@@ -48,6 +48,11 @@ public abstract class ByteEncoding : MonoEncoding
 	protected bool isMailNewsDisplay;
 	protected bool isMailNewsSave;
 	protected int windowsCodePage;
+#if NET_2_0
+	static byte [] isNormalized;
+	static byte [] isNormalizedComputed;
+	static byte [] normalization_bytes;
+#endif
 
 	// Constructor.
 	protected ByteEncoding(int codePage, char[] toChars,
@@ -58,6 +63,9 @@ public abstract class ByteEncoding : MonoEncoding
 						   int windowsCodePage)
 			: base(codePage)
 			{
+				if (toChars.Length != byte.MaxValue + 1)
+					throw new ArgumentException("toChars");
+
 				this.toChars = toChars;
 				this.encodingName = encodingName;
 				this.bodyName = bodyName;
@@ -71,6 +79,38 @@ public abstract class ByteEncoding : MonoEncoding
 			}
 
 #if NET_2_0
+	public override bool IsAlwaysNormalized (NormalizationForm form)
+	{
+		if (form != NormalizationForm.FormC)
+			return false;
+
+		if (isNormalized == null)
+			isNormalized = new byte [0x10000 / 8];
+		if (isNormalizedComputed == null)
+			isNormalizedComputed = new byte [0x10000 / 8];
+
+		if (normalization_bytes == null) {
+			normalization_bytes = new byte [0x100];
+			lock (normalization_bytes) {
+				for (int i = 0; i < 0x100; i++)
+					normalization_bytes [i] = (byte) i;
+			}
+		}
+
+		byte offset = (byte) (1 << (CodePage % 8));
+		if ((isNormalizedComputed [CodePage / 8] & offset) == 0) {
+			Encoding e = Clone () as Encoding;
+			e.DecoderFallback = new DecoderReplacementFallback ("");
+			string s = e.GetString (normalization_bytes);
+			// note that the flag only stores FormC information.
+			if (s != s.Normalize (form))
+				isNormalized [CodePage / 8] |= offset;
+			isNormalizedComputed [CodePage / 8] |= offset;
+		}
+
+		return (isNormalized [CodePage / 8] & offset) == 0;
+	}
+
 	public override bool IsSingleByte {
 		get { return true; }
 	}
@@ -314,7 +354,7 @@ public abstract class ByteEncoding : MonoEncoding
 			}
 
 	// Decode a buffer of bytes into a string.
-	public override String GetString(byte[] bytes, int index, int count)
+	public unsafe override String GetString(byte[] bytes, int index, int count)
 			{
 				if(bytes == null)
 				{
@@ -330,13 +370,22 @@ public abstract class ByteEncoding : MonoEncoding
 					throw new ArgumentOutOfRangeException
 						("count", Strings.GetString("ArgRange_Array"));
 				}
-				StringBuilder s = new StringBuilder();
-				char[] cvt = toChars;
-				while(count-- > 0)
-				{
-					s.Append(cvt[(int)(bytes[index++])]);
-				}
-				return s.ToString();
+
+				if (count == 0)
+					return string.Empty;
+
+				string s = new string ((char) 0, count);
+
+				fixed (byte* bytePtr = bytes)
+					fixed (char* charPtr = s)
+						fixed (char* cvt = toChars) {
+							byte* b = bytePtr + index;
+							char* c = charPtr;
+							while(count-- != 0)
+								*(c++) = cvt[*(b++)];
+						}
+
+				return s;
 			}
 	public override String GetString(byte[] bytes)
 			{
@@ -344,15 +393,8 @@ public abstract class ByteEncoding : MonoEncoding
 				{
 					throw new ArgumentNullException("bytes");
 				}
-				int count = bytes.Length;
-				int posn = 0;
-				StringBuilder s = new StringBuilder();
-				char[] cvt = toChars;
-				while(count-- > 0)
-				{
-					s.Append(cvt[(int)(bytes[posn++])]);
-				}
-				return s.ToString();
+
+				return GetString (bytes, 0, bytes.Length);
 			}
 
 #if !ECMA_COMPAT
