@@ -38,14 +38,115 @@ using System.IO;
 using System.Reflection;
 using System.Runtime.InteropServices;
 using System.Threading;
+#if NET_2_0
+using System.Windows.Forms.VisualStyles;
+#endif
 
 namespace System.Windows.Forms {
 	public sealed class Application {
+		internal class MWFThread {
+			#region Fields
+			private ApplicationContext	context;
+			private bool			messageloop_started;
+			private int			thread_id;
+
+			private static Hashtable	threads = new Hashtable();
+			#endregion	// Fields
+
+			#region Constructors
+			private MWFThread() {
+			}
+			#endregion	// Constructors
+
+			#region Properties
+			public ApplicationContext Context {
+				get {
+					return context;
+				}
+
+				set {
+					context = value;
+				}
+			}
+
+			public bool MessageLoop {
+				get {
+					return messageloop_started;
+				}
+
+				set {
+					messageloop_started = value;
+				}
+			}
+
+
+			public static int LoopCount {
+				get {
+					IEnumerator	e;
+					int		loops;
+					MWFThread	thread;
+
+					e = threads.Values.GetEnumerator();
+					loops = 0;
+
+					while (e.MoveNext()) {
+						thread = (MWFThread)e.Current;
+						if (thread != null && thread.messageloop_started) {
+							loops++;
+						}
+					}
+
+					return loops;
+				}
+			}
+
+			public static MWFThread Current {
+				get {
+					MWFThread	thread;
+
+					thread = null;
+					lock (threads) {
+						thread = (MWFThread)threads[Thread.CurrentThread.GetHashCode()];
+						if (thread == null) {
+							thread = new MWFThread();
+							thread.thread_id = Thread.CurrentThread.GetHashCode();
+							threads[thread.thread_id] = thread;
+						}
+					}
+
+					return thread;
+				}
+			}
+			#endregion	// Properties
+
+			#region Methods
+			public void Exit() {
+				if (context != null) {
+					context.ExitThread();
+				}
+				context = null;
+
+				if (Application.ThreadExit != null) {
+					Application.ThreadExit(null, EventArgs.Empty);
+				}
+
+				if (LoopCount == 0) {
+					if (Application.ApplicationExit != null) {
+						Application.ApplicationExit(null, EventArgs.Empty);
+					}
+				}
+				threads[thread_id] = null;
+			}
+			#endregion	// Methods
+		}
+
 		private static bool			browser_embedded	= false;
 		private static InputLanguage		input_language		= InputLanguage.CurrentInputLanguage;
-		private static bool			messageloop_started	= false;
 		private static string			safe_caption_format	= "{1} - {0} - {2}";
 		private static ArrayList		message_filters		= new ArrayList();
+#if NET_2_0
+		private static VisualStyleState visual_style_state = VisualStyleState.ClientAndNonClientAreasEnabled;
+#endif
 
 		private Application () {
 		}
@@ -154,7 +255,7 @@ namespace System.Windows.Forms {
 
 		public static bool MessageLoop {
 			get {
-				return messageloop_started;
+				return MWFThread.Current.MessageLoop;
 			}
 		}
 
@@ -176,9 +277,6 @@ namespace System.Windows.Forms {
 
 				version = Assembly.GetEntryAssembly().GetName().Version.ToString();
 
-				if (version.StartsWith("0.")) {
-					version="1." + version.Substring(2);
-				}
 				return version;
 			}
 		}
@@ -214,6 +312,35 @@ namespace System.Windows.Forms {
 				return key;
 			}
 		}
+
+#if NET_2_0
+		public static bool RenderWithVisualStyles {
+		      get {
+				if (VisualStyleInformation.IsSupportedByOS)
+				{
+					if (!VisualStyleInformation.IsEnabledByUser)
+						return false;
+				  
+					if (!XplatUI.ThemesEnabled)
+						return false;
+				  
+					if (Application.VisualStyleState == VisualStyleState.ClientAndNonClientAreasEnabled)
+						return true;
+				  
+					if (Application.VisualStyleState == VisualStyleState.ClientAreaEnabled)
+				  		return true;
+				}
+				
+				return false;
+		      }
+		}
+
+		public static VisualStyleState VisualStyleState {
+			get { return Application.visual_style_state; }
+			set { Application.visual_style_state = value; }
+		}
+#endif
+
 		#endregion
 
 		#region Public Static Methods
@@ -230,10 +357,6 @@ namespace System.Windows.Forms {
 		}
 
 #if NET_2_0
-		public static void EnableRTLMirroring () 
-		{
-		}
-
 		//
 		// If true, it uses GDI+, performance reasons were quoted
 		//
@@ -258,6 +381,7 @@ namespace System.Windows.Forms {
 
 		public static void ExitThread() {
 			CloseForms(Thread.CurrentThread);
+			MWFThread.Current.Exit();
 		}
 
 		public static ApartmentState OleRequired() {
@@ -299,6 +423,11 @@ namespace System.Windows.Forms {
 			Queue		toplevels;
 			IEnumerator	control;
 			MSG		msg;
+			Object		queue_id;
+			MWFThread	thread;
+
+
+			thread = MWFThread.Current;
 
 			msg = new MSG();
 
@@ -306,13 +435,16 @@ namespace System.Windows.Forms {
 				context = new ApplicationContext();
 			}
 
+			thread.Context = context;
+
 			if (context.MainForm != null) {
+				context.MainForm.is_modal = Modal;
+				context.MainForm.context = context;
+				context.MainForm.closing = false;
 				context.MainForm.Visible = true;	// Cannot use Show() or scaling gets confused by menus
 				// FIXME - do we need this?
 				//context.MainForm.PerformLayout();
-				context.MainForm.context = context;
 				context.MainForm.Activate();
-				context.MainForm.closing = false;
 			}
 
 			#if DebugRunLoop
@@ -322,11 +454,6 @@ namespace System.Windows.Forms {
 			if (Modal) {
 				Control c;
 
-				if (context.MainForm.Modal) {
-					throw new Exception("fixme");
-				}
-				context.MainForm.is_modal = true;
-
 				toplevels = new Queue();
 				control = Control.controls.GetEnumerator();
 
@@ -350,10 +477,10 @@ namespace System.Windows.Forms {
 				toplevels = null;
 			}
 
-			XplatUI.StartLoop(Thread.CurrentThread);
-			messageloop_started = true;
+			queue_id = XplatUI.StartLoop(Thread.CurrentThread);
+			thread.MessageLoop = true;
 
-			while (XplatUI.GetMessage(ref msg, IntPtr.Zero, 0, 0)) {
+			while (XplatUI.GetMessage(queue_id, ref msg, IntPtr.Zero, 0, 0)) {
 				if ((message_filters != null) && (message_filters.Count > 0)) {
 					Message	m;
 					bool	drop;
@@ -397,7 +524,7 @@ namespace System.Windows.Forms {
 				}
 
 				// Handle exit, Form might have received WM_CLOSE and set 'closing' in response
-				if ((context.MainForm != null) && context.MainForm.closing) {
+				if ((context.MainForm != null) && (context.MainForm.closing || (Modal && !context.MainForm.Visible))) {
 					if (!Modal) {
 						XplatUI.PostQuitMessage(0);
 					} else {
@@ -409,7 +536,7 @@ namespace System.Windows.Forms {
 				Console.WriteLine("   RunLoop loop left");
 			#endif
 
-			messageloop_started = false;
+			thread.MessageLoop = false;
 			XplatUI.EndLoop(Thread.CurrentThread);
 
 			if (Modal) {
@@ -446,11 +573,7 @@ namespace System.Windows.Forms {
 			}
 
 			if (!Modal) {
-				if (ThreadExit != null) {
-					ThreadExit(null, EventArgs.Empty);
-				}
-
-				context.ExitThread();
+				thread.Exit();
 			}
 		}
 
