@@ -29,6 +29,8 @@
 
 namespace System.Windows.Forms
 {
+#if !NET_2_0
 	[Serializable]
+#endif
 	public delegate void ColumnClickEventHandler (object sender, ColumnClickEventArgs e);
 }
