@@ -27,6 +27,7 @@
 
 // NOT COMPLETE
 
+using System.Collections;
 using System.ComponentModel;
 using System.ComponentModel.Design;
 using System.Drawing;
@@ -36,6 +37,10 @@ namespace System.Windows.Forms {
 		private Control		active_control;
 		private Control		focused_control;
 		private Control		unvalidated_control;
+
+		// This is an internal hack that allows some container controls
+		// to not auto select their child when they are activated
+		internal bool 		auto_select_child = true;
 #if NET_2_0
 		private SizeF		auto_scale_dimensions;
 		private AutoScaleMode	auto_scale_mode;
@@ -67,20 +72,89 @@ namespace System.Windows.Forms {
 					return;
 				}
 
-				active_control = value;
-
 				if (!Contains(value) && this != value) {
 					throw new ArgumentException("Not a child control");
 				}
 
+				// Fire the enter and leave events if possible
+				Form form = FindForm ();
+				if (form != null) {
+					Control active = form.ActiveControl;
+					Control common_container = GetCommonContainer (form.ActiveControl, value);
+					ArrayList chain = new ArrayList ();
+					Control walk = active;
+
+					// Generate the leave messages	
+					while (walk != common_container) {
+						walk.FireLeave ();
+						if (walk.CausesValidation && !ValidateControl (walk))
+							return;
+						walk = walk.Parent;
+					}
+
+					walk = value;
+					while (walk != common_container) {
+						chain.Add (walk);
+						walk = walk.Parent;
+					}
+
+					for (int i = chain.Count - 1; i >= 0; i--) {
+						walk = (Control) chain [i];
+						walk.FireEnter ();
+					}
+				}
+
+				active_control = value;
+
 				if (this is Form)
 					CheckAcceptButton();
-				
+
 				// Scroll control into view
 				ScrollControlIntoView(active_control);
 
 				// Let the control know it's selected
-				Select(value);
+				SendControlFocus (value);
+			}
+		}
+
+		private bool ValidateControl (Control c)
+		{
+			CancelEventArgs e = new CancelEventArgs ();
+
+			c.FireValidating (e);
+
+			if (e.Cancel)
+				return false;
+
+			c.FireValidated ();
+			return true;
+		}
+
+		// Just in a separate method to make debugging a little easier,
+		// should eventually be rolled into ActiveControl setter
+		private Control GetCommonContainer (Control active_control, Control value)
+		{
+			Control new_container = null;
+			Control prev_container = active_control;
+
+			while (prev_container != null) {
+				new_container = value.Parent;
+				while (new_container != null) {
+					if (new_container == prev_container)
+						return new_container;
+					new_container = new_container.Parent;
+				}
+
+				prev_container = prev_container.Parent;
+			}
+
+			return null;
+		}
+
+		private void SendControlFocus (Control c)
+		{
+			if (c.IsHandleCreated) {
+				XplatUI.SetFocus (c.window.Handle);
 			}
 		}
 
@@ -319,11 +393,21 @@ namespace System.Windows.Forms {
 		}
 
 		protected virtual bool ProcessTabKey(bool forward) {
-			return SelectNextControl(active_control, forward, true, true, true);
+			return SelectNextControl(active_control, forward, true, true, false);
 		}
 
-		protected override void Select(bool directed, bool forward) {
-			base.Select(directed, forward);
+		protected override void Select(bool directed, bool forward)
+		{
+			if (Parent != null) {
+				IContainerControl parent = Parent.GetContainerControl ();
+				if (parent != null) {
+					parent.ActiveControl = this;
+				}
+			}
+
+			if (directed && auto_select_child) {
+				SelectNextControl (null, forward, true, true, false);
+			}
 		}
 
 		protected virtual void UpdateDefaultButton() {
@@ -332,13 +416,32 @@ namespace System.Windows.Forms {
 
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		protected override void WndProc(ref Message m) {
+			switch ((Msg) m.Msg) {
+			case Msg.WM_LBUTTONDOWN:
+				OnMouseDown (new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
+				        mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()),
+					HighOrder ((int) m.LParam.ToInt32 ()), 0));
+				return;
+				/*
+			case Msg.WM_SETFOCUS:
+				if (active_control != null)
+					Select (active_control);
+				else
+					SelectNextControl (null, true, true, true, false);
+				break;
+				*/
+			case Msg.WM_KILLFOCUS:
+				break;
+			}
 			base.WndProc(ref m);
 		}
 		#endregion	// Protected Instance Methods
-		
+
+		#region Internal Methods
 		internal virtual void CheckAcceptButton()
 		{
 			// do nothing here, only called if it is a Form
 		}
+		#endregion	// Internal Methods
 	}
 }
