@@ -30,59 +30,25 @@ using System.Collections;
 using System.ComponentModel;
 
 namespace System.Windows.Forms {
-	[DefaultMember("Item")]
 	public class CurrencyManager : BindingManagerBase {
 
-		protected Type finalType;
 		protected int listposition;
+		protected Type finalType;
 
 		private IList list;
 		private bool binding_suspended;
 
-		internal CurrencyManager (object data_source, string data_member)
+		private object data_source;
+
+		bool editing;
+
+		internal CurrencyManager ()
 		{
-			if (data_source is IList) {
-				list = (IList) data_source;
-			} else if (data_source is IListSource) {
-				list = ((IListSource) data_source).GetList ();
-			} else {
-				throw new Exception ("Attempted to create currency manager " +
-					"from invalid type: " + data_source.GetType ());
-			}
+		}
 
-			if (data_source as ArrayList != null) {
-				finalType = ((ArrayList)data_source).GetType ();
-			} else {
-				if (data_source as Array != null) {
-					finalType = ((Array) data_source).GetType ();
-				} else {
-					finalType = null;
-				}
-			}
-
-			DataTable table = data_source as DataTable;
-			if (table == null && data_source is DataView)
-				table = ((DataView) data_source).Table;
-
-			
-			if (table == null) {
-				DataSet dataset = data_source as DataSet;
-				int sp = data_member.IndexOf ('.');
-				if (sp != -1) {
-					data_member = data_member.Substring (0, sp);
-				}
-				if (dataset != null) {
-					table = dataset.Tables [data_member];
-				}
-			}
-
-			if (table != null) {
-				list = new DataView (table);
-				table.Columns.CollectionChanged  += new CollectionChangeEventHandler (MetaDataChangedHandler);
-				table.ChildRelations.CollectionChanged  += new CollectionChangeEventHandler (MetaDataChangedHandler);
-				table.ParentRelations.CollectionChanged  += new CollectionChangeEventHandler (MetaDataChangedHandler);
-				table.Constraints.CollectionChanged += new CollectionChangeEventHandler (MetaDataChangedHandler);
-			}
+		internal CurrencyManager (object data_source)
+		{
+			SetDataSource (data_source);
 		}
 
 		public IList List {
@@ -91,6 +57,8 @@ namespace System.Windows.Forms {
 
 		public override object Current {
 			get {
+				if (listposition == -1 || listposition >= list.Count)
+					throw new IndexOutOfRangeException ("list position");
 				return list [listposition];
 			}
 		}
@@ -110,38 +78,65 @@ namespace System.Windows.Forms {
 					value = list.Count - 1;
 				if (listposition == value)
 					return;
+
+				if (listposition != -1)
+					EndCurrentEdit ();
+
 				listposition = value;
 				OnCurrentChanged (EventArgs.Empty);
 				OnPositionChanged (EventArgs.Empty);
 			}
 		}
-		
-		internal string ListName {
-			get {
-				ITypedList typed = list as ITypedList;
-				
-				if (typed == null) {
-					return finalType.Name;
-				} else {
-					return typed.GetListName (null);
-				}
-			}		
+
+		[MonoTODO]
+		internal void SetDataSource (object data_source)
+		{
+			if (this.data_source is IBindingList)
+				((IBindingList)this.data_source).ListChanged -= new ListChangedEventHandler (ListChangedHandler);
+			if (this.data_source is DataView) {
+				DataView dataview = (DataView)this.data_source;
+				dataview.Table.Columns.CollectionChanged  -= new CollectionChangeEventHandler (MetaDataChangedHandler);
+				dataview.Table.ChildRelations.CollectionChanged  -= new CollectionChangeEventHandler (MetaDataChangedHandler);
+				dataview.Table.ParentRelations.CollectionChanged  -= new CollectionChangeEventHandler (MetaDataChangedHandler);
+				dataview.Table.Constraints.CollectionChanged -= new CollectionChangeEventHandler (MetaDataChangedHandler);
+			}
+
+			if (data_source is IListSource)
+				data_source = ((IListSource)data_source).GetList();
+
+			this.data_source = data_source;
+			if (data_source != null)
+				this.finalType = data_source.GetType();
+
+			listposition = -1;
+			if (this.data_source is IBindingList)
+				((IBindingList)this.data_source).ListChanged += new ListChangedEventHandler (ListChangedHandler);
+			if (this.data_source is DataView) {
+				DataView dataview = (DataView)this.data_source;
+				dataview.Table.Columns.CollectionChanged  += new CollectionChangeEventHandler (MetaDataChangedHandler);
+				dataview.Table.ChildRelations.CollectionChanged  += new CollectionChangeEventHandler (MetaDataChangedHandler);
+				dataview.Table.ParentRelations.CollectionChanged  += new CollectionChangeEventHandler (MetaDataChangedHandler);
+				dataview.Table.Constraints.CollectionChanged += new CollectionChangeEventHandler (MetaDataChangedHandler);
+			}
+
+			list = (IList)data_source;
+
+			// XXX this is wrong.  MS invokes OnItemChanged directly, which seems to call PushData.
+			ListChangedHandler (null, new ListChangedEventArgs (ListChangedType.Reset, -1));
 		}
 
 		public override PropertyDescriptorCollection GetItemProperties ()
 		{
-			ITypedList typed = list as ITypedList;
-
 			if (list is Array) {
 				Type element = list.GetType ().GetElementType ();
 				return TypeDescriptor.GetProperties (element);
 			}
 
-			if (typed != null) {
-				return typed.GetItemProperties (null);
+			if (list is ITypedList) {
+				return ((ITypedList)list).GetItemProperties (null);
 			}
 
-			PropertyInfo [] props = finalType.GetProperties ();
+			PropertyInfo [] props = data_source.GetType().GetProperties ();
 			for (int i = 0; i < props.Length; i++) {
 				if (props [i].Name == "Item") {
 					Type t = props [i].PropertyType;
@@ -176,14 +171,16 @@ namespace System.Windows.Forms {
                 internal override bool IsSuspended {
                         get { return binding_suspended; }
                 }
-                
+
+                [MonoTODO ("this needs re-addressing once DataViewManager.AllowNew is implemented")]
                 internal bool CanAddRows {
                 	get {
-				if (list as IBindingList == null) {
-					return false;
+				if (list is IBindingList) {
+					return true;
+					//return ((IBindingList)list).AllowNew;
 				}
-				
-				return true;
+
+				return false;
 			}
 		}
 
@@ -191,32 +188,63 @@ namespace System.Windows.Forms {
 		{
 			if (list as IBindingList == null)
 				throw new NotSupportedException ();
-				
+
 			(list as IBindingList).AddNew ();
+
+			EndCurrentEdit ();
+
+			listposition = list.Count - 1;
+
+			OnCurrentChanged (EventArgs.Empty);
+			OnPositionChanged (EventArgs.Empty);
+		}
+
+
+		void BeginEdit ()
+		{
+			IEditableObject editable = Current as IEditableObject;
+
+			if (editable != null) {
+				try {
+					editable.BeginEdit ();
+					editing = true;
+				}
+				catch {
+					/* swallow exceptions in IEditableObject.BeginEdit () */
+				}
+			}
 		}
 
 		public override void CancelCurrentEdit ()
 		{
+			if (listposition == -1)
+				return;
+
 			IEditableObject editable = Current as IEditableObject;
 
-			if (editable == null)
-				return;
-			editable.CancelEdit ();
-			OnItemChanged (new ItemChangedEventArgs (Position));
+			if (editable != null) {
+				editing = false;
+				editable.CancelEdit ();
+				OnItemChanged (new ItemChangedEventArgs (Position));
+			}
 		}
 		
 		public override void EndCurrentEdit ()
 		{
+			if (listposition == -1)
+				return;
+
 			IEditableObject editable = Current as IEditableObject;
 
-			if (editable == null)
-				return;
-			editable.EndEdit ();
+			if (editable != null) {
+				editing = false;
+				editable.EndEdit ();
+			}
 		}
 
 		public void Refresh ()
 		{
-			PullData ();
+			ListChangedHandler (null, new ListChangedEventArgs (ListChangedType.Reset, -1));
 		}
 
 		protected void CheckEmpty ()
@@ -228,7 +256,7 @@ namespace System.Windows.Forms {
 
 		protected internal override void OnCurrentChanged (EventArgs e)
 		{
-			PullData ();
+			BeginEdit ();
 
 			if (onCurrentChangedHandler != null) {
 				onCurrentChangedHandler (this, e);
@@ -237,31 +265,32 @@ namespace System.Windows.Forms {
 
 		protected virtual void OnItemChanged (ItemChangedEventArgs e)
 		{
-			PushData ();
-
 			if (ItemChanged != null)
 				ItemChanged (this, e);
 		}
 
 		protected virtual void OnPositionChanged (EventArgs e)
 		{
-			if (onPositionChangedHandler == null)
-				return;
-			onPositionChangedHandler (this, e);
+			if (onPositionChangedHandler != null)
+				onPositionChangedHandler (this, e);
 		}
 
 		protected internal override string GetListName (ArrayList accessors)
 		{
 			if (list is ITypedList) {
-				PropertyDescriptor [] pds;
-				pds = new PropertyDescriptor [accessors.Count];
-				accessors.CopyTo (pds, 0);
+				PropertyDescriptor [] pds = null;
+				if (accessors != null) {
+					pds = new PropertyDescriptor [accessors.Count];
+					accessors.CopyTo (pds, 0);
+				}
 				return ((ITypedList) list).GetListName (pds);
 			}
+			else if (finalType != null) {
+				return finalType.Name;
+			}
 			return String.Empty;
 		}
 
-		[MonoTODO ("Not totally sure how this works, its doesn't seemt to do a pull/push like i originally assumed")]
 		protected override void UpdateIsBinding ()
 		{
 			UpdateItem ();
@@ -273,12 +302,19 @@ namespace System.Windows.Forms {
 		private void UpdateItem ()
 		{
 			// Probably should be validating or something here
-			EndCurrentEdit ();
+			if (listposition != -1) {
+				EndCurrentEdit ();
+			}
+			else if (list.Count > 0) {
+
+				listposition = 0;
+				
+				BeginEdit ();
+			}
 		}
 		
-		internal object GetItem (int index)
-		{
-			return list [index];
+		internal object this [int index] {
+			get { return list [index]; }
 		}		
 		
 		private PropertyDescriptorCollection GetBrowsableProperties (Type t)
@@ -294,6 +330,39 @@ namespace System.Windows.Forms {
 				MetaDataChanged (this, EventArgs.Empty);
 		}
 
+		private void ListChangedHandler (object sender, ListChangedEventArgs e)
+		{
+			switch (e.ListChangedType) {
+			case ListChangedType.ItemDeleted:
+				if (listposition == e.NewIndex) {
+					listposition = e.NewIndex - 1;
+					OnCurrentChanged (EventArgs.Empty);
+					OnPositionChanged (EventArgs.Empty);
+				}
+					
+				OnItemChanged (new ItemChangedEventArgs (-1));
+				break;
+			case ListChangedType.ItemAdded:
+				if (listposition == -1) {
+					listposition = e.NewIndex - 1;
+					OnCurrentChanged (EventArgs.Empty);
+					OnPositionChanged (EventArgs.Empty);
+				}
+
+				OnItemChanged (new ItemChangedEventArgs (-1));
+				break;
+			case ListChangedType.ItemChanged:
+				if (editing)
+					OnItemChanged (new ItemChangedEventArgs (e.NewIndex));
+				break;
+			default:
+				PushData ();
+				OnItemChanged (new ItemChangedEventArgs (-1));
+				//				UpdateIsBinding ();
+				break;
+			}
+		}
+
 		public event ItemChangedEventHandler ItemChanged;
 		public event EventHandler MetaDataChanged;
 	}
