@@ -33,6 +33,7 @@ using System.IO;
 using System.ComponentModel;
 using System.Runtime.InteropServices;
 using System.Runtime.Serialization;
+using System.Reflection;
 
 namespace System.Windows.Forms {
 	[Editor("System.Drawing.Design.CursorEditor, " + Consts.AssemblySystem_Drawing_Design, typeof(System.Drawing.Design.UITypeEditor))]
@@ -139,11 +140,20 @@ namespace System.Windows.Forms {
 
 		public Cursor(Type type, string resource) {
 			using (Stream s = type.Assembly.GetManifestResourceStream (type, resource)) {
-				if (s == null) {
-					throw new FileNotFoundException ("Resource name was not found: `" + resource + "'");
+				if (s != null) {
+					CreateCursor(s);
+					return;
 				}
-				CreateCursor(s);
 			}
+
+			// Try a different way, previous failed
+			using (Stream s = Assembly.GetExecutingAssembly().GetManifestResourceStream(resource)) {
+				if (s != null) {
+					CreateCursor(s);
+					return;
+				}
+			}
+			throw new FileNotFoundException ("Resource name was not found: `" + resource + "'");
 		}
 		#endregion	// Public Constructors
 
@@ -391,7 +401,7 @@ namespace System.Windows.Forms {
 			cursor_dir.idType = reader.ReadUInt16();
 			if (cursor_dir.idReserved != 0 || !(cursor_dir.idType == 2 || cursor_dir.idType == 1))
 				throw new System.ArgumentException ("Invalid Argument, format error", "stream");
-		
+
 			entry_count = reader.ReadUInt16();
 			cursor_dir.idCount = entry_count;
 			cursor_dir.idEntries = new CursorEntry[entry_count];
@@ -407,6 +417,10 @@ namespace System.Windows.Forms {
 				ce.reserved = reader.ReadByte();
 				ce.xHotspot = reader.ReadUInt16();
 				ce.yHotspot = reader.ReadUInt16();
+				if (cursor_dir.idType == 1) {
+					ce.xHotspot = (ushort)(ce.width / 2);
+					ce.yHotspot = (ushort)(ce.height / 2);
+				}
 				ce.sizeInBytes = reader.ReadUInt32();
 				ce.fileOffset = reader.ReadUInt32();
 
@@ -694,8 +708,9 @@ namespace System.Windows.Forms {
 					pal = bmp.Palette;				// Managed palette
 
 					for (int i = 0; i < ci.cursorColors.Length; i++) {
-						pal.Entries[i] = Color.FromArgb((int)ci.cursorColors[i] & unchecked((int)0xff000000));
+						pal.Entries[i] = Color.FromArgb((int)ci.cursorColors[i] | unchecked((int)0xff000000));
 					}
+					bmp.Palette = pal;
 				}
 
 				bytesPerLine = (int)((((cih.biWidth * cih.biBitCount) + 31) & ~31) >> 3);
