@@ -134,20 +134,14 @@ namespace System.Windows.Forms
 		#endregion	// Local Variables
 
 		#region Constructors
-		public DataGridColumnStyle ()
+		public DataGridColumnStyle () : this (null)
 		{
-			CommmonConstructor ();
-			property_descriptor = null;
 		}
 
 		public DataGridColumnStyle (PropertyDescriptor prop)
 		{
-			CommmonConstructor ();
 			property_descriptor = prop;
-		}
 
-		private void CommmonConstructor ()
-		{
 			fontheight = -1;
 			table_style = null;
 			header_text = string.Empty;
@@ -190,9 +184,7 @@ namespace System.Windows.Forms
 
 		[Browsable(false)]
 		public virtual DataGridTableStyle DataGridTableStyle {
-			get {
-				return table_style;
-			}			
+			get { return table_style; }			
 		}
 		
 		protected int FontHeight {
@@ -227,9 +219,7 @@ namespace System.Windows.Forms
 				if (value != header_text) {
 					header_text = value;
 					
-					if (table_style != null && table_style.DataGrid != null) {
-						table_style.DataGrid.Invalidate ();
-					}
+					Invalidate ();
 
 					if (HeaderTextChanged != null) {
 						HeaderTextChanged (this, EventArgs.Empty);
@@ -442,7 +432,9 @@ namespace System.Windows.Forms
 		protected internal virtual object GetColumnValueAtRow (CurrencyManager source, int rowNum)
 		{			
 			CheckValidDataSource (source);
-			return property_descriptor.GetValue (source.GetItem (rowNum));
+			if (rowNum >= source.Count)
+				return DBNull.Value;
+			return property_descriptor.GetValue (source [rowNum]);
 		}
 
 		protected internal abstract int GetMinimumHeight ();
@@ -453,12 +445,13 @@ namespace System.Windows.Forms
 
 		void  IDataGridColumnStyleEditingNotificationService.ColumnStartedEditing (Control editingControl)
 		{
-
+			ColumnStartedEditing (editingControl);
 		}
 
 		protected virtual void Invalidate ()
 		{
-			grid.grid_drawing.InvalidateColumn (this);
+			if (grid != null)
+				grid.InvalidateColumn (this);
 		}
 
 		protected internal abstract void Paint (Graphics g, Rectangle bounds, CurrencyManager source, int rowNum);
@@ -477,7 +470,13 @@ namespace System.Windows.Forms
 		protected internal virtual void SetColumnValueAtRow (CurrencyManager source, int rowNum,  object value)
 		{
 			CheckValidDataSource (source);
-			property_descriptor.SetValue (source.GetItem (rowNum), value);			
+
+			IEditableObject editable = source [rowNum] as IEditableObject;
+
+			if (editable != null) {
+				editable.BeginEdit ();
+				property_descriptor.SetValue (source [rowNum], value);
+			}
 		}
 
 		protected virtual void SetDataGrid (DataGrid value)
