@@ -40,16 +40,17 @@ namespace System.Windows.Forms
 	{
 
 		#region	Local Variables
-		private bool isedit;
+		private bool isnavigating;
 		private DataGrid grid;
 		#endregion	// Local Variables
 
 		#region Constructors
 		public DataGridTextBox ()
 		{
-			isedit = true;
+			isnavigating = true;
 			grid = null;
 			accepts_tab = true;
+			accepts_return = false;
 
 			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick | ControlStyles.StandardDoubleClick, false);
 			SetStyle (ControlStyles.FixedHeight, true);
@@ -58,23 +59,19 @@ namespace System.Windows.Forms
 
 		#region Public Instance Properties
 		public bool IsInEditOrNavigateMode {
-			get {
-				return isedit;
-			}
-			set {
-				if (value != isedit) {
-					isedit = value;
-				}
-			}
+			get { return isnavigating; }
+			set { isnavigating = value; }
 		}
 
 		#endregion	// Public Instance Properties
 
 		#region Public Instance Methods
 		protected override void OnKeyPress (KeyPressEventArgs e)
-		{			
-			grid.is_changing = true;
-			grid.InvalidateCurrentRowHeader ();
+		{
+			if (!ReadOnly) {
+				isnavigating = false;
+				grid.ColumnStartedEditing (Bounds);
+			}
 			base.OnKeyPress (e);
 		}
 
@@ -87,85 +84,47 @@ namespace System.Windows.Forms
 		{
 			Keys key = (Keys) m.WParam.ToInt32 ();
 
-			// If we decide DataGrid needs to process we call grid.ProcessKeyPreviewInternal and return true
-			// If we want TextBox to handle the key , we return false;
+			if (isnavigating && ProcessKeyPreview (ref m))
+				return true;
 
-			// We only care about KEYDOWN messages
-			if (m.Msg != (int)Msg.WM_KEYDOWN) {
+			switch ((Msg)m.Msg) {
+			case Msg.WM_CHAR:
+				switch (key) {
+				case Keys.Enter:
+					isnavigating = true;
+					return false;
+				default:
+					return ProcessKeyEventArgs (ref m);
+				}
+
+			case Msg.WM_KEYDOWN:
+				switch (key) {
+				case Keys.F2:
+					SelectionStart = Text.Length;
+					SelectionLength = 0;
+					return false;
+
+				case Keys.Left:
+					if (SelectionStart == 0)
+						return ProcessKeyPreview (ref m);
+					return false;
+				case Keys.Right:
+					// Arrow keys go right until we hit the end of the text
+					if (SelectionStart + SelectionLength >= Text.Length) {
+						return ProcessKeyPreview (ref m);
+					}
+					return false;
+				case Keys.Enter:
+					return true;
+				case Keys.Up:
+				case Keys.Down:
+					return ProcessKeyPreview (ref m);
+				}
+
+				return ProcessKeyEventArgs(ref m);
+			default:
 				return false;
 			}
-
-			// Anything with control key pressed is for DataGrid
-			if ((Control.ModifierKeys & Keys.Control) != 0) {
-				return grid.ProcessKeyPreviewInternal(ref m);
-			}
-
-			if (isedit) {
-				switch (key) {
-					case Keys.F2: {
-						this.SelectionStart = this.Text.Length;
-						this.SelectionLength = 0;
-						return true;
-					}
-
-					case Keys.Up:
-					case Keys.Down:
-					case Keys.PageDown:
-					case Keys.PageUp: {
-						isedit = false;
-						grid.EndEdit(false);
-						grid.ProcessKeyPreviewInternal(ref m);
-						return true;
-					}
-
-					case Keys.Escape: {
-						isedit = false;
-						grid.EndEdit (true);
-						return true;
-					}
-
-					case Keys.Enter: {
-						isedit = false;
-						grid.EndEdit (false);
-						grid.ProcessKeyPreviewInternal(ref m);
-						return true;
-					}
-
-					case Keys.Left: {
-						if (base.SelectionStart == 0) {
-							isedit = false;
-							grid.EndEdit (false);
-							grid.ProcessKeyPreviewInternal(ref m);
-							return true;
-						}
-						return false;
-					}
-
-					case Keys.Right: {
-						// Arrow keys go right until we hit the end of the text
-						if ((base.SelectionStart + base.SelectionLength) == base.Text.Length) {
-							isedit = false;
-							grid.EndEdit (false);
-							grid.ProcessKeyPreviewInternal(ref m);
-							return true;
-						}
-						return false;
-					}
-
-					case Keys.Tab: {
-						isedit = false;
-						grid.EndEdit(false);
-						grid.ProcessKeyPreviewInternal(ref m);
-						return true;
-					}
-
-					default: {
-						return base.ProcessKeyMessage(ref m);
-					}
-				}
-			}
-
-			return base.ProcessKeyMessage(ref m);
 		}
 
 		public void SetDataGrid (DataGrid parentGrid)
@@ -175,6 +134,13 @@ namespace System.Windows.Forms
 
 		protected override void WndProc (ref Message m)
 		{
+			switch ((Msg)m.Msg) {
+			case Msg.WM_LBUTTONDOWN:
+			case Msg.WM_LBUTTONDBLCLK:
+				isnavigating = false;
+				break;
+			}
+
 			base.WndProc (ref m);
 		}
 
