@@ -39,37 +39,38 @@ namespace System.Windows.Forms
 		private string format;
 		private IFormatProvider format_provider = null;
 		private StringFormat string_format =  new StringFormat ();
-		private DataGridTextBox textbox = null;
-		private static readonly int offset_x = 2;
-		private static readonly int offset_y = 2;
+		private DataGridTextBox textbox;
+		private static readonly int offset_x = 0;
+		private static readonly int offset_y = 0;
 		#endregion	// Local Variables
 
 		#region Constructors
-		public DataGridTextBoxColumn ()
+		public DataGridTextBoxColumn () : this (null, String.Empty, false)
 		{
-			format = string.Empty;
 		}
 
-		public DataGridTextBoxColumn (PropertyDescriptor prop) : base (prop)
+		public DataGridTextBoxColumn (PropertyDescriptor prop) : this (prop, String.Empty, false)
 		{
-			format = string.Empty;
 		}
 		
-		public DataGridTextBoxColumn (PropertyDescriptor prop,  bool isDefault) : base (prop)
+		public DataGridTextBoxColumn (PropertyDescriptor prop,  bool isDefault) : this (prop, String.Empty, isDefault)
 		{
-			format = string.Empty;
-			is_default = isDefault;
 		}
 
-		public DataGridTextBoxColumn (PropertyDescriptor prop,  string format) : base (prop)
+		public DataGridTextBoxColumn (PropertyDescriptor prop,  string format) : this (prop, format, false)
 		{
-			this.format = format;
 		}
 		
 		public DataGridTextBoxColumn (PropertyDescriptor prop,  string format, bool isDefault) : base (prop)
 		{
-			this.format = format;
+			Format = format;
 			is_default = isDefault;
+
+			textbox = new DataGridTextBox ();
+			textbox.Multiline = true;
+			textbox.WordWrap = false;
+			textbox.BorderStyle = BorderStyle.None;
+			textbox.Visible = false;
 		}
 
 		#endregion
@@ -77,12 +78,11 @@ namespace System.Windows.Forms
 		#region Public Instance Properties
 		[Editor("System.Windows.Forms.Design.DataGridColumnStyleFormatEditor, " + Consts.AssemblySystem_Design, typeof(System.Drawing.Design.UITypeEditor))]
 		public string Format {
-			get {
-				return format;
-			}
+			get { return format; }
 			set {
 				if (value != format) {
 					format = value;
+					Invalidate ();
 				}
 			}
 		}
@@ -90,9 +90,7 @@ namespace System.Windows.Forms
 		[Browsable(false)]
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		public IFormatProvider FormatInfo {
-			get {
-				return format_provider;
-			}
+			get { return format_provider; }
 			set {
 				if (value != format_provider) {
 					format_provider = value;
@@ -102,32 +100,22 @@ namespace System.Windows.Forms
 
 		[DefaultValue(null)]
 		public override PropertyDescriptor PropertyDescriptor {
-			set {
-				base.PropertyDescriptor = value;
-			}
+			set { base.PropertyDescriptor = value; }
 		}
 
 		public override bool ReadOnly {
-			get {
-				return base.ReadOnly;
-			}
-			set {
-				base.ReadOnly = value;
-			}
+			get { return base.ReadOnly; }
+			set { base.ReadOnly = value; }
 		}
 		
 		[Browsable(false)]
 		public virtual TextBox TextBox {
-			get {
-				EnsureTextBox();
-				return textbox;
-			}
+			get { return textbox; }
 		}
 		#endregion	// Public Instance Properties
 
 		#region Public Instance Methods
 
-
 		protected internal override void Abort (int rowNum)
 		{
 			EndEdit ();			
@@ -135,24 +123,26 @@ namespace System.Windows.Forms
 		
 		protected internal override bool Commit (CurrencyManager dataSource, int rowNum)
 		{
-			string text;
-			
-			if (textbox.Text == NullText) {
-				text = string.Empty;
-			} else {
-				text = textbox.Text;
-			}
-			
+			/* Do not write data if not editing. */
+			if (textbox.IsInEditOrNavigateMode)
+				return true;
+
 			try {
-				SetColumnValueAtRow (dataSource, rowNum, text);
+				object obj = GetColumnValueAtRow (dataSource, rowNum);
+				string existing_text = GetFormattedString (obj);
+
+				if (existing_text != textbox.Text) {
+					if (textbox.Text == NullText)
+						SetColumnValueAtRow (dataSource, rowNum, DBNull.Value);
+					else
+						SetColumnValueAtRow (dataSource, rowNum, textbox.Text);
+				}
 			}
-			
 			catch (Exception e) {
-				string message = "The data entered in column ["+ MappingName +"] has an invalid format.";
-				MessageBox.Show( message);
+				Console.WriteLine ("exception!?!?!?! {0}", e);
+				return false;
 			}
 			
-			
 			EndEdit ();			
 			return true;
 		}
@@ -160,14 +150,12 @@ namespace System.Windows.Forms
 		[MonoTODO]
 		protected internal override void ConcedeFocus ()
 		{
-
+			HideEditBox ();
 		}
 
 		protected internal override void Edit (CurrencyManager source, int rowNum,  Rectangle bounds,  bool _ro, string instantText, bool cellIsVisible)
 		{
-			object obj;
-			
-			EnsureTextBox();
+			grid.SuspendLayout ();
 
 			textbox.TextAlign = alignment;
 			
@@ -179,28 +167,35 @@ namespace System.Windows.Forms
 				textbox.ReadOnly = false;
 			}			
 			
+			if (instantText != null && instantText != "") {
+				textbox.Text = instantText;
+			}
+			else {
+				object obj = GetColumnValueAtRow (source, rowNum);
+				if (obj == null)
+					textbox.Text = "";
+				else
+					textbox.Text = GetFormattedString (obj);
+			}
+
 			textbox.Location = new Point (bounds.X + offset_x, bounds.Y + offset_y);
 			textbox.Size = new Size (bounds.Width - offset_x, bounds.Height - offset_y);
 
-			obj = GetColumnValueAtRow (source, rowNum);
-			textbox.Text = GetFormattedString (obj);
-
+			textbox.IsInEditOrNavigateMode = true;
 			textbox.Visible = cellIsVisible;
 			textbox.Focus ();
-			textbox.SelectAll ();			
+			textbox.SelectAll ();
+			grid.ResumeLayout (false);
 		}
 
 		protected void EndEdit ()
 		{
-			ReleaseHostedControl ();
-			grid.Invalidate (grid.GetCurrentCellBounds ());
+			HideEditBox ();
 		}
 
 		protected internal override void EnterNullValue ()
 		{
-			if (textbox != null) {
-				textbox.Text = NullText;
-			}
+			textbox.Text = NullText;
 		}
 
 		protected internal override int GetMinimumHeight ()
@@ -223,7 +218,11 @@ namespace System.Windows.Forms
 		[MonoTODO]
 		protected void HideEditBox ()
 		{
-
+			grid.SuspendLayout ();
+			textbox.Bounds = Rectangle.Empty;
+			textbox.Visible = false;
+			textbox.IsInEditOrNavigateMode = true;
+			grid.ResumeLayout (false);
 		}
 
 		protected internal override void Paint (Graphics g, Rectangle bounds, CurrencyManager source, int rowNum)
@@ -285,15 +284,25 @@ namespace System.Windows.Forms
 		protected internal override void ReleaseHostedControl ()
 		{			
 			if (textbox != null) {
-				DataGridTableStyle.DataGrid.Controls.Remove (textbox);
+				grid.SuspendLayout ();
+				grid.Controls.Remove (textbox);
+				grid.Invalidate (new Rectangle (textbox.Location, textbox.Size));
 				textbox.Dispose ();
 				textbox = null;
+				grid.ResumeLayout (false);
 			}
 		}
 
 		protected override void SetDataGridInColumn (DataGrid value)
 		{
 			base.SetDataGridInColumn (value);
+
+			if (value != null) {
+				textbox.SetDataGrid (grid);			
+				grid.SuspendLayout ();
+				grid.Controls.Add (textbox);
+				grid.ResumeLayout (false);
+			}
 		}
 		
 		protected internal override void UpdateUI (CurrencyManager source, int rowNum, string instantText)
@@ -306,17 +315,13 @@ namespace System.Windows.Forms
 
 		#region Private Instance Methods
 
-		// We use DataGridTextBox to render everything that DataGridBoolColumn does not
-		internal static bool CanRenderType (Type type)
-		{
-			return (type != typeof (Boolean));
-		}
-
 		private string GetFormattedString (object obj)
 		{
-			if (obj == DBNull.Value) {
+			if (obj == null)
+				return "";
+
+			if (obj == DBNull.Value)
 				return NullText;
-			}
 			
 			if (format != null && obj as IFormattable != null) {
 				return ((IFormattable)obj).ToString (format, format_provider);
@@ -325,18 +330,6 @@ namespace System.Windows.Forms
 			return obj.ToString ();
 
 		}
-
-		private void EnsureTextBox() {
-			if (textbox == null) {
-				textbox = new DataGridTextBox ();
-				textbox.SetDataGrid (DataGridTableStyle.DataGrid);
-				textbox.Multiline = true;
-				textbox.BorderStyle = BorderStyle.None;
-				textbox.Visible = false;
-
-				DataGridTableStyle.DataGrid.Controls.Add (textbox);
-			}			
-		}
 		#endregion Private Instance Methods
 	}
 }
