@@ -78,8 +78,6 @@ namespace System.Windows.Forms {
 		private int firstDisplayedScrollingColumnHiddenWidth;
 		private int firstDisplayedScrollingColumnIndex;
 		private int firstDisplayedScrollingRowIndex;
-		private Font font = Control.DefaultFont;
-		private Color foreColor = Control.DefaultForeColor;
 		private Color gridColor = Color.FromKnownColor(KnownColor.ControlDarkDark);
 		private int horizontalScrollingOffset;
 		private bool isCurrentCellDirty;
@@ -614,23 +612,13 @@ namespace System.Windows.Forms {
 		}
 
 		public override Font Font {
-			get { return font; }
-			set {
-				if (font != value) {
-					font = value;
-					OnFontChanged(EventArgs.Empty);
-				}
-			}
+			get { return base.Font; }
+			set { base.Font = value; }
 		}
 
 		public override Color ForeColor {
-			get { return foreColor; }
-			set {
-				if (foreColor != value) {
-					foreColor = value;
-					OnForeColorChanged(EventArgs.Empty);
-				}
-			}
+			get { return base.ForeColor; }
+			set { base.ForeColor = value; }
 		}
 
 		public Color GridColor {
@@ -995,11 +983,17 @@ namespace System.Windows.Forms {
 
 		public event DataGridViewAutoSizeModeEventHandler AutoSizeRowsModeChanged;
 
-		public new event EventHandler BackColorChanged;
+		public new event EventHandler BackColorChanged {
+			add { base.BackColorChanged += value; }
+			remove { base.BackColorChanged -= value; }
+		}
 
 		public event EventHandler BackgroundColorChanged;
 
-		public new event EventHandler BackgroundImageChanged;
+		public new event EventHandler BackgroundImageChanged {
+			add { base.BackgroundImageChanged += value; }
+			remove { base.BackgroundImageChanged -= value; }
+		}
 
 		public new event EventHandler BackgroundImageLayoutChanged;
 
@@ -1135,9 +1129,15 @@ namespace System.Windows.Forms {
 
 		public event EventHandler EditModeChanged;
 
-		public new event EventHandler FontChanged;
+		public new event EventHandler FontChanged {
+			add { base.FontChanged += value; }
+			remove { base.FontChanged -= value; }
+		}
 
-		public new event EventHandler ForeColorChanged;
+		public new event EventHandler ForeColorChanged {
+			add { base.ForeColorChanged += value; }
+			remove { base.ForeColorChanged -= value; }
+		}
 
 		public event EventHandler GridColorChanged;
 
@@ -1221,9 +1221,15 @@ namespace System.Windows.Forms {
 
 		public event DataGridViewRowCancelEventHandler UserDeletingRow;
 
-		public new event EventHandler StyleChanged;
+		public new event EventHandler StyleChanged {
+			add { base.StyleChanged += value; }
+			remove { base.StyleChanged -= value; }
+		}
 
-		public new event EventHandler TextChanged;
+		public new event EventHandler TextChanged {
+			add { base.TextChanged += value; }
+			remove { base.TextChanged -= value; }
+		}
 
 		public virtual DataGridViewAdvancedBorderStyle AdjustColumnHeaderBorderStyle (DataGridViewAdvancedBorderStyle dataGridViewAdvancedBorderStyleInput, DataGridViewAdvancedBorderStyle dataGridViewAdvancedBorderStylePlaceholder, bool isFirstDisplayedColumn, bool isLastVisibleColumn) {
 			return (DataGridViewAdvancedBorderStyle) dataGridViewAdvancedBorderStyleInput.Clone();
@@ -2149,16 +2155,10 @@ namespace System.Windows.Forms {
 
 		protected override void OnFontChanged (EventArgs e) {
 			base.OnFontChanged(e);
-			if (FontChanged != null) {
-				FontChanged(this, e);
-			}
 		}
 
 		protected override void OnForeColorChanged (EventArgs e) {
 			base.OnForeColorChanged(e);
-			if (ForeColorChanged != null) {
-				ForeColorChanged(this, e);
-			}
 		}
 
 		protected virtual void OnGridColorChanged (EventArgs e) {
