@@ -65,7 +65,7 @@ namespace System.Windows.Forms {
 		private Form			owner;
 		private Form.ControlCollection	owned_forms;
 		private MdiClient		mdi_container;
-		private InternalWindowManager	window_manager;
+		internal InternalWindowManager	window_manager;
 		private Form			mdi_parent;
 		private bool			key_preview;
 		private MainMenu		menu;
@@ -87,6 +87,15 @@ namespace System.Windows.Forms {
 		{
 			default_icon = (Icon)Locale.GetResource("mono.ico");
 		}
+
+		// warning: this is only hooked up when an mdi container is created.
+		private void ControlAddedHandler (object sender, ControlEventArgs e)
+		{
+			if (mdi_container != null) {
+				Console.WriteLine ("SENDING TO BACK");
+				mdi_container.SendToBack ();
+			}
+		}
 		#endregion	// Private & Internal Methods
 
 		#region Public Classes
@@ -198,11 +207,17 @@ namespace System.Windows.Forms {
 			}
 
 			set {
+				if (value == allow_transparency) {
+					return;
+				}
+
 				if (XplatUI.SupportsTransparency()) {
 					allow_transparency = value;
 
 					if (value) {
-						XplatUI.SetWindowTransparency(Handle, Opacity, TransparencyKey);
+						if (IsHandleCreated) {
+							XplatUI.SetWindowTransparency(Handle, Opacity, TransparencyKey);
+						}
 					} else {
 						UpdateStyles(); // Remove the WS_EX_LAYERED style
 					}
@@ -211,6 +226,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(true)]
+		[MWFCategory("Layout")]
 		public bool AutoScale {
 			get {
 				return autoscale;
@@ -277,6 +293,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(true)]
+		[MWFCategory("Window Style")]
 		public bool ControlBox {
 			get {
 				return control_box;
@@ -322,6 +339,10 @@ namespace System.Windows.Forms {
 			}
 
 			set {
+				if (value < DialogResult.None || value > DialogResult.No)
+					throw new InvalidEnumArgumentException ("value", (int) value, 
+							typeof (DialogResult));
+
 				dialog_result = value;
 				if (is_modal) {
 					closing = true;
@@ -331,6 +352,7 @@ namespace System.Windows.Forms {
 
 		[DefaultValue(FormBorderStyle.Sizable)]
 		[DispId(-504)]
+		[MWFCategory("Appearance")]
 		public FormBorderStyle FormBorderStyle {
 			get {
 				return form_border_style;
@@ -342,10 +364,6 @@ namespace System.Windows.Forms {
 					if (IsHandleCreated) {
 						XplatUI.SetBorderStyle(window.Handle, form_border_style);
 					}
-
-					if (value == FormBorderStyle.FixedToolWindow ||
-							value == FormBorderStyle.SizableToolWindow)
-						window_manager = new InternalWindowManager (this);
 				} else {
 					window_manager.UpdateBorderStyle (value);
 				}
@@ -355,6 +373,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(false)]
+		[MWFCategory("Window Style")]
 		public bool HelpButton {
 			get {
 				return help_button;
@@ -370,6 +389,7 @@ namespace System.Windows.Forms {
 
 		[Localizable(true)]
 		[AmbientValue(null)]
+		[MWFCategory("Window Style")]
 		public Icon Icon {
 			get {
 				return icon;
@@ -395,6 +415,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(false)]
+		[MWFCategory("Window Style")]
 		public bool IsMdiContainer {
 			get {
 				return mdi_container != null;
@@ -402,8 +423,10 @@ namespace System.Windows.Forms {
 
 			set {
 				if (value && mdi_container == null) {
-					mdi_container = new MdiClient();
+					mdi_container = new MdiClient ();
 					Controls.Add(mdi_container);
+					ControlAdded += new ControlEventHandler (ControlAddedHandler);
+					mdi_container.SendToBack ();
 				} else if (!value && mdi_container != null) {
 					Controls.Remove(mdi_container);
 					mdi_container = null;
@@ -441,6 +464,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(true)]
+		[MWFCategory("Window Style")]
 		public bool MaximizeBox {
 			get {
 				return maximize_box;
@@ -459,6 +483,7 @@ namespace System.Windows.Forms {
 		[DefaultValue("{Width=0, Height=0}")]
 		[Localizable(true)]
 		[RefreshProperties(RefreshProperties.Repaint)]
+		[MWFCategory("Layout")]
 		public Size MaximumSize {
 			get {
 				return maximum_size;
@@ -479,17 +504,10 @@ namespace System.Windows.Forms {
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public Form[] MdiChildren {
 			get {
-				if (mdi_container != null) {
-					Form[] form_list;
-
-					form_list = new Form[mdi_container.Controls.Count];
-					for (int i = 0; i < mdi_container.Controls.Count; i++) {
-						form_list[i] = (Form)mdi_container.Controls[i];
-					}
-					return form_list;
-				} else {
+				if (mdi_container != null)
+					return mdi_container.MdiChildren;
+				else
 					return new Form[0];
-				}
 			}
 		}
 
@@ -501,26 +519,30 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				SuspendLayout ();
-
-				// TopLevel = true;
-
-				if (!value.IsMdiContainer)
+				if (value != null && !value.IsMdiContainer)
 					throw new ArgumentException ();
 
 				if (mdi_parent != null) {
 					mdi_parent.MdiContainer.Controls.Remove (this);
 				}
 
-				mdi_parent = value;
-				if (mdi_parent != null) {
+				if (value != null) {
+					mdi_parent = value;
 					window_manager = new MdiWindowManager (this,
 							mdi_parent.MdiContainer);
 					mdi_parent.MdiContainer.Controls.Add (this);
-					UpdateStyles ();
-				}
 
-				ResumeLayout ();
+					RecreateHandle ();
+
+				} else if (mdi_parent != null) {
+					mdi_parent = null;
+
+					// Create a new window manager
+					window_manager = null;
+					FormBorderStyle = form_border_style;
+
+					RecreateHandle ();
+				}
 			}
 		}
 
@@ -544,6 +566,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(null)]
+		[MWFCategory("Window Style")]
 		public MainMenu Menu {
 			get {
 				return menu;
@@ -615,6 +638,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(true)]
+		[MWFCategory("Window Style")]
 		public bool MinimizeBox {
 			get {
 				return minimize_box;
@@ -633,6 +657,7 @@ namespace System.Windows.Forms {
 		[DefaultValue("{Width=0, Height=0}")]
 		[Localizable(true)]
 		[RefreshProperties(RefreshProperties.Repaint)]
+		[MWFCategory("Layout")]
 		public Size MinimumSize {
 			get {
 				return minimum_size;
@@ -641,6 +666,12 @@ namespace System.Windows.Forms {
 			set {
 				if (minimum_size != value) {
 					minimum_size = value;
+
+					if ((Size.Width < value.Width) || (Size.Height < value.Height)) {
+						Size = new Size(Math.Max(Size.Width, value.Width), Math.Max(Size.Height, value.Height));
+					}
+  
+
 					OnMinimumSizeChanged(EventArgs.Empty);
 					if (IsHandleCreated) {
 						XplatUI.SetWindowMinMax(Handle, maximized_bounds, minimum_size, maximum_size);
@@ -659,6 +690,7 @@ namespace System.Windows.Forms {
 
 		[DefaultValue(1D)]
 		[TypeConverter(typeof(OpacityConverter))]
+		[MWFCategory("Window Style")]
 		public double Opacity {
 			get {
 				return opacity;
@@ -668,8 +700,11 @@ namespace System.Windows.Forms {
 				opacity = value;
 
 				AllowTransparency = true;
-				UpdateStyles();
-				XplatUI.SetWindowTransparency(Handle, opacity, TransparencyKey);
+
+				if (IsHandleCreated) {
+					UpdateStyles();
+					XplatUI.SetWindowTransparency(Handle, opacity, TransparencyKey);
+				}
 			}
 		}
 			
@@ -716,6 +751,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(true)]
+		[MWFCategory("Window Style")]
 		public bool ShowInTaskbar {
 			get {
 				return show_in_taskbar;
@@ -745,6 +781,7 @@ namespace System.Windows.Forms {
 
 		[MonoTODO("Trigger something when GripStyle is set")]
 		[DefaultValue(SizeGripStyle.Auto)]
+		[MWFCategory("Window Style")]
 		public SizeGripStyle SizeGripStyle {
 			get {
 				return size_grip_style;
@@ -757,6 +794,7 @@ namespace System.Windows.Forms {
 
 		[DefaultValue(FormStartPosition.WindowsDefaultLocation)]
 		[Localizable(true)]
+		[MWFCategory("Layout")]
 		public FormStartPosition StartPosition {
 			get {
 				return start_position;
@@ -821,6 +859,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(false)]
+		[MWFCategory("Window Style")]
 		public bool TopMost {
 			get {
 				return topmost;
@@ -835,6 +874,7 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		[MWFCategory("Window Style")]
 		public Color TransparencyKey {
 			get {
 				return transparency_key;
@@ -850,6 +890,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(FormWindowState.Normal)]
+		[MWFCategory("Layout")]
 		public FormWindowState WindowState {
 			get {
 				if (IsHandleCreated) {
@@ -869,11 +910,12 @@ namespace System.Windows.Forms {
 			}
 
 			set {
+				FormWindowState old_state = window_state;
 				window_state = value;
 				if (IsHandleCreated) {
 
 					if (window_manager != null) {
-						window_manager.SetWindowState (window_state);
+						window_manager.SetWindowState (old_state, value);
 						return;
 					}
 
@@ -903,7 +945,7 @@ namespace System.Windows.Forms {
 				cp.Parent = IntPtr.Zero;
 				cp.menu = ActiveMenu;
 
-				if (start_position == FormStartPosition.WindowsDefaultLocation) {
+				if (start_position == FormStartPosition.WindowsDefaultLocation && !IsMdiChild) {
 					cp.X = unchecked((int)0x80000000);
 					cp.Y = unchecked((int)0x80000000);
 				} else {
@@ -917,9 +959,12 @@ namespace System.Windows.Forms {
 
 				if (IsMdiChild) {
 					cp.Style |= (int)(WindowStyles.WS_CHILD | WindowStyles.WS_CAPTION);
-					cp.Parent = Parent.Handle;
+					if (Parent != null) {
+						cp.Parent = Parent.Handle;
+					}
 
 					cp.ExStyle |= (int) (WindowExStyles.WS_EX_WINDOWEDGE | WindowExStyles.WS_EX_MDICHILD);
+
 					switch (FormBorderStyle) {
 					case FormBorderStyle.None:
 						break;
@@ -1033,7 +1078,7 @@ namespace System.Windows.Forms {
 
 		protected override Size DefaultSize {
 			get {
-				return new Size (250, 250);
+				return new Size (300, 300);
 			}
 		}
 
@@ -1094,7 +1139,10 @@ namespace System.Windows.Forms {
 		}
 
 		public void AddOwnedForm(Form ownedForm) {
-			owned_forms.Add(ownedForm);
+			if (!owned_forms.Contains(ownedForm)) {
+				owned_forms.Add(ownedForm);
+			}
+			ownedForm.Owner = this;
 		}
 
 		public void Close () {
@@ -1127,6 +1175,9 @@ namespace System.Windows.Forms {
 
 		public DialogResult ShowDialog(IWin32Window ownerWin32) {
 			Form		previous;
+			Rectangle	area;
+			bool		confined;
+			IntPtr		capture_window;
 
 			owner = null;
 
@@ -1161,11 +1212,24 @@ namespace System.Windows.Forms {
 			form_parent_window.Parent = this.owner;
 			#endif
 
+			// Release any captures
+			XplatUI.GrabInfo(out capture_window, out confined, out area);
+			if (capture_window != IntPtr.Zero) {
+				XplatUI.UngrabWindow(capture_window);
+			}
+
 			previous = Form.ActiveForm;
 
+#if not
+			// Commented out; we instead let the Visible=true inside the runloop create the control
+			// otherwise setting DialogResult inside any of the events that are triggered by the
+			// create will not actually cause the form to not be displayed.
+			// Leaving this comment here in case there was an actual purpose to creating the control
+			// in here.
 			if (!IsHandleCreated) {
 				CreateControl();
 			}
+#endif
 
 			Application.RunLoop(true, new ApplicationContext(this));
 
@@ -1174,7 +1238,11 @@ namespace System.Windows.Forms {
 				XplatUI.Activate(previous.window.Handle);
 			}
 
-			return DialogResult;
+			if (DialogResult != DialogResult.None) {
+				return DialogResult;
+			}
+			DialogResult = DialogResult.Cancel;
+			return DialogResult.Cancel;
 		}
 
 		public override string ToString() {
@@ -1200,6 +1268,8 @@ namespace System.Windows.Forms {
 		{
 			SizeF current_size_f = GetAutoScaleSize (DeviceContext, Font);
 			Size current_size = new Size ((int) current_size_f.Width, (int) current_size_f.Height);
+			float	dx;
+			float	dy;
 
 			if (current_size == autoscale_base_size)
 				return;
@@ -1214,8 +1284,19 @@ namespace System.Windows.Forms {
 			// http://blogs.msdn.com/mharsh/archive/2004/01/25/62621.aspx
 			// but it makes things larger without looking better.
 			//
-			Scale (current_size_f.Width / AutoScaleBaseSize.Width + 0.08f,
-			       current_size_f.Height / AutoScaleBaseSize.Height + 0.08f);
+			if (current_size_f.Width != AutoScaleBaseSize.Width) {
+				dx = current_size_f.Width / AutoScaleBaseSize.Width + 0.08f;
+			} else {
+				dx = 1;
+			}
+
+			if (current_size_f.Height != AutoScaleBaseSize.Height) {
+				dy = current_size_f.Height / AutoScaleBaseSize.Height + 0.08f;
+			} else {
+				dy = 1;
+			}
+
+			Scale (dx, dy);
 			
 			AutoScaleBaseSize = current_size;
 		}
@@ -1279,10 +1360,29 @@ namespace System.Windows.Forms {
 		protected override void CreateHandle() {
 			base.CreateHandle ();
 
+			if (XplatUI.SupportsTransparency()) {
+				if (allow_transparency) {
+					XplatUI.SetWindowTransparency(Handle, Opacity, TransparencyKey);
+				}
+			}
+
 			XplatUI.SetWindowMinMax(window.Handle, maximized_bounds, minimum_size, maximum_size);
 			if (icon != null) {
 				XplatUI.SetIcon(window.Handle, icon);
 			}
+
+			if ((owner != null) && (owner.IsHandleCreated)) {
+				XplatUI.SetTopmost(window.Handle, owner.window.Handle, true);
+			}
+
+			if (owned_forms.Count > 0) {
+				for (int i = 0; i < owned_forms.Count; i++) {
+					if (owned_forms[i].IsHandleCreated) {
+						XplatUI.SetTopmost(owned_forms[i].window.Handle, window.Handle, true);
+					}
+				}
+			}
+
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1343,6 +1443,10 @@ namespace System.Windows.Forms {
 
 			// Send initial location
 			OnLocationChanged(EventArgs.Empty);
+
+			if (IsMdiContainer) {
+				mdi_container.LayoutMdi (MdiLayout.Cascade);
+			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1384,15 +1488,15 @@ namespace System.Windows.Forms {
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected virtual void OnLoad(EventArgs e) {
-			if (Load != null) {
-				Load(this, e);
-			}
-
 			if (AutoScale){
 				ApplyAutoScaling ();
 				AutoScale = false;
 			}
 
+			if (Load != null) {
+				Load(this, e);
+			}
+
 			if (!IsMdiChild) {
 				switch (StartPosition) {
 					case FormStartPosition.CenterScreen:
@@ -1406,12 +1510,7 @@ namespace System.Windows.Forms {
 						Top = CreateParams.Y;
 						break;
 				}
-			} else {
-				Left = 25 * MdiParent.MdiContainer.ChildrenCreated + 1;
-				Top = 25 * MdiParent.MdiContainer.ChildrenCreated + 1;
-				MdiParent.MdiContainer.ChildrenCreated++;
 			}
-
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1528,31 +1627,35 @@ namespace System.Windows.Forms {
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void ScaleCore(float dx, float dy) {
-			// We can't scale max or min windows
-			if (WindowState == FormWindowState.Normal) {
-				// We cannot call base since base also adjusts X/Y, but
-				// a form is toplevel and doesn't move
-				Size	size;
-
+			try {
 				SuspendLayout();
 
-				size = ClientSize;
-				if (!GetStyle(ControlStyles.FixedWidth)) {
-					size.Width = (int)(size.Width * dx);
-				}
+				// We can't scale max or min windows
+				if (WindowState == FormWindowState.Normal) {
+					// We cannot call base since base also adjusts X/Y, but
+					// a form is toplevel and doesn't move
+					Size	size;
 
-				if (!GetStyle(ControlStyles.FixedHeight)) {
-					size.Height = (int)(size.Height * dy);
-				}
+					size = ClientSize;
+					if (!GetStyle(ControlStyles.FixedWidth)) {
+						size.Width = (int)(size.Width * dx);
+					}
 
-				ClientSize = size;
+					if (!GetStyle(ControlStyles.FixedHeight)) {
+						size.Height = (int)(size.Height * dy);
+					}
+
+					ClientSize = size;
+				}
 
 				/* Now scale our children */
 				Control [] controls = child_controls.GetAllControls ();
 				for (int i=0; i < controls.Length; i++) {
 					controls[i].Scale(dx, dy);
 				}
-				
+			}
+
+			finally {
 				ResumeLayout();
 			}
 		}
@@ -1641,17 +1744,35 @@ namespace System.Windows.Forms {
 				}
 
 				case Msg.WM_CLOSE: {
-					if (!is_modal) {
-						CancelEventArgs args = new CancelEventArgs ();
+					Form act = Form.ActiveForm;
+					if (act != null && act != this && act.Modal == true) {
+						return;
+					}
 
+					CancelEventArgs args = new CancelEventArgs ();
+
+					if (mdi_container != null) {
+						foreach (Form mdi_child in mdi_container.MdiChildren) {
+							mdi_child.OnClosing (args);
+						}
+					}
+
+					if (!is_modal) {
 						OnClosing (args);
 						if (!args.Cancel) {
 							OnClosed (EventArgs.Empty);
-							DestroyHandle();
+							base.Dispose();
 						}
 						return;
 					} else {
-						closing = true;
+						OnClosing (args);
+						if (!args.Cancel) {
+							OnClosed (EventArgs.Empty);
+							closing = true;
+						} else {
+							DialogResult = DialogResult.None;
+							closing = false;
+						}
 					}
 					return;
 				}
@@ -1689,7 +1810,7 @@ namespace System.Windows.Forms {
 				// Menu drawing
 				case Msg.WM_NCLBUTTONDOWN: {
 					if (native_enabled && ActiveMenu != null) {
-						ActiveMenu.OnMouseDown(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0));
+						ActiveMenu.OnMouseDown(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, Control.MousePosition.X, Control.MousePosition.Y, 0));
 					}
 
 					if (ActiveMaximizedMdiChild != null) {
@@ -1720,7 +1841,7 @@ namespace System.Windows.Forms {
 						ActiveMenu.Draw (pe, new Rectangle (pnt.X, pnt.Y, ClientSize.Width, 0));
 
 						if (ActiveMaximizedMdiChild != null) {
-							ActiveMaximizedMdiChild.DrawMaximizedButtons (pe, ActiveMenu);
+							ActiveMaximizedMdiChild.DrawMaximizedButtons (ActiveMenu, pe);
 						}
 
 						XplatUI.PaintEventEnd(Handle, false);
@@ -1765,7 +1886,7 @@ namespace System.Windows.Forms {
 
 						args = new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
 							mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0);
-						active_tracker.OnClick(new MouseEventArgs (args.Button, args.Clicks, Control.MousePosition.X, Control.MousePosition.Y, args.Delta));
+						active_tracker.OnMouseDown(new MouseEventArgs (args.Button, args.Clicks, Control.MousePosition.X, Control.MousePosition.Y, args.Delta));
 						return;
 					}
 					base.WndProc(ref m);
@@ -1807,7 +1928,7 @@ namespace System.Windows.Forms {
 							mmi.ptMinTrackSize.y = minimum_size.Height;
 						}
 
-						if (minimum_size != Size.Empty) {
+						if (maximum_size != Size.Empty) {
 							mmi.ptMaxTrackSize.x = maximum_size.Width;
 							mmi.ptMaxTrackSize.y = maximum_size.Height;
 						}
