@@ -58,7 +58,7 @@ namespace System.Windows.Forms {
 		internal uint		opacity;
 		internal bool		enabled;
 		internal bool		zero_sized;
-		internal Rectangle	invalid;
+		internal ArrayList	invalid_list;
 		internal Rectangle	nc_invalid;
 		internal bool		expose_pending;
 		internal bool		nc_expose_pending;
@@ -73,7 +73,9 @@ namespace System.Windows.Forms {
 		internal int		tool_caption_height;
 		internal bool		whacky_wm;
 		internal bool		fixed_size;
+		internal Region		user_clip;
 		internal static Bitmap	bmp = new Bitmap(1, 1, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
+		internal XEventQueue	queue;
 		#endregion	// Local Variables
 
 		#region Constructors and destructors
@@ -89,7 +91,7 @@ namespace System.Windows.Forms {
 			whole_window = IntPtr.Zero;
 			handle = IntPtr.Zero;
 			parent = null;
-			invalid = Rectangle.Empty;
+			invalid_list = new ArrayList();
 			expose_pending = false;
 			nc_expose_pending = false;
 			enabled = true;
@@ -101,6 +103,8 @@ namespace System.Windows.Forms {
 		}
 
 		public void Dispose() {
+			expose_pending = false;
+			nc_expose_pending = false;
 			windows.Remove(client_window);
 			windows.Remove(whole_window);
 			for (int i = 0; i < marshal_free_list.Count; i++) {
@@ -265,6 +269,16 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		public Region UserClip {
+			get {
+				return user_clip;
+			}
+
+			set {
+				user_clip = value;
+			}
+		}
+
 		public Rectangle DefaultClientRect {
 			get {
 				// We pass a Zero for the menu handle so the menu size is
@@ -333,6 +347,16 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		public XEventQueue Queue {
+			get {
+				return queue;
+			}
+
+			set {
+				queue = value;
+			}
+		}
+
 		public bool Enabled {
 			get {
 				if (!enabled) {
@@ -388,14 +412,23 @@ namespace System.Windows.Forms {
 
 		public Rectangle Invalid {
 			get {
-				return invalid;
-			}
+				if (invalid_list.Count == 0)
+					return Rectangle.Empty;
 
-			set {
-				invalid = value;
+				Rectangle result = (Rectangle)invalid_list[0];
+				for (int i = 1; i < invalid_list.Count; i ++) {
+					result = Rectangle.Union (result, (Rectangle)invalid_list[i]);
+				}
+				return result;
 			}
 		}
 
+		public Rectangle[] ClipRectangles {
+			get {
+				return (Rectangle[]) invalid_list.ToArray (typeof (Rectangle));
+ 			}
+ 		}
+
 		public Rectangle NCInvalid {
 			get { return nc_invalid; }
 			set { nc_invalid = value; }
@@ -429,7 +462,7 @@ namespace System.Windows.Forms {
 				}
 
 				if (parent != null) {
-					return parent.mapped;
+					return parent.Mapped;
 				}
 
 				return true;
@@ -528,31 +561,22 @@ namespace System.Windows.Forms {
 
 		#region Methods
 		public void AddInvalidArea(int x, int y, int width, int height) {
-			if (invalid == Rectangle.Empty) {
-				invalid = new Rectangle (x, y, width, height);
-				return;
-			}
-
-			int right, bottom;
-			right = Math.Max (invalid.Right, x + width);
-			bottom = Math.Max (invalid.Bottom, y + height);
-			invalid.X = Math.Min (invalid.X, x);
-			invalid.Y = Math.Min (invalid.Y, y);
-
-			invalid.Width = right - invalid.X;
-			invalid.Height = bottom - invalid.Y;
+			AddInvalidArea(new Rectangle(x, y, width, height));
 		}
 
 		public void AddInvalidArea(Rectangle rect) {
-			if (invalid == Rectangle.Empty) {
-				invalid = rect;
-				return;
+			ArrayList tmp = new ArrayList ();
+			foreach (Rectangle r in invalid_list) {
+				if (!rect.Contains (r)) {
+					tmp.Add (r);
+				}
 			}
-			invalid = Rectangle.Union (invalid, rect);
+			tmp.Add (rect);
+			invalid_list = tmp;
 		}
 
 		public void ClearInvalidArea() {
-			invalid = Rectangle.Empty;
+			invalid_list.Clear();
 			expose_pending = false;
 		}
 
