@@ -36,30 +36,29 @@ namespace System.Windows.Forms {
 
 		private static Color titlebar_color;
 
-		internal int BorderWidth = 3;
 		private Size MinTitleBarSize = new Size (115, 25);
 
 		internal Form form;
 
-		private TitleButton close_button;
-		private TitleButton maximize_button;
-		private TitleButton minimize_button;
-
+		internal TitleButton close_button;
+		internal TitleButton maximize_button;
+		internal TitleButton minimize_button;
+		protected Rectangle icon_rect;
+		
 		private TitleButton [] title_buttons = new TitleButton [3];
 		
 		// moving windows
 		internal Point start;
-		private State state;
+		internal State state;
 		private FormPos sizing_edge;
 		internal Rectangle virtual_position;
-		private Rectangle prev_virtual_position;
 
-		private class TitleButton {
+		public class TitleButton {
 			public Rectangle Rectangle;
 			public ButtonState State;
 			public CaptionButton Caption;
 			public EventHandler Clicked;
-			
+
 			public TitleButton (CaptionButton caption, EventHandler clicked)
 			{
 				Caption = caption;
@@ -67,14 +66,14 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		private enum State {
+		public enum State {
 			Idle,
 			Moving,
 			Sizing,
 		}
 
 		[Flags]
-		private enum FormPos {
+		public enum FormPos {
 			None,
 
 			TitleBar = 1,
@@ -98,6 +97,8 @@ namespace System.Windows.Forms {
 			titlebar_color = Color.FromArgb (255, 0, 0, 255);
 			this.form = form;
 
+			form.SizeChanged += new EventHandler (FormSizeChangedHandler);
+
 			CreateButtons ();
 		}
 
@@ -105,6 +106,26 @@ namespace System.Windows.Forms {
 			get { return form; }
 		}
 
+		public Rectangle CloseButtonRect {
+			get { return close_button.Rectangle; }
+			set { close_button.Rectangle = value; }
+		}
+
+		public Rectangle MinimizeButtonRect {
+			get { return minimize_button.Rectangle; }
+			set { minimize_button.Rectangle = value; }
+		}
+
+		public Rectangle MaximizeButtonRect {
+			get { return maximize_button.Rectangle; }
+			set { maximize_button.Rectangle = value; }
+		}
+
+		public Rectangle IconRect {
+			get { return icon_rect; }
+			set { value = icon_rect; }
+		}
+
 		public int IconWidth {
 			get { return TitleBarHeight - 5; }
 		}
@@ -132,9 +153,8 @@ namespace System.Windows.Forms {
 			case Msg.WM_NCHITTEST:
 				int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
 				int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
-
+				
 				form.PointToClient (ref x, ref y);
-				y += TitleBarHeight;
 
 				FormPos pos = FormPosForCoords (x, y);
 				
@@ -177,15 +197,16 @@ namespace System.Windows.Forms {
 				}
 				return true;
 
-			case Msg.WM_NCMOUSEMOVE:
-				return HandleNCMouseMove (form, ref m);
-
 			case Msg.WM_NCLBUTTONUP:
 				return HandleNCLButtonUp (ref m);
 
 			case Msg.WM_NCLBUTTONDOWN:
 				return HandleNCLButtonDown (ref m);
 
+			case Msg.WM_NCLBUTTONDBLCLK:
+				HandleNCLButtonDblClick (ref m);
+				break;
+
 			case Msg.WM_MOUSE_LEAVE:
 				FormMouseLeave (ref m);
 				break;
@@ -197,7 +218,7 @@ namespace System.Windows.Forms {
 					ncp = (XplatUIWin32.NCCALCSIZE_PARAMS) Marshal.PtrToStructure (m.LParam,
 							typeof (XplatUIWin32.NCCALCSIZE_PARAMS));
 
-					int bw = BorderWidth;
+					int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
 
 					if (HasBorders) {
 						ncp.rgrc1.top += TitleBarHeight + bw;
@@ -212,15 +233,7 @@ namespace System.Windows.Forms {
 				break;
 
 			case Msg.WM_NCPAINT:
-				PaintEventArgs pe;
-
-				pe = XplatUI.PaintEventStart(m.HWnd, false);
-				PaintWindowDecorations (pe);
-				XplatUI.PaintEventEnd(m.HWnd, false);
-
-				// We don't want the form.WndProc to handle this because it
-				// will call a PaintEventEnd
-				
+				PaintDecorations ();
 				return true;
 			}
 			return false;
@@ -251,9 +264,8 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		public virtual void SetWindowState (FormWindowState window_state)
+		public virtual void SetWindowState (FormWindowState old_state, FormWindowState window_state)
 		{
-			form.window_state = window_state;
 		}
 
 		public virtual FormWindowState GetWindowState ()
@@ -263,7 +275,19 @@ namespace System.Windows.Forms {
 
 		public virtual void PointToClient (ref int x, ref int y)
 		{
-			// toolwindows stay in screencoords
+			// toolwindows stay in screencoords we just have to make sure
+			// they obey the working area
+			Rectangle working = SystemInformation.WorkingArea;
+
+			if (x > working.Right)
+				x = working.Right;
+			if (x < working.Left)
+				x = working.Left;
+
+			if (y < working.Top)
+				y = working.Top;
+			if (y > working.Bottom)
+				y = working.Bottom;
 		}
 
 		public virtual void PointToScreen (ref int x, ref int y)
@@ -283,18 +307,48 @@ namespace System.Windows.Forms {
 			form.Refresh ();
 		}
 
-		protected virtual bool IsActive ()
+		public virtual bool IsActive ()
 		{
 			return true;
 		}
 
-		private void CreateButtons ()
+		private void FormSizeChangedHandler (object sender, EventArgs e)
+		{
+			ThemeEngine.Current.ManagedWindowSetButtonLocations (this);
+
+			PaintDecorations ();
+		}
+
+		public void PaintDecorations ()
+		{
+			PaintEventArgs pe = XplatUI.PaintEventStart (form.Handle, false);
+			ThemeEngine.Current.DrawManagedWindowDecorations (pe.Graphics, pe.ClipRectangle, this);
+
+			if (IsMaximized) {
+				MainMenu menu = form.ActiveMenu;
+				DrawMaximizedButtons (menu, pe);
+				if (menu != null)
+					menu.Draw (pe);
+			}
+
+			XplatUI.PaintEventEnd (form.Handle, false);
+		}
+
+		protected void CreateButtons ()
 		{
 			switch (form.FormBorderStyle) {
 			case FormBorderStyle.None:
 				close_button = null;
 				minimize_button = null;
 				maximize_button = null;
+				if (IsMaximized || IsMinimized)
+					goto case FormBorderStyle.Sizable;
+				break;
+			case FormBorderStyle.FixedToolWindow:
+			case FormBorderStyle.SizableToolWindow:
+				close_button = new TitleButton (CaptionButton.Close, new EventHandler (CloseClicked));
+				if (IsMaximized || IsMinimized)
+					goto case FormBorderStyle.Sizable;
 				break;
 			case FormBorderStyle.FixedSingle:
 			case FormBorderStyle.Fixed3D:
@@ -304,15 +358,13 @@ namespace System.Windows.Forms {
 				minimize_button = new TitleButton (CaptionButton.Minimize, new EventHandler (MinimizeClicked));
 				maximize_button = new TitleButton (CaptionButton.Maximize, new EventHandler (MaximizeClicked));
 				break;
-			case FormBorderStyle.FixedToolWindow:
-			case FormBorderStyle.SizableToolWindow:
-				close_button = new TitleButton (CaptionButton.Close, new EventHandler (CloseClicked));
-				break;
 			}
 
 			title_buttons [0] = close_button;
 			title_buttons [1] = minimize_button;
 			title_buttons [2] = maximize_button;
+
+			ThemeEngine.Current.ManagedWindowSetButtonLocations (this);
 		}
 
 		protected virtual bool HandleButtonDown (ref Message m)
@@ -325,28 +377,28 @@ namespace System.Windows.Forms {
 		{
 			Activate ();
 
-			int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
-			int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
-
 			start = Cursor.Position;
 			virtual_position = form.Bounds;
 
+			int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
+			int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
+			Console.WriteLine ("MOUSE DOWN:   ({0}, {1})", x, y);
+			
 			form.PointToClient (ref x, ref y);
 			// Need to adjust because we are in NC land
-			y += TitleBarHeight;
 			FormPos pos = FormPosForCoords (x, y);
 
+			
 			if (pos == FormPos.TitleBar) {
 				HandleTitleBarDown (x, y);
 				return true;
 			}
 
 			if (IsSizable) {
-				SetCursorForPos (pos);
-			
 				if ((pos & FormPos.AnyEdge) == 0)
 					return false;
 
+				virtual_position = form.Bounds;
 				state = State.Sizing;
 				sizing_edge = pos;
 				form.Capture = true;
@@ -356,7 +408,11 @@ namespace System.Windows.Forms {
 			return false;
 		}
 
-		private void HandleTitleBarDown (int x, int y)
+		protected virtual void HandleNCLButtonDblClick (ref Message m)
+		{
+		}
+
+		protected virtual void HandleTitleBarDown (int x, int y)
 		{
 			foreach (TitleButton button in title_buttons) {
 				if (button != null && button.Rectangle.Contains (x, y)) {
@@ -399,48 +455,10 @@ namespace System.Windows.Forms {
 			return false;
 		}
 
-		private bool HandleNCMouseMove (Form form, ref Message m)
-		{
-			int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
-			int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
-			
-			if (IsSizable) {
-				form.PointToClient (ref x, ref y);
-				y += TitleBarHeight;
-			}
-
-			return false;
-		}
-
 		private void FormMouseLeave (ref Message m)
 		{
 			form.ResetCursor ();
 		}
-
-		private void SetCursorForPos (FormPos pos)
-		{
-			switch (pos) {
-			case FormPos.TopLeft:
-			case FormPos.BottomRight:
-				form.Cursor = Cursors.SizeNWSE;
-				break;
-			case FormPos.TopRight:
-			case FormPos.BottomLeft:
-				form.Cursor = Cursors.SizeNESW;
-				break;
-			case FormPos.Top:
-			case FormPos.Bottom:
-				form.Cursor = Cursors.SizeNS;
-				break;
-			case FormPos.Left:
-			case FormPos.Right:
-				form.Cursor = Cursors.SizeWE;
-				break;
-			default:
-				form.ResetCursor ();
-				break;
-			}
-		}
 	
 		protected virtual void HandleWindowMove (Message m)
 		{
@@ -452,49 +470,51 @@ namespace System.Windows.Forms {
 
 		private void HandleSizing (Message m)
 		{
-			Point move = MouseMove (m);
 			Rectangle pos = virtual_position;
-			int mw = MinTitleBarSize.Width + (BorderWidth * 2);
-			int mh = MinTitleBarSize.Height + (BorderWidth * 2);
-			
+			int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
+			int mw = MinTitleBarSize.Width + (bw * 2);
+			int mh = MinTitleBarSize.Height + (bw * 2);
+			int x = Cursor.Position.X;
+			int y = Cursor.Position.Y;
+
+			PointToClient (ref x, ref y);
+
 			if ((sizing_edge & FormPos.Top) != 0) {
-				int height = form.Height - move.Y;
-				if (height <= mh) {
-					move.Y += height - mh;
-					height = mh;
-				}
-				pos.Y = form.Top + move.Y;
-				pos.Height = height;
+				if (pos.Bottom - y < mh)
+					y = pos.Bottom - mh;
+				pos.Height = pos.Bottom - y;
+				pos.Y = y;
 			} else if ((sizing_edge & FormPos.Bottom) != 0) {
-				int height = form.Height + move.Y;
+				int height = y - pos.Top;
 				if (height <= mh)
-					move.Y -= height - mh;
-				pos.Height = form.Height + move.Y;
+					height = mh;
+				pos.Height = height;
 			}
 
 			if ((sizing_edge & FormPos.Left) != 0) {
-				int width = form.Width - move.X;
-				if (width <= mw) {
-					move.X += width - mw;
-					width = mw;
-				}
-				pos.X = form.Left + move.X;
-				pos.Width = width;
+				if (pos.Right - x < mw)
+					x = pos.Right - mw;
+				pos.Width = pos.Right - x;
+				pos.X = x;
 			} else if ((sizing_edge & FormPos.Right) != 0) {
-				int width = form.Width + move.X;
+				int width = x - form.Left;
 				if (width <= mw)
-					move.X -= width - mw;
-				pos.Width = form.Width + move.X;
+					width = mw;
+				pos.Width = width;
 			}
 
 			UpdateVP (pos);
 		}
 
-		private bool IsMaximized {
+		public bool IsMaximized {
 			get { return GetWindowState () == FormWindowState.Maximized; }
 		}
 
-		private bool IsSizable {
+		public bool IsMinimized {
+			get { return GetWindowState () == FormWindowState.Minimized; }
+		}
+
+		public bool IsSizable {
 			get {
 				switch (form.FormBorderStyle) {
 				case FormBorderStyle.Sizable:
@@ -506,13 +526,13 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		private bool HasBorders {
+		public bool HasBorders {
 			get {
 				return form.FormBorderStyle != FormBorderStyle.None;
 			}
 		}
 
-		private bool IsToolWindow {
+		public bool IsToolWindow {
 			get {
 				if (form.FormBorderStyle == FormBorderStyle.SizableToolWindow ||
 						form.FormBorderStyle == FormBorderStyle.FixedToolWindow)
@@ -523,24 +543,7 @@ namespace System.Windows.Forms {
 
 		public int TitleBarHeight {
 			get {
-				if (IsToolWindow)
-					return SystemInformation.ToolWindowCaptionHeight;
-				if (form.FormBorderStyle == FormBorderStyle.None)
-					return 0;
-				return SystemInformation.CaptionHeight;
-			}
-		}
-
-		private Size ButtonSize {
-			get {
-				int height = TitleBarHeight;
-				if (IsToolWindow)
-					return new Size (SystemInformation.ToolWindowCaptionButtonSize.Width - 2,
-							height - 5);
-				if (form.FormBorderStyle == FormBorderStyle.None)
-					return Size.Empty;
-				return new Size (SystemInformation.CaptionButtonSize.Width - 2,
-						height - 5);
+				return ThemeEngine.Current.ManagedWindowTitleBarHeight (this);
 			}
 		}
 
@@ -580,14 +583,18 @@ namespace System.Windows.Forms {
 
 		private bool HandleNCLButtonUp (ref Message m)
 		{
+			if (form.Capture) {
+				ClearVirtualPosition ();
+
+				form.Capture = false;
+				state = State.Idle;
+			}
+				
 			int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
 			int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
 
 			form.PointToClient (ref x, ref y);
 
-			// Need to adjust because we are in NC land
-			y += TitleBarHeight;
-
 			foreach (TitleButton button in title_buttons) {
 				if (button != null && button.Rectangle.Contains (x, y)) {
 					button.Clicked (this, EventArgs.Empty);
@@ -597,85 +604,8 @@ namespace System.Windows.Forms {
 
 			return true;
 		}
-
-		private void PaintWindowDecorations (PaintEventArgs pe)
-		{
-			Graphics dc = pe.Graphics;
-
-			if (HasBorders) {
-				Rectangle borders = new Rectangle (0, 0, form.Width, form.Height);
-			
-				ControlPaint.DrawBorder3D (dc, borders,	Border3DStyle.Raised);
-			}
-
-			Color color = ThemeEngine.Current.ColorControlDark;
-
-			if (IsActive () && !IsMaximized)
-				color = titlebar_color;
-
-			Rectangle tb = new Rectangle (BorderWidth, BorderWidth,
-					form.Width - (BorderWidth * 2), TitleBarHeight - 1);
-
-			// HACK: For now always draw the titlebar until we get updates better
-			// Rectangle vis = Rectangle.Intersect (tb, pe.ClipRectangle);	
-			//if (vis != Rectangle.Empty)
-				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (color), tb);
-
-			dc.DrawLine (new Pen (SystemColors.ControlLight, 1), BorderWidth,
-					TitleBarHeight + BorderWidth, form.Width - (BorderWidth * 2),
-					TitleBarHeight + BorderWidth);
-
-			if (!IsToolWindow) {
-				tb.X += 18; // Room for the icon and the buttons
-				tb.Width = (form.Width - 62) - tb.X;
-			}
-
-			if (form.Text != null) {
-				StringFormat format = new StringFormat ();
-				format.FormatFlags = StringFormatFlags.NoWrap;
-				format.Trimming = StringTrimming.EllipsisCharacter;
-				format.LineAlignment = StringAlignment.Center;
-
-				if (tb.IntersectsWith (pe.ClipRectangle))
-					dc.DrawString (form.Text, form.Font,
-						ThemeEngine.Current.ResPool.GetSolidBrush (Color.White),
-						tb, format);
-			}
-
-			if (!IsToolWindow && HasBorders) {
-				if (form.Icon != null) {
-					Rectangle icon = new Rectangle (BorderWidth + 3,
-							BorderWidth + 2, IconWidth, IconWidth);
-					if (icon.IntersectsWith (pe.ClipRectangle))
-						dc.DrawIcon (form.Icon, icon);
-				}
-
-				Size bs = ButtonSize;
-				close_button.Rectangle = new Rectangle (form.Width - BorderWidth - bs.Width - 2,
-						BorderWidth + 2, bs.Width, bs.Height);
-
-				maximize_button.Rectangle = new Rectangle (close_button.Rectangle.Left - 2 - bs.Width,
-						BorderWidth + 2, bs.Width, bs.Height);
-				
-				minimize_button.Rectangle = new Rectangle (maximize_button.Rectangle.Left - bs.Width,
-						BorderWidth + 2, bs.Width, bs.Height);
-
-				
-				
-				
-
-				DrawTitleButton (dc, minimize_button, pe.ClipRectangle);
-				DrawTitleButton (dc, maximize_button, pe.ClipRectangle);
-				DrawTitleButton (dc, close_button, pe.ClipRectangle);
-			} else if (IsToolWindow) {
-				Size bs = ButtonSize;
-				close_button.Rectangle = new Rectangle (form.Width - BorderWidth - 2 - bs.Width,
-						BorderWidth + 2, bs.Width, bs.Height);
-				DrawTitleButton (dc, close_button, pe.ClipRectangle);
-			}
-		}
 		
-		private void DrawTitleButton (Graphics dc, TitleButton button, Rectangle clip)
+		protected void DrawTitleButton (Graphics dc, TitleButton button, Rectangle clip)
 		{
 			if (!button.Rectangle.IntersectsWith (clip))
 				return;
@@ -686,25 +616,11 @@ namespace System.Windows.Forms {
 					button.Caption, ButtonState.Normal);
 		}
 
-		public void DrawMaximizedButtons (PaintEventArgs pe, MainMenu menu)
+		public virtual void DrawMaximizedButtons (object sender, PaintEventArgs pe)
 		{
-			Size bs = ButtonSize;
-
-			close_button.Rectangle = new Rectangle (menu.Width - BorderWidth - bs.Width - 2,
-						2, bs.Width, bs.Height);
-
-			maximize_button.Rectangle = new Rectangle (close_button.Rectangle.Left - 2 - bs.Width,
-					2, bs.Width, bs.Height);
-				
-			minimize_button.Rectangle = new Rectangle (maximize_button.Rectangle.Left - bs.Width,
-					2, bs.Width, bs.Height);
-
-			DrawTitleButton (pe.Graphics, minimize_button, pe.ClipRectangle);
-			DrawTitleButton (pe.Graphics, maximize_button, pe.ClipRectangle);
-			DrawTitleButton (pe.Graphics, close_button, pe.ClipRectangle);
 		}
 
-		private void CloseClicked (object sender, EventArgs e)
+		protected virtual void CloseClicked (object sender, EventArgs e)
 		{
 			form.Close ();
 		}
@@ -712,10 +628,8 @@ namespace System.Windows.Forms {
 		private void MinimizeClicked (object sender, EventArgs e)
 		{
 			if (GetWindowState () != FormWindowState.Minimized) {
-				minimize_button.Caption = CaptionButton.Restore;
 				form.WindowState = FormWindowState.Minimized;
 			} else {
-				minimize_button.Caption = CaptionButton.Minimize;
 				form.WindowState = FormWindowState.Normal;
 			}
 		}
@@ -723,10 +637,8 @@ namespace System.Windows.Forms {
 		private void MaximizeClicked (object sender, EventArgs e)
 		{
 			if (GetWindowState () != FormWindowState.Maximized) {
-				maximize_button.Caption = CaptionButton.Restore;
 				form.WindowState = FormWindowState.Maximized;
 			} else {
-				maximize_button.Caption = CaptionButton.Maximize;
 				form.WindowState = FormWindowState.Normal;
 			}
 		}
@@ -739,7 +651,7 @@ namespace System.Windows.Forms {
 
 		protected virtual void DrawVirtualPosition (Rectangle virtual_position)
 		{
-			form.Location = virtual_position.Location;
+			form.Bounds = virtual_position;
 			start = Cursor.Position;
 		}
 
@@ -752,47 +664,48 @@ namespace System.Windows.Forms {
 		{
 		}
 
-		private FormPos FormPosForCoords (int x, int y)
+		protected FormPos FormPosForCoords (int x, int y)
 		{
-			if (y < TitleBarHeight + BorderWidth) {
+			int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
+			if (y < TitleBarHeight + bw) {
 				//	Console.WriteLine ("A");
-				if (y > BorderWidth && x > BorderWidth &&
-						x < form.Width - BorderWidth)
+				if (y > bw && x > bw &&
+						x < form.Width - bw)
 					return FormPos.TitleBar;
 
-				if (x < BorderWidth || (x < 20 && y < BorderWidth))
+				if (x < bw || (x < 20 && y < bw))
 					return FormPos.TopLeft;
 
-				if (x > form.Width - BorderWidth ||
-					(x > form.Width - 20 && y < BorderWidth))
+				if (x > form.Width - bw ||
+					(x > form.Width - 20 && y < bw))
 					return FormPos.TopRight;
 
-				if (y < BorderWidth)
+				if (y < bw)
 					return FormPos.Top;
 
 			} else if (y > form.Height - 20) {
 				//	Console.WriteLine ("B");
-				if (x < BorderWidth ||
-						(x < 20 && y > form.Height - BorderWidth))
+				if (x < bw ||
+						(x < 20 && y > form.Height - bw))
 					return FormPos.BottomLeft;
 
-				if (x > form.Width - (BorderWidth * 2) ||
+				if (x > form.Width - (bw * 2) ||
 						(x > form.Width - 20 &&
-						 y > form.Height - BorderWidth))
+						 y > form.Height - bw))
 					return FormPos.BottomRight;
 
-				if (y > form.Height - (BorderWidth * 2))
+				if (y > form.Height - (bw * 2))
 					return FormPos.Bottom;
 
 
-			} else if (x < BorderWidth) {
+			} else if (x < bw) {
 				//	Console.WriteLine ("C");
 				return FormPos.Left;
-			} else if (x > form.Width - (BorderWidth * 2)) {
+			} else if (x > form.Width - (bw * 2)) {
 //				Console.WriteLine ("D");
 				return FormPos.Right;
 			} else {
-				//			Console.WriteLine ("E   {0}", form.Width - BorderWidth);
+				//			Console.WriteLine ("E   {0}", form.Width - bw);
 			}
 			
 			return FormPos.None;
