@@ -62,7 +62,7 @@ namespace System.Windows.Forms
 		private ColumnHeaderStyle header_style = ColumnHeaderStyle.Clickable;
 		private bool hide_selection = true;
 		private bool hover_selection = false;
-		private IComparer item_sorter;
+		private IComparer item_sorter = new ItemComparer ();
 		private ListViewItemCollection items;
 		private bool label_edit = false;
 		private bool label_wrap = true;
@@ -143,11 +143,17 @@ namespace System.Windows.Forms
 
 			header_control = new HeaderControl (this);
 			header_control.Visible = false;
+			Controls.AddImplicit (header_control);
+
 			item_control = new ItemControl (this);
-			item_control.Visible = true;
+			Controls.AddImplicit (item_control);
 
 			h_scroll = new HScrollBar ();
+			Controls.AddImplicit (this.h_scroll);
+
 			v_scroll = new VScrollBar ();
+			Controls.AddImplicit (this.v_scroll);
+
 			h_marker = v_marker = 0;
 			keysearch_tickcnt = 0;
 
@@ -160,6 +166,8 @@ namespace System.Windows.Forms
 			// event handlers
 			base.KeyDown += new KeyEventHandler(ListView_KeyDown);
 			SizeChanged += new EventHandler (ListView_SizeChanged);
+			GotFocus += new EventHandler (FocusChanged);
+			LostFocus += new EventHandler (FocusChanged);
 
 			this.SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
 		}
@@ -322,8 +330,6 @@ namespace System.Windows.Forms
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		public ListViewItem FocusedItem {
 			get {
-				if (focused_item == null && Focused && items.Count > 0)
-					focused_item = items [0];
 				return focused_item;
 			}
 		}
@@ -465,6 +471,18 @@ namespace System.Windows.Forms
 			get { return selected_items; }
 		}
 
+#if NET_2_0
+		[MonoTODO("Implement")]
+		public bool ShowGroups {
+			get {
+				return false;
+			}
+
+			set {
+			}
+		}
+#endif
+
 		[DefaultValue (null)]
 		public ImageList SmallImageList {
 			get { return small_image_list; }
@@ -538,6 +556,18 @@ namespace System.Windows.Forms
 			}
 		}
 
+#if NET_2_0
+		[MonoTODO("Implement")]
+		public bool UseCompatibleStateImageBehavior {
+			get {
+				return false;
+			}
+
+			set {
+			}
+		}
+#endif
+
 		[DefaultValue (View.LargeIcon)]
 		public View View {
 			get { return view; }
@@ -580,9 +610,14 @@ namespace System.Windows.Forms
 		
 		internal int LastVisibleIndex {			
 			get {							
-				for (int i = FirstVisibleIndex; i < Items.Count; i++) {						
-					if (Items[i].Bounds.Y > ClientRectangle.Bottom)						
-							return i -1;					
+				for (int i = FirstVisibleIndex; i < Items.Count; i++) {
+					if (View == View.List || Alignment == ListViewAlignment.Left) {
+						if (Items[i].Bounds.X > ClientRectangle.Right)
+							return i - 1;					
+					} else {
+						if (Items[i].Bounds.Y > ClientRectangle.Bottom)
+							return i - 1;					
+					}
 				}
 				
 				return Items.Count - 1;
@@ -745,6 +780,10 @@ namespace System.Windows.Forms
 				return;
 			}
 
+			// Don't calculate if the view is not displayable
+			if (client_area.Height < 0 || client_area.Width < 0)
+				return;
+
 			// making a scroll bar visible might make
 			// other scroll bar visible			
 			if (layout_wd > client_area.Right) {
@@ -878,6 +917,7 @@ namespace System.Windows.Forms
 		{
 			header_control.Visible = false;
 			header_control.Size = Size.Empty;
+			item_control.Visible = true;
 			item_control.Location = Point.Empty;
 
 			if (items.Count == 0)
@@ -930,12 +970,6 @@ namespace System.Windows.Forms
 
 		void LayoutHeader ()
 		{
-			if (columns.Count == 0 || header_style == ColumnHeaderStyle.None) {
-				header_control.Visible = false;
-				header_control.Size = Size.Empty;
-				return;
-			}
-
 			int x = 0;
 			for (int i = 0; i < Columns.Count; i++) {
 			       	ColumnHeader col = GetReorderedColumn (i);
@@ -945,15 +979,30 @@ namespace System.Windows.Forms
 				x += col.Wd;
 			}
 
-			header_control.Width = x;
-			header_control.Height = columns [0].Ht;
-			header_control.Visible = true;
+			if (x < ClientRectangle.Width)
+				x = ClientRectangle.Width;
+
+			if (header_style == ColumnHeaderStyle.None) {
+				header_control.Visible = false;
+				header_control.Size = Size.Empty;
+			} else {
+				header_control.Width = x;
+				header_control.Height = columns [0].Ht;
+				header_control.Visible = true;
+			}
 		}
 
 		void LayoutDetails ()
 		{
+			if (columns.Count == 0) {
+				header_control.Visible = false;
+				item_control.Visible = false;
+				return;
+			}
+
 			LayoutHeader ();
 
+			item_control.Visible = true;
 			item_control.Location = new Point (0, header_control.Height);
 
 			int y = 0; 
@@ -1084,7 +1133,7 @@ namespace System.Windows.Forms
 				return item_matrix [row - 1, col].Index;
 
 			case Keys.Down:
-				if (row == (rows - 1))
+				if (row == (rows - 1) || row == Items.Count - 1)
 					return -1;
 				while (item_matrix [row + 1, col] == null)
 				       col--;	
@@ -1097,16 +1146,23 @@ namespace System.Windows.Forms
 
 		ListViewItem selection_start;
 
-		private void SelectItems (ArrayList sel_items)
+		private bool SelectItems (ArrayList sel_items)
 		{
+			bool changed = false;
 			multiselecting = true;
 			ArrayList curr_items = (ArrayList) SelectedItems.list.Clone ();
 			foreach (ListViewItem item in curr_items)
-				if (!sel_items.Contains (item))
+				if (!sel_items.Contains (item)) {
 					item.Selected = false;
+					changed = true;
+				}
 			foreach (ListViewItem item in sel_items)
-				item.Selected = true;
+				if (!item.Selected) {
+					item.Selected = true;
+					changed = true;
+				}
 			multiselecting = false;
+			return changed;
 		}
 
 		private void UpdateMultiSelection (int index)
@@ -1132,23 +1188,28 @@ namespace System.Windows.Forms
 						    curr.col >= left && curr.col <= right)
 							list.Add (curr);
 				}
-				SelectItems (list);
+				if (SelectItems (list))
+					OnSelectedIndexChanged (EventArgs.Empty);
 			} else if (!ctrl_pressed) {
 				SelectedItems.Clear ();
 				SelectedIndices.list.Clear ();
 				item.Selected = true;
 				selection_start = item;
+				OnSelectedIndexChanged (EventArgs.Empty);
 			}
 		}
 
 		private void ListView_KeyDown (object sender, KeyEventArgs ke)
 		{			
-			if (ke.Handled || Items.Count == 0)
+			if (ke.Handled || Items.Count == 0 || !item_control.Visible)
 				return;
 
 			int index = -1;
 			ke.Handled = true;
 
+			if (FocusedItem == null)
+				SetFocusedItem (Items [0]);
+
 			switch (ke.KeyCode) {
 
 			case Keys.End:
@@ -1176,8 +1237,10 @@ namespace System.Windows.Forms
 
 			if (MultiSelect)
 				UpdateMultiSelection (index);
-			else
+			else if (!items [index].Selected) {
 				items [index].Selected = true;
+				OnSelectedIndexChanged (EventArgs.Empty);
+			}
 
 			SetFocusedItem (items [index]);				
 			EnsureVisible (index);
@@ -1190,13 +1253,12 @@ namespace System.Windows.Forms
 			ListViewItem clicked_item;
 			ListViewItem last_clicked_item;
 			bool hover_processed = false;
+			bool checking = false;
 
 			public ItemControl (ListView owner)
 			{
 				this.owner = owner;
 				DoubleClick += new EventHandler(ItemsDoubleClick);
-				KeyDown += new KeyEventHandler (ItemsKeyDown);
-				KeyUp += new KeyEventHandler (ItemsKeyUp);
 				MouseDown += new MouseEventHandler(ItemsMouseDown);
 				MouseMove += new MouseEventHandler(ItemsMouseMove);
 				MouseHover += new EventHandler(ItemsMouseHover);
@@ -1210,16 +1272,6 @@ namespace System.Windows.Forms
 					owner.ItemActivate (this, e);
 			}
 
-			void ItemsKeyDown (object sender, KeyEventArgs args)
-			{
-				owner.OnKeyDown (args);
-			}
-
-			void ItemsKeyUp (object sender, KeyEventArgs args)
-			{
-				owner.OnKeyUp (args);
-			}
-
 			enum BoxSelect {
 				None,
 				Normal,
@@ -1336,6 +1388,16 @@ namespace System.Windows.Forms
 				return true;
 			}
 
+			private void ToggleCheckState (ListViewItem item)
+			{
+				CheckState curr_state = item.Checked ?  CheckState.Checked : CheckState.Unchecked;
+				item.Checked = !item.Checked;
+				CheckState new_state = item.Checked ?  CheckState.Checked : CheckState.Unchecked;
+
+				ItemCheckEventArgs ice = new ItemCheckEventArgs (item.Index, curr_state, new_state);
+				owner.OnItemCheck (ice);
+			}
+
 			private void ItemsMouseDown (object sender, MouseEventArgs me)
 			{
 				if (owner.items.Count == 0)
@@ -1343,16 +1405,12 @@ namespace System.Windows.Forms
 
 				Point pt = new Point (me.X, me.Y);
 				foreach (ListViewItem item in owner.items) {
-					if (item.CheckRectReal.Contains (pt)) {
-						CheckState curr_state = item.Checked ?  CheckState.Checked : CheckState.Unchecked;
-						item.Checked = !item.Checked;
-
-						CheckState new_state = item.Checked ?  CheckState.Checked : CheckState.Unchecked;
-
-						// Raise the ItemCheck event
-						ItemCheckEventArgs ice = new ItemCheckEventArgs (item.Index, curr_state, new_state);
-						owner.OnItemCheck (ice);
-						break;
+					if (me.Clicks == 1 && item.CheckRectReal.Contains (pt)) {
+						checking = true;
+						if (me.Clicks > 1)
+							return;
+						ToggleCheckState (item);
+						return;
 					}
 
 					if (owner.View == View.Details && !owner.FullRowSelect) {
@@ -1368,9 +1426,9 @@ namespace System.Windows.Forms
 					}
 				}
 
-				owner.SetFocusedItem (clicked_item);
 
 				if (clicked_item != null) {
+					owner.SetFocusedItem (clicked_item);
 					bool changed = !clicked_item.Selected;
 					if (owner.MultiSelect && (XplatUI.State.ModifierKeys & Keys.Control) == 0)
 						owner.UpdateMultiSelection (clicked_item.Index);
@@ -1382,9 +1440,11 @@ namespace System.Windows.Forms
 
 					// Raise double click if the item was clicked. On MS the
 					// double click is only raised if you double click an item
-					if (me.Clicks > 1 && clicked_item != null)
+					if (me.Clicks > 1) {
 						owner.OnDoubleClick (EventArgs.Empty);
-					else if (me.Clicks == 1 && clicked_item != null)
+						if (owner.CheckBoxes)
+							ToggleCheckState (clicked_item);
+					} else if (me.Clicks == 1)
 						owner.OnClick (EventArgs.Empty);
 				} else {
 					if (owner.MultiSelect) {
@@ -1472,7 +1532,7 @@ namespace System.Windows.Forms
 							break;
 						}
 					}
-				} else if (owner.SelectedItems.Count > 0 && BoxSelectRectangle.Size.IsEmpty) {
+				} else if (!checking && owner.SelectedItems.Count > 0 && BoxSelectRectangle.Size.IsEmpty) {
 					// Need this to clean up background clicks
 					owner.SelectedItems.Clear ();
 					owner.SelectedIndices.list.Clear ();
@@ -1484,6 +1544,7 @@ namespace System.Windows.Forms
 				BoxSelectRectangle = Rectangle.Empty;
 				prev_selection = null;
 				box_select_mode = BoxSelect.None;
+				checking = false;
 			}
 
 			private void ItemsMouseWheel (object sender, MouseEventArgs me)
@@ -1514,6 +1575,11 @@ namespace System.Windows.Forms
 			{
 				ThemeEngine.Current.DrawListViewItems (pe.Graphics, pe.ClipRectangle, owner);
 			}
+
+			internal override void OnGotFocusInternal (EventArgs e)
+			{
+				owner.Focus ();
+			}
 		}
 
 		internal override void OnPaintInternal (PaintEventArgs pe)
@@ -1524,6 +1590,17 @@ namespace System.Windows.Forms
 			CalculateScrollBars ();
 		}
 
+		void FocusChanged (object o, EventArgs args)
+		{
+			if (Items.Count == 0)
+				return;
+
+			if (FocusedItem == null)
+				SetFocusedItem (Items [0]);
+
+			item_control.Invalidate (FocusedItem.Bounds);
+		}
+
 		private void ListView_SizeChanged (object sender, EventArgs e)
 		{
 			CalculateListView (alignment);
@@ -1641,12 +1718,6 @@ namespace System.Windows.Forms
 		protected override void OnHandleCreated (EventArgs e)
 		{
 			base.OnHandleCreated (e);
-			SuspendLayout ();
-			Controls.AddImplicit (header_control);
-			Controls.AddImplicit (item_control);
-			Controls.AddImplicit (this.v_scroll);
-			Controls.AddImplicit (this.h_scroll);
-			ResumeLayout ();
 		}
 
 		protected override void OnHandleDestroyed (EventArgs e)
@@ -1775,7 +1846,7 @@ namespace System.Windows.Forms
 		public Rectangle GetItemRect (int index, ItemBoundsPortion portion)
 		{
 			if (index < 0 || index >= items.Count)
-				throw new IndexOutOfRangeException ("Invalid Index");
+				throw new IndexOutOfRangeException ("index");
 
 			return items [index].GetBounds (portion);
 		}
@@ -1921,6 +1992,8 @@ namespace System.Windows.Forms
 					zone.X = zone.Right - 5;
 					zone.Width = 10;
 					if (zone.Contains (pt)) {
+						if (i < owner.Columns.Count - 1 && owner.Columns [i + 1].Width == 0)
+							i++;
 						resize_column = owner.Columns [i];
 						break;
 					}
@@ -1982,7 +2055,28 @@ namespace System.Windows.Forms
 					target_x = owner.GetReorderedColumn (drag_to_index).Rect.X - owner.h_marker;
 				theme.DrawListViewHeaderDragDetails (pe.Graphics, owner, drag_column, target_x);
 			}
-					
+
+			protected override void WndProc (ref Message m)
+			{
+				switch ((Msg)m.Msg) {
+				case Msg.WM_SETFOCUS:
+					owner.Focus ();
+					break;
+				default:
+					base.WndProc (ref m);
+					break;
+				}
+			}
+		}
+
+		private class ItemComparer : IComparer {
+
+			public int Compare (object x, object y)
+			{
+				ListViewItem item_x = x as ListViewItem;
+				ListViewItem item_y = y as ListViewItem;
+				return String.Compare (item_x.Text, item_y.Text);
+			}
 		}
 
 		public class CheckedIndexCollection : IList, ICollection, IEnumerable
@@ -2011,7 +2105,7 @@ namespace System.Windows.Forms
 			public int this [int index] {
 				get {
 					if (index < 0 || index >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("index");
 					return (int) list [index];
 				}
 			}
@@ -2119,7 +2213,7 @@ namespace System.Windows.Forms
 			public ListViewItem this [int index] {
 				get {
 					if (index < 0 || index >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("index");
 					return (ListViewItem) list [index];
 				}
 			}
@@ -2227,7 +2321,7 @@ namespace System.Windows.Forms
 			public virtual ColumnHeader this [int index] {
 				get {
 					if (index < 0 || index >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("index");
 					return (ColumnHeader) list [index];
 				}
 			}
@@ -2256,7 +2350,9 @@ namespace System.Windows.Forms
 				int idx;
 				value.owner = this.owner;
 				idx = list.Add (value);
-				owner.Redraw (true); 
+				if (owner.IsHandleCreated) {
+					owner.Redraw (true); 
+				}
 				return idx;
 			}
 
@@ -2353,7 +2449,7 @@ namespace System.Windows.Forms
 				// LAMESPEC: MSDOCS say greater than or equal to the value of the Count property
 				// but it's really only greater.
 				if (index < 0 || index > list.Count)
-					throw new ArgumentOutOfRangeException ("Index out of range.");
+					throw new ArgumentOutOfRangeException ("index");
 
 				value.owner = this.owner;
 				list.Insert (index, value);
@@ -2376,7 +2472,7 @@ namespace System.Windows.Forms
 			public virtual void RemoveAt (int index)
 			{
 				if (index < 0 || index >= list.Count)
-					throw new ArgumentOutOfRangeException ("Index out of range.");
+					throw new ArgumentOutOfRangeException ("index");
 
 				// TODO: Update Column internal index ?
 				list.RemoveAt (index);
@@ -2413,13 +2509,13 @@ namespace System.Windows.Forms
 			public virtual ListViewItem this [int displayIndex] {
 				get {
 					if (displayIndex < 0 || displayIndex >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("displayIndex");
 					return (ListViewItem) list [displayIndex];
 				}
 
 				set {
 					if (displayIndex < 0 || displayIndex >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("displayIndex");
 
 					if (list.Contains (value))
 						throw new ArgumentException ("An item cannot be added more than once. To add an item again, you need to clone it.", "value");
@@ -2582,7 +2678,7 @@ namespace System.Windows.Forms
 				// LAMESPEC: MSDOCS say greater than or equal to the value of the Count property
 				// but it's really only greater.
 				if (index < 0 || index > list.Count)
-					throw new ArgumentOutOfRangeException ("Index out of range.");
+					throw new ArgumentOutOfRangeException ("index");
 
 				if (list.Contains (item))
 					throw new ArgumentException ("An item cannot be added more than once. To add an item again, you need to clone it.", "item");
@@ -2619,7 +2715,7 @@ namespace System.Windows.Forms
 			public virtual void RemoveAt (int index)
 			{
 				if (index < 0 || index >= list.Count)
-					throw new ArgumentOutOfRangeException ("Index out of range.");
+					throw new ArgumentOutOfRangeException ("index");
 
 				list.RemoveAt (index);
 				owner.SelectedItems.list.RemoveAt (index);
@@ -2658,7 +2754,7 @@ namespace System.Windows.Forms
 			public int this [int index] {
 				get {
 					if (index < 0 || index >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("index");
 					return (int) list [index];
 				}
 			}
@@ -2766,7 +2862,7 @@ namespace System.Windows.Forms
 			public ListViewItem this [int index] {
 				get {
 					if (index < 0 || index >= list.Count)
-						throw new ArgumentOutOfRangeException ("Index out of range.");
+						throw new ArgumentOutOfRangeException ("index");
 					return (ListViewItem) list [index];
 				}
 			}
