@@ -36,51 +36,79 @@ namespace System.Windows.Forms {
 	public sealed class MdiClient : Control {
 		#region Local Variables
 		private int mdi_created;
-		private Form active;
 		private HScrollBar hbar;
 		private VScrollBar vbar;
 		private SizeGrip sizegrip;
 		private int hbar_value;
 		private int vbar_value;
 		private bool lock_sizing;
-		
+		private int prev_bottom;
+		private LayoutEventHandler initial_layout_handler;
+		internal ArrayList mdi_child_list;
+
 		#endregion	// Local Variables
 
 		#region Public Classes
 		public new class ControlCollection : Control.ControlCollection {
-			MdiClient	owner;
+
+			private MdiClient owner;
 			
 			public ControlCollection(MdiClient owner) : base(owner) {
 				this.owner = owner;
-				controls = new ArrayList ();
+				owner.mdi_child_list = new ArrayList ();
 			}
 
 			public override void Add(Control value) {
 				if ((value is Form) == false || !(((Form)value).IsMdiChild)) {
 					throw new ArgumentException("Form must be MdiChild");
 				}
+				owner.mdi_child_list.Add (value);
 				base.Add (value);
-				SetChildIndex (value, 0); // always insert at front
-				// newest member is the active one
-				owner.ActiveMdiChild = (Form) value;
 
-				value.LocationChanged += new EventHandler (owner.FormLocationChanged);
+				// newest member is the active one
+				Form form = (Form) value;
+				owner.ActiveMdiChild = form;
 			}
 
-			public override void Remove(Control value) {
+			public override void Remove(Control value)
+			{
+				Form form = value as Form;
+				if (form != null) {
+					MdiWindowManager wm = form.WindowManager as MdiWindowManager;
+					if (wm != null) {
+						form.Closed -= wm.form_closed_handler;
+					}
+				}
+
+				owner.mdi_child_list.Remove (value);
 				base.Remove (value);
 			}
 		}
 		#endregion	// Public Classes
 
 		#region Public Constructors
-		public MdiClient() {
+		public MdiClient()
+		{
 			BackColor = SystemColors.AppWorkspace;
 			Dock = DockStyle.Fill;
 			SetStyle (ControlStyles.Selectable, false);
 		}
 		#endregion	// Public Constructors
 
+		internal override void OnPaintBackgroundInternal (PaintEventArgs pe)
+		{
+			if (BackgroundImage != null)
+				return;
+
+			if (Parent == null || Parent.BackgroundImage == null)
+				return;
+			Parent.PaintControlBackground (pe);
+		}
+
+		internal Form ParentForm {
+			get { return (Form) Parent; }
+		}
+
 		protected override Control.ControlCollection CreateControlsInstance ()
 		{
 			return new MdiClient.ControlCollection (this);
@@ -104,7 +132,7 @@ namespace System.Windows.Forms {
 
 			// Should probably make this into one loop
 			SizeScrollBars ();
-			SizeMaximized ();
+			ArrangeWindows ();
 		}
 
 		protected override void ScaleCore (float dx, float dy)
@@ -128,14 +156,11 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		public Form[] MdiChildren {
+		public Form [] MdiChildren {
 			get {
-				Form[]	children;
-
-				children = new Form[Controls.Count];
-				Controls.CopyTo(children, 0);
-
-				return children;
+				if (mdi_child_list == null)
+					return new Form [0];
+				return (Form []) mdi_child_list.ToArray (typeof (Form));
 			}
 		}
 		#endregion	// Public Instance Properties
@@ -149,8 +174,40 @@ namespace System.Windows.Forms {
 		#endregion	// Protected Instance Properties
 
 		#region Public Instance Methods
-		public void LayoutMdi(MdiLayout value) {
-			throw new NotImplementedException();
+		public void LayoutMdi (MdiLayout value) {
+
+			int max_width = Int32.MaxValue;
+			int max_height = Int32.MaxValue;
+
+			if (Parent != null) {
+				max_width = Parent.Width;
+				max_height = Parent.Height;
+			}
+
+			switch (value) {
+			case MdiLayout.Cascade:
+				int i = 0;
+				for (int c = Controls.Count - 1; c >= 0; c--) {
+					Form form = (Form) Controls [c];
+
+					int l = 22 * i;
+					int t = 22 * i;
+
+					if (i != 0 && (l + form.Width > max_width || t + form.Height > max_height)) {
+						i = 0;
+						l = 22 * i;
+						t = 22 * i;
+					}
+
+					form.Left = l;
+					form.Top = t;
+
+					i++;
+				}
+				break;
+			default:
+				throw new NotImplementedException();
+			}
 		}
 		#endregion	// Public Instance Methods
 
@@ -274,6 +331,7 @@ namespace System.Windows.Forms {
 			hbar.LargeChange = 50;
 			hbar.Maximum = diff + 51;
 			hbar.Value = -virtual_left;
+
 			hbar.ValueChanged += new EventHandler (HBarValueChanged);
 		}
 
@@ -330,15 +388,26 @@ namespace System.Windows.Forms {
 			lock_sizing = false;
 		}
 
-		private void SizeMaximized ()
+		private void ArrangeWindows ()
 		{
+			int change = 0;
+			if (prev_bottom != -1)
+				change = Bottom - prev_bottom;
+
 			foreach (Form child in Controls) {
 				if (!child.Visible)
 					continue;
 				MdiWindowManager wm = (MdiWindowManager) child.WindowManager;
 				if (wm.GetWindowState () == FormWindowState.Maximized)
 					wm.SizeMaximized ();
+
+				if (wm.GetWindowState () == FormWindowState.Minimized) {
+					child.Top += change;
+				}
+					
 			}
+
+			prev_bottom = Bottom;
 		}
 
 		private void FormLocationChanged (object sender, EventArgs e)
@@ -364,7 +433,10 @@ namespace System.Windows.Forms {
 					continue;
 
 				MdiWindowManager wm = (MdiWindowManager) form.WindowManager;
-
+				// Need to get the width in the loop cause some themes might have
+				// different widths for different styles
+				int bw = ThemeEngine.Current.ManagedWindowBorderWidth (wm);
+				
 				if (wm.IconicBounds != Rectangle.Empty) {
 					form.Bounds = wm.IconicBounds;
 					continue;
@@ -372,7 +444,7 @@ namespace System.Windows.Forms {
 					
 				// The extra one pixel is a cheap hack for now until we
 				// handle 0 client sizes properly in the driver
-				int height = wm.TitleBarHeight + (wm.BorderWidth * 2) + 1; 
+				int height = wm.TitleBarHeight + (bw * 2) + 1; 
 				Rectangle rect = new Rectangle (iconic_x, iconic_y, xspacing, height);
 				form.Bounds = wm.IconicBounds = rect;
 
@@ -385,15 +457,43 @@ namespace System.Windows.Forms {
 			lock_sizing = false;
 		}
 
+		internal void CloseChildForm (Form form)
+		{
+			if (Controls.Count > 1) {
+				Form next = (Form) Controls [1];
+				if (form.WindowState == FormWindowState.Maximized)
+					next.WindowState = FormWindowState.Maximized;
+				ActivateChild (next);
+			}
+
+			Controls.Remove (form);
+			form.Close ();
+		}
+
+		internal void ActivateNextChild ()
+		{
+			if (Controls.Count < 1)
+				return;
+
+			Form front = (Form) Controls [0];
+			Form form = (Form) Controls [1];
+
+			front.SendToBack ();
+			ActivateChild (form);
+		}
+
 		internal void ActivateChild (Form form)
 		{
-			form.BringToFront ();
-			active = form;
+			if (Controls.Count < 1)
+				return;
 
-			foreach (Form child in Controls) {
-				if (child == form)
-					continue;
-				// TODO: We need to repaint the decorations here
+			Form current = (Form) Controls [0];
+
+			form.BringToFront ();
+
+			if (current != form) {
+				current.WindowManager.PaintDecorations ();
+				form.WindowManager.PaintDecorations ();
 			}
 		}
 
@@ -403,8 +503,15 @@ namespace System.Windows.Forms {
 		}
 
 		internal Form ActiveMdiChild {
-			get { return active; }
-			set { active = value; }
+			get {
+				if (Controls.Count < 1)
+					return null;
+				return (Form) Controls [0];
+			}
+			set {
+				ActivateChild (value);
+			}
 		}
 	}
 }
+
