@@ -24,10 +24,9 @@
 //	Mike Kestner  <mkestner@novell.com>
 //
 
-// NOT COMPLETE
-
 using System.Collections;
 using System.Drawing;
+using System.Threading;
 
 namespace System.Windows.Forms {
 
@@ -85,8 +84,16 @@ namespace System.Windows.Forms {
 				PopUpWindow puw = TopMenu.Wnd as PopUpWindow;
 				puw.HideWindow ();
 			} else {
+#if false
+				MenuItem item = TopMenu.SelectedItem;
+				if (item.IsPopup)
+					HideSubPopups (item);
+
+				Menu menu = item.Parent;
+				menu.InvalidateItem (item);
+#else
 				DeselectItem (TopMenu.SelectedItem);
-				(TopMenu as MainMenu).Draw ();
+#endif
 			}
 			CurrentMenu = TopMenu;
 		}
@@ -109,14 +116,19 @@ namespace System.Windows.Forms {
 		MenuItem GetItemAtXY (int x, int y)
 		{
 			Point pnt = new Point (x, y);
-			MenuItem item = FindItemByCoords (TopMenu, pnt);
-			if (item == null && TopMenu.SelectedItem != null)
+			MenuItem item = null;
+			if (TopMenu.SelectedItem != null)
 				item = FindSubItemByCoord (TopMenu.SelectedItem, Control.MousePosition);
+			if (item == null)
+				item = FindItemByCoords (TopMenu, pnt);
 			return item;
 		}
 
-		public void OnClick (MouseEventArgs args)
+		public void OnMouseDown (MouseEventArgs args)
 		{
+			if ((args.Button & MouseButtons.Left) == 0)
+				return;
+
 			MenuItem item = GetItemAtXY (args.X, args.Y);
 
 			if (item != null && !item.Enabled)
@@ -131,8 +143,7 @@ namespace System.Windows.Forms {
 			if (item.IsPopup) {
 				active = true;
 				grab_control.ActiveTracker = this;
-				if (item.Parent is MainMenu)
-					(item.Parent as MainMenu).Draw ();
+				item.Parent.InvalidateItem (item);
 			} else if (item.Parent is MainMenu)
 				active = false;
 			else
@@ -180,6 +191,8 @@ namespace System.Windows.Forms {
 
 		static public bool TrackPopupMenu (Menu menu, Point pnt)
 		{
+			Object	queue_id;
+
 			if (menu.MenuItems.Count <= 0)	// No submenus to track
 				return true;				
 
@@ -194,9 +207,11 @@ namespace System.Windows.Forms {
 
 			bool no_quit = true;
 
+			queue_id = XplatUI.StartLoop(Thread.CurrentThread);
+
 			while ((menu.Wnd != null) && menu.Wnd.Visible && no_quit) {
 				MSG msg = new MSG ();
-				no_quit = XplatUI.GetMessage(ref msg, IntPtr.Zero, 0, 0);
+				no_quit = XplatUI.GetMessage(queue_id, ref msg, IntPtr.Zero, 0, 0);
 				XplatUI.TranslateMessage(ref msg);
 				XplatUI.DispatchMessage(ref msg);				
 			}
@@ -223,10 +238,7 @@ namespace System.Windows.Forms {
 				HideSubPopups (item);
 
 			Menu menu = item.Parent;
-			if (menu is MainMenu)
-				(menu as MainMenu).Draw ();
-			else if (menu.Wnd != null)
-				menu.Wnd.Invalidate (item.bounds);
+			menu.InvalidateItem (item);
 		}
 
 		void SelectItem (Menu menu, MenuItem item, bool execute)
@@ -238,13 +250,10 @@ namespace System.Windows.Forms {
 				if (CurrentMenu != menu)
 					CurrentMenu = menu;
 				item.Selected = true;
-				if (menu is MainMenu)
-					(menu as MainMenu).Draw ();
-				else
-					menu.Wnd.Invalidate (item.bounds);
+				menu.InvalidateItem (item);
 				item.PerformSelect ();					
 			}
-			
+
 			if (execute)
 				ExecFocusedItem (menu, item);
 		}
@@ -280,7 +289,7 @@ namespace System.Windows.Forms {
 			
 			Point pnt;
 			if (menu is MainMenu)
-				pnt = new Point (item.X, 0);
+				pnt = new Point (item.X, item.Y + item.Height - 2 - menu.Height);
 			else
 				pnt = new Point (item.X + item.Width, item.Y + 1);
 			pnt = menu.Wnd.PointToScreen (pnt);
@@ -479,9 +488,6 @@ namespace System.Windows.Forms {
 					break;
 				}
 				break;
-
-			default:
-				throw new ArgumentException ("Invalid msg_type " + msg_type);
 			}
 		}
 
@@ -525,6 +531,7 @@ namespace System.Windows.Forms {
 						ShowSubPopup (CurrentMenu, item);
 						SelectItem (item, item.MenuItems [0], false);
 						CurrentMenu = item;
+						active = true;
 					}
 					return;
 				}
@@ -631,7 +638,7 @@ namespace System.Windows.Forms {
 				return ProcessMnemonic (msg, keyData);
 			else if ((Msg)msg.Msg == Msg.WM_SYSKEYUP || keynav_state == KeyNavState.Idle)
 				return false;
-			else if (!active)
+			else if (!active && !Navigating)
 				return false;
 
 			switch (keyData) {
@@ -684,9 +691,8 @@ namespace System.Windows.Forms {
 
 		public void ShowWindow ()
 		{
-			Show ();
 			RefreshItems ();
-			Refresh ();
+			Show ();
 		}
 		
 		internal override void OnPaintInternal (PaintEventArgs args)
