@@ -96,6 +96,13 @@ namespace System.Windows.Forms
 				msgbox_buttons = buttons;
 				msgbox_default = MessageBoxDefaultButton.Button1;
 
+				if (owner != null) {
+					Owner = Control.FromHandle(owner.Handle).FindForm();
+				} else {
+					if (Application.MWFThread.Current.Context != null) {
+						Owner = Application.MWFThread.Current.Context.MainForm;
+					}
+				}
 				this.Text = caption;
 			}
 
@@ -119,6 +126,7 @@ namespace System.Windows.Forms
 					cp = base.CreateParams;
 
 					cp.Style = (int)(WindowStyles.WS_DLGFRAME | WindowStyles.WS_POPUP | WindowStyles.WS_CLIPCHILDREN | WindowStyles.WS_CLIPSIBLINGS | WindowStyles.WS_CAPTION);
+					cp.ExStyle = (int)(WindowExStyles.WS_EX_TOPMOST);
 					if (!is_enabled) {
 						cp.Style |= (int)(WindowStyles.WS_DISABLED);
 					}
@@ -166,7 +174,8 @@ namespace System.Windows.Forms
 					} else {
 						textleft_up = new Point (icon_image.Width + space_image_text + space_border, 2 + space_border);
 					}
-					tsize.Height = icon_image.Height;
+					if (tsize.Height < icon_image.Height)
+						tsize.Height = icon_image.Height;
 				} else {
 					tsize.Width += space_border * 2;
 					textleft_up = new Point (space_border + 12, space_border + 12);
