@@ -101,12 +101,22 @@ namespace System.Windows.Forms
 		
 		private Mime( )
 		{
+#if NET_2_0
+			Aliases = new NameValueCollection (StringComparer.CurrentCultureIgnoreCase);
+			SubClasses = new NameValueCollection (StringComparer.CurrentCultureIgnoreCase);
+			GlobalPatternsShort = new NameValueCollection (StringComparer.CurrentCultureIgnoreCase);
+			GlobalPatternsLong = new NameValueCollection (StringComparer.CurrentCultureIgnoreCase);
+			GlobalLiterals = new NameValueCollection (StringComparer.CurrentCultureIgnoreCase);
+			GlobalSufPref = new NameValueCollection (StringComparer.CurrentCultureIgnoreCase);
+#else
 			Aliases = new NameValueCollection (new CaseInsensitiveHashCodeProvider (), new Comparer (System.Globalization.CultureInfo.CurrentUICulture));
 			SubClasses = new NameValueCollection (new CaseInsensitiveHashCodeProvider (), new Comparer (System.Globalization.CultureInfo.CurrentUICulture));
 			GlobalPatternsShort = new NameValueCollection (new CaseInsensitiveHashCodeProvider (), new Comparer (System.Globalization.CultureInfo.CurrentUICulture));
 			GlobalPatternsLong = new NameValueCollection (new CaseInsensitiveHashCodeProvider (), new Comparer (System.Globalization.CultureInfo.CurrentUICulture));
 			GlobalLiterals = new NameValueCollection (new CaseInsensitiveHashCodeProvider (), new Comparer (System.Globalization.CultureInfo.CurrentUICulture));
 			GlobalSufPref = new NameValueCollection (new CaseInsensitiveHashCodeProvider (), new Comparer (System.Globalization.CultureInfo.CurrentUICulture));
+#endif
+			
 			Matches80Plus = new ArrayList ();
 			MatchesBelow80 = new ArrayList ();
 			
@@ -177,17 +187,19 @@ namespace System.Windows.Forms
 			GoByFileName( );
 //			}
 			
-//			if ( !mime_file_cache.ContainsKey( current_file_name ) )
 			mime_file_cache.Add( current_file_name, global_result );
 			
-			// not tested
-			if ( mime_file_cache.Count > mime_file_cache_max_size )
-			{
-				IEnumerator enumerator = mime_file_cache.GetEnumerator( );
+			if (mime_file_cache.Count > mime_file_cache_max_size) {
+				IEnumerator enumerator = mime_file_cache.GetEnumerator ();
 				
-				for ( int i = 0; i < mime_file_cache_max_size - 1000; i++ )
-				{
-					mime_file_cache.Remove( enumerator.Current.ToString( ) );
+				int counter = mime_file_cache_max_size - 1000;
+				
+				while (enumerator.MoveNext ()) {
+					mime_file_cache.Remove (enumerator.Current.ToString ());
+					counter--;
+					
+					if (counter == 0)
+						break;
 				}
 			}
 		}
@@ -702,7 +714,7 @@ namespace System.Windows.Forms
 				
 				file_stream.Close( );
 			}
-			catch (Exception e)
+			catch (Exception)
 			{
 				return false;
 			}
