@@ -43,6 +43,7 @@
 // to get the image itself for a mime type with a specific size
 
 using System;
+using System.Reflection;
 using System.Drawing;
 using System.Collections;
 using System.Collections.Specialized;
@@ -69,6 +70,22 @@ namespace System.Windows.Forms
 		GNOME
 		// Win, Mac OSX...
 	}
+
+	internal class ResourceImageLoader {
+		static Assembly assembly = typeof (ResourceImageLoader).Assembly;
+		
+		static internal Bitmap Get (string name)
+		{
+			using (Stream stream = assembly.GetManifestResourceStream (name)){
+				if (stream == null){
+					Console.WriteLine ("Failed to read {0}", name);
+					return null;
+				}
+				
+				return new Bitmap (stream);
+			}
+		}
+	}
 	
 	internal class MimeIconEngine
 	{
@@ -278,7 +295,7 @@ namespace System.Windows.Forms
 			{
 				if ( platform == EPlatformHandler.Default )
 				{
-					if ( mime_type == "inode/directory" )
+					if (mime_type =="inode/directory")
 					{
 						return (int)MimeTypeIconIndexHash[ "inode/directory" ];
 					}
@@ -357,12 +374,12 @@ namespace System.Windows.Forms
 			
 			string[] split = mime_type.Split( new char[] { ',' } );
 			
-			foreach ( string s in split )
+			for (int i = 0; i < split.Length; i++)
 			{
-				if ( MimeTypeIconIndexHash.ContainsKey( s ) )
+				if ( MimeTypeIconIndexHash.ContainsKey( split[i] ) )
 					continue;
 				
-				MimeTypeIconIndexHash.Add( s, path_or_index );
+				MimeTypeIconIndexHash.Add( split[i], path_or_index );
 			}
 		}
 		
@@ -400,9 +417,9 @@ namespace System.Windows.Forms
 					{
 						string[] split = alias.Split( new char[] { ',' } );
 						
-						foreach ( string s in split )
+						for (int i = 0; i < split.Length; i++)
 						{
-							oindex = MimeTypeIconIndexHash[ s ];
+							oindex = MimeTypeIconIndexHash[ split[i] ];
 							
 							if ( oindex != null )
 								return oindex;
@@ -412,8 +429,12 @@ namespace System.Windows.Forms
 					// if oindex is still null check if mime_type is a sub class of an other mime type
 					string sub_class = Mime.SubClasses[ mime_type ];
 					
-					if ( sub_class != null )
-						return MimeTypeIconIndexHash[ sub_class ];
+					if ( sub_class != null ) {
+						oindex = MimeTypeIconIndexHash[ sub_class ];
+						
+						if ( oindex != null )
+							return oindex;
+					}
 					
 					// last check, see if we find an entry for the main mime type class
 					string mime_class_main = mime_type.Substring( 0, mime_type.IndexOf( '/' ) );
@@ -453,6 +474,7 @@ namespace System.Windows.Forms
 	
 	internal class PlatformDefaultHandler : PlatformMimeIconHandler
 	{
+		
 		public override MimeExtensionHandlerStatus Start( )
 		{
 			MimeIconEngine.AddMimeTypeAndIconName( "unknown/unknown", "paper" );
@@ -463,13 +485,15 @@ namespace System.Windows.Forms
 			MimeIconEngine.AddMimeTypeAndIconName( "recently/recently", "last_open" );
 			MimeIconEngine.AddMimeTypeAndIconName( "workplace/workplace", "monitor-computer" );
 			
-			MimeIconEngine.AddIconByImage( "folder",  (Image)Locale.GetResource( "folder" ) );
-			MimeIconEngine.AddIconByImage( "paper",  (Image)Locale.GetResource( "paper" ) );
-			MimeIconEngine.AddIconByImage( "desktop",  (Image)Locale.GetResource( "desktop" ) );
-			MimeIconEngine.AddIconByImage( "folder_with_paper",  (Image)Locale.GetResource( "folder_with_paper" ) );
-			MimeIconEngine.AddIconByImage( "monitor-planet",  (Image)Locale.GetResource( "monitor-planet" ) );
-			MimeIconEngine.AddIconByImage( "last_open",  (Image)Locale.GetResource( "last_open" ) );
-			MimeIconEngine.AddIconByImage( "monitor-computer",  (Image)Locale.GetResource( "monitor-computer" ) );
+			MimeIconEngine.AddIconByImage( "folder",  ResourceImageLoader.Get ("folder.png") );
+			MimeIconEngine.AddIconByImage( "paper",  ResourceImageLoader.Get("text-x-generic.png") );
+			MimeIconEngine.AddIconByImage( "desktop",  ResourceImageLoader.Get( "user-desktop.png" ) );
+			MimeIconEngine.AddIconByImage( "folder_with_paper",  ResourceImageLoader.Get( "document-open.png" ) );
+
+			// fix
+			MimeIconEngine.AddIconByImage( "monitor-planet",  ResourceImageLoader.Get( "document-open.png" ) );
+			MimeIconEngine.AddIconByImage( "last_open",  ResourceImageLoader.Get( "document-open.png" ) );
+			MimeIconEngine.AddIconByImage( "monitor-computer",  ResourceImageLoader.Get( "document-open.png" ) );
 			
 			return MimeExtensionHandlerStatus.OK; // return always ok
 		}
@@ -529,9 +553,9 @@ namespace System.Windows.Forms
 				string icon_path = icon_paths[ 0 ] + icon_theme;
 				string[] dirs = Directory.GetDirectories( icon_path );
 				
-				foreach( string path in dirs )
+				for (int i = 0; i < dirs.Length; i++)
 				{
-					if ( path.EndsWith( "48x48" ) )
+					if ( dirs[i].EndsWith( "48x48" ) )
 						return false;
 				}
 			}
@@ -605,17 +629,17 @@ namespace System.Windows.Forms
 				
 				string[] directories = Directory.GetDirectories( icon_path );
 				
-				foreach ( string d in directories )
+				for (int i = 0; i < directories.Length; i++)
 				{
-					DirectoryInfo di = new DirectoryInfo( d );
+					DirectoryInfo di = new DirectoryInfo( directories [i] );
 					
 					FileInfo[] fileinfo = di.GetFiles( );
 					
-					foreach ( FileInfo fi in fileinfo )
+					for (int z = 0; z < fileinfo.Length; z++)
 					{
-						string name = Path.GetFileNameWithoutExtension( fi.Name );
+						string name = Path.GetFileNameWithoutExtension( fileinfo [z].Name );
 						
-						MimeIconEngine.AddIcon( name, fi.FullName );
+						MimeIconEngine.AddIcon( name, fileinfo [z].FullName );
 					}
 				}
 			}
@@ -641,15 +665,15 @@ namespace System.Windows.Forms
 			{
 				string[] directories = Directory.GetDirectories( mime_path );
 				
-				foreach ( string d in directories )
+				for (int i = 0; i < directories.Length; i++)
 				{
-					string[] files = Directory.GetFiles( d );
+					string[] files = Directory.GetFiles( directories [i] );
 					
-					foreach ( string f in files )
+					for (int z = 0; z < files.Length; z++)
 					{
 					    try {
-						ReadDotDesktop( f );
-					    } catch {
+							ReadDotDesktop( files [z] );
+						} catch {
 						// Ignore errors if the file can not be read.
 					    }
 					}
@@ -815,7 +839,7 @@ namespace System.Windows.Forms
 					icon_theme = "gnome";
 					return false;
 				}
-			} catch ( Exception e ) {
+			} catch (Exception) {
 				return false;
 			}
 		}
@@ -888,7 +912,7 @@ namespace System.Windows.Forms
 				}
 				
 				sr.Close ();
-			} catch (Exception e) {
+			} catch (Exception) {
 				
 			}
 			
@@ -896,10 +920,12 @@ namespace System.Windows.Forms
 				foreach (string icon_theme in tmp_inherits) {
 					foreach (string path in icon_paths) {
 						if (Directory.Exists (path + "/" + icon_theme)) {
-							if (!inherits_path_collection.Contains (path + "/" + icon_theme))
+							if (!inherits_path_collection.Contains (path + "/" + icon_theme)) {
 								inherits_path_collection.Add (path + "/" + icon_theme);
-							if (File.Exists (path + "/" + icon_theme + "/" + "index.theme"))
-								GetIndexThemeInherits (path + "/" + icon_theme + "/" + "index.theme");
+								
+								if (File.Exists (path + "/" + icon_theme + "/" + "index.theme"))
+									GetIndexThemeInherits (path + "/" + icon_theme + "/" + "index.theme");
+							}
 							break;
 						}
 					}
@@ -910,17 +936,19 @@ namespace System.Windows.Forms
 		private void CreateUIIcons ()
 		{
 			string resolv_path = ResolvePath (main_icon_theme_path);
+			string default_gnome_path = "";
 			
+			// get the default gnome icon theme path
+			foreach (string path in icon_paths)
+				if (Directory.Exists (path + "/gnome")) {
+					default_gnome_path = path + "/gnome/48x48/";
+					break;
+				}
+
 			// use default gnome icon theme if there isn't a "/48x48" or "/scalable" dir
-			// for the current theme
+			// for the current theme			
 			if (resolv_path == String.Empty)
-				foreach (string path in icon_paths)
-					if (Directory.Exists (path + "/gnome")) {
-						resolv_path = path + "/gnome/48x48/";
-						break;
-					}
-			
-			string[] dirs = Directory.GetDirectories (resolv_path);
+				resolv_path = default_gnome_path;
 			
 			Hashtable name_mime_hash = new Hashtable ();
 			
@@ -939,51 +967,72 @@ namespace System.Windows.Forms
 			name_mime_hash ["gnome-dev-harddisk"] = "harddisk/harddisk";
 			name_mime_hash ["gnome-dev-removable"] = "removable/removable";
 			
-			if (!CheckAndAddUIIcons (dirs, name_mime_hash)) {
-				//could be a kde icon theme, so we check kde icon names also
-				name_mime_hash.Clear ();
-				name_mime_hash ["folder"] = "inode/directory";
-				name_mime_hash ["unknown"] = "unknown/unknown";
-				name_mime_hash ["desktop"] = "desktop/desktop";
-				name_mime_hash ["folder_home"] = "directory/home";
-				name_mime_hash ["network"] = "network/network";
-				name_mime_hash ["folder_man"] = "recently/recently";
-				name_mime_hash ["system"] = "workplace/workplace";
+			int initial_name_mime_hash_count = name_mime_hash.Count;
+			
+			// first check the current icon theme path
+			string[] dirs = Directory.GetDirectories (resolv_path);
+			ArrayList objects = CheckAndAddUIIcons (dirs, name_mime_hash);
+			
+			if (objects.Count != name_mime_hash.Count) {
+				// remove found icons
+				foreach (object o in objects) {
+					name_mime_hash.Remove (o);
+				}
 				
-				name_mime_hash ["nfs_mount"] = "nfs/nfs";
-				name_mime_hash ["server"] = "smb/smb";
+				// check the default gnome path
+				dirs = Directory.GetDirectories (default_gnome_path);
+				objects = CheckAndAddUIIcons (dirs, name_mime_hash);
 				
-				name_mime_hash ["cdrom_mount"] = "cdrom/cdrom";
-				name_mime_hash ["hdd_mount"] = "harddisk/harddisk";
-				name_mime_hash ["usbpendrive_mount"] = "removable/removable";
-				
-				CheckAndAddUIIcons (dirs, name_mime_hash);
+				//could be a kde icon theme, so we check kde icon names too
+				if (objects.Count == initial_name_mime_hash_count) {
+					dirs = Directory.GetDirectories (resolv_path);
+					
+					name_mime_hash.Clear ();
+					name_mime_hash ["folder"] = "inode/directory";
+					name_mime_hash ["unknown"] = "unknown/unknown";
+					name_mime_hash ["desktop"] = "desktop/desktop";
+					name_mime_hash ["folder_home"] = "directory/home";
+					name_mime_hash ["network"] = "network/network";
+					name_mime_hash ["folder_man"] = "recently/recently";
+					name_mime_hash ["system"] = "workplace/workplace";
+					
+					name_mime_hash ["nfs_mount"] = "nfs/nfs";
+					name_mime_hash ["server"] = "smb/smb";
+					
+					name_mime_hash ["cdrom_mount"] = "cdrom/cdrom";
+					name_mime_hash ["hdd_mount"] = "harddisk/harddisk";
+					name_mime_hash ["usbpendrive_mount"] = "removable/removable";
+					
+					CheckAndAddUIIcons (dirs, name_mime_hash);
+				}
 			}
 		}
 		
-		private bool CheckAndAddUIIcons (string[] dirs, Hashtable name_mime_hash)
+		private ArrayList CheckAndAddUIIcons (string[] dirs, Hashtable name_mime_hash)
 		{
-			string extension = is_svg_icon_theme ? "svg" : "png";
-			int counter = 0;
+			ArrayList al = new ArrayList (name_mime_hash.Count);
 			
-			foreach (string place in dirs) {
+			string extension = is_svg_icon_theme ? "svg" : "png";
+			
+			for (int i = 0; i < dirs.Length; i++) {
 				foreach (DictionaryEntry entry in name_mime_hash) {
 					string key = (string)entry.Key;
-					if (File.Exists (place + "/" + key + "." + extension)) {
+					if (File.Exists (dirs [i] + "/" + key + "." + extension)) {
 						string value = (string)entry.Value;
+						
 						MimeIconEngine.AddMimeTypeAndIconName (value, key);
+						
 						if (!is_svg_icon_theme)
-							MimeIconEngine.AddIcon (key, place + "/" + key + "." + extension);
+							MimeIconEngine.AddIcon (key, dirs [i] + "/" + key + "." + extension);
 						else
-							MimeIconEngine.AddSVGIcon (key, place + "/" + key + "." + extension);
-						counter++;
-						if (counter == name_mime_hash.Count)
-							return true;
+							MimeIconEngine.AddSVGIcon (key, dirs [i] + "/" + key + "." + extension);
+						
+						al.Add (entry.Key);
 					}
 				}
 			}
 			
-			return false;
+			return al;
 		}
 		
 		private void CreateMimeTypeIcons ()
@@ -999,8 +1048,8 @@ namespace System.Windows.Forms
 				
 				string[] files = Directory.GetFiles (path_to_use + "mimetypes");
 				
-				foreach (string file in files) {
-					string extension = Path.GetExtension (file);
+				for (int z = 0; z < files.Length; z++) {
+					string extension = Path.GetExtension (files [z]);
 					
 					if (!is_svg_icon_theme) {
 						if (extension != ".png")
@@ -1009,7 +1058,7 @@ namespace System.Windows.Forms
 					if (extension != ".svg")
 						continue;
 					
-					string file_name = Path.GetFileNameWithoutExtension (file);
+					string file_name = Path.GetFileNameWithoutExtension (files [z]);
 					
 					if (!file_name.StartsWith ("gnome-mime-"))
 						continue;
@@ -1025,9 +1074,9 @@ namespace System.Windows.Forms
 					MimeIconEngine.AddMimeTypeAndIconName (mime_type.ToString (), file_name);
 					
 					if (!is_svg_icon_theme)
-						MimeIconEngine.AddIcon (file_name, file);
+						MimeIconEngine.AddIcon (file_name, files [z]);
 					else
-						MimeIconEngine.AddSVGIcon (file_name, file);
+						MimeIconEngine.AddSVGIcon (file_name, files [z]);
 				}
 			}
 		}
