@@ -395,8 +395,8 @@ namespace System.Windows.Forms
 			if (Value >= Maximum)
 				return;
 
-			Value = Value + Step;
-			Refresh ();
+			Value = Maximum < Value + Step ? Maximum : Value + Step;
+			Refresh ();	// FIXME - calculate delta and only expose that
 		}
 
 		public override string ToString()
@@ -424,6 +424,7 @@ namespace System.Windows.Forms
     				return;
 
 			UpdateAreas ();
+			Invalidate();	// Invalidate the full surface, blocks will not match
     		}
 
 		internal override void OnPaintInternal (PaintEventArgs pevent)
