@@ -262,8 +262,6 @@ namespace System.Windows.Forms {
 			get {
 				Line		start_line;
 				Line		end_line;
-				int		current;
-				int		total;
 
 				start_line = document.GetLine(1);
 				end_line = document.GetLine(document.Lines);
@@ -279,6 +277,8 @@ namespace System.Windows.Forms {
 				InsertRTFFromStream(data, 0, 1);
 
 				data.Close();
+
+				Invalidate();
 			}
 		}
 
@@ -849,8 +849,6 @@ namespace System.Windows.Forms {
 		}
 
 		public void LoadFile(System.IO.Stream data, RichTextBoxStreamType fileType) {
-			RTF.RTF rtf;	// Not 'using SWF.RTF' to avoid ambiguities with font and color
-
 			document.Empty();
 
 			// FIXME - ignoring unicode
@@ -1279,7 +1277,7 @@ namespace System.Windows.Forms {
 				}
 
 				case RTF.Major.SpecialChar: {
-					Console.Write("[Got SpecialChar control {0}]", rtf.Minor);
+					//Console.Write("[Got SpecialChar control {0}]", rtf.Minor);
 					SpecialChar(rtf);
 					break;
 				}
@@ -1323,32 +1321,32 @@ namespace System.Windows.Forms {
 				}
 
 				case Minor.EmDash: {
-					Console.Write("");
+					Console.Write("\u2014");
 					break;
 				}
 
 				case Minor.EnDash: {
-					Console.Write("");
+					Console.Write("\u2013");
 					break;
 				}
 
 				case Minor.LQuote: {
-					Console.Write("");
+					Console.Write("\u2018");
 					break;
 				}
 
 				case Minor.RQuote: {
-					Console.Write("");
+					Console.Write("\u2019");
 					break;
 				}
 
 				case Minor.LDblQuote: {
-					Console.Write("");
+					Console.Write("\u201C");
 					break;
 				}
 
 				case Minor.RDblQuote: {
-					Console.Write("");
+					Console.Write("\u201D");
 					break;
 				}
 
@@ -1472,9 +1470,9 @@ namespace System.Windows.Forms {
 				FlushText(rtf, false);
 			}
 
-			catch (RTF.RTFException) {
-				// Seems to be plain text...
-				
+			catch (RTF.RTFException e) {
+				// Seems to be plain text or broken RTF
+				Console.WriteLine("RTF Parsing failure: {0}", e.Message);
 			}
 
 			to_x = rtf_cursor_x;
