@@ -61,11 +61,13 @@ namespace System.Windows.Forms {
 		internal ImplicitVScrollBar	vscroll;
 		internal RichTextBoxScrollBars	scrollbars;
 		internal bool			richtext;
+		internal bool			show_selection;		// set to true to always show selection, even if no focus is set
+		internal int			selection_length;	// set to the user-specified selection length, or -1 if none
 		internal int			requested_height;
 		internal int			canvas_width;
 		internal int			canvas_height;
-		static internal int		track_width = 20;
-		static internal int		track_border = 5;
+		static internal int		track_width = 2;	//
+		static internal int		track_border = 5;	//
 		internal DateTime		click_last;
 		internal CaretSelection		click_mode;
 		internal Bitmap			bmp;
@@ -93,6 +95,7 @@ namespace System.Windows.Forms {
 			read_only = false;
 			word_wrap = true;
 			richtext = false;
+			show_selection = false;
 			document = new Document(this);
 			document.WidthChanged += new EventHandler(document_WidthChanged);
 			document.HeightChanged += new EventHandler(document_HeightChanged);
@@ -135,6 +138,10 @@ namespace System.Windows.Forms {
 
 			canvas_width = ClientSize.Width;
 			canvas_height = ClientSize.Height;
+			document.ViewPortWidth = canvas_width;
+			document.ViewPortHeight = canvas_height;
+
+			Cursor = Cursors.IBeam;
 
 			CalculateScrollBars();
 		}
@@ -151,10 +158,19 @@ namespace System.Windows.Forms {
 				return s.ToUpper();
 			}
 		}
+
+		internal override void HandleClick(int clicks, MouseEventArgs me) {
+			// MS seems to fire the click event in spite of the styles they set
+			control_style |= ControlStyles.StandardClick | ControlStyles.StandardDoubleClick;
+			base.HandleClick (clicks, me);
+			control_style ^= ControlStyles.StandardClick | ControlStyles.StandardDoubleClick;
+		}
+
 		#endregion	// Private and Internal Methods
 
 		#region Public Instance Properties
 		[DefaultValue(false)]
+		[MWFCategory("Behavior")]
 		public bool AcceptsTab {
 			get {
 				return accepts_tab;
@@ -171,6 +187,7 @@ namespace System.Windows.Forms {
 		[DefaultValue(true)]
 		[Localizable(true)]
 		[RefreshProperties(RefreshProperties.Repaint)]
+		[MWFCategory("Behavior")]
 		public virtual bool AutoSize {
 			get {
 				return auto_size;
@@ -217,6 +234,7 @@ namespace System.Windows.Forms {
 
 		[DefaultValue(BorderStyle.Fixed3D)]
 		[DispId(-504)]
+		[MWFCategory("Appearance")]
 		public BorderStyle BorderStyle {
 			get { return InternalBorderStyle; }
 			set { 
@@ -244,6 +262,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(true)]
+		[MWFCategory("Behavior")]
 		public bool HideSelection {
 			get {
 				return hide_selection;
@@ -267,6 +286,7 @@ namespace System.Windows.Forms {
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		[Editor("System.Windows.Forms.Design.StringArrayEditor, " + Consts.AssemblySystem_Design, typeof(System.Drawing.Design.UITypeEditor))]
 		[Localizable(true)]
+		[MWFCategory("Layout")]
 		public string[] Lines {
 			get {
 				string[]	lines;
@@ -274,6 +294,12 @@ namespace System.Windows.Forms {
 				int		l;
 
 				l = document.Lines;
+
+				// Handle empty document
+				if ((l == 1) && (document.GetLine(1).text.Length == 0)) {
+					return new string[0];
+				}
+
 				lines = new string[l];
 
 				for (i = 1; i <= l; i++) {
@@ -303,6 +329,7 @@ namespace System.Windows.Forms {
 
 		[DefaultValue(32767)]
 		[Localizable(true)]
+		[MWFCategory("Behavior")]
 		public virtual int MaxLength {
 			get {
 				if (max_length == 2147483646) {	// We don't distinguish between single and multi-line limits
@@ -336,6 +363,7 @@ namespace System.Windows.Forms {
 		[DefaultValue(false)]
 		[Localizable(true)]
 		[RefreshProperties(RefreshProperties.All)]
+		[MWFCategory("Behavior")]
 		public virtual bool Multiline {
 			get {
 				return multiline;
@@ -380,6 +408,7 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(false)]
+		[MWFCategory("Behavior")]
 		public bool ReadOnly {
 			get {
 				return read_only;
@@ -417,18 +446,26 @@ namespace System.Windows.Forms {
 			}
 
 			set {
+				if (value < 0) {
+					throw new ArgumentException(String.Format("{0} is not a valid value", value), "value");
+				}
+
 				if (value != 0) {
 					int	start;
 					Line	line;
 					LineTag	tag;
 					int	pos;
 
+					selection_length = value;
+
 					start = document.LineTagToCharIndex(document.selection_start.line, document.selection_start.pos);
 
 					document.CharIndexToLineTag(start + value, out line, out tag, out pos);
 					document.SetSelectionEnd(line, pos);
 					document.PositionCaret(line, pos);
 				} else {
+					selection_length = -1;
+
 					document.SetSelectionEnd(document.selection_start.line, document.selection_start.pos);
 					document.PositionCaret(document.selection_start.line, document.selection_start.pos);
 				}
@@ -447,12 +484,12 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				Line	line;
-				LineTag	tag;
-				int	pos;
-
-				document.CharIndexToLineTag(value, out line, out tag, out pos);
-				document.SetSelectionStart(line, pos);
+				document.SetSelectionStart(value);
+				if (selection_length > -1 ) {
+					document.SetSelectionEnd(value + selection_length);
+				} else {
+					document.SetSelectionEnd(value);
+				}
 				ScrollToCaret();
 			}
 		}
@@ -473,7 +510,9 @@ namespace System.Windows.Forms {
 					sb = new StringBuilder();
 
 					for (i = 1; i < document.Lines; i++) {
-						sb.Append(document.GetLine(i).text.ToString() + Environment.NewLine);
+						if (i > 1)
+							sb.Append (Environment.NewLine);
+						sb.Append(document.GetLine(i).text.ToString());
 					}
 					sb.Append(document.GetLine(document.Lines).text.ToString());
 					return sb.ToString();
@@ -497,27 +536,44 @@ namespace System.Windows.Forms {
 						this.Lines = lines;
 
 						line = document.GetLine(1);
-						document.SetSelectionStart(line, 0);
 
-						line = document.GetLine(document.Lines);
-						document.SetSelectionEnd(line, line.text.Length);
-						document.PositionCaret(line, line.text.Length);
+						if (!Focused) {
+							document.SetSelectionStart(line, 0);
+
+							line = document.GetLine(document.Lines);
+							document.SetSelectionEnd(line, line.text.Length);
+							selection_length = -1;
+							document.PositionCaret(line, line.text.Length);
+						} else {
+							document.SetSelectionStart(line, 0);
+							document.SetSelectionEnd(line, 0);
+							selection_length = -1;
+							document.PositionCaret(line, 0);
+						}
 					} else {
 						document.Clear();
 						document.Add(1, CaseAdjust(value), alignment, Font, ThemeEngine.Current.ResPool.GetSolidBrush(ForeColor));
 						CalculateDocument();
 						line = document.GetLine(1);
-						document.SetSelectionStart(line, 0);
-						document.SetSelectionEnd(line, value.Length);
-						document.PositionCaret(line, value.Length);
+
+						if (!Focused) {
+							document.SetSelectionStart(line, 0);
+							document.SetSelectionEnd(line, value.Length);
+							selection_length = -1;
+							document.PositionCaret(line, value.Length);
+						} else {
+							document.SetSelectionStart(line, 0);
+							document.SetSelectionEnd(line, 0);
+							selection_length = -1;
+							document.PositionCaret(line, 0);
+						}
 					}
 				} else {
 					document.Empty();
 					CalculateDocument();
 				}
-				base.Text = value;
-				// Not needed, base.Text already fires it
-				// OnTextChanged(EventArgs.Empty);
+
+				OnTextChanged(EventArgs.Empty);
 			}
 		}
 
@@ -547,6 +603,7 @@ namespace System.Windows.Forms {
 
 		[DefaultValue(true)]
 		[Localizable(true)]
+		[MWFCategory("Behavior")]
 		public bool WordWrap {
 			get {
 				return word_wrap;
@@ -599,6 +656,7 @@ namespace System.Windows.Forms {
 			document.MoveCaret(CaretDirection.CtrlEnd);
 			document.SetSelectionStart(document.CaretLine, document.CaretPosition);
 			document.SetSelectionEnd(document.CaretLine, document.CaretPosition);
+			selection_length = -1;
 
 			OnTextChanged(EventArgs.Empty);
 		}
@@ -637,7 +695,9 @@ namespace System.Windows.Forms {
 		}
 
 		public void ScrollToCaret() {
-			this.CaretMoved(this, EventArgs.Empty);
+			if (IsHandleCreated) {
+				CaretMoved(this, EventArgs.Empty);
+			}
 		}
 
 		public void Select(int start, int length) {
@@ -652,10 +712,11 @@ namespace System.Windows.Forms {
 			last = document.GetLine(document.Lines);
 			document.SetSelectionStart(document.GetLine(1), 0);
 			document.SetSelectionEnd(last, last.text.Length);
+			selection_length = -1;
 		}
 
 		public override string ToString() {
-			return Text;
+			return String.Concat (base.ToString (), ", Text: ", Text);
 		}
 
 		public void Undo() {
@@ -666,6 +727,8 @@ namespace System.Windows.Forms {
 		#region Protected Instance Methods
 		protected override void CreateHandle() {
 			base.CreateHandle ();
+			document.AlignCaret();
+			ScrollToCaret();
 		}
 
 		protected override bool IsInputKey(Keys keyData) {
@@ -726,7 +789,7 @@ namespace System.Windows.Forms {
 		protected override void OnFontChanged(EventArgs e) {
 			base.OnFontChanged (e);
 
-			if (auto_size) {
+			if (auto_size && !multiline) {
 				if (PreferredHeight != ClientSize.Height) {
 					Height = PreferredHeight;
 				}
@@ -808,6 +871,14 @@ namespace System.Windows.Forms {
 					return false;
 				}
 
+				case Keys.A: {	// Select All (Ctrl-A)
+					if (control) {
+						SelectAll();
+						return true;
+					}
+					return false;
+				}
+
 				case Keys.Left: {
 					if (control) {
 						document.MoveCaret(CaretDirection.WordBack);
@@ -962,55 +1033,6 @@ namespace System.Windows.Forms {
 					break;
 				}
 
-
-				case Keys.Back: {
-					if (read_only) {
-						break;
-					}
-
-					// delete only deletes on the line, doesn't do the combine
-					if (document.selection_visible) {
-						document.ReplaceSelection("");
-					}
-					document.SetSelectionToCaret(true);
-
-					if (document.CaretPosition == 0) {
-						if (document.CaretLine.LineNo > 1) {
-							Line	line;
-							int	new_caret_pos;
-
-							line = document.GetLine(document.CaretLine.LineNo - 1);
-							new_caret_pos = line.text.Length;
-
-							document.Combine(line, document.CaretLine);
-							document.UpdateView(line, 1, 0);
-							document.PositionCaret(line, new_caret_pos);
-							//document.MoveCaret(CaretDirection.CharForward);
-							document.UpdateCaret();
-							OnTextChanged(EventArgs.Empty);
-						}
-					} else {
-						if (!control || document.CaretPosition == 0) {
-							document.DeleteChar(document.CaretTag, document.CaretPosition, false);
-							document.MoveCaret(CaretDirection.CharBack);
-						} else {
-							int start_pos;
-
-							start_pos = document.CaretPosition - 1;
-
-							while ((start_pos > 0) && !Document.IsWordSeparator(document.CaretLine.Text[start_pos - 1])) {
-								start_pos--;
-							}
-							document.DeleteChars(document.CaretTag, start_pos, document.CaretPosition - start_pos);
-							document.PositionCaret(document.CaretLine, start_pos);
-						}
-						document.UpdateCaret();
-						OnTextChanged(EventArgs.Empty);
-					}
-					CaretMoved(this, null);
-					return true;
-				}
-
 				case Keys.Insert: {
 					if (shift) {
 						Paste(null, true);
@@ -1026,6 +1048,24 @@ namespace System.Windows.Forms {
 					return false;
 				}
 
+				case Keys.PageUp: {
+					if ((Control.ModifierKeys & Keys.Control) != 0) {
+						document.MoveCaret(CaretDirection.CtrlPgUp);
+					} else {
+						document.MoveCaret(CaretDirection.PgUp);
+					}
+					return true;
+				}
+
+				case Keys.PageDown: {
+					if ((Control.ModifierKeys & Keys.Control) != 0) {
+						document.MoveCaret(CaretDirection.CtrlPgDn);
+					} else {
+						document.MoveCaret(CaretDirection.PgDn);
+					}
+					return true;
+				}
+
 				case Keys.Delete: {
 					if (shift) {
 						Cut();
@@ -1090,6 +1130,57 @@ namespace System.Windows.Forms {
 			return false;
 		}
 
+		private void HandleBackspace(bool control) {
+			bool	fire_changed;
+
+			fire_changed = false;
+
+			// delete only deletes on the line, doesn't do the combine
+			if (document.selection_visible) {
+				document.ReplaceSelection("");
+				fire_changed = true;
+			}
+			document.SetSelectionToCaret(true);
+
+			if (document.CaretPosition == 0) {
+				if (document.CaretLine.LineNo > 1) {
+					Line	line;
+					int	new_caret_pos;
+
+					line = document.GetLine(document.CaretLine.LineNo - 1);
+					new_caret_pos = line.text.Length;
+
+					document.Combine(line, document.CaretLine);
+					document.UpdateView(line, 1, 0);
+					document.PositionCaret(line, new_caret_pos);
+					//document.MoveCaret(CaretDirection.CharForward);
+					document.UpdateCaret();
+					fire_changed = true;
+				}
+			} else {
+				if (!control || document.CaretPosition == 0) {
+					document.DeleteChar(document.CaretTag, document.CaretPosition, false);
+					document.MoveCaret(CaretDirection.CharBack);
+				} else {
+					int start_pos;
+
+					start_pos = document.CaretPosition - 1;
+
+					while ((start_pos > 0) && !Document.IsWordSeparator(document.CaretLine.Text[start_pos - 1])) {
+						start_pos--;
+					}
+					document.DeleteChars(document.CaretTag, start_pos, document.CaretPosition - start_pos);
+					document.PositionCaret(document.CaretLine, start_pos);
+				}
+				document.UpdateCaret();
+				fire_changed = true;
+			}
+			if (fire_changed) {
+				OnTextChanged(EventArgs.Empty);
+			}
+			CaretMoved(this, null);
+		}
+
 		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
 			// Make sure we don't get sized bigger than we want to be
 			if (!richtext) {
@@ -1110,21 +1201,6 @@ namespace System.Windows.Forms {
 
 		protected override void WndProc(ref Message m) {
 			switch ((Msg)m.Msg) {
-
-				case Msg.WM_SETFOCUS: {
-					// Set caret
-					document.CaretHasFocus();
-					base.WndProc(ref m);
-					return;
-				}
-
-				case Msg.WM_KILLFOCUS: {
-					// Kill caret
-					document.CaretLostFocus();
-					base.WndProc(ref m);
-					return;
-				}
-
 				case Msg.WM_KEYDOWN: {
 					if (ProcessKeyMessage(ref m) || ProcessKey((Keys)m.WParam.ToInt32() | XplatUI.State.ModifierKeys)) {
 						m.Result = IntPtr.Zero;
@@ -1135,57 +1211,50 @@ namespace System.Windows.Forms {
 				}
 
 				case Msg.WM_CHAR: {
+					int	ch;
+
 					if (ProcessKeyMessage(ref m)) {
 						m.Result = IntPtr.Zero;
 						return;
 					}
 
-					// Ctrl-Backspace generates a real char, whack it
-					if (m.WParam.ToInt32() == 127) {
-						base.WndProc(ref m);
+					if (read_only) {
 						return;
 					}
 
-					if (!read_only && (m.WParam.ToInt32() >= 32)) {	// FIXME, tabs should probably go through
+					m.Result = IntPtr.Zero;
+
+					ch = m.WParam.ToInt32();
+
+					if (ch == 127) {
+						HandleBackspace(true);
+					} else if (ch >= 32) {
 						if (document.selection_visible) {
 							document.ReplaceSelection("");
 						}
 
+						char c = (char)m.WParam;
 						switch (character_casing) {
-							case CharacterCasing.Normal: {
-								if (document.Length < max_length) {
-									document.InsertCharAtCaret((char)m.WParam, true);
-									OnTextChanged(EventArgs.Empty);
-									CaretMoved(this, null);
-								} else {
-									XplatUI.AudibleAlert();
-								}
-								return;
-							}
-
-							case CharacterCasing.Lower: {
-								if (document.Length < max_length) {
-									document.InsertCharAtCaret(Char.ToLower((char)m.WParam), true);
-									OnTextChanged(EventArgs.Empty);
-									CaretMoved(this, null);
-								} else {
-									XplatUI.AudibleAlert();
-								}
-								return;
-							}
-
-							case CharacterCasing.Upper: {
-								if (document.Length < max_length) {
-									document.InsertCharAtCaret(Char.ToUpper((char)m.WParam), true);
-									OnTextChanged(EventArgs.Empty);
-									CaretMoved(this, null);
-								} else {
-									XplatUI.AudibleAlert();
-								}
-								return;
-							}
+						case CharacterCasing.Upper:
+							c = Char.ToUpper((char) m.WParam);
+							break;
+						case CharacterCasing.Lower:
+							c = Char.ToLower((char) m.WParam);
+							break;
 						}
+
+						if (document.Length < max_length) {
+							document.InsertCharAtCaret(c, true);
+							OnTextChanged(EventArgs.Empty);
+							CaretMoved(this, null);
+						} else {
+							XplatUI.AudibleAlert();
+						}
+						return;
+					} else if (ch == 8) {
+						HandleBackspace(false);
 					}
+
 					return;
 				}
 
@@ -1210,7 +1279,11 @@ namespace System.Windows.Forms {
 		public event EventHandler	BorderStyleChanged;
 		[Browsable(false)]
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
-		public event EventHandler	Click;
+		public event EventHandler	Click {
+			add { base.Click += value; }
+			remove { base.Click -= value; }
+		}
+
 		public event EventHandler	HideSelectionChanged;
 		public event EventHandler	ModifiedChanged;
 		public event EventHandler	MultilineChanged;
@@ -1234,6 +1307,25 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal bool ShowSelection {
+			get {
+				if (show_selection) {
+					return true;
+				}
+
+				return has_focus;
+			}
+
+			set {
+				if (show_selection == value)
+					return;
+
+				show_selection = value;
+				// Currently InvalidateSelectionArea is commented out so do a full invalidate
+				document.InvalidateSelectionArea();
+			}
+		}
+
 		internal Graphics CreateGraphicsInternal() {
 			if (IsHandleCreated) {
 				return base.CreateGraphics();
@@ -1294,6 +1386,18 @@ namespace System.Windows.Forms {
 			#endif
 		}
 
+		internal override void OnGotFocusInternal (EventArgs e)
+		{
+			document.CaretHasFocus ();
+			base.OnGotFocusInternal (e);
+		}
+
+		internal override void OnLostFocusInternal (EventArgs e)
+		{
+			document.CaretLostFocus ();
+			base.OnLostFocusInternal (e);
+		}
+
 		private void TextBoxBase_MouseDown(object sender, MouseEventArgs e) {
 			if (e.Button == MouseButtons.Left) {
 				TimeSpan interval;
@@ -1335,6 +1439,18 @@ namespace System.Windows.Forms {
 				return;
 			}
 
+			if ((e.Button == MouseButtons.Middle) && (((int)Environment.OSVersion.Platform == 4) || ((int)Environment.OSVersion.Platform == 128))) {
+				Document.Marker	marker;
+
+				marker.tag = document.FindCursor(e.X + document.ViewPortX, e.Y + document.ViewPortY, out marker.pos);
+				marker.line = marker.tag.line;
+				marker.height = marker.tag.height;
+
+				document.SetSelection(marker.line, marker.pos, marker.line, marker.pos);
+				Paste();
+
+			}
+
 			#if Debug
 				LineTag	tag;
 				Line	line;
@@ -1391,21 +1507,33 @@ namespace System.Windows.Forms {
 		}
 
 		private void TextBoxBase_SizeChanged(object sender, EventArgs e) {
-			canvas_width = ClientSize.Width;
-			canvas_height = ClientSize.Height;
+			if (hscroll.Visible) {
+				//vscroll.Maximum += hscroll.Height;
+				canvas_height = ClientSize.Height - hscroll.Height;
+			} else {
+				canvas_height = ClientSize.Height;
+			}
+
+			if (vscroll.Visible) {
+				//hscroll.Maximum += vscroll.Width;
+				canvas_width = ClientSize.Width - vscroll.Width;
+			} else {
+				canvas_width = ClientSize.Width;
+			}
+
+
 			document.ViewPortWidth = canvas_width;
 			document.ViewPortHeight = canvas_height;
 
 			// We always move them, they just might not be displayed
-			hscroll.Bounds = new Rectangle (ClientRectangle.Left, ClientRectangle.Bottom - hscroll.Height, ClientSize.Width - (vscroll.Visible ? SystemInformation.VerticalScrollBarWidth : 0), hscroll.Height);
-			vscroll.Bounds = new Rectangle (ClientRectangle.Right - vscroll.Width, ClientRectangle.Top, vscroll.Width, ClientSize.Height - (hscroll.Visible ? SystemInformation.HorizontalScrollBarHeight : 0));
+			hscroll.Bounds = new Rectangle (ClientRectangle.Left, ClientRectangle.Height - hscroll.Height, ClientSize.Width - (vscroll.Visible ? vscroll.Width : 0), hscroll.Height);
+			vscroll.Bounds = new Rectangle (ClientRectangle.Right - vscroll.Width, ClientRectangle.Top, vscroll.Width, ClientSize.Height - (hscroll.Visible ? hscroll.Height : 0));
 			
 		}
 
 		private void TextBoxBase_MouseWheel(object sender, MouseEventArgs e) {
 			Line	line;
 			int	line_no;
-			int	target;
 
 			if (!vscroll.Enabled) {
 				return;
@@ -1437,25 +1565,28 @@ namespace System.Windows.Forms {
 			}
 			document.RecalculateDocument(CreateGraphicsInternal());
 			CalculateScrollBars();
-			Invalidate();	// FIXME - do we need this?
+			Invalidate();
 		}
 
 		internal void CalculateScrollBars() {
 			// FIXME - need separate calculations for center and right alignment
 			// No scrollbars for a single line
-			if (document.Width >= ClientSize.Width) {
+
+			if (document.Width >= document.ViewPortWidth) {
 				hscroll.Enabled = true;
 				hscroll.Minimum = 0;
-				hscroll.Maximum = document.Width - ClientSize.Width + track_border;
+				hscroll.LargeChange = document.ViewPortWidth;
+				hscroll.Maximum = document.Width;
 			} else {
 				hscroll.Maximum = document.ViewPortWidth;
 				hscroll.Enabled = false;
 			}
 
-			if (document.Height >= ClientSize.Height) {
+			if (document.Height >= document.ViewPortHeight) {
 				vscroll.Enabled = true;
 				vscroll.Minimum = 0;
-				vscroll.Maximum = document.Height - ClientSize.Height + 1;
+				vscroll.LargeChange = document.ViewPortHeight;
+				vscroll.Maximum = document.Height;
 			} else {
 				vscroll.Maximum = document.ViewPortHeight;
 				vscroll.Enabled = false;
@@ -1470,24 +1601,22 @@ namespace System.Windows.Forms {
 				if ((scrollbars & RichTextBoxScrollBars.Horizontal) != 0) {
 					if (((scrollbars & RichTextBoxScrollBars.ForcedHorizontal) != 0) || hscroll.Enabled) {
 						hscroll.Visible = true;
+					} else {
+						hscroll.Visible = false;
 					}
+				} else {
+					hscroll.Visible = false;
 				}
 			}
 
 			if ((scrollbars & RichTextBoxScrollBars.Vertical) != 0) {
 				if (((scrollbars & RichTextBoxScrollBars.ForcedVertical) != 0) || vscroll.Enabled) {
 					vscroll.Visible = true;
+				} else {
+					vscroll.Visible = false;
 				}
-			}
-
-			if (hscroll.Visible) {
-				vscroll.Maximum += hscroll.Height;
-				canvas_height = ClientSize.Height - hscroll.Height;
-			}
-
-			if (vscroll.Visible) {
-				hscroll.Maximum += vscroll.Width * 2;
-				canvas_width = ClientSize.Width - vscroll.Width * 2;
+			} else {
+				vscroll.Visible = false;
 			}
 
 			TextBoxBase_SizeChanged(this, EventArgs.Empty);
@@ -1502,8 +1631,16 @@ namespace System.Windows.Forms {
 		}
 
 		private void hscroll_ValueChanged(object sender, EventArgs e) {
-			XplatUI.ScrollWindow(this.Handle, document.ViewPortX-this.hscroll.Value, 0, false);
+			int old_viewport_x;
+
+			old_viewport_x = document.ViewPortX;
 			document.ViewPortX = this.hscroll.Value;
+
+			if (vscroll.Visible) {
+				XplatUI.ScrollWindow(this.Handle, new Rectangle(0, 0, ClientSize.Width - vscroll.Width, ClientSize.Height), old_viewport_x - this.hscroll.Value, 0, false);
+			} else {
+				XplatUI.ScrollWindow(this.Handle, ClientRectangle, old_viewport_x - this.hscroll.Value, 0, false);
+			}
 			document.UpdateCaret();
 
 			if (HScrolled != null) {
@@ -1512,8 +1649,16 @@ namespace System.Windows.Forms {
 		}
 
 		private void vscroll_ValueChanged(object sender, EventArgs e) {
-			XplatUI.ScrollWindow(this.Handle, 0, document.ViewPortY-this.vscroll.Value, false);
+			int old_viewport_y;
+
+			old_viewport_y = document.ViewPortY;
 			document.ViewPortY = this.vscroll.Value;
+
+			if (hscroll.Visible) {
+				XplatUI.ScrollWindow(this.Handle, new Rectangle(0, 0, ClientSize.Width, ClientSize.Height - hscroll.Height), 0, old_viewport_y - this.vscroll.Value, false);
+			} else {
+				XplatUI.ScrollWindow(this.Handle, ClientRectangle, 0, old_viewport_y - this.vscroll.Value, false);
+			}
 			document.UpdateCaret();
 
 			if (VScrolled != null) {
@@ -1557,40 +1702,45 @@ namespace System.Windows.Forms {
 			pos = document.Caret;
 			//Console.WriteLine("Caret now at {0} (Thumb: {1}x{2}, Canvas: {3}x{4}, Document {5}x{6})", pos, hscroll.Value, vscroll.Value, canvas_width, canvas_height, document.Width, document.Height);
 
+
+			// Horizontal scrolling:
+			// If the caret moves to the left outside the visible area, we jump the document into view, not just one
+			// character, but 1/3 of the width of the document
+			// If the caret moves to the right outside the visible area, we scroll just enough to keep the caret visible
+
 			// Handle horizontal scrolling
 			if (document.CaretLine.alignment == HorizontalAlignment.Left) {
-				if (pos.X < (document.ViewPortX + track_width)) {
+				// Check if we moved out of view to the left
+				if (pos.X < (document.ViewPortX)) {
 					do {
-						if ((hscroll.Value - track_width) >= hscroll.Minimum) {
-							hscroll.Value -= track_width;
+						if ((hscroll.Value - document.ViewPortWidth / 3) >= hscroll.Minimum) {
+							hscroll.Value -= document.ViewPortWidth / 3;
 						} else {
 							hscroll.Value = hscroll.Minimum;
 						}
 					} while (hscroll.Value > pos.X);
 				}
 
-				if ((pos.X > (this.canvas_width + document.ViewPortX - track_width)) && (hscroll.Enabled && (hscroll.Value != hscroll.Maximum))) {
-					do {
-						if ((hscroll.Value + track_width) <= hscroll.Maximum) {
-							hscroll.Value += track_width;
+				// Check if we moved out of view to the right
+				if ((pos.X >= (document.ViewPortWidth + document.ViewPortX)) && (hscroll.Enabled && (hscroll.Value != hscroll.Maximum))) {
+					if ((pos.X - document.ViewPortWidth + 1) <= hscroll.Maximum) {
+						if (pos.X - document.ViewPortWidth >= 0) {
+							hscroll.Value = pos.X - document.ViewPortWidth + 1;
 						} else {
-							hscroll.Value = hscroll.Maximum;
+							hscroll.Value = 0;
 						}
-					} while (pos.X > (hscroll.Value + this.canvas_width));
-				}
-			} else if (document.CaretLine.alignment == HorizontalAlignment.Right) {
-				if (pos.X < document.ViewPortX) {
-					if (pos.X > hscroll.Minimum) {
-						hscroll.Value = pos.X;
 					} else {
-						hscroll.Value = hscroll.Minimum;
+						hscroll.Value = hscroll.Maximum;
 					}
 				}
+			} else if (document.CaretLine.alignment == HorizontalAlignment.Right) {
+//				hscroll.Value = pos.X;
 
-				if ((pos.X > (this.canvas_width + document.ViewPortX)) && (hscroll.Enabled && (hscroll.Value != hscroll.Maximum))) {
-					hscroll.Value = hscroll.Maximum;
-				}
+//				if ((pos.X > (this.canvas_width + document.ViewPortX)) && (hscroll.Enabled && (hscroll.Value != hscroll.Maximum))) {
+//					hscroll.Value = hscroll.Maximum;
+//				}
 			} else {
+				// FIXME - implement center cursor alignment
 			}
 
 			if (!multiline) {
