@@ -468,7 +468,7 @@ namespace System.Windows.Forms
 
 		public virtual int MenuHeight {
 			get {
-				return 19;
+				return XplatUI.MenuHeight;
 			}
 		}
 
@@ -548,52 +548,45 @@ namespace System.Windows.Forms
 			}
 		}
 
-		private Image GetSizedResourceImage(string name, int size) {
-			
-			Image image = ResPool.GetUIImage (name, size);
+		//
+		// This routine fetches images embedded as assembly resources (not
+		// resgen resources).  It optionally scales the image to fit the
+		// specified size x dimension (it adjusts y automatically to fit that).
+		//
+		private Image GetSizedResourceImage(string name, int width)
+		{
+			Image image = ResPool.GetUIImage (name, width);
 			if (image != null)
 				return image;
 			
 			string	fullname;
 
-			if (size > 0) {
-				// Try name name_sizexsize
-				fullname = String.Format("{0}_{1}x{1}", name, size);
-				image = ResPool.GetUIImage (fullname, size);
-				if (image != null)
-					return image;
-				else {
-					image = (Image)Locale.GetResource(fullname);
-					if (image != null) {
-						ResPool.AddUIImage (image, fullname, size);
-						return image;
-					}
-				}
-
-				// Try name_size
-				fullname = String.Format("{0}_{1}", name, size);
-				image = ResPool.GetUIImage (fullname, size);
-				if (image != null)
-					return image;
-				else {
-					image = (Image)Locale.GetResource(fullname);
-					if (image != null) {
-						ResPool.AddUIImage (image, fullname, size);
-						return image;
-					}
-				}
-				
-				image = (Image)Locale.GetResource(name);
-				if (image != null) {
-					image = new Bitmap (image, new Size (size, size));
-					ResPool.AddUIImage (image, name, size);
+			if (width > 0) {
+				// Try name_width
+				fullname = String.Format("{1}_{0}", name, width);
+				image = ResourceImageLoader.Get (fullname);
+				if (image != null){
+					ResPool.AddUIImage (image, name, width);
 					return image;
 				}
 			}
 
 			// Just try name
-			image = (Image)Locale.GetResource(name);
-			ResPool.AddUIImage (image, name, size);
+			image = ResourceImageLoader.Get (name);
+			if (image == null)
+				return null;
+			
+			ResPool.AddUIImage (image, name, 0);
+			if (image.Width != width && width != 0){
+				Console.Error.WriteLine ("warning: requesting icon that not been tuned {0}_{1} {2}", width, name, image.Width);
+				int height = (image.Height * width)/image.Width;
+				Bitmap b = new Bitmap (width, height);
+				Graphics g = Graphics.FromImage (b);
+				g.DrawImage (image, 0, 0, width, height);
+				ResPool.AddUIImage (b, name, width);
+
+				return b;
+			}
 			return image;
 		}
 		
@@ -603,26 +596,35 @@ namespace System.Windows.Forms
 			
 		public virtual Image Images(UIIcon index, int size) {
 			switch (index) {
-				case UIIcon.PlacesRecentDocuments:	return GetSizedResourceImage ("last_open", size);
-				case UIIcon.PlacesDesktop:		return GetSizedResourceImage ("desktop", size);
-				case UIIcon.PlacesPersonal:		return GetSizedResourceImage ("folder_with_paper", size);
-				case UIIcon.PlacesMyComputer:		return GetSizedResourceImage ("monitor-computer", size);
-				case UIIcon.PlacesMyNetwork:		return GetSizedResourceImage ("monitor-planet", size);
+				case UIIcon.PlacesRecentDocuments:
+					return GetSizedResourceImage ("document-open.png", size);
+				case UIIcon.PlacesDesktop:
+					return GetSizedResourceImage ("user-desktop.png", size);
+				case UIIcon.PlacesPersonal:
+					return GetSizedResourceImage ("document-open.png", size);
+				case UIIcon.PlacesMyComputer:
+					return GetSizedResourceImage ("computer.png", size);
+				case UIIcon.PlacesMyNetwork:
+					return GetSizedResourceImage ("folder-remote.png", size);
 
 				// Icons for message boxes
-				case UIIcon.MessageBoxError:		return GetSizedResourceImage ("mbox_error.png", size);
-				case UIIcon.MessageBoxInfo:		return GetSizedResourceImage ("mbox_info.png", size);
-				case UIIcon.MessageBoxQuestion:		return GetSizedResourceImage ("mbox_question.png", size);
-				case UIIcon.MessageBoxWarning:		return GetSizedResourceImage ("mbox_warn.png", size);
+				case UIIcon.MessageBoxError:
+					return GetSizedResourceImage ("dialog-error.png", size);
+				case UIIcon.MessageBoxInfo:
+					return GetSizedResourceImage ("dialog-information.png", size);
+				case UIIcon.MessageBoxQuestion:
+					return GetSizedResourceImage ("dialog-question.png", size);
+				case UIIcon.MessageBoxWarning:
+					return GetSizedResourceImage ("dialog-warning.png", size);
 				
 				// misc Icons
-				case UIIcon.NormalFolder:		return GetSizedResourceImage ("folder", size);
+				case UIIcon.NormalFolder:
+					return GetSizedResourceImage ("folder.png", size);
 
 				default: {
 					throw new ArgumentException("Invalid Icon type requested", "index");
 				}
 			}
-			return null;
 		}
 
 		public virtual Image Images(string mimetype, string extension, int size) {
@@ -660,7 +662,6 @@ namespace System.Windows.Forms
 		#region CheckedListBox
 		// Drawing		
 		public abstract void DrawCheckedListBoxItem (CheckedListBox ctrl, DrawItemEventArgs e);
-		public abstract Rectangle CheckedListBoxCheckRectangle ();
 		#endregion // CheckedListBox
 		
 		#region ComboBox
@@ -694,13 +695,15 @@ namespace System.Windows.Forms
 		// Paint		
 		public abstract void DataGridPaint (PaintEventArgs pe, DataGrid grid);
 		public abstract void DataGridPaintCaption (Graphics g, Rectangle clip, DataGrid grid);
-		public abstract void DataGridPaintColumnsHdrs (Graphics g, Rectangle clip, DataGrid grid);
-		public abstract void DataGridPaintRowsHeaders (Graphics g, Rectangle clip, DataGrid grid);
+		public abstract void DataGridPaintColumnHeaders (Graphics g, Rectangle clip, DataGrid grid);
+		public abstract void DataGridPaintRowContents (Graphics g, int row, Rectangle row_rect, bool is_newrow, Rectangle clip, DataGrid grid);
 		public abstract void DataGridPaintRowHeader (Graphics g, Rectangle bounds, int row, DataGrid grid);
 		public abstract void DataGridPaintRowHeaderArrow (Graphics g, Rectangle bounds, DataGrid grid);
+		public abstract void DataGridPaintParentRows (Graphics g, Rectangle bounds, DataGrid grid);
+		public abstract void DataGridPaintParentRow (Graphics g, Rectangle bounds, DataGridDataSource row, DataGrid grid);
 		public abstract void DataGridPaintRows (Graphics g, Rectangle cells, Rectangle clip, DataGrid grid);
-		public abstract void DataGridPaintRow (Graphics g, int row, Rectangle row_rect, bool is_newrow, DataGrid grid);
-		
+		public abstract void DataGridPaintRow (Graphics g, int row, Rectangle row_rect, bool is_newrow, Rectangle clip, DataGrid grid);
+		public abstract void DataGridPaintRelationRow (Graphics g, int row, Rectangle row_rect, bool is_newrow, Rectangle clip, DataGrid grid);
 		
 		#endregion // Datagrid
 
@@ -781,6 +784,12 @@ namespace System.Windows.Forms
 		public abstract Size PictureBoxDefaultSize{get;}
 		#endregion	// PictureBox
 
+		#region PrintPreviewControl
+		public abstract int PrintPreviewControlPadding{get;}
+		public abstract Size PrintPreviewControlGetPageSize (PrintPreviewControl preview);
+		public abstract void PrintPreviewControlPaint (PaintEventArgs pe, PrintPreviewControl preview, Size page_image_size);
+		#endregion      // PrintPreviewControl
+
 		#region ProgressBar
 		// Drawing
 		public abstract void DrawProgressBar (Graphics dc, Rectangle clip_rectangle, ProgressBar progress_bar);
@@ -840,6 +849,7 @@ namespace System.Windows.Forms
 		public abstract int ToolBarDropDownArrowWidth { get; }	 // width for the dropdown arrow on the ToolBarButton
 		public abstract int ToolBarDropDownArrowHeight { get; }	 // height for the dropdown arrow on the ToolBarButton
 		public abstract Size ToolBarDefaultSize{get;}
+		public abstract bool ToolBarInvalidateEntireButton { get; }
 		#endregion	// ToolBar
 
 		#region ToolTip
@@ -864,6 +874,40 @@ namespace System.Windows.Forms
 		public abstract Size TreeViewDefaultSize { get; }
 		#endregion
 
+		public virtual void DrawManagedWindowDecorations (Graphics dc, Rectangle clip, InternalWindowManager wm)
+		{
+			// Just making virtual for now so all the themes still build.
+		}
+
+		public virtual int ManagedWindowTitleBarHeight (InternalWindowManager wm)
+		{
+			// Just making virtual for now so all the themes still build.
+			return 15;
+		}
+
+		public virtual int ManagedWindowBorderWidth (InternalWindowManager wm)
+		{
+			// Just making virtual for now so all the themes still build.
+			return 3;
+		}
+
+		public virtual int ManagedWindowIconWidth (InternalWindowManager wm)
+		{
+			// Just making virtual for now so all the themes still build.
+			return ManagedWindowTitleBarHeight (wm) - 5;
+		}
+
+		public virtual Size ManagedWindowButtonSize (InternalWindowManager wm)
+		{
+			// Just making virtual for now so all the themes still build.
+			return new Size (10, 10);
+		}
+
+		public virtual void ManagedWindowSetButtonLocations (InternalWindowManager wm)
+		{
+			// Just making virtual for now so all the themes still build.
+		}
+
 		#region	ControlPaint Methods
 		public abstract void CPDrawBorder (Graphics graphics, Rectangle bounds, Color leftColor, int leftWidth,
 			ButtonBorderStyle leftStyle, Color topColor, int topWidth, ButtonBorderStyle topStyle,
