@@ -464,19 +464,6 @@ namespace System.Windows.Forms
 		}
 		#endregion	// ProgressBar
 		
-		#region RadioButton
-		protected override void RadioButton_DrawFocus(RadioButton radio_button, Graphics dc, Rectangle text_rectangle)
-		{
-			if (radio_button.Focused && radio_button.appearance != Appearance.Button) 
-			{
-				if (radio_button.FlatStyle != FlatStyle.Flat && radio_button.FlatStyle != FlatStyle.Popup) 
-				{
-					DrawInnerFocusRectangle (dc, text_rectangle, radio_button.BackColor);
-				} 
-			}
-		}
-		#endregion	// RadioButton
-		
 		#region ScrollBar
 		protected override void ScrollBar_DrawThumb( ScrollBar bar, Rectangle thumb_pos, Rectangle clip, Graphics dc )
 		{
@@ -963,6 +950,10 @@ namespace System.Windows.Forms
 		}
 		
 		#region ToolBar
+		public override bool ToolBarInvalidateEntireButton {
+			get { return true; }
+		}
+
 		public  override void DrawToolBar (Graphics dc, Rectangle clip_rectangle, ToolBar control) 
 		{
 			StringFormat format = new StringFormat ();
@@ -1097,6 +1088,7 @@ namespace System.Windows.Forms
 			else
 				CPDrawStringDisabled (dc, button.Text, control.Font, ColorControlLight, button.TextRectangle, format);
 		}
+
 		#endregion	// ToolBar
 		
 //		public override void CPDrawBorder3D (Graphics graphics, Rectangle rectangle, Border3DStyle style, Border3DSide sides) {
