@@ -113,6 +113,8 @@ namespace System.Windows.Forms
 			buttons = new ToolBarButtonCollection (this);
 			dock_style = DockStyle.Top;
 			
+			GotFocus += new EventHandler (FocusChanged);
+			LostFocus += new EventHandler (FocusChanged);
 			MouseDown += new MouseEventHandler (ToolBar_MouseDown);
 			MouseLeave += new EventHandler (ToolBar_MouseLeave);
 			MouseMove += new MouseEventHandler (ToolBar_MouseMove);
@@ -150,7 +152,7 @@ namespace System.Windows.Forms
 					return;
 
 				appearance = value;
-				Redraw (false);
+				Redraw (true);
 			}
 		}
 
@@ -221,14 +223,17 @@ namespace System.Windows.Forms
 			get {
 				if (button_size.IsEmpty) {
 					if (buttons.Count == 0)
-						return new Size (39, 36);
+						return new Size (24, 22);
+					Size result = CalcButtonSize ();
+					if (result.IsEmpty)
+						return new Size (24, 22);
 					else
-						return CalcButtonSize ();
+						return result;
 				}
 				return button_size;
 			}
 			set {
-				size_specified = true;
+				size_specified = value != Size.Empty;
 				if (button_size == value)
 					return;
 
@@ -292,7 +297,12 @@ namespace System.Windows.Forms
 		[DefaultValue (null)]
 		public ImageList ImageList {
 			get { return image_list; }
-			set { image_list = value; }
+			set { 
+				if (image_list == value)
+					return;
+				image_list = value;
+				Redraw (true);
+			}
 		}
 
 		[Browsable (false)]
@@ -434,8 +444,7 @@ namespace System.Windows.Forms
 			}
 			e.Button.pressed = false;
 
-			Invalidate (e.Button.Rectangle);
-			Redraw (false);
+			e.Button.InvalidateBorder ();
 
 			if (ButtonClick != null)
 				ButtonClick (this, e);
@@ -474,11 +483,18 @@ namespace System.Windows.Forms
 			if (Width <= 0 || Height <= 0 || !Visible)
 				return;
 
-			Redraw (true);
+			Redraw (true, background_image != null);
 		}
 
+		bool height_specified = false;
+		int requested_height = -1;
+
 		protected override void SetBoundsCore (int x, int y, int width, int height, BoundsSpecified specified)
 		{
+			if ((specified & BoundsSpecified.Height) != 0) {
+				requested_height = height;
+				height_specified = true;
+			}
 			base.SetBoundsCore (x, y, width, height, specified);
 		}
 
@@ -487,9 +503,91 @@ namespace System.Windows.Forms
 			base.WndProc (ref m);
 		}
 
+		internal override bool InternalPreProcessMessage (ref Message msg)
+		{
+			if (msg.Msg == (int)Msg.WM_KEYDOWN) {
+				Keys key_data = (Keys)msg.WParam.ToInt32();
+				if (HandleKeyDown (key_data))
+					return true;
+			} 
+			return base.InternalPreProcessMessage (ref msg);
+		}
+			
 		#endregion Protected Methods
 
 		#region Private Methods
+		private void FocusChanged (object sender, EventArgs args)
+		{
+			if (Appearance != ToolBarAppearance.Flat || Buttons.Count == 0)
+				return;
+
+			ToolBarButton prelit = null;
+			foreach (ToolBarButton b in Buttons)
+				if (b.Hilight) {
+					prelit = b;
+					break;
+				}
+
+			if (Focused && prelit == null)
+				foreach (ToolBarButton btn in Buttons) {
+					if (btn.Enabled) {
+						btn.Hilight = true;
+						break;
+					}
+				}
+			else if (prelit != null)
+				prelit.Hilight = false;
+		}
+
+		private bool HandleKeyDown (Keys key_data)
+		{
+			if (Appearance != ToolBarAppearance.Flat || Buttons.Count == 0)
+				return false;
+
+			switch (key_data) {
+			case Keys.Left:
+			case Keys.Up:
+				HighlightButton (-1);
+				return true;
+			case Keys.Right:
+			case Keys.Down:
+				HighlightButton (1);
+				return true;
+			default:
+				return false;
+			}
+		}
+
+		void HighlightButton (int offset)
+		{
+			ArrayList enabled = new ArrayList ();
+			int count = 0;
+			int start = -1;
+			ToolBarButton curr_button = null;
+			foreach (ToolBarButton btn in Buttons) {
+				if (btn.Hilight) {
+					start = count;
+					curr_button = btn;
+				}
+
+				if (btn.Enabled) {
+					enabled.Add (btn);
+					count++;
+				}
+			}
+
+			int next = (start + offset) % count;
+			if (next < 0)
+				next = count - 1;
+
+			if (next == start)
+				return;
+
+			if (curr_button != null)
+				curr_button.Hilight = false;
+			(enabled [next] as ToolBarButton).Hilight = true;
+		}
+
 		private void ToolBar_MouseDown (object sender, MouseEventArgs me)
 		{
 			if (!Enabled) 
@@ -516,7 +614,7 @@ namespace System.Windows.Forms
 					}
 					button.pressed = true;
 					button.inside = true;
-					Invalidate (button.Rectangle);
+					button.InvalidateBorder ();
 					break;
 				}
 			}
@@ -530,6 +628,8 @@ namespace System.Windows.Forms
 			Point loc = new Point (me.X, me.Y);
 
 			// draw the normal button
+			// Make a copy in case the list is modified during enumeration
+			ArrayList buttons = new ArrayList (this.buttons);
 			foreach (ToolBarButton button in buttons) {
 				if (button.Enabled && button.Rectangle.Contains (loc)) {
 					if (button.Style == ToolBarButtonStyle.DropDownButton) {
@@ -547,7 +647,7 @@ namespace System.Windows.Forms
 						OnButtonClick (new ToolBarButtonClickEventArgs (button));
 				} else if (button.pressed) {
 					button.pressed = false;
-					Invalidate (button.Rectangle);
+					button.InvalidateBorder ();
 				}
 			}
 		}
@@ -559,8 +659,6 @@ namespace System.Windows.Forms
 
 			if (current_button.Hilight) {
 				current_button.Hilight = false;
-				Invalidate (current_button.Rectangle);
-				Redraw (false);
 			}
 			current_button = null;
 		}
@@ -580,8 +678,6 @@ namespace System.Windows.Forms
 					    (button.inside != button.Rectangle.Contains (loc))) {
 						button.inside = button.Rectangle.Contains (loc);
 						button.Hilight = false;
-						Invalidate (button.Rectangle);
-						Redraw (false);
 						break;
 					}
 				}
@@ -592,8 +688,6 @@ namespace System.Windows.Forms
 					if (current_button.Hilight || current_button.Pushed)
 						return;
 					current_button.Hilight = true;
-					Invalidate (current_button.Rectangle);
-					Redraw (false);
 				}
 				else {
 					foreach (ToolBarButton button in buttons) {
@@ -602,13 +696,9 @@ namespace System.Windows.Forms
 							if (current_button.Hilight || current_button.Pushed)
 								continue;
 							current_button.Hilight = true;
-							Invalidate (current_button.Rectangle);
-							Redraw (false);
 						}
 						else if (button.Hilight) {
 							button.Hilight = false;
-							Invalidate (button.Rectangle);
-							Redraw (false);
 						}
 					}
 				}
@@ -622,48 +712,95 @@ namespace System.Windows.Forms
 
 		internal void Redraw (bool recalculate)
 		{
-			if (recalculate)
-				Layout ();
-
-			Refresh ();
+			Redraw (recalculate, true);
 		}
 
+		internal void Redraw (bool recalculate, bool force)
+		{
+			bool invalidate = true;
+			if (recalculate) {
+				invalidate = Layout ();
+			}
+
+			if (force || invalidate)
+				Invalidate ();
+		}
+
+		internal bool SizeSpecified {
+			get { return size_specified; }
+		}
+
+		const int text_padding = 3;
+
 		private Size CalcButtonSize ()
 		{
-			String longestText = buttons [0].Text;
-			for (int i = 1; i < buttons.Count; i++) {
-				if (buttons[i].Text.Length > longestText.Length)
-					longestText = buttons[i].Text;
+			if (Buttons.Count == 0)
+				return Size.Empty;
+
+			string longest_text = Buttons [0].Text;
+			for (int i = 1; i < Buttons.Count; i++) {
+				if (Buttons[i].Text.Length > longest_text.Length)
+					longest_text = Buttons[i].Text;
 			}
 
-			SizeF sz = this.DeviceContext.MeasureString (longestText, this.Font);
-			Size size = new Size ((int) Math.Ceiling (sz.Width), (int) Math.Ceiling (sz.Height));
-
-			if (ImageList != null) {
-				// adjustment for the image grip 
-				int imgWidth = this.ImageSize.Width + 2 * ThemeEngine.Current.ToolBarImageGripWidth; 
-				int imgHeight = this.ImageSize.Height + 2 * ThemeEngine.Current.ToolBarImageGripWidth;
-
-				if (text_alignment == ToolBarTextAlign.Right) {
-					size.Width = imgWidth + size.Width;
-					size.Height = (size.Height > imgHeight) ? size.Height : imgHeight;
-				}
-				else {
-					size.Height = imgHeight + size.Height;
-					size.Width = (size.Width > imgWidth) ? size.Width : imgWidth;
-				}
+			Size size = Size.Empty;
+			if (longest_text != null && longest_text.Length > 0) {
+				SizeF sz = DeviceContext.MeasureString (longest_text, Font);
+				if (sz != SizeF.Empty)
+					size = new Size ((int) Math.Ceiling (sz.Width) + 2 * text_padding, (int) Math.Ceiling (sz.Height));
 			}
+
+			Size img_size = ImageList == null ? new Size (16, 16) : ImageSize;
+
+			Theme theme = ThemeEngine.Current;
+			int imgWidth = img_size.Width + 2 * theme.ToolBarImageGripWidth; 
+			int imgHeight = img_size.Height + 2 * theme.ToolBarImageGripWidth;
+
+			if (text_alignment == ToolBarTextAlign.Right) {
+				size.Width = imgWidth + size.Width;
+				size.Height = (size.Height > imgHeight) ? size.Height : imgHeight;
+			} else {
+				size.Height = imgHeight + size.Height;
+				size.Width = (size.Width > imgWidth) ? size.Width : imgWidth;
+			}
+
+			size.Width += theme.ToolBarImageGripWidth;
+			size.Height += theme.ToolBarImageGripWidth;
 			return size;
 		}
 
-		void Layout ()
+		// Flat toolbars disregard specified sizes.  Normal toolbars grow the
+		// button size to be at least large enough to show the image.
+		Size AdjustedButtonSize {
+			get {
+				Size size = ButtonSize;
+				if (size_specified) {
+					if (Appearance == ToolBarAppearance.Flat)
+						size = CalcButtonSize ();
+					else {
+						int grip = ThemeEngine.Current.ToolBarImageGripWidth;
+						if (size.Width < ImageSize.Width + 2 * grip )
+							size.Width = ImageSize.Width + 2 * grip;
+						if (size.Height < ImageSize.Height + 2 * grip)
+							size.Height = ImageSize.Height + 2 * grip;
+					}
+				}
+				return size;
+			}
+		}
+
+		bool Layout ()
 		{
+			bool changed = false;
 			Theme theme = ThemeEngine.Current;
-			int ht = ButtonSize.Height + theme.ToolBarGripWidth;
 			int x = theme.ToolBarGripWidth;
 			int y = theme.ToolBarGripWidth;
 
-			if (Wrappable) {
+			Size button_size = AdjustedButtonSize;
+
+			int ht = button_size.Height + theme.ToolBarGripWidth;
+
+			if (Wrappable && Parent != null) {
 				int separator_index = -1;
 
 				for (int i = 0; i < buttons.Count; i++) {
@@ -672,14 +809,20 @@ namespace System.Windows.Forms
 					if (!button.Visible)
 						continue;
 
-					if (size_specified)
-						button.Layout (ButtonSize);
-					else
-						button.Layout ();
+					if (size_specified) {
+						if (button.Layout (button_size))
+							changed = true;
+					}
+					else {
+						if (button.Layout ())
+							changed = true;
+					}
 
 					bool is_separator = button.Style == ToolBarButtonStyle.Separator;
 
 					if (x + button.Rectangle.Width < Width || is_separator) {
+						if (button.Location.X != x || button.Location.Y != y)
+							changed = true;
 						button.Location = new Point (x, y);
 						x += button.Rectangle.Width;
 						if (is_separator)
@@ -692,6 +835,8 @@ namespace System.Windows.Forms
 					} else {
 						x = theme.ToolBarGripWidth;
 						y += ht; 
+						if (button.Location.X != x || button.Location.Y != y)
+							changed = true;
 						button.Location = new Point (x, y);
 						x += button.Rectangle.Width;
 					}
@@ -701,17 +846,25 @@ namespace System.Windows.Forms
 			} else {
 				if (AutoSize)
 					Height = ht;
-				else
+				else if (!height_specified)
 					Height = DefaultSize.Height;
 				foreach (ToolBarButton button in buttons) {
-					if (size_specified)
-						button.Layout (ButtonSize);
-					else
-						button.Layout ();
+					if (size_specified) {
+						if (button.Layout (button_size))
+							changed = true;
+					}
+					else {
+						if (button.Layout ())
+							changed = true;
+					}
+					if (button.Location.X != x || button.Location.Y != y)
+						changed = true;
 					button.Location = new Point (x, y);
 					x += button.Rectangle.Width;
 				}
 			}
+
+			return changed;
 		}
  		#endregion Private Methods
 
