@@ -81,7 +81,7 @@ namespace System.Windows.Forms {
 				if (index < 0 || index >= Count)
 					throw new ArgumentOutOfRangeException ("index");
 				TreeNode node = (TreeNode) value;
-				node.parent = owner;
+				SetupNode (node);
 				nodes [index] = node;
 			}
 		}
@@ -95,7 +95,7 @@ namespace System.Windows.Forms {
 			set {
 				if (index < 0 || index >= Count)
 					throw new ArgumentOutOfRangeException ("index");
-				value.parent = owner;
+				SetupNode (value);
 				nodes [index] = value;
 			}
 		}
@@ -112,29 +112,19 @@ namespace System.Windows.Forms {
 			if (node == null)
 				throw new ArgumentNullException("node");
 
-			// Remove it from any old parents
-			node.Remove ();
-
 			int res;
 			TreeView tree_view = null;
-			if (owner != null)
-				tree_view = owner.TreeView;
+
 			if (tree_view != null && tree_view.Sorted) {
 				res = AddSorted (node);
 			} else {
-				node.parent = owner;
 				if (count >= nodes.Length)
 					Grow ();
 				nodes [count++] = node;
 				res = count;
 			}
 
-			if (owner != null && tree_view != null && (owner.IsExpanded || owner.IsRoot)) {
-				// XXX: Need to ensure the boxes for the nodes have been created
-				tree_view.UpdateBelow (owner);
-			} else if (owner != null && tree_view != null) {
-				tree_view.UpdateBelow (owner);
-			}
+			SetupNode (node);
 
 			return res;
 		}
@@ -152,8 +142,8 @@ namespace System.Windows.Forms {
 
 		public virtual void Clear ()
 		{
-			for (int i = 0; i < count; i++)
-				RemoveAt (i, false);
+			while (count > 0)
+				RemoveAt (0, false);
 			
 			Array.Clear (nodes, 0, count);
 			count = 0;
@@ -163,8 +153,10 @@ namespace System.Windows.Forms {
 				tree_view = owner.TreeView;
 				if (owner.IsRoot)
 					tree_view.top_node = null;
-				if (tree_view != null)
+				if (tree_view != null) {
 					tree_view.UpdateBelow (owner);
+					tree_view.RecalculateVisibleOrder (owner);
+				}
 			}
 		}
 
@@ -175,7 +167,7 @@ namespace System.Windows.Forms {
 
 		public void CopyTo (Array dest, int index)
 		{
-			nodes.CopyTo (dest, index);
+			Array.Copy (nodes, index, dest, index, count);
 		}
 
 		public IEnumerator GetEnumerator ()
@@ -190,14 +182,14 @@ namespace System.Windows.Forms {
 
 		public virtual void Insert (int index, TreeNode node)
 		{
-			node.parent = owner;
-
 			if (count >= nodes.Length)
 				Grow ();
 
 			Array.Copy (nodes, index, nodes, index + 1, count - index);
 			nodes [index] = node;
 			count++;
+
+			SetupNode (node);
 		}
 
 		public void Remove (TreeNode node)
@@ -215,28 +207,104 @@ namespace System.Windows.Forms {
 		private void RemoveAt (int index, bool update)
 		{
 			TreeNode removed = nodes [index];
-			
+			TreeNode prev = GetPrevNode (removed);
+			TreeNode new_selected = null;
+			bool visible = removed.IsVisible;
+
+                        TreeView tree_view = null;
+			if (owner != null)
+				tree_view = owner.TreeView;
+
+			if (tree_view != null) {
+				tree_view.RecalculateVisibleOrder (prev);
+				if (removed == tree_view.top_node) {
+
+					if (removed.IsRoot) {
+						tree_view.top_node = null;
+					} else {
+						OpenTreeNodeEnumerator oe = new OpenTreeNodeEnumerator (removed);
+						if (oe.MovePrevious () && oe.MovePrevious ()) {
+							tree_view.top_node = oe.CurrentNode;
+						} else {
+							removed.is_expanded = false;
+							oe = new OpenTreeNodeEnumerator (removed);
+							if (oe.MoveNext () && oe.MoveNext ()) {
+								tree_view.top_node = oe.CurrentNode;
+							} else {
+								tree_view.top_node = null;
+							}
+						}
+					}
+				}
+				if (removed == tree_view.selected_node) {
+					OpenTreeNodeEnumerator oe = new OpenTreeNodeEnumerator (removed);
+					if (oe.MoveNext () && oe.MoveNext ()) {
+						new_selected = oe.CurrentNode;
+					} else {
+						oe = new OpenTreeNodeEnumerator (removed);
+						oe.MovePrevious ();
+						new_selected = oe.CurrentNode;
+					}
+				}
+			}
+
 			Array.Copy (nodes, index + 1, nodes, index, count - index);
 			count--;
 			if (nodes.Length > OrigSize && nodes.Length > (count * 2))
 				Shrink ();
 
-                        TreeView tree_view = null;
-			if (owner != null)
-				tree_view = owner.TreeView;
-			if (tree_view != null && removed == tree_view.top_node) {
-				OpenTreeNodeEnumerator oe = new OpenTreeNodeEnumerator (removed);
-				if (oe.MoveNext () && oe.MoveNext ())
-					tree_view.top_node = oe.CurrentNode;
-				else
-					tree_view.top_node = null;
+			if (tree_view != null && new_selected != null) {
+				tree_view.SelectedNode = new_selected;
 			}
 
 			TreeNode parent = removed.parent;
 			removed.parent = null;
 
-			if (tree_view != null)
+			if (update && tree_view != null && visible) {
+				tree_view.RecalculateVisibleOrder (prev);
+				tree_view.UpdateScrollBars ();
 				tree_view.UpdateBelow (parent);
+			}
+		}
+
+		private TreeNode GetPrevNode (TreeNode node)
+		{
+			OpenTreeNodeEnumerator one = new OpenTreeNodeEnumerator (node);
+
+			if (one.MovePrevious () && one.MovePrevious ())
+				return one.CurrentNode;
+			return null;
+		}
+
+		private void SetupNode (TreeNode node)
+		{
+			// Remove it from any old parents
+			node.Remove ();
+
+			node.parent = owner;
+
+			TreeView tree_view = null;
+			if (owner != null)
+				tree_view = owner.TreeView;
+
+			if (tree_view != null) {
+				TreeNode prev = GetPrevNode (node);
+
+				if (tree_view.top_node == null)
+					tree_view.top_node = node;
+
+				if (node.IsVisible)
+					tree_view.RecalculateVisibleOrder (prev);
+				tree_view.UpdateScrollBars ();
+			}
+
+			if (owner != null && tree_view != null && (owner.IsExpanded || owner.IsRoot)) {
+				// tree_view.UpdateBelow (owner);
+				tree_view.UpdateNode (owner);
+				tree_view.UpdateNode (node);
+			} else if (owner != null && tree_view != null) {
+				tree_view.UpdateBelow (owner);
+			}
 		}
 
 		int IList.Add (object node)
@@ -292,7 +360,6 @@ namespace System.Windows.Forms {
 			count++;
 			nodes [pos] = node;
 
-			node.parent = owner;
 			return count;
 		}
 
@@ -304,6 +371,10 @@ namespace System.Windows.Forms {
 			for (int i = 0; i < count; i++) {
 				nodes [i].Nodes.Sort ();
 			}
+
+			// No null checks since sort can only be called from the treeviews root node collection
+			owner.TreeView.RecalculateVisibleOrder (owner);
+			owner.TreeView.UpdateScrollBars ();
 		}
 
 		private void Grow ()
@@ -315,7 +386,7 @@ namespace System.Windows.Forms {
 
 		private void Shrink ()
 		{
-			int len = (count > OrigSize ? count : OrigSize);
+			int len = (count + 1 > OrigSize ? count + 1 : OrigSize);
 			TreeNode [] nn = new TreeNode [len];
 			Array.Copy (nodes, nn, count);
 			nodes = nn;
