@@ -45,7 +45,8 @@ namespace System.Windows.Forms {
 		internal bool nodes_added;
 		private TreeNodeCollection nodes;
 
-		private TreeNode selected_node = null;
+		private TreeViewAction selection_action = TreeViewAction.Unknown;
+		internal TreeNode selected_node = null;
 		private TreeNode focused_node = null;
 		private bool select_mmove = false;
 
@@ -57,26 +58,25 @@ namespace System.Windows.Forms {
 		private bool hot_tracking;
 		private int indent = 19;
 
-		private TextBox edit_text_box;
-		private TreeNode edit_node;
+		private NodeLabelEditEventArgs edit_args;
+		private LabelEditTextBox edit_text_box;
+		internal TreeNode edit_node;
 		
 		private bool checkboxes;
 		private bool label_edit;
-		private bool scrollable;
+		private bool scrollable = true;
 		private bool show_lines = true;
 		private bool show_root_lines = true;
 		private bool show_plus_minus = true;
 		private bool hide_selection = true;
 
-		private bool add_hscroll;
-		private bool add_vscroll;
-		private int max_node_width;
+		private int max_visible_order = -1;
 		private VScrollBar vbar;
-		private int skipped_nodes;
 		private HScrollBar hbar;
-		private int hbar_offset;
-		private int used_height;
-		private bool update_node_bounds;
+		private bool vbar_bounds_set;
+		private bool hbar_bounds_set;
+		internal int skipped_nodes;
+		internal int hbar_offset;
 		
 		private int update_stack;
 		private bool update_needed;
@@ -94,7 +94,6 @@ namespace System.Windows.Forms {
 
 		private Pen dash;
 		private StringFormat string_format;
-		private int open_node_count = -1;
 
 		private int drag_begin_x = 0;
 		private int drag_begin_y = 0;
@@ -105,6 +104,7 @@ namespace System.Windows.Forms {
 		#region Public Constructors	
 		public TreeView ()
 		{
+			border_style = BorderStyle.Fixed3D;
 			base.background_color = ThemeEngine.Current.ColorWindow;
 			base.foreground_color = ThemeEngine.Current.ColorWindowText;
 
@@ -118,8 +118,8 @@ namespace System.Windows.Forms {
 			MouseMove += new MouseEventHandler(MouseMoveHandler);
 			SizeChanged += new EventHandler (SizeChangedHandler);
 			FontChanged += new EventHandler (FontChangedHandler);
-			LostFocus += new EventHandler (FocusChangedHandler);
-			GotFocus += new EventHandler (FocusChangedHandler);
+			LostFocus += new EventHandler (LostFocusHandler);
+			GotFocus += new EventHandler (GotFocusHandler);
 			MouseWheel += new MouseEventHandler(MouseWheelHandler);
 
 			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
@@ -127,6 +127,19 @@ namespace System.Windows.Forms {
 			string_format = new StringFormat ();
 			string_format.LineAlignment = StringAlignment.Center;
 			string_format.Alignment = StringAlignment.Center;
+
+			vbar = new ImplicitVScrollBar ();
+			hbar = new ImplicitHScrollBar ();
+
+			vbar.Visible = false;
+			hbar.Visible = false;
+			vbar.ValueChanged += new EventHandler (VScrollBarValueChanged);
+			hbar.ValueChanged += new EventHandler (HScrollBarValueChanged);
+
+			SuspendLayout ();
+			Controls.AddImplicit (vbar);
+			Controls.AddImplicit (hbar);
+			ResumeLayout ();
 		}
 
 		#endregion	// Public Constructors
@@ -166,7 +179,7 @@ namespace System.Windows.Forms {
 				if (!checkboxes)
 					root_node.CollapseAllUncheck ();
 
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -181,7 +194,7 @@ namespace System.Windows.Forms {
 				if (value == full_row_select)
 					return;
 				full_row_select = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 		[DefaultValue(true)]
@@ -191,7 +204,7 @@ namespace System.Windows.Forms {
 				if (hide_selection == value)
 					return;
 				hide_selection = value;
-				this.Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -215,7 +228,7 @@ namespace System.Windows.Forms {
 				if (image_index == value)
 					return;
 				image_index = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -224,7 +237,7 @@ namespace System.Windows.Forms {
 			get { return image_list; }
 			set {
 				image_list = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -243,7 +256,7 @@ namespace System.Windows.Forms {
 						"'Indent' must be greater than or equal to 0.");
 				}
 				indent = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -258,7 +271,7 @@ namespace System.Windows.Forms {
 				if (value == item_height)
 					return;
 				item_height = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -288,6 +301,7 @@ namespace System.Windows.Forms {
 				if (scrollable == value)
 					return;
 				scrollable = value;
+				UpdateScrollBars ();
 			}
 		}
 
@@ -302,6 +316,7 @@ namespace System.Windows.Forms {
 					throw new ArgumentException ("'" + value + "' is not a valid value for 'value'. " +
 						"'value' must be greater than or equal to 0.");
 				}
+				UpdateNode (SelectedNode);
 			}
 		}
 
@@ -310,10 +325,12 @@ namespace System.Windows.Forms {
 		public TreeNode SelectedNode {
 			get { return selected_node; }
 			set {
-				if (selected_node == value)
+				if (selected_node == value) {
+					selection_action = TreeViewAction.Unknown;
 					return;
+				}
 
-				TreeViewCancelEventArgs e = new TreeViewCancelEventArgs (value, false, TreeViewAction.Unknown);
+				TreeViewCancelEventArgs e = new TreeViewCancelEventArgs (value, false, selection_action);
 				OnBeforeSelect (e);
 
 				if (e.Cancel)
@@ -349,6 +366,7 @@ namespace System.Windows.Forms {
 					selected_node.EnsureVisible ();
 
 				OnAfterSelect (new TreeViewEventArgs (value, TreeViewAction.Unknown));
+				selection_action = TreeViewAction.Unknown;
 			}
 		}
 
@@ -368,7 +386,7 @@ namespace System.Windows.Forms {
 				if (show_lines == value)
 					return;
 				show_lines = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -379,7 +397,7 @@ namespace System.Windows.Forms {
 				if (show_plus_minus == value)
 					return;
 				show_plus_minus = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -390,7 +408,7 @@ namespace System.Windows.Forms {
 				if (show_root_lines == value)
 					return;
 				show_root_lines = value;
-				Refresh ();
+				Invalidate ();
 			}
 		}
 
@@ -403,7 +421,7 @@ namespace System.Windows.Forms {
 				if (sorted) {
 					Nodes.Sort ();
 					top_node = null;
-					Refresh ();
+					Invalidate ();
 				}
 			}
 		}
@@ -426,7 +444,7 @@ namespace System.Windows.Forms {
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public int VisibleCount {
 			get {
-				return ClientRectangle.Height / ItemHeight;
+				return ViewportRectangle.Height / ItemHeight;
 			}
 		}
 
@@ -447,13 +465,9 @@ namespace System.Windows.Forms {
 		#endregion	// Protected Instance Properties
 
 		#region Public Instance Methods
-		public void BeginUpdate () {
-			update_stack++;
-		}
-
-		public void CollapseAll ()
+		public void BeginUpdate ()
 		{
-			root_node.CollapseAll ();
+			update_stack++;
 		}
 
 		public void EndUpdate ()
@@ -463,14 +477,29 @@ namespace System.Windows.Forms {
 			} else {
 				update_stack = 0;
 				if (update_needed) {
+					RecalculateVisibleOrder (root_node);
+					UpdateScrollBars ();
+					if (SelectedNode != null)
+						SelectedNode.EnsureVisible ();
 					Invalidate (ViewportRectangle);
 					update_needed = false;
 				}
 			}
 		}
 
-		public void ExpandAll () {
+		public void ExpandAll ()
+		{
+			BeginUpdate ();
 			root_node.ExpandAll ();
+			EndUpdate ();
+		}
+
+		
+		public void CollapseAll ()
+		{
+			BeginUpdate ();
+			root_node.CollapseAll ();
+			EndUpdate ();
 		}
 
 		public TreeNode GetNodeAt (Point pt) {
@@ -507,6 +536,8 @@ namespace System.Windows.Forms {
 		#region Protected Instance Methods
 		protected override void CreateHandle () {
 			base.CreateHandle ();
+
+			UpdateScrollBars ();
 		}
 
 		protected override void Dispose (bool disposing) {
@@ -520,8 +551,8 @@ namespace System.Windows.Forms {
 			return node.prop_bag;
 		}
 
-		protected override bool IsInputKey (Keys key_data) {
-
+		protected override bool IsInputKey (Keys key_data)
+		{
 			if ((key_data & Keys.Alt) == 0) {
 				switch (key_data & Keys.KeyCode) {
 				case Keys.Enter:
@@ -558,9 +589,11 @@ namespace System.Windows.Forms {
 					if (selected_node.IsExpanded)
 						selected_node.Collapse ();
 					else {
-						ne = new OpenTreeNodeEnumerator (selected_node);
-						if (ne.MovePrevious () && ne.MovePrevious ())
-							SelectedNode = ne.CurrentNode;
+						TreeNode parent = selected_node.Parent;
+						if (parent != null) {
+							selection_action = TreeViewAction.ByKeyboard;
+							SelectedNode = parent;
+						}
 					}
 				}
 				break;
@@ -569,31 +602,37 @@ namespace System.Windows.Forms {
 					if (!selected_node.IsExpanded)
 						selected_node.Expand ();
 					else {
-						ne = new OpenTreeNodeEnumerator (selected_node);
-						if (ne.MoveNext () && ne.MoveNext ())
-							SelectedNode = ne.CurrentNode;
+						TreeNode child = selected_node.FirstNode;
+						if (child != null)
+							SelectedNode = child;
 					}
 				}
 				break;
 			case Keys.Up:
 				if (selected_node != null) {
 					ne = new OpenTreeNodeEnumerator (selected_node);
-					if (ne.MovePrevious () && ne.MovePrevious ())
+					if (ne.MovePrevious () && ne.MovePrevious ()) {
+						selection_action = TreeViewAction.ByKeyboard;
 						SelectedNode = ne.CurrentNode;
+					}
 				}
 				break;
 			case Keys.Down:
 				if (selected_node != null) {
 					ne = new OpenTreeNodeEnumerator (selected_node);
-					if (ne.MoveNext () && ne.MoveNext ())
+					if (ne.MoveNext () && ne.MoveNext ()) {
+						selection_action = TreeViewAction.ByKeyboard;
 						SelectedNode = ne.CurrentNode;
+					}
 				}
 				break;
 			case Keys.Home:
 				if (root_node.Nodes.Count > 0) {
 					ne = new OpenTreeNodeEnumerator (root_node.Nodes [0]);
-					if (ne.MoveNext ())
+					if (ne.MoveNext ()) {
+						selection_action = TreeViewAction.ByKeyboard;
 						SelectedNode = ne.CurrentNode;
+					}
 				}
 				break;
 			case Keys.End:
@@ -601,25 +640,28 @@ namespace System.Windows.Forms {
 					ne = new OpenTreeNodeEnumerator (root_node.Nodes [0]);
 					while (ne.MoveNext ())
 					{ }
+					selection_action = TreeViewAction.ByKeyboard;
 					SelectedNode = ne.CurrentNode;
 				}
 				break;
 			case Keys.PageDown:
 				if (selected_node != null) {
 					ne = new OpenTreeNodeEnumerator (selected_node);
-					int move = ViewportRectangle.Height / ItemHeight;
+					int move = VisibleCount;
 					for (int i = 0; i < move && ne.MoveNext (); i++) {
 						
 					}
+					selection_action = TreeViewAction.ByKeyboard;
 					SelectedNode = ne.CurrentNode;
 				}
 				break;
 			case Keys.PageUp:
 				if (selected_node != null) {
 					ne = new OpenTreeNodeEnumerator (selected_node);
-					int move = ViewportRectangle.Height / ItemHeight;
+					int move = VisibleCount;
 					for (int i = 0; i < move && ne.MovePrevious (); i++)
 					{ }
+					selection_action = TreeViewAction.ByKeyboard;
 					SelectedNode = ne.CurrentNode;
 				}
 				break;
@@ -711,19 +753,6 @@ namespace System.Windows.Forms {
 
 		protected override void OnHandleCreated (EventArgs e) {
 			base.OnHandleCreated (e);
-
-			vbar = new ImplicitVScrollBar ();
-			hbar = new ImplicitHScrollBar ();
-
-			vbar.Visible = false;
-			hbar.Visible = false;
-			vbar.ValueChanged += new EventHandler (VScrollBarValueChanged);
-			hbar.ValueChanged += new EventHandler (HScrollBarValueChanged);
-
-			SuspendLayout ();
-			Controls.AddImplicit (vbar);
-			Controls.AddImplicit (hbar);
-			ResumeLayout ();
 		}
 
 		protected override void OnHandleDestroyed (EventArgs e) {
@@ -732,14 +761,7 @@ namespace System.Windows.Forms {
 
 		protected override void WndProc(ref Message m) {
 			switch ((Msg) m.Msg) {
-			case Msg.WM_PAINT: {				
-				PaintEventArgs	paint_event;
 
-				paint_event = XplatUI.PaintEventStart (Handle, true);
-				DoPaint (paint_event);
-				XplatUI.PaintEventEnd (Handle, true);
-				return;
-			}
 			case Msg.WM_LBUTTONDBLCLK:
 				int val = m.LParam.ToInt32();
 				DoubleClickHandler (null, new
@@ -754,14 +776,6 @@ namespace System.Windows.Forms {
 		#endregion	// Protected Instance Methods
 
 		#region	Internal & Private Methods and Properties
-		internal string LabelEditText {
-			get {
-				if (edit_text_box == null)
-					return String.Empty;
-				return edit_text_box.Text;
-			}
-		}
-
 		internal IntPtr CreateNodeHandle ()
 		{
 			return (IntPtr) handle_count++;
@@ -786,7 +800,6 @@ namespace System.Windows.Forms {
 			return null;
 		}
 
-		// TODO: we shouldn't have to compute this on the fly
 	        internal Rectangle ViewportRectangle {
 			get {
 				Rectangle res = ClientRectangle;
@@ -860,11 +873,39 @@ namespace System.Windows.Forms {
 			return (x > l && x < l + 10);
 		}
 
+		internal void RecalculateVisibleOrder (TreeNode start)
+		{
+			if (update_stack > 0)
+				return;
+
+			int order;
+			if (start == null) {
+				start = root_node;
+				order = 0;
+			} else
+				order = start.visible_order;
+
+			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (start);
+			while (walk.MoveNext ()) {
+				walk.CurrentNode.visible_order = order;
+				order++;
+			}
+
+			max_visible_order = order;
+		}
+
 		internal void SetTop (TreeNode node)
 		{
-			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (root_node);
-			int offset = 0;
+			if (!vbar.is_visible)
+				return;
 
+			TreeNode first = root_node.FirstNode;
+
+			if (first == null)
+				return;  // I don't think its possible for this to happen
+
+			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (first);
+			int offset = -1;
 			while (walk.CurrentNode != node && walk.MoveNext ())
 				offset++;
 
@@ -873,6 +914,9 @@ namespace System.Windows.Forms {
 
 		internal void SetBottom (TreeNode node)
 		{
+			if (!vbar.is_visible)
+				return;
+
 			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (node);
 
 			int bottom = ViewportRectangle.Bottom;
@@ -882,7 +926,17 @@ namespace System.Windows.Forms {
 					break;
 				offset++;
 			}
-			vbar.Value += offset;
+
+			int nv = vbar.Value + offset;
+			if (vbar.Value + offset < vbar.Maximum) {
+				vbar.Value = nv;
+			} else {
+#if DEBUG
+				Console.Error.WriteLine ("setting bottom to value greater then maximum ({0}, {1})",
+						nv, vbar.Maximum);
+#endif
+			}
+				
 		}
 
 		internal void UpdateBelow (TreeNode node)
@@ -893,25 +947,29 @@ namespace System.Windows.Forms {
 			}
 
 			if (node == root_node) {
-				Refresh ();
+				Invalidate (ViewportRectangle);
 				return;
 			}
 				
 			// We need to update the current node so the plus/minus block gets update too
-			Rectangle invalid = new Rectangle (0, node.Bounds.Top - 1,
-					Width, Height - node.Bounds.Top + 1);
+			int top = Math.Max (node.Bounds.Top - 1, 0);
+			Rectangle invalid = new Rectangle (0, top,
+					Width, Height - top);
 			Invalidate (invalid);
 		}
 
 		internal void UpdateNode (TreeNode node)
 		{
+			if (node == null)
+				return;
+
 			if (update_stack > 0) {
 				update_needed = true;
 				return;
 			}
 
 			if (node == root_node) {
-				Refresh ();
+				Invalidate ();
 				return;
 			}
 
@@ -939,29 +997,13 @@ namespace System.Windows.Forms {
 			Invalidate (new Rectangle (l, node.Bounds.Top, 8, node.Bounds.Height));
 		}
 
-		private void DoPaint (PaintEventArgs pe)
+		internal override void OnPaintInternal (PaintEventArgs pe)
 		{
-			if (Width <= 0 || Height <=  0 || Visible == false)
-				return;
-
 			Draw (pe.ClipRectangle, pe.Graphics);
 		}
 
 		private void Draw (Rectangle clip, Graphics dc)
 		{
-			if (top_node == null && Nodes.Count > 0)
-				top_node = nodes [0];
-
-			if (selected_node == null && Nodes.Count > 0)
-				SelectedNode = nodes [0];
-
-			// Decide if we need a scrollbar
-			int old_open_node_count = open_node_count;
-
-			//Rectangle fill = ClientRectangle;
-			add_vscroll = false;
-			add_hscroll = false;
-			
 			dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor), clip);
 
 			Color dash_color = ControlPaint.Dark (BackColor);
@@ -970,45 +1012,31 @@ namespace System.Windows.Forms {
 			dash = new Pen (dash_color, 1);
 			dash.DashStyle = DashStyle.Dot;
 
-			int depth = 0;
-			int item_height = ItemHeight;
-			int height = ClientRectangle.Height;
+			Rectangle viewport = ViewportRectangle;
+			Rectangle original_clip = clip;
+			if (clip.Bottom > viewport.Bottom)
+				clip.Height = viewport.Bottom - clip.Top;
 
-			open_node_count = 0;
-			used_height = 0;
-			foreach (TreeNode node in nodes) {
-				DrawNode (node, dc, clip, ref depth, item_height, height);
-				depth = 0;
+			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (TopNode);
+			while (walk.MoveNext ()) {
+				TreeNode current = walk.CurrentNode;
+
+				// Haven't gotten to visible nodes yet
+				if (current.GetY () + ItemHeight < clip.Top)
+					continue;
+
+				// Past the visible nodes
+				if (current.GetY () > clip.Bottom)
+					break;
+
+				DrawNode (current, dc, clip);
 			}
 
-			add_vscroll = (open_node_count * ItemHeight) > ClientRectangle.Height;
-
-			if (max_node_width > ClientRectangle.Width)
-				add_hscroll = true;
-
-			if (add_vscroll)
-				add_hscroll = max_node_width > ClientRectangle.Width - ThemeEngine.Current.VScrollBarDefaultSize.Width;
-			if (add_hscroll)
-				add_vscroll = (open_node_count * ItemHeight) > ClientRectangle.Height - ThemeEngine.Current.HScrollBarDefaultSize.Height;
-			
-			if (add_hscroll) {
-				AddHorizontalScrollBar ();
-			} else if (hbar != null) {
-				hbar_offset = 0;
-				hbar.Visible = false;
-			}
-
-			if (add_vscroll) {
-				AddVerticalScrollBar (open_node_count, old_open_node_count != open_node_count);
-			} else if (vbar != null) {
-				vbar.Visible = false;
-				skipped_nodes = 0;
-			}
-
-			if (add_hscroll && add_vscroll) {
+			if (hbar.Visible && vbar.Visible) {
 				Rectangle corner = new Rectangle (hbar.Right, vbar.Bottom, vbar.Width, hbar.Height);
-				if (clip.IntersectsWith (corner))
-					dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorControl), corner);
+				if (original_clip.IntersectsWith (corner))
+					dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorControl),
+							corner);
 			}
 		}
 
@@ -1040,33 +1068,63 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		private void DrawNodeLines (TreeNode node, Graphics dc, bool visible, Pen dash, int x, int y,
-				int middle, int item_height, int node_count)
+		private void DrawNodeLines (TreeNode node, Graphics dc, Rectangle clip, Pen dash, int x, int y,	int middle)
 		{
-			int ladjust = 9; // left adjust
-			int radjust = 0; // right adjust
+			int ladjust = 9;
+			int radjust = 0;
 
-			if (node_count > 0 && show_plus_minus)
+			if (node.nodes.Count > 0 && show_plus_minus)
 				ladjust = 13;
 			if (checkboxes)
 				radjust = 3;
 
 			dc.DrawLine (dash, x - indent + ladjust, middle, x + radjust, middle);
 
-			//if (!visible)
-			//	return;
+			if (node.PrevNode != null || node.Parent != null) {
+				ladjust = 9;
+				dc.DrawLine (dash, x - indent + ladjust, node.Bounds.Top,
+						x - indent + ladjust, middle - (show_plus_minus && node.Nodes.Count > 0 ? 4 : 0));
+			}
 
-			int ly = 0;
-			if (node.PrevNode != null) {
-				int prevadjust = (node.Nodes.Count > 0 && show_plus_minus ? (node.PrevNode.Nodes.Count == 0 ? 0 : 4) :
-						(node.PrevNode.Nodes.Count == 0 ? 0 : 4));
-				int myadjust = (node.Nodes.Count > 0 && show_plus_minus ? 4 : 0);
-				ly = node.PrevNode.Bounds.Bottom - (item_height / 2) + prevadjust;
-				dc.DrawLine (dash, x - indent + 9, middle - myadjust, x - indent + 9, ly);
-			} else if (node.Parent != null) {
-				int myadjust = (node.Nodes.Count > 0 && show_plus_minus ? 4 : 0);
-				ly = node.Parent.Bounds.Bottom + 1;
-				dc.DrawLine (dash, x - indent + 9, middle - myadjust, x - indent + 9, ly);
+			if (node.NextNode != null) {
+				ladjust = 9;
+				dc.DrawLine (dash, x - indent + ladjust, middle + (show_plus_minus && node.Nodes.Count > 0 ? 4 : 0),
+						x - indent + ladjust, node.Bounds.Bottom);
+				
+			}
+
+			ladjust = 0;
+			if (show_plus_minus)
+				ladjust = 9;
+			TreeNode parent = node.Parent;
+			while (parent != null) {
+				if (parent.NextNode != null) {
+					int px = parent.GetLinesX () - indent + ladjust;
+					dc.DrawLine (dash, px, node.Bounds.Top, px, node.Bounds.Bottom);
+				}
+				parent = parent.Parent;
+			}
+		}
+
+		private void DrawLinesToNext (TreeNode node, Graphics dc, Rectangle clip, Pen dash, int x, int y)
+		{
+			int middle = y + (ItemHeight / 2);
+
+			if (node.NextNode != null) {
+				int top = (node.Nodes.Count > 0 && show_plus_minus ? middle + 4 : middle);
+				int ncap = (node.NextNode.Nodes.Count > 0 && show_plus_minus ? 4 : 8);
+				int bottom = Math.Min (node.NextNode.GetY () + ncap, clip.Bottom);
+
+				dc.DrawLine (dash, x - indent + 9, top, x - indent + 9, bottom);
+			}
+
+			if (node.IsExpanded && node.Nodes.Count > 0) {
+				int top = node.Bounds.Bottom;
+				int ncap = (node.Nodes [0].Nodes.Count > 0 && show_plus_minus ? 4 : 8);
+				int bottom = Math.Min (node.Nodes [0].GetY () + ncap, clip.Bottom);
+				int nx = node.Nodes [0].GetLinesX ();
+
+				dc.DrawLine (dash, nx - indent + 9, top, nx - indent + 9, bottom);
 			}
 		}
 
@@ -1077,25 +1135,41 @@ namespace System.Windows.Forms {
 			if (!RectsIntersect (clip, x, y + 2, ImageList.ImageSize.Width, ImageList.ImageSize.Height))
 				return;
 
-			if (node.ImageIndex > -1 && ImageList != null && node.ImageIndex < ImageList.Images.Count) {
+			if (ImageList == null)
+				return;
+
+			int use_index = -1;
+			if (node.ImageIndex > -1 && node.ImageIndex < ImageList.Images.Count) {
+				use_index = node.ImageIndex;
+			} else if (ImageIndex > -1 && ImageIndex < ImageList.Images.Count) {
+				use_index = ImageIndex;
+			}
+
+			if (use_index == -1 && ImageList.Images.Count > 0) {
+				use_index = 0;
+			}
+
+			if (use_index != -1) {
 				ImageList.Draw (dc, x, y + 2, ImageList.ImageSize.Width, 
-						ImageList.ImageSize.Height, node.ImageIndex);
-			} else if (ImageIndex > -1 && ImageList != null && ImageIndex < ImageList.Images.Count) {
-				ImageList.Draw (dc, x, y + 2, ImageList.ImageSize.Width, 
-						ImageList.ImageSize.Height, ImageIndex);
+						ImageList.ImageSize.Height, use_index);
 			}
 		}
 
-		private void DrawEditNode (TreeNode node)
+		private void LabelEditFinished (object sender, EventArgs e)
 		{
-			SuspendLayout ();
+			EndEdit (edit_node);
+		}
+
+		internal void BeginEdit (TreeNode node)
+		{
+			if (edit_node != null)
+				EndEdit (edit_node);
 
 			if (edit_text_box == null) {
-				edit_text_box = new FixedSizeTextBox ();
+				edit_text_box = new LabelEditTextBox ();
 				edit_text_box.BorderStyle = BorderStyle.FixedSingle;
-				edit_text_box.KeyUp += new KeyEventHandler (EditTextBoxKeyDown);
-				edit_text_box.Leave += new EventHandler (EditTextBoxLeave);
-				Controls.AddImplicit (edit_text_box);
+				edit_text_box.EditingFinished += new EventHandler (LabelEditFinished);
+				Controls.Add (edit_text_box);
 			}
 
 			edit_text_box.Bounds = node.Bounds;
@@ -1106,40 +1180,46 @@ namespace System.Windows.Forms {
 			edit_text_box.Focus ();
 			edit_text_box.SelectAll ();
 
-			ResumeLayout ();
+			edit_args = new NodeLabelEditEventArgs (edit_node);
+			OnBeforeLabelEdit (edit_args);
+
+			if (edit_args.CancelEdit)
+				EndEdit (node);
+			
+			edit_node = node;
 		}
 
-		private void EditTextBoxKeyDown (object sender, KeyEventArgs e)
+		internal void EndEdit (TreeNode node)
 		{
-			if (e.KeyCode == Keys.Return)
-				EndEdit ();
+			if (edit_text_box != null && edit_text_box.Visible) {
+				edit_text_box.Visible = false;
+			}
+
+			if (edit_node != null && edit_node == node) {
+				OnAfterLabelEdit (edit_args);
+
+				if (!edit_args.CancelEdit) {
+					if (edit_args.Label != null)
+						edit_node.Text = edit_args.Label;
+					else
+						edit_node.Text = edit_text_box.Text;
+				}
+
+			}
+
+			
+			edit_node = null;
+			UpdateNode (node);
 		}
 
-		private void EditTextBoxLeave (object sender, EventArgs e)
-		{
-			EndEdit ();
-		}
-
-		private void EndEdit ()
-		{
-			edit_text_box.Visible = false;
-			edit_node.EndEdit (false);
-			UpdateNode(edit_node);
-		}
-
-		private void UpdateNodeBounds (TreeNode node, int x, int y, int item_height, Graphics dc)
+		internal int GetNodeWidth (TreeNode node)
 		{
 			Font font = node.NodeFont;
 			if (node.NodeFont == null)
 				font = Font;
-
-			if (node.NeedsWidth || update_node_bounds)
-				node.SetWidth ((int) dc.MeasureString (node.Text, font, 0,
-							       string_format).Width + 3);
-			node.SetHeight (item_height);
-			node.SetPosition (x, y);
+			return (int) DeviceContext.MeasureString (node.Text, font, 0, string_format).Width + 3;
 		}
-		
+
 		private void DrawSelectionAndFocus(TreeNode node, Graphics dc, Rectangle r)
 		{
 			if (Focused && focused_node == node) {
@@ -1167,146 +1247,144 @@ namespace System.Windows.Forms {
 					node.Bounds, string_format);
 		}
 
-		private void DrawNode (TreeNode node, Graphics dc, Rectangle clip, ref int depth, int item_height, int max_height)
+		private void DrawNode (TreeNode node, Graphics dc, Rectangle clip)
 		{
-			open_node_count++;
-			int x = (!show_root_lines && node.Parent != null ? depth  - 1 : depth) * indent - hbar_offset;
-			int y = item_height * (open_node_count - skipped_nodes - 1);
-			bool visible = (y >= 0 && y < max_height);
-			int _n_count = node.nodes.Count;
-			int middle = y + (item_height / 2);
+			int child_count = node.nodes.Count;
+			int y = node.GetY ();
+			int middle = y + (ItemHeight / 2);
 
-			// The thing is totally out of the clipping rectangle
-			if (clip.Top > y + ItemHeight || clip.Bottom < y)
-				visible = false;
-
-			if (visible && full_row_select) {
-				Rectangle r = new Rectangle(1, y+2, ViewportRectangle.Width-2, item_height);
-				DrawSelectionAndFocus(node, dc, r);
+			if (full_row_select) {
+				Rectangle r = new Rectangle (1, y + 2, ViewportRectangle.Width - 2, ItemHeight);
+				DrawSelectionAndFocus (node, dc, r);
 			}
 
-			if (show_root_lines || node.Parent != null) {
-				x += 5;
-				if (_n_count > 0) {
-					if (show_plus_minus && visible) {
-						DrawNodePlusMinus (node, dc, x, middle);
-					}
-				}
-				x += indent - 5; 
-			}
+			if ((show_root_lines || node.Parent != null) && show_plus_minus && child_count > 0)
+				DrawNodePlusMinus (node, dc, node.GetLinesX () - Indent + 5, middle);
 
-			int ox = x;
-
-			if (checkboxes) {
-				if (visible)
-					DrawNodeCheckBox (node, dc, ox, middle);
-				ox += 19;
-			}
+			if (checkboxes)
+				DrawNodeCheckBox (node, dc, node.GetX () - 19, middle);
 
 			if (show_lines)
-				DrawNodeLines (node, dc, visible, dash, x, y, middle, item_height, _n_count);
+				DrawNodeLines (node, dc, clip, dash, node.GetLinesX (), y, middle);
 
-			if (ImageList != null) {
-                                if (visible)
-                                        DrawNodeImage (node, dc, clip, ox, y);
-				// MS leaves the space for the image if the ImageList is
-				// non null regardless of whether or not an image is drawn
-				ox += ImageList.ImageSize.Width + 3; // leave a little space so the text isn't against the image
-			}
-
-			UpdateNodeBounds (node, ox, y, item_height, dc);
-
-			bool bounds_in_clip = clip.IntersectsWith (node.Bounds) || full_row_select;
-			if (visible &&	bounds_in_clip) {
-				if (node.IsEditing)
-					DrawEditNode (node);
-				else
-					DrawStaticNode (node, dc);
-			}
-			
-			if (node.Bounds.Right > max_node_width) {
-				max_node_width = node.Bounds.Right;
-				if (max_node_width > ClientRectangle.Width && !add_hscroll) {
-					max_height -= ItemHeight;
-					add_hscroll = true;
-				}
-			}
-
-			if (node.Bounds.Bottom > used_height)
-				used_height = node.Bounds.Bottom;
-
-			depth++;
-			if (node.IsExpanded) {
-				for (int i = 0; i < _n_count; i++) {
-					int tdepth = depth;
-					DrawNode (node.nodes [i], dc, clip, ref tdepth, item_height, max_height);
-				}
-			}
+			if (ImageList != null)
+                                DrawNodeImage (node, dc, clip, node.GetImageX (), y);
 
+			if (!node.IsEditing)
+				DrawStaticNode (node, dc);
 		}
 
-		private void AddVerticalScrollBar (int total_nodes, bool count_changed)
+		internal void UpdateScrollBars ()
 		{
-			vbar.Bounds = new Rectangle (ClientRectangle.Width - vbar.Width,
-				0, vbar.Width, (add_hscroll ? Height - ThemeEngine.Current.HScrollBarDefaultSize.Height : Height));
+			if (update_stack > 0)
+				return;
 
-			if (count_changed) {
-				vbar.Maximum = total_nodes;
-				int height = ClientRectangle.Height;
-				vbar.LargeChange = height / ItemHeight;
+			bool vert = false;
+			bool horz = false;
+			int height = -1;
+			int width = -1;
+
+			if (scrollable) {
+				OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (root_node);
+				
+				while (walk.MoveNext ()) {
+					int r = walk.CurrentNode.Bounds.Right;
+					int b = walk.CurrentNode.Bounds.Bottom;
+
+					if (r > width)
+						width = r;
+					if (b > height)
+						height = b;
+				}
+
+				// Remove scroll adjustments
+				if (nodes.Count > 0)
+					height -= nodes [0].Bounds.Top;
+				width += hbar_offset;
+
+				if (height > ClientRectangle.Height) {
+					vert = true;
+
+					if (width > ClientRectangle.Width - SystemInformation.VerticalScrollBarWidth)
+						horz = true;
+				} else if (width > ClientRectangle.Width) {
+					horz = true;
+				}
+
+				if (!vert && horz && height > ClientRectangle.Height - SystemInformation.HorizontalScrollBarHeight)
+					vert = true;
 			}
 
-			vbar.Visible = true;
-		}
+			if (vert) {
+				vbar.SetValues (Math.Max (0, max_visible_order - 2), VisibleCount);
+				/*
+				vbar.Maximum = max_visible_order;
+				vbar.LargeChange = ClientRectangle.Height / ItemHeight;
+				*/
 
-		private void AddHorizontalScrollBar ()
-		{
-			hbar.Bounds = new Rectangle (ClientRectangle.Left,
-					ClientRectangle.Bottom - hbar.Height,
-					(add_vscroll ? Width - ThemeEngine.Current.VScrollBarDefaultSize.Width : Width), hbar.Height);
+				if (!vbar_bounds_set) {
+					vbar.Bounds = new Rectangle (ClientRectangle.Width - vbar.Width, 0, vbar.Width,
+							ClientRectangle.Height -
+							(horz ? SystemInformation.VerticalScrollBarWidth : 0));
+					vbar_bounds_set = true;
 
-			int width = ClientRectangle.Width;
-			if (vbar.Visible)
-				width -= vbar.Width;
+					// We need to recalc the hbar if the vbar is now visible
+					hbar_bounds_set = false;
+				}
 
-			hbar.SmallChange = 15;
-			hbar.LargeChange = 50;
-			int num_pixels = max_node_width - width;
-			hbar.Maximum = num_pixels + 50;
-			hbar.Visible = true;
+				vbar.Visible = true;
+			} else {
+				skipped_nodes = 0;
+				top_node = root_node.FirstNode;
+				RecalculateVisibleOrder (root_node);
+				vbar.Visible = false;
+				vbar_bounds_set = false;
+			}
+
+			if (horz) {
+				hbar.SetValues (width + 1, ClientRectangle.Width);
+				/*
+				hbar.LargeChange = ClientRectangle.Width;
+				hbar.Maximum = width + 1;
+				*/
+
+				if (!hbar_bounds_set) {
+					hbar.Bounds = new Rectangle (0, ClientRectangle.Height - hbar.Height,
+							ClientRectangle.Width - (vert ? SystemInformation.VerticalScrollBarWidth : 0),
+							hbar.Height);
+					hbar_bounds_set = true;
+				}
+				hbar.Visible = true;
+			} else {
+				hbar_offset = 0;
+				hbar.Visible = false;
+				hbar_bounds_set = false;
+			}
 		}
 
 		private void SizeChangedHandler (object sender, EventArgs e)
 		{
-			SuspendLayout ();
-
-			if (max_node_width > ClientRectangle.Width && ClientRectangle.Width >= 0) {
-				add_hscroll = true;
-				AddHorizontalScrollBar ();
-				return;
+			if (IsHandleCreated) {
+				if (max_visible_order == -1)
+					RecalculateVisibleOrder (root_node);
+				UpdateScrollBars ();
 			}
 
-			if (used_height > ClientRectangle.Height && ClientRectangle.Height >= 0) {
-				add_vscroll = true;
-				AddVerticalScrollBar (open_node_count, true);
-				return;
+			if (vbar.Visible) {
+				vbar.Bounds = new Rectangle (ClientRectangle.Width - vbar.Width, 0, vbar.Width,
+						ClientRectangle.Height - (hbar.Visible ? SystemInformation.HorizontalScrollBarHeight : 0));
 			}
 
-			if (vbar != null && vbar.Visible) {
-				int height = (hbar != null && hbar.Visible ? Height - hbar.Height : Height);
-				vbar.SetBounds (Right - vbar.Width, 0, 0, height, BoundsSpecified.X | BoundsSpecified.Height);
+			if (hbar.Visible) {
+				hbar.Bounds = new Rectangle (0, ClientRectangle.Height - hbar.Height,
+						ClientRectangle.Width - (vbar.Visible ? SystemInformation.VerticalScrollBarWidth : 0), hbar.Height);
 			}
-
-			if (hbar != null && hbar.Visible) {
-				int width = (vbar != null && vbar.Visible ? Width - vbar.Width : Width);
-				hbar.SetBounds (0, Bottom - hbar.Height, width, 0, BoundsSpecified.Y | BoundsSpecified.Width);
-			}
-
-			ResumeLayout ();
 		}
 
 		private void VScrollBarValueChanged (object sender, EventArgs e)
 		{
+			EndEdit (edit_node);
+
 			SetVScrollPos (vbar.Value, null);
 		}
 
@@ -1346,22 +1424,67 @@ namespace System.Windows.Forms {
 
 		private void HScrollBarValueChanged(object sender, EventArgs e)
 		{
+			EndEdit (edit_node);
+
 			int old_offset = hbar_offset;
 			hbar_offset = hbar.Value;
 
-			if (hbar_offset < 0)
+			if (hbar_offset < 0) {
 				hbar_offset = 0;
+			}
 
 			XplatUI.ScrollWindow (Handle, ViewportRectangle, old_offset - hbar_offset, 0, false);
 		}
 
+		internal void ExpandBelow (TreeNode node, int count_to_next)
+		{
+			if (update_stack > 0) {
+				update_needed = true;
+				return;
+			}
+
+			Rectangle below = new Rectangle (0, node.Bounds.Bottom, ViewportRectangle.Width,
+					ViewportRectangle.Height - node.Bounds.Bottom);
+				
+			int amount = count_to_next * ItemHeight;
+
+			if (amount > 0)
+				XplatUI.ScrollWindow (Handle, below, 0, amount, false);
+
+			if (show_plus_minus) {
+				//int linesx = node.GetLinesX ();
+				Invalidate (new Rectangle (0, node.GetY (), Width, ItemHeight));
+			}
+		}
+
+		internal void CollapseBelow (TreeNode node, int count_to_next)
+		{
+			if (update_stack > 0) {
+				update_needed = true;
+				return;
+			}
+
+			Rectangle below = new Rectangle (0, node.Bounds.Bottom, ViewportRectangle.Width,
+					ViewportRectangle.Height - node.Bounds.Bottom);
+			int amount = count_to_next * ItemHeight;
+
+			if (amount > 0)
+				XplatUI.ScrollWindow (Handle, below, 0, -amount, false);
+
+			if (show_plus_minus) {
+				//int linesx = node.GetLinesX ();
+				Invalidate (new Rectangle (0, node.GetY (), Width, ItemHeight));
+			}
+		}
+
 		private void MouseWheelHandler(object sender, MouseEventArgs e) {
-			if (vbar == null || !vbar.Visible) {
+
+			if (vbar == null || !vbar.is_visible) {
 				return;
 			}
 
 			if (e.Delta < 0) {
-				vbar.Value = Math.Min(vbar.Value + SystemInformation.MouseWheelScrollLines, vbar.Maximum);
+				vbar.Value = Math.Min(vbar.Value + SystemInformation.MouseWheelScrollLines, vbar.Maximum - VisibleCount + 1);
 			} else {
 				vbar.Value = Math.Max(0, vbar.Value - SystemInformation.MouseWheelScrollLines);
 			}
@@ -1369,15 +1492,30 @@ namespace System.Windows.Forms {
 
 		private void FontChangedHandler (object sender, EventArgs e)
 		{
-			update_node_bounds = true;
+			InvalidateNodeWidthRecursive (root_node);
 		}
 
-		private void FocusChangedHandler (object sender, EventArgs e)
+		private void InvalidateNodeWidthRecursive (TreeNode node)
 		{
-			if (selected_node != null)
+			node.InvalidateWidth ();
+			foreach (TreeNode child in node.Nodes) {
+				InvalidateNodeWidthRecursive (child);
+			}
+		}
+
+		private void GotFocusHandler (object sender, EventArgs e)
+		{
+			if (selected_node == null)
+				SelectedNode = top_node;
+			else
 				UpdateNode (selected_node);
 		}
 
+		private void LostFocusHandler (object sender, EventArgs e)
+		{
+			UpdateNode (SelectedNode);
+		}
+
 		private void MouseDownHandler (object sender, MouseEventArgs e)
 		{
 			TreeNode node = GetNodeAt (e.Y);
@@ -1394,15 +1532,10 @@ namespace System.Windows.Forms {
 				return;
 			} else if (IsSelectableArea (node, e.X) || full_row_select) {
 				TreeNode old_selected = selected_node;
+				selection_action = TreeViewAction.ByMouse;
 				SelectedNode = node;
 				if (label_edit && e.Clicks == 1 && selected_node == old_selected) {
-					node.BeginEdit ();
-					if (edit_node != null) {
-						edit_node.EndEdit (false);
-						UpdateNode (edit_node);
-					}
-					edit_node = node;
-					UpdateNode (edit_node);
+					BeginEdit (node);
 				} else if (selected_node != focused_node) {
 					select_mmove = true;
 				}
@@ -1473,7 +1606,8 @@ namespace System.Windows.Forms {
 
 			selected_node = focused_node;
 			select_mmove = false;
-			Refresh();
+			Invalidate (focused_node.Bounds);
+			Invalidate (selected_node.Bounds);
 		}
 
 		private void DoubleClickHandler (object sender, MouseEventArgs e) {
