@@ -65,7 +65,7 @@ namespace System.Windows.Forms
 				this.SetStyle(ControlStyles.Opaque, true);
 				this.SetStyle(ControlStyles.ResizeRedraw, true);
 				this.SetStyle(ControlStyles.UserPaint, true);
-				this.SetStyle(ControlStyles.Selectable, false);
+				this.SetStyle(ControlStyles.FixedHeight, true);
 
 				tmrRepeat = new Timer();
 
@@ -137,8 +137,6 @@ namespace System.Windows.Forms
 
 			#region Protected Instance Methods
 			protected override void OnMouseDown(MouseEventArgs e) {
-				this.Select(owner.txtView);
-
 				if (e.Button != MouseButtons.Left) {
 					return;
 				}
@@ -222,11 +220,63 @@ namespace System.Windows.Forms
 				compute_rects();
 			}
 			#endregion	// Protected Instance Methods
+
+			
+			internal void SetSelectable (bool selectable)
+			{
+				SetStyle (ControlStyles.Selectable, selectable);
+			}
+
+			internal override void OnGotFocusInternal (EventArgs e)
+			{
+				owner.Select (false, true);
+			}
+
+			internal override void OnLostFocusInternal (EventArgs e)
+			{
+				owner.Select (false, true);
+			}
 		}
 		#endregion	// UpDownSpinner Sub-class
 
+		internal class UpDownTextBox : TextBox {
+
+			private UpDownBase owner;
+
+			public UpDownTextBox (UpDownBase owner)
+			{
+				this.owner = owner;
+
+				SetStyle (ControlStyles.FixedWidth, false);
+			}
+
+			internal void SetSelectable (bool selectable)
+			{
+				SetStyle (ControlStyles.Selectable, selectable);
+			}
+
+			internal void ActivateCaret (bool active)
+			{
+				if (active)
+					document.CaretHasFocus ();
+				else
+					document.CaretLostFocus ();
+			}
+
+			
+			internal override void OnGotFocusInternal (EventArgs e)
+			{
+				owner.Select (false, true);
+			}
+
+			internal override void OnLostFocusInternal (EventArgs e)
+			{
+				owner.Select (false, true);
+			}			
+		}
+
 		#region Local Variables
-		internal TextBox		txtView;
+		internal UpDownTextBox		txtView;
 		private UpDownSpinner		spnSpinner;
 		private bool			_InterceptArrowKeys = true;
 		private LeftRightAlignment	_UpDownAlign;
@@ -241,7 +291,7 @@ namespace System.Windows.Forms
 
 			spnSpinner = new UpDownSpinner(this);
 
-			txtView = new FixedSizeTextBox();
+			txtView = new UpDownTextBox (this);
 			txtView.ModifiedChanged += new EventHandler(OnChanged);
 			txtView.AcceptsReturn = true;
 			txtView.AutoSize = false;
@@ -250,8 +300,8 @@ namespace System.Windows.Forms
 			txtView.TabIndex = TabIndex;
 
 			SuspendLayout ();
-			Controls.AddImplicit (txtView);
-			Controls.AddImplicit (spnSpinner);
+			Controls.Add (spnSpinner);
+			Controls.Add (txtView);
 			ResumeLayout ();
 
 			this.ActiveControl = txtView;
@@ -259,18 +309,19 @@ namespace System.Windows.Forms
 			Height = PreferredHeight;
 			base.BackColor = txtView.BackColor;
 
-			GotFocus += new EventHandler (GotFocusHandler);
 			TabIndexChanged += new EventHandler (TabIndexChangedHandler);
 			
 			txtView.MouseWheel += new MouseEventHandler(txtView_MouseWheel);
 			txtView.KeyDown += new KeyEventHandler(OnTextBoxKeyDown);
 			txtView.KeyPress += new KeyPressEventHandler(OnTextBoxKeyPress);
-			txtView.LostFocus += new EventHandler(OnTextBoxLostFocus);
+//			txtView.LostFocus += new EventHandler(OnTextBoxLostFocus);
 			txtView.Resize += new EventHandler(OnTextBoxResize);
 			txtView.TextChanged += new EventHandler(OnTextBoxTextChanged);
 
 			txtView.Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right;
 
+			// So the child controls don't get auto selected when the updown is selected
+			auto_select_child = false;
 			SetStyle(ControlStyles.FixedHeight, true);
 
 			UpdateEditText();
@@ -306,11 +357,6 @@ namespace System.Windows.Forms
 			}
 		}
 
-		private void GotFocusHandler (object sender, EventArgs e)
-		{
-			txtView.Focus ();
-		}
-
 		private void TabIndexChangedHandler (object sender, EventArgs e)
 		{
 			txtView.TabIndex = TabIndex;
@@ -319,6 +365,22 @@ namespace System.Windows.Forms
 		internal override void OnPaintInternal (PaintEventArgs e) {
 			e.Graphics.FillRectangle(ThemeEngine.Current.ResPool.GetSolidBrush(BackColor), ClientRectangle);
 		}
+
+		internal override void OnGotFocusInternal (EventArgs e)
+		{
+			base.OnGotFocusInternal (e);
+			txtView.ActivateCaret (true);
+			txtView.SetSelectable (false);
+			spnSpinner.SetSelectable (false);
+		}
+
+		internal override void OnLostFocusInternal (EventArgs e)
+		{
+			base.OnLostFocusInternal (e);
+			txtView.ActivateCaret (false);
+			txtView.SetSelectable (true);
+			spnSpinner.SetSelectable (true);
+		}
 		#endregion	// Private Methods
 
 		#region Public Instance Properties
@@ -417,7 +479,7 @@ namespace System.Windows.Forms
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public override bool Focused {
 			get {
-				return txtView.Focused;
+				return base.Focused;
 			}
 		}
 
@@ -479,7 +541,10 @@ namespace System.Windows.Forms
 		[Localizable(true)]
 		public override string Text {
 			get {
-				return txtView.Text;
+				if (txtView != null) {
+					return txtView.Text;
+				}
+				return "";
 			}
 			set {
 				bool suppress_validation = changing_text;
@@ -653,6 +718,14 @@ namespace System.Windows.Forms
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void WndProc(ref Message m) {
+
+			switch((Msg) m.Msg) {
+			case Msg.WM_KEYUP:
+			case Msg.WM_KEYDOWN:
+			case Msg.WM_CHAR:
+				XplatUI.SendMessage (txtView.Handle, (Msg) m.Msg, m.WParam, m.LParam);
+				break;
+			}
 			base.WndProc (ref m);
 		}
 		#endregion	// Protected Instance Methods
