@@ -332,6 +332,13 @@ namespace System.Windows.Forms {
 			Init ();
 		}
 
+		public bool InDrag()
+		{
+			if (drag_data == null)
+				return false;
+			return drag_data.State != DragState.None;
+		}
+		
 		public void SetAllowDrop (Hwnd hwnd, bool allow)
 		{
 			int[] atoms;
@@ -464,7 +471,7 @@ namespace System.Windows.Forms {
 							xevent.MotionEvent.y_root = y_temp;
 						}
 					}
-					
+
 					if (child == IntPtr.Zero)
 						break;
 					
@@ -634,9 +641,10 @@ namespace System.Windows.Forms {
 
 			allowed = EffectFromAction (xevent.ClientMessageEvent.ptr5);
 
-			IntPtr parent, child, new_child;
+			IntPtr parent, child, new_child, last_drop_child;
 			parent = XplatUIX11.XRootWindow (display, 0);
 			child = toplevel;
+			last_drop_child = IntPtr.Zero;
 			while (true) {
 				int xd, yd;
 				new_child = IntPtr.Zero;
@@ -648,8 +656,16 @@ namespace System.Windows.Forms {
 				if (new_child == IntPtr.Zero)
 					break;
 				child = new_child;
+
+				Hwnd h = Hwnd.ObjectFromHandle (child);
+				Control d = Control.FromHandle (h.client_window);
+				if (d != null && d.allow_drop)
+					last_drop_child = child;
 			}
 
+			if (last_drop_child != IntPtr.Zero)
+				child = last_drop_child;
+
 			if (target != child) {
 				// We have moved into a new control 
 				// or into a control for the first time
