@@ -41,28 +41,35 @@ namespace System.Windows.Forms {
 	internal class X11Keyboard {
 
 		private IntPtr display;
-		private IntPtr xim;
+		private IntPtr window;
 		private IntPtr xic;
 		private StringBuilder lookup_buffer;
 		private int min_keycode, max_keycode, keysyms_per_keycode, syms;
 		private int [] keyc2vkey = new int [256];
 		private int [] keyc2scan = new int [256];
 		private byte [] key_state_table = new byte [256];
+		private int lcid;
 		private bool num_state, cap_state;
-		private KeyboardLayout layout;
+		private bool initialized;
 
-		// TODO
 		private int NumLockMask;
 		private int AltGrMask;
-		
+
 		public X11Keyboard (IntPtr display, IntPtr window)
 		{
 			this.display = display;
+			this.window = window;
 			lookup_buffer = new StringBuilder (24);
+		}
+
+		public void EnsureLayoutInitialized ()
+		{
+			if (initialized)
+				return;
 
 			KeyboardLayouts layouts = new KeyboardLayouts ();
-			layout = layouts.Layouts [0];
-			DetectLayout (layouts);
+			KeyboardLayout layout = DetectLayout (layouts);
+			lcid = layout.Lcid;
 			CreateConversionArray (layouts, layout);
 
 			if (!XSupportsLocale ()) {
@@ -73,12 +80,14 @@ namespace System.Windows.Forms {
 				Console.Error.WriteLine ("Could not set X locale modifiers");
 			}
 
-			xim = XOpenIM (display, IntPtr.Zero, IntPtr.Zero, IntPtr.Zero);
+			IntPtr xim = XOpenIM (display, IntPtr.Zero, IntPtr.Zero, IntPtr.Zero);
 			if (xim == IntPtr.Zero) {
 				Console.Error.WriteLine ("Could not get XIM");
 			}
 
-			xic = CreateXic (window);
+			xic = CreateXic (window, xim);
+			
+			initialized = true;
 		}
 
 		public Keys ModifierKeys {
@@ -125,6 +134,8 @@ namespace System.Windows.Forms {
 
 		public void KeyEvent (IntPtr hwnd, XEvent xevent, ref MSG msg)
 		{
+			EnsureLayoutInitialized ();
+
 			XKeySym keysym;
 			int ascii_chars;
 
@@ -198,6 +209,8 @@ namespace System.Windows.Forms {
 			if (msg.message != Msg.WM_KEYDOWN && msg.message != Msg.WM_SYSKEYDOWN)
 				return res;
 
+			EnsureLayoutInitialized ();
+
 			string buffer;
 			Msg message;
 			int tu = ToUnicode ((int) msg.wParam, Control.HighOrder ((int) msg.lParam), out buffer);
@@ -263,6 +276,9 @@ namespace System.Windows.Forms {
 
 			if (vkey == (int) VirtualKeys.VK_DECIMAL)
 				e.KeyEvent.keycode = XKeysymToKeycode (display, (int) KeypadKeys.XK_KP_Decimal);
+			
+			if (vkey == (int) VirtualKeys.VK_SEPARATOR)
+				e.KeyEvent.keycode = XKeysymToKeycode(display, (int) KeypadKeys.XK_KP_Separator);
 
 			if (e.KeyEvent.keycode == 0 && vkey != (int) VirtualKeys.VK_NONAME) {
 				// And I couldn't find the keycode so i returned the vkey and was like whatever
@@ -282,7 +298,7 @@ namespace System.Windows.Forms {
 				if (dead_char != 0) {
 					byte [] bytes = new byte [1];
 					bytes [0] = (byte) dead_char;
-					Encoding encoding = Encoding.GetEncoding (new CultureInfo (layout.Lcid).TextInfo.ANSICodePage);
+					Encoding encoding = Encoding.GetEncoding (new CultureInfo (lcid).TextInfo.ANSICodePage);
 					buffer = new string (encoding.GetChars (bytes));
 					res = -1;
 				}
@@ -407,14 +423,17 @@ namespace System.Windows.Forms {
 
 		public int EventToVkey (XEvent e)
 		{
+			EnsureLayoutInitialized ();
+
 			IntPtr status;
 			XKeySym ks;
 
 			LookupString (ref e, 0, out ks, out status);
 			int keysym = (int) ks;
 
-			if ((keysym >= 0xFFAE) && (keysym <= 0xFFB9) && (keysym != 0xFFAF)
-					&& ((e.KeyEvent.state & NumLockMask) !=0)) {
+			if (((e.KeyEvent.state & NumLockMask) != 0) &&
+			    (keysym == (int)KeypadKeys.XK_KP_Separator || keysym == (int)KeypadKeys.XK_KP_Decimal ||
+			    (keysym >= (int)KeypadKeys.XK_KP_0 && keysym <= (int)KeypadKeys.XK_KP_9))) {
 				// Only the Keypad keys 0-9 and . send different keysyms
 				// depending on the NumLock state
 				return nonchar_key_vkey [keysym & 0xFF];
@@ -423,7 +442,7 @@ namespace System.Windows.Forms {
 			return keyc2vkey [e.KeyEvent.keycode];
 		}
 
-		public void CreateConversionArray (KeyboardLayouts layouts, KeyboardLayout layout)
+		private void CreateConversionArray (KeyboardLayouts layouts, KeyboardLayout layout)
 		{
 			XEvent e2 = new XEvent ();
 			uint keysym = 0;
@@ -482,7 +501,6 @@ namespace System.Windows.Forms {
 							}
 						}
 						if (maxval >= 0) {
-							/// XXX
 							scan = layouts.scan_table [(int) layout.ScanIndex][maxval];
 							vkey = layouts.vkey_table [(int) layout.VKeyIndex][maxval];
 						}
@@ -496,7 +514,7 @@ namespace System.Windows.Forms {
 			
 		}
 
-		public void DetectLayout (KeyboardLayouts layouts)
+		private KeyboardLayout DetectLayout (KeyboardLayouts layouts)
 		{
 			XDisplayKeycodes (display, out min_keycode, out max_keycode);
 			IntPtr ksp = XGetKeyboardMapping (display, (byte) min_keycode,
@@ -592,11 +610,13 @@ namespace System.Windows.Forms {
 			}
 
 			if (layout != null)  {
-                                this.layout = layout;
-				Console.WriteLine (Locale.GetText("Keyboard") + ": " + layout.Name);
+                                return layout;
 			} else {
-				Console.WriteLine (Locale.GetText("Keyboard layout not recognized, using default layout: " + this.layout.Name));
+				Console.WriteLine (Locale.GetText("Keyboard layout not recognized, using default layout: " +
+								   layouts.Layouts [0].Name));
 			}
+
+			return layouts.Layouts [0];
 		}
 
 		// TODO
@@ -639,7 +659,7 @@ namespace System.Windows.Forms {
 			return 0;
 		}
 
-		internal IntPtr CreateXic (IntPtr window)
+		private IntPtr CreateXic (IntPtr window, IntPtr xim)
 		{
 			xic = XCreateIC (xim, 
 				"inputStyle", XIMProperties.XIMPreeditNothing | XIMProperties.XIMStatusNothing,
@@ -738,7 +758,7 @@ namespace System.Windows.Forms {
 		internal extern static int XFreeModifiermap (IntPtr modmap);
 
 
-		public readonly static int [] nonchar_key_vkey = new int []
+		private readonly static int [] nonchar_key_vkey = new int []
 		{
 			/* unused */
 			0, 0, 0, 0, 0, 0, 0, 0,					    /* FF00 */
@@ -788,7 +808,7 @@ namespace System.Windows.Forms {
 			0, 0, 0, 0, 0, 0, 0, (int) VirtualKeys.VK_DELETE			      /* FFF8 */
 		};
 
-		public static readonly int [] nonchar_key_scan = new int []
+		private static readonly int [] nonchar_key_scan = new int []
 		{
 			/* unused */
 			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		     /* FF00 */
