@@ -85,7 +85,7 @@ namespace System.Windows.Forms {
 			// http://www.mono-project.com/FAQ:_Technical#How_to_detect_the_execution_platform_.3F
 			int platform = (int) Environment.OSVersion.Platform;
 			if ((platform == 4) || (platform == 128)) {
-				if (Environment.GetEnvironmentVariable ("MONO_MWF_USE_QUARTZ_BACKEND") != null)
+				if (Environment.GetEnvironmentVariable ("not_supported_MONO_MWF_USE_QUARTZ_BACKEND") != null)
 					driver=XplatUIOSX.GetInstance();
 				else
 					driver=XplatUIX11.GetInstance();
@@ -201,6 +201,12 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		static public int MenuHeight {
+			get {
+				return driver.MenuHeight;
+			}
+		}
+
 		static public int MouseButtonCount {
 			get {
 				return driver.MouseButtonCount;
@@ -231,6 +237,12 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		static public bool UserClipWontExposeParent {
+			get {
+				return driver.UserClipWontExposeParent;
+			}
+		}
+
 		static public int VerticalScrollBarWidth {
 			get {
 				return driver.VerticalScrollBarWidth;
@@ -248,6 +260,14 @@ namespace System.Windows.Forms {
 				return driver.WorkingArea;
 			}
 		}
+
+		public static bool ThemesEnabled {
+			get {
+				return XplatUI.driver.ThemesEnabled;
+			}
+		}
+ 
+
 		#endregion	// Public Static Properties
 
 		#region Events
@@ -450,6 +470,13 @@ namespace System.Windows.Forms {
 			return driver.GetAutoScaleSize(font);
 		}
 
+		internal static Region GetClipRegion(IntPtr handle) {
+			#if DriverDebug
+				Console.WriteLine("GetClipRegion({0}): Called", Window(handle));
+			#endif
+			return driver.GetClipRegion(handle);
+		}
+
 		internal static void GetCursorInfo(IntPtr cursor, out int width, out int height, out int hotspot_x, out int hotspot_y) {
 			#if DriverDebug
 				Console.WriteLine("GetCursorInfo({0}): Called", cursor.ToInt32());
@@ -471,6 +498,13 @@ namespace System.Windows.Forms {
 			driver.GetDisplaySize(out size);
 		}
 
+		internal static IntPtr GetFocus() {
+			#if DriverDebug
+				Console.WriteLine("GetFocus(): Called, Result:{0}", Window(driver.GetFocus()));
+			#endif
+			return driver.GetFocus();
+		}
+
 		internal static bool GetFontMetrics(Graphics g, Font font, out int ascent, out int descent) {
 			#if DriverDebug
 				Console.WriteLine("GetFontMetrics(): Called");
@@ -485,8 +519,8 @@ namespace System.Windows.Forms {
 			return driver.GetMenuOrigin(handle);
 		}
 
-		internal static bool GetMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
-			return driver.GetMessage(ref msg, hWnd, wFilterMin, wFilterMax);
+		internal static bool GetMessage(object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
+			return driver.GetMessage(queue_id, ref msg, hWnd, wFilterMin, wFilterMax);
 		}
 
 		internal static IntPtr GetParent(IntPtr handle) {
@@ -592,8 +626,8 @@ namespace System.Windows.Forms {
 			return driver.PaintEventStart(handle, client);
 		}
 
-		internal static bool PeekMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
-			return driver.PeekMessage(ref msg, hWnd, wFilterMin, wFilterMax, flags);
+		internal static bool PeekMessage(Object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
+			return driver.PeekMessage(queue_id, ref msg, hWnd, wFilterMin, wFilterMax, flags);
 		}
 
 		internal static bool PostMessage(IntPtr hwnd, Msg message, IntPtr wParam, IntPtr lParam) {
@@ -702,6 +736,13 @@ namespace System.Windows.Forms {
 			driver.SetCaretPos(handle, x, y);
 		}
 
+		internal static void SetClipRegion(IntPtr handle, Region region) {
+			#if DriverDebug
+				Console.WriteLine("SetClipRegion({0}, {1}): Called", Window(handle), region);
+			#endif
+			driver.SetClipRegion(handle, region);
+		}
+
 		internal static void SetCursor(IntPtr handle, IntPtr cursor) {
 			#if DriverDebug
 				Console.WriteLine("SetCursor({0}, {1:X}): Called", Window(handle), cursor.ToInt32());
@@ -830,16 +871,16 @@ namespace System.Windows.Forms {
 			return driver.StartDrag (handle, data, allowedEffects);
 		}
 
-		internal static void StartLoop(Thread thread) {
+		internal static object StartLoop(Thread thread) {
 			#if DriverDebug
 				Console.WriteLine("StartLoop({0:X}): Called", thread.GetHashCode());
 			#endif
-			driver.StartLoop(thread);
+			return driver.StartLoop(thread);
 		}
 
 		internal static bool SupportsTransparency() {
 			#if DriverDebug
-				Console.WriteLine("SupportsTransparency(): Called, result={1}", driver.SupportsTransparency());
+				Console.WriteLine("SupportsTransparency(): Called, result={0}", driver.SupportsTransparency());
 			#endif
 			return driver.SupportsTransparency();
 		}
