@@ -86,6 +86,12 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal virtual int MenuHeight {
+			get {
+				return 19;
+			}
+		}
+
 		internal virtual bool DropTarget {
 			get {
 				return false;
@@ -101,6 +107,12 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal virtual bool UserClipWontExposeParent {
+			get {
+				return true;
+			}
+		}
+
 		internal virtual int VerticalScrollBarWidth {
 			get {
 				return 16;
@@ -124,6 +136,8 @@ namespace System.Windows.Forms {
 		internal abstract bool MouseWheelPresent { get; }
 		internal abstract Rectangle VirtualScreen { get; }
 		internal abstract Rectangle WorkingArea { get; }
+		internal abstract bool ThemesEnabled { get; }
+
 		#endregion	// XplatUI Driver Properties
 
                 internal abstract event EventHandler Idle;
@@ -184,9 +198,9 @@ namespace System.Windows.Forms {
 		internal abstract IntPtr DefWndProc(ref Message msg);
 		internal abstract void HandleException(Exception e);
 		internal abstract void DoEvents();
-		internal abstract bool PeekMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags);
+		internal abstract bool PeekMessage(Object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags);
 		internal abstract void PostQuitMessage(int exitCode);
-		internal abstract bool GetMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax);
+		internal abstract bool GetMessage(object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax);
 		internal abstract bool TranslateMessage(ref MSG msg);
 		internal abstract IntPtr DispatchMessage(ref MSG msg);
 
@@ -195,6 +209,9 @@ namespace System.Windows.Forms {
 
 		internal abstract bool CalculateWindowRect(ref Rectangle ClientRect, int Style, int ExStyle, Menu menu, out Rectangle WindowRect);
 
+		internal abstract Region GetClipRegion(IntPtr hwnd);
+		internal abstract void SetClipRegion(IntPtr hwnd, Region region);
+
 		internal abstract void SetCursor(IntPtr hwnd, IntPtr cursor);
 		internal abstract void ShowCursor(bool show);
 		internal abstract void OverrideCursor(IntPtr cursor);
@@ -222,6 +239,7 @@ namespace System.Windows.Forms {
 		internal abstract void SetCaretPos(IntPtr hwnd, int x, int y);
 		internal abstract void CaretVisible(IntPtr hwnd, bool visible);
 
+		internal abstract IntPtr GetFocus();
 		internal abstract void SetFocus(IntPtr hwnd);
 		internal abstract IntPtr GetActive();
 
@@ -254,7 +272,7 @@ namespace System.Windows.Forms {
 		internal abstract IntPtr SendMessage(IntPtr hwnd, Msg message, IntPtr wParam, IntPtr lParam);
 		internal abstract bool PostMessage(IntPtr hwnd, Msg message, IntPtr wParam, IntPtr lParam);
 
-		internal abstract void StartLoop(Thread thread);
+		internal abstract object StartLoop(Thread thread);
 		internal abstract void EndLoop(Thread thread);
 
 		internal abstract void RequestNCRecalc(IntPtr hwnd);
