@@ -51,7 +51,7 @@ namespace System.Windows.Forms {
 		// General driver variables
 		private static XplatUIOSX Instance;
 		private static int RefCount;
-		private static bool ThemesEnabled;
+		private static bool themes_enabled;
 		private static IntPtr FocusWindow;
 
 		// Mouse 
@@ -759,7 +759,7 @@ namespace System.Windows.Forms {
 		}
 
 		internal override void EnableThemes() {
-			ThemesEnabled = true;
+			themes_enabled = true;
 		}
 
 		internal override void Activate(IntPtr handle) {
@@ -1154,7 +1154,11 @@ namespace System.Windows.Forms {
 
 			return IntPtr.Zero;
 		}
-		
+
+		internal override Region GetClipRegion(IntPtr hwnd) {
+			return null;
+		}
+
 		[MonoTODO]
 		internal override void GetCursorInfo(IntPtr cursor, out int width, out int height, out int hotspot_x, out int hotspot_y) {
 			throw new NotImplementedException ();
@@ -1181,6 +1185,11 @@ namespace System.Windows.Forms {
 			x = pt.x;
 			y = pt.y;
 		}
+
+		internal override IntPtr GetFocus() {
+			return FocusWindow;
+		}
+
 		
 		internal override bool GetFontMetrics(Graphics g, Font font, out int ascent, out int descent) {
 			return GetFontMetrics(g.GetHdc(), font.ToHfont(), out ascent, out descent);
@@ -1191,7 +1200,7 @@ namespace System.Windows.Forms {
 			throw new NotImplementedException();
 		}
 
-		internal override bool GetMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
+		internal override bool GetMessage(object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
 			IntPtr evtRef = IntPtr.Zero;
 			IntPtr target = GetEventDispatcherTarget();
 			CheckTimers (DateTime.Now);
@@ -1343,7 +1352,7 @@ namespace System.Windows.Forms {
 			}
 
 			hwnd.client_dc  = Graphics.FromHwnd (hwnd.client_window);
-			paint_event = new PaintEventArgs(hwnd.client_dc, hwnd.invalid);
+			paint_event = new PaintEventArgs(hwnd.client_dc, hwnd.Invalid);
 			
 			return paint_event;
 		}
@@ -1366,7 +1375,7 @@ namespace System.Windows.Forms {
 			}
 		}
 		
-		internal override bool PeekMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
+		internal override bool PeekMessage(Object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
 			Console.WriteLine("XplatUIOSX.PeekMessage");
 			return true;
 		}
@@ -1511,6 +1520,10 @@ namespace System.Windows.Forms {
 				}
 			}
 		}
+
+		internal override void SetClipRegion(IntPtr hwnd, Region region) {
+			throw new NotImplementedException();
+		}
 		
 		internal override void SetCursor(IntPtr window, IntPtr cursor) {
 			SetThemeCursor ((uint) cursor);
@@ -1711,7 +1724,7 @@ namespace System.Windows.Forms {
 				CGDisplayHideCursor (CGMainDisplayID ());
 		}
 
-		internal override void StartLoop(Thread thread) {
+		internal override object StartLoop(Thread thread) {
 			throw new NotImplementedException();
 		}
 		
@@ -1834,6 +1847,13 @@ namespace System.Windows.Forms {
 				return new Rectangle ((int)bounds.origin.x, (int)bounds.origin.y, (int)bounds.size.width, (int)bounds.size.height);
 			}
 		}
+		internal override bool ThemesEnabled {
+			get {
+				return XplatUIOSX.themes_enabled;
+			}
+		}
+ 
+
 		#endregion
 		
 		[DllImport("/System/Library/Frameworks/Carbon.framework/Versions/Current/Carbon")]
