@@ -722,7 +722,7 @@ namespace System.Windows.Forms {
 			BITMAPINFOHEADER	bmi;
 			int			ncolors;
 			int			imagesize;
-			int			palettesize;
+			//int			palettesize;
 			Bitmap			bmp;
 			BitmapData		bits;
 			ColorPalette		pal;
@@ -738,7 +738,7 @@ namespace System.Windows.Forms {
 					ncolors = (int)(1 << bmi.biBitCount);
 				}
 			}
-			palettesize = ncolors * 4;
+			//palettesize = ncolors * 4;
 
 			imagesize = (int)bmi.biSizeImage;
 			if (imagesize == 0) {
@@ -781,8 +781,9 @@ namespace System.Windows.Forms {
 				Marshal.Copy((IntPtr)((int)dib_data + Marshal.SizeOf(typeof(BITMAPINFOHEADER))), palette, 0, palette.Length);
 
 				for (int i = 0; i < ncolors; i++) {
-					pal.Entries[i] = Color.FromArgb(palette[i] & unchecked((int)0xff000000));
+					pal.Entries[i] = Color.FromArgb(palette[i] | unchecked((int)0xff000000));
 				}
+				bmp.Palette = pal;
 			}
 
 			bytesPerLine = (int)((((bmi.biWidth * bmi.biBitCount) + 31) & ~31) >> 3);
@@ -903,12 +904,25 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal override bool UserClipWontExposeParent {
+			get {
+				return false;
+			}
+		}
+
+
 		internal override int VerticalScrollBarWidth {
 			get {
 				return scroll_width;
 			}
 		}
 
+		internal override int MenuHeight {
+			get {
+				return Win32GetSystemMetrics(SystemMetrics.SM_CYMENU);
+			}
+		}
+
 		internal override bool DropTarget {
 			get {
 				return false;
@@ -1028,6 +1042,14 @@ namespace System.Windows.Forms {
 				//return new Rectangle(0, 0, Win32GetSystemMetrics(SystemMetrics.SM.SM_CXSCREEN), Win32GetSystemMetrics(SystemMetrics.SM_CYSCREEN));
 			}
 		}
+
+		internal override bool ThemesEnabled {
+			get {
+				return XplatUIWin32.themes_enabled;
+			}
+		}
+ 
+
 		#endregion	// Static Properties
 
 		#region Singleton Specific Code
@@ -1296,11 +1318,7 @@ namespace System.Windows.Forms {
 			pt.y=rect.top;
 
 			parent = Win32GetParent(handle);
-			if (parent == FosterParent) {
-				Win32ScreenToClient(IntPtr.Zero, ref pt);
-			} else {
-				Win32ScreenToClient(parent, ref pt);
-			}
+			Win32ScreenToClient(parent, ref pt);
 
 			x = pt.x;
 			y = pt.y;
@@ -1349,7 +1367,7 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		internal override bool PeekMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
+		internal override bool PeekMessage(Object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
 			return Win32PeekMessage(ref msg, hWnd, wFilterMin, wFilterMax, flags);
 		}
 
@@ -1372,7 +1390,7 @@ namespace System.Windows.Forms {
 		}
 
 
-		internal override bool GetMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
+		internal override bool GetMessage(Object queue_id, ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
 			return GetMessage(ref msg, hWnd, wFilterMin, wFilterMax, true);
 		}
 
@@ -1700,6 +1718,20 @@ namespace System.Windows.Forms {
 			Win32SetCursorPos(x, y);
 		}
 
+		internal override Region GetClipRegion(IntPtr hwnd) {
+			Region region;
+
+			region = new Region();
+
+			Win32GetWindowRgn(hwnd, region.GetHrgn(Graphics.FromHwnd(hwnd)));
+
+			return region;
+		}
+
+		internal override void SetClipRegion(IntPtr hwnd, Region region) {
+			Win32SetWindowRgn(hwnd, region.GetHrgn(Graphics.FromHwnd(hwnd)), true);
+		}
+
 		internal override void EnableWindow(IntPtr handle, bool Enable) {
 			Win32EnableWindow(handle, Enable);
 		}
@@ -1708,8 +1740,8 @@ namespace System.Windows.Forms {
 			// Nothing to do
 		}
 
-		internal override void StartLoop(System.Threading.Thread thread) {
-			// Nothing to do
+		internal override object StartLoop(System.Threading.Thread thread) {
+			return null;
 		}
 
 		internal override void SetModal(IntPtr handle, bool Modal) {
@@ -1840,6 +1872,10 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal override IntPtr GetFocus() {
+			return Win32GetFocus();
+		}
+
 		internal override void SetFocus(IntPtr hwnd) {
 			Win32SetFocus(hwnd);
 		}
@@ -1879,7 +1915,7 @@ namespace System.Windows.Forms {
 			rect.right = rectangle.Right;
 			rect.bottom = rectangle.Bottom;
 
-			Win32ScrollWindowEx(hwnd, XAmount, YAmount, ref rect, ref rect, IntPtr.Zero, IntPtr.Zero, ScrollWindowExFlags.SW_INVALIDATE | ScrollWindowExFlags.SW_ERASE | (with_children ? ScrollWindowExFlags.SW_SCROLLCHILDREN : ScrollWindowExFlags.SW_NONE));
+			Win32ScrollWindowEx(hwnd, XAmount, YAmount, IntPtr.Zero, ref rect, IntPtr.Zero, IntPtr.Zero, ScrollWindowExFlags.SW_INVALIDATE | ScrollWindowExFlags.SW_ERASE | (with_children ? ScrollWindowExFlags.SW_SCROLLCHILDREN : ScrollWindowExFlags.SW_NONE));
 			Win32UpdateWindow(hwnd);
 		}
 
@@ -1959,6 +1995,7 @@ namespace System.Windows.Forms {
 			style = Win32GetWindowLong(handle, WindowLong.GWL_STYLE);
 			exstyle = Win32GetWindowLong(handle, WindowLong.GWL_EXSTYLE);
 
+
 			switch (border_style) {
 				case FormBorderStyle.None: {
 					style &= ~(uint)WindowStyles.WS_BORDER;
@@ -1973,7 +2010,7 @@ namespace System.Windows.Forms {
 				}
 
 				case FormBorderStyle.Fixed3D: {
-					style |= (uint)WindowStyles.WS_BORDER;
+					style &= ~(uint)WindowStyles.WS_BORDER;
 					exstyle |= (uint)WindowExStyles.WS_EX_CLIENTEDGE;
 					break;
 				}
@@ -1983,7 +2020,7 @@ namespace System.Windows.Forms {
 			Win32SetWindowLong(handle, WindowLong.GWL_EXSTYLE, exstyle);
 			
 			Win32SetWindowPos(handle, IntPtr.Zero, 0, 0, 0, 0, 
-				SetWindowPosFlags.SWP_FRAMECHANGED | SetWindowPosFlags.SWP_NOMOVE | SetWindowPosFlags.SWP_NOSIZE | SetWindowPosFlags.SWP_NOACTIVATE);
+				SetWindowPosFlags.SWP_FRAMECHANGED | SetWindowPosFlags.SWP_NOMOVE | SetWindowPosFlags.SWP_NOSIZE | SetWindowPosFlags.SWP_NOACTIVATE | SetWindowPosFlags.SWP_NOOWNERZORDER | SetWindowPosFlags.SWP_NOZORDER);
 		}
 
 		internal override void SetMenu(IntPtr handle, Menu menu) {
@@ -2472,6 +2509,9 @@ namespace System.Windows.Forms {
 		[DllImport ("user32.dll", EntryPoint="SetFocus", CallingConvention=CallingConvention.StdCall)]
 		internal extern static IntPtr Win32SetFocus(IntPtr hwnd);
 
+		[DllImport ("user32.dll", EntryPoint="GetFocus", CallingConvention=CallingConvention.StdCall)]
+		internal extern static IntPtr Win32GetFocus();
+
 		[DllImport ("user32.dll", EntryPoint="CreateCaret", CallingConvention=CallingConvention.StdCall)]
 		internal extern static bool Win32CreateCaret(IntPtr hwnd, IntPtr hBitmap, int nWidth, int nHeight);
 
@@ -2500,11 +2540,20 @@ namespace System.Windows.Forms {
 		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, ref RECT prcScroll, ref RECT prcClip, IntPtr hrgnUpdate, out RECT prcUpdate, ScrollWindowExFlags flags);
 
 		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
-		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, ref RECT prcScroll, ref RECT prcClip, IntPtr hrgnUpdate, IntPtr prcUpdate, ScrollWindowExFlags flags);
+		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, IntPtr prcScroll, ref RECT prcClip, IntPtr hrgnUpdate, out RECT prcUpdate, ScrollWindowExFlags flags);
+
+		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
+		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, ref RECT prcScroll, IntPtr prcClip, IntPtr hrgnUpdate, out RECT prcUpdate, ScrollWindowExFlags flags);
+
+		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
+		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, IntPtr prcScroll, ref RECT prcClip, IntPtr hrgnUpdate, IntPtr prcUpdate, ScrollWindowExFlags flags);
 
 		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
 		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, ref RECT prcScroll, IntPtr prcClip, IntPtr hrgnUpdate, IntPtr prcUpdate, ScrollWindowExFlags flags);
 
+		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
+		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, ref RECT prcScroll, ref RECT prcClip, IntPtr hrgnUpdate, IntPtr prcUpdate, ScrollWindowExFlags flags);
+
 		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
 		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, IntPtr prcScroll, IntPtr prcClip, IntPtr hrgnUpdate, IntPtr prcUpdate, ScrollWindowExFlags flags);
 
@@ -2612,6 +2661,12 @@ namespace System.Windows.Forms {
 
 		[DllImport ("winmm.dll", EntryPoint="PlaySoundW", CallingConvention=CallingConvention.StdCall, CharSet=CharSet.Unicode)]
 		internal extern static IntPtr Win32PlaySound(string pszSound, IntPtr hmod, SndFlags fdwSound);
+
+		[DllImport ("user32.dll", EntryPoint="SetWindowRgn", CallingConvention=CallingConvention.StdCall, CharSet=CharSet.Unicode)]
+		internal extern static int Win32SetWindowRgn(IntPtr hWnd, IntPtr hRgn, bool redraw);
+
+		[DllImport ("user32.dll", EntryPoint="GetWindowRgn", CallingConvention=CallingConvention.StdCall, CharSet=CharSet.Unicode)]
+		internal extern static IntPtr Win32GetWindowRgn(IntPtr hWnd, IntPtr hRgn);
 		#endregion
 	}
 }
