@@ -250,13 +250,13 @@ namespace MonoTests.System.Windows.Forms
 			listBox.SelectionMode = (SelectionMode) 10;
 		}
 
-		[Test][ExpectedException(typeof(InvalidOperationException))]
+		[Test]
 		public void SelectedValueNull()
 		{
 			listBox.SelectedValue = null;
 		}
 
-		[Test][ExpectedException(typeof(InvalidOperationException))]
+		[Test]
 		public void SelectedValueEmptyString()
 		{
 			listBox.SelectedValue = String.Empty;
@@ -349,5 +349,21 @@ namespace MonoTests.System.Windows.Forms
 			Assert.AreEqual (true, col.Contains ("Item2"), "#H1");
 		}
 
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void IndexerNullTest ()
+		{
+			col.Add ("Item1");
+			col [0] = null;
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void AddNullTest ()
+		{
+			col.Add (null);
+		}
+
+
 	}
 }
