@@ -28,39 +28,39 @@
 #if NET_2_0
 
 using System;
-using System.Collections;
+using System.Collections.Generic;
 using System.Xml;
 
 namespace Microsoft.Build.BuildEngine {
 	internal class BuildChoose {
 		
-		bool		isImported;
 		BuildWhen	otherwise;
 		Project		project;
-		IList		whens;
+		XmlElement	xmlElement;
+		List <BuildWhen>	whens;
 		
-		public BuildChoose (bool imported, Project project)
+		public BuildChoose (XmlElement xmlElement, Project project)
 		{
-			this.isImported = imported;
-			this.project = project; 
-			this.whens = new ArrayList ();
+			this.xmlElement = xmlElement;
+			this.project = project;
+			this.whens = new List <BuildWhen> ();
 		}
 		
-		public void BindToXml (XmlElement chooseElement)
+		public void Evaluate ()
 		{
 		}
 		
-		public bool IsImported {
-			get { return isImported; }
-			set { isImported = value; }
-		}
+		//public bool IsImported {
+		//	get { return isImported; }
+		//	set { isImported = value; }
+		//}
 		
 		public BuildWhen Otherwise {
 			get { return otherwise; }
 			set { otherwise = value; }
 		}
 		
-		public IList Whens {
+		public List <BuildWhen> Whens {
 			get { return whens; }
 			set { whens = value; }
 		}
