@@ -35,6 +35,7 @@ using System.Text;
 using System.Xml;
 using Microsoft.Build.Framework;
 using Microsoft.Build.Utilities;
+using Mono.XBuild.Utilities;
 
 namespace Microsoft.Build.BuildEngine {
 	public class BuildItem {
@@ -42,6 +43,7 @@ namespace Microsoft.Build.BuildEngine {
 		XmlElement	itemElement;
 		string		finalItemSpec;
 		bool		isImported;
+		string		itemInclude;
 		string		name;
 		BuildItemGroup	parentItemGroup;
 		string		recursiveDir;
@@ -56,6 +58,7 @@ namespace Microsoft.Build.BuildEngine {
 		{
 			this.name = itemName;
 			this.finalItemSpec = taskItem.ItemSpec;
+			this.itemInclude = Utilities.Escape (taskItem.ItemSpec);
 			this.evaluatedMetadata = (Hashtable) taskItem.CloneCustomMetadata ();
 			this.unevaluatedMetadata = (Hashtable) taskItem.CloneCustomMetadata ();
 		}
@@ -108,8 +111,8 @@ namespace Microsoft.Build.BuildEngine {
 
 		public string GetMetadata (string metadataName)
 		{
-			if (TaskItem.IsReservedMetadataName (metadataName))
-				return TaskItem.GetReservedMetadata (FinalItemSpec, metadataName);
+			if (ReservedNameUtils.IsReservedMetadataName (metadataName))
+				return ReservedNameUtils.GetReservedMetadata (FinalItemSpec, metadataName);
 			else if (evaluatedMetadata.Contains (metadataName) == true)
 				return (string) evaluatedMetadata [metadataName];
 			else
@@ -126,7 +129,7 @@ namespace Microsoft.Build.BuildEngine {
 			if (metadataName == null)
 				throw new ArgumentNullException ("metadataName");
 			
-			if (TaskItem.IsReservedMetadataName (metadataName))
+			if (ReservedNameUtils.IsReservedMetadataName (metadataName))
 				throw new ArgumentException ("Can't remove reserved metadata.");
 			
 			if (evaluatedMetadata.Contains (metadataName))
@@ -154,13 +157,19 @@ namespace Microsoft.Build.BuildEngine {
 			if (metadataValue == null)
 				throw new ArgumentNullException ("metadataValue");
 			
-			if (TaskItem.IsReservedMetadataName (metadataName))
+			if (ReservedNameUtils.IsReservedMetadataName (metadataName))
 				throw new ArgumentException ("Can't modify reserved metadata.");
 			
 			RemoveMetadata (metadataName);
+			
 			unevaluatedMetadata.Add (metadataName, metadataValue);
-			Expression finalValue = new Expression (parentItemGroup.Project, metadataValue);
-			evaluatedMetadata.Add (metadataName, (string) finalValue.ToNonArray (typeof (string)));
+				
+			if (parentItemGroup != null && treatMetadataValueAsLiteral == false) {	
+				OldExpression finalValue = new OldExpression (parentItemGroup.Project);
+				finalValue.ParseSource (metadataValue);
+				evaluatedMetadata.Add (metadataName, (string) finalValue.ConvertTo (typeof (string)));
+			} else
+				evaluatedMetadata.Add (metadataName, metadataValue);
 		}
 		
 		private void BindToXml (XmlElement xmlElement)
@@ -182,14 +191,16 @@ namespace Microsoft.Build.BuildEngine {
 		internal void Evaluate ()
 		{
 			DirectoryScanner directoryScanner;
-			Expression includeExpr, excludeExpr;
+			OldExpression includeExpr, excludeExpr;
 			string includes, excludes;
 
-			includeExpr = new Expression (parentItemGroup.Project, Include);
-			excludeExpr = new Expression (parentItemGroup.Project, Exclude);
+			includeExpr = new OldExpression (parentItemGroup.Project);
+			includeExpr.ParseSource (Include);
+			excludeExpr = new OldExpression (parentItemGroup.Project);
+			excludeExpr.ParseSource (Exclude);
 			
-			includes = (string) includeExpr.ToNonArray (typeof (string));
-			excludes = (string) excludeExpr.ToNonArray (typeof (string));
+			includes = (string) includeExpr.ConvertTo (typeof (string));
+			excludes = (string) excludeExpr.ConvertTo (typeof (string));
 
 			this.finalItemSpec = includes;
 			
@@ -226,19 +237,19 @@ namespace Microsoft.Build.BuildEngine {
 			}
 		}
 		
-		internal string ToString (Expression transform)
+		internal string ConvertToString (OldExpression transform)
 		{
 			return GetItemSpecFromTransform (transform);
 		}
 		
-		internal ITaskItem ToITaskItem (Expression transform)
+		internal ITaskItem ConvertToITaskItem (OldExpression transform)
 		{
 			TaskItem taskItem;
 			taskItem = new TaskItem (GetItemSpecFromTransform (transform), evaluatedMetadata);
 			return taskItem;
 		}
 
-		private string GetItemSpecFromTransform (Expression transform)
+		private string GetItemSpecFromTransform (OldExpression transform)
 		{
 			StringBuilder sb;
 		
@@ -246,13 +257,13 @@ namespace Microsoft.Build.BuildEngine {
 				return finalItemSpec;
 			else {
 				sb = new StringBuilder ();
-				foreach (object o in transform) {
+				foreach (object o in transform.Collection) {
 					if (o is string) {
 						sb.Append ((string)o);
 					} else if (o is PropertyReference) {
-						sb.Append (((PropertyReference)o).ToString ());
+						sb.Append (((PropertyReference)o).ConvertToString ());
 					} else if (o is ItemReference) {
-						sb.Append (((ItemReference)o).ToString ());
+						sb.Append (((ItemReference)o).ConvertToString ());
 					} else if (o is MetadataReference) {
 						sb.Append (GetMetadata (((MetadataReference)o).MetadataName));
 					}
@@ -262,13 +273,29 @@ namespace Microsoft.Build.BuildEngine {
 		}
 
 		public string Condition {
-			get { return itemElement.GetAttribute ("Condition"); }
-			set { itemElement.SetAttribute ("Condition", value); }
+			get {
+				if (FromXml)
+					return itemElement.GetAttribute ("Condition");
+				else
+					return String.Empty;
+			}
+			set {
+				if (FromXml)
+					itemElement.SetAttribute ("Condition", value);
+			}
 		}
 
 		public string Exclude {
-			get { return itemElement.GetAttribute ("Exclude"); }
-			set { itemElement.SetAttribute ("Exclude", value); }
+			get {
+				if (FromXml)
+					return itemElement.GetAttribute ("Exclude");
+				else
+					return String.Empty;
+			}
+			set {
+				if (FromXml)
+					itemElement.SetAttribute ("Exclude", value);
+			}
 		}
 
 		public string FinalItemSpec {
@@ -276,8 +303,18 @@ namespace Microsoft.Build.BuildEngine {
 		}
 
 		public string Include {
-			get { return itemElement.GetAttribute ("Include"); }
-			set { itemElement.SetAttribute ("Include", value); }
+			get {
+				if (FromXml)
+					return itemElement.GetAttribute ("Include");
+				else if (itemInclude != null)
+					return itemInclude;
+				else
+					return finalItemSpec;
+			}
+			set {
+				if (FromXml)
+					itemElement.SetAttribute ("Include", value);
+			}
 		}
 
 		public bool IsImported {
