@@ -30,6 +30,7 @@
 using System;
 using System.Reflection;
 using System.Collections;
+using System.Collections.Generic;
 using System.Xml;
 using Microsoft.Build.Framework;
 using Microsoft.Build.Utilities;
@@ -39,7 +40,7 @@ namespace Microsoft.Build.BuildEngine {
 	
 		XmlAttribute		condition;
 		bool			isImported;
-		IList			buildItems;
+		List <BuildItem>	buildItems;
 		GroupingCollection	parentCollection;
 		Project			parentProject;
 		XmlElement		itemGroupElement;
@@ -52,7 +53,7 @@ namespace Microsoft.Build.BuildEngine {
 		internal BuildItemGroup (XmlElement xmlElement, Project project)
 		{
 			this.itemGroupElement = xmlElement;
-			this.buildItems = new ArrayList ();
+			this.buildItems = new List <BuildItem> ();
 			this.isImported = false;
 			this.parentProject = project;
 			
@@ -73,7 +74,13 @@ namespace Microsoft.Build.BuildEngine {
 		internal void Evaluate ()
 		{
 			foreach (BuildItem bi in buildItems) {
-				bi.Evaluate ();
+				if (bi.Condition == String.Empty)
+					bi.Evaluate ();
+				else {
+					ConditionExpression ce = ConditionParser.ParseCondition (bi.Condition);
+					if (ce.BoolEvaluate (parentProject))
+						bi.Evaluate ();
+				}
 			}
 		}
 
@@ -110,7 +117,7 @@ namespace Microsoft.Build.BuildEngine {
 		public void Clear ()
 		{
 			//FIXME: should this remove all build items?
-			buildItems = new ArrayList ();
+			buildItems = new List <BuildItem> ();
 		}
 
 		[MonoTODO]
@@ -140,27 +147,32 @@ namespace Microsoft.Build.BuildEngine {
 
 		public BuildItem[] ToArray ()
 		{
-			BuildItem[] array;
-			array = new BuildItem [Count];
-			buildItems.CopyTo (array, 0);
-			return array;
+			return buildItems.ToArray ();
 		}
 		
-		internal string ToString (Expression transform, string separator)
+		internal string ConvertToString (OldExpression transform,
+						 OldExpression separator)
 		{
+			string separatorString;
+			
+			if (separator == null)
+				separatorString = ";";
+			else
+				separatorString = (string) separator.ConvertTo (typeof (string));
+		
 			string[] items = new string [buildItems.Count];
 			int i = 0;
 			foreach (BuildItem bi in  buildItems)
-				items [i++] = bi.ToString (transform);
-			return String.Join (separator, items);
+				items [i++] = bi.ConvertToString (transform);
+			return String.Join (separatorString, items);
 		}
 		
-		internal ITaskItem[] ToITaskItemArray (Expression transform)
+		internal ITaskItem[] ConvertToITaskItemArray (OldExpression transform)
 		{
 			ITaskItem[] array = new ITaskItem [buildItems.Count];
 			int i = 0;
 			foreach (BuildItem item in buildItems)
-				array [i++] = item.ToITaskItem (transform);
+				array [i++] = item.ConvertToITaskItem (transform);
 			return array;
 		}
 
@@ -169,7 +181,7 @@ namespace Microsoft.Build.BuildEngine {
 				if (condition != null)
 					return condition.Value;
 				else
-					return null;
+					return String.Empty;
 			}
 			set {
 				if (condition != null)
@@ -194,7 +206,7 @@ namespace Microsoft.Build.BuildEngine {
 
 		public BuildItem this [int index] {
 			get {
-				return (BuildItem) buildItems [index];
+				return buildItems [index];
 			}
 		}
 		
