@@ -42,19 +42,19 @@ namespace Microsoft.Build.BuildEngine {
 		Project		parentProject;
 		PropertyType	propertyType;
 
-		internal bool FromXml {
-			get {
-				return propertyElement != null;
-			}
-		}
-	
 		private BuildProperty ()
 		{
 		}
 
-		public BuildProperty (string propertyName, string propertyValue):
-			this (propertyName, propertyValue, PropertyType.Normal)
+		public BuildProperty (string propertyName, string propertyValue)
+			: this (propertyName, propertyValue, PropertyType.Normal)
 		{
+			if (propertyName == null)
+				throw new ArgumentNullException (null,
+					"Parameter \"propertyName\" cannot be null.");
+			if (propertyValue == null)
+				throw new ArgumentNullException (null,
+					"Parameter \"propertyValue\" cannot be null.");
 		}
 
 		internal BuildProperty (string propertyName,
@@ -83,15 +83,16 @@ namespace Microsoft.Build.BuildEngine {
 		[MonoTODO]
 		public BuildProperty Clone (bool deepClone)
 		{
-			return (BuildProperty) this.MemberwiseClone ();
-		}
-
-		internal void Evaluate ()
-		{
-			if (FromXml) {
-				Expression exp = new Expression (parentProject, Value);
-				finalValue = (string) exp.ToNonArray (typeof (string));
-				parentProject.EvaluatedProperties.AddProperty (this);
+			if (deepClone) {
+				if (FromXml == false) {
+					return (BuildProperty) this.MemberwiseClone ();
+				} else {
+					throw new NotImplementedException ();
+				}
+			} else {
+				if (FromXml == false)
+					throw new InvalidOperationException ("A shallow clone of this object cannot be created.");
+				throw new NotImplementedException ();
 			}
 		}
 
@@ -110,9 +111,33 @@ namespace Microsoft.Build.BuildEngine {
 				return Value;
 		}
 
+		internal void Evaluate ()
+		{
+			if (FromXml) {
+				OldExpression exp = new OldExpression (parentProject);
+				exp.ParseSource (Value);
+				finalValue = (string) exp.ConvertTo (typeof (string));
+				parentProject.EvaluatedProperties.AddProperty (this);
+			}
+		}
+
+		private bool FromXml {
+			get {
+				return propertyElement != null;
+			}
+		}
+	
 		public string Condition {
-			get { return propertyElement.GetAttribute ("Condition"); }
-			set { propertyElement.SetAttribute ("Condition", value); }
+			get {
+				if (FromXml)
+					return propertyElement.GetAttribute ("Condition");
+				else
+					return String.Empty;
+			}
+			set {
+				if (FromXml)
+					propertyElement.SetAttribute ("Condition", value);
+			}
 		}
 
 		public string FinalValue {
