@@ -29,20 +29,26 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
+using System.IO;
 using Microsoft.Build.Framework;
+using Mono.XBuild.Utilities;
 
 namespace Microsoft.Build.BuildEngine {
 	public class Engine {
 		
 		string			binPath;
 		bool			buildEnabled;
+		TaskDatabase		defaultTasks;
+		bool			defaultTasksRegistered;
+		const string		defaultTasksProjectName = "Microsoft.Common.tasks";
 		EventSource		eventSource;
 		bool			buildStarted;
 		BuildPropertyGroup	globalProperties;
-		IDictionary		importedProjects;
-		IList			loggers;
+		//IDictionary		importedProjects;
+		List <ILogger>		loggers;
 		bool			onlyLogCriticalEvents;
-		IDictionary		projects;
+		Dictionary <string, Project>	projects;
 
 		static Engine		globalEngine;
 		static Version		version;
@@ -63,11 +69,13 @@ namespace Microsoft.Build.BuildEngine {
 		{
 			this.binPath = binPath;
 			this.buildEnabled = true;
-			this.projects = new Hashtable ();
+			this.projects = new Dictionary <string, Project> ();
 			this.eventSource = new EventSource ();
-			this.loggers = new ArrayList ();
+			this.loggers = new List <ILogger> ();
 			this.buildStarted = false;
 			this.globalProperties = new BuildPropertyGroup ();
+			
+			RegisterDefaultTasks ();
 		}
 		
 		[MonoTODO]
@@ -160,7 +168,7 @@ namespace Microsoft.Build.BuildEngine {
 			bool result;
 			Project project;
 			
-			if (projects.Contains (projectFile)) {
+			if (projects.ContainsKey (projectFile)) {
 				project = (Project) projects [projectFile];
 				LogProjectStarted (project, targetNames);
 				result = project.Build (targetNames, targetOutputs);
@@ -184,7 +192,8 @@ namespace Microsoft.Build.BuildEngine {
 
 		public Project CreateNewProject ()
 		{
-			CheckBinPath ();
+			if (defaultTasksRegistered == true)
+				CheckBinPath ();
 			// FIXME: I don't really know if it should be here
 			LogBuildStarted ();
 			return new Project (this);
@@ -225,7 +234,7 @@ namespace Microsoft.Build.BuildEngine {
 
 		public void UnloadAllProjects ()
 		{
-			foreach (DictionaryEntry e in projects)
+			foreach (KeyValuePair <string, Project> e in projects)
 				UnloadProject ((Project) e.Value);
 		}
 
@@ -285,6 +294,26 @@ namespace Microsoft.Build.BuildEngine {
 			bfea = new BuildFinishedEventArgs ("Build finished.", null, succeeded);
 			eventSource.FireBuildFinished (this, bfea);
 		}
+		
+		private void RegisterDefaultTasks ()
+		{
+			this.defaultTasksRegistered = false;
+			
+			Project defaultTasksProject = CreateNewProject ();
+			
+			if (binPath != null) {
+				if (File.Exists (Path.Combine (binPath, defaultTasksProjectName)) == true) {
+					defaultTasksProject.Load (Path.Combine (binPath, defaultTasksProjectName));
+					defaultTasks = defaultTasksProject.TaskDatabase;
+				} else {
+					defaultTasks = new TaskDatabase ();
+				}
+			} else {
+				defaultTasks = new TaskDatabase ();
+			}
+			
+			this.defaultTasksRegistered = true;
+		}
 
 		public string BinPath {
 			get { return binPath; }
@@ -321,6 +350,14 @@ namespace Microsoft.Build.BuildEngine {
 		internal EventSource EventSource {
 			get { return eventSource; }
 		}
+		
+		internal bool DefaultTasksRegistered {
+			get { return defaultTasksRegistered; }
+		}
+		
+		internal TaskDatabase DefaultTasks {
+			get { return defaultTasks; }
+		}
 	}
 }
 
