@@ -29,6 +29,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.Collections.Specialized;
 using System.IO;
 using System.Text;
@@ -39,16 +40,16 @@ using Mono.XBuild.Framework;
 
 namespace Microsoft.Build.BuildEngine {
 	public class Project {
-		static string separator = ";";
 	
 		bool				buildEnabled;
-		IDictionary			conditionedProperties;
+		//IDictionary			conditionedProperties;
+		Dictionary <string, List <string>>	conditionedProperties;
 		string[]			defaultTargets;
 		Encoding			encoding;
 		BuildItemGroup			evaluatedItems;
 		BuildItemGroup			evaluatedItemsIgnoringCondition;
-		IDictionary			evaluatedItemsByName;
-		IDictionary			evaluatedItemsByNameIgnoringCondition;
+		Dictionary <string, BuildItemGroup>	evaluatedItemsByName;
+		Dictionary <string, BuildItemGroup>	evaluatedItemsByNameIgnoringCondition;
 		BuildPropertyGroup		evaluatedProperties;
 		string				firstTargetName;
 		string				fullFileName;
@@ -84,7 +85,11 @@ namespace Microsoft.Build.BuildEngine {
 			itemGroups = new BuildItemGroupCollection (groups);
 			propertyGroups = new BuildPropertyGroupCollection (groups);
 			targets = new TargetCollection (this);
+			
 			taskDatabase = new TaskDatabase ();
+			if (engine.DefaultTasksRegistered == true)
+				taskDatabase.CopyTasks (engine.DefaultTasks);
+			
 			globalProperties = new BuildPropertyGroup ();
 			fullFileName = String.Empty;
 
@@ -156,8 +161,7 @@ namespace Microsoft.Build.BuildEngine {
 			if (targets.Exists (targetName) == false)
 				throw new Exception ("Target specified to build does not exist.");
 			
-			this.targets [targetName].Build ();
-			return true;
+			return this.targets [targetName].Build ();
 		}
 		
 		[MonoTODO]
@@ -198,24 +202,29 @@ namespace Microsoft.Build.BuildEngine {
 			return true;
 		}
 
+		[MonoTODO]
 		public string[] GetConditionedPropertyValues (string propertyName)
 		{
-			StringCollection sc = (StringCollection) conditionedProperties [propertyName];
-			string[] propertyValues = new string [sc.Count];
-			int i  = 0;
-			foreach (string propertyValue in sc)
-				propertyValues [i++] = propertyValue;
-			return propertyValues;
+			if (conditionedProperties.ContainsKey (propertyName))
+				return conditionedProperties [propertyName].ToArray ();
+			else
+				return new string [0];
 		}
 
 		public BuildItemGroup GetEvaluatedItemsByName (string itemName)
 		{
-			return (BuildItemGroup) evaluatedItemsByName [itemName];
+			if (evaluatedItemsByName.ContainsKey (itemName))
+				return evaluatedItemsByName [itemName];
+			else
+				return null;
 		}
 
 		public BuildItemGroup GetEvaluatedItemsByNameIgnoringCondition (string itemName)
 		{
-			return (BuildItemGroup) evaluatedItemsByNameIgnoringCondition [itemName];
+			if (evaluatedItemsByNameIgnoringCondition.ContainsKey (itemName))
+				return evaluatedItemsByNameIgnoringCondition [itemName];
+			else
+				return null;
 		}
 
 		public string GetEvaluatedProperty (string propertyName)
@@ -325,18 +334,35 @@ namespace Microsoft.Build.BuildEngine {
 		{
 			evaluatedItems = new BuildItemGroup (null, this);
 			evaluatedItemsIgnoringCondition = new BuildItemGroup (null, this);
-			evaluatedItemsByName = CollectionsUtil.CreateCaseInsensitiveHashtable ();
-			evaluatedItemsByNameIgnoringCondition = CollectionsUtil.CreateCaseInsensitiveHashtable ();
+			evaluatedItemsByName = new Dictionary <string, BuildItemGroup> (StringComparer.InvariantCultureIgnoreCase);
+			evaluatedItemsByNameIgnoringCondition = new Dictionary <string, BuildItemGroup> (StringComparer.InvariantCultureIgnoreCase);
 			evaluatedProperties = new BuildPropertyGroup ();
 
 			InitializeProperties ();
 
-			foreach (BuildPropertyGroup bpg in PropertyGroups)
-				bpg.Evaluate ();
+			foreach (BuildPropertyGroup bpg in PropertyGroups) {
+				if (bpg.Condition == String.Empty)
+					bpg.Evaluate ();
+				else {
+					ConditionExpression ce = ConditionParser.ParseCondition (bpg.Condition);
+					if (ce.BoolEvaluate (this))
+						bpg.Evaluate ();
+				}
+			}
+			
 			foreach (Import import in Imports)
 				import.Evaluate ();
-			foreach (BuildItemGroup big in ItemGroups)
-				big.Evaluate ();
+			
+			foreach (BuildItemGroup big in ItemGroups) {
+				if (big.Condition == String.Empty)
+					big.Evaluate ();
+				else {
+					ConditionExpression ce = ConditionParser.ParseCondition (big.Condition);
+					if (ce.BoolEvaluate (this))
+						big.Evaluate ();
+				}
+			}
+			
 			foreach (UsingTask usingTask in UsingTasks)
 				usingTask.Evaluate ();
 		}
@@ -579,7 +605,7 @@ namespace Microsoft.Build.BuildEngine {
 				throw new ArgumentNullException ("xmlElement");
 			BuildItemGroup big = new BuildItemGroup (xmlElement, this);
 			ItemGroups.Add (big);
-			big.Evaluate ();
+			//big.Evaluate ();
 		}
 		
 		private void AddPropertyGroup (XmlElement xmlElement)
@@ -588,13 +614,17 @@ namespace Microsoft.Build.BuildEngine {
 				throw new ArgumentNullException ("xmlElement");
 			BuildPropertyGroup bpg = new BuildPropertyGroup (xmlElement, this);
 			PropertyGroups.Add (bpg);
-			bpg.Evaluate ();
+			//bpg.Evaluate ();
 		}
 		
 		private void AddChoose (XmlElement xmlElement)
 		{
 			if (xmlElement == null)
 				throw new ArgumentNullException ("xmlElement");
+				
+			BuildChoose bc = new BuildChoose (xmlElement, this);
+			groups.Add (bc);
+			
 		}
 		
 		private static void ValidationCallBack (object sender, ValidationEventArgs e)
