@@ -29,6 +29,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.Xml;
 using Microsoft.Build.Framework;
 
@@ -44,9 +45,8 @@ namespace Microsoft.Build.BuildEngine {
 		string		name;
 		Project		project;
 		XmlElement	targetElement;
-		ArrayList	onErrorElements;
-		
-		ArrayList	buildTasks;
+		List <XmlElement>	onErrorElements;
+		List <BuildTask>	buildTasks;
 		
 		internal Target (XmlElement targetElement, Project project)
 		{
@@ -64,9 +64,9 @@ namespace Microsoft.Build.BuildEngine {
 			this.engine = project.ParentEngine;
 			this.isImported = false;;
 
-			this.onErrorElements  = new ArrayList ();
+			this.onErrorElements  = new List <XmlElement> ();
 			this.buildState = BuildState.NotStarted;
-			this.buildTasks = new ArrayList ();
+			this.buildTasks = new List <BuildTask> ();
 			this.batchingImpl = new BatchingImpl (project, this.targetElement);
 
 			foreach (XmlNode xn in targetElement.ChildNodes) {
@@ -81,16 +81,20 @@ namespace Microsoft.Build.BuildEngine {
 			}
 		}
 		
-		internal void Build ()
+		internal bool Build ()
 		{
+			bool result;
+		
 			buildState = BuildState.Started;
 			if (dependsOnTargets == null) {
 				;
 			} else if (dependsOnTargets.Value == "") {
 				;
 			} else {
-				Expression dependencies = new Expression (Project, dependsOnTargets.Value);
-				string[] targetsToBuildFirst = (string[]) dependencies.ToArray (typeof (string[]));
+				OldExpression dependencies = new OldExpression (Project);
+				dependencies.ParseSource (dependsOnTargets.Value);
+				
+				string[] targetsToBuildFirst = (string[]) dependencies.ConvertTo (typeof (string[]));
 				foreach (string target in targetsToBuildFirst) {
 					string trimmed = target.Trim ();
 					Target t = (Target) project.Targets [trimmed];
@@ -103,11 +107,14 @@ namespace Microsoft.Build.BuildEngine {
 						throw new InvalidProjectFileException ("Cycle in target dependencies detected.");
 				}
 			}
-			RealBuild ();
+			
+			result = RealBuild ();
 			buildState = BuildState.Finished;
+			
+			return result;
 		}
 		
-		private void RealBuild ()
+		private bool RealBuild ()
 		{
 			bool executeOnErrors = false;
 			bool result = true;
@@ -130,6 +137,8 @@ namespace Microsoft.Build.BuildEngine {
 			
 			if (executeOnErrors == true)
 				ExecuteOnErrors ();
+				
+			return result;
 		}
 		
 		private void ExecuteOnErrors ()
