@@ -28,7 +28,7 @@
 #if NET_2_0
 
 using System;
-using System.Collections;
+using System.Collections.Generic;
 using System.Collections.Specialized;
 using System.Reflection;
 using System.Xml;
@@ -40,6 +40,7 @@ namespace Microsoft.Build.BuildEngine {
 		
 		ITask		task;
 		XmlElement	taskElement;
+		Type		taskType;
 		Project		parentProject;
 		
 		static Type	requiredAttribute;
@@ -57,22 +58,23 @@ namespace Microsoft.Build.BuildEngine {
 		}
 		
 		public void Prepare (ITask task, XmlElement taskElement,
-				     IDictionary parameters, Type taskType)
+				     IDictionary <string, string> parameters, Type taskType)
 		{
-			Hashtable	values;
+			Dictionary <string, object>	values;
 			PropertyInfo	currentProperty;
 			PropertyInfo[]	properties;
-		
+			
 			this.task = task;
 			this.taskElement = taskElement;
-			values = new Hashtable ();
+			this.taskType = taskType;
+			values = new Dictionary <string, object> ();
 			
-			foreach (DictionaryEntry de in parameters) {
-				currentProperty = taskType.GetProperty ((string) de.Key);
+			foreach (KeyValuePair <string, string> de in parameters) {
+				currentProperty = taskType.GetProperty (de.Key);
 				if (currentProperty == null)
 					throw new InvalidProjectFileException (String.Format ("Task does not have property \"{0}\" defined",
 						de.Key));
-				values.Add ((string) de.Key, GetObjectFromString ((string) de.Value, currentProperty.PropertyType)); 
+				values.Add (de.Key, GetObjectFromString (de.Value, currentProperty.PropertyType));
 			}
 			
 			properties = taskType.GetProperties ();
@@ -114,7 +116,7 @@ namespace Microsoft.Build.BuildEngine {
 					itemName = xmlElement.GetAttribute ("ItemName");
 					propertyName = xmlElement.GetAttribute ("PropertyName");
 					
-					propertyInfo = GetType ().GetProperty (taskParameter);
+					propertyInfo = taskType.GetProperty (taskParameter);
 					if (propertyInfo == null)
 						throw new Exception ("Could not get property info.");
 					if (propertyInfo.IsDefined (outputAttribute, false) == false)
@@ -221,21 +223,18 @@ namespace Microsoft.Build.BuildEngine {
 				
 		private object GetObjectFromString (string raw, Type type)
 		{
-			Expression e;
+			OldExpression e;
 			object result;
 			
-			e = new Expression (parentProject, raw);
+			e = new OldExpression (parentProject);
+			e.ParseSource (raw);
 			
 			if (type == typeof (ITaskItem)) {
-				result = (object) e.ToITaskItem ();
+				result = e.ConvertTo (typeof (ITaskItem));
 			} else if (type == typeof (ITaskItem[])) {
-				result = (object) e.ToITaskItemArray ();
+				result = e.ConvertTo (typeof (ITaskItem[]));
 			} else {
-				if (type.IsArray) {
-					result = e.ToArray (type);
-				} else {
-					result = e.ToNonArray (type);
-				}
+				result = e.ConvertTo (type);
 			}
 			
 			return result;
