@@ -27,6 +27,7 @@
 
 #if NET_2_0
 
+using System;
 using Microsoft.Build.Framework;
 
 namespace Microsoft.Build.Tasks {
@@ -41,7 +42,7 @@ namespace Microsoft.Build.Tasks {
 
 		public override bool Execute ()
 		{
-			return true;
+			throw new NotImplementedException ();
 		}
 
 		[Output]
@@ -58,4 +59,4 @@ namespace Microsoft.Build.Tasks {
 	}
 }
 
-#endif
\ No newline at end of file
+#endif
