@@ -52,10 +52,7 @@ namespace Microsoft.Build.Tasks {
 		
 		public Exec ()
 		{
-			process = new Process ();
-			//timeout = Int32.MaxValue;
 			ignoreExitCode = false;
-			executionTime = 0;
 		}
 
 		/*public override bool Execute ()
@@ -104,13 +101,13 @@ namespace Microsoft.Build.Tasks {
 						    string responseFileCommands,
 						    string commandLineCommands)
 		{
-			return 0;
+			return base.ExecuteTool (GenerateFullPathToTool (), String.Empty, String.Empty);
 		}
 		
 		[MonoTODO]
 		protected override string GenerateFullPathToTool ()
 		{
-			return null;
+			return command;
 		}
 		
 		[MonoTODO]
