@@ -126,6 +126,13 @@ namespace Microsoft.JScript {
 				false_stm.Emit (ec);			
 			ig.MarkLabel (merge_lbl);
 		}
+
+		internal override void PropagateParent (AST parent)
+		{
+			base.PropagateParent (parent);
+			true_stm.PropagateParent (this);
+			false_stm.PropagateParent (this);
+		}
  	}
 
 	abstract class Jump : AST {
@@ -149,6 +156,11 @@ namespace Microsoft.JScript {
 				throw new Exception ("error JS1026: Label not found");
                         return true;
 		}
+
+		internal override void PropagateParent (AST parent)
+		{
+			base.PropagateParent (parent);
+		}
 	}
 
 	internal class Continue : Jump {
@@ -202,6 +214,11 @@ namespace Microsoft.JScript {
 			}
 			ec.ig.Emit (OpCodes.Br, (binding as Labelled).EndAddrs);
 		}
+
+		internal override void PropagateParent (AST parent)
+		{
+			base.PropagateParent (parent);
+		}
 	}
 
 	internal class NotVoidReturnEventArgs : EventArgs {
@@ -209,7 +226,7 @@ namespace Microsoft.JScript {
 		
 	internal delegate void NotVoidReturnEventHandler (object sender, NotVoidReturnEventArgs args);
 	
-	internal class Return : AST {
+	internal class Return : AST, ICanModifyContext {
 
 		internal AST expression;
 		private bool exp_returns_void = false;
@@ -242,6 +259,18 @@ namespace Microsoft.JScript {
 				return String.Empty;
 		}
 
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			if (expression is ICanModifyContext)
+				((ICanModifyContext) expression).PopulateContext (env, ns);
+		}
+
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			if (expression is ICanModifyContext)
+				((ICanModifyContext) expression).EmitDecls (ec);
+		}
+
 		internal override bool Resolve (Environment env)
 		{
 			if (!InFunction)
