@@ -2,6 +2,46 @@
 // TokenStream.cs: Port of Mozilla's Rhino TokenStream
 //		   This class implements the JScript scanner
 //
+
+/*
+ * The contents of this file are subject to the Netscape Public
+ * License Version 1.1 (the "License"); you may not use this file
+ * except in compliance with the License. You may obtain a copy of
+ * the License at http://www.mozilla.org/NPL/
+ *
+ * Software distributed under the License is distributed on an "AS
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+ * implied. See the License for the specific language governing
+ * rights and limitations under the License.
+ *
+ * The Original Code is Rhino code, released
+ * May 6, 1999.
+ *
+ * The Initial Developer of the Original Code is Netscape
+ * Communications Corporation.  Portions created by Netscape are
+ * Copyright (C) 1997-1999 Netscape Communications Corporation. All
+ * Rights Reserved.
+ *
+ * Contributor(s):
+ * Roger Lawrence
+ * Mike McCabe
+ * Igor Bukanov
+ * Ethan Hugg
+ * Terry Lucas
+ * Milen Nankov
+ *
+ * Alternatively, the contents of this file may be used under the
+ * terms of the GNU Public License (the "GPL"), in which case the
+ * provisions of the GPL are applicable instead of those above.
+ * If you wish to allow use of your version of this file only
+ * under the terms of the GPL and not to allow others to use your
+ * version of this file under the NPL, indicate your decision by
+ * deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL.  If you do not delete
+ * the provisions above, a recipient may use your version of this
+ * file under either the NPL or the GPL.
+ */
+
 // Author:
 //	Cesar Lopez Nataren (cesar@ciencias.unam.mx)
 //
@@ -1047,9 +1087,9 @@ internal class TokenStream {
 							ReportCurrentLineError ("msg.invalid.re.flag");
 							return Token.ERROR;
 						}
-						
-						_string = to_string (string_buffer).Substring (0, re_end);
-						reg_exp_flags = to_string (string_buffer).Substring (re_end, string_buffer_top - re_end);
+
+						_string = new String (string_buffer, 0, re_end);
+						reg_exp_flags = new String (string_buffer, re_end, string_buffer_top - re_end);
 						return Token.REGEXP;
 					}
 					
@@ -1465,13 +1505,5 @@ internal class TokenStream {
 			unicode_category == UnicodeCategory.CurrencySymbol || 
 			unicode_category == UnicodeCategory.ConnectorPunctuation;
 	}		
-
-	internal static string to_string (Array a)
-	{
-		string s = String.Empty;
-		foreach (char c in a)
-			s += c;
-		return s;
-	}
 }
 }
