@@ -108,7 +108,12 @@ namespace Cairo
 		// FontFace
 		[DllImport (cairo)]
 		internal static extern void cairo_font_face_destroy (IntPtr font_face);
-		
+
+#if CAIRO_1_2		
+		[DllImport (cairo)]
+		internal static extern FontType cairo_font_face_get_type (IntPtr font_face);
+#endif
+
 		//[DllImport (cairo)]
 		//internal static extern void cairo_font_face_get_user_data (IntPtr font_face);
 		
@@ -201,6 +206,9 @@ namespace Cairo
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_get_font_options (IntPtr cr, IntPtr options);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_get_group_target (IntPtr cr);
 		
 		[DllImport (cairo)]
 		internal static extern LineCap cairo_get_line_cap (IntPtr cr);
@@ -246,16 +254,25 @@ namespace Cairo
 		internal static extern IntPtr cairo_image_surface_create (Cairo.Format format, int width, int height);
 		
 		[DllImport (cairo)]
-		internal static extern IntPtr cairo_image_surface_create_for_data (string data, Cairo.Format format, int width, int height, int stride);
+		internal static extern IntPtr cairo_image_surface_create_for_data (byte[] data, Cairo.Format format, int width, int height, int stride);
 		
 		[DllImport (cairo)]
 		internal static extern IntPtr cairo_image_surface_create_from_png  (string filename);
 		
 		//[DllImport (cairo)]
 		//internal static extern IntPtr cairo_image_surface_create_from_png_stream  (string filename);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_image_surface_get_data (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern Format cairo_image_surface_get_format (IntPtr surface);
 		
 		[DllImport (cairo)]
 		internal static extern int cairo_image_surface_get_height (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern int cairo_image_surface_get_stride (IntPtr surface);
 		
 		[DllImport (cairo)]
 		internal static extern int cairo_image_surface_get_width  (IntPtr surface);
@@ -319,6 +336,9 @@ namespace Cairo
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_new_path (IntPtr cr);
+
+		[DllImport (cairo)]
+		internal static extern void cairo_new_sub_path (IntPtr cr);
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_paint (IntPtr cr);
@@ -362,6 +382,11 @@ namespace Cairo
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_pattern_get_matrix (IntPtr pattern, Matrix matrix);
+
+#if CAIRO_1_2
+		[DllImport (cairo)]
+		internal static extern PatternType cairo_pattern_get_type (IntPtr pattern);
+#endif
 		
 		[DllImport (cairo)]
 		internal static extern IntPtr cairo_pattern_reference (IntPtr pattern);
@@ -386,7 +411,7 @@ namespace Cairo
 		//internal static extern IntPtr cairo_pdf_surface_create_for_stream (string filename, double width, double height);
 		
 		[DllImport (cairo)]
-		internal static extern void cairo_pdf_surface_set_dpi (IntPtr surface, double x_dpi, double y_dpi);
+		internal static extern void cairo_pdf_surface_set_size (IntPtr surface, double x, double y);
 		
 		// PostscriptSurface
 		[DllImport (cairo)]
@@ -394,9 +419,29 @@ namespace Cairo
 		
 		//[DllImport (cairo)]
 		//internal static extern IntPtr cairo_ps_surface_create_for_stream (string filename, double width, double height);
+		[DllImport (cairo)]
+		internal static extern void cairo_ps_surface_begin_page_setup (IntPtr surface);
 		
 		[DllImport (cairo)]
-		internal static extern void cairo_ps_surface_set_dpi (IntPtr surface, double x_dpi, double y_dpi);
+		internal static extern void cairo_ps_surface_begin_setup (IntPtr surface);
+		
+		[DllImport (cairo)]
+		internal static extern void cairo_ps_surface_dsc_comment (IntPtr surface, string comment);
+		
+		[DllImport (cairo)]
+		internal static extern void cairo_ps_surface_set_size (IntPtr surface, double x, double y);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_pop_group (IntPtr cr);
+
+		[DllImport (cairo)]
+		internal static extern void cairo_pop_group_to_source (IntPtr cr);
+
+		[DllImport (cairo)]
+		internal static extern void cairo_push_group (IntPtr cr);
+
+		[DllImport (cairo)]
+		internal static extern void cairo_push_group_with_content (IntPtr cr, Content content);
 
 		[DllImport (cairo)]
 		internal static extern IntPtr cairo_quartz_surface_create (IntPtr context, bool flipped, int width, int height);
@@ -440,7 +485,24 @@ namespace Cairo
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_scaled_font_extents (IntPtr scaled_font, TextExtents extents);
-		
+
+#if CAIRO_1_2		
+		[DllImport (cairo)]
+		internal static extern void cairo_scaled_font_get_ctm (IntPtr scaled_font, out Matrix matrix);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_scaled_font_get_font_face (IntPtr scaled_font);
+
+		[DllImport (cairo)]
+		internal static extern void cairo_scaled_font_get_font_matrix (IntPtr scaled_font, out Matrix matrix);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_scaled_font_get_font_options (IntPtr scaled_font);
+
+		[DllImport (cairo)]
+		internal static extern FontType cairo_scaled_font_get_type (IntPtr scaled_font);
+#endif
+
 		[DllImport (cairo)]
 		internal static extern void cairo_scaled_font_glyph_extents (IntPtr scaled_font, IntPtr glyphs, int num_glyphs, TextExtents extents);
 		
@@ -449,6 +511,11 @@ namespace Cairo
 
 		[DllImport (cairo)]
 		internal static extern Status cairo_scaled_font_status (IntPtr scaled_font);
+
+#if CAIRO_1_2		
+		[DllImport (cairo)]
+		internal static extern void cairo_scaled_font_text_extents (IntPtr scaled_font, string utf8, ref TextExtents extents);
+#endif
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_select_font_face (IntPtr cr, string family, FontSlant slant, FontWeight weight);
@@ -543,9 +610,20 @@ namespace Cairo
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_surface_flush (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern Content cairo_surface_get_content (IntPtr surface);
 		
+		[DllImport (cairo)]
+		internal static extern void cairo_surface_get_device_offset (IntPtr surface, out double x, out double y);
+
 		[DllImport (cairo)]
 		internal static extern void cairo_surface_get_font_options (IntPtr surface, IntPtr FontOptions);
+
+#if CAIRO_1_2
+		[DllImport (cairo)]
+		internal static extern SurfaceType cairo_surface_get_type (IntPtr surface);
+#endif
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_surface_mark_dirty (IntPtr surface);
@@ -558,6 +636,9 @@ namespace Cairo
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_surface_set_device_offset (IntPtr surface, double x, double y);
+
+		[DllImport (cairo)]
+		internal static extern void cairo_surface_set_fallback_resolution (IntPtr surface, double x, double y);
 		
 		[DllImport (cairo)]
 		internal static extern Status cairo_surface_status (IntPtr surface);
@@ -574,8 +655,10 @@ namespace Cairo
 		//[DllImport (cairo)]
 		//internal static extern IntPtr cairo_svg_surface_create_for_stream (double width, double height);
 		
+#if CAIRO_1_2
 		[DllImport (cairo)]
-		internal static extern IntPtr cairo_svg_surface_set_dpi (IntPtr handle, double width, double height);
+		internal static extern IntPtr cairo_svg_surface_restrict_to_version (IntPtr surface, SvgVersion version);
+#endif
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_text_extents (IntPtr cr, string utf8, ref TextExtents extents);
@@ -601,6 +684,10 @@ namespace Cairo
 		[DllImport (cairo)]
 		internal static extern IntPtr cairo_version_string ();
 		
+		// DirectFBSurface
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_directfb_surface_create (IntPtr dfb, IntPtr surface);
+		
 		// win32 fonts
 		[DllImport (cairo)]
 		internal static extern IntPtr cairo_win32_font_face_create_for_logfontw (IntPtr logfontw);
@@ -620,10 +707,10 @@ namespace Cairo
 		
 		// XcbSurface
 		[DllImport (cairo)]
-		internal static extern IntPtr cairo_xcb_surface_create (IntPtr connection, IntPtr drawable, IntPtr visual, int width, int height);
+		internal static extern IntPtr cairo_xcb_surface_create (IntPtr connection, uint drawable, IntPtr visual, int width, int height);
 		
 		[DllImport (cairo)]
-		internal static extern IntPtr cairo_xcb_surface_create_for_bitmap (IntPtr connection, IntPtr bitmap, int width, int height);
+		internal static extern IntPtr cairo_xcb_surface_create_for_bitmap (IntPtr connection, uint bitmap, IntPtr screen, int width, int height);
 		
 		[DllImport (cairo)]
 		internal static extern void cairo_xcb_surface_set_size (IntPtr surface, int width, int height);
@@ -635,6 +722,27 @@ namespace Cairo
 		[DllImport (cairo)]
 		internal static extern IntPtr cairo_xlib_surface_create_for_bitmap (IntPtr display, IntPtr bitmap, IntPtr screen, int width, int height);
 		
+		[DllImport (cairo)]
+		internal static extern int cairo_xlib_surface_get_depth (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_xlib_surface_get_display (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_xlib_surface_get_drawable (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern int cairo_xlib_surface_get_height (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_xlib_surface_get_screen (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern IntPtr cairo_xlib_surface_get_visual (IntPtr surface);
+
+		[DllImport (cairo)]
+		internal static extern int cairo_xlib_surface_get_width (IntPtr surface);
+		
 		[DllImport (cairo)]
 		internal static extern void cairo_xlib_surface_set_drawable (IntPtr surface, IntPtr drawable, int width, int height);
 		
