@@ -87,12 +87,14 @@ namespace Mono.Security.Protocol.Tls
 		#region Receive Record Async Result
 		private class ReceiveRecordAsyncResult : IAsyncResult
 		{
+			private object locker = new object ();
 			private AsyncCallback _userCallback;
 			private object _userState;
 			private Exception _asyncException;
-			private ManualResetEvent _complete;
+			private ManualResetEvent handle;
 			private byte[] _resultingBuffer;
 			private Stream _record;
+			private bool completed;
 
 			private byte[] _initialBuffer;
 
@@ -100,7 +102,6 @@ namespace Mono.Security.Protocol.Tls
 			{
 				_userCallback = userCallback;
 				_userState = userState;
-				_complete = new ManualResetEvent(false);
 				_initialBuffer = initialBuffer;
 				_record = record;
 			}
@@ -132,12 +133,24 @@ namespace Mono.Security.Protocol.Tls
 
 			public bool CompletedWithError
 			{
-				get { return null != _asyncException; }
+				get {
+					if (!IsCompleted)
+						return false; // Perhaps throw InvalidOperationExcetion?
+
+					return null != _asyncException;
+				}
 			}
 
 			public WaitHandle AsyncWaitHandle
 			{
-				get { return _complete; }
+				get {
+					lock (locker) {
+						if (handle == null)
+							handle = new ManualResetEvent (completed);
+					}
+					return handle;
+				}
+				
 			}
 
 			public bool CompletedSynchronously
@@ -147,27 +160,28 @@ namespace Mono.Security.Protocol.Tls
 
 			public bool IsCompleted
 			{
-				get { return _complete.WaitOne(0, false); }
+				get {
+					lock (locker) {
+						return completed;
+					}
+				}
 			}
 
 			private void SetComplete(Exception ex, byte[] resultingBuffer)
 			{
-				if (this.IsCompleted)
-					return;
-
-				lock (this)
-				{
-					if (this.IsCompleted)
+				lock (locker) {
+					if (completed)
 						return;
 
+					completed = true;
 					_asyncException = ex;
 					_resultingBuffer = resultingBuffer;
-					_complete.Set();
+					if (handle != null)
+						handle.Set ();
+
+					if (_userCallback != null)
+						_userCallback.BeginInvoke (this, null, null);
 				}
-
-				if (null != _userCallback)
-					_userCallback (this);
-
 			}
 
 			public void SetComplete(Exception ex)
@@ -190,17 +204,18 @@ namespace Mono.Security.Protocol.Tls
 		#region Receive Record Async Result
 		private class SendRecordAsyncResult : IAsyncResult
 		{
+			private object locker = new object ();
 			private AsyncCallback _userCallback;
 			private object _userState;
 			private Exception _asyncException;
-			private ManualResetEvent _complete;
+			private ManualResetEvent handle;
 			private HandshakeMessage _message;
+			private bool completed;
 
 			public SendRecordAsyncResult(AsyncCallback userCallback, object userState, HandshakeMessage message)
 			{
 				_userCallback = userCallback;
 				_userState = userState;
-				_complete = new ManualResetEvent(false);
 				_message = message;
 			}
 
@@ -221,12 +236,24 @@ namespace Mono.Security.Protocol.Tls
 
 			public bool CompletedWithError
 			{
-				get { return null != _asyncException; }
+				get {
+					if (!IsCompleted)
+						return false; // Perhaps throw InvalidOperationExcetion?
+
+					return null != _asyncException;
+				}
 			}
 
 			public WaitHandle AsyncWaitHandle
 			{
-				get { return _complete; }
+				get {
+					lock (locker) {
+						if (handle == null)
+							handle = new ManualResetEvent (completed);
+					}
+					return handle;
+				}
+				
 			}
 
 			public bool CompletedSynchronously
@@ -236,26 +263,28 @@ namespace Mono.Security.Protocol.Tls
 
 			public bool IsCompleted
 			{
-				get { return _complete.WaitOne(0, false); }
+				get {
+					lock (locker) {
+						return completed;
+					}
+				}
 			}
 
 			public void SetComplete(Exception ex)
 			{
-				if (this.IsCompleted)
-					return;
-
-				lock (this)
-				{
-					if (this.IsCompleted)
+				lock (locker) {
+					if (completed)
 						return;
 
+					completed = true;
+					if (handle != null)
+						handle.Set ();
+
+					if (_userCallback != null)
+						_userCallback.BeginInvoke (this, null, null);
+
 					_asyncException = ex;
-					_complete.Set();
 				}
-
-				if (null != _userCallback)
-					_userCallback (this);
-
 			}
 
 			public void SetComplete()
@@ -358,19 +387,14 @@ namespace Mono.Security.Protocol.Tls
 						}
 						break;
 
-	// FIXME / MCS bug - http://bugzilla.ximian.com/show_bug.cgi?id=67711
-	//				case (ContentType)0x80:
-	//					this.context.HandshakeMessages.Write (result);
-	//					break;
+					case (ContentType)0x80:
+						this.context.HandshakeMessages.Write (buffer);
+						break;
 
 					default:
-						if (contentType != (ContentType)0x80)
-						{
-							throw new TlsException(
-								AlertDescription.UnexpectedMessage,
-								"Unknown record received from server.");
-						}
-						this.context.HandshakeMessages.Write (buffer);
+						throw new TlsException(
+							AlertDescription.UnexpectedMessage,
+							"Unknown record received from server.");
 						break;
 				}
 
@@ -390,7 +414,8 @@ namespace Mono.Security.Protocol.Tls
 			if (null == internalResult)
 				throw new ArgumentException("Either the provided async result is null or was not created by this RecordProtocol.");
 
-			internalResult.AsyncWaitHandle.WaitOne();
+			if (!internalResult.IsCompleted)
+				internalResult.AsyncWaitHandle.WaitOne();
 
 			if (internalResult.CompletedWithError)
 				throw internalResult.AsyncException;
@@ -606,9 +631,6 @@ namespace Mono.Security.Protocol.Tls
 
 			// Make the pending state to be the current state
 			this.context.IsActual = true;
-
-			// Send Finished message
-			this.SendRecord(HandshakeType.Finished);			
 		}
 
 		public IAsyncResult BeginSendRecord(HandshakeType handshakeType, AsyncCallback callback, object state)
@@ -667,7 +689,8 @@ namespace Mono.Security.Protocol.Tls
 			if (asyncResult is SendRecordAsyncResult)
 			{
 				SendRecordAsyncResult internalResult = asyncResult as SendRecordAsyncResult;
-				internalResult.AsyncWaitHandle.WaitOne();
+				if (!internalResult.IsCompleted)
+					internalResult.AsyncWaitHandle.WaitOne();
 				if (internalResult.CompletedWithError)
 					throw internalResult.AsyncException;
 			}
@@ -715,13 +738,13 @@ namespace Mono.Security.Protocol.Tls
 				short	fragmentLength = 0;
 				byte[]	fragment;
 
-				if ((count - position) > Context.MAX_FRAGMENT_SIZE)
+				if ((count + offset - position) > Context.MAX_FRAGMENT_SIZE)
 				{
 					fragmentLength = Context.MAX_FRAGMENT_SIZE;
 				}
 				else
 				{
-					fragmentLength = (short)(count - position);
+					fragmentLength = (short)(count + offset - position);
 				}
 
 				// Fill the fragment data
