@@ -109,7 +109,7 @@ namespace System.Configuration {
 				rootGroup.StreamName = streamName;
 			}
 			
-			if (configPath != null)
+			if (streamName != null)
 				Load ();
 		}
 		
@@ -231,18 +231,24 @@ namespace System.Configuration {
 			ConfigurationSection sec = data as ConfigurationSection;
 			if (sec != null || !createDefaultInstance) return sec;
 			
-			object secObj = config.CreateInstance () as ConfigurationSection;
-			if (secObj == null)
-				sec = new IgnoreSection ();
-			else
-				sec = (ConfigurationSection) secObj;
-				
+			object secObj = config.CreateInstance ();
+			sec = secObj as ConfigurationSection;
+			if (sec == null) {
+				DefaultSection ds = new DefaultSection ();
+				ds.SectionHandler = secObj as IConfigurationSectionHandler;
+				sec = ds;
+			}
+
 			ConfigurationSection parentSection = parent != null ? parent.GetSectionInstance (config, true) : null;
-			sec.RawXml = data as string;
+
+			string xml = data as string;
+			if (xml == null && parentSection != null)
+				xml = parentSection.RawXml;
+			sec.RawXml = xml;
 			sec.Reset (parentSection);
-			
-			if (data != null) {
-				XmlTextReader r = new XmlTextReader (new StringReader (data as string));
+
+			if (xml != null && xml == data) {
+				XmlTextReader r = new XmlTextReader (new StringReader (xml));
 				sec.DeserializeSection (r);
 				r.Close ();
 			}
@@ -289,7 +295,7 @@ namespace System.Configuration {
 			if (sec.SectionInformation.Type == null)
 				sec.SectionInformation.Type = system.Host.GetConfigTypeName (sec.GetType ());
 			
-			SectionInfo section = new SectionInfo (name, sec.SectionInformation.Type, sec.SectionInformation.AllowLocation, sec.SectionInformation.AllowDefinition, sec.SectionInformation.AllowExeDefinition);
+			SectionInfo section = new SectionInfo (name, sec.SectionInformation);
 			section.StreamName = streamName;
 			section.ConfigHost = system.Host;
 			group.AddChild (section);
