@@ -45,11 +45,6 @@ using System.Collections.Specialized;
 using System.ComponentModel;
 using System.Data;
 using System.Data.Common;
-
-#if NET_2_0
-using System.Data.ProviderBase;
-#endif // NET_2_0
-
 using System.EnterpriseServices;
 using System.Globalization;
 using System.Net;
@@ -60,7 +55,7 @@ using System.Xml;
 namespace System.Data.SqlClient {
 	[DefaultEvent ("InfoMessage")]
 #if NET_2_0
-	public sealed class SqlConnection : DbConnectionBase, IDbConnection, ICloneable	
+	public sealed class SqlConnection : DbConnection, IDbConnection, ICloneable	
 #else
 	public sealed class SqlConnection : Component, IDbConnection, ICloneable	                
 #endif // NET_2_0
@@ -116,14 +111,6 @@ namespace System.Data.SqlClient {
                         Init (connectionString);
 		}
 
-#if NET_2_0
-                internal SqlConnection (DbConnectionFactory connectionFactory) : base (connectionFactory)
-                {
-                        Init (String.Empty);
-                }
-
-#endif //NET_2_0
-
                 private void Init (string connectionString)
                 {
                         connectionTimeout       = 15; // default timeout
@@ -436,27 +423,24 @@ namespace System.Data.SqlClient {
 
 		protected override void Dispose (bool disposing) 
 		{
-			if (!disposed) { 
-                                try {
-                                        if (disposing) {
-                                                if (State == ConnectionState.Open) 
-                                                        Close ();
-                                                parms.Reset ();
-                                                ConnectionString = "";
-						SetDefaultConnectionParameters (this.connStringParameters); 
-                                        }
-                                } finally {
-                                        disposed = true;
-                                        base.Dispose (disposing);
-                                }
+			if (disposed)
+				return;
+
+			try {
+				if (disposing) {
+					if (State == ConnectionState.Open) 
+						Close ();
+					ConnectionString = "";
+					SetDefaultConnectionParameters (this.connStringParameters); 
+				}
+			} finally {
+				disposed = true;
+				base.Dispose (disposing);
 			}
 		}
 
 		[MonoTODO ("Not sure what this means at present.")]
 		public 
-#if NET_2_0
-		override
-#endif // NET_2_0
                 void EnlistDistributedTransaction (ITransaction transaction)
 		{
 			throw new NotImplementedException ();
@@ -476,6 +460,17 @@ namespace System.Data.SqlClient {
 		{
 			return BeginTransaction (iso);
 		}
+#if NET_2_0
+		protected override DbTransaction BeginDbTransaction (IsolationLevel level)
+		{
+			return (DbTransaction)BeginTransaction (level);
+		}
+
+		protected override DbCommand CreateDbCommand ()
+		{
+			return CreateCommand ();
+		}
+#endif
 
 		IDbCommand IDbConnection.CreateCommand ()
 		{
@@ -488,11 +483,6 @@ namespace System.Data.SqlClient {
 			GC.SuppressFinalize (this);
 		}
 
-		~SqlConnection ()
-		{
-			Dispose (false);
-		}
-
 		public 
 #if NET_2_0
 		override
@@ -503,7 +493,7 @@ namespace System.Data.SqlClient {
 			if (state == ConnectionState.Open)
 				throw new InvalidOperationException ("The Connection is already Open (State=Open)");
 
-			if (connectionString == null)
+			if (connectionString == null || connectionString.Trim().Length == 0)
 				throw new InvalidOperationException ("Connection string has not been initialized.");
 
 			try {
@@ -552,7 +542,7 @@ namespace System.Data.SqlClient {
 			string theInstanceName = "";
 	
 			if (theDataSource == null)
-				throw new ArgumentException("Format of initialization string doesnot conform to specifications");
+				throw new ArgumentException("Format of initialization string does not conform to specifications");
 
 			thePort = 1433; // default TCP port for SQL Server
 			bool success = true;
@@ -633,8 +623,9 @@ namespace System.Data.SqlClient {
                         NameValueCollection parameters = new NameValueCollection ();
                         SetDefaultConnectionParameters (parameters);
 
-			if ((connectionString == null) || (connectionString.Length == 0)) {
-                                this.connectionString = connectionString;
+			if ((connectionString == null) || (connectionString.Trim().Length == 0)) {
+				this.connectionString = connectionString;
+				this.connStringParameters = parameters;
 				return;
                         }
 
