@@ -41,9 +41,6 @@ using System;
 using System.ComponentModel;
 using System.Data;
 using System.Data.Common;
-#if NET_2_0
-using System.Data.ProviderBase;
-#endif // NET_2_0
 using System.Data.SqlTypes;
 using System.Runtime.InteropServices;
 using System.Text;
@@ -51,7 +48,7 @@ using System.Text;
 namespace System.Data.SqlClient {
 	[TypeConverterAttribute (typeof (SqlParameterConverter))]
 #if NET_2_0
-	public sealed class SqlParameter : DbParameterBase, IDbDataParameter, IDataParameter, ICloneable
+	public sealed class SqlParameter : DbParameter, IDbDataParameter, IDataParameter, ICloneable
 #else
 	public sealed class SqlParameter : MarshalByRefObject, IDbDataParameter, IDataParameter, ICloneable
 #endif // NET_2_0
@@ -192,7 +189,7 @@ namespace System.Data.SqlClient {
 #if NET_2_0
 		override
 #endif // NET_2_0
-	 DbType DbType {
+	 	DbType DbType {
 			get { return dbType; }
 			set { 
 				SetDbType (value); 
@@ -234,11 +231,6 @@ namespace System.Data.SqlClient {
 			get { return metaParameter; }
 		}
 
-		string IDataParameter.ParameterName {
-			get { return metaParameter.ParameterName; }
-			set { metaParameter.ParameterName = value; }
-		}
-
 #if ONLY_1_0 || ONLY_1_1
 		[Browsable (false)]
 		[DataSysDescription ("a design-time property used for strongly typed code-generation.")]
@@ -264,11 +256,7 @@ namespace System.Data.SqlClient {
 #if NET_2_0
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
 #endif
-		public 
-#if NET_2_0
-		override
-#endif
-	 	int Offset {
+		public int Offset {
 			get { return offset; }
 			set { offset = value; }
 		}
@@ -295,11 +283,7 @@ namespace System.Data.SqlClient {
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 #endif		
-		public 
-#if NET_2_0
-		override
-#endif // NET_2_0
-	 byte Precision {
+		public byte Precision {
 			get { return metaParameter.Precision; }
 			set { metaParameter.Precision = value; }
 		}
@@ -313,11 +297,7 @@ namespace System.Data.SqlClient {
 		[Browsable (false)]
                 [EditorBrowsable (EditorBrowsableState.Never)]
 #endif
-                public 
-#if NET_2_0
-		override
-#endif // NET_2_0
-	 byte Scale {
+                public byte Scale {
 			get { return metaParameter.Scale; }
 			set { metaParameter.Scale = value; }
 		}
@@ -427,6 +407,9 @@ namespace System.Data.SqlClient {
 		// infer type information.
 		private void InferSqlType (object value)
 		{
+			if (value == null || value == DBNull.Value)
+				return;
+
 			Type type = value.GetType ();
 
 			string exception = String.Format ("The parameter data type of {0} is invalid.", type.Name);
@@ -486,10 +469,8 @@ namespace System.Data.SqlClient {
 				SetSqlDbType (SqlDbType.Money);
 				break;
 			case "System.Object":
-			case "System.DBNull":
-				SetSqlDbType (SqlDbType.Variant); // variant can contain numeric,
-								//string,binary or data and also nul								    //values, so we can later resolve 									// it to correct type.	
-				break;	
+				SetSqlDbType (SqlDbType.Variant); 
+				break;
 			default:
 				throw new ArgumentException (exception);				
 			}
@@ -828,12 +809,6 @@ namespace System.Data.SqlClient {
 		}
 
 #if NET_2_0
-                [MonoTODO]
-                public override void CopyTo (DbParameter param)
-                {
-                        throw new NotImplementedException ();
-                }
-                
                 [MonoTODO]
                 public override void ResetDbType ()
                 {
