@@ -90,11 +90,6 @@ namespace System.Data {
 			_rowId = builder._rowId;
 
 			rowError = String.Empty;
-
-			// create mapped XmlDataElement
-			DataSet ds = _table.DataSet;
-			if (ds != null && ds._xmlDataDocument != null)
-				mappedElement = new XmlDataDocument.XmlDataElement (this, _table.Prefix, _table.TableName, _table.Namespace, ds._xmlDataDocument);
 		}
 
 		internal DataRow(DataTable table,int rowId)
@@ -341,6 +336,23 @@ namespace System.Data {
 			}
 		}
 
+#if NET_2_0
+		public void SetAdded ()
+		{
+			if (RowState != DataRowState.Unchanged)
+				throw new InvalidOperationException ("SetAdded and SetModified can only be called on DataRows with Unchanged DataRowState.");
+			Original = -1;
+		}
+
+		public void SetModified ()
+		{
+			if (RowState != DataRowState.Unchanged)
+				throw new InvalidOperationException ("SetAdded and SetModified can only be called on DataRows with Unchanged DataRowState.");
+			Current = _table.RecordCache.NewRecord ();
+			_table.RecordCache.CopyRecord (_table, Original, Current);
+		}
+#endif
+
 		/// <summary>
 		/// Gets the DataTable for which this row has a schema.
 		/// </summary>
@@ -476,6 +488,8 @@ namespace System.Data {
 				column.UpdateAutoIncrementValue(column.DataContainer.GetInt64(Proposed));
 			}
 
+			foreach (DataColumn col in Table.Columns)
+				CheckValue (this [col], col);
 		}
 
 		private void CheckValue (object v, DataColumn col) 
@@ -491,11 +505,9 @@ namespace System.Data {
 
 				//Constraint violations during data load is raise in DataTable EndLoad
 				this._nullConstraintViolation = true;
-				if (this.Table._duringDataLoad) {
+				if (this.Table._duringDataLoad || (Table.DataSet != null && !Table.DataSet.EnforceConstraints))
 					this.Table._nullConstraintViolationDuringDataLoad = true;
-				}
 				_nullConstraintMessage = "Column '" + col.ColumnName + "' does not allow nulls.";
-			
 			}
 		}
 
@@ -560,7 +572,16 @@ namespace System.Data {
 		}
 
 		internal XmlDataDocument.XmlDataElement DataElement {
-			get { return mappedElement; }
+			get { 
+				if (mappedElement != null || _table.DataSet == null || 
+					_table.DataSet._xmlDataDocument == null)
+					return mappedElement;
+
+				// create mapped XmlDataElement
+				mappedElement = new XmlDataDocument.XmlDataElement (this, _table.Prefix, XmlHelper.Encode (_table.TableName),
+									_table.Namespace, _table.DataSet._xmlDataDocument);
+				return mappedElement;
+			}
 			set { mappedElement = value; }
 		}
 
@@ -1361,10 +1382,10 @@ namespace System.Data {
 			if (columnIndex < 0 || columnIndex >= Table.Columns.Count)
 				throw new IndexOutOfRangeException ();
 
-			while(ColumnErrors.Count < columnIndex) {
-				ColumnErrors.Add(null);
-			}
-			ColumnErrors.Add(error);
+			while (columnIndex >= ColumnErrors.Count) 
+				ColumnErrors.Add (null);
+
+			ColumnErrors [columnIndex] = error;
 		}
 
 		/// <summary>
