@@ -217,10 +217,18 @@ namespace System.Data {
 						table.ResetIndexes();
 				}
 
+				// TODO : Need to take care of Error handling and settting of RowErrors 
+				bool constraintViolated = false;
 				foreach (DataTable table in Tables) {
 					foreach (Constraint constraint in table.Constraints)
 						constraint.AssertConstraint();
+					table.AssertNotNullConstraints ();
+					if (!constraintViolated && table.HasErrors)
+						constraintViolated = true;
 				}
+
+				if (constraintViolated)
+					Constraint.ThrowConstraintException ();
 			}
 			enforceConstraints = value;
 		}
@@ -234,7 +242,7 @@ namespace System.Data {
 		{
 			Merge (dataSet, false, MissingSchemaAction.Add);
 		}
-		
+
 		public void Merge (DataTable table)
 		{
 			Merge (table, false, MissingSchemaAction.Add);
@@ -592,13 +600,13 @@ namespace System.Data {
 		[MonoTODO]
 		public DataTableReader CreateDataReader (DataTable[] dataTables)
 		{
-			throw new NotImplementedException ();
+			return new DataTableReader (dataTables);
 		}
 
 		[MonoTODO]
 		public DataTableReader CreateDataReader ()
 		{
-			throw new NotImplementedException ();
+			return new DataTableReader ((DataTable[])Tables.ToArray (typeof (DataTable)));
 		}
 #endif
 		
@@ -804,7 +812,7 @@ namespace System.Data {
 				shouldOutputContent = tableCollection [n].Rows.Count > 0;
 				
 			if (shouldOutputContent) {
-				WriteStartElement (writer, mode, Namespace, Prefix, XmlConvert.EncodeName (DataSetName));
+				WriteStartElement (writer, mode, Namespace, Prefix, XmlHelper.Encode (DataSetName));
 				
 				if (mode == XmlWriteMode.WriteSchema)
 					DoWriteXmlSchema (writer);
@@ -1319,22 +1327,10 @@ namespace System.Data {
 		
 		private void WriteTables (XmlWriter writer, XmlWriteMode mode, DataTableCollection tableCollection, DataRowVersion version)
 		{
-			//Write out each table in order, providing it is not
-			//part of another table structure via a nested parent relationship
-			foreach (DataTable table in tableCollection) {
-				bool isTopLevel = true;
-				/*
-				foreach (DataRelation rel in table.ParentRelations) {
-					if (rel.Nested) {
-						isTopLevel = false;
-						break;
-					}
-				}
-				*/
-				if (isTopLevel) {
-					WriteTable ( writer, table, mode, version);
-				}
-			}
+			//WriteTable takes care of skipping a table if it has a 
+			//Nested Parent Relationship
+			foreach (DataTable table in tableCollection)
+				WriteTable ( writer, table, mode, version);
 		}
 
 		private void WriteTable (XmlWriter writer, DataTable table, XmlWriteMode mode, DataRowVersion version)
@@ -1366,30 +1362,18 @@ namespace System.Data {
 			foreach (DataRow row in rows) {
 				if (skipIfNested) {
 					// Skip rows that is a child of any tables.
-					switch (relationCount) {
-					case 0:
-						break;
-					case 1:
-						if (!oneRel.Nested)
-							break;
-						if (row.GetParentRow (oneRel) != null)
+					bool skip = false;
+					for (int i = 0; i < table.ParentRelations.Count; i++) {
+						DataRelation prel = table.ParentRelations [i];
+						if (!prel.Nested)
+							continue;
+						if (row.GetParentRow (prel) != null) {
+							skip = true;
 							continue;
-						break;
-					case 2:
-						bool skip = false;
-						for (int i = 0; i < table.ParentRelations.Count; i++) {
-							DataRelation prel = table.ParentRelations [i];
-							if (!prel.Nested)
-								continue;
-							if (row.GetParentRow (prel) != null) {
-								skip = true;
-								continue;
-							}
 						}
-						if (skip)
-							continue;
-						break;
 					}
+					if (skip)
+						continue;
 				}
 
 				if (!row.HasVersion(version) || 
@@ -1409,7 +1393,7 @@ namespace System.Data {
 
 				// If all of the columns were null, we have to write empty element
 				if (AllNulls) {
-					writer.WriteElementString (XmlConvert.EncodeLocalName (table.TableName), "");
+					writer.WriteElementString (XmlHelper.Encode (table.TableName), "");
 					continue;
 				}
 				
@@ -1451,7 +1435,7 @@ namespace System.Data {
 				colnspc = col.Namespace;
 	
 			//TODO check if I can get away with write element string
-			WriteStartElement (writer, mode, colnspc, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName));
+			WriteStartElement (writer, mode, colnspc, col.Prefix, XmlHelper.Encode (col.ColumnName));
 			writer.WriteString (WriteObjectXml (rowObject));
 			writer.WriteEndElement ();
 		}
@@ -1459,7 +1443,7 @@ namespace System.Data {
 		private void WriteColumnAsAttribute (XmlWriter writer, XmlWriteMode mode, DataColumn col, DataRow row, DataRowVersion version)
 		{
 			if (!row.IsNull (col))
-				WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), WriteObjectXml (row[col, version]));
+				WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlHelper.Encode (col.ColumnName), WriteObjectXml (row[col, version]));
 		}
 
 		private void WriteTableElement (XmlWriter writer, XmlWriteMode mode, DataTable table, DataRow row, DataRowVersion version)
@@ -1467,7 +1451,7 @@ namespace System.Data {
 			//sort out the namespacing
 			string nspc = table.Namespace.Length > 0 ? table.Namespace : Namespace;
 
-			WriteStartElement (writer, mode, nspc, table.Prefix, XmlConvert.EncodeLocalName (table.TableName));
+			WriteStartElement (writer, mode, nspc, table.Prefix, XmlHelper.Encode (table.TableName));
 
 			if (mode == XmlWriteMode.DiffGram) {
 				WriteAttributeString (writer, mode, XmlConstants.DiffgrNamespace, XmlConstants.DiffgrPrefix, "id", table.TableName + (row.XmlRowID + 1));
@@ -1510,7 +1494,7 @@ namespace System.Data {
 				WriteDiffGramElement (writer);
 			}
 			
-			WriteStartElement (writer, mode, Namespace, Prefix, XmlConvert.EncodeName (DataSetName));
+			WriteStartElement (writer, mode, Namespace, Prefix, XmlHelper.Encode (DataSetName));
 			
 			WriteTable (writer, table, mode, DataRowVersion.Default);
 			
