@@ -96,7 +96,7 @@ namespace System.Data {
 		private ArrayList _indexes;
 		private RecordCache _recordCache;
 		private int _defaultValuesRowIndex = -1;
-		protected internal bool initInProgress;
+		protected internal bool fInitInProgress;
 
 		// If CaseSensitive property is changed once it does not anymore follow owner DataSet's 
 		// CaseSensitive property. So when you lost you virginity it's gone for ever
@@ -617,39 +617,54 @@ namespace System.Data {
 		internal bool EnforceConstraints {
 			get { return enforceConstraints; }
 			set {
-				if (value != enforceConstraints) {
-					if (value) {
-						// reset indexes since they may be outdated
-						ResetIndexes();
+				if (value == enforceConstraints)
+					return;
 
-						bool violatesConstraints = false;
+				if (value) {
+					// reset indexes since they may be outdated
+					ResetIndexes();
 
-						//FIXME: use index for AllowDBNull
-						for (int i = 0; i < Columns.Count; i++) {
-							DataColumn column = Columns[i];
-							if (!column.AllowDBNull) {
-								for (int j = 0; j < Rows.Count; j++){
-									if (Rows[j].IsNull(column)) {
-										violatesConstraints = true;
-										Rows[j].RowError = String.Format("Column '{0}' does not allow DBNull.Value.", column.ColumnName);
-									}
-								}
-							}
-						}
+					// assert all constraints
+					foreach (Constraint constraint in Constraints)
+						constraint.AssertConstraint();
 
-						if (violatesConstraints)
-							Constraint.ThrowConstraintException();
+					AssertNotNullConstraints ();
 
-						// assert all constraints
-						foreach (Constraint constraint in Constraints) {
-							constraint.AssertConstraint();
-						}
-					}
-					enforceConstraints = value;
+					if (HasErrors)
+						Constraint.ThrowConstraintException ();
 				}
+				enforceConstraints = value;
 			}
 		}
 
+		internal void AssertNotNullConstraints ()
+		{
+			if (_duringDataLoad && !_nullConstraintViolationDuringDataLoad)
+				return;
+
+			bool result = false;
+			String errMsg;
+			for (int j = 0; j < Rows.Count; j++) {
+				if (!Rows [j].HasVersion (DataRowVersion.Default))
+					continue;	
+				errMsg = String.Empty;
+				for (int i = 0; i < Columns.Count; i++) {
+					DataColumn column = Columns[i];
+					if (column.AllowDBNull || !Rows[j].IsNull (column))
+						continue;
+					result = true;
+					errMsg =  String.Format("Column '{0}' does not allow DBNull.Value.",
+							column.ColumnName);
+					Rows [j].SetColumnError (i, errMsg);
+				}
+				// ms.net sets the last ColumnError as RowError
+				if (errMsg != String.Empty)
+					Rows [j].RowError = errMsg;
+			}
+			if (!result)
+				_nullConstraintViolationDuringDataLoad = false;
+		}
+
 		internal bool RowsExist(DataColumn[] columns, DataColumn[] relatedColumns,DataRow row)
 		{
 			int curIndex = row.IndexFromVersion(DataRowVersion.Default);
@@ -721,6 +736,7 @@ namespace System.Data {
 				if (myRow.RowState != DataRowState.Detached)
 					i++;
 			}
+			_rows.OnListChanged (this, new ListChangedEventArgs (ListChangedType.Reset, -1, -1));
 		}
 
 		/// <summary>
@@ -728,7 +744,11 @@ namespace System.Data {
 		/// on a form or used by another component. The initialization
 		/// occurs at runtime.
 		/// </summary>
-		public virtual void BeginInit () 
+		public 
+#if NET_2_0
+		virtual
+#endif
+		void BeginInit () 
 		{
 			InitInProgress = true;
 		}
@@ -925,15 +945,19 @@ namespace System.Data {
 		/// initialization occurs at runtime.
 		/// </summary>
 		[MonoTODO]
-		public virtual void EndInit () 
+		public
+#if NET_2_0
+		virtual
+#endif
+		void EndInit () 
 		{
 			InitInProgress = false;
 			FinishInit ();
 		}
 
 		internal bool InitInProgress {
-			get { return initInProgress; }
-			set { initInProgress = value; }
+			get { return fInitInProgress; }
+			set { fInitInProgress = value; }
 		}
 
 		internal void FinishInit ()
@@ -959,21 +983,12 @@ namespace System.Data {
 		public void EndLoadData() 
 		{
 			if (this._duringDataLoad) {
-				if(this._nullConstraintViolationDuringDataLoad) {
-					this._nullConstraintViolationDuringDataLoad = false;
-					throw new ConstraintException ("Failed to enable constraints. One or more rows contain values violating non-null, unique, or foreign-key constraints.");
-				}
-				
-				if (this.dataSet !=null) {
-					//Getting back to previous EnforceConstraint state
+				//Getting back to previous EnforceConstraint state
+				if (this.dataSet != null)
 					this.dataSet.InternalEnforceConstraints(this.dataSetPrevEnforceConstraints,true);
-				}
-				else {
-					//Getting back to the table's previous EnforceConstraint state
+				else
 					this.EnforceConstraints = true;
-				}
 
-				//Returning from loading mode, raising exceptions as usual
 				this._duringDataLoad = false;
 			}
 		}
@@ -1016,9 +1031,9 @@ namespace System.Data {
 
 #if NET_2_0
 		[MonoTODO]
-		public DataTableReader GetDataReader ()
+		public DataTableReader CreateDataReader ()
 		{
-			throw new NotImplementedException ();
+			return new DataTableReader (this);
 		}
 #endif
 
@@ -1190,42 +1205,39 @@ namespace System.Data {
 		public DataRow LoadDataRow (object[] values, bool fAcceptChanges) 
 		{
 			DataRow row = null;
-			if (PrimaryKey.Length == 0) {
+			if (PrimaryKey.Length == 0)
 				row = Rows.Add (values);
-				if (fAcceptChanges)
-					row.AcceptChanges ();
-			}
 			else {
 				EnsureDefaultValueRowIndex();
 				int newRecord = CreateRecord(values);
-				int existingRecord = _primaryKeyConstraint.Index.Find(newRecord);
+				int existingRecord = _primaryKeyConstraint.Index.Find (newRecord);
 
 				if (existingRecord < 0) {
 					row = NewRowFromBuilder (RowBuilder);
 					row.Proposed = newRecord;
-					AddRowToIndexes (row);
 					Rows.AddInternal(row);
-				}
-				else {
+					if (!_duringDataLoad)
+						AddRowToIndexes (row);
+				} else {
 					row = RecordCache[existingRecord];
 					row.BeginEdit();
 					row.ImportRecord(newRecord);
 					row.EndEdit();
 				}
-				
-				if (fAcceptChanges)
-					row.AcceptChanges ();
 			}
+			
+			if (fAcceptChanges)
+				row.AcceptChanges ();
 				
 			return row;
 		}
 
-		internal DataRow LoadDataRow(IDataRecord record, int[] mapping, int length, bool fAcceptChanges)
+		internal DataRow LoadDataRow (IDataRecord record, int[] mapping, int length, bool fAcceptChanges)
 		{
 			DataRow row = null;
-				int tmpRecord = this.RecordCache.NewRecord();
-				try {
-				RecordCache.ReadIDataRecord(tmpRecord,record,mapping,length);
+			int tmpRecord = this.RecordCache.NewRecord ();
+			try {
+				RecordCache.ReadIDataRecord (tmpRecord,record,mapping,length);
 				if (PrimaryKey.Length != 0) {
 					bool hasPrimaryValues = true;
 					foreach(DataColumn col in PrimaryKey) {
@@ -1234,35 +1246,28 @@ namespace System.Data {
 							break;
 						}
 					}
-					
+
 					if (hasPrimaryValues) {
-						// find the row in the table.
-						row = Rows.Find(tmpRecord);
+						int existingRecord = _primaryKeyConstraint.Index.Find (tmpRecord);
+						if (existingRecord != -1)
+							row  = RecordCache [existingRecord];
 					}
 				}
-					
-				bool shouldUpdateIndex = false;
+
 				if (row == null) {
 					row = NewNotInitializedRow();
-					row.ImportRecord(tmpRecord);
+					row.Proposed = tmpRecord;
 					Rows.AddInternal (row);
-					shouldUpdateIndex = true;
-				}
-				else {
-					// Proposed = tmpRecord
-					row.ImportRecord(tmpRecord);
-				}
-				
-				if (fAcceptChanges) {
-					row.AcceptChanges();
-				}
-				
-				if (shouldUpdateIndex && !fAcceptChanges) {
-					AddRowToIndexes(row);
+				} else {
+					row.BeginEdit ();
+					row.ImportRecord (tmpRecord);
+					row.EndEdit ();
 				}
 
-			}
-			catch(Exception e) {
+				if (fAcceptChanges)
+					row.AcceptChanges();
+
+			} catch(Exception e) {
 				this.RecordCache.DisposeRecord(tmpRecord);
 				throw e;
 			}				
@@ -1326,19 +1331,19 @@ namespace System.Data {
 		[MonoTODO]
 		public void Merge (DataTable table)
 		{
-			throw new NotImplementedException ();
+			Merge (table, false, MissingSchemaAction.Add);
 		}
 
 		[MonoTODO]
 		public void Merge (DataTable table, bool preserveChanges)
 		{
-			throw new NotImplementedException ();
+			Merge (table, preserveChanges, MissingSchemaAction.Add);
 		}
 
 		[MonoTODO]
 		public void Merge (DataTable table, bool preserveChanges, MissingSchemaAction missingSchemaAction)
 		{
-			throw new NotImplementedException ();
+			MergeManager.Merge (this, table, preserveChanges, missingSchemaAction);
 		}
 #endif
 
@@ -1553,18 +1558,37 @@ namespace System.Data {
 			if (filterExpression == null)
 				filterExpression = String.Empty;
 
-			DataColumn[] columns = _emptyColumnArray;
-			ListSortDirection[] sorts = null;
-			if (sort != null && !sort.Equals(String.Empty))
-				columns = ParseSortString (this, sort, out sorts, false);
-
 			IExpression filter = null;
 			if (filterExpression != String.Empty) {
 				Parser parser = new Parser ();
 				filter = parser.Compile (filterExpression);
 			}
 
-			Index index = GetIndex(columns, sorts, recordStates, filter, false);
+			DataColumn[] columns = _emptyColumnArray;
+			ListSortDirection[] sorts = null;
+			
+			if (sort != null && !sort.Equals(String.Empty))
+				columns = ParseSortString (this, sort, out sorts, false);
+
+			if (Rows.Count == 0)
+				return new DataRow [0];
+
+			//if sort order is not given, sort it in Ascending order of the
+			//columns involved in the filter
+			if (columns.Length ==0 && filter != null) {
+				ArrayList list = new ArrayList ();
+				for (int i=0, j=0; i < Columns.Count; ++i) {
+					if (!filter.DependsOn (Columns [i]))
+						continue;
+					list.Add (Columns [i]);
+				}
+				columns = (DataColumn[])list.ToArray (typeof (DataColumn));
+			}
+
+			bool addIndex = true;
+			if (filterExpression != String.Empty)
+				addIndex = false;
+			Index index = GetIndex(columns, sorts, recordStates, filter, false, addIndex);
 
 			int[] records = index.GetAll();
 			DataRow[] dataRows = NewRowArray(index.Size);
@@ -1573,7 +1597,6 @@ namespace System.Data {
 
 			return dataRows;
 		}
-
 		
 		private void AddIndex (Index index)
 		{
@@ -1610,6 +1633,24 @@ namespace System.Data {
 			return index;
 		}
 
+		internal Index GetIndex (DataColumn[] columns, ListSortDirection[] sort,
+					 DataViewRowState rowState, IExpression filter,
+					 bool reset, bool addIndex)
+		{
+			Index index = FindIndex(columns,sort,rowState,filter);
+			if (index == null ) {
+				index = new Index(new Key(this,columns,sort,rowState,filter));
+
+				if (addIndex)
+					AddIndex (index);
+			}
+			else if (reset) {
+				// reset existing index only if asked for this
+				index.Reset ();
+			}
+			return index;
+		}
+
 		internal Index FindIndex(DataColumn[] columns)
 		{
 			return FindIndex(columns,null,DataViewRowState.None, null);
