@@ -162,6 +162,7 @@ namespace System.Data {
 
 #if NET_2_0
 		[MonoTODO]
+		[Browsable (false)]
 		public ForeignKeyConstraint (string constraintName, string parentTableName, string parentTableNamespace, string[] parentColumnNames, string[] childColumnNames, AcceptRejectRule acceptRejectRule, Rule deleteRule, Rule updateRule)
 		{
 			throw new NotImplementedException ();
@@ -256,14 +257,12 @@ namespace System.Data {
 				if (pc == cc)
 					throw new InvalidOperationException("Parent and child columns can't be the same column.");
 
-				if (! pc.DataType.Equals(cc.DataType))
-				{
+				if (!pc.DataTypeMatches (cc)) {
 					//LAMESPEC: spec says throw InvalidConstraintException
 					//		implementation throws InvalidOperationException
-					throw new InvalidConstraintException("Parent column is not type compatible with it's child"
+					throw new InvalidOperationException ("Parent column is not type compatible with it's child"
 						+ " column.");
 				}
-					
 			}
 			
 		}
