@@ -261,7 +261,7 @@ namespace System.Data
 			TableName = tname;
 			Columns = cols;
 			IsAttribute = isAttr;
-			ConstraintName = XmlConvert.DecodeName (cname);
+			ConstraintName = XmlHelper.Decode (cname);
 			IsPrimaryKey = isPK;
 			ReferName = refName;
 			IsNested = isNested;
@@ -518,7 +518,7 @@ el.ElementType != schemaAnyType)
 
 		private void ProcessDataTableElement (XmlSchemaElement el)
 		{
-			string tableName = XmlConvert.DecodeName (el.QualifiedName.Name);
+			string tableName = XmlHelper.Decode (el.QualifiedName.Name);
 			// If it is already registered, just ignore.
 			if (dataset.Tables.Contains (tableName))
 				return;
@@ -585,7 +585,7 @@ el.ElementType != schemaAnyType)
 			DataTable ctab = dataset.Tables [rs.ChildTableName];
 
 			DataRelation rel ;
-			string name = rs.ExplicitName != null ? rs.ExplicitName : XmlConvert.DecodeName (ptab.TableName) + '_' + XmlConvert.DecodeName (ctab.TableName);
+			string name = rs.ExplicitName != null ? rs.ExplicitName : XmlHelper.Decode (ptab.TableName) + '_' + XmlHelper.Decode (ctab.TableName);
 
 			// Annotation Relations belonging to a DataSet can contain multiple colnames
 			// in parentkey and childkey.
@@ -595,24 +595,20 @@ el.ElementType != schemaAnyType)
 
 				DataColumn[] pcol = new DataColumn [pcolnames.Length];
 				for (int i=0; i<pcol.Length; ++i)
-					pcol [i] = ptab.Columns [XmlConvert.DecodeName (pcolnames [i])];
+					pcol [i] = ptab.Columns [XmlHelper.Decode (pcolnames [i])];
 
 				DataColumn[] ccol = new DataColumn [ccolnames.Length];
-				for (int i=0; i < ccol.Length; ++i)
-					ccol [i] = ctab.Columns [XmlConvert.DecodeName (ccolnames [i])];
-
+				for (int i=0; i < ccol.Length; ++i) {
+					ccol [i] = ctab.Columns [XmlHelper.Decode (ccolnames [i])];
+					if (ccol [i] == null)
+						ccol [i] = CreateChildColumn (pcol [i], ctab);
+				}
 				rel = new DataRelation (name, pcol, ccol, rs.CreateConstraint);
 			} else {
-				DataColumn pcol = ptab.Columns [XmlConvert.DecodeName (rs.ParentColumnName)];
-				DataColumn ccol = ctab.Columns [XmlConvert.DecodeName (rs.ChildColumnName)];
-				if (ccol == null) {
-					ccol = new DataColumn ();
-					ccol.ColumnName = pcol.ColumnName;
-					ccol.Namespace = String.Empty; // don't copy
-					ccol.ColumnMapping = MappingType.Hidden;
-					ccol.DataType = pcol.DataType;
-					ctab.Columns.Add (ccol);
-				}
+				DataColumn pcol = ptab.Columns [XmlHelper.Decode (rs.ParentColumnName)];
+				DataColumn ccol = ctab.Columns [XmlHelper.Decode (rs.ChildColumnName)];
+				if (ccol == null) 
+					ccol = CreateChildColumn (pcol, ctab);
 				rel = new DataRelation (name, pcol, ccol, rs.CreateConstraint);
 			}
 			rel.Nested = rs.IsNested;
@@ -621,6 +617,15 @@ el.ElementType != schemaAnyType)
 			return rel;
 		}
 
+		private DataColumn CreateChildColumn (DataColumn parentColumn, DataTable childTable)
+		{
+			DataColumn col = childTable.Columns.Add (parentColumn.ColumnName, 
+								parentColumn.DataType);
+			col.Namespace = String.Empty;
+			col.ColumnMapping = MappingType.Hidden;
+			return col;
+		}
+
 		private void ImportColumnGroupBase (XmlSchemaElement parent, XmlSchemaGroupBase gb)
 		{
 			foreach (XmlSchemaParticle p in gb.Items) {
@@ -732,24 +737,23 @@ el.ElementType != schemaAnyType)
 			if (targetElements.Contains (el))
 				return; // do nothing
 
-			string elName = XmlConvert.DecodeName (el.QualifiedName.Name);
+			string elName = XmlHelper.Decode (el.QualifiedName.Name);
 			if (elName == dataset.DataSetName)
 				// Well, why it is ArgumentException :-?
 				throw new ArgumentException ("Nested element must not have the same name as DataSet's name.");
 
 			if (el.Annotation != null)
 				HandleAnnotations (el.Annotation, true);
-			// If xsd:keyref for this table exists, then don't add
+			// If xsd:keyref xsd:key for this table exists, then don't add
 			// relation here manually.
-			else if (!DataSetDefinesPrimaryKey (elName)) {
+			else if (!DataSetDefinesKey (elName)) {
 				AddParentKeyColumn (parent, el, col);
-				DataColumn pkey = currentTable.PrimaryKey;
 
 				RelationStructure rel = new RelationStructure ();
-				rel.ParentTableName = XmlConvert.DecodeName (parent.QualifiedName.Name);
+				rel.ParentTableName = XmlHelper.Decode (parent.QualifiedName.Name);
 				rel.ChildTableName = elName;
-				rel.ParentColumnName = pkey.ColumnName;
-				rel.ChildColumnName = pkey.ColumnName;
+				rel.ParentColumnName = col.ColumnName;
+				rel.ChildColumnName = col.ColumnName;
 				rel.CreateConstraint = true;
 				rel.IsNested = true;
 				relations.Add (rel);
@@ -761,10 +765,10 @@ el.ElementType != schemaAnyType)
 
 		}
 
-		private bool DataSetDefinesPrimaryKey (string name)
+		private bool DataSetDefinesKey (string name)
 		{
 			foreach (ConstraintStructure c in reservedConstraints.Values)
-				if (c.TableName == name && c.IsPrimaryKey)
+				if (c.TableName == name && (c.IsPrimaryKey || c.IsNested))
 					return true;
 			return false;
 		}
@@ -775,9 +779,10 @@ el.ElementType != schemaAnyType)
 				return;
 
 			// check name identity
-			string name = XmlConvert.DecodeName (parent.QualifiedName.Name) + "_Id";
-			if (currentTable.ContainsColumn (name))
-				throw new DataException (String.Format ("There is already a column that has the same name: {0}", name));
+			string name = XmlHelper.Decode (parent.QualifiedName.Name) + "_Id";
+			int count = 0;
+			while (currentTable.ContainsColumn (name))
+				name = String.Format ("{0}_{1}", name, count++);
 			// check existing primary key
 			if (currentTable.Table.PrimaryKey.Length > 0)
 				throw new DataException (String.Format ("There is already primary key columns in the table \"{0}\".", currentTable.Table.TableName));
@@ -786,7 +791,6 @@ el.ElementType != schemaAnyType)
 			col.ColumnMapping = MappingType.Hidden;
 			col.Namespace = parent.QualifiedName.Namespace;
 			col.DataType = typeof (int);
-			col.Unique = true;
 			col.AutoIncrement = true;
 			col.AllowDBNull = false;
 
@@ -803,8 +807,8 @@ el.ElementType != schemaAnyType)
 			AddParentKeyColumn (parent, el, col);
 			DataColumn pkey = currentTable.PrimaryKey;
 
-			string elName = XmlConvert.DecodeName (el.QualifiedName.Name);
-			string parentName = XmlConvert.DecodeName (parent.QualifiedName.Name);
+			string elName = XmlHelper.Decode (el.QualifiedName.Name);
+			string parentName = XmlHelper.Decode (parent.QualifiedName.Name);
 
 			DataTable dt = new DataTable ();
 			dt.TableName = elName;
@@ -840,7 +844,7 @@ el.ElementType != schemaAnyType)
 
 		private void FillDataColumnSimpleElement (XmlSchemaElement el, DataColumn col)
 		{
-			col.ColumnName = XmlConvert.DecodeName (el.QualifiedName.Name);
+			col.ColumnName = XmlHelper.Decode (el.QualifiedName.Name);
 			col.Namespace = el.QualifiedName.Namespace;
 			col.ColumnMapping = MappingType.Element;
 			col.DataType = ConvertDatatype (GetSchemaPrimitiveType (el.ElementType));
@@ -913,7 +917,7 @@ el.ElementType != schemaAnyType)
 			if (index > 0)
 				tableName = tableName.Substring (index + 1);
 
-			return XmlConvert.DecodeName (tableName);
+			return XmlHelper.Decode (tableName);
 		}
 
 		private void ReserveSelfIdentity (XmlSchemaIdentityConstraint ic)
@@ -933,7 +937,7 @@ el.ElementType != schemaAnyType)
 				else if (isAttr)
 					colName = colName.Substring (1);
 
-				colName = XmlConvert.DecodeName (colName);
+				colName = XmlHelper.Decode (colName);
 				cols [i] = colName;
 				isAttrSpec [i] = isAttr;
 				i++;
@@ -1011,7 +1015,7 @@ el.ElementType != schemaAnyType)
 				else if (isAttr)
 					colName = colName.Substring (1);
 
-				colName = XmlConvert.DecodeName (colName);
+				colName = XmlHelper.Decode (colName);
 				cols [i] = colName;
 				isAttrSpec [i] = isAttr;
 				i++;
@@ -1137,9 +1141,9 @@ el.ElementType != schemaAnyType)
 			string fkn = el.GetAttribute ("childkey", XmlConstants.MsdataNamespace);
 
 			RelationStructure rel = new RelationStructure ();
-			rel.ExplicitName = XmlConvert.DecodeName (name);
-			rel.ParentTableName = XmlConvert.DecodeName (ptn);
-			rel.ChildTableName = XmlConvert.DecodeName (ctn);
+			rel.ExplicitName = XmlHelper.Decode (name);
+			rel.ParentTableName = XmlHelper.Decode (ptn);
+			rel.ChildTableName = XmlHelper.Decode (ctn);
 			// ColumnNames will be decoded wherever they are used as they can
 			// contain 'space' separated list of column-names.
 			rel.ParentColumnName = pkn;
