@@ -1130,6 +1130,8 @@ namespace MonoTests.System.Data
 
 			// error string
 			Assert.AreEqual(sColErr  ,  dr.GetColumnError(0) , "DRW117");
+			dr.SetColumnError (0, "");
+			Assert.AreEqual("",  dr.GetColumnError (0) , "DRW118");
 		}
 
 		[Test] public void SetColumnError_ByColumnNameError()
@@ -1319,6 +1321,41 @@ namespace MonoTests.System.Data
 			ds.Tables[0].EndLoadData(); //Foreign constraint violation
 		}
 
+
+		[Test]
+		public void TestRowErrors ()
+		{
+			DataTable table = new DataTable ();
+			DataColumn col1 = table.Columns.Add ("col1", typeof (int));
+			DataColumn col2 = table.Columns.Add ("col2", typeof (int));
+			DataColumn col3 = table.Columns.Add ("col3", typeof (int));
+
+			col1.AllowDBNull = false;
+			table.Constraints.Add ("uc", new DataColumn[] {col2,col3}, false);
+			table.BeginLoadData ();
+			table.Rows.Add (new object[] {null,1,1});
+			table.Rows.Add (new object[] {1,1,1});
+			try {
+				table.EndLoadData ();
+				Assert.Fail ("#0");
+			} catch (ConstraintException) {}
+			Assert.IsTrue (table.HasErrors, "#1");
+			DataRow[] rows = table.GetErrors ();
+
+			Assert.AreEqual (2, rows.Length, "#2");
+			Assert.AreEqual ("Column 'col1' does not allow DBNull.Value.", table.Rows [0].RowError, "#3");
+			Assert.AreEqual ("Column 'col2, col3' is constrained to be unique.  Value '1, 1' is already present."
+					, table.Rows [1].RowError, "#4");
+
+			Assert.AreEqual (table.Rows [0].RowError, table.Rows [0].GetColumnError (0), "#5");
+			Assert.AreEqual (table.Rows [1].RowError, table.Rows [0].GetColumnError (1), "#6");
+			Assert.AreEqual (table.Rows [1].RowError, table.Rows [0].GetColumnError (2), "#7");
+
+			Assert.AreEqual ("", table.Rows [1].GetColumnError (0), "#8");
+			Assert.AreEqual (table.Rows [1].RowError, table.Rows [1].GetColumnError (1), "#9");
+			Assert.AreEqual (table.Rows [1].RowError, table.Rows [1].GetColumnError (2), "#10");
+		}
+
 		[Test]
 		public void BeginEdit()
 		{
@@ -1579,5 +1616,123 @@ namespace MonoTests.System.Data
 				Assert.Fail("DRW147: Wrong exception type. Got:" + exc);
 			}
 		}
+
+#if NET_2_0
+		string SetAddedModified_ErrMsg = "SetAdded and SetModified can only be called on DataRows with Unchanged DataRowState.";
+		[Test]
+		public void SetAdded_test()
+		{
+			DataTable table = new DataTable();
+
+			DataRow row = table.NewRow();
+			try {
+				row.SetAdded();
+				Assert.Fail ("#1");
+			} catch (InvalidOperationException e) {
+				Assert.AreEqual (SetAddedModified_ErrMsg, e.Message, "#2");
+			}
+
+			table.Columns.Add("col1", typeof(int));
+			table.Columns.Add("col2", typeof(int));
+			table.Columns.Add("col3", typeof(int));
+
+			row = table.Rows.Add(new object[] { 1, 2, 3 });
+			Assert.AreEqual(DataRowState.Added, row.RowState, "#1");
+			try {
+				row.SetAdded();
+				Assert.Fail ("#2");
+			} catch (InvalidOperationException e) {
+				Assert.AreEqual (SetAddedModified_ErrMsg, e.Message, "#2");
+			}
+			Assert.AreEqual(DataRowState.Added, row.RowState, "#2");
+
+			row.AcceptChanges();
+			row[0] = 10;
+			Assert.AreEqual(DataRowState.Modified, row.RowState, "#5");
+			try {
+				row.SetAdded();
+				Assert.Fail ("#3");
+			} catch (InvalidOperationException e) {
+				Assert.AreEqual (SetAddedModified_ErrMsg, e.Message, "#2");
+			}
+
+			row.AcceptChanges();
+			Assert.AreEqual(DataRowState.Unchanged, row.RowState, "#3");
+			row.SetAdded();
+			Assert.AreEqual(DataRowState.Added, row.RowState, "#4");
+		}
+
+		[Test]
+		public void setAdded_testRollback ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (int));
+			table.Columns.Add ("col2", typeof (int));
+
+			table.Rows.Add (new object[] {1,1});
+			table.AcceptChanges ();
+
+			table.Rows [0].SetAdded ();
+			table.RejectChanges ();
+			Assert.AreEqual (0, table.Rows.Count, "#1");
+		}
+
+		[Test]
+		public void SetModified_test()
+		{
+			DataTable table = new DataTable();
+
+			DataRow row = table.NewRow();
+			try {
+				row.SetModified ();
+			} catch (InvalidOperationException) {}
+
+			table.Columns.Add("col1", typeof(int));
+			table.Columns.Add("col2", typeof(int));
+			table.Columns.Add("col3", typeof(int));
+
+			row = table.Rows.Add(new object[] { 1, 2, 3 });
+			Assert.AreEqual(DataRowState.Added, row.RowState, "#1");
+			try {
+				row.SetModified();
+				Assert.Fail ("#1");
+			} catch (InvalidOperationException e) {
+				Assert.AreEqual (SetAddedModified_ErrMsg, e.Message, "#2");
+			}
+
+			row.AcceptChanges();
+			row[0] = 10;
+			Assert.AreEqual(DataRowState.Modified, row.RowState, "#5");
+			try {
+				row.SetModified ();
+				Assert.Fail ("#2");
+			} catch (InvalidOperationException e) {
+				Assert.AreEqual (SetAddedModified_ErrMsg, e.Message, "#2");
+			}
+
+			row.AcceptChanges();
+			Assert.AreEqual(DataRowState.Unchanged, row.RowState, "#3");
+			row.SetModified ();
+			Assert.AreEqual(DataRowState.Modified, row.RowState, "#4");
+		}
+
+		[Test]
+		public void setModified_testRollback()
+		{
+			DataTable table = new DataTable();
+			table.Columns.Add("col1", typeof(int));
+			table.Columns.Add("col2", typeof(int));
+
+			DataRow row = table.Rows.Add(new object[] { 1, 1 });
+			table.AcceptChanges();
+
+			row.SetModified ();
+			Assert.AreEqual(row.RowState, DataRowState.Modified, "#0");
+			Assert.AreEqual(1, row [0, DataRowVersion.Current], "#1");
+			Assert.AreEqual(1, row [0, DataRowVersion.Original], "#2");
+			table.RejectChanges ();
+			Assert.AreEqual(row.RowState, DataRowState.Unchanged, "#3");
+		}
+#endif
 	}
 }
