@@ -4,10 +4,7 @@
 //      Alejandro Snchez Acosta  <raciel@es.gnu.org>
 // 
 // (C) Alejandro Snchez Acosta
-// 
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004,2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,9 +26,11 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-namespace System.Drawing.Design
-{
+namespace System.Drawing.Design {
+
+#if ONLY_1_1
 	[Serializable]
+#endif
 	public delegate void ToolboxComponentsCreatedEventHandler (object sender, ToolboxComponentsCreatedEventArgs e);
 }
 
