@@ -10,10 +10,7 @@
 //   Jordi Mas i Hernandez (jordi@ximian.com)
 //
 // (C) 2002 Ximian, Inc.  http://www.ximian.com
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004,2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -35,16 +32,16 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Runtime.InteropServices;
 using System.Collections;
 using System.IO;
 
 namespace System.Drawing.Imaging {
 
+#if ONLY_1_1
 	[ComVisible (false)]
-	public sealed class ImageCodecInfo 
-	{
+#endif
+	public sealed class ImageCodecInfo {
 		private Guid clsid;
 		private string codecName;
 		private string dllName;
