@@ -45,7 +45,9 @@ using System.Reflection;
 namespace System.Drawing 
 {
 	[TypeConverter(typeof(ColorConverter))]
+#if ONLY_1_1
 	[ComVisible (true)]
+#endif
 #if !TARGET_JVM
 	[Editor ("System.Drawing.Design.ColorEditor, " + Consts.AssemblySystem_Drawing_Design, typeof (System.Drawing.Design.UITypeEditor))]
 #endif
@@ -56,28 +58,25 @@ namespace System.Drawing
 		private static Hashtable systemColors;
 		static Color [] knownColors;
 		
-		// Private transparancy (A) and R,G,B fields.
-		private byte a;
-		private byte r;
-		private byte g;
-		private byte b;
+		// Private transparency (A) and R,G,B fields.
+		private long value;
 		private static string creatingColorNames = "creatingColorNames";
 
-		// The specs also indicate that all three of these propities are true
+		// The specs also indicate that all three of these properties are true
 		// if created with FromKnownColor or FromNamedColor, false otherwise (FromARGB).
 		// Per Microsoft and ECMA specs these varibles are set by which constructor is used, not by their values.
 		[Flags]
-		enum ColorType {
+		enum ColorType : short {
 			Empty=0,
-			ARGB=1,
-			Known=2,
+			Known=1,
+			ARGB=2,
 			Named=4,
 			System=8
 		}
-		private ColorType colorType;
-		private KnownColor knownColor;
+		private short state;
+		private short knownColor;
 
-		private string myname;
+		private string name;
 
 #if TARGET_JVM
 		internal java.awt.Color NativeObject {
@@ -89,27 +88,34 @@ namespace System.Drawing
 
 		public string Name {
 			get{
-				if (myname == null || myname == String.Empty)
+				if (name == null) {
+					if (IsNamedColor) { // Can happen with stuff deserialized from MS
+						FillColorNames ();
+						object o = knownColors [knownColor];
+						if (o != null)
+							return ((Color) o).name;
+					}
 					return String.Format ("{0:x}", ToArgb ());
-				return myname;
+				}
+				return name;
 			}
 		}
 
 		public bool IsKnownColor {
 			get{
-				return (colorType & ColorType.Known) != 0;
+				return (state & ((short) ColorType.Known)) != 0;
 			}
 		}
 
 		public bool IsSystemColor {
 			get{
-				return (colorType & ColorType.System) != 0;
+				return (state & ((short) ColorType.System)) != 0;
 			}
 		}
 
 		public bool IsNamedColor {
 			get{
-				return (colorType & (ColorType.Known|ColorType.Named)) != 0;
+				return (state & (short)(ColorType.Known|ColorType.Named)) != 0;
 			}
 		}
 
@@ -123,42 +129,38 @@ namespace System.Drawing
 		{
 			CheckARGBValues (alpha, red, green, blue);
 			Color color = new Color ();
-			color.colorType = ColorType.ARGB;
-			color.a = (byte) alpha;
-			color.r = (byte) red;
-			color.g = (byte) green;
-			color.b = (byte) blue;
-			color.myname = String.Empty;
+			color.state = (short) ColorType.ARGB;
+			color.value = ((uint) alpha << 24) + (red << 16) + (green << 8) + blue;
 			return color;
 		}
 
 		private static Color FromArgbNamed (int alpha, int red, int green, int blue, string name, KnownColor knownColor)
 		{
 			Color color = FromArgb (alpha, red, green, blue);
-			color.colorType = ColorType.Known|ColorType.Named;
+			color.state = (short) (ColorType.Known|ColorType.Named);
 			//color.issystemcolor = false; //???
-			color.myname = name;
+			color.name = name;
 			// FIXME: here happens SEGFAULT.
 			//color.knownColor = (KnownColor) Enum.Parse (typeof (KnownColor), name, false);
-			color.knownColor = knownColor;
+			color.knownColor = (short) knownColor;
 			return color;
 		}
 
 		internal static Color FromArgbSystem (int alpha, int red, int green, int blue, string name, KnownColor knownColor)
 		{
 			Color color = FromArgbNamed (alpha, red, green, blue, name, knownColor);
-			color.colorType |= ColorType.System;
+			color.state |= (short) ColorType.System;
 			return color;
 		}
 
 		public int ToArgb()
 		{
-			return a << 24 | r << 16 | g << 8 | b;
+			return (int) value;
 		} 
 
 		public static Color FromArgb (int alpha, Color baseColor)
 		{
-			return FromArgb (alpha, baseColor.r, baseColor.g, baseColor.b);
+			return FromArgb (alpha, baseColor.R, baseColor.G, baseColor.B);
 		}
 
 		public static Color FromArgb (int argb)
@@ -178,9 +180,9 @@ namespace System.Drawing
 #endif			
 				// This is what it returns!
 				Color d = FromArgb (0, 0, 0, 0);
-				d.myname = c.ToString ();
-				d.colorType |= ColorType.Named;
-				d.knownColor = c;
+				d.name = c.ToString ();
+				d.state |= (short) ColorType.Named;
+				d.knownColor = (short) c;
 				return d;
 			}
 			
@@ -245,8 +247,8 @@ namespace System.Drawing
 				if (c == null) {
 					// This is what it returns!
 					Color d = FromArgb (0, 0, 0, 0);
-					d.myname = colorName;
-					d.colorType |= ColorType.Named;
+					d.name = colorName;
+					d.state |= (short) ColorType.Named;
 					c = d;
 				}
 			}
@@ -296,13 +298,7 @@ namespace System.Drawing
 
 		public static bool operator == (Color colorA, Color colorB)
 		{
-			if (colorA.a != colorB.a)
-				return false;
-			if (colorA.r != colorB.r)
-				return false;
-            if (colorA.g != colorB.g)
-				return false;
-			if (colorA.b != colorB.b)
+			if (colorA.value != colorB.value)
 				return false;
 			if (colorA.IsNamedColor != colorB.IsNamedColor)
 				return false;
@@ -332,16 +328,16 @@ namespace System.Drawing
 		
 		public float GetBrightness ()
 		{
-			byte minval = Math.Min (r, Math.Min (g, b));
-			byte maxval = Math.Max (r, Math.Max (g, b));
+			byte minval = Math.Min (R, Math.Min (G, B));
+			byte maxval = Math.Max (R, Math.Max (G, B));
 	
 			return (float)(maxval + minval) / 510;
 		}
 
 		public float GetSaturation ()
 		{
-			byte minval = Math.Min (r, Math.Min (g, b));
-			byte maxval = Math.Max (r, Math.Max (g, b));
+			byte minval = (byte) Math.Min (R, Math.Min (G, B));
+			byte maxval = (byte) Math.Max (R, Math.Max (G, B));
 			
 			if (maxval == minval)
 					return 0.0f;
@@ -355,8 +351,11 @@ namespace System.Drawing
 
 		public float GetHue ()
 		{
-			byte minval = Math.Min (r, Math.Min (g, b));
-			byte maxval = Math.Max (r, Math.Max (g, b));
+			int r = R;
+			int g = G;
+			int b = B;
+			byte minval = (byte) Math.Min (r, Math.Min (g, b));
+			byte maxval = (byte) Math.Max (r, Math.Max (g, b));
 			
 			if (maxval == minval)
 					return 0.0f;
@@ -392,7 +391,7 @@ namespace System.Drawing
 		/// </remarks>
 		public KnownColor ToKnownColor ()
 		{
-			return knownColor;
+			return (KnownColor) knownColor;
 		}
 
 		/// <summary>
@@ -406,7 +405,7 @@ namespace System.Drawing
 		public bool IsEmpty 
 		{
 			get {
-				return colorType == ColorType.Empty;
+				return state == (short) ColorType.Empty;
 			}
 		}
 
@@ -421,7 +420,16 @@ namespace System.Drawing
 		public byte A
 		{
 			get {
-				return a;
+				// Optimization for known colors that were deserialized
+				// from an MS serialized stream.  
+				if (value == 0 && IsKnownColor) {
+					FillColorNames ();
+					object o = knownColors [knownColor];
+					if (o != null) {
+						value = ((Color) o).ToArgb ();
+					}
+				}
+				return (byte) ((value >> 24 & 0x0ff));
 			}
 		}
 
@@ -436,7 +444,16 @@ namespace System.Drawing
 		public byte R
 		{
 			get {
-				return r;
+				// Optimization for known colors that were deserialized
+				// from an MS serialized stream.  
+				if (value == 0 && IsKnownColor) {
+					FillColorNames ();
+					object o = knownColors [knownColor];
+					if (o != null) {
+						value = ((Color) o).ToArgb ();
+					}
+				}
+				return (byte) ((value >> 16 & 0x0ff));
 			}
 		}
 
@@ -451,7 +468,16 @@ namespace System.Drawing
 		public byte G
 		{
 			get {
-				return g;
+				// Optimization for known colors that were deserialized
+				// from an MS serialized stream.  
+				if (value == 0 && IsKnownColor) {
+					FillColorNames ();
+					object o = knownColors [knownColor];
+					if (o != null) {
+						value = ((Color) o).ToArgb ();
+					}
+				}
+				return (byte) ((value >> 8 & 0x0ff));
 			}
 		}
 
@@ -466,7 +492,16 @@ namespace System.Drawing
 		public byte B
 		{
 			get {
-				return b;
+				// Optimization for known colors that were deserialized
+				// from an MS serialized stream.  
+				if (value == 0 && IsKnownColor) {
+					FillColorNames ();
+					object o = knownColors [knownColor];
+					if (o != null) {
+						value = ((Color) o).ToArgb ();
+					}
+				}
+				return (byte) (value & 0x0ff);
 			}
 		}
 
@@ -513,7 +548,7 @@ namespace System.Drawing
 		
 		public override int GetHashCode ()
 		{
-			return ToArgb().GetHashCode () ^ Name.GetHashCode ();
+			return ((int) value) ^ Name.GetHashCode ();
 		}
 
 		/// <summary>
@@ -529,10 +564,11 @@ namespace System.Drawing
 			if (IsEmpty)
 				return "Color [Empty]";
 
-			if (myname != "")
-				return "Color [" + myname + "]";
+			// Use the property here, not the field.
+			if (IsNamedColor)
+				return "Color [" + Name + "]";
 
-			return String.Format ("Color [A={0}, R={1}, G={2}, B={3}]", a, r, g, b);
+			return String.Format ("Color [A={0}, R={1}, G={2}, B={3}]", A, R, G, B);
 		}
  
 		private static void CheckRGBValues (int red,int green,int blue)
