@@ -52,15 +52,22 @@ namespace System.Drawing
 		private bool disposed = false;
 		private static float defDpiX = 0;
 		private static float defDpiY = 0;
+#if NET_2_0
+		private IntPtr deviceContextHdc;
+#endif
 
+#if !NET_2_0
 		[ComVisible(false)]
+#endif
 		public delegate bool EnumerateMetafileProc (EmfPlusRecordType recordType,
 							    int flags,
 							    int dataSize,
 							    IntPtr data,
 							    PlayRecordCallback callbackData);
 		
+#if !NET_2_0
 		[ComVisible (false)]
+#endif
 		public delegate bool DrawImageAbort (IntPtr callbackData);
 
 		internal Graphics (IntPtr nativeGraphics)
@@ -113,33 +120,33 @@ namespace System.Drawing
 			throw new NotImplementedException ();
 		}
 
-		
 		public GraphicsContainer BeginContainer ()
 		{
-			int state;
+			uint state;
 			Status status;
 			status = GDIPlus.GdipBeginContainer2 (nativeObject, out state);
         		GDIPlus.CheckStatus (status);
 
                         return new GraphicsContainer(state);
 		}
-		
+
+		[MonoTODO ("rectangles and unit aren't supported in libgdiplus")]		
 		public GraphicsContainer BeginContainer (Rectangle dstrect, Rectangle srcrect, GraphicsUnit unit)
 		{
-			int state;
+			uint state;
 			Status status;
-			status = GDIPlus.GdipBeginContainerI (nativeObject, dstrect, srcrect, unit, out state);
+			status = GDIPlus.GdipBeginContainerI (nativeObject, ref dstrect, ref srcrect, unit, out state);
 			GDIPlus.CheckStatus (status);
 
 			return new GraphicsContainer (state);
 		}
 
-		
+		[MonoTODO ("rectangles and unit aren't supported in libgdiplus")]		
 		public GraphicsContainer BeginContainer (RectangleF dstrect, RectangleF srcrect, GraphicsUnit unit)
 		{
-			int state;
+			uint state;
 			Status status;
-			status = GDIPlus.GdipBeginContainer (nativeObject, dstrect, srcrect, unit, out state);
+			status = GDIPlus.GdipBeginContainer (nativeObject, ref dstrect, ref srcrect, unit, out state);
 			GDIPlus.CheckStatus (status);
 
 			return new GraphicsContainer (state);
@@ -483,7 +490,6 @@ namespace System.Drawing
 			GDIPlus.CheckStatus (status);
 		}
 		
-		
 		public void DrawCurve (Pen pen, PointF [] points, int offset, int numberOfSegments)
 		{
 			if (pen == null)
@@ -512,7 +518,6 @@ namespace System.Drawing
 			GDIPlus.CheckStatus (status);
 		}
 
-		
 		public void DrawCurve (Pen pen, PointF [] points, int offset, int numberOfSegments, float tension)
 		{
 			if (pen == null)
@@ -561,20 +566,23 @@ namespace System.Drawing
 
 		public void DrawIcon (Icon icon, Rectangle targetRect)
 		{
-			Image img = icon.ToBitmap ();
-			DrawImage (img, targetRect);
+			using (Image img = icon.ToBitmap ()) {
+				DrawImage (img, targetRect);
+			}
 		}
 
 		public void DrawIcon (Icon icon, int x, int y)
 		{
-			Image img = icon.ToBitmap ();
-			DrawImage (img, x, y);
+			using (Image img = icon.ToBitmap ()) {
+				DrawImage (img, x, y);
+			}
 		}
 
 		public void DrawIconUnstretched (Icon icon, Rectangle targetRect)
 		{
-			Image img = icon.ToBitmap ();
-			DrawImageUnscaled (img, targetRect);
+			using (Image img = icon.ToBitmap ()) {
+				DrawImageUnscaled (img, targetRect);
+			}
 		}
 		
 		public void DrawImage (Image image, RectangleF rect)
@@ -1169,8 +1177,10 @@ namespace System.Drawing
 
 		public void EndContainer (GraphicsContainer container)
 		{
+#if NET_2_0
 			if (container == null)
 				throw new ArgumentNullException ("container");
+#endif
 			Status status = GDIPlus.GdipEndContainer(nativeObject, container.NativeObject);
 			GDIPlus.CheckStatus (status);
 		}
@@ -1606,7 +1616,9 @@ namespace System.Drawing
 		{
 			if (brush == null)
 				throw new ArgumentNullException ("brush");
-			
+			if (rects == null)
+				throw new ArgumentNullException ("rects");
+
 			Status status = GDIPlus.GdipFillRectanglesI (nativeObject, brush.nativeObject, rects, rects.Length);
 			GDIPlus.CheckStatus (status);
 		}
@@ -1615,7 +1627,9 @@ namespace System.Drawing
 		{
 			if (brush == null)
 				throw new ArgumentNullException ("brush");
-			
+			if (rects == null)
+				throw new ArgumentNullException ("rects");
+
 			Status status = GDIPlus.GdipFillRectangles (nativeObject, brush.nativeObject, rects, rects.Length);
 			GDIPlus.CheckStatus (status);
 		}
@@ -1760,6 +1774,9 @@ namespace System.Drawing
 		{
 			IntPtr hdc;
 			GDIPlus.CheckStatus (GDIPlus.GdipGetDC (this.nativeObject, out hdc));
+#if NET_2_0
+			deviceContextHdc = hdc;
+#endif
 			return hdc;
 		}
 
@@ -2023,13 +2040,22 @@ namespace System.Drawing
 		{
 			Status status = GDIPlus.GdipReleaseDC (nativeObject, hdc);
 			GDIPlus.CheckStatus (status);
-		}
 #if NET_2_0
-		public void ReleaseHdc()
+			if (hdc == deviceContextHdc)
+				deviceContextHdc = IntPtr.Zero;
+#endif
+		}
+
+#if NET_2_0
+		public void ReleaseHdc ()
 		{
-		      
+			if (deviceContextHdc == IntPtr.Zero)
+				throw new ArgumentException ("Invalid Handle");
+
+			ReleaseHdc (deviceContextHdc);
 		}
 #endif
+
 		[MonoTODO]
 #if NET_2_0
 		[EditorBrowsable (EditorBrowsableState.Never)]
@@ -2346,6 +2372,7 @@ namespace System.Drawing
 			}
 		}
 
+		[MonoTODO ("not supported by libgdiplus")]
 		public PixelOffsetMode PixelOffsetMode {
 			get {
 			        PixelOffsetMode pixelOffset = PixelOffsetMode.Invalid;
@@ -2389,6 +2416,7 @@ namespace System.Drawing
 			}
 		}
 
+		[MonoTODO ("not supported by libgdiplus")]
 		public int TextContrast {
 			get {	
                                 int contrast;
@@ -2444,6 +2472,14 @@ namespace System.Drawing
                                 return rect;
 			}
 		}
+
+#if NET_2_0
+		[MonoTODO]
+		[EditorBrowsable (EditorBrowsableState.Never)]
+		public object GetContextInfo ()
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 	}
 }
-
