@@ -195,7 +195,7 @@ namespace System.Drawing
 					height = iconSize.Height;
 				}
 			}
-			if (dataStream != null && width != 0 && height != 0) {
+			if ((dataStream != null) && (width == height)) {
 				dataStream.Seek (0, SeekOrigin.Begin);
 				InitFromStreamWithSize (dataStream, width, height);
 			}
@@ -358,7 +358,10 @@ namespace System.Drawing
 					break;
 				}
 
-				case 24:
+				case 24: {
+					bmp = new Bitmap(bih.biWidth, biHeight, PixelFormat.Format24bppRgb);
+					break;
+				}
 				case 32: {	// 32bpp
 					bmp = new Bitmap(bih.biWidth, biHeight, PixelFormat.Format32bppArgb);
 					break;
@@ -373,8 +376,9 @@ namespace System.Drawing
 				pal = bmp.Palette;				// Managed palette
 
 				for (int i = 0; i < ii.iconColors.Length; i++) {
-					pal.Entries[i] = Color.FromArgb((int)ii.iconColors[i] & unchecked((int)0xff000000));
+					pal.Entries[i] = Color.FromArgb((int)ii.iconColors[i] | unchecked((int)0xff000000));
 				}
+				bmp.Palette = pal;
 			}
 
 			bytesPerLine = (int)((((bih.biWidth * bih.biBitCount) + 31) & ~31) >> 3);
@@ -386,12 +390,14 @@ namespace System.Drawing
 			
 			bmp.UnlockBits(bits);
 
-			bmp = new Bitmap(bmp);	// This makes a 32bpp image out of an indexed one
+			bmp = new Bitmap (bmp);// This makes a 32bpp image out of an indexed one
+
 			// Apply the mask to make properly transparent
+			bytesPerLine = (int)((((bih.biWidth) + 31) & ~31) >> 3);
 			for (int y = 0; y < biHeight; y++) {
 				for (int x = 0; x < bih.biWidth / 8; x++) {
 					for (int bit = 7; bit >= 0; bit--) {
-						if (((ii.iconAND[y * bih.biWidth / 8 +x] >> bit) & 1) != 0) {
+						if (((ii.iconAND[y * bytesPerLine +x] >> bit) & 1) != 0) {
 							bmp.SetPixel(x*8 + 7-bit, biHeight - y - 1, Color.Transparent);
 						}
 					}
@@ -448,7 +454,7 @@ namespace System.Drawing
 				throw new System.ArgumentException ("The argument 'stream' must be a picture that can be used as a Icon", "stream");
 			
 			BinaryReader reader = new BinaryReader (stream);
-            
+
 			//iconDir = new IconDir ();
 			iconDir.idReserved = reader.ReadUInt16();
 			if (iconDir.idReserved != 0) //must be 0
@@ -476,7 +482,7 @@ namespace System.Drawing
 				ide.imageOffset = reader.ReadUInt32 ();
 				iconDir.idEntries [i] = ide;
 				//is this is the best fit??
-				if (!sizeObtained)   
+				if (!sizeObtained)
 					if (ide.height==height && ide.width==width) {
 						this.id = (ushort) i;
 						sizeObtained = true;
@@ -551,16 +557,17 @@ namespace System.Drawing
 				//Determine the XOR array Size
 				int xorSize = numBytesPerLine * iconHeight;
 				iidata.iconXOR = new byte [xorSize];
-				for (int i=0; i<xorSize; i++)
-					iidata.iconXOR[i] = bihReader.ReadByte();		
+				int nread = bihReader.Read (iidata.iconXOR, 0, xorSize);
+				if (nread != xorSize)
+					throw new Exception ("Short file");
 				
 				//Determine the AND array size
-				//For this i subtract the current position from the length.
-				//ugly hack...
-				int andSize = (int) (bihReader.BaseStream.Length - bihReader.BaseStream.Position);
+				numBytesPerLine = (int)((((bih.biWidth) + 31) & ~31) >> 3);
+				int andSize = numBytesPerLine * iconHeight;
 				iidata.iconAND = new byte [andSize];
-				for (int i=0; i<andSize; i++)
-					iidata.iconAND[i] = bihReader.ReadByte();		
+				nread = bihReader.Read (iidata.iconAND, 0, andSize);
+				if (nread != andSize)
+					throw new Exception ("Short file");
 				
 				imageData [j] = iidata;
 				bihReader.Close();
