@@ -37,7 +37,6 @@ namespace System.Drawing
 #if !NET_2_0
 	[ComVisible (false)]
 #endif
-	[MonoTODO ("Region are limited to rectangles when using libgdiplus")]
 	public sealed class Region : MarshalByRefObject, IDisposable
 	{
                 private IntPtr nativeRegion = IntPtr.Zero;
@@ -78,6 +77,8 @@ namespace System.Drawing
 			if (rgnData == null)
 				throw new ArgumentNullException ("rgnData");
 			// a NullReferenceException can be throw for rgnData.Data.Length (if rgnData.Data is null) just like MS
+			if (rgnData.Data.Length == 0)
+				throw new ArgumentException ("rgnData");
 			Status status = GDIPlus.GdipCreateRegionRgnData (rgnData.Data, rgnData.Data.Length, out nativeRegion);
 			GDIPlus.CheckStatus (status);
 		}
@@ -86,7 +87,6 @@ namespace System.Drawing
 		// Union
 		//
 
-		[MonoTODO ("GdipCombineRegionPath isn't implemented in libgdiplus")]
 		public void Union (GraphicsPath path)
 		{
 			if (path == null)
@@ -518,21 +518,32 @@ namespace System.Drawing
 		{
 			if (hrgn == IntPtr.Zero)
 				throw new ArgumentException ("hrgn");
-			return new Region (hrgn);
+
+			IntPtr handle;
+			Status status = GDIPlus.GdipCreateRegionHrgn (hrgn, out handle);
+			GDIPlus.CheckStatus (status);
+
+			return new Region (handle);
 		}
 		
 		
-		public IntPtr GetHrgn(Graphics g)
+		public IntPtr GetHrgn (Graphics g)
 		{
-#if false
-			//
 			// Our WindowsForms implementation uses null to avoid
 			// creating a Graphics context when not needed
-			//
+#if false
+			// this is MS behaviour
 			if (g == null)
 				throw new ArgumentNullException ("g");
+#else
+			// this is an hack for MWF (libgdiplus would reject that)
+			if (g == null)
+				return nativeRegion;
 #endif
-			return nativeRegion;
+			IntPtr handle = IntPtr.Zero;
+			Status status = GDIPlus.GdipGetRegionHRgn (nativeRegion, g.NativeObject, ref handle);
+			GDIPlus.CheckStatus (status);
+			return handle;
 		}
 		
 		
@@ -583,7 +594,6 @@ namespace System.Drawing
 			return rects;			
 		}		
 
-		[MonoTODO ("GdipTransformRegion isn't implemented in libgdiplus")]		
 		public void Transform(Matrix matrix)
 		{
 			if (matrix == null)
