@@ -479,7 +479,6 @@ namespace MonoTests.System.Drawing.Drawing2D {
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void AddLines_Point_1 ()
 		{
 			GraphicsPath gp = new GraphicsPath ();
@@ -514,7 +513,6 @@ namespace MonoTests.System.Drawing.Drawing2D {
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void AddLines_PointF_1 ()
 		{
 			GraphicsPath gp = new GraphicsPath ();
@@ -827,12 +825,12 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			Assert.AreEqual (10, path.PathTypes.Length, "PathPoints");
 			Assert.AreEqual (10, path.PathData.Points.Length, "PathData");
 
-			// GetBounds (well GdipGetPathWorldBounds) isn't implemented
+			// GetBounds (well GdipGetPathWorldBounds) isn't very precise with curves
 			RectangleF rect = path.GetBounds ();
-			Assert.AreEqual (0.8333333f, rect.X, "Bounds.X");
-			Assert.AreEqual (0.8333333f, rect.Y, "Bounds.Y");
-			Assert.AreEqual (2.33333278f, rect.Width, "Bounds.Width");
-			Assert.AreEqual (2.33333278f, rect.Height, "Bounds.Height");
+			Assert.AreEqual (0.8333333f, rect.X, 0.2f, "Bounds.X");
+			Assert.AreEqual (0.8333333f, rect.Y, 0.2f, "Bounds.Y");
+			Assert.AreEqual (2.33333278f, rect.Width, 0.4f, "Bounds.Width");
+			Assert.AreEqual (2.33333278f, rect.Height, 0.4f, "Bounds.Height");
 
 			Assert.AreEqual (0, path.PathData.Types[0], "PathData.Types[0]");
 			for (int i = 1; i < 9; i++)
@@ -980,6 +978,27 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			GraphicsPath gp = new GraphicsPath ();
 			gp.AddCurve (new Point[2] { new Point (1, 1), new Point (2, 2) });
 			CheckCurve (gp);
+			// note: GdipAddPathCurveI allows adding a "curve" with only 2 points (a.k.a. a line ;-)
+			gp.Dispose ();
+		}
+
+		[Test]
+		public void AddCurve_Point_2_Tension ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new Point[2] { new Point (1, 1), new Point (2, 2) }, 1.0f);
+			CheckCurve (gp);
+			// note: GdipAddPathCurve2I allows adding a "curve" with only 2 points (a.k.a. a line ;-)
+			gp.Dispose ();
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void AddCurve3_Point_2 ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new Point[2] { new Point (1, 1), new Point (2, 2) }, 0, 2, 0.5f);
+			// adding only two points isn't supported by GdipAddCurve3I
 		}
 
 		[Test]
@@ -1011,6 +1030,135 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			GraphicsPath gp = new GraphicsPath ();
 			gp.AddCurve (new PointF[2] { new PointF (1f, 1f), new PointF (2f, 2f) });
 			CheckCurve (gp);
+			// note: GdipAddPathCurve allows adding a "curve" with only 2 points (a.k.a. a line ;-)
+			gp.Dispose ();
+		}
+
+		[Test]
+		public void AddCurve_PoinFt_2_Tension ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[2] { new PointF (1f, 1f), new PointF (2f, 2f) }, 1.0f);
+			CheckCurve (gp);
+			// note: GdipAddPathCurve2 allows adding a "curve" with only 2 points (a.k.a. a line ;-)
+			gp.Dispose ();
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void AddCurve3_PointF_2 ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[2] { new PointF (1f, 1f), new PointF (2f, 2f) }, 0, 2, 0.5f);
+			// adding only two points isn't supported by GdipAddCurve3
+		}
+
+		[Test]
+		public void AddCurve_LargeTension ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[3] { new PointF (1f, 1f), new PointF (0f, 20f), new PointF (20f, 0f) }, 0, 2, Single.MaxValue);
+			Assert.AreEqual (7, gp.PointCount, "PointCount");
+			gp.Dispose ();
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void AddCurve_ZeroSegments ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[2] { new PointF (1f, 1f), new PointF (2f, 2f) }, 0, 0, 0.5f);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void AddCurve_NegativeSegments ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[2] { new PointF (1f, 1f), new PointF (2f, 2f) }, 0, -1, 0.5f);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void AddCurve_OffsetTooLarge ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[3] { new PointF (1f, 1f), new PointF (0f, 20f), new PointF (20f, 0f) }, 1, 2, 0.5f);
+		}
+
+		[Test]
+		public void AddCurve_Offset ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddCurve (new PointF[4] { new PointF (1f, 1f), new PointF (0f, 20f), new PointF (20f, 0f), new PointF (0f, 10f) }, 1, 2, 0.5f);
+			Assert.AreEqual (7, gp.PointCount, "PointCount");
+			gp.Dispose ();
+		}
+
+		private FontFamily GetFontFamily ()
+		{
+			try {
+				return FontFamily.GenericMonospace;
+			}
+			catch (ArgumentException) {
+				Assert.Ignore ("GenericMonospace FontFamily couldn't be found");
+				return null;
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (NullReferenceException))]
+		public void AddString_NullString ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			FontFamily ff = GetFontFamily ();
+			gp.AddString (null, ff, 0, 10, new Point (10, 10), StringFormat.GenericDefault);
+		}
+
+		[Test]
+		public void AddString_EmptyString ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			FontFamily ff = GetFontFamily ();
+			gp.AddString (String.Empty, ff, 0, 10, new Point (10, 10), StringFormat.GenericDefault);
+			Assert.AreEqual (0, gp.PointCount, "PointCount");
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void AddString_NullFontFamily ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddString ("mono", null, 0, 10, new Point (10, 10), StringFormat.GenericDefault);
+		}
+
+		[Test]
+		public void AddString_NegativeSize ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			FontFamily ff = GetFontFamily ();
+			gp.AddString ("mono", ff, 0, -10, new Point (10, 10), StringFormat.GenericDefault);
+			Assert.IsTrue (gp.PointCount > 0, "PointCount");
+		}
+
+		[Test]
+		[Category ("NotWorking")] // StringFormat not yet supported in libgdiplus
+		public void AddString_StringFormat ()
+		{
+			FontFamily ff = GetFontFamily ();
+			// null maps to ?
+			GraphicsPath gp1 = new GraphicsPath ();
+			gp1.AddString ("mono", ff, 0, 10, new RectangleF (10, 10, 10, 10), null);
+
+			// StringFormat.GenericDefault
+			GraphicsPath gp2 = new GraphicsPath ();
+			gp2.AddString ("mono", ff, 0, 10, new RectangleF (10, 10, 10, 10), StringFormat.GenericDefault);
+			Assert.AreEqual (gp1.PointCount, gp2.PointCount, "GenericDefault");
+
+			// StringFormat.GenericTypographic
+			GraphicsPath gp3 = new GraphicsPath ();
+			gp3.AddString ("mono", ff, 0, 10, new RectangleF (10, 10, 10, 10), StringFormat.GenericTypographic);
+			Assert.IsFalse (gp1.PointCount == gp3.PointCount, "GenericTypographic");
 		}
 
 		[Test]
@@ -1065,7 +1213,7 @@ namespace MonoTests.System.Drawing.Drawing2D {
 		}
 
 		[Test]
-		[Category ("NotWorking")] // bounds+pen support is missing in libgdiplus
+		[Category ("NotWorking")] // can't/wont duplicate the lack of precision
 		public void GetBounds_WithPen ()
 		{
 			Rectangle rect = new Rectangle (1, 1, 2, 2);
@@ -1074,18 +1222,20 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			gp.AddRectangle (rect);
 
 			RectangleF bounds = gp.GetBounds (null, p);
+			// those bounds doesn't make any sense (even visually)
+			// probably null gets mis-interpreted ???
 			Assert.AreEqual (-6.09999943f, bounds.X, "NullMatrix.Bounds.X");
 			Assert.AreEqual (-6.09999943f, bounds.Y, "NullMatrix.Bounds.Y");
 			Assert.AreEqual (16.1999989f, bounds.Width, "NullMatrix.Bounds.Width");
 			Assert.AreEqual (16.1999989f, bounds.Height, "NullMatrix.Bounds.Height");
 
 			Matrix m = new Matrix ();
-			// an empty matrix is different than a null matrix
 			bounds = gp.GetBounds (m, p);
 			Assert.AreEqual (-0.419999957f, bounds.X, "EmptyMatrix.Bounds.X");
 			Assert.AreEqual (-0.419999957f, bounds.Y, "EmptyMatrix.Bounds.Y");
 			Assert.AreEqual (4.83999968f, bounds.Width, "EmptyMatrix.Bounds.Width");
 			Assert.AreEqual (4.83999968f, bounds.Height, "EmptyMatrix.Bounds.Height");
+			// visually we can see the bounds just a pixel bigger than the rectangle
 
 			gp = new GraphicsPath ();
 			gp.AddRectangle (rect);
@@ -1103,6 +1253,66 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			Assert.AreEqual (3.000001f, bounds.Height, "WidenEmptyMatrix.Bounds.Height");
 		}
 
+		private void CheckPieBounds (RectangleF rect)
+		{
+			Assert.AreEqual (60.0f, rect.X, 1, "Bounds.X");
+			Assert.AreEqual (60.0f, rect.Y, 1, "Bounds.Y");
+			Assert.AreEqual (43.3f, rect.Width, 1, "Bounds.Width");
+			Assert.AreEqual (48.3f, rect.Height, 1, "Bounds.Height");
+		}
+
+		[Test]
+		public void GetBounds_Empty_Pie ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddPie (10, 10, 100, 100, 30, 45);
+			CheckPieBounds (gp.GetBounds ());
+			gp.Dispose ();
+		}
+
+		[Test]
+		public void GetBounds_Null_Pie ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddPie (10, 10, 100, 100, 30, 45);
+			CheckPieBounds (gp.GetBounds (null));
+			gp.Dispose ();
+		}
+
+		[Test]
+		public void GetBounds_MatrixEmpty_Pie ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddPie (10, 10, 100, 100, 30, 45);
+			CheckPieBounds (gp.GetBounds (new Matrix ()));
+			gp.Dispose ();
+		}
+
+		[Test]
+		public void GetBounds_NullNull_Pie ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddPie (10, 10, 100, 100, 30, 45);
+			CheckPieBounds (gp.GetBounds (null, null));
+			gp.Dispose ();
+		}
+
+		[Test]
+		public void GetBounds_Empty_ClosedCurve ()
+		{
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddClosedCurve (new Point[4] { new Point (20, 100), new Point (70, 10),
+				new Point (130, 200), new Point (180, 100) });
+#if false
+			// so far from reality that it's totally useless
+			Assert.AreEqual (1.666666f, rect.X, 0.00001, "Bounds.X");
+			Assert.AreEqual (-6.66666f, rect.Y, 1, "Bounds.Y");
+			Assert.AreEqual (196.6666f, rect.Width, 1, "Bounds.Width");
+			Assert.AreEqual (221.6666f, rect.Height, 1, "Bounds.Height");
+#endif
+			gp.Dispose ();
+		}
+
 		[Test]
 		[ExpectedException (typeof (ArgumentNullException))]
 		public void Transform_Null ()
@@ -1265,6 +1475,42 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			ComparePaths (path, clone);
 		}
 
+		private void CheckWrap (GraphicsPath path)
+		{
+			Assert.AreEqual (3, path.PointCount, "Count");
+
+			PointF[] pts = path.PathPoints;
+			Assert.AreEqual (0, pts[0].X, 1e-30, "0.X");
+			Assert.AreEqual (0, pts[0].Y, 1e-30, "0.Y");
+			Assert.AreEqual (0, pts[1].X, 1e-30, "1.X");
+			Assert.AreEqual (0, pts[1].Y, 1e-30, "1.Y");
+			Assert.AreEqual (0, pts[2].X, 1e-30, "2.X");
+			Assert.AreEqual (0, pts[2].Y, 1e-30, "2.Y");
+
+			byte[] types = path.PathTypes;
+			Assert.AreEqual (0, types[0], "0");
+			Assert.AreEqual (1, types[1], "1");
+			Assert.AreEqual (129, types[2], "2");
+		}
+
+		private void CheckWrapNaN (GraphicsPath path, bool closed)
+		{
+			Assert.AreEqual (3, path.PointCount, "Count");
+
+			PointF[] pts = path.PathPoints;
+			Assert.AreEqual (Single.NaN, pts[0].X, "0.X");
+			Assert.AreEqual (Single.NaN, pts[0].Y, "0.Y");
+			Assert.AreEqual (Single.NaN, pts[1].X, "1.X");
+			Assert.AreEqual (Single.NaN, pts[1].Y, "1.Y");
+			Assert.AreEqual (Single.NaN, pts[2].X, "2.X");
+			Assert.AreEqual (Single.NaN, pts[2].Y, "2.Y");
+
+			byte[] types = path.PathTypes;
+			Assert.AreEqual (0, types[0], "0");
+			Assert.AreEqual (1, types[1], "1");
+			Assert.AreEqual (closed ? 129 : 1, types[2], "2");
+		}
+
 		[Test]
 		[ExpectedException (typeof (ArgumentNullException))]
 		public void Warp_Null ()
@@ -1272,6 +1518,202 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			new GraphicsPath ().Warp (null, new RectangleF ());
 		}
 
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void Warp_NoPoints ()
+		{
+			new GraphicsPath ().Warp (new PointF[0], new RectangleF ());
+		}
+
+		[Test]
+		public void Wrap_NoPoint ()
+		{
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				Assert.AreEqual (0, gp.PointCount, "PointCount-1");
+
+				PointF[] pts = new PointF[1] { new PointF (0, 0) };
+				RectangleF r = new RectangleF (10, 20, 30, 40);
+				gp.Warp (pts, r, new Matrix ());
+				Assert.AreEqual (0, gp.PointCount, "PointCount-2");
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Wrap_SinglePoint ()
+		{
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddLines (new Point[1] { new Point (1, 1) });
+				// Special case - a line with a single point is valid
+				Assert.AreEqual (1, gp.PointCount, "PointCount-1");
+
+				PointF[] pts = new PointF[1] { new PointF (0, 0) };
+				RectangleF r = new RectangleF (10, 20, 30, 40);
+				gp.Warp (pts, r, new Matrix ());
+				Assert.AreEqual (0, gp.PointCount, "PointCount-2");
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Wrap_Line ()
+		{
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddLine (new Point (1, 1), new Point (20, 20));
+				Assert.AreEqual (2, gp.PointCount, "PointCount-1");
+
+				PointF[] pts = new PointF[1] { new PointF (0, 0) };
+				RectangleF r = new RectangleF (10, 20, 30, 40);
+				gp.Warp (pts, r, new Matrix ());
+				Assert.AreEqual (2, gp.PointCount, "PointCount-2");
+			}
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Warp_NullMatrix ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0,0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, null);
+			CheckWrap (path);
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Warp_EmptyMatrix ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, new Matrix ());
+			CheckWrap (path);
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Warp_Rectangle_Empty ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			path.Warp (pts, new RectangleF (), null);
+			CheckWrapNaN (path, true);
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Warp_Rectangle_NegativeWidthHeight ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, -30, -40);
+			path.Warp (pts, r, null);
+			Assert.AreEqual (3, path.PointCount, "Count");
+
+			pts = path.PathPoints;
+			Assert.AreEqual (1.131355e-39, pts[0].X, 1e40, "0.X");
+			Assert.AreEqual (-2.0240637E-33, pts[0].Y, 1e40, "0.Y");
+			Assert.AreEqual (1.070131E-39, pts[1].X, 1e40, "1.X");
+			Assert.AreEqual (-2.02406389E-33, pts[1].Y, 1e40, "1.Y");
+			Assert.AreEqual (3.669146E-40, pts[2].X, 1e40, "2.X");
+			Assert.AreEqual (-6.746879E-34, pts[2].Y, 1e40, "2.Y");
+			byte[] types = path.PathTypes;
+			Assert.AreEqual (0, types[0], "0");
+			Assert.AreEqual (1, types[1], "1");
+			Assert.AreEqual (129, types[2], "2");
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Warp_Matrix_NonInvertible ()
+		{
+			Matrix matrix = new Matrix (123, 24, 82, 16, 47, 30);
+			Assert.IsFalse (matrix.IsInvertible, "!IsInvertible");
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, matrix);
+
+			Assert.AreEqual (3, path.PointCount, "Count");
+			pts = path.PathPoints;
+			Assert.AreEqual (47, pts[0].X, "0.X");
+			Assert.AreEqual (30, pts[0].Y, "0.Y");
+			Assert.AreEqual (47, pts[1].X, "1.X");
+			Assert.AreEqual (30, pts[1].Y, "1.Y");
+			Assert.AreEqual (47, pts[2].X, "2.X");
+			Assert.AreEqual (30, pts[2].Y, "2.Y");
+			byte[] types = path.PathTypes;
+			Assert.AreEqual (0, types[0], "0");
+			Assert.AreEqual (1, types[1], "1");
+			Assert.AreEqual (129, types[2], "2");
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Warp_Bilinear ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, new Matrix (), WarpMode.Bilinear);
+			// note that the last point is no more closed
+			CheckWrapNaN (path, false);
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Warp_Perspective ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, new Matrix (), WarpMode.Perspective);
+			CheckWrap (path);
+		}
+
+		[Test]
+		public void Warp_Invalid ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, new Matrix (), (WarpMode) Int32.MinValue);
+			Assert.AreEqual (0, path.PointCount, "Count");
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Warp_Flatness_Negative ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, new Matrix (), WarpMode.Perspective, -1f);
+			CheckWrap (path);
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Warp_Flatness_OverOne ()
+		{
+			PointF[] pts = new PointF[1] { new PointF (0, 0) };
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			RectangleF r = new RectangleF (10, 20, 30, 40);
+			path.Warp (pts, r, new Matrix (), WarpMode.Perspective, 2.0f);
+			CheckWrap (path);
+		}
+
 		[Test]
 		public void SetMarkers_EmptyPath ()
 		{
@@ -1553,5 +1995,455 @@ namespace MonoTests.System.Drawing.Drawing2D {
 			Assert.AreEqual (1, types[path.PointCount - 2], "start/Line2");
 			Assert.AreEqual (1, types[path.PointCount - 1], "end/Line2");
 		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void Widen_Pen_Null ()
+		{
+			new GraphicsPath ().Widen (null);
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Widen_Pen ()
+		{
+			Pen pen = new Pen (Color.Blue);
+			GraphicsPath path = new GraphicsPath ();
+			path.AddRectangle (new Rectangle (1, 1, 2, 2));
+			Assert.AreEqual (4, path.PointCount, "Count-1");
+			path.Widen (pen);
+			Assert.AreEqual (12, path.PointCount, "Count-2");
+
+			PointF[] pts = path.PathPoints;
+			Assert.AreEqual (0.5, pts[0].X, 0.25, "0.X");
+			Assert.AreEqual (0.5, pts[0].Y, 0.25, "0.Y");
+			Assert.AreEqual (3.5, pts[1].X, 0.25, "1.X");
+			Assert.AreEqual (0.5, pts[1].Y, 0.25, "1.Y");
+			Assert.AreEqual (3.5, pts[2].X, 0.25, "2.X");
+			Assert.AreEqual (3.5, pts[2].Y, 0.25, "2.Y");
+			Assert.AreEqual (0.5, pts[3].X, 0.25, "3.X");
+			Assert.AreEqual (3.5, pts[3].Y, 0.25, "3.Y");
+			Assert.AreEqual (1.5, pts[4].X, 0.25, "4.X");
+			Assert.AreEqual (3.0, pts[4].Y, 0.25, "4.Y");
+			Assert.AreEqual (1.0, pts[5].X, 0.25, "5.X");
+			Assert.AreEqual (2.5, pts[5].Y, 0.25, "5.Y");
+			Assert.AreEqual (3.0, pts[6].X, 0.25, "6.X");
+			Assert.AreEqual (2.5, pts[6].Y, 0.25, "6.Y");
+			Assert.AreEqual (2.5, pts[7].X, 0.25, "7.X");
+			Assert.AreEqual (3.0, pts[7].Y, 0.25, "7.Y");
+			Assert.AreEqual (2.5, pts[8].X, 0.25, "8.X");
+			Assert.AreEqual (1.0, pts[8].Y, 0.25, "8.Y");
+			Assert.AreEqual (3.0, pts[9].X, 0.25, "9.X");
+			Assert.AreEqual (1.5, pts[9].Y, 0.25, "9.Y");
+			Assert.AreEqual (1.0, pts[10].X, 0.25, "10.X");
+			Assert.AreEqual (1.5, pts[10].Y, 0.25, "10.Y");
+			Assert.AreEqual (1.5, pts[11].X, 0.25, "11.X");
+			Assert.AreEqual (1.0, pts[11].Y, 0.25, "11.Y");
+
+			byte[] types = path.PathTypes;
+			Assert.AreEqual (0, types[0], "0");
+			Assert.AreEqual (1, types[1], "1");
+			Assert.AreEqual (1, types[2], "2");
+			Assert.AreEqual (129, types[3], "3");
+			Assert.AreEqual (0, types[4], "4");
+			Assert.AreEqual (1, types[5], "5");
+			Assert.AreEqual (1, types[6], "6");
+			Assert.AreEqual (1, types[7], "7");
+			Assert.AreEqual (1, types[8], "8");
+			Assert.AreEqual (1, types[9], "9");
+			Assert.AreEqual (1, types[10], "10");
+			Assert.AreEqual (129, types[11], "11");
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void Widen_Pen_Null_Matrix ()
+		{
+			new GraphicsPath ().Widen (null, new Matrix ());
+		}
+
+		[Test]
+#if ONLY_1_1
+		[ExpectedException (typeof (OutOfMemoryException))]
+#endif
+		public void Widen_NoPoint ()
+		{
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				Assert.AreEqual (0, gp.PointCount, "PointCount-1");
+				Pen pen = new Pen (Color.Blue);
+				gp.Widen (pen);
+				Assert.AreEqual (0, gp.PointCount, "PointCount-2");
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (OutOfMemoryException))]
+		public void Widen_SinglePoint ()
+		{
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddLines (new Point[1] { new Point (1, 1) });
+				// Special case - a line with a single point is valid
+				Assert.AreEqual (1, gp.PointCount, "PointCount");
+				gp.Widen (Pens.Red);
+				// oops ;-)
+			}
+		}
+
+		private void CheckWiden3 (GraphicsPath path)
+		{
+			PointF[] pts = path.PathPoints;
+			Assert.AreEqual (4.2, pts[0].X, 0.25, "0.X");
+			Assert.AreEqual (4.5, pts[0].Y, 0.25, "0.Y");
+			Assert.AreEqual (15.8, pts[1].X, 0.25, "1.X");
+			Assert.AreEqual (4.5, pts[1].Y, 0.25, "1.Y");
+			Assert.AreEqual (10.0, pts[2].X, 0.25, "2.X");
+			Assert.AreEqual (16.1, pts[2].Y, 0.25, "2.Y");
+			Assert.AreEqual (10.4, pts[3].X, 0.25, "3.X");
+			Assert.AreEqual (14.8, pts[3].Y, 0.25, "3.Y");
+			Assert.AreEqual (9.6, pts[4].X, 0.25, "4.X");
+			Assert.AreEqual (14.8, pts[4].Y, 0.25, "4.Y");
+			Assert.AreEqual (14.6, pts[5].X, 0.25, "7.X");
+			Assert.AreEqual (4.8, pts[5].Y, 0.25, "7.Y");
+			Assert.AreEqual (15.0, pts[6].X, 0.25, "5.X");
+			Assert.AreEqual (5.5, pts[6].Y, 0.25, "5.Y");
+			Assert.AreEqual (5.0, pts[7].X, 0.25, "6.X");
+			Assert.AreEqual (5.5, pts[7].Y, 0.25, "6.Y");
+			Assert.AreEqual (5.4, pts[8].X, 0.25, "8.X");
+			Assert.AreEqual (4.8, pts[8].Y, 0.25, "8.Y");
+
+			byte[] types = path.PathTypes;
+			Assert.AreEqual (0, types[0], "0");
+			Assert.AreEqual (1, types[1], "1");
+			Assert.AreEqual (129, types[2], "2");
+			Assert.AreEqual (0, types[3], "3");
+			Assert.AreEqual (1, types[4], "4");
+			Assert.AreEqual (1, types[5], "5");
+			Assert.AreEqual (1, types[6], "6");
+			Assert.AreEqual (1, types[7], "7");
+			Assert.AreEqual (129, types[8], "8");
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Widen_Pen_Matrix_Null ()
+		{
+			Pen pen = new Pen (Color.Blue);
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			path.Widen (pen, null);
+			Assert.AreEqual (9, path.PointCount, "Count");
+			CheckWiden3 (path);
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Widen_Pen_Matrix_Empty ()
+		{
+			Pen pen = new Pen (Color.Blue);
+			GraphicsPath path = new GraphicsPath ();
+			path.AddPolygon (new Point[3] { new Point (5, 5), new Point (15, 5), new Point (10, 15) });
+			path.Widen (pen, new Matrix ());
+			Assert.AreEqual (9, path.PointCount, "Count");
+			CheckWiden3 (path);
+		}
+
+		[Test]
+		[Ignore ("results aren't always constant and differs from 1.x and 2.0")]
+		public void Widen_Pen_Matrix_NonInvertible ()
+		{
+			Matrix matrix = new Matrix (123, 24, 82, 16, 47, 30);
+			Assert.IsFalse (matrix.IsInvertible, "!IsInvertible");
+			GraphicsPath path = new GraphicsPath ();
+			path.Widen (new Pen (Color.Blue), matrix);
+			Assert.AreEqual (0, path.PointCount, "Points");
+		}
+
+		private void CheckWidenedBounds (string message, GraphicsPath gp, Matrix m)
+		{
+			RectangleF bounds = gp.GetBounds (m);
+			Assert.AreEqual (0.5f, bounds.X, 0.00001f, message + ".Bounds.X");
+			Assert.AreEqual (0.5f, bounds.Y, 0.00001f, message + ".Bounds.Y");
+			Assert.AreEqual (3.0f, bounds.Width, 0.00001f, message + ".Bounds.Width");
+			Assert.AreEqual (3.0f, bounds.Height, 0.00001f, message + ".Bounds.Height");
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void Widen_Pen_SmallWidth ()
+		{
+			Matrix m = new Matrix ();
+			Rectangle rect = new Rectangle (1, 1, 2, 2);
+
+			// pen's smaller than 1.0 (width) are "promoted" to 1
+			Pen p = new Pen (Color.Aqua, 0);
+			GraphicsPath gp = new GraphicsPath ();
+			gp.AddRectangle (rect);
+			gp.Widen (p);
+			CheckWidenedBounds ("Width == 0, Null matrix", gp, null);
+			CheckWidenedBounds ("Width == 0, Empty matrix", gp, m);
+
+			p.Width = 0.5f;
+			gp = new GraphicsPath ();
+			gp.AddRectangle (rect);
+			gp.Widen (p);
+			CheckWidenedBounds ("Width == 0.5, Null matrix", gp, null);
+			CheckWidenedBounds ("Width == 0.5, Empty matrix", gp, m);
+
+			p.Width = 1.0f;
+			gp = new GraphicsPath ();
+			gp.AddRectangle (rect);
+			gp.Widen (p);
+			CheckWidenedBounds ("Width == 1.0, Null matrix", gp, null);
+			CheckWidenedBounds ("Width == 1.0, Empty matrix", gp, m);
+
+			p.Width = 1.1f;
+			gp = new GraphicsPath ();
+			gp.AddRectangle (rect);
+			gp.Widen (p);
+			RectangleF bounds = gp.GetBounds (m);
+			Assert.AreEqual (0.45f, bounds.X, 0.00001f, "1.1.Bounds.X");
+			Assert.AreEqual (0.45f, bounds.Y, 0.00001f, "1.1.Bounds.Y");
+			Assert.AreEqual (3.10f, bounds.Width, 0.00001f, "1.1.Bounds.Width");
+			Assert.AreEqual (3.10f, bounds.Height, 0.00001f, "1.1.Bounds.Height");
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void IsOutlineVisible_IntNull ()
+		{
+			new GraphicsPath ().IsOutlineVisible (1, 1, null);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void IsOutlineVisible_FloatNull ()
+		{
+			new GraphicsPath ().IsOutlineVisible (1.0f, 1.0f, null);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void IsOutlineVisible_PointNull ()
+		{
+			new GraphicsPath ().IsOutlineVisible (new Point (), null);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void IsOutlineVisible_PointFNull ()
+		{
+			new GraphicsPath ().IsOutlineVisible (new PointF (), null);
+		}
+
+		private void IsOutlineVisible_Line (Graphics graphics)
+		{
+			Pen p2 = new Pen (Color.Red, 3.0f);
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddLine (10, 1, 14, 1);
+				Assert.IsTrue (gp.IsOutlineVisible (10, 1, Pens.Red, graphics), "Int1");
+				Assert.IsTrue (gp.IsOutlineVisible (10, 2, p2, graphics), "Int2");
+				Assert.IsFalse (gp.IsOutlineVisible (10, 2, Pens.Red, graphics), "Int3");
+
+				Assert.IsTrue (gp.IsOutlineVisible (11.0f, 1.0f, Pens.Red, graphics), "Float1");
+				Assert.IsTrue (gp.IsOutlineVisible (11.0f, 1.0f, p2, graphics), "Float2");
+				Assert.IsFalse (gp.IsOutlineVisible (11.0f, 2.0f, Pens.Red, graphics), "Float3");
+
+				Point pt = new Point (12, 2);
+				Assert.IsFalse (gp.IsOutlineVisible (pt, Pens.Red, graphics), "Point1");
+				Assert.IsTrue (gp.IsOutlineVisible (pt, p2, graphics), "Point2");
+				pt.Y = 1;
+				Assert.IsTrue (gp.IsOutlineVisible (pt, Pens.Red, graphics), "Point3");
+
+				PointF pf = new PointF (13.0f, 2.0f);
+				Assert.IsFalse (gp.IsOutlineVisible (pf, Pens.Red, graphics), "PointF1");
+				Assert.IsTrue (gp.IsOutlineVisible (pf, p2, graphics), "PointF2");
+				pf.Y = 1;
+				Assert.IsTrue (gp.IsOutlineVisible (pf, Pens.Red, graphics), "PointF3");
+			}
+			p2.Dispose ();
+		}
+
+		[Test]
+		public void IsOutlineVisible_Line_WithoutGraphics ()
+		{
+			IsOutlineVisible_Line (null);
+		}
+
+		[Test]
+		public void IsOutlineVisible_Line_WithGraphics_Inside ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					IsOutlineVisible_Line (g);
+				}
+			}
+		}
+
+		[Test]
+		public void IsOutlineVisible_Line_WithGraphics_Outside ()
+		{
+			using (Bitmap bitmap = new Bitmap (5, 5)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					IsOutlineVisible_Line (g);
+				}
+				// graphics "seems" ignored as the line is outside the bitmap!
+			}
+		}
+
+		// docs ways the point is in world coordinates and that the graphics transform 
+		// should be applied
+
+		[Test]
+		public void IsOutlineVisible_Line_WithGraphics_Transform ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.Transform = new Matrix (2, 0, 0, 2, 50, -50);
+					IsOutlineVisible_Line (g);
+				}
+				// graphics still "seems" ignored (Transform)
+			}
+		}
+
+		[Test]
+		public void IsOutlineVisible_Line_WithGraphics_PageUnit ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.PageUnit = GraphicsUnit.Millimeter;
+					IsOutlineVisible_Line (g);
+				}
+				// graphics still "seems" ignored (PageUnit)
+			}
+		}
+
+		[Test]
+		public void IsOutlineVisible_Line_WithGraphics_PageScale ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.PageScale = 2.0f;
+					IsOutlineVisible_Line (g);
+				}
+				// graphics still "seems" ignored (PageScale)
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void IsOutlineVisible_Line_WithGraphics ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.Transform = new Matrix (2, 0, 0, 2, 50, -50);
+					g.PageUnit = GraphicsUnit.Millimeter;
+					g.PageScale = 2.0f;
+					using (GraphicsPath gp = new GraphicsPath ()) {
+						gp.AddLine (10, 1, 14, 1);
+						Assert.IsFalse (gp.IsOutlineVisible (10, 1, Pens.Red, g), "Int1");
+					}
+				}
+				// graphics ISN'T ignored (Transform+PageUnit+PageScale)
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")] // looks buggy - reported to MS as FDBK50868
+		public void IsOutlineVisible_Line_End ()
+		{
+			// horizontal line
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddLine (10, 1, 14, 1);
+				Assert.IsFalse (gp.IsOutlineVisible (14, 1, Pens.Red, null), "Int1h");
+				Assert.IsFalse (gp.IsOutlineVisible (13.5f, 1.0f, Pens.Red, null), "Float1h");
+				Assert.IsTrue (gp.IsOutlineVisible (13.4f, 1.0f, Pens.Red, null), "Float2h");
+				Assert.IsFalse (gp.IsOutlineVisible (new Point (14, 1), Pens.Red, null), "Point1h");
+				Assert.IsFalse (gp.IsOutlineVisible (new PointF (13.5f, 1.0f), Pens.Red, null), "PointF1h");
+				Assert.IsTrue (gp.IsOutlineVisible (new PointF (13.49f, 1.0f), Pens.Red, null), "PointF2h");
+			}
+			// vertical line
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddLine (1, 10, 1, 14);
+				Assert.IsFalse (gp.IsOutlineVisible (1, 14, Pens.Red, null), "Int1v");
+				Assert.IsFalse (gp.IsOutlineVisible (1.0f, 13.5f, Pens.Red, null), "Float1v");
+				Assert.IsTrue (gp.IsOutlineVisible (1.0f, 13.4f, Pens.Red, null), "Float2v");
+				Assert.IsFalse (gp.IsOutlineVisible (new Point (1, 14), Pens.Red, null), "Point1v");
+				Assert.IsFalse (gp.IsOutlineVisible (new PointF (1.0f, 13.5f), Pens.Red, null), "PointF1v");
+				Assert.IsTrue (gp.IsOutlineVisible (new PointF (1.0f, 13.49f), Pens.Red, null), "PointF2v");
+			}
+		}
+
+		private void IsOutlineVisible_Rectangle (Graphics graphics)
+		{
+			Pen p2 = new Pen (Color.Red, 3.0f);
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddRectangle (new Rectangle (10, 10, 20, 20));
+				Assert.IsTrue (gp.IsOutlineVisible (10, 10, Pens.Red, graphics), "Int1");
+				Assert.IsTrue (gp.IsOutlineVisible (10, 11, p2, graphics), "Int2");
+				Assert.IsFalse (gp.IsOutlineVisible (11, 11, Pens.Red, graphics), "Int3");
+
+				Assert.IsTrue (gp.IsOutlineVisible (11.0f, 10.0f, Pens.Red, graphics), "Float1");
+				Assert.IsTrue (gp.IsOutlineVisible (11.0f, 11.0f, p2, graphics), "Float2");
+				Assert.IsFalse (gp.IsOutlineVisible (11.0f, 11.0f, Pens.Red, graphics), "Float3");
+
+				Point pt = new Point (15, 10);
+				Assert.IsTrue (gp.IsOutlineVisible (pt, Pens.Red, graphics), "Point1");
+				Assert.IsTrue (gp.IsOutlineVisible (pt, p2, graphics), "Point2");
+				pt.Y = 15;
+				Assert.IsFalse (gp.IsOutlineVisible (pt, Pens.Red, graphics), "Point3");
+
+				PointF pf = new PointF (29.0f, 29.0f);
+				Assert.IsFalse (gp.IsOutlineVisible (pf, Pens.Red, graphics), "PointF1");
+				Assert.IsTrue (gp.IsOutlineVisible (pf, p2, graphics), "PointF2");
+				pf.Y = 31.0f;
+				Assert.IsTrue (gp.IsOutlineVisible (pf, p2, graphics), "PointF3");
+			}
+			p2.Dispose ();
+		}
+
+		[Test]
+		public void IsOutlineVisible_Rectangle_WithoutGraphics ()
+		{
+			IsOutlineVisible_Rectangle (null);
+		}
+
+		private void IsVisible_Rectangle (Graphics graphics)
+		{
+			using (GraphicsPath gp = new GraphicsPath ()) {
+				gp.AddRectangle (new Rectangle (10, 10, 20, 20));
+				Assert.IsFalse (gp.IsVisible (9, 9, graphics), "Int0");
+				Assert.IsTrue (gp.IsVisible (10, 10, graphics), "Int1");
+				Assert.IsTrue (gp.IsVisible (20, 20, graphics), "Int2");
+				Assert.IsTrue (gp.IsVisible (29, 29, graphics), "Int3");
+				Assert.IsFalse (gp.IsVisible (30, 29, graphics), "Int4");
+				Assert.IsFalse (gp.IsVisible (29, 30, graphics), "Int5");
+				Assert.IsFalse (gp.IsVisible (30, 30, graphics), "Int6");
+
+				Assert.IsFalse (gp.IsVisible (9.4f, 9.4f, graphics), "Float0");
+				Assert.IsTrue (gp.IsVisible (9.5f, 9.5f, graphics), "Float1");
+				Assert.IsTrue (gp.IsVisible (10f, 10f, graphics), "Float2");
+				Assert.IsTrue (gp.IsVisible (20f, 20f, graphics), "Float3");
+				Assert.IsTrue (gp.IsVisible (29.4f, 29.4f, graphics), "Float4");
+				Assert.IsFalse (gp.IsVisible (29.5f, 29.5f, graphics), "Float5");
+				Assert.IsFalse (gp.IsVisible (29.5f, 29.4f, graphics), "Float6");
+				Assert.IsFalse (gp.IsVisible (29.4f, 29.5f, graphics), "Float7");
+			}
+		}
+
+		[Test]
+		public void IsVisible_Rectangle_WithoutGraphics ()
+		{
+			IsVisible_Rectangle (null);
+		}
+
+		[Test]
+		public void IsVisible_Rectangle_WithGraphics ()
+		{
+			using (Bitmap bitmap = new Bitmap (40, 40)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					IsVisible_Rectangle (g);
+				}
+			}
+		}
 	}
 }
