@@ -1,3 +1,28 @@
+//
+// ColorConverter class testing unit
+//
+// Copyright (C) 2006 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -121,19 +146,6 @@ namespace MonoTests.System.Drawing {
 			Assert.AreEqual (0, color.G, "CF7#3");
 			Assert.AreEqual (0, color.B, "CF7#4");
 
-			color = (Color) colconv.ConvertFrom (null, CultureInfo.InvariantCulture,
-				"0x123");
-			Assert.AreEqual (255, color.A, "CF8#1");
-			Assert.AreEqual (17, color.R, "CF8#2");
-			Assert.AreEqual (34, color.G, "CF8#3");
-			Assert.AreEqual (51, color.B, "CF8#4");
-
-			color = (Color) colconv.ConvertFrom (null, CultureInfo.InvariantCulture,
-				"#123");
-			Assert.AreEqual (255, color.A, "CF9#1");
-			Assert.AreEqual (17, color.R, "CF9#2");
-			Assert.AreEqual (34, color.G, "CF9#3");
-			Assert.AreEqual (51, color.B, "CF9#4");
 
 			Assert.AreEqual (col, (Color) colconv.ConvertFrom (null,
 				CultureInfo.InvariantCulture, colStrInvariant), "CF#1");
@@ -144,7 +156,25 @@ namespace MonoTests.System.Drawing {
 			Assert.AreEqual (Color.Empty, colconv.ConvertFrom (" "), "CF#4");
 			Assert.AreEqual (Color.Red, colconv.ConvertFrom ("Red"), "CF#5");
 			Assert.AreEqual (Color.Red, colconv.ConvertFrom (" Red "), "CF#6");
+		}
 
+		[Test]
+		[NUnit.Framework.Category ("NotWorking")]
+		public void ConvertFrom_NotWorking ()
+		{
+			Color color = (Color) colconv.ConvertFrom (null, CultureInfo.InvariantCulture,
+				"0x123");
+			Assert.AreEqual (0, color.A, "CF8#1");
+			Assert.AreEqual (0, color.R, "CF8#2");
+			Assert.AreEqual (1, color.G, "CF8#3");
+			Assert.AreEqual (35, color.B, "CF8#4");
+
+			color = (Color) colconv.ConvertFrom (null, CultureInfo.InvariantCulture,
+				"#123");
+			Assert.AreEqual (0, color.A, "CF9#1");
+			Assert.AreEqual (0, color.R, "CF9#2");
+			Assert.AreEqual (1, color.G, "CF9#3");
+			Assert.AreEqual (35, color.B, "CF9#4");
 		}
 
 		[Test]
