@@ -165,11 +165,21 @@ namespace MonoTests.System.Drawing
 		}
 
 		[Test]
+#if NET_2_0
+		[Category ("NotWorking")]
+		public void GetHashCodeTest ()
+		{
+			Assert.AreEqual (0, Size.Empty.GetHashCode (), "GHC#2a");
+			Assert.IsTrue (new SizeF (0, 0).IsEmpty, "GHC#2b");
+			Assert.IsFalse (new SizeF (0, 0).GetHashCode () == 0, "GHC#2c"); // that doesn't work!
+		}
+#else
 		public void GetHashCodeTest ()
 		{
 			Assert.AreEqual (sz11_0.GetHashCode (), new SizeF (1.1f, 0).GetHashCode (), "GHC#1");
 			Assert.AreEqual (Size.Empty.GetHashCode (), new SizeF (0, 0).GetHashCode (), "GHC#2");
 		}
+#endif
 
 		[Test]
 		public void ToStringTest () {
