@@ -38,6 +38,10 @@ using System.Xml;
 using System.Xml.Schema;
 using System.Xml.Serialization;
 
+#if NET_2_0
+using System.Collections.Generic;
+#endif
+
 namespace System.Web.Services.Description
 {
 	[XmlFormatExtensionPoint ("Extensions")]
@@ -67,6 +71,9 @@ namespace System.Web.Services.Description
 		string targetNamespace;
 		Types types;
 		static ServiceDescriptionSerializer serializer;
+#if NET_2_0
+		internal static List<XmlAttribute> extensible_attributes;
+#endif
 
 		#endregion // Fields
 
@@ -125,7 +132,11 @@ namespace System.Web.Services.Description
 		}
 
 		[XmlIgnore]
-		public ServiceDescriptionFormatExtensionCollection Extensions { 	
+		public 
+#if NET_2_0
+		override
+#endif
+		ServiceDescriptionFormatExtensionCollection Extensions { 	
 			get { return extensions; }
 		}
 
@@ -292,7 +303,7 @@ namespace System.Web.Services.Description
 			info.Serializer.Serialize (writer, ext, ns);
 		}
 		
-		internal static void ReadExtension (XmlReader reader, object ob)
+		internal static void ReadExtension (XmlDocument doc, XmlReader reader, object ob)
 		{
 			ServiceDescriptionFormatExtensionCollection extensions = ExtensionManager.GetExtensionPoint (ob);
 			if (extensions != null)
@@ -305,9 +316,52 @@ namespace System.Web.Services.Description
 					return;
 				}
 			}
+
+			//No XmlFormatExtensionPoint attribute found
+
+#if NET_2_0
+			//Add to DocumentableItem.Extensions property
+			DocumentableItem item = ob as DocumentableItem;
+			if (item == null) {
+				reader.Skip ();
+				return;
+			}
+
+			item.Extensions.Add (doc.ReadNode (reader));
+#else
 			reader.Skip ();
+#endif
 		}
 
+#if NET_2_0
+		internal static void AddUnknownAttribute (XmlAttribute xa)
+		{
+			if (extensible_attributes == null)
+				extensible_attributes = new List<XmlAttribute> ();
+
+			extensible_attributes.Add (xa);
+		}
+
+		internal static void SetExtensibleAttributes (object ob)
+		{
+			if (extensible_attributes == null)
+				return;
+
+			DocumentableItem item = ob as DocumentableItem;
+			if (item == null)
+				return;
+
+#if TARGET_JVM //No support for 'new T()' in generics yet.
+			XmlAttribute[] array = new XmlAttribute[extensible_attributes.Count];
+			extensible_attributes.CopyTo (array);
+			item.ExtensibleAttributes = array;
+#else
+			item.ExtensibleAttributes = extensible_attributes.ToArray ();
+#endif
+			extensible_attributes.Clear ();
+		}
+#endif
+
 
 		#endregion
 
@@ -316,13 +370,13 @@ namespace System.Web.Services.Description
 			protected override void Serialize (object o, XmlSerializationWriter writer)
 			{
 				ServiceDescriptionWriterBase xsWriter = writer as ServiceDescriptionWriterBase;
-				xsWriter.WriteTree ((ServiceDescription)o);
+				xsWriter.WriteRoot_ServiceDescription (o);
 			}
 			
 			protected override object Deserialize (XmlSerializationReader reader)
 			{
 				ServiceDescriptionReaderBase xsReader = reader as ServiceDescriptionReaderBase;
-				return xsReader.ReadTree ();
+				return xsReader.ReadRoot_ServiceDescription ();
 			}
 			
 			protected override XmlSerializationWriter CreateWriter ()
