@@ -374,7 +374,7 @@ namespace System.Web.Services.Protocols
 			SoapWebClientAsyncResult sar = (SoapWebClientAsyncResult) ar;
 			InvokeCompletedEventArgs args = new InvokeCompletedEventArgs (sar.Exception, false, info.UserState, (object[]) sar.Result);
 			if (info.Context != null)
-				info.Context.SendOrPost (info.Callback, args);
+				info.Context.Send (info.Callback, args);
 			else
 				info.Callback (args);
 		}
