@@ -399,7 +399,7 @@ namespace System.Web.Compilation
 		void AddCodeForPropertyOrField (ControlBuilder builder, Type type, string var_name, string att, MemberInfo member, bool isDataBound)
 		{
 			CodeMemberMethod method = builder.method;
-			if (isDataBound) {
+			if (isDataBound && IsWritablePropertyOrField (member)) {
 				string dbMethodName = DataBoundProperty (builder, type, var_name, att);
 				AddEventAssign (method, "DataBinding", typeof (EventHandler), dbMethodName);
 				return;
@@ -476,6 +476,17 @@ namespace System.Web.Compilation
 			return member;
 		}
 
+		static bool IsWritablePropertyOrField (MemberInfo member)
+		{
+			PropertyInfo pi = member as PropertyInfo;
+			if (pi != null)
+				return pi.CanWrite;
+			FieldInfo fi = member as FieldInfo;
+			if (fi != null)
+				return !fi.IsInitOnly;
+			throw new ArgumentException ("Argument must be of PropertyInfo or FieldInfo type", "member");
+		}
+
 		bool ProcessPropertiesAndFields (ControlBuilder builder, MemberInfo member, string id,
 						string attValue, string prefix)
 		{
@@ -486,8 +497,6 @@ namespace System.Web.Compilation
 			Type type;
 			if (isPropertyInfo) {
 				type = ((PropertyInfo) member).PropertyType;
-				if (hyphen == -1 && ((PropertyInfo) member).CanWrite == false)
-					return false;
 			} else {
 				type = ((FieldInfo) member).FieldType;
 			}
@@ -496,6 +505,9 @@ namespace System.Web.Compilation
 #if NET_2_0
 				if (isDataBound) RegisterBindingInfo (builder, member.Name, ref attValue);
 #endif
+				if (!IsWritablePropertyOrField (member))
+					return false;
+
 				AddCodeForPropertyOrField (builder, type, member.Name, attValue, member, isDataBound);
 				return true;
 			}
@@ -633,7 +645,6 @@ namespace System.Web.Compilation
 					continue;
 #endif
 				CreateAssignStatementFromAttribute (builder, id);
-
 			}
 		}
 
@@ -1222,17 +1233,45 @@ namespace System.Web.Compilation
 			prop.GetStatements.Add (new CodeMethodReturnStatement (new CodePrimitiveExpression (false)));
 			mainClass.Members.Add (prop);
 		}
-		
-		CodeExpression GetExpressionFromString (Type type, string str, MemberInfo member)
+
+#if NET_2_0
+		protected virtual string HandleUrlProperty (string str, MemberInfo member)
 		{
-			if (type == typeof (string))
+			return str;
+		}
+#endif
+    
+		CodeExpression GetExpressionFromString (Type type, string str, MemberInfo member)
+		{			
+#if NET_2_0
+			Type origType = type;
+			bool wasNullable = false;
+			
+			if (type.IsGenericType && type.GetGenericTypeDefinition() == typeof(Nullable<>)) {
+				Type[] types = type.GetGenericArguments();
+				type = types[0]; // we're interested only in the first type here
+				wasNullable = true;
+			}
+#endif
+
+			if (type == typeof (string)) {
+#if NET_2_0
+				object[] urlAttr = member.GetCustomAttributes (typeof (UrlPropertyAttribute), true);
+				if (urlAttr.Length != 0)
+					str = HandleUrlProperty (str, member);
+#endif
 				return new CodePrimitiveExpression (str);
+			}
 
 			if (type == typeof (bool)) {
 				if (str == null || str == "" || InvariantCompareNoCase (str, "true"))
 					return new CodePrimitiveExpression (true);
 				else if (InvariantCompareNoCase (str, "false"))
 					return new CodePrimitiveExpression (false);
+#if NET_2_0
+				else if (wasNullable && InvariantCompareNoCase(str, "null"))
+					return new CodePrimitiveExpression (null);
+#endif
 				else
 					throw new ParseException (currentLocation,
 							"Value '" + str  + "' is not a valid boolean.");
