@@ -87,6 +87,7 @@ namespace System.Web.Configuration
 			return configPath + "/" + locatinSubPath;
 		}
 		
+		[MonoTODO ("this should consult with the build provider machinery")]
 		public virtual Type GetConfigType (string typeName, bool throwOnError)
 		{
 			Type type = Type.GetType (typeName);
@@ -116,7 +117,11 @@ namespace System.Web.Configuration
 				string mdir;
 
 				if (map == null)
+#if TARGET_JVM
+					return "/web.config";
+#else
 					mdir = Path.GetDirectoryName (System.Runtime.InteropServices.RuntimeEnvironment.SystemConfigurationFile);
+#endif
 				else
 					mdir = Path.GetDirectoryName (map.MachineConfigFilename);
 
@@ -170,7 +175,8 @@ namespace System.Web.Configuration
 				else
 					configPath = locationSubPath;
 
-				if (configPath == "/")
+				if (configPath == HttpRuntime.AppDomainAppVirtualPath
+				    || configPath == "/")
 					i = -1;
 				else
 					i = configPath.LastIndexOf ("/");
@@ -201,6 +207,11 @@ namespace System.Web.Configuration
 			else if (HttpContext.Current != null
 				 && HttpContext.Current.Request != null)
 				return HttpContext.Current.Request.MapPath (virtualPath);
+			else if (HttpRuntime.AppDomainAppVirtualPath != null && virtualPath.StartsWith (HttpRuntime.AppDomainAppVirtualPath)) {
+				if (virtualPath == HttpRuntime.AppDomainAppVirtualPath)
+					return HttpRuntime.AppDomainAppPath;
+				return UrlUtils.Combine (HttpRuntime.AppDomainAppPath, virtualPath.Substring (HttpRuntime.AppDomainAppVirtualPath.Length));
+			}
 			else
 				return virtualPath;
 		}
@@ -297,8 +308,20 @@ namespace System.Web.Configuration
 		
 		public virtual Stream OpenStreamForRead (string streamName)
 		{
-			if (!File.Exists (streamName))
+			if (!File.Exists (streamName)) {
+#if TARGET_J2EE
+				if (streamName != null && (streamName.EndsWith ("machine.config") || streamName.EndsWith ("web.config"))) {
+					if (streamName.StartsWith ("/"))
+						streamName = streamName.Substring (1);
+					java.lang.ClassLoader cl = (java.lang.ClassLoader) AppDomain.CurrentDomain.GetData ("GH_ContextClassLoader");
+					if (cl != null) {
+						java.io.InputStream inputStream = cl.getResourceAsStream (streamName);
+						return (Stream) vmw.common.IOUtils.getStream (inputStream);
+					}
+				}
+#endif
 				throw new ConfigurationException ("File '" + streamName + "' not found");
+			}
 				
 			return new FileStream (streamName, FileMode.Open, FileAccess.Read);
 		}
