@@ -189,16 +189,19 @@ namespace System.Web.Configuration {
 			return OpenMappedMachineConfiguration (fileMap);
 		}
 
-		[MonoTODO ("apparently this bad boy can return null, but GetWebApplicationSection doesn't")]
 		public static object GetSection (string sectionName)
 		{
-			try {
-				_Configuration c = OpenWebConfiguration (HttpContext.Current.Request.Path);
+			_Configuration c;
+			if (HttpContext.Current != null
+			    && HttpContext.Current.Request != null)
+				c = OpenWebConfiguration (HttpContext.Current.Request.Path);
+			else
+				c = OpenWebConfiguration (HttpRuntime.AppDomainAppVirtualPath);
+
+			if (c == null)
+				return null;
+			else
 				return c.GetSection (sectionName);
-			}
-			catch {
-				return GetWebApplicationSection (sectionName);
-			}
 		}
 
 		[MonoTODO]
@@ -230,6 +233,8 @@ namespace System.Web.Configuration {
 			return config;
 		}
 
+		static MethodInfo get_runtime_object = typeof (ConfigurationSection).GetMethod ("GetRuntimeObject", BindingFlags.NonPublic | BindingFlags.Instance);
+
 		[MonoTODO]
 		public static object GetWebApplicationSection (string sectionName)
 		{
@@ -237,7 +242,7 @@ namespace System.Web.Configuration {
 
 			ConfigurationSection section = config.GetSection (sectionName);
 
-			return section;
+			return get_runtime_object.Invoke (section, new object [0]);
 		}
 
 		public static NameValueCollection AppSettings {
@@ -256,16 +261,27 @@ namespace System.Web.Configuration {
 		{
  			if (path == "/" || path == "")
 				return path;
-			
-			string pd = HttpContext.Current.Request.MapPath (path);
 
-			if (!Directory.Exists (pd)) {
-				int i = path.LastIndexOf ('/');
-				path = path.Substring (0, i);
-			} 
+			/* first if we can, map it to a physical path
+			 * to see if it corresponds to a file */
+			if (HttpContext.Current != null
+			    && HttpContext.Current.Request != null) {
+				string pd = HttpContext.Current.Request.MapPath (path);
+
+				if (!Directory.Exists (pd)) {
+					/* if it does, remove the file from the url */
+					int i = path.LastIndexOf ('/');
+					path = path.Substring (0, i);
+				} 
+			}
 			
+			if (path.Length == 0)
+				return path;
+
+			/* remove excess /'s from the end of the virtual path */
 			while (path [path.Length - 1] == '/')
 				path = path.Substring (0, path.Length - 1);
+
 			return path;
 		}
 
@@ -283,10 +299,19 @@ namespace System.Web.Configuration {
 
 		static private Web20DefaultConfig config {
 			get {
-				return (Web20DefaultConfig)AppDomain.CurrentDomain.GetData("WebConfigurationManager.config");
+				return (Web20DefaultConfig) AppDomain.CurrentDomain.GetData ("Web20DefaultConfig.config");
 			}
 			set {
-				AppDomain.CurrentDomain.SetData("WebConfigurationManager.config", value);
+				AppDomain.CurrentDomain.SetData ("Web20DefaultConfig.config", value);
+			}
+		}
+
+		static private IInternalConfigSystem configSystem {
+			get {
+				return (IInternalConfigSystem) AppDomain.CurrentDomain.GetData ("IInternalConfigSystem.configSystem");
+			}
+			set {
+				AppDomain.CurrentDomain.SetData ("IInternalConfigSystem.configSystem", value);
 			}
 		}
 #else
