@@ -58,7 +58,7 @@ namespace System.Web.Hosting {
 		}
 
 		//
-		// For furthe details see `Hosting the ASP.NET runtime'
+		// For further details see `Hosting the ASP.NET runtime'
 		//
 		//    http://www.west-wind.com/presentations/aspnetruntime/aspnetruntime.asp
 		// 
@@ -68,6 +68,9 @@ namespace System.Web.Hosting {
 			if (physicalDir == null)
 				throw new NullReferenceException ();
 
+#if NET_2_0
+			physicalDir = Path.GetFullPath (physicalDir);
+#endif
 			// This might throw
 			Uri u = new Uri (physicalDir);
 			physicalDir = HttpUtility.UrlDecode (u.AbsolutePath);
@@ -96,11 +99,11 @@ namespace System.Web.Hosting {
 			setup.ApplicationName = domain_id;
 			setup.ConfigurationFile = FindWebConfig (physicalDir);
 			setup.DisallowCodeDownload = true;
-			setup.PrivateBinPath = "bin";
+			string bin_path = Path.Combine (physicalDir, "bin");
+			setup.PrivateBinPath = bin_path;
 			setup.PrivateBinPathProbe = "*";
 			setup.ShadowCopyFiles = "true";
-			UriBuilder b = new UriBuilder ("file://", null, 0, Path.Combine (physicalDir, "bin"));
-			setup.ShadowCopyDirectories = b.Uri.ToString ();
+			setup.ShadowCopyDirectories = bin_path;
 
 			string dynamic_dir = null;
 			string user = Environment.UserName;
