@@ -6,7 +6,7 @@
 //	Stefan Grling (stefan@gorling.se)
 //	Jackson Harper (jackson@ximian.com)
 //
-// Copyright (C) 2002,2003,2004,2005 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2002-2006 Novell, Inc (http://www.novell.com)
 // (C) 2003 Stefan Grling (http://www.gorling.se)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
@@ -42,11 +42,34 @@ namespace System.Web.SessionState
 	[AspNetHostingPermission (SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	public sealed class SessionStateModule : IHttpModule
 	{
-		internal static readonly string CookieName = "ASPSESSION";
-		internal static readonly string HeaderName = "AspFilterSessionId";
+		internal const string CookieName = "ASPSESSION";
+		internal const string HeaderName = "AspFilterSessionId";
 		static object locker = new object ();
 		
 #if TARGET_J2EE		
+#if NET_2_0
+		static private SessionStateSection config {
+			get
+			{
+				return (SessionStateSection) AppDomain.CurrentDomain.GetData ("SessionStateModule.config");
+			}
+			set
+			{
+				AppDomain.CurrentDomain.SetData ("SessionStateModule.config", value);
+			}
+		}
+		static private Type handlerType
+		{
+			get
+			{
+				return (Type) AppDomain.CurrentDomain.GetData ("SessionStateModule.handlerType");
+			}
+			set
+			{
+				AppDomain.CurrentDomain.SetData ("SessionStateModule.handlerType", value);
+			}
+		}
+#else
 		static private SessionConfig config {
 			get {
 				return (SessionConfig)AppDomain.CurrentDomain.GetData("SessionStateModule.config");
@@ -63,6 +86,7 @@ namespace System.Web.SessionState
 				AppDomain.CurrentDomain.SetData("SessionStateModule.handlerType", value);
 			}
 		}
+#endif
 #else
 #if NET_2_0
 		static SessionStateSection config;
