@@ -167,7 +167,7 @@ namespace System.Web.UI.HtmlControls {
 
 #if NET_2_0
 		[DefaultValue ("")]
-		public string ValidationGroup
+		public virtual string ValidationGroup
 		{
 			get {
 				return ViewState.GetString ("ValidationGroup", "");
