@@ -45,6 +45,7 @@ namespace System.Web.UI.WebControls {
 		
 		object dataSource;
 		bool initialized;
+		bool preRendered;
 		bool requiresDataBinding;
 		
 		protected BaseDataBoundControl ()
@@ -97,7 +98,11 @@ namespace System.Web.UI.WebControls {
 		
 		protected bool RequiresDataBinding {
 			get { return requiresDataBinding; }
-			set { requiresDataBinding = value; }
+			set { 
+				requiresDataBinding = value;
+				if (value && preRendered && IsBoundUsingDataSourceID && Page != null && !Page.IsCallback)
+					EnsureDataBound ();
+			}
 		}
 		
 		protected void ConfirmInitState ()
@@ -132,6 +137,9 @@ namespace System.Web.UI.WebControls {
 		{
 			base.OnInit (e);
 			Page.PreLoad += new EventHandler (OnPagePreLoad);
+
+			if (!IsViewStateEnabled && Page != null && Page.IsPostBack)
+				RequiresDataBinding = true;
 		}
 		
 		protected virtual void OnPagePreLoad (object sender, EventArgs e)
@@ -141,6 +149,7 @@ namespace System.Web.UI.WebControls {
 		
 		protected internal override void OnPreRender (EventArgs e)
 		{
+			preRendered = true;
 			EnsureDataBound ();
 			base.OnPreRender (e);
 		}
