@@ -301,14 +301,14 @@ namespace System.Web.UI.WebControls {
 						AddSeparator (cell);
 						cell.Controls.Add (CreateButton (DeleteText, DeleteImageUrl, DataControlCommands.DeleteCommandName, index));
 					}
-					if (ShowSelectButton) {
-						AddSeparator (cell);
-						cell.Controls.Add (CreateButton (SelectText, SelectImageUrl, DataControlCommands.SelectCommandName, index));
-					}
 					if (ShowInsertButton) {
 						AddSeparator (cell);
 						cell.Controls.Add (CreateButton (NewText, NewImageUrl, DataControlCommands.NewCommandName, index));
 					}
+					if (ShowSelectButton) {
+						AddSeparator (cell);
+						cell.Controls.Add (CreateButton (SelectText, SelectImageUrl, DataControlCommands.SelectCommandName, index));
+					}
 				}
 			} else
 				base.InitializeCell (cell, cellType, rowState, rowIndex);
@@ -362,6 +362,8 @@ namespace System.Web.UI.WebControls {
 		
 		public override void ValidateSupportsCallback ()
 		{
+			if (ShowSelectButton)
+				throw new NotSupportedException ("ShowSelectButton = true");
 		}
 	}
 }
