@@ -515,7 +515,7 @@ namespace System.Web.UI.WebControls {
 				IEnumerator items = (data_source.DataSource != null) ? data_source.GetEnumerator () : null;
 				if (items != null && items.MoveNext ()) {
 					object data = items.Current;
-					if (data is ICustomTypeDescriptor)
+					if ((data is ICustomTypeDescriptor) || (!IsBindableType(data.GetType())))
 						props = TypeDescriptor.GetProperties (data);
 					else if (data != null)
 						ptype = data.GetType ();
@@ -755,7 +755,9 @@ namespace System.Web.UI.WebControls {
 					link.CausesValidation = false;
 					number = link;
 				} else {
-					number = new LiteralControl (page);
+					Label pageLabel = new Label();
+					pageLabel.Text = page;
+					number = pageLabel;
 				}
 
 				res.Controls.Add (number);
@@ -858,6 +860,29 @@ namespace System.Web.UI.WebControls {
 
 		class NCollection : ICollection {
 			int n;
+#if TARGET_JVM
+			class SequenceEnumerator : IEnumerator {
+				readonly int _max;
+				object _current;
+				public SequenceEnumerator(int max) {
+					_max = max;
+					Reset();
+				}
+
+				public bool MoveNext() {
+					_current = (int)_current + 1;
+					return (int)_current >= 0 && (int)_current < _max;
+				}
+
+				public void Reset() {
+					_current = -1;
+				}
+
+				public object Current {
+					get { return _current; }
+				}
+			}
+#endif
 
 			public NCollection (int n)
 			{
@@ -866,8 +891,12 @@ namespace System.Web.UI.WebControls {
 
 			public IEnumerator GetEnumerator ()
 			{
+#if TARGET_JVM
+				return new SequenceEnumerator(n);
+#else
 				for (int i = 0; i < n; i++)
 					yield return i;
+#endif
 			}
 
 			public int Count {
@@ -913,6 +942,9 @@ namespace System.Web.UI.WebControls {
 			pds.PageSize = PageSize;
 			pds.VirtualCount = VirtualItemCount;
 
+			if ((pds.IsPagingEnabled) && (pds.PageCount < pds.CurrentPageIndex))
+				throw new HttpException ("Invalid DataGrid PageIndex");
+				
 			CreateRenderColumns (paged_data_source, useDataSource);
 			if (useDataSource) {
 				if (DataSource != null)
@@ -930,6 +962,8 @@ namespace System.Web.UI.WebControls {
 			// this silly foreach and index++
 			if (items_list == null)
 				items_list = new ArrayList ();
+			else
+				items_list.Clear();
 
 			bool skip_first = false;
 			IEnumerator enumerator = null;
@@ -946,6 +980,7 @@ namespace System.Web.UI.WebControls {
 			int index = 0;
 			bool first = true;
 			string key = null;
+			int dataset_index = pds.FirstIndexInPage;
 			int selected_index = SelectedIndex;
 			int edit_item_index = EditItemIndex;
 			while (enumerator != null && (skip_first || enumerator.MoveNext ())) {
@@ -973,15 +1008,16 @@ namespace System.Web.UI.WebControls {
 				else if (index % 2 != 0) 
 					type = ListItemType.AlternatingItem;
 
-				items_list.Add (CreateItem (index, index, type, useDataSource, data, pds));
+				items_list.Add (CreateItem (index, dataset_index, type, useDataSource, data, pds));
 				index++;
+				dataset_index++;
 			}
 
 			CreateItem (-1, -1, ListItemType.Footer, useDataSource, null, paged_data_source);
 			if (pds.IsPagingEnabled) {
 				CreateItem (-1, -1, ListItemType.Pager, false, null, paged_data_source);
 				if (useDataSource)
-					ViewState ["Items"] = pds.DataSourceCount;
+					ViewState ["Items"] = pds.IsCustomPagingEnabled ? index : pds.DataSourceCount;
 			} else if (useDataSource) {
 				ViewState ["Items"] = index;
 			}
@@ -1017,6 +1053,10 @@ namespace System.Web.UI.WebControls {
 			rt.CopyBaseAttributes (this);
 			rt.ApplyStyle (ControlStyle);
 
+			rt.Caption = Caption;
+			rt.CaptionAlign = CaptionAlign;
+			rt.Enabled = Enabled;
+
 			bool top_pager = true;
 			Style alt = null;
 			foreach (DataGridItem item in rt.Rows) {
@@ -1029,7 +1069,7 @@ namespace System.Web.UI.WebControls {
 				case ListItemType.AlternatingItem:
 					if (alt == null) {
 						if (alt_item_style != null) {
-							alt = new Style ();
+							alt = new TableItemStyle ();
 							alt.CopyFrom (item_style);
 							alt.CopyFrom (alt_item_style);
 						} else {
