@@ -632,10 +632,10 @@ namespace System.Web.UI.WebControls {
 			case ListItemType.SelectedItem:
 			case ListItemType.EditItem:
 				int index = item.ItemIndex;
-				if ((SelectedIndex == index) && (SelectedItemTemplate != null))
-					t = SelectedItemTemplate;
-				else if ((EditItemIndex == index) && (EditItemTemplate != null))
+				if ((EditItemIndex == index) && (EditItemTemplate != null))
 					t = EditItemTemplate;
+				else if ((SelectedIndex == index) && (SelectedItemTemplate != null))
+					t = SelectedItemTemplate;
 				else if (((index & 1) == 0) || (alternatingItemTemplate == null))
 					t = ItemTemplate;
 				else
@@ -650,7 +650,11 @@ namespace System.Web.UI.WebControls {
 		protected override void LoadViewState (object savedState)
 		{
 			object[] state = (object[]) savedState;
+#if NET_2_0
+			base.LoadViewState (state [8]);
+#else
 			base.LoadViewState (state [0]);
+#endif
 			if (state [1] != null)
 				ItemStyle.LoadViewState (state [1]);
 			if (state [2] != null)
