@@ -108,10 +108,12 @@ namespace System.Web.UI.WebControls
 		// Control state
 		int pageIndex;
 		DetailsViewMode currentMode = DetailsViewMode.ReadOnly; 
-		int pageCount = -1;
+		int pageCount = 0;
 		
 		public DetailsView ()
 		{
+			rows = new DetailsViewRowCollection (new ArrayList ());
+			key = new DataKey (new OrderedDictionary ());
 		}
 		
 		public event EventHandler PageIndexChanged {
@@ -422,7 +424,9 @@ namespace System.Web.UI.WebControls
 				return -1;
 			}
 			set {
-				ViewState ["CellPadding"] = value;
+				if (value < -1)
+					throw new ArgumentOutOfRangeException ("< -1");
+				ViewState["CellPadding"] = value;
 				RequireBinding ();
 			}
 		}
@@ -437,6 +441,8 @@ namespace System.Web.UI.WebControls
 				return 0;
 			}
 			set {
+				if (value < -1)
+					throw new ArgumentOutOfRangeException ("< -1");
 				ViewState ["CellSpacing"] = value;
 				RequireBinding ();
 			}
@@ -465,17 +471,16 @@ namespace System.Web.UI.WebControls
 				return currentMode;
 			}
 		}
-	
+
+		DetailsViewMode defaultMode = DetailsViewMode.ReadOnly;
 		[DefaultValueAttribute (DetailsViewMode.ReadOnly)]
 		[WebCategoryAttribute ("Behavior")]
 		public virtual DetailsViewMode DefaultMode {
 			get {
-				object o = ViewState ["DefaultMode"];
-				if (o != null) return (DetailsViewMode) o;
-				return DetailsViewMode.ReadOnly;
+				return defaultMode;
 			}
 			set {
-				ViewState ["DefaultMode"] = value;
+				defaultMode = value;
 				RequireBinding ();
 			}
 		}
@@ -497,6 +502,8 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+		string[] dataKeyNames = null;
+
 		[DefaultValueAttribute (null)]
 		[WebCategoryAttribute ("Data")]
 		[TypeConverter (typeof(StringArrayConverter))]
@@ -504,12 +511,13 @@ namespace System.Web.UI.WebControls
 		public virtual string[] DataKeyNames
 		{
 			get {
-				object o = ViewState ["DataKeyNames"];
-				if (o != null) return (string[]) o;
-				return emptyKeys;
+				if (dataKeyNames == null)
+					return emptyKeys;
+				else
+					return dataKeyNames;
 			}
 			set {
-				ViewState ["DataKeyNames"] = value;
+				dataKeyNames = value;
 				RequireBinding ();
 			}
 		}
@@ -662,12 +670,10 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (GridLines.Both)]
 		public virtual GridLines GridLines {
 			get {
-				object ob = ViewState ["GridLines"];
-				if (ob != null) return (GridLines) ob;
-				return GridLines.Both;
+				return ((TableStyle)ControlStyle).GridLines;
 			}
 			set {
-				ViewState ["GridLines"] = value;
+				((TableStyle)ControlStyle).GridLines = value;
 			}
 		}
 
@@ -754,7 +760,7 @@ namespace System.Web.UI.WebControls
 		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
 		public virtual int PageCount {
 			get {
-				if (pageCount != -1) return pageCount;
+				if (pageCount != 0) return pageCount;
 				EnsureDataBound ();
 				return pageCount;
 			}
@@ -818,7 +824,7 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual DetailsViewRowCollection Rows {
 			get {
-				EnsureDataBound ();
+				EnsureChildControls ();
 				return rows;
 			}
 		}
@@ -881,7 +887,7 @@ namespace System.Web.UI.WebControls
 	
 		public virtual bool IsBindableType (Type type)
 		{
-			return type.IsPrimitive || type == typeof(string) || type == typeof(DateTime) || type == typeof(Guid);
+			return type.IsPrimitive || type == typeof (string) || type == typeof (DateTime) || type == typeof (Guid) || type == typeof (Decimal);
 		}
 		
 		protected override DataSourceSelectArguments CreateDataSourceSelectArguments ()
@@ -961,7 +967,11 @@ namespace System.Web.UI.WebControls
 		
 		protected virtual DetailsViewRow CreateRow (int rowIndex, DataControlRowType rowType, DataControlRowState rowState)
 		{
-			DetailsViewRow row = new DetailsViewRow (rowIndex, rowType, rowState);
+			DetailsViewRow row;
+			if (rowType == DataControlRowType.Pager)
+				row = new DetailsViewPagerRow (rowIndex, rowType, rowState);
+			else
+				row = new DetailsViewRow (rowIndex, rowType, rowState);
 			OnItemCreated (EventArgs.Empty);
 			return row;
 		}
@@ -970,7 +980,7 @@ namespace System.Web.UI.WebControls
 		{
 			if (Initialized) {
 				RequiresDataBinding = true;
-				pageCount = -1;
+				pageCount = 0;
 			}
 		}
 		
@@ -980,16 +990,20 @@ namespace System.Web.UI.WebControls
 			table.Caption = Caption;
 			table.CaptionAlign = CaptionAlign;
 			table.CellPadding = CellPadding;
-			table.CellSpacing = CellSpacing;
+			//CellSpacing defaults to 0 and Table.CellSpacing to -1
+			if (CellSpacing != 0)
+				table.CellSpacing = CellSpacing;
 			table.HorizontalAlign = HorizontalAlign;
 			table.BackImageUrl = BackImageUrl;
 			return table;
 		}
-	
-		[MonoTODO]
+
 		protected override Style CreateControlStyle ()
 		{
-			throw new NotImplementedException ();
+			TableStyle style = new TableStyle (ViewState);
+			style.GridLines = GridLines.Both;
+			style.CellSpacing = 0;
+			return style;
 		}
 		
 		protected override int CreateChildControls (IEnumerable data, bool dataBinding)
@@ -1009,15 +1023,10 @@ namespace System.Web.UI.WebControls
 						dataSource.AllowServerPaging = true;
 						if (view.CanRetrieveTotalRowCount)
 							dataSource.VirtualCount = SelectArguments.TotalRowCount;
-						else {
-							dataSource.DataSourceView = view;
-							dataSource.DataSourceSelectArguments = SelectArguments;
-							dataSource.SetItemCountFromPageIndex (PageIndex + PagerSettings.PageButtonCount);
-						}
 					}
 				}
 				
-				pageCount = dataSource.PageCount;
+				pageCount = dataSource.DataSourceCount;
 			}
 			else
 			{
@@ -1038,7 +1047,7 @@ namespace System.Web.UI.WebControls
 				
 			ArrayList list = new ArrayList ();
 
-			if (!Page.IsPostBack)
+			if (Page == null || !Page.IsPostBack)
 				currentMode = DefaultMode;
 
 			
@@ -1064,17 +1073,15 @@ namespace System.Web.UI.WebControls
 
 			// Main table creation
 			
-			if (HeaderText.Length != 0 || headerTemplate != null) {
-				headerRow = CreateRow (-1, DataControlRowType.Header, DataControlRowState.Normal);
-				DataControlFieldCell cell = new DataControlFieldCell (null);
-				cell.ColumnSpan = 2;
-				if (headerTemplate != null)
-					headerTemplate.InstantiateIn (cell);
-				else
-					cell.Text = HeaderText;
-				headerRow.Cells.Add (cell);
-				table.Rows.Add (headerRow);
-			}
+			headerRow = CreateRow (-1, DataControlRowType.Header, DataControlRowState.Normal);
+			DataControlFieldCell headerCell = new DataControlFieldCell (null);
+			headerCell.ColumnSpan = 2;
+			if (headerTemplate != null)
+				headerTemplate.InstantiateIn (headerCell);
+			else
+				headerCell.Text = HeaderText;
+			headerRow.Cells.Add (headerCell);
+			table.Rows.Add (headerRow);
 			
 			if (showPager && PagerSettings.Position == PagerPosition.Top || PagerSettings.Position == PagerPosition.TopAndBottom) {
 				topPagerRow = CreateRow (-1, DataControlRowType.Pager, DataControlRowState.Normal);
@@ -1110,17 +1117,15 @@ namespace System.Web.UI.WebControls
 				table.Rows.Add (bottomPagerRow);
 			}
 
-			if (FooterText.Length != 0 || footerTemplate != null) {
-				footerRow = CreateRow (-1, DataControlRowType.Footer, DataControlRowState.Normal);
-				DataControlFieldCell cell = new DataControlFieldCell (null);
-				cell.ColumnSpan = 2;
-				if (footerTemplate != null)
-					footerTemplate.InstantiateIn (cell);
-				else
-					cell.Text = FooterText;
-				footerRow.Cells.Add (cell);
-				table.Rows.Add (footerRow);
-			}
+			footerRow = CreateRow (-1, DataControlRowType.Footer, DataControlRowState.Normal);
+			DataControlFieldCell footerCell = new DataControlFieldCell (null);
+			footerCell.ColumnSpan = 2;
+			if (footerTemplate != null)
+				footerTemplate.InstantiateIn (footerCell);
+			else
+				footerCell.Text = FooterText;
+			footerRow.Cells.Add (footerCell);
+			table.Rows.Add (footerRow);
 			
 			if (dataBinding)
 				DataBind (false);
@@ -1131,7 +1136,7 @@ namespace System.Web.UI.WebControls
 		[MonoTODO]
 		protected override void EnsureDataBound ()
 		{
-			throw new NotImplementedException ();
+			base.EnsureDataBound ();
 		}
 		
 		DataControlRowState GetRowState (int index)
@@ -1464,7 +1469,8 @@ namespace System.Web.UI.WebControls
 			OnItemInserting (args);
 			if (!args.Cancel) {
 				DataSourceView view = GetData ();
-				if (view == null) throw new HttpException ("The DataSourceView associated to data bound control was null");
+				if (view == null)
+					return;
 				view.Insert (currentEditNewValues, new DataSourceViewOperationCallback (InsertCallback));
 			} else
 				EndRowEdit ();
@@ -1483,7 +1489,7 @@ namespace System.Web.UI.WebControls
 
 		public virtual void DeleteItem ()
 		{
-			currentEditRowKeys = DataKey.Values;
+			currentEditRowKeys = DataKey == null ? null : DataKey.Values;
 			currentEditNewValues = GetRowValues (true, true);
 			
 			DetailsViewDeleteEventArgs args = new DetailsViewDeleteEventArgs (PageIndex, currentEditRowKeys, currentEditNewValues);
@@ -1528,13 +1534,15 @@ namespace System.Web.UI.WebControls
 			pageIndex = (int) state[1];
 			pageCount = (int) state[2];
 			currentMode = (DetailsViewMode) state[3];
+			dataKeyNames = (string[]) state[4];
+			defaultMode = (DetailsViewMode) state[5];
 		}
 		
 		protected internal override object SaveControlState ()
 		{
 			object bstate = base.SaveControlState ();
 			return new object[] {
-				bstate, pageIndex, pageCount, currentMode
+				bstate, pageIndex, pageCount, currentMode, dataKeyNames, defaultMode
 					};
 		}
 		
@@ -1680,7 +1688,7 @@ namespace System.Web.UI.WebControls
 		[MonoTODO]
 		protected override void OnPagePreLoad (object sender, EventArgs e)
 		{
-			throw new NotImplementedException ();
+			base.OnPagePreLoad (sender, e);
 		}
 		
 		protected internal override void OnPreRender (EventArgs e)
@@ -1710,35 +1718,21 @@ namespace System.Web.UI.WebControls
 		{
 			if (EnablePagingCallbacks)
 				base.RenderBeginTag (writer);
+			else
+				writer.RenderBeginTag (HtmlTextWriterTag.Div);
 
 			RenderGrid (writer);
 			
 			if (EnablePagingCallbacks)
 				base.RenderEndTag (writer);
+			else
+				writer.RenderEndTag ();
 		}
 		
 		void RenderGrid (HtmlTextWriter writer)
 		{
-			switch (GridLines) {
-			case GridLines.Horizontal:
-				writer.AddAttribute (HtmlTextWriterAttribute.Rules, "rows");
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "1");
-				break;
-			case GridLines.Vertical:
-				writer.AddAttribute (HtmlTextWriterAttribute.Rules, "cols");
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "1");
-				break;
-			case GridLines.Both:
-				writer.AddAttribute (HtmlTextWriterAttribute.Rules, "all");
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "1");
-				break;
-			default:
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "0");
-				break;
-			}
-			
-			writer.AddAttribute (HtmlTextWriterAttribute.Cellspacing, "0");
-			writer.AddStyleAttribute (HtmlTextWriterStyle.BorderCollapse, "collapse");
+			table.GridLines = GridLines;
+			table.ControlStyle.MergeWith (ControlStyle);
 			table.RenderBeginTag (writer);
 			
 			foreach (DetailsViewRow row in table.Rows)
@@ -1795,7 +1789,8 @@ namespace System.Web.UI.WebControls
 		[MonoTODO]
 		PostBackOptions IPostBackContainer.GetPostBackOptions (IButtonControl control)
 		{
-			throw new NotImplementedException ();
+			PostBackOptions pbo = new PostBackOptions (this, control.CommandName + "$" + control.CommandArgument);
+			return pbo;
 		}
 	}
 }
