@@ -130,19 +130,10 @@ namespace System.Web.UI.WebControls {
 		{
 			get 
 			{
-				string[] names;
+				string [] names = Names;
 
-				if ((fontstyles & FontStyles.Names) == 0) 
-				{
+				if (names.Length == 0)
 					return string.Empty;
-				}
-
-				names = (string[])bag["Font_Names"];
-
-				if (names.Length == 0) 
-				{
-					return string.Empty;
-				}
 
 				return names[0];
 			}
@@ -152,7 +143,7 @@ namespace System.Web.UI.WebControls {
 				// Seems to be a special case in MS, removing the names from the bag when Name is set to empty, 
 				// but not when setting Names to an empty array
 				if (value == string.Empty) {
-					bag.Remove("Font_Names");
+					Names = null;
 					return;
 				}
 
@@ -192,8 +183,14 @@ namespace System.Web.UI.WebControls {
 
 			set 
 			{
-				fontstyles |= FontStyles.Names;
-				bag["Font_Names"] = value;
+				if (value == null) {
+					fontstyles &= ~FontStyles.Names;
+					bag.Remove ("Font_Names");
+				}
+				else {
+					fontstyles |= FontStyles.Names;
+					bag ["Font_Names"] = value;
+				}
 			}
 		}
 
@@ -311,11 +308,42 @@ namespace System.Web.UI.WebControls {
 		#region Public Instance Methods
 		public void CopyFrom(FontInfo f) 
 		{
+			//Methods CopyFrom and MergeWith behave differently between 1.1 and 2.0
+			if (f == null || f.IsEmpty)
+				return;
+
 			if (f == this)
 				return;
 
-			this.Reset();
+#if NET_2_0
+			// MS stores the property in the bag if it's value is false
+			if ((f.fontstyles & FontStyles.Bold) != 0) {
+				this.Bold = f.Bold;
+			}
 
+			if ((f.fontstyles & FontStyles.Italic) != 0) {
+				this.Italic = f.Italic;
+			}
+
+			// MS seems to have some weird behaviour, even if f's Name has been set to String.Empty we still get an empty array
+			this.Names = f.Names;
+
+			if ((f.fontstyles & FontStyles.Overline) != 0) {
+				this.Overline = f.Overline;
+			}
+
+			if ((f.fontstyles & FontStyles.Size) != 0) {
+				this.Size = f.Size;
+			}
+
+			if ((f.fontstyles & FontStyles.Strikeout) != 0) {
+				this.Strikeout = f.Strikeout;
+			}
+
+			if ((f.fontstyles & FontStyles.Underline) != 0) {
+				this.Underline = f.Underline;
+			}
+#else
 			// MS does not store the property in the bag if it's value is false
 			if (((f.fontstyles & FontStyles.Bold) != 0) && f.Bold) 
 			{
@@ -328,10 +356,7 @@ namespace System.Web.UI.WebControls {
 			}
 
 			// MS seems to have some weird behaviour, even if f's Name has been set to String.Empty we still get an empty array
-			if (((f.fontstyles & FontStyles.Names) != 0)) 
-			{
-				this.Names = f.Names;
-			}
+			this.Names = f.Names;
 
 			if (((f.fontstyles & FontStyles.Overline) != 0) && f.Overline) 
 			{
@@ -352,10 +377,41 @@ namespace System.Web.UI.WebControls {
 			{
 				this.Underline = true;
 			}
+#endif
 		}
 
 		public void MergeWith(FontInfo f) 
 		{
+			//Methods CopyFrom and MergeWith behave differently between 1.1 and 2.0
+#if NET_2_0
+			if (((fontstyles & FontStyles.Bold) == 0) && ((f.fontstyles & FontStyles.Bold) != 0)) {
+				this.Bold = f.Bold;
+			}
+
+			if (((fontstyles & FontStyles.Italic) == 0) && ((f.fontstyles & FontStyles.Italic) != 0)) {
+				this.Italic = f.Italic;
+			}
+
+			if (((fontstyles & FontStyles.Names) == 0) && ((f.fontstyles & FontStyles.Names) != 0)) {
+				this.Names = f.Names;
+			}
+
+			if (((fontstyles & FontStyles.Overline) == 0) && ((f.fontstyles & FontStyles.Overline) != 0)) {
+				this.Overline = f.Overline;
+			}
+
+			if (((fontstyles & FontStyles.Size) == 0) && ((f.fontstyles & FontStyles.Size) != 0)) {
+				this.Size = f.Size;
+			}
+
+			if (((fontstyles & FontStyles.Strikeout) == 0) && ((f.fontstyles & FontStyles.Strikeout) != 0)) {
+				this.Strikeout = f.Strikeout;
+			}
+
+			if (((fontstyles & FontStyles.Underline) == 0) && ((f.fontstyles & FontStyles.Underline) != 0)) {
+				this.Underline = f.Underline;
+			}
+#else
 			if (((fontstyles & FontStyles.Bold) == 0) && ((f.fontstyles & FontStyles.Bold) != 0) && f.Bold) 
 			{
 				this.Bold = true;
@@ -390,6 +446,7 @@ namespace System.Web.UI.WebControls {
 			{
 				this.Underline = true;
 			}
+#endif
 		}
 
 		public bool ShouldSerializeNames() 
@@ -406,6 +463,13 @@ namespace System.Web.UI.WebControls {
 
 			return this.Name + ", " + this.Size.ToString();
 		}
+#if NET_2_0
+
+		public void ClearDefaults () {
+			Reset ();
+		}
+
+#endif
 		#endregion	// Public Instance Methods
 
 		#region Private Methods
@@ -453,6 +517,61 @@ namespace System.Web.UI.WebControls {
 				fontstyles |= FontStyles.Underline;
 			}
 		}
+
+#if NET_2_0
+		internal void FillStyleAttributes (CssStyleCollection attributes, bool alwaysRenderTextDecoration) {
+			if (IsEmpty) {
+				if(alwaysRenderTextDecoration)
+					attributes.Add (HtmlTextWriterStyle.TextDecoration, "none");
+				return;
+			}
+
+			string s;
+			// Fonts are a bit weird
+			s = String.Join (",", Names);
+			if (s.Length > 0) {
+				attributes.Add (HtmlTextWriterStyle.FontFamily, s);
+			}
+
+			if ((fontstyles & FontStyles.Bold) != 0) {
+				attributes.Add (HtmlTextWriterStyle.FontWeight, Bold ? "bold" : "normal");
+			}
+
+			if ((fontstyles & FontStyles.Italic) != 0) {
+				attributes.Add (HtmlTextWriterStyle.FontStyle, Italic ? "italic" : "normal");
+			}
+
+			if (!Size.IsEmpty) {
+				attributes.Add (HtmlTextWriterStyle.FontSize, Size.ToString ());
+			}
+
+			// These styles are munged into a attribute decoration
+			s = string.Empty;
+			bool hasTextDecoration = false;
+
+			if ((fontstyles & FontStyles.Overline) != 0) {
+				if (Overline)
+					s += "overline ";
+				hasTextDecoration = true;
+			}
+
+			if ((fontstyles & FontStyles.Strikeout) != 0) {
+				if (Strikeout)
+					s += "line-through ";
+				hasTextDecoration = true;
+			}
+
+			if ((fontstyles & FontStyles.Underline) != 0) {
+				if (Underline)
+					s += "underline ";
+				hasTextDecoration = true;
+			}
+
+			s = (s.Length > 0) ? s.Trim () : (alwaysRenderTextDecoration || hasTextDecoration) ? "none" : "";
+			if (s.Length > 0)
+				attributes.Add (HtmlTextWriterStyle.TextDecoration, s);
+		}
+#endif
 		#endregion	// Private Methods
 
 
