@@ -85,7 +85,6 @@ namespace System.Web.UI.WebControls
 		
 		DataKey key;
 		DataKey oldEditValues;
-		int dataSourceCount;
 		
 		private static readonly object PageIndexChangedEvent = new object();
 		private static readonly object PageIndexChangingEvent = new object();
@@ -103,10 +102,11 @@ namespace System.Web.UI.WebControls
 		// Control state
 		int pageIndex;
 		FormViewMode currentMode = FormViewMode.ReadOnly; 
-		int pageCount = -1;
+		int pageCount = 0;
 		
 		public FormView ()
 		{
+			key = new DataKey (new OrderedDictionary ());
 		}
 		
 		public event EventHandler PageIndexChanged {
@@ -345,6 +345,8 @@ namespace System.Web.UI.WebControls
 				return -1;
 			}
 			set {
+				if (value < -1)
+					throw new ArgumentOutOfRangeException ("< -1");
 				ViewState ["CellPadding"] = value;
 				RequireBinding ();
 			}
@@ -360,7 +362,9 @@ namespace System.Web.UI.WebControls
 				return 0;
 			}
 			set {
-				ViewState ["CellSpacing"] = value;
+				if (value < -1)
+					throw new ArgumentOutOfRangeException ("< -1");
+				ViewState["CellSpacing"] = value;
 				RequireBinding ();
 			}
 		}
@@ -372,21 +376,22 @@ namespace System.Web.UI.WebControls
 				return currentMode;
 			}
 		}
-	
+
+		FormViewMode defaultMode;
+
 		[DefaultValueAttribute (FormViewMode.ReadOnly)]
 		[WebCategoryAttribute ("Behavior")]
 		public virtual FormViewMode DefaultMode {
 			get {
-				object o = ViewState ["DefaultMode"];
-				if (o != null) return (FormViewMode) o;
-				return FormViewMode.ReadOnly;
+				return defaultMode;
 			}
 			set {
-				ViewState ["DefaultMode"] = value;
+				defaultMode = value;
 				RequireBinding ();
 			}
 		}
-	
+
+		string[] dataKeyNames;
 		[DefaultValueAttribute (null)]
 		[WebCategoryAttribute ("Data")]
 		[TypeConverter (typeof(StringArrayConverter))]
@@ -394,12 +399,12 @@ namespace System.Web.UI.WebControls
 		public virtual string[] DataKeyNames
 		{
 			get {
-				object o = ViewState ["DataKeyNames"];
-				if (o != null) return (string[]) o;
-				return emptyKeys;
+				if (dataKeyNames == null)
+					return emptyKeys;
+				return dataKeyNames;
 			}
 			set {
-				ViewState ["DataKeyNames"] = value;
+				dataKeyNames = value;
 				RequireBinding ();
 			}
 		}
@@ -531,12 +536,10 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (GridLines.None)]
 		public virtual GridLines GridLines {
 			get {
-				object ob = ViewState ["GridLines"];
-				if (ob != null) return (GridLines) ob;
-				return GridLines.None;
+				return ((TableStyle) ControlStyle).GridLines;
 			}
 			set {
-				ViewState ["GridLines"] = value;
+				((TableStyle) ControlStyle).GridLines = value;
 			}
 		}
 
@@ -641,7 +644,7 @@ namespace System.Web.UI.WebControls
 		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
 		public virtual int PageCount {
 			get {
-				if (pageCount != -1) return pageCount;
+				if (pageCount != 0) return pageCount;
 				EnsureDataBound ();
 				return pageCount;
 			}
@@ -655,6 +658,8 @@ namespace System.Web.UI.WebControls
 				return pageIndex;
 			}
 			set {
+				if (pageIndex == value)
+					return;
 				pageIndex = value;
 				RequireBinding ();
 			}
@@ -760,7 +765,11 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual int DataItemIndex {
 			get { return PageIndex; }
-		}		
+		}
+
+		int IDataItemContainer.DataItemIndex {
+			get { return DataItemIndex; }
+		}
 	
 		int IDataItemContainer.DisplayIndex {
 			get { return PageIndex; }
@@ -768,7 +777,7 @@ namespace System.Web.UI.WebControls
 	
 		public virtual bool IsBindableType (Type type)
 		{
-			return type.IsPrimitive || type == typeof(string) || type == typeof(DateTime) || type == typeof(Guid);
+			return type.IsPrimitive || type == typeof (string) || type == typeof (DateTime) || type == typeof (Guid) || type == typeof (Decimal);
 		}
 		
 		protected override DataSourceSelectArguments CreateDataSourceSelectArguments ()
@@ -786,8 +795,8 @@ namespace System.Web.UI.WebControls
 		void RequireBinding ()
 		{
 			if (Initialized) {
-				RequiresDataBinding = true;
 				pageCount = -1;
+				RequiresDataBinding = true;
 			}
 		}
 		
@@ -806,55 +815,35 @@ namespace System.Web.UI.WebControls
 		[MonoTODO]
 		protected override void EnsureDataBound ()
 		{
-			throw new NotImplementedException ();
+			base.EnsureDataBound ();
 		}
 	
-		[MonoTODO]
 		protected override Style CreateControlStyle ()
 		{
-			throw new NotImplementedException ();
+			TableStyle style = new TableStyle (ViewState);
+			style.CellSpacing = 0;
+			return style;
 		}
 		
 		protected override int CreateChildControls (IEnumerable data, bool dataBinding)
 		{
-			PagedDataSource dataSource;
+			PagedDataSource dataSource = new PagedDataSource ();
+			dataSource.DataSource = data;
+			dataSource.AllowPaging = AllowPaging;
+			dataSource.PageSize = 1;
+			dataSource.CurrentPageIndex = PageIndex;
 
 			if (dataBinding) {
 				DataSourceView view = GetData ();
-				dataSource = new PagedDataSource ();
-				dataSource.DataSource = data;
-				
-				if (AllowPaging) {
-					dataSource.AllowPaging = true;
-					dataSource.PageSize = 1;
-					dataSource.CurrentPageIndex = PageIndex;
-					if (view.CanPage) {
-						dataSource.AllowServerPaging = true;
-						if (view.CanRetrieveTotalRowCount)
-							dataSource.VirtualCount = SelectArguments.TotalRowCount;
-						else {
-							dataSource.DataSourceView = view;
-							dataSource.DataSourceSelectArguments = SelectArguments;
-							dataSource.SetItemCountFromPageIndex (PageIndex + PagerSettings.PageButtonCount);
-						}
-					}
-				}
-				
-				pageCount = dataSource.PageCount;
-			}
-			else
-			{
-				dataSource = new PagedDataSource ();
-				dataSource.DataSource = data;
-				if (AllowPaging) {
-					dataSource.AllowPaging = true;
-					dataSource.PageSize = 1;
-					dataSource.CurrentPageIndex = PageIndex;
+				if (view != null && view.CanPage) {
+					dataSource.AllowServerPaging = true;
+					if (view.CanRetrieveTotalRowCount)
+						dataSource.VirtualCount = SelectArguments.TotalRowCount;
 				}
 			}
 
-			bool showPager = AllowPaging && (PageCount > 1);
-			dataSourceCount = dataSource.Count;
+			pageCount = dataSource.DataSourceCount;
+			bool showPager = AllowPaging && (dataSource.Count > 0);
 			
 			Controls.Clear ();
 			table = CreateTable ();
@@ -866,10 +855,16 @@ namespace System.Web.UI.WebControls
 			// Gets the current data item
 			
 			IEnumerator e = dataSource.GetEnumerator (); 
-			if (e.MoveNext ())
-				dataItem = e.Current;
+			dataItem = null;
+
+			if (AllowPaging) {
+				if (e.MoveNext ())
+					dataItem = e.Current;
+			}
 			else
-				dataItem = null;
+			for (int page = 0; e.MoveNext (); page++ )
+				if (page == PageIndex)
+					dataItem = e.Current;
 			
 			// Main table creation
 			
@@ -885,7 +880,7 @@ namespace System.Web.UI.WebControls
 				table.Rows.Add (topPagerRow);
 			}
 
-			if (dataSourceCount > 0) {
+			if (pageCount > 0) {
 				DataControlRowState rstate = GetRowState ();
 				itemRow = CreateRow (0, DataControlRowType.DataRow, rstate);
 				InitializeRow (itemRow);
@@ -897,13 +892,23 @@ namespace System.Web.UI.WebControls
 					key = new DataKey (new OrderedDictionary (), DataKeyNames);
 				}
 			} else {
-				itemRow = CreateRow (-1, DataControlRowType.EmptyDataRow, DataControlRowState.Normal);
+				switch (CurrentMode) {
+				case FormViewMode.Edit:
+					itemRow = CreateRow (-1, DataControlRowType.DataRow, DataControlRowState.Edit);
+					break;
+				case FormViewMode.Insert:
+					itemRow = CreateRow (-1, DataControlRowType.DataRow, DataControlRowState.Insert);
+					break;
+				default:
+					itemRow = CreateRow (-1, DataControlRowType.EmptyDataRow, DataControlRowState.Normal);
+					break;
+				}
 				table.Rows.Add (itemRow);
 				InitializeRow (itemRow);
 			}
 				
 			if (showPager && PagerSettings.Position == PagerPosition.Bottom || PagerSettings.Position == PagerPosition.TopAndBottom) {
-				bottomPagerRow = CreateRow (-1, DataControlRowType.Pager, DataControlRowState.Normal);
+				bottomPagerRow = CreateRow (0, DataControlRowType.Pager, DataControlRowState.Normal);
 				InitializePager (bottomPagerRow, dataSource);
 				table.Rows.Add (bottomPagerRow);
 			}
@@ -931,7 +936,8 @@ namespace System.Web.UI.WebControls
 		protected virtual void InitializePager (FormViewRow row, PagedDataSource dataSource)
 		{
 			TableCell cell = new TableCell ();
-			
+			cell.ColumnSpan = 2;
+
 			if (pagerTemplate != null)
 				pagerTemplate.InstantiateIn (cell);
 			else
@@ -976,6 +982,7 @@ namespace System.Web.UI.WebControls
 				else
 					cell.Text = HeaderText;
 			}
+			cell.ColumnSpan = 2;
 			row.Cells.Add (cell);
 		}
 		
@@ -1007,6 +1014,9 @@ namespace System.Web.UI.WebControls
 		
 		protected virtual void ExtractRowValues (IOrderedDictionary fieldValues, bool includeKeys)
 		{
+			if (Row == null)
+				return;
+
 			DataControlRowState rowState = Row.RowState;
 			IBindableTemplate bt;
 			
@@ -1045,7 +1055,7 @@ namespace System.Web.UI.WebControls
 			cachedKeyProperties = null;
 			base.DataBind ();
 			
-			if (dataSourceCount > 0) {
+			if (pageCount > 0) {
 				if (CurrentMode == FormViewMode.Edit)
 					oldEditValues = new DataKey (GetRowValues (true));
 				key = new DataKey (CreateRowDataKey (dataItem), DataKeyNames);
@@ -1168,8 +1178,11 @@ namespace System.Web.UI.WebControls
 			FormViewPageEventArgs args = new FormViewPageEventArgs (newIndex);
 			OnPageIndexChanging (args);
 			if (!args.Cancel) {
+				newIndex = args.NewPageIndex;
+				if (newIndex < 0 || newIndex >= PageCount)
+					return;
 				EndRowEdit ();
-				PageIndex = args.NewPageIndex;
+				PageIndex = newIndex;
 				OnPageIndexChanged (EventArgs.Empty);
 			}
 		}
@@ -1248,7 +1261,8 @@ namespace System.Web.UI.WebControls
 			OnItemInserting (args);
 			if (!args.Cancel) {
 				DataSourceView view = GetData ();
-				if (view == null) throw new HttpException ("The DataSourceView associated to data bound control was null");
+				if (view == null) 
+					return;
 				view.Insert (currentEditNewValues, new DataSourceViewOperationCallback (InsertCallback));
 			} else
 				EndRowEdit ();
@@ -1314,14 +1328,16 @@ namespace System.Web.UI.WebControls
 			pageIndex = (int) state[1];
 			pageCount = (int) state[2];
 			currentMode = (FormViewMode) state[3];
+			defaultMode = (FormViewMode) state[4];
+			dataKeyNames = (string[]) state[5];
 		}
 		
 		protected internal override object SaveControlState ()
 		{
 			object bstate = base.SaveControlState ();
 			return new object[] {
-				bstate, pageIndex, pageCount, currentMode
-					};
+				bstate, pageIndex, pageCount, currentMode, defaultMode, dataKeyNames
+			};
 		}
 		
 		protected override void TrackViewState()
@@ -1406,7 +1422,10 @@ namespace System.Web.UI.WebControls
 			}
 			
 			writer.AddAttribute (HtmlTextWriterAttribute.Cellspacing, "0");
-			writer.AddStyleAttribute (HtmlTextWriterStyle.BorderCollapse, "collapse");
+			if (!string.IsNullOrEmpty (ControlStyle.CssClass))
+				writer.AddAttribute (HtmlTextWriterAttribute.Class, ControlStyle.CssClass);
+
+			table.ControlStyle.MergeWith (ControlStyle);
 			table.RenderBeginTag (writer);
 			
 			foreach (FormViewRow row in table.Rows)
@@ -1444,10 +1463,10 @@ namespace System.Web.UI.WebControls
 			table.RenderEndTag (writer);
 		}
 
-		[MonoTODO]
 		PostBackOptions IPostBackContainer.GetPostBackOptions (IButtonControl control)
 		{
-			throw new NotImplementedException ();
+			Control ctrl = control as Control;
+			return new PostBackOptions(ctrl);
 		}
 
 	}
