@@ -174,19 +174,19 @@ namespace System.Web.UI.WebControls {
 			base.AddAttributesToRender (writer);
 #if NET_2_0
 			// src is always present, even if empty, in 2.0
-			writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveUrl (ImageUrl));
+			writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveClientUrl (ImageUrl));
 			string s = AlternateText;
 			if ((s.Length > 0) || GenerateEmptyAlternateText)
 				writer.AddAttribute (HtmlTextWriterAttribute.Alt, s);
 			s = DescriptionUrl;
 			if (s.Length > 0)
-				writer.AddAttribute (HtmlTextWriterAttribute.Longdesc, ResolveUrl (s));
+				writer.AddAttribute (HtmlTextWriterAttribute.Longdesc, ResolveClientUrl (s));
 #else
 			// alt is always present, even if empty, in 1.x
 			writer.AddAttribute (HtmlTextWriterAttribute.Alt, AlternateText);
 			string s = ImageUrl;
 			if (s.Length > 0)
-				writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveUrl (s));
+				writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveClientUrl (s));
 			// added in Fx 1.1 SP1 but the HtmlTextWriterAttribute wasn't
 			s = DescriptionUrl;
 			if (s.Length > 0)
@@ -224,12 +224,21 @@ namespace System.Web.UI.WebControls {
 				writer.AddAttribute (HtmlTextWriterAttribute.Align, "texttop");
 				break;
 			}
-
+#if NET_2_0
+#if BUG_78875_FIXED
+			if (Context.Request.Browser.SupportsCss)
+#endif
+			if (BorderWidth.IsEmpty)
+				writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, "0px");
+#if BUG_78875_FIXED
+			else
+#endif
+#else
 			// if border-with is not specified in style or 
 			// no style is defined - set image to no border
-			if (!ControlStyleCreated || ControlStyle.BorderWidth.IsEmpty) {
+			if (!ControlStyleCreated || ControlStyle.BorderWidth.IsEmpty)
 				writer.AddAttribute (HtmlTextWriterAttribute.Border, "0");
-			}
+#endif
 		}
 
 #if NET_2_0
