@@ -392,7 +392,7 @@ namespace System.Web.UI.WebControls
 		public string OldValuesParameterFormatString {
 			get {
 				object ret = ViewState ["OldValuesParameterFormatString"];
-				return ret != null ? (string)ret : "original_{0}";
+				return ret != null ? (string)ret : "{0}";
 			}
 			set {
 				ViewState ["OldValuesParameterFormatString"] = value;
@@ -552,6 +552,8 @@ namespace System.Web.UI.WebControls
 			if (rargs.Exception != null && !rargs.ExceptionHandled)
 				throw rargs.Exception;
 
+			OnDataSourceViewChanged (EventArgs.Empty);
+
 			return -1;
 		}
 
@@ -577,7 +579,7 @@ namespace System.Web.UI.WebControls
 			MethodInfo method;
 			
 			if (DataObjectTypeName.Length == 0) {
-				paramValues = MergeParameterValues (DeleteParameters, null, oldDataValues, true);
+				paramValues = MergeParameterValues (DeleteParameters, null, oldDataValues, false);
 				method = GetObjectMethod (DeleteMethod, paramValues);
 			} else {
 				method = ResolveDataObjectMethod (DeleteMethod, oldDataValues, null, out paramValues);
@@ -595,6 +597,8 @@ namespace System.Web.UI.WebControls
 			if (rargs.Exception != null && !rargs.ExceptionHandled)
 				throw rargs.Exception;
 
+			OnDataSourceViewChanged (EventArgs.Empty);
+
 			return -1;
 		}
 		
@@ -656,6 +660,8 @@ namespace System.Web.UI.WebControls
 			if (rargs.Exception != null && !rargs.ExceptionHandled)
 				throw rargs.Exception;
 
+			OnDataSourceViewChanged (EventArgs.Empty);
+
 			return -1;
 		}
 		
@@ -876,6 +882,15 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
+		/// <summary>
+		/// Merge the current data item fields with view parameter default values
+		/// </summary>
+		/// <param name="viewParams">default parameters</param>
+		/// <param name="values">new parameters for update and insert</param>
+		/// <param name="oldValues">old parameters for update and delete</param>
+		/// <param name="allwaysAddNewValues">true for insert, as current item is
+		/// irrelevant for insert</param>
+		/// <returns>merged values</returns>
 		IOrderedDictionary MergeParameterValues (ParameterCollection viewParams, IDictionary values, IDictionary oldValues, bool allwaysAddNewValues)
 		{
 			OrderedDictionary mergedValues = new OrderedDictionary ();
