@@ -356,7 +356,11 @@ namespace System.Web.UI.WebControls {
 		{
 			get 
 			{
+#if NET_2_0
+				return (styles == 0 && (fontinfo == null || fontinfo.IsEmpty) && RegisteredCssClass.Length == 0);
+#else
 				return (styles == 0 && (fontinfo == null || fontinfo.IsEmpty));
+#endif
 			}
 		}
 
@@ -379,6 +383,23 @@ namespace System.Web.UI.WebControls {
 		}
 		#endregion	// Protected Instance Properties
 
+		#region Internal Instance Properties
+		internal bool AlwaysRenderTextDecoration
+		{
+			get
+			{
+				if (viewstate["AlwaysRenderTextDecoration"] == null)
+					return false;
+				return (bool)viewstate["AlwaysRenderTextDecoration"];
+			}
+		    
+			set
+			{
+				viewstate["AlwaysRenderTextDecoration"] = value;
+			}
+		}
+		#endregion	// Internal Instance Properties
+		
 		#region Public Instance Methods
 		public void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer) 
 		{
@@ -387,19 +408,32 @@ namespace System.Web.UI.WebControls {
 
 		public virtual void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer, WebControl owner)
 		{
-			if ((styles & Styles.CssClass) != 0) 
+#if NET_2_0
+			if (RegisteredCssClass.Length > 0) {
+				if (CssClass.Length > 0)
+					writer.AddAttribute (HtmlTextWriterAttribute.Class, CssClass + " " + RegisteredCssClass);
+				else
+					writer.AddAttribute (HtmlTextWriterAttribute.Class, RegisteredCssClass);
+			}
+			else 
+#endif
 			{
-				string s = (string)viewstate["CssClass"];
-				if (s != string.Empty)
-					writer.AddAttribute (HtmlTextWriterAttribute.Class, s);
+				if (CssClass.Length > 0)
+					writer.AddAttribute (HtmlTextWriterAttribute.Class, CssClass);
+				WriteStyleAttributes (writer);
 			}
-
-			WriteStyleAttributes (writer);
 		}
 
 		void WriteStyleAttributes (HtmlTextWriter writer) 
 		{
-			string		s;
+#if NET_2_0
+			CssStyleCollection col = new CssStyleCollection (new StateBag ());
+			FillStyleAttributes (col, null);
+			foreach (string key in col.Keys) {
+				writer.AddStyleAttribute (key, col [key]);
+			}
+#else
+			string s;
 			Color		color;
 			BorderStyle	bs;
 			Unit		u;
@@ -420,7 +454,8 @@ namespace System.Web.UI.WebControls {
 			if ((styles & Styles.BorderWidth) != 0) {
 				u = (Unit)viewstate["BorderWidth"];
 				if (!u.IsEmpty) {
-					have_width = true;
+					if (u.Value > 0)
+						have_width = true;
 					writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, u.ToString());
 				}
 			}
@@ -455,7 +490,7 @@ namespace System.Web.UI.WebControls {
 					writer.AddStyleAttribute (HtmlTextWriterStyle.Width, u.ToString());
 			}
 
-			if (fontinfo != null) {
+			if (!Font.IsEmpty) {
 				// Fonts are a bit weird
 				if (fontinfo.Name != string.Empty) {
 					s = fontinfo.Names[0];
@@ -485,14 +520,16 @@ namespace System.Web.UI.WebControls {
 				if (fontinfo.Underline)
 					s += "underline ";
 
-				if (s != string.Empty)
+				s = (s != "") ? s : AlwaysRenderTextDecoration ? "none" : "";
+				if (s != "")
 					writer.AddStyleAttribute (HtmlTextWriterStyle.TextDecoration, s);
 			}
+#endif
 		}
 
+#if NET_2_0
 		void FillStyleAttributes (CssStyleCollection attributes) 
 		{
-			string		s;
 			Color		color;
 			BorderStyle	bs;
 			Unit		u;
@@ -511,18 +548,25 @@ namespace System.Web.UI.WebControls {
 					attributes.Add (HtmlTextWriterStyle.BorderColor, ColorTranslator.ToHtml(color));
 			}
 
-			if ((styles & Styles.BorderStyle) != 0) 
-			{
-				bs = (BorderStyle)viewstate["BorderStyle"];
-				if (bs != BorderStyle.NotSet) 
-					attributes.Add (HtmlTextWriterStyle.BorderStyle, bs.ToString());
+			bool have_width = false;
+			if ((styles & Styles.BorderWidth) != 0) {
+				u = (Unit) viewstate ["BorderWidth"];
+				if (!u.IsEmpty) {
+					if (u.Value > 0)
+						have_width = true;
+					attributes.Add (HtmlTextWriterStyle.BorderWidth, u.ToString ());
+				}
 			}
 
-			if ((styles & Styles.BorderWidth) != 0) 
-			{
-				u = (Unit)viewstate["BorderWidth"];
-				if (!u.IsEmpty)
-					attributes.Add (HtmlTextWriterStyle.BorderWidth, u.ToString());
+			if ((styles & Styles.BorderStyle) != 0) {
+				bs = (BorderStyle) viewstate ["BorderStyle"];
+				if (bs != BorderStyle.NotSet)
+					attributes.Add (HtmlTextWriterStyle.BorderStyle, bs.ToString ());
+				else if (have_width)
+						attributes.Add (HtmlTextWriterStyle.BorderStyle, "solid");
+			}
+			else if (have_width) {
+				attributes.Add (HtmlTextWriterStyle.BorderStyle, "solid");
 			}
 
 			if ((styles & Styles.ForeColor) != 0) 
@@ -546,57 +590,9 @@ namespace System.Web.UI.WebControls {
 					attributes.Add (HtmlTextWriterStyle.Width, u.ToString());
 			}
 
-			if (fontinfo != null) {
-				// Fonts are a bit weird
-				if (fontinfo.Name != string.Empty) 
-				{
-					s = fontinfo.Names[0];
-					for (int i = 1; i < fontinfo.Names.Length; i++) 
-					{
-						s += "," + fontinfo.Names[i];
-					}
-					attributes.Add (HtmlTextWriterStyle.FontFamily, s);
-				}
-
-				if (fontinfo.Bold) 
-				{
-					attributes.Add (HtmlTextWriterStyle.FontWeight, "bold");
-				}
-
-				if (fontinfo.Italic) 
-				{
-					attributes.Add (HtmlTextWriterStyle.FontStyle, "italic");
-				}
-
-				if (!fontinfo.Size.IsEmpty) 
-				{
-					attributes.Add (HtmlTextWriterStyle.FontSize, fontinfo.Size.ToString());
-				}
-
-				// These styles are munged into a attribute decoration
-				s = string.Empty;
-
-				if (fontinfo.Overline) 
-				{
-					s += "overline ";
-				}
-
-				if (fontinfo.Strikeout) 
-				{
-					s += "line-through ";
-				}
-
-				if (fontinfo.Underline) 
-				{
-					s += "underline ";
-				}
-
-				if (s != string.Empty) 
-				{
-					attributes.Add (HtmlTextWriterStyle.TextDecoration, s);
-				}
-			}
+			Font.FillStyleAttributes (attributes, AlwaysRenderTextDecoration);
 		}
+#endif
 
 		public virtual void CopyFrom(Style s) 
 		{
@@ -768,9 +764,7 @@ namespace System.Web.UI.WebControls {
 			{
 				styles |= Styles.Width;
 			}
-			if (fontinfo != null) {
-				fontinfo.LoadViewState();
-			}
+			Font.LoadViewState();
 
 			LoadViewStateInternal();
 		}
@@ -782,7 +776,7 @@ namespace System.Web.UI.WebControls {
 
 		protected internal virtual object SaveViewState () 
 		{
-			if (styles != Styles.None) 
+			if (styles != Styles.None || !Font.IsEmpty) 
 			{
 				return viewstate.SaveViewState();
 			}
