@@ -224,15 +224,15 @@ namespace System.Web.UI.WebControls {
 				writer.AddAttribute (HtmlTextWriterAttribute.Border, BorderWidth.Value.ToString (CultureInfo.InvariantCulture));
 			}
 
+#if !NET_2_0
 			string s = BackImageUrl;
 			if (s.Length > 0) {
 				if (owner != null)
-					s = owner.ResolveUrl (s);
-#if ONLY_1_1
+					s = owner.ResolveClientUrl (s);
 				s = String.Concat ("url(", s, ")");
-#endif
 				writer.AddStyleAttribute (HtmlTextWriterStyle.BackgroundImage, s);
 			}
+#endif
 		}
 
 		private void Copy (string name, Styles s, Style source)
