@@ -189,10 +189,10 @@ namespace System.Web.UI.WebControls
 	    [BrowsableAttribute (false)]
 		public WizardStepBase ActiveStep {
 			get {
-				if (activeStepIndex < -1 || activeStepIndex >= steps.Count)
+				if (ActiveStepIndex < -1 || ActiveStepIndex >= WizardSteps.Count)
 					throw new InvalidOperationException ("ActiveStepIndex has an invalid value.");
-				if (activeStepIndex == -1) return null;
-				return steps [activeStepIndex];
+				if (ActiveStepIndex == -1) return null;
+				return WizardSteps [activeStepIndex];
 			}
 		}
 		
@@ -200,6 +200,9 @@ namespace System.Web.UI.WebControls
 	    [ThemeableAttribute (false)]
 		public virtual int ActiveStepIndex {
 			get {
+				if (WizardSteps.Count == 0)
+					return -1;
+
 				return activeStepIndex;
 			}
 			set {
@@ -569,11 +572,14 @@ namespace System.Web.UI.WebControls
 		[MonoTODO]
 		public virtual string SkipLinkText 
 		{
-			get {
-				throw new NotImplementedException ();
+			get
+			{
+				object v = ViewState ["SkipLinkText"];
+				return v != null ? (string) v : "Skip Navigation Links.";
 			}
-			set {
-				throw new NotImplementedException ();
+			set
+			{
+				ViewState ["SkipLinkText"] = value;
 			}
 		}
 		
@@ -785,9 +791,14 @@ namespace System.Web.UI.WebControls
 		protected virtual new HtmlTextWriterTag TagKey
 		{
 			get {
-				throw new NotImplementedException ();
+				return HtmlTextWriterTag.Table;
 			}
 		}
+
+		internal virtual ITemplate SideBarItemTemplate
+		{
+			get { return sideBarItemTemplate; }
+		}
 		
 		public ICollection GetHistory ()
 		{
@@ -808,10 +819,10 @@ namespace System.Web.UI.WebControls
 		public WizardStepType GetStepType (WizardStepBase wizardStep, int index)
 		{
 			if (wizardStep.StepType == WizardStepType.Auto) {
-				if (index == 0)
-					return WizardStepType.Start;
-				else if (index == WizardSteps.Count - 1)
+				if (index == WizardSteps.Count - 1)
 					return WizardStepType.Finish;
+				else if (index == 0)
+					return WizardStepType.Start;
 				else
 					return WizardStepType.Step;
 			} else
@@ -852,29 +863,34 @@ namespace System.Web.UI.WebControls
 			wizardTable = new Table ();
 			wizardTable.CellPadding = CellPadding; 
 			wizardTable.CellSpacing = CellSpacing; 
+			wizardTable.ID = this.ID;
 			
 			AddHeaderRow (wizardTable);
 			
 			TableRow viewRow = new TableRow ();
 			TableCell viewCell = new TableCell ();
-			
-			if (multiView == null) {
-				multiView = new MultiView ();
-				foreach (View v in WizardSteps)
+
+			if (multiView == null)
+			{
+				multiView = new MultiView();
+				foreach (View v in WizardSteps) {
+					if (v is TemplatedWizardStep) 
+						InstantiateTemplateStep ((TemplatedWizardStep) v);
 					multiView.Views.Add (v);
+				}
 			}
 			
 			multiView.ActiveViewIndex = activeStepIndex;
 			
 			RegisterApplyStyle (viewCell, StepStyle);
 			viewCell.Controls.Add (multiView);
-			
-			viewCell.Height = new Unit ("100%");
 			viewRow.Cells.Add (viewCell);
+			viewRow.Height = new Unit ("100%");
 			wizardTable.Rows.Add (viewRow);
 			
 			TableRow buttonRow = new TableRow ();
 			TableCell buttonCell = new TableCell ();
+			buttonCell.HorizontalAlign = HorizontalAlign.Right;
 			CreateButtonBar (buttonCell);
 			buttonRow.Cells.Add (buttonCell);
 			wizardTable.Rows.Add (buttonRow);
@@ -882,18 +898,21 @@ namespace System.Web.UI.WebControls
 			if (DisplaySideBar && ActiveStep.StepType != WizardStepType.Complete) {
 				Table contentTable = wizardTable;
 				contentTable.Height = new Unit ("100%");
+				contentTable.Width = new Unit ("100%");
 				
 				wizardTable = new Table ();
 				wizardTable.CellPadding = CellPadding; 
 				wizardTable.CellSpacing = CellSpacing;
 				TableRow row = new TableRow ();
-				
-				TableCell sideBarCell = new TableCell ();
+
+				TableCellNamingContainer sideBarCell = new TableCellNamingContainer ();
+				sideBarCell.ControlStyle.Height = Unit.Percentage (100);
 				CreateSideBar (sideBarCell);
 				row.Cells.Add (sideBarCell);
 				
 				TableCell contentCell = new TableCell ();
 				contentCell.Controls.Add (contentTable);
+				contentCell.Height = new Unit ("100%");
 				row.Cells.Add (contentCell);
 				
 				wizardTable.Rows.Add (row);
@@ -903,10 +922,34 @@ namespace System.Web.UI.WebControls
 			Controls.Add (wizardTable);
 			Controls.SetReadonly (true);
 		}
+
+		internal virtual void InstantiateTemplateStep(TemplatedWizardStep step)
+		{
+			step.InstantiateInContainer ();
+
+			if (step.CustomNavigationTemplate != null) {
+				WizardStepType stepType = GetStepType (step, ActiveStepIndex);
+				switch (stepType) {
+				case WizardStepType.Start:
+					startNavigationTemplate = step.CustomNavigationTemplate;
+					break;
+
+				case WizardStepType.Step:
+					stepNavigationTemplate = step.CustomNavigationTemplate;
+					break;
+
+				case WizardStepType.Finish:
+					finishNavigationTemplate = step.CustomNavigationTemplate;
+					break;
+				}
+			}
+		}
 		
 		void CreateButtonBar (TableCell buttonBarCell)
 		{
 			Table t = new Table ();
+			t.CellPadding = 5;
+			t.CellSpacing = 5;
 			TableRow row = new TableRow ();
 			RegisterApplyStyle (buttonBarCell, NavigationStyle);
 			
@@ -914,35 +957,38 @@ namespace System.Web.UI.WebControls
 			switch (stepType) {
 				case WizardStepType.Start:
 					if (startNavigationTemplate != null) {
-						AddTemplateCell (row, startNavigationTemplate, StartNextButtonID, CancelButtonID);
+						AddTemplateButtonBar (buttonBarCell, startNavigationTemplate, StartNextButtonID, CancelButtonID);
+						return;
 					} else {
-						if (DisplayCancelButton)
-							AddButtonCell (row, CreateButton (CancelButtonID, CancelCommandName, CancelButtonType, CancelButtonText, CancelButtonImageUrl, CancelButtonStyle));
 						if (AllowNavigationToStep (ActiveStepIndex + 1))
 							AddButtonCell (row, CreateButton (StartNextButtonID, MoveNextCommandName, StartNextButtonType, StartNextButtonText, StartNextButtonImageUrl, StartNextButtonStyle));
+						if (DisplayCancelButton)
+							AddButtonCell (row, CreateButton (CancelButtonID, CancelCommandName, CancelButtonType, CancelButtonText, CancelButtonImageUrl, CancelButtonStyle));
 					}
 					break;
 				case WizardStepType.Step:
 					if (stepNavigationTemplate != null) {
-						AddTemplateCell (row, stepNavigationTemplate, StepPreviousButtonID, StepNextButtonID, CancelButtonID);
+						AddTemplateButtonBar (buttonBarCell, stepNavigationTemplate, StepPreviousButtonID, StepNextButtonID, CancelButtonID);
+						return;
 					} else {
-						if (DisplayCancelButton)
-							AddButtonCell (row, CreateButton (CancelButtonID, CancelCommandName, CancelButtonType, CancelButtonText, CancelButtonImageUrl, CancelButtonStyle));
 						if (AllowNavigationToStep (ActiveStepIndex - 1))
 							AddButtonCell (row, CreateButton (StepPreviousButtonID, MovePreviousCommandName, StepPreviousButtonType, StepPreviousButtonText, StepPreviousButtonImageUrl, StepPreviousButtonStyle));
 						if (AllowNavigationToStep (ActiveStepIndex + 1))
 							AddButtonCell (row, CreateButton (StepNextButtonID, MoveNextCommandName, StepNextButtonType, StepNextButtonText, StepNextButtonImageUrl, StepNextButtonStyle));
+						if (DisplayCancelButton)
+							AddButtonCell (row, CreateButton (CancelButtonID, CancelCommandName, CancelButtonType, CancelButtonText, CancelButtonImageUrl, CancelButtonStyle));
 					}
 					break;
 				case WizardStepType.Finish:
 					if (finishNavigationTemplate != null) {
-						AddTemplateCell (row, finishNavigationTemplate, FinishPreviousButtonID, FinishButtonID, CancelButtonID);
+						AddTemplateButtonBar (buttonBarCell, finishNavigationTemplate, FinishPreviousButtonID, FinishButtonID, CancelButtonID);
+						return;
 					} else {
-						if (DisplayCancelButton)
-							AddButtonCell (row, CreateButton (CancelButtonID, CancelCommandName, CancelButtonType, CancelButtonText, CancelButtonImageUrl, CancelButtonStyle));
 						if (AllowNavigationToStep (ActiveStepIndex - 1))
 							AddButtonCell (row, CreateButton (FinishPreviousButtonID, MovePreviousCommandName, FinishPreviousButtonType, FinishPreviousButtonText, FinishPreviousButtonImageUrl, FinishPreviousButtonStyle));
 						AddButtonCell (row, CreateButton (FinishButtonID, MoveCompleteCommandName, FinishCompleteButtonType, FinishCompleteButtonText, FinishCompleteButtonImageUrl, FinishCompleteButtonStyle));
+						if (DisplayCancelButton)
+							AddButtonCell (row, CreateButton (CancelButtonID, CancelCommandName, CancelButtonType, CancelButtonText, CancelButtonImageUrl, CancelButtonStyle));
 					}
 					break;
 			}
@@ -960,22 +1006,20 @@ namespace System.Web.UI.WebControls
 			return b;
 		}
 		
-		void AddTemplateCell (TableRow row, ITemplate template, params string[] buttonIds)
+		void AddTemplateButtonBar (TableCell cell, ITemplate template, params string[] buttonIds)
 		{
-			TableCell cell = new TableCell ();
 			template.InstantiateIn (cell);
 			
 			foreach (string id in buttonIds) {
 				IButtonControl b = cell.FindControl (id) as IButtonControl;
 				if (b != null) RegisterCommandEvents (b);
 			}
-			
-			row.Cells.Add (cell);
 		}
 		
 		void AddButtonCell (TableRow row, Control control)
 		{
 			TableCell cell = new TableCell ();
+			cell.HorizontalAlign = HorizontalAlign.Right;
 			cell.Controls.Add (control);
 			row.Cells.Add (cell);
 		}
@@ -983,22 +1027,57 @@ namespace System.Web.UI.WebControls
 		void CreateSideBar (TableCell sideBarCell)
 		{
 			RegisterApplyStyle (sideBarCell, SideBarStyle);
-			
+
+			if (SkipLinkText != "") {
+				System.Web.UI.HtmlControls.HtmlAnchor anchor = new System.Web.UI.HtmlControls.HtmlAnchor ();
+				anchor.HRef = "#" + ClientID + "_SkipLink";
+
+				Image img = new Image ();
+				ClientScriptManager csm = new ClientScriptManager (null);
+				img.ImageUrl = csm.GetWebResourceUrl (typeof (SiteMapPath), "transparent.gif");
+				img.Attributes.Add ("height", "0");
+				img.Attributes.Add ("width", "0");
+				img.AlternateText = SkipLinkText;
+
+				anchor.Controls.Add (img);
+				sideBarCell.Controls.Add (anchor);
+			}
+
 			if (sideBarTemplate != null) {
 				sideBarTemplate.InstantiateIn (sideBarCell);
 				stepDatalist = sideBarCell.FindControl (DataListID) as DataList;
+				stepDatalist.ItemDataBound += new DataListItemEventHandler(StepDatalistItemDataBound);
 				if (stepDatalist == null)
 					throw new InvalidOperationException ("The side bar template must contain a DataList control with id '" + DataListID + "'.");
 			} else {
 				stepDatalist = new DataList ();
 				stepDatalist.ID = DataListID;
+				stepDatalist.ItemTemplate = SideBarItemTemplate;
 				sideBarCell.Controls.Add (stepDatalist);
 			}
 
+			if (SkipLinkText != "") {
+				System.Web.UI.HtmlControls.HtmlAnchor anchor = new System.Web.UI.HtmlControls.HtmlAnchor ();
+				anchor.ID = "SkipLink";
+				sideBarCell.Controls.Add (anchor);
+			}
+
+			stepDatalist.CellSpacing = 0;
 			stepDatalist.DataSource = WizardSteps;
-			stepDatalist.ItemTemplate = sideBarItemTemplate;
 			stepDatalist.DataBind ();
 		}
+
+		void StepDatalistItemDataBound (object sender, DataListItemEventArgs e)
+		{
+			if (e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem) {
+				IButtonControl button = (IButtonControl) e.Item.FindControl (SideBarButtonID);
+				WizardStep step = (WizardStep) e.Item.DataItem;
+
+				button.CommandName = Wizard.MoveToCommandName;
+				button.CommandArgument = WizardSteps.IndexOf (step).ToString ();
+				button.Text = step.Name;
+			}
+		}
 		
 		void AddHeaderRow (Table table)
 		{
@@ -1022,7 +1101,10 @@ namespace System.Web.UI.WebControls
 		
 		protected override Style CreateControlStyle ()
 		{
-			return new TableStyle ();
+			TableStyle style = new TableStyle ();
+			style.CellPadding = 0;
+			style.CellSpacing = 0;
+			return style;
 		}
 
 		[MonoTODO]
@@ -1146,6 +1228,7 @@ namespace System.Web.UI.WebControls
 			CommandEventArgs args = e as CommandEventArgs;
 			if (args != null) {
 				ProcessEvent (args.CommandName, args.CommandArgument as string);
+				return true;
 			}
 			return base.OnBubbleEvent (source, e);
 		}
@@ -1219,6 +1302,9 @@ namespace System.Web.UI.WebControls
 		
 		protected internal override void Render (HtmlTextWriter writer)
 		{
+			if (multiView.ActiveViewIndex != ActiveStepIndex)
+				CreateControlHierarchy ();
+
 			wizardTable.ApplyStyle (ControlStyle);
 
 			foreach (object[] styleDef in styles)
@@ -1226,7 +1312,11 @@ namespace System.Web.UI.WebControls
 			
 			wizardTable.Render (writer);
 		}
-		
+
+		class TableCellNamingContainer : TableCell, INamingContainer
+		{
+		}
+
 		class SideBarButtonTemplate: ITemplate
 		{
 			Wizard wizard;
@@ -1248,16 +1338,16 @@ namespace System.Web.UI.WebControls
 			{
 				WizardStepBase step = DataBinder.GetDataItem (s) as WizardStepBase;
 				if (step != null) {
-					Control c = (Control)s;
+					DataListItem c = (DataListItem) s;
 					LinkButton b = (LinkButton) c.Controls[0];
 					b.ID = SideBarButtonID;
 					b.CommandName = Wizard.MoveToCommandName;
 					b.CommandArgument = wizard.WizardSteps.IndexOf (step).ToString ();
-					b.Text = step.Title;
+					b.Text = step.Name;
 					if (step.StepType == WizardStepType.Complete)
 						b.Enabled = false;
 					if (step == wizard.ActiveStep)
-						b.Font.Bold = true;
+						c.Font.Bold = true;
 					wizard.RegisterCommandEvents (b);
 				}
 			}
