@@ -38,6 +38,7 @@ using System.Xml;
 using System.Xml.Xsl;
 using System.ComponentModel;
 using System.IO;
+using System.Web.Caching;
 using System.Security.Permissions;
 
 namespace System.Web.UI.WebControls {
@@ -55,6 +56,15 @@ namespace System.Web.UI.WebControls {
 //	[WebSysDisplayName ("XML file")]
 	public class XmlDataSource : HierarchicalDataSourceControl, IDataSource, IListSource {
 
+		private string _data = string.Empty;
+		private string _transform = string.Empty;
+		private string _xpath = string.Empty;
+		private string _dataFile = string.Empty;
+		private string _transformFile = string.Empty;
+		private string _cacheKeyDependency = string.Empty;
+		private bool _enableCaching = true;
+		private int _cacheDuration = 0;
+		private DataSourceCacheExpiry _cacheExpirationPolicy = DataSourceCacheExpiry.Absolute;
 		
 		event EventHandler IDataSource.DataSourceChanged {
 			add { ((IHierarchicalDataSource)this).DataSourceChanged += value; }
@@ -77,26 +87,98 @@ namespace System.Web.UI.WebControls {
 		XmlDocument xmlDocument;
 		public XmlDocument GetXmlDocument ()
 		{
+			if (xmlDocument == null && EnableCaching)
+				xmlDocument = GetXmlDocumentFromCache ();
+
 			if (xmlDocument == null) {
-				xmlDocument = new XmlDocument ();
-				LoadXmlDocument (xmlDocument);
+				xmlDocument = LoadXmlDocument ();
+				UpdateCache ();
 			}
+
 			return xmlDocument;
 		}
-		
-		[MonoTODO ("XSLT, schema")]
-		void LoadXmlDocument (XmlDocument document)
+
+		[MonoTODO ("schema")]
+		XmlDocument LoadXmlDocument ()
 		{
-			if (Transform == "" && TransformFile == "") {
-				if (DataFile != "")
-					document.Load (MapPathSecure (DataFile));
-				else
-					document.LoadXml (Data);
-			} else {
-				throw new NotImplementedException ("XSLT transform not implemented");
+			XmlDocument document = LoadFileOrData (DataFile, Data);
+
+			if (TransformFile == "" && Transform == "")
+				return document;
+
+			XslTransform xslTransform = new XslTransform ();
+			XmlDocument xsl = LoadFileOrData (TransformFile, Transform);
+			xslTransform.Load (xsl);
+
+			OnTransforming (EventArgs.Empty);
+
+			XmlDocument transofrResult = new XmlDocument ();
+			transofrResult.Load (xslTransform.Transform (document, TransformArgumentList));
+
+			return transofrResult;
+		}
+
+		XmlDocument LoadFileOrData (string filename, string data)
+		{
+			XmlDocument document = new XmlDocument ();
+			if (filename != "")
+				document.Load (MapPathSecure (filename));
+			else
+				document.LoadXml (data);
+
+			return document;
+		}
+
+		XmlDocument GetXmlDocumentFromCache ()
+		{
+			if (DataCache != null)
+				return (XmlDocument) DataCache [GetDataKey ()];
+
+			return null;
+		}
+
+		string GetDataKey ()
+		{
+			return TemplateSourceDirectory + "_" + Page.ToString () + "_" + ID;
+		}
+
+		Cache DataCache
+		{
+			get
+			{
+				if (HttpContext.Current != null)
+					return HttpContext.Current.Cache;
+
+				return null;
 			}
 		}
 
+		void UpdateCache ()
+		{
+			if (!EnableCaching)
+				return;
+
+			if (DataCache == null)
+				return;
+
+			if (DataCache [GetDataKey ()] != null)
+				DataCache.Remove (GetDataKey ());
+
+			DateTime absoluteExpiration = Cache.NoAbsoluteExpiration;
+			TimeSpan slidindExpiraion = Cache.NoSlidingExpiration;
+
+			if (CacheDuration > 0) {
+				if (CacheExpirationPolicy == DataSourceCacheExpiry.Absolute)
+					absoluteExpiration = DateTime.Now.AddSeconds (CacheDuration);
+				else
+					slidindExpiraion = new TimeSpan (CacheDuration * 10000);
+			}
+
+			DataCache.Add (GetDataKey (), xmlDocument,
+				new CacheDependency (new string [] { }, new string [] { CacheKeyDependency }),
+				absoluteExpiration, slidindExpiraion, CacheItemPriority.Default, null);
+		}
+
 		public void Save ()
 		{
 			if (!CanBeSaved)
@@ -156,10 +238,10 @@ namespace System.Web.UI.WebControls {
 		[MonoTODO]
 		public virtual int CacheDuration {
 			get {
-				throw new NotImplementedException ();
+				return _cacheDuration;
 			}
 			set {
-				throw new NotImplementedException ();
+				_cacheDuration = value;
 			}
 		}
 
@@ -167,10 +249,10 @@ namespace System.Web.UI.WebControls {
 		[MonoTODO]
 		public virtual DataSourceCacheExpiry CacheExpirationPolicy {
 			get {
-				throw new NotImplementedException ();
+				return _cacheExpirationPolicy;
 			}
 			set {
-				throw new NotImplementedException ();
+				_cacheExpirationPolicy = value;
 			}
 		}
 
@@ -178,10 +260,10 @@ namespace System.Web.UI.WebControls {
 		[MonoTODO]
 		public virtual string CacheKeyDependency {
 			get {
-				throw new NotImplementedException ();
+				return _cacheKeyDependency;
 			}
 			set {
-				throw new NotImplementedException ();
+				_cacheKeyDependency = value;
 			}
 		}
 
@@ -189,10 +271,10 @@ namespace System.Web.UI.WebControls {
 		[MonoTODO]
 		public virtual bool EnableCaching {
 			get {
-				throw new NotImplementedException ();
+				return _enableCaching;
 			}
 			set {
-				throw new NotImplementedException ();
+				_enableCaching = value;
 			}
 		}
 
@@ -203,13 +285,10 @@ namespace System.Web.UI.WebControls {
 		[EditorAttribute ("System.ComponentModel.Design.MultilineStringEditor," + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 //		[TypeConverter (typeof(MultilineStringConverter))]
 		public virtual string Data {
-			get {
-				string ret = ViewState ["Data"] as string;
-				return ret != null ? ret : "";
-			}
+			get { return _data; }
 			set {
-				if (Data != value) {
-					ViewState ["Data"] = value;
+				if (_data != value) {
+					_data = value;
 					xmlDocument = null;
 					OnDataSourceChanged(EventArgs.Empty);
 				}
@@ -219,13 +298,10 @@ namespace System.Web.UI.WebControls {
 		[DefaultValueAttribute ("")]
 		[EditorAttribute ("System.Web.UI.Design.XmlDataFileEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public virtual string DataFile {
-			get {
-				string ret = ViewState ["DataFile"] as string;
-				return ret != null ? ret : "";
-			}
+			get { return _dataFile; }
 			set {
-				if (DataFile != value) {
-					ViewState ["DataFile"] = value;
+				if (_dataFile != value) {
+					_dataFile = value;
 					xmlDocument = null;
 					OnDataSourceChanged(EventArgs.Empty);
 				}
@@ -245,13 +321,10 @@ namespace System.Web.UI.WebControls {
 		[DefaultValueAttribute ("")]
 //		[TypeConverterAttribute (typeof(System.ComponentModel.MultilineStringConverter))]
 		public virtual string Transform {
-			get {
-				string ret = ViewState ["Transform"] as string;
-				return ret != null ? ret : "";
-			}
+			get { return _transform; }
 			set {
-				if (Transform != value) {
-					ViewState ["Transform"] = value;
+				if (_transform != value) {
+					_transform = value;
 					xmlDocument = null;
 					OnDataSourceChanged(EventArgs.Empty);
 				}
@@ -261,13 +334,10 @@ namespace System.Web.UI.WebControls {
 		[EditorAttribute ("System.Web.UI.Design.XslTransformFileEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		[DefaultValueAttribute ("")]
 		public virtual string TransformFile {
-			get {
-				string ret = ViewState ["TransformFile"] as string;
-				return ret != null ? ret : "";
-			}
+			get { return _transformFile; }
 			set {
-				if (TransformFile != value) {
-					ViewState ["TransformFile"] = value;
+				if (_transformFile != value) {
+					_transformFile = value;
 					xmlDocument = null;
 					OnDataSourceChanged(EventArgs.Empty);
 				}
@@ -276,13 +346,10 @@ namespace System.Web.UI.WebControls {
 		
 		[DefaultValueAttribute ("")]
 		public virtual string XPath {
-			get {
-				string ret = ViewState ["XPath"] as string;
-				return ret != null ? ret : "";
-			}
+			get { return _xpath; }
 			set {
-				if (XPath != value) {
-					ViewState ["XPath"] = value;
+				if (_xpath != value) {
+					_xpath = value;
 					OnDataSourceChanged(EventArgs.Empty);
 				}
 			}
