@@ -141,9 +141,6 @@ namespace System.Web.UI
 			stateMask = ENABLE_VIEWSTATE | VISIBLE | AUTOID | BINDING_CONTAINER | AUTO_EVENT_WIREUP;
                         if (this is INamingContainer)
 				stateMask |= IS_NAMING_CONTAINER;
-#if NET_2_0
-			stateMask |= ENABLE_THEMING;
-#endif
                 }
 
 #if NET_2_0
@@ -162,18 +159,16 @@ namespace System.Web.UI
 			}
 		}
 
+		string _appRelativeTemplateSourceDirectory = "~/";
+
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		[Browsable (false)]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		public string AppRelativeTemplateSourceDirectory 
 		{
-			get {
-				throw new NotImplementedException ();
-			}
+			get { return _appRelativeTemplateSourceDirectory; }
 			[EditorBrowsable (EditorBrowsableState.Never)]
-			set {
-				throw new NotImplementedException ();
-			}
+			set	{ _appRelativeTemplateSourceDirectory = value; }
 		}
 		
 #endif		
@@ -183,7 +178,7 @@ namespace System.Web.UI
 		public Control BindingContainer {
 			get {
 				Control container = NamingContainer;
-				if ((container.stateMask & BINDING_CONTAINER) == 0)
+				if (container != null && (container.stateMask & BINDING_CONTAINER) == 0)
 					container = container.BindingContainer;
 				return container;
 			}
@@ -281,7 +276,14 @@ namespace System.Web.UI
 		protected internal bool IsViewStateEnabled 
 		{
 			get {
-				throw new NotImplementedException ();
+				if (Page == null)
+					return false;
+
+				for (Control control = this; control != null; control = control.Parent)
+					if (!control.EnableViewState)
+						return false;
+
+				return true;
 			}
 		}
 
@@ -601,7 +603,7 @@ namespace System.Web.UI
 				return;
 
 			/* apply the style sheet skin here */
-			if (page.StyleSheetTheme != null) {
+			if (page.StyleSheetPageTheme != null) {
 				ControlSkin cs = page.StyleSheetPageTheme.GetControlSkin (GetType(), SkinID);
 				if (cs != null)
 					cs.ApplySkin (this);
@@ -1125,6 +1127,34 @@ namespace System.Web.UI
 			HttpResponse resp = Context.Response;
 			return resp.ApplyAppPathModifier (UrlUtils.Combine (ts, relativeUrl));
 		}
+
+
+#if NET_2_0		
+		public
+#else
+		internal
+#endif
+		string ResolveClientUrl (string relativeUrl)
+		{
+			if (relativeUrl == null)
+				throw new ArgumentNullException ("relativeUrl");
+
+			if (relativeUrl == "")
+				return "";
+
+			if (relativeUrl [0] == '#')
+				return relativeUrl;
+			
+			string ts = TemplateSourceDirectory;
+			if (ts == "" || !UrlUtils.IsRelativeUrl (relativeUrl))
+				return relativeUrl;
+
+			HttpResponse resp = Context.Response;
+			string absoluteUrl = resp.ApplyAppPathModifier (UrlUtils.Combine (ts, relativeUrl));
+			if (absoluteUrl.StartsWith (ts + "/"))
+				return absoluteUrl.Substring (ts.Length + 1);
+			return absoluteUrl;
+		}
 		
 		internal bool HasRenderMethodDelegate () {
 			return _renderMethodDelegate != null;
@@ -1246,7 +1276,6 @@ namespace System.Web.UI
 				trace.Write ("control", String.Format ("InitRecursive {0} {1}", _userId, type_name));
 			}
 #endif
-
                         if (HasControls ()) {
 				if ((stateMask & IS_NAMING_CONTAINER) != 0)
 					namingContainer = this;
@@ -1270,6 +1299,8 @@ namespace System.Web.UI
 
 			stateMask |= INITING;
 #if NET_2_0
+			ApplyTheme ();
+			
 			if (Adapter != null)
 				Adapter.OnInit (EventArgs.Empty);
 			else
@@ -1388,7 +1419,7 @@ namespace System.Web.UI
 #if NET_2_0
 		internal ControlSkin controlSkin;
 
-                internal void ApplyThemeRecursive ()
+                internal void ApplyTheme ()
                 {
 #if MONO_TRACE
 			TraceContext trace = (Context != null && Context.Trace.IsEnabled) ? Context.Trace : null;
@@ -1398,17 +1429,10 @@ namespace System.Web.UI
 				trace.Write ("control", String.Format ("ApplyThemeRecursive {0} {1}", _userId, type_name));
 			}
 #endif
-			ControlSkin controlSkin = Page.PageTheme.GetControlSkin (GetType(), SkinID);
-			if (controlSkin != null)
-				controlSkin.ApplySkin (this);
-
-                        if (HasControls ()) {
-				int len = _controls.Count;
-				for (int i=0;i<len;i++)
-				{
-					Control c = _controls[i];
-					c.ApplyThemeRecursive ();
-				}
+			if (Page.PageTheme != null && EnableTheming) {
+				ControlSkin controlSkin = Page.PageTheme.GetControlSkin (GetType (), SkinID);
+				if (controlSkin != null)
+					controlSkin.ApplySkin (this);
 			}
 
 #if MONO_TRACE
@@ -1441,14 +1465,28 @@ namespace System.Web.UI
 #if NET_2_0
 
 		string skinId = string.Empty;
+		bool _enableTheming = true;
 		
 		[Browsable (false)]
 		[Themeable (false)]
 		[DefaultValue (true)]
 		public virtual bool EnableTheming
 		{
-			get { return (stateMask & ENABLE_THEMING) != 0; }
-			set { SetMask (ENABLE_THEMING, value); }
+			get
+			{
+				if ((stateMask & ENABLE_THEMING) != 0)
+					return _enableTheming;
+
+				if (_parent != null)
+					return _parent.EnableTheming;
+
+				return true;
+			}
+			set 
+			{ 
+				SetMask (ENABLE_THEMING, true);
+				_enableTheming = value;
+			}
 		}
 		
 		[Browsable (false)]
@@ -1459,11 +1497,6 @@ namespace System.Web.UI
 			get { return skinId; }
 			set { skinId = value; }
 		}
-		
-		public string ResolveClientUrl (string url)
-		{
-			throw new NotImplementedException ();               
-		}
 
 		ControlBuilder IControlBuilderAccessor.ControlBuilder { 
 			get {throw new NotImplementedException (); }
