@@ -204,11 +204,11 @@ namespace System.Web.UI {
 			// faster than a linear search?
 			
 			foreach (HtmlAttribute t in htmlattrs) {
-				if (t.name == attrName)
+				if (String.Compare(t.name, attrName, true, CultureInfo.InvariantCulture) == 0)
 					return t.key;
 			}
 
-			return 0;		
+			return (HtmlTextWriterAttribute)(-1);		
 		}
 
 		[MonoTODO]
@@ -229,11 +229,11 @@ namespace System.Web.UI {
 			// faster than a linear search?
 			
 			foreach (HtmlStyle t in htmlstyles) {
-				if (t.name == styleName)
+				if (String.Compare(t.name, styleName, true, CultureInfo.InvariantCulture) == 0)
 					return t.key;
 			}
 
-			return 0;			
+			return (HtmlTextWriterStyle)(-1);			
 		}
 		
 		[MonoTODO]
@@ -251,7 +251,7 @@ namespace System.Web.UI {
 			// faster than a linear search?
 			
 			foreach (HtmlTag t in tags) {
-				if (t.name == tagName)
+				if (String.Compare(t.name, tagName, true, CultureInfo.InvariantCulture) == 0)
 					return t.key;
 			}
 
@@ -333,7 +333,7 @@ namespace System.Web.UI {
 			if (! newline)
 				return;
 			newline = false;
-			
+
 			for (int i = 0; i < Indent; i ++)
 				b.Write (tab_string);
 		}
@@ -1113,5 +1113,25 @@ namespace System.Web.UI {
 			new HtmlStyle (HtmlTextWriterStyle.ZIndex,             "z-index"),
 #endif
 		};
+
+#if NET_2_0
+		public virtual bool IsValidFormAttribute (string attribute)
+		{
+			return true;
+		}
+
+		// writes <br />
+		public virtual void WriteBreak ()
+		{
+			string br = GetTagName (HtmlTextWriterTag.Br);
+			WriteBeginTag (br);
+			Write (SelfClosingTagEnd);
+		}
+
+		public virtual void WriteEncodedText (string text)
+		{
+			Write (HttpUtility.HtmlEncode (text));
+		}
+#endif
 	}
 }
