@@ -46,12 +46,16 @@ using System.Web.Util;
 using System.Diagnostics;
 
 namespace System.Web.UI {
-	#if NET_2_0
-		public
-	#else
-		internal
-	#endif
-	sealed class ObjectStateFormatter : IFormatter {
+#if NET_2_0
+	public
+#else
+	internal
+#endif
+	sealed class ObjectStateFormatter : IFormatter
+#if NET_2_0
+		, IStateFormatter
+#endif
+	{
 		public object Deserialize (Stream inputStream)
 		{
 			if (inputStream == null)
@@ -661,21 +665,29 @@ namespace System.Web.UI {
 			{
 				Color c = (Color) o;
 				
-				if (!c.IsKnownColor) {
+				if (c.IsEmpty || c.IsKnownColor) {
+					w.Write (SecondaryId);
+					if (c.IsEmpty)
+						w.Write (-1); //isempty marker
+					else
+						w.Write ((int) c.ToKnownColor ());
+				}
+				else {
 					w.Write (PrimaryId);
 					w.Write (c.ToArgb ());
-				} else {
-					w.Write (SecondaryId);
-					w.Write ((int) c.ToKnownColor ());
 				}
 			}
 			
 			protected override object Read (byte token, BinaryReader r, ReaderContext ctx)
 			{
+				int value = r.ReadInt32 ();
 				if (token == PrimaryId)
-					return Color.FromArgb (r.ReadInt32 ());
-				else
-					return Color.FromKnownColor ((KnownColor) r.ReadInt32 ());
+					return Color.FromArgb (value);
+				else {
+					if (value == -1) //isempty marker
+						return Color.Empty;
+					return Color.FromKnownColor ((KnownColor)value);
+				}
 			}
 			
 			protected override Type Type {
@@ -721,7 +733,7 @@ namespace System.Web.UI {
 				} else {
 					w.Write (PrimaryId);
 					w.Write (((Type) o).FullName);
-					
+
 					// We should cache the name of the assembly
 					w.Write (((Type) o).Assembly.FullName);
 				}
@@ -732,7 +744,6 @@ namespace System.Web.UI {
 				if (token == PrimaryId) {
 					string type = r.ReadString ();
 					string assembly = r.ReadString ();
-					
 					Type t = Assembly.Load (assembly).GetType (type);
 					ctx.CacheItem (t);
 					return t;
