@@ -111,6 +111,11 @@ public class Page : TemplateControl, IHttpHandler
 #endif
 	const string postEventSourceID = "__EVENTTARGET";
 
+#if NET_2_0
+	const string ScrollPositionXID = "__SCROLLPOSITIONX";
+	const string ScrollPositionYID = "__SCROLLPOSITIONY";
+#endif
+
 #if NET_2_0
 	internal const string CallbackArgumentID = "__CALLBACKARGUMENT";
 	internal const string CallbackSourceID = "__CALLBACKTARGET";
@@ -130,6 +135,9 @@ public class Page : TemplateControl, IHttpHandler
 	string _title;
 	string _theme;
 	string _styleSheetTheme;
+	Hashtable items;
+
+	bool _maintainScrollPositionOnPostBack;
 #endif
 
 	#region Constructor
@@ -355,6 +363,15 @@ public class Page : TemplateControl, IHttpHandler
 			return ValidateCollection (_validators);
 		}
 	}
+#if NET_2_0
+	public IDictionary Items {
+		get {
+			if (items == null)
+				items = new Hashtable ();
+			return items;
+		}
+	}
+#endif
 
 	[EditorBrowsable (EditorBrowsableState.Never)]
 #if NET_2_0
@@ -370,6 +387,15 @@ public class Page : TemplateControl, IHttpHandler
 	}
 #endif
 
+#if NET_2_0
+	[Browsable (false)]
+	[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
+	public bool MaintainScrollPositionOnPostBack {
+		get { return _maintainScrollPositionOnPostBack; }
+		set { _maintainScrollPositionOnPostBack = value; }
+	}
+#endif
+
 #if NET_2_0
 	public PageAdapter PageAdapter {
 		get {
@@ -432,6 +458,9 @@ public class Page : TemplateControl, IHttpHandler
 	public virtual HttpSessionState Session
 	{
 		get {
+			if (_context == null)
+				_context = HttpContext.Current;
+
 			if (_context == null)
 				throw new HttpException ("Session is not available without context");
 
@@ -473,15 +502,24 @@ public class Page : TemplateControl, IHttpHandler
 
 	void InitializeStyleSheet ()
 	{
+		if (_styleSheetTheme == null) {
+			PagesSection ps = WebConfigurationManager.GetSection ("system.web/pages") as PagesSection;
+			if (ps != null)
+				_styleSheetTheme = ps.StyleSheetTheme;
+		}
 		if (_styleSheetTheme != null && _styleSheetTheme != "")
-			_styleSheetPageTheme = ThemeDirectoryCompiler.GetCompiledInstance ("./App_Themes/" + _styleSheetTheme + "/", _context);
+			_styleSheetPageTheme = ThemeDirectoryCompiler.GetCompiledInstance ("~/App_Themes/" + _styleSheetTheme + "/", _context);
 	}
 
 	void InitializeTheme ()
 	{
+		if (_theme == null) {
+			PagesSection ps = WebConfigurationManager.GetSection ("system.web/pages") as PagesSection;
+			if (ps != null)
+				_theme = ps.Theme;
+		}
 		if (_theme != null && _theme != "") {
-			_pageTheme = ThemeDirectoryCompiler.GetCompiledInstance ("./App_Themes/" + _theme + "/", _context);
-			ApplyThemeRecursive ();
+			_pageTheme = ThemeDirectoryCompiler.GetCompiledInstance ("~/App_Themes/" + _theme + "/", _context);
 		}
 	}
 
@@ -648,8 +686,7 @@ public class Page : TemplateControl, IHttpHandler
 			coll = req.QueryString;
 		}
 
-		
-		if (coll == null || coll ["__VIEWSTATE"] == null)
+		if (coll != null && coll ["__VIEWSTATE"] == null && coll ["__EVENTTARGET"] == null)
 			return null;
 
 		return coll;
@@ -794,7 +831,39 @@ public class Page : TemplateControl, IHttpHandler
 	{
 		return Request.MapPath (virtualPath);
 	}
-	
+
+#if NET_2_0
+	protected internal override void Render (HtmlTextWriter writer) {
+		if (MaintainScrollPositionOnPostBack) {
+			RequiresPostBackScript ();
+
+			string scriptUrl = ClientScript.GetWebResourceUrl (typeof (Page), "MaintainScrollPositionOnPostBack.js");
+
+			ClientScript.RegisterClientScriptInclude (typeof (Page), "MaintainScrollPositionOnPostBack.js", scriptUrl);
+
+			ClientScript.RegisterHiddenField (ScrollPositionXID, Request [ScrollPositionXID]);
+			ClientScript.RegisterHiddenField (ScrollPositionYID, Request [ScrollPositionYID]);
+			
+			StringBuilder script = new StringBuilder ();
+			script.AppendLine ("<script type=\"text/javascript\">");
+			script.AppendLine ("<!--");
+			script.AppendLine ("theForm.oldSubmit = theForm.submit");
+			script.AppendLine ("theForm.submit = WebForm_SaveScrollPositionSubmit");
+			script.AppendLine ("theForm.oldOnSubmit = theForm.onsubmit");
+			script.AppendLine ("theForm.onsubmit = WebForm_SaveScrollPositionOnSubmit");
+			if (IsPostBack) {
+				script.AppendLine ("theForm.oldOnLoad = window.onload");
+				script.AppendLine ("window.onload = WebForm_RestoreScrollPosition");
+			}
+			script.AppendLine ("// -->");
+			script.AppendLine ("</script>");
+			
+			ClientScript.RegisterStartupScript (typeof (Page), "MaintainScrollPositionOnPostBackStartup", script.ToString());
+		}
+		base.Render (writer);
+	}
+#endif
+
 	private void RenderPostBackScript (HtmlTextWriter writer, string formUniqueID)
 	{
 		writer.WriteLine ("<input type=\"hidden\" name=\"{0}\" value=\"\" />", postEventSourceID);
@@ -866,9 +935,7 @@ public class Page : TemplateControl, IHttpHandler
 			return null;
 
 		LosFormatter fmt = GetFormatter ();
-		MemoryStream ms = new MemoryStream ();
-		fmt.Serialize (ms, _savedViewState);
-		return Convert.ToBase64String (ms.GetBuffer (), 0, (int) ms.Length);
+		return fmt.SerializeToBase64 (_savedViewState);
 	}
 
 	internal object GetSavedViewState ()
@@ -1023,6 +1090,7 @@ public class Page : TemplateControl, IHttpHandler
 		OnPreInit (EventArgs.Empty);
 
 		InitializeTheme ();
+		ApplyMasterPage ();
 #endif
 		Trace.Write ("aspx.page", "Begin Init");
 		InitRecursive (null);
@@ -1031,7 +1099,6 @@ public class Page : TemplateControl, IHttpHandler
 #if NET_2_0
 		OnInitComplete (EventArgs.Empty);
 		
-		ApplyMasterPage ();
 
 		if (_title != null && htmlHeader != null)
 			htmlHeader.Title = _title;
@@ -1570,7 +1637,7 @@ public class Page : TemplateControl, IHttpHandler
 	
 	void ApplyMasterPage ()
 	{
-		if (masterPageFile != null) {
+		if (masterPageFile != null && masterPageFile.Length > 0) {
 			ArrayList appliedMasterPageFiles = new ArrayList ();
 
 			if (Master != null) {
@@ -1730,6 +1797,46 @@ public class Page : TemplateControl, IHttpHandler
 		return dataItemCtx.Peek ();
 	}
 
+	protected internal override void OnInit (EventArgs e)
+	{
+		base.OnInit (e);
+		if (Header == null)
+			return;
+
+		ArrayList themes = new ArrayList();
+
+		if (StyleSheetPageTheme != null && StyleSheetPageTheme.GetStyleSheets () != null)
+			themes.AddRange (StyleSheetPageTheme.GetStyleSheets ());
+		if (PageTheme != null && PageTheme.GetStyleSheets () != null)
+			themes.AddRange (PageTheme.GetStyleSheets ());
+
+		foreach (string lss in themes) {
+			HtmlLink hl = new HtmlLink ();
+			hl.Href = ResolveUrl (lss);
+			hl.Attributes["type"] = "text/css";
+			hl.Attributes["rel"] = "stylesheet";
+			Header.Controls.Add (hl);
+		}
+	}
+
 	#endif
+
+#if NET_2_0
+	[MonoTODO ("Used by .net aspx parser")]
+	protected object GetWrappedFileDependencies (string [] list)
+	{
+		return null;
+	}
+
+	[MonoTODO ("Used by .net aspx parser")]
+	protected virtual void InitializeCulture ()
+	{
+	}
+
+	[MonoTODO ("Used by .net aspx parser")]
+	protected internal void AddWrappedFileDependencies (object virtualFileDependencies)
+	{
+	}
+#endif
 }
 }
