@@ -390,6 +390,22 @@ namespace System.Web.UI {
 			}
 		}
 
+		internal virtual Assembly AddAssemblyByFileName (string filename)
+		{
+			Assembly assembly = null;
+			Exception error = null;
+
+			try {
+				assembly = Assembly.LoadFrom (filename);
+			} catch (Exception e) { error = e; }
+
+			if (assembly == null)
+				ThrowParseException ("Assembly " + filename + " not found", error);
+
+			AddAssembly (assembly, true);
+			return assembly;
+		}
+
 		internal virtual Assembly AddAssemblyByName (string name)
 		{
 			if (anames == null)
@@ -416,7 +432,7 @@ namespace System.Web.UI {
 					assembly = Assembly.LoadWithPartialName (name);
 				} catch (Exception e) { error = e; }
 			}
-
+			
 			if (assembly == null)
 				ThrowParseException ("Assembly " + name + " not found", error);
 
@@ -467,7 +483,7 @@ namespace System.Web.UI {
 
 				// Add the code file as an option to the
 				// compiler. This lets both files be compiled at once.
-				compilerOptions += " " + realPath;
+				compilerOptions += " \"" + realPath + "\"";
 			} else if (inherits != null) {
 				// We just set the inherits directly because this is a
 				// Single-Page model.
