@@ -863,11 +863,12 @@ namespace System.Web {
 			IHttpHandler handler = null;
 			try {
 				handler = GetHandler (context);
+				context.Handler = handler;
 			} catch (FileNotFoundException fnf){
 				if (context.Request.IsLocal)
 					ProcessError (new HttpException (404, String.Format ("File not found {0}", fnf.FileName), fnf));
 				else
-					ProcessError (new HttpException (404, "File not found", fnf));
+					ProcessError (new HttpException (404, "File not found: " + Path.GetFileName (fnf.FileName)));
 			} catch (DirectoryNotFoundException dnf){
 				ProcessError (new HttpException (404, "Directory not found", dnf));
 			} catch (Exception e) {
@@ -1036,6 +1037,7 @@ namespace System.Web {
 				return;
 			}
 
+			HttpContext.Current = Context;
 			PreStart ();
 			pipeline = Pipeline ();
 			Tick ();
@@ -1064,7 +1066,6 @@ namespace System.Web {
 			} else {
 				handler = factory.GetHandler (context, verb, url, request.PhysicalPath);
 			}
-			context.Handler = handler;
 
 			return handler;
 		}
@@ -1100,8 +1101,11 @@ namespace System.Web {
 			done.Reset ();
 			
 			begin_iar = new AsyncRequestState (done, cb, extraData);
-
+#if TARGET_JVM
+			if (true)
+#else
 			if (Thread.CurrentThread.IsThreadPoolThread)
+#endif
 				Start (null);
 			else
 				ThreadPool.QueueUserWorkItem (new WaitCallback (Start), null);
