@@ -370,17 +370,6 @@ namespace System.Web {
 			return e.GetChars (b.GetBuffer (), 0, (int) b.Length);
 		}
 
-		static bool TryParseHexa (string str, out uint result)
-		{
-			result = 0;
-			try {
-				result = UInt32.Parse (str, NumberStyles.HexNumber, CultureInfo.InvariantCulture);
-				return true;
-			} catch {
-				return false;
-			}
-		}
-
 		public static string UrlDecode (string s, Encoding e)
 		{
 			if (null == s) 
@@ -395,7 +384,7 @@ namespace System.Web {
 			StringBuilder output = new StringBuilder ();
 			long len = s.Length;
 			MemoryStream bytes = new MemoryStream ();
-			uint xchar;
+			int xchar;
 	
 			for (int i = 0; i < len; i++) {
 				if (s [i] == '%' && i + 2 < len && s [i + 1] != '%') {
@@ -405,13 +394,14 @@ namespace System.Web {
 							bytes.SetLength (0);
 						}
 
-						if (TryParseHexa (s.Substring (i + 2, 4), out xchar)) {
+						xchar = GetChar (s, i + 2, 4);
+						if (xchar != -1) {
 							output.Append ((char) xchar);
 							i += 5;
 						} else {
 							output.Append ('%');
 						}
-					} else if (TryParseHexa (s.Substring (i + 1, 2), out xchar)) {
+					} else if ((xchar = GetChar (s, i + 1, 2)) != -1) {
 						bytes.WriteByte ((byte) xchar);
 						i += 2;
 					} else {
@@ -476,6 +466,24 @@ namespace System.Web {
 
 			return value;
 		}
+
+		private static int GetChar (string str, int offset, int length)
+		{
+			int val = 0;
+			int end = length + offset;
+			for (int i = offset; i < end; i++) {
+				char c = str [i];
+				if (c > 127)
+					return -1;
+
+				int current = GetInt ((byte) c);
+				if (current == -1)
+					return -1;
+				val = (val << 4) + current;
+			}
+
+			return val;
+		}
 		
 		public static string UrlDecode (byte [] bytes, int offset, int count, Encoding e)
 		{
@@ -955,19 +963,64 @@ namespace System.Web {
 #endif
 
 #if NET_2_0
-		[MonoTODO]
 		public static NameValueCollection ParseQueryString (string query)
 		{
-			// LAMESPEC: default encoding not specified
-			throw new NotImplementedException ();
+			return ParseQueryString (query, Encoding.UTF8);
 		}
 
-		[MonoTODO]
 		public static NameValueCollection ParseQueryString (string query, Encoding encoding)
 		{
-			throw new NotImplementedException ();
-		}
+			if (query == null)
+				throw new ArgumentNullException ("query");
+			if (encoding == null)
+				throw new ArgumentNullException ("encoding");
+			if (query.Length == 0 || (query.Length == 1 && query[0] == '?'))
+				return new NameValueCollection ();
+			if (query[0] == '?')
+				query = query.Substring (1);
+				
+			NameValueCollection result = new NameValueCollection ();
+			ParseQueryString (query, encoding, result);
+			return result;
+		} 				
 #endif
+
+		internal static void ParseQueryString (string query, Encoding encoding, NameValueCollection result)
+		{
+			if (query.Length == 0)
+				return;
+
+			int namePos = 0;
+			while (namePos <= query.Length) {
+				int valuePos = -1, valueEnd = -1;
+				for (int q = namePos; q < query.Length; q++) {
+					if (valuePos == -1 && query[q] == '=') {
+						valuePos = q + 1;
+					} else if (query[q] == '&') {
+						valueEnd = q;
+						break;
+					}
+				}
+
+				string name, value;
+				if (valuePos == -1) {
+					name = null;
+					valuePos = namePos;
+				} else {
+					name = UrlDecode (query.Substring (namePos, valuePos - namePos - 1), encoding);
+				}
+				if (valueEnd < 0) {
+					namePos = -1;
+					valueEnd = query.Length;
+				} else {
+					namePos = valueEnd + 1;
+				}
+				value = UrlDecode (query.Substring (valuePos, valueEnd - valuePos), encoding);
+
+				result.Add (name, value);
+				if (namePos == -1) break;
+			}
+		}
 		#endregion // Methods
 	}
 }
