@@ -47,7 +47,9 @@ namespace System.Web
 		bool building;
 		string file;
 		SiteMapNode root = null;
+#if !TARGET_JVM // Java platform does not support file notifications
 		FileSystemWatcher watcher;
+#endif
 
 		[MonoTODO]
 		protected internal override void AddNode (SiteMapNode node, SiteMapNode parentNode)
@@ -139,6 +141,11 @@ namespace System.Web
 							rolesList.Add (ss);
 					}
 				}
+
+				if (!string.IsNullOrEmpty (url)) {
+					if (UrlUtils.IsRelativeUrl (url))
+						url = UrlUtils.Combine (HttpRuntime.AppDomainAppVirtualPath, url);
+				}
 				
 				SiteMapNode node = new SiteMapNode (this, url, url, title, description,
 					/*ArrayList.ReadOnly (keywordsList), */ArrayList.ReadOnly (rolesList), null,
@@ -162,8 +169,10 @@ namespace System.Web
 
 		protected virtual void Dispose (bool disposing)
 		{
+#if !TARGET_JVM // Java platform does not support file notifications
 			if (disposing)
 				watcher.Dispose ();
+#endif
 		}
 
 		public void Dispose ()
@@ -196,7 +205,8 @@ namespace System.Web
 				file = Path.Combine(HttpRuntime.AppDomainAppPath, file);
 			else
 				file = UrlUtils.ResolvePhysicalPathFromAppAbsolute (file);
-				
+
+#if !TARGET_JVM // Java platform does not support file notifications
 			if (File.Exists (file)) {
 				watcher = new FileSystemWatcher ();
 				watcher.Path = Path.GetFullPath (Path.GetDirectoryName (file));
@@ -204,6 +214,7 @@ namespace System.Web
 				watcher.Changed += new FileSystemEventHandler (OnFileChanged);
 				watcher.EnableRaisingEvents = true;
 			}
+#endif
 		}
 
 		[MonoTODO]
