@@ -36,6 +36,7 @@ using System.Xml.XPath;
 
 namespace System.Xml.Schema
 {
+	[MonoTODO ("This class is unused and thus won't be finished.")]
 	public sealed class XmlAtomicValue : XPathItem, ICloneable
 	{
 		bool booleanValue;
@@ -53,7 +54,6 @@ namespace System.Xml.Schema
 
 		#region Constructors
 
-		[MonoTODO]
 		internal XmlAtomicValue (bool value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -68,7 +68,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (DateTime value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -83,7 +82,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (decimal value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -98,7 +96,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (double value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -113,7 +110,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (int value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -128,7 +124,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (long value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -143,7 +138,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (float value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -158,7 +152,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (string value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -175,7 +168,6 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
-		[MonoTODO]
 		internal XmlAtomicValue (object value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -281,13 +273,11 @@ namespace System.Xml.Schema
 			return this.Clone ();
 		}
 
-		[MonoTODO]
 		public XmlAtomicValue Clone ()
 		{
 			return new XmlAtomicValue (this, schemaType);
 		}
 
-		[MonoTODO]
 		public override object ValueAs (Type type, IXmlNamespaceResolver nsResolver)
 		{
 			switch (XmlTypeCodeFromRuntimeType (type, false)) {
@@ -338,7 +328,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override object TypedValue {
 			get {
 				switch (ResolvedTypeCode) {
@@ -360,7 +349,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		// This method works like ValueAsString.
 		public override string Value {
 			get {
@@ -425,7 +413,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override bool ValueAsBoolean {
 			get {
 				switch (xmlTypeCode) {
@@ -457,7 +444,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override DateTime ValueAsDateTime {
 			get {
 				switch (xmlTypeCode) {
@@ -478,7 +464,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override double ValueAsDouble {
 			get {
 				switch (xmlTypeCode) {
@@ -509,7 +494,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override int ValueAsInt {
 			get {
 				switch (xmlTypeCode) {
@@ -540,7 +524,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override long ValueAsLong {
 			get {
 				switch (xmlTypeCode) {
@@ -571,12 +554,10 @@ namespace System.Xml.Schema
 			}
 		}
 
-		[MonoTODO]
 		public override Type ValueType {
 			get { return schemaType.Datatype.ValueType; }
 		}
 
-		[MonoTODO]
 		public override XmlSchemaType XmlType {
 			get { return schemaType; }
 		}
