@@ -65,8 +65,6 @@ namespace System.Xml.Schema
 		private string version;
 
 		// other post schema compilation infoset
-		private Hashtable idCollection;
-		private XmlSchemaObjectTable namedIdentities;
 		private XmlSchemaSet schemas;
 
 		private XmlNameTable nameTable;
@@ -75,7 +73,6 @@ namespace System.Xml.Schema
 
 		// Only compilation-time use
 		private XmlSchemaObjectCollection compilationItems;
-		private Hashtable handledUris;
 
 		// Compiler specific things
 		const string xmlname = "schema";
@@ -95,8 +92,6 @@ namespace System.Xml.Schema
 			groups = new XmlSchemaObjectTable();
 			notations = new XmlSchemaObjectTable();
 			schemaTypes = new XmlSchemaObjectTable();
-			idCollection = new Hashtable ();
-			namedIdentities = new XmlSchemaObjectTable();
 		}
 
 		#region Properties
@@ -235,20 +230,20 @@ namespace System.Xml.Schema
 			get{ return notations; }
 		}
 
-		internal Hashtable IDCollection
-		{
-			get { return idCollection; }
-		}
-
 		internal XmlSchemaObjectTable NamedIdentities
 		{
-			get { return namedIdentities; }
+			get { return schemas.NamedIdentities; }
 		}
 
 		internal XmlSchemaSet Schemas
 		{
 			get { return schemas; }
 		}
+
+		internal Hashtable IDCollection
+		{
+			get { return schemas.IDCollection; }
+		}
 		#endregion
 
 		#region Compile
@@ -284,29 +279,30 @@ namespace System.Xml.Schema
 		internal void Compile (ValidationEventHandler handler, XmlResolver resolver)
 #endif
 		{
-			Compile (handler, new Stack (), this, null, resolver);
+			XmlSchemaSet xss = new XmlSchemaSet ();
+			if (handler != null)
+				xss.ValidationEventHandler += handler;
+			xss.XmlResolver = resolver;
+			xss.Add (this);
+			xss.Compile ();
 		}
 
-		internal void Compile (ValidationEventHandler handler, XmlSchemaSet col, XmlResolver resolver)
+		// It is used by XmlSchemaCollection.Add(),
+		// XmlSchemaSet.Compile() and XmlSchemaSet.Remove().
+		internal void CompileSubset (ValidationEventHandler handler, XmlSchemaSet col, XmlResolver resolver)
 		{
-			Compile (handler, new Stack (), this, col, resolver);
+			Hashtable handledUris = new Hashtable ();
+			// Add this schema itself.
+			if (SourceUri != null && SourceUri.Length > 0)
+				handledUris.Add (SourceUri, SourceUri);
+
+			DoCompile (handler, handledUris, col, resolver);
 		}
 
-		private void Compile (ValidationEventHandler handler, Stack schemaLocationStack, XmlSchema rootSchema, XmlSchemaSet col, XmlResolver resolver)
+		void DoCompile (ValidationEventHandler handler, Hashtable handledUris, XmlSchemaSet col, XmlResolver resolver)
 		{
-			if (rootSchema != this) {
-				CompilationId = rootSchema.CompilationId;
-				schemas = rootSchema.schemas;
-			}
-			else {
-				schemas = col;
-				if (schemas == null) {
-					schemas = new XmlSchemaSet ();
-					schemas.CompilationId = Guid.NewGuid ();
-				}
-				CompilationId = schemas.CompilationId;
-				this.idCollection.Clear ();
-			}
+			CompilationId = col.CompilationId;
+			schemas = col;
 			if (!schemas.Contains (this)) // e.g. xs:import
 				schemas.Add (this);
 
@@ -316,7 +312,6 @@ namespace System.Xml.Schema
 			groups.Clear ();
 			notations.Clear ();
 			schemaTypes.Clear ();
-			namedIdentities.Clear ();
 
 			//1. Union and List are not allowed in block default
 			if (BlockDefault != XmlSchemaDerivationMethod.All) {
@@ -333,7 +328,7 @@ namespace System.Xml.Schema
 			}
 
 			//3. id must be of type ID
-			XmlSchemaUtil.CompileID(Id, this, this.IDCollection, handler);
+			XmlSchemaUtil.CompileID(Id, this, col.IDCollection, handler);
 
 			//4. targetNamespace should be of type anyURI or absent
 			if (TargetNamespace != null) {
@@ -357,8 +352,6 @@ namespace System.Xml.Schema
 #endif
 				compilationItems.Add (Items [i]);
 			}
-			if (this == rootSchema)
-				handledUris = new Hashtable ();
 
 			// First, we run into inclusion schemas to collect 
 			// compilation target items into compiledItems.
@@ -379,18 +372,10 @@ namespace System.Xml.Schema
 				string url = null;
 				if (resolver != null) {
 					url = GetResolvedUri (resolver, ext.SchemaLocation);
-					if (schemaLocationStack.Contains (url)) {
-						// Just skip nested inclusion. 
-						// The spec is "carefully written"
-						// not to handle it as an error.
-//						error (handler, "Nested inclusion was found: " + url);
-						// must skip this inclusion
-						continue;
-					}
-					if (rootSchema.handledUris.Contains (url))
+					if (handledUris.Contains (url))
 						// This schema is already handled, so simply skip (otherwise, duplicate definition errrors occur.
 						continue;
-					rootSchema.handledUris.Add (url, url);
+					handledUris.Add (url, url);
 					try {
 						stream = resolver.GetEntity (new Uri (url), null, typeof (Stream)) as Stream;
 					} catch (Exception) {
@@ -422,7 +407,6 @@ namespace System.Xml.Schema
 					missedSubComponents = true;
 					continue;
 				} else {
-					schemaLocationStack.Push (url);
 					XmlTextReader xtr = null;
 					try {
 						xtr = new XmlTextReader (url, stream, nameTable);
@@ -457,28 +441,9 @@ namespace System.Xml.Schema
 				}
 
 				// Compile included schema.
-				includedSchema.idCollection = this.IDCollection;
-				includedSchema.Compile (handler, schemaLocationStack, rootSchema, col, resolver);
-				schemaLocationStack.Pop ();
+				includedSchema.DoCompile (handler, handledUris, col, resolver);
 
-				if (import != null)
-					rootSchema.schemas.Add (includedSchema);
-
-				// Note that we use compiled items. Items
-				// may not exist in Items, since included
-				// schema also includes another schemas.
-				foreach (DictionaryEntry entry in includedSchema.Attributes)
-					compilationItems.Add ((XmlSchemaObject) entry.Value);
-				foreach (DictionaryEntry entry in includedSchema.Elements)
-					compilationItems.Add ((XmlSchemaObject) entry.Value);
-				foreach (DictionaryEntry entry in includedSchema.SchemaTypes)
-					compilationItems.Add ((XmlSchemaObject) entry.Value);
-				foreach (DictionaryEntry entry in includedSchema.AttributeGroups)
-					compilationItems.Add ((XmlSchemaObject) entry.Value);
-				foreach (DictionaryEntry entry in includedSchema.Groups)
-					compilationItems.Add ((XmlSchemaObject) entry.Value);
-				foreach (DictionaryEntry entry in includedSchema.Notations)
-					compilationItems.Add ((XmlSchemaObject) entry.Value);
+				AddExternalComponentsTo (includedSchema, compilationItems);
 			}
 
 			// Compilation phase.
@@ -574,13 +539,6 @@ namespace System.Xml.Schema
 						XmlSeverityType.Error);
 				}
 			}
-
-			if (rootSchema == this)
-				Validate(handler);
-
-			if (errorCount == 0)
-				isCompiled = true;
-			errorCount = 0;
 		}
 
 		private string GetResolvedUri (XmlResolver resolver, string relativeUri)
@@ -592,6 +550,15 @@ namespace System.Xml.Schema
 			return abs != null ? abs.ToString () : String.Empty;
 		}
 
+		void AddExternalComponentsTo (XmlSchema s, XmlSchemaObjectCollection items)
+		{
+			foreach (XmlSchemaExternal ext in s.Includes)
+				if (ext.Schema != null)
+					AddExternalComponentsTo (ext.Schema, items);
+			foreach (XmlSchemaObject obj in s.Items)
+				items.Add (obj);
+		}
+
 		internal bool IsNamespaceAbsent (string ns)
 		{
 			return !schemas.Contains (ns);
@@ -599,7 +566,51 @@ namespace System.Xml.Schema
 
 		#endregion
 
-		private void Validate (ValidationEventHandler handler)
+		internal XmlSchemaAttribute FindAttribute (XmlQualifiedName name)
+		{
+			XmlSchemaAttribute a;
+			foreach (XmlSchema schema in schemas.Schemas ()) {
+				a = schema.Attributes [name] as XmlSchemaAttribute;
+				if (a != null)
+					return a;
+			}
+			return null;
+		}
+
+		internal XmlSchemaAttributeGroup FindAttributeGroup (XmlQualifiedName name)
+		{
+			XmlSchemaAttributeGroup a;
+			foreach (XmlSchema schema in schemas.Schemas ()) {
+				a = schema.AttributeGroups [name] as XmlSchemaAttributeGroup;
+				if (a != null)
+					return a;
+			}
+			return null;
+		}
+
+		internal XmlSchemaElement FindElement (XmlQualifiedName name)
+		{
+			XmlSchemaElement a;
+			foreach (XmlSchema schema in schemas.Schemas ()) {
+				a = schema.Elements [name] as XmlSchemaElement;
+				if (a != null)
+					return a;
+			}
+			return null;
+		}
+
+		internal XmlSchemaType FindSchemaType (XmlQualifiedName name)
+		{
+			XmlSchemaType a;
+			foreach (XmlSchema schema in schemas.Schemas ()) {
+				a = schema.SchemaTypes [name] as XmlSchemaType;
+				if (a != null)
+					return a;
+			}
+			return null;
+		}
+
+		internal void Validate (ValidationEventHandler handler)
 		{
 			ValidationId = CompilationId;
 
@@ -620,6 +631,10 @@ namespace System.Xml.Schema
 				errorCount += grp.Validate (handler, this);
 			foreach (XmlSchemaNotation ntn in Notations.Values)
 				errorCount += ntn.Validate (handler, this);
+
+			if (errorCount == 0)
+				isCompiled = true;
+			errorCount = 0;
 		}
 
 		#region Read
