@@ -350,7 +350,7 @@ namespace System.Xml.Schema
 		internal override int Compile(ValidationEventHandler h, XmlSchema schema)
 		{
 			// If this is already compiled this time, simply skip.
-			if (this.IsComplied (schema.CompilationId))
+			if (CompilationId == schema.CompilationId)
 				return 0;
 			InitPostCompileInformations ();
 			this.schema = schema;
@@ -653,7 +653,7 @@ namespace System.Xml.Schema
 			if (schemaType != null)
 				elementType = schemaType;
 			else if (SchemaTypeName != XmlQualifiedName.Empty) {
-				XmlSchemaType type = schema.SchemaTypes [SchemaTypeName] as XmlSchemaType;
+				XmlSchemaType type = schema.FindSchemaType (SchemaTypeName);
 				if (type != null) {
 					type.Validate (h, schema);
 					elementType = type;
