@@ -81,7 +81,7 @@ namespace System.Xml.Schema
 		internal override int Compile(ValidationEventHandler h, XmlSchema schema)
 		{
 			// If this is already compiled this time, simply skip.
-			if (this.IsComplied (schema.CompilationId))
+			if (CompilationId == schema.CompilationId)
 				return 0;
 
 #if NET_2_0
@@ -149,7 +149,7 @@ namespace System.Xml.Schema
 			if (IsValidated (schema.ValidationId))
 				return errorCount;
 
-			XmlSchemaType st = schema.SchemaTypes [baseTypeName] as XmlSchemaType;
+			XmlSchemaType st = schema.FindSchemaType (baseTypeName);
 			if (st != null) {
 				XmlSchemaComplexType ct = st as XmlSchemaComplexType;
 				if (ct != null && ct.ContentModel is XmlSchemaComplexContent)
