@@ -102,7 +102,7 @@ namespace System.Xml.Schema
 		internal override int Compile(ValidationEventHandler h, XmlSchema schema)
 		{
 			// If this is already compiled this time, simply skip.
-			if (this.IsComplied (schema.CompilationId))
+			if (CompilationId == schema.CompilationId)
 				return 0;
 
 #if NET_2_0
@@ -383,7 +383,7 @@ namespace System.Xml.Schema
 
 			XmlSchemaSimpleType type = baseType;
 			if (type == null)
-				type = schema.SchemaTypes [baseTypeName] as XmlSchemaSimpleType;
+				type = schema.FindSchemaType (baseTypeName) as XmlSchemaSimpleType;
 			if (type != null) {
 				if (validate)
 					errorCount += type.Validate (h, schema);
