@@ -31,9 +31,15 @@
 using System.CodeDom;
 using System.CodeDom.Compiler;
 using System.Collections;
+#if NET_2_0
+using System.ComponentModel;
+using System.Diagnostics;
+#endif
 using System.Globalization;
 using System.Xml.Schema;
 
+using Microsoft.CSharp;
+
 namespace System.Xml.Serialization {
 	internal class MapCodeGenerator {
 
@@ -42,24 +48,24 @@ namespace System.Xml.Serialization {
 		CodeAttributeDeclarationCollection includeMetadata;
 		XmlTypeMapping exportedAnyType = null;
 		protected bool includeArrayTypes;
-		ICodeGenerator codeGen;
+		CodeDomProvider codeProvider;
 		CodeGenerationOptions options;
 
 		Hashtable exportedMaps = new Hashtable ();
 		Hashtable includeMaps = new Hashtable ();
 
-		public MapCodeGenerator (CodeNamespace codeNamespace, CodeCompileUnit codeCompileUnit)
+		public MapCodeGenerator (CodeNamespace codeNamespace, CodeCompileUnit codeCompileUnit, CodeGenerationOptions options)
 		{
 			this.codeCompileUnit = codeCompileUnit;
 			this.codeNamespace = codeNamespace;
-			this.options = CodeGenerationOptions.GenerateOldAsync;
+			this.options = options;
 		}
 
-		public MapCodeGenerator (CodeNamespace codeNamespace, CodeCompileUnit codeCompileUnit, ICodeGenerator codeGen, CodeGenerationOptions options, Hashtable mappings)
+		public MapCodeGenerator (CodeNamespace codeNamespace, CodeCompileUnit codeCompileUnit, CodeDomProvider codeProvider, CodeGenerationOptions options, Hashtable mappings)
 		{
 			this.codeCompileUnit = codeCompileUnit;
 			this.codeNamespace = codeNamespace;
-			this.codeGen = codeGen;
+			this.codeProvider = codeProvider;
 			this.options = options;
 //			this.mappings = mappings;
 		}
@@ -86,18 +92,18 @@ namespace System.Xml.Serialization {
 			ExportMembersMapCode (dummyClass, (ClassMap)xmlMembersMapping.ObjectMap, xmlMembersMapping.Namespace, null);
 		}
 
-		public void ExportTypeMapping (XmlTypeMapping xmlTypeMapping)
+		public void ExportTypeMapping (XmlTypeMapping xmlTypeMapping, bool isTopLevel)
 		{
-			ExportMapCode (xmlTypeMapping);
+			ExportMapCode (xmlTypeMapping, isTopLevel);
 			RemoveInclude (xmlTypeMapping);
 		}
 
-		void ExportMapCode (XmlTypeMapping map)
+		void ExportMapCode (XmlTypeMapping map, bool isTopLevel)
 		{
 			switch (map.TypeData.SchemaType)
 			{
 				case SchemaTypes.Enum:
-					ExportEnumCode (map);
+					ExportEnumCode (map, isTopLevel);
 					break;
 
 				case SchemaTypes.Array:
@@ -105,7 +111,7 @@ namespace System.Xml.Serialization {
 					break;
 
 				case SchemaTypes.Class:
-					ExportClassCode (map);
+					ExportClassCode (map, isTopLevel);
 					break;
 
 				case SchemaTypes.XmlSerializable:
@@ -116,16 +122,18 @@ namespace System.Xml.Serialization {
 			}
 		}
 
-		void ExportClassCode (XmlTypeMapping map)
+		void ExportClassCode (XmlTypeMapping map, bool isTopLevel)
 		{
 			CodeTypeDeclaration codeClass;
-			
 			if (IsMapExported (map)) {
-				// Regenerate attributes, since things may have changed
 				codeClass = GetMapDeclaration (map);
 				if (codeClass != null) {
-					codeClass.CustomAttributes = null;
-					GenerateClass (map, codeClass);
+					// Regenerate attributes, since things may have changed
+					codeClass.CustomAttributes.Clear ();
+#if NET_2_0
+					AddClassAttributes (codeClass);
+#endif
+					GenerateClass (map, codeClass, isTopLevel);
 					ExportDerivedTypeAttributes (map, codeClass);
 				}
 				return;
@@ -137,7 +145,7 @@ namespace System.Xml.Serialization {
 				SetMapExported (map, null);
 				foreach (XmlTypeMapping dmap in exportedAnyType.DerivedTypes) {
 					if (IsMapExported (dmap) || !dmap.IncludeInSchema) continue;
-					ExportTypeMapping (dmap);
+					ExportTypeMapping (dmap, false);
 					AddInclude (dmap);
 				}
 				return;
@@ -149,7 +157,12 @@ namespace System.Xml.Serialization {
 			AddCodeType (codeClass, map.Documentation);
 			codeClass.Attributes = MemberAttributes.Public;
 
-			GenerateClass (map, codeClass);
+#if NET_2_0
+			codeClass.IsPartial = CodeProvider.Supports(GeneratorSupport.PartialTypes);
+			AddClassAttributes (codeClass);
+#endif
+
+			GenerateClass (map, codeClass, isTopLevel);
 			ExportDerivedTypeAttributes (map, codeClass);
 			
 			ExportMembersMapCode (codeClass, (ClassMap)map.ObjectMap, map.XmlTypeNamespace, map.BaseMap);
@@ -159,7 +172,7 @@ namespace System.Xml.Serialization {
 				CodeTypeReference ctr = new CodeTypeReference (map.BaseMap.TypeData.FullTypeName);
 				codeClass.BaseTypes.Add (ctr);
 				if (map.BaseMap.IncludeInSchema) {
-					ExportMapCode (map.BaseMap);
+					ExportMapCode (map.BaseMap, false);
 					AddInclude (map.BaseMap);
 				}
 			}
@@ -182,13 +195,23 @@ namespace System.Xml.Serialization {
 				if (codeClass.CustomAttributes == null) 
 					codeClass.CustomAttributes = new CodeAttributeDeclarationCollection ();
 
-				ExportMapCode (tm);
+				ExportMapCode (tm, false);
 				ExportDerivedTypes (tm, codeClass);
 			}
 		}
 
 		void ExportMembersMapCode (CodeTypeDeclaration codeClass, ClassMap map, string defaultNamespace, XmlTypeMapping baseMap)
 		{
+			// Write attributes
+
+			ICollection attributes = map.AttributeMembers;
+			if (attributes != null) {
+				foreach (XmlTypeMapMemberAttribute attr in attributes) {
+					if (baseMap != null && DefinedInBaseMap (baseMap, attr)) continue;
+					AddAttributeFieldMember (codeClass, attr, defaultNamespace);
+				}
+			}
+
 			ICollection members = map.ElementMembers;
 				
 			if (members != null)
@@ -221,17 +244,6 @@ namespace System.Xml.Serialization {
 				}
 			}
 
-			// Write attributes
-
-			ICollection attributes = map.AttributeMembers;
-			if (attributes != null)
-			{
-				foreach (XmlTypeMapMemberAttribute attr in attributes) {
-					if (baseMap != null && DefinedInBaseMap (baseMap, attr)) continue;
-					AddAttributeFieldMember (codeClass, attr, defaultNamespace);
-				}
-			}
-
 			XmlTypeMapMember anyAttrMember = map.DefaultAnyAttributeMember;
 			if (anyAttrMember != null)
 			{
@@ -261,7 +273,7 @@ namespace System.Xml.Serialization {
 		{
 			CodeMemberField codeField = null;
 			CodeTypeMember codeProp = null;
-			
+
 			if ((options & CodeGenerationOptions.GenerateProperties) > 0) {
 				string field = CodeIdentifier.MakeCamel (name + "Field");
 				codeField = new CodeMemberField (type, field);
@@ -304,7 +316,7 @@ namespace System.Xml.Serialization {
 			if (attributes.Count > 0) codeField.CustomAttributes = attributes;
 
 			if (attinfo.MappedType != null) {
-				ExportMapCode (attinfo.MappedType);
+				ExportMapCode (attinfo.MappedType, false);
 				RemoveInclude (attinfo.MappedType);
 			}
 
@@ -357,7 +369,7 @@ namespace System.Xml.Serialization {
 
 				GenerateElementInfoMember (attributes, member, einfo, defaultType, defaultNamespace, addAlwaysAttr, forceUseMemberName);
 				if (einfo.MappedType != null) {
-					ExportMapCode (einfo.MappedType);
+					ExportMapCode (einfo.MappedType, false);
 					RemoveInclude (einfo.MappedType);
 				}
 			}
@@ -389,8 +401,7 @@ namespace System.Xml.Serialization {
 		void AddArrayElementFieldMember (CodeTypeDeclaration codeClass, XmlTypeMapMemberList member, string defaultNamespace)
 		{
 			CodeTypeMember codeField = CreateFieldMember (codeClass, member.TypeData, member.Name);
-			codeField.Attributes = MemberAttributes.Public;
-			
+
 			CodeAttributeDeclarationCollection attributes = new CodeAttributeDeclarationCollection ();
 			AddArrayAttributes (attributes, member, defaultNamespace, false);
 
@@ -417,7 +428,7 @@ namespace System.Xml.Serialization {
 				if (ainfo.MappedType != null) {
 					if (!IsMapExported (ainfo.MappedType) && includeArrayTypes)
 						AddInclude (ainfo.MappedType);
-					ExportMapCode (ainfo.MappedType);
+					ExportMapCode (ainfo.MappedType, false);
 				}
 			}
 
@@ -436,7 +447,7 @@ namespace System.Xml.Serialization {
 				if (ainfo.MappedType != null) {
 					if (!IsMapExported (ainfo.MappedType) && includeArrayTypes)
 						AddInclude (ainfo.MappedType);
-					ExportMapCode (ainfo.MappedType);
+					ExportMapCode (ainfo.MappedType, false);
 				}
 			}
 		}
@@ -454,7 +465,7 @@ namespace System.Xml.Serialization {
 			return false;
 		}
 
-		void ExportEnumCode (XmlTypeMapping map)
+		void ExportEnumCode (XmlTypeMapping map, bool isTopLevel)
 		{
 			if (IsMapExported (map)) return;
 
@@ -465,12 +476,25 @@ namespace System.Xml.Serialization {
 			codeEnum.IsEnum = true;
 			AddCodeType (codeEnum, map.Documentation);
 
-			GenerateEnum (map, codeEnum);
 			EnumMap emap = (EnumMap) map.ObjectMap;
-			
 			if (emap.IsFlags)
-				codeEnum.CustomAttributes.Add (new CodeAttributeDeclaration ("System.Flags"));
+				codeEnum.CustomAttributes.Add (new CodeAttributeDeclaration ("System.FlagsAttribute"));
 
+#if NET_2_0
+			CodeAttributeDeclaration generatedCodeAttribute = new CodeAttributeDeclaration (
+				new CodeTypeReference (typeof(GeneratedCodeAttribute)));
+			generatedCodeAttribute.Arguments.Add (new CodeAttributeArgument (
+				new CodePrimitiveExpression ("System.Xml")));
+			generatedCodeAttribute.Arguments.Add (new CodeAttributeArgument (
+				new CodePrimitiveExpression ("2.0.50727.42")));
+			codeEnum.CustomAttributes.Add (generatedCodeAttribute);
+
+			codeEnum.CustomAttributes.Add (new CodeAttributeDeclaration (
+				new CodeTypeReference (typeof (SerializableAttribute))));
+#endif
+
+			GenerateEnum (map, codeEnum, isTopLevel);
+			
 			int flag = 1;
 			foreach (EnumMap.EnumMapMember emem in emap.Members)
 			{
@@ -563,6 +587,30 @@ namespace System.Xml.Serialization {
 			AddComments (type, comments);
 			codeNamespace.Types.Add (type);
 		}
+
+#if NET_2_0
+		void AddClassAttributes (CodeTypeDeclaration codeClass)
+		{
+			CodeAttributeDeclaration generatedCodeAttribute = new CodeAttributeDeclaration (
+				new CodeTypeReference (typeof (GeneratedCodeAttribute)));
+			generatedCodeAttribute.Arguments.Add (new CodeAttributeArgument (
+				new CodePrimitiveExpression ("System.Xml")));
+			generatedCodeAttribute.Arguments.Add (new CodeAttributeArgument (
+				new CodePrimitiveExpression ("2.0.50727.42")));
+			codeClass.CustomAttributes.Add (generatedCodeAttribute);
+
+			codeClass.CustomAttributes.Add (new CodeAttributeDeclaration (
+				new CodeTypeReference (typeof (SerializableAttribute))));
+			codeClass.CustomAttributes.Add (new CodeAttributeDeclaration (
+				new CodeTypeReference (typeof (DebuggerStepThroughAttribute))));
+
+			CodeAttributeDeclaration designerCategoryAttribute = new CodeAttributeDeclaration (
+				new CodeTypeReference (typeof (DesignerCategoryAttribute)));
+			designerCategoryAttribute.Arguments.Add (new CodeAttributeArgument (
+				new CodePrimitiveExpression ("code")));
+			codeClass.CustomAttributes.Add (designerCategoryAttribute);
+		}
+#endif
 		
 		CodeTypeReference GetDomType (TypeData data)
 		{
@@ -573,10 +621,25 @@ namespace System.Xml.Serialization {
 		}
 		
 		#endregion
-		
+
+		#region Private Properties
+
+#if NET_2_0
+		private CodeDomProvider CodeProvider {
+			get {
+				if (codeProvider == null) {
+					codeProvider = new CSharpCodeProvider ();
+				}
+				return codeProvider;
+			}
+		}
+#endif
+
+		#endregion
+
 		#region Overridable methods
-		
-		protected virtual void GenerateClass (XmlTypeMapping map, CodeTypeDeclaration codeClass)
+
+		protected virtual void GenerateClass (XmlTypeMapping map, CodeTypeDeclaration codeClass, bool isTopLevel)
 		{
 		}
 		
@@ -636,10 +699,10 @@ namespace System.Xml.Serialization {
 		protected virtual void GenerateUnnamedAnyElementAttribute (CodeAttributeDeclarationCollection attributes, XmlTypeMapElementInfo einfo, string defaultNamespace)
 		{
 		}
-		
-		protected virtual void GenerateEnum (XmlTypeMapping map, CodeTypeDeclaration codeEnum)
+
+		protected virtual void GenerateEnum (XmlTypeMapping map, CodeTypeDeclaration codeEnum, bool isTopLevel)
 		{
-		}		
+		}
 		
 		protected virtual void GenerateEnumItem (CodeMemberField codeField, EnumMap.EnumMapMember emem)
 		{
