@@ -1163,14 +1163,14 @@ namespace System.Xml.Serialization
 
 		string GenerateGetMemberValue (XmlTypeMapMember member, string ob, bool isValueList)
 		{
-			if (isValueList) return GetCast (member.TypeData, TypeTranslator.GetTypeData (typeof(object)), ob + "[" + member.Index + "]");
+			if (isValueList) return GetCast (member.TypeData, TypeTranslator.GetTypeData (typeof(object)), ob + "[" + member.GlobalIndex + "]");
 			else return ob + ".@" + member.Name;
 		}
 		
 		string GenerateMemberHasValueCondition (XmlTypeMapMember member, string ob, bool isValueList)
 		{
 			if (isValueList) {
-				return ob + ".Length > " + member.Index;
+				return ob + ".Length > " + member.GlobalIndex;
 			}
 			else if (member.DefaultValue != System.DBNull.Value) {
 				string mem = ob + ".@" + member.Name;
@@ -1298,14 +1298,18 @@ namespace System.Xml.Serialization
 
 				if (_format == SerializationFormat.Literal)
 				{
-					if (typeMap.TypeData.SchemaType == SchemaTypes.XmlNode)
-						throw new Exception ("Not supported for XmlNode types");
-						
-					WriteLineInd ("if (Reader.LocalName != " + GetLiteral (typeMap.ElementName) + " || Reader.NamespaceURI != " + GetLiteral (typeMap.Namespace) + ")");
-					WriteLine ("throw CreateUnknownNodeException();");
-					Unindent ();
+					if (typeMap.TypeData.SchemaType == SchemaTypes.XmlNode) {
+						if (typeMap.TypeData.Type == typeof (XmlDocument))
+							WriteLine ("return ReadXmlDocument (false);");
+						else
+							WriteLine ("return ReadXmlNode (false);");
+					} else {
+						WriteLineInd ("if (Reader.LocalName != " + GetLiteral (typeMap.ElementName) + " || Reader.NamespaceURI != " + GetLiteral (typeMap.Namespace) + ")");
+						WriteLine ("throw CreateUnknownNodeException();");
+						Unindent ();
 
-					WriteLine ("return " + GetReadObjectCall (typeMap, GetLiteral(typeMap.IsNullable), "true") + ";");
+						WriteLine ("return " + GetReadObjectCall (typeMap, GetLiteral(typeMap.IsNullable), "true") + ";");
+					}
 				}
 				else
 				{
@@ -1906,7 +1910,7 @@ namespace System.Xml.Serialization
 					
 					if (map.XmlTextCollector != null)
 					{
-						WriteLine ("else if (Reader.NodeType == System.Xml.XmlNodeType.Text)");
+						WriteLine ("else if (Reader.NodeType == System.Xml.XmlNodeType.Text || Reader.NodeType == System.Xml.XmlNodeType.CDATA)");
 						WriteLineInd ("{");
 		
 						if (map.XmlTextCollector is XmlTypeMapMemberExpandable)
@@ -2009,7 +2013,7 @@ namespace System.Xml.Serialization
 
 		void GenerateSetMemberValue (XmlTypeMapMember member, string ob, string value, bool isValueList)
 		{
-			if (isValueList) WriteLine (ob + "[" + member.Index + "] = " + value + ";");
+			if (isValueList) WriteLine (ob + "[" + member.GlobalIndex + "] = " + value + ";");
 			else {
 				WriteLine (ob + ".@" + member.Name + " = " + value + ";");
 				if (member.IsOptionalValueType)
@@ -2175,8 +2179,8 @@ namespace System.Xml.Serialization
 			string valueArray = GetObTempVar ();
 			WriteLine ("string[] " + valueArray + " = " + var + ".Split (' ');");
 			
-			WriteLine (list + " = new " + itemType + " [" + valueArray + ".Length];");
-
+			WriteLine (list + " = new " + GetArrayDeclaration (listType, valueArray + ".Length") + ";");
+			
 			XmlTypeMapElementInfo info = (XmlTypeMapElementInfo)listMap.ItemInfo[0];
 
 			string index = GetNumTempVar ();
@@ -2185,10 +2189,23 @@ namespace System.Xml.Serialization
 			Unindent ();
 			WriteLineUni ("}");
 			WriteLine ("else");
-			WriteLine ("\t" + list + " = new " + itemType + " [0];");
+			WriteLine ("\t" + list + " = new " + GetArrayDeclaration (listType, "0") + ";");
 			
 			return list;
 		}
+		
+		string GetArrayDeclaration (Type type, string length)
+		{
+			Type t = type.GetElementType();
+			System.Text.StringBuilder sb = new System.Text.StringBuilder ();
+			sb.Append ('[').Append (length).Append (']');
+			while (t.IsArray) {
+				sb.Append ("[]");
+				t = t.GetElementType();
+			}
+			sb.Insert (0, t.FullName);
+			return sb.ToString ();
+		}
 
 		void GenerateAddListValue (TypeData listType, string list, string index, string value, bool canCreateInstance)
 		{
@@ -2282,10 +2299,8 @@ namespace System.Xml.Serialization
 				WriteLineInd ("{");
 				WriteLine ("xmlName = xmlName.Trim();");
 				WriteLine ("if (xmlName.Length == 0) return (" + typeMap.TypeFullName + ")0;");
-				WriteLine ("if (xmlName.IndexOf (' ') != -1)");
-				WriteLineInd ("{");
 				WriteLine (typeMap.TypeFullName + " sb = (" + typeMap.TypeFullName + ")0;");
-				WriteLine ("string[] enumNames = xmlName.ToString().Split (' ');");
+				WriteLine ("string[] enumNames = xmlName.Split (null);");
 				WriteLine ("foreach (string name in enumNames)");
 				WriteLineInd ("{");
 				WriteLine ("if (name == string.Empty) continue;");
@@ -2293,9 +2308,6 @@ namespace System.Xml.Serialization
 				WriteLineUni ("}");
 				WriteLine ("return sb;");
 				WriteLineUni ("}");
-				WriteLine ("else");
-				WriteLine ("\treturn " + switchMethod + " (xmlName);");
-				WriteLineUni ("}");
 				WriteLine ("");
 				metName = switchMethod;
 			}
