@@ -30,6 +30,7 @@
 //
 
 using System.Xml;
+using System.CodeDom.Compiler;
 using System.Xml.Schema;
 using System.Collections;
 
@@ -91,7 +92,7 @@ namespace System.Xml.Serialization
 		
 #if NET_2_0
 		[MonoTODO]
-		public XmlSchemaImporter (XmlSchemas schemas, CodeGenerationOptions options, System.CodeDom.Compiler.ICodeGenerator codeGenerator, ImportContext context)
+		public XmlSchemaImporter (XmlSchemas schemas, CodeGenerationOptions options, CodeDomProvider codeProvider, ImportContext context)
 		{
 			this.schemas = schemas;
 			this.options = options;
@@ -246,8 +247,8 @@ namespace System.Xml.Serialization
 			
 			// Does not have the requested base type.
 			// Then, get/create a map for that base type.
-			
-			XmlTypeMapping baseMap = ReflectType (baseType, null);
+
+			XmlTypeMapping baseMap = ReflectType (baseType);
 			
 			// Add this map as a derived map of the base map
 
@@ -414,12 +415,14 @@ namespace System.Xml.Serialization
 		{
 			XmlQualifiedName qname;
 			XmlSchemaType stype;
-			if (!LocateElement (name, out qname, out stype)) return null;
+
+			XmlSchemaElement elem = (XmlSchemaElement) schemas.Find (name, typeof (XmlSchemaElement));
+			if (!LocateElement (elem, out qname, out stype)) return null;
 			
 			if (stype == null) {
 				// Importing a primitive type
 				TypeData td = TypeTranslator.GetPrimitiveTypeData (qname.Name);
-				return ReflectType (td.Type, name.Namespace);
+				return ReflectType (td, name.Namespace);
 			}
 			
 			XmlTypeMapping map = GetRegisteredTypeMapping (qname);
@@ -427,6 +430,7 @@ namespace System.Xml.Serialization
 			
 			map = CreateTypeMapping (qname, SchemaTypes.Class, name);
 			map.Documentation = GetDocumentation (stype);
+			map.IsNullable = elem.IsNillable;
 			RegisterMapFixup (map, qname, (XmlSchemaComplexType)stype);
 			
 			BuildPendingMaps ();
@@ -434,11 +438,16 @@ namespace System.Xml.Serialization
 		}
 
 		bool LocateElement (XmlQualifiedName name, out XmlQualifiedName qname, out XmlSchemaType stype)
+		{
+			XmlSchemaElement elem = (XmlSchemaElement) schemas.Find (name, typeof (XmlSchemaElement));
+			return LocateElement (elem, out qname, out stype);
+		}
+
+		bool LocateElement (XmlSchemaElement elem, out XmlQualifiedName qname, out XmlSchemaType stype)
 		{
 			qname = null;
 			stype = null;
 			
-			XmlSchemaElement elem = (XmlSchemaElement) schemas.Find (name, typeof (XmlSchemaElement));
 			if (elem == null) return false;
 
 			// The root element must be an element with complex type
@@ -446,7 +455,7 @@ namespace System.Xml.Serialization
 			if (elem.SchemaType != null)
 			{
 				stype = elem.SchemaType;
-				qname = name;
+				qname = elem.QualifiedName;
 			}
 			else
 			{
@@ -1792,18 +1801,24 @@ namespace System.Xml.Serialization
 			}
 			return grp;
 		}
-		
-		XmlTypeMapping ReflectType (Type type, string ns)
+
+		XmlTypeMapping ReflectType (Type type)
+		{
+			TypeData typeData = TypeTranslator.GetTypeData (type);
+			return ReflectType (typeData, (string) null);
+		}
+
+		XmlTypeMapping ReflectType (TypeData typeData, string ns)
 		{
 			if (!encodedFormat)
 			{
 				if (auxXmlRefImporter == null) auxXmlRefImporter = new XmlReflectionImporter ();
-				return auxXmlRefImporter.ImportTypeMapping (type, ns);
+				return auxXmlRefImporter.ImportTypeMapping (typeData, ns);
 			}
 			else
 			{
 				if (auxSoapRefImporter == null) auxSoapRefImporter = new SoapReflectionImporter ();
-				return auxSoapRefImporter.ImportTypeMapping (type, ns);
+				return auxSoapRefImporter.ImportTypeMapping (typeData, ns);
 			}
 		}
 
