@@ -171,8 +171,11 @@ namespace System.Xml
 
 		public override XmlNodeType NodeType {
 			get {
-				if (Current == entity)
-					return entity.EOF ? XmlNodeType.EndEntity : entity.NodeType;
+				if (entity != null)
+					return entity.ReadState == ReadState.Initial ?
+						source.NodeType :
+						entity.EOF ? XmlNodeType.EndEntity :
+						entity.NodeType;
 				else
 					return source.NodeType;
 			}
@@ -426,21 +429,9 @@ namespace System.Xml
 		}
 #endif
 
-#if NET_1_0
-		public override string ReadInnerXml ()
-		{
-			return ReadInnerXmlInternal ();
-		}
-
-		public override string ReadOuterXml ()
-		{
-			return ReadOuterXmlInternal ();
-		}
-#endif
-
 		public override string ReadString ()
 		{
-			return ReadStringInternal ();
+			return base.ReadString ();
 		}
 
 		public override void ResolveEntity ()
