@@ -310,9 +310,10 @@ namespace System.Xml
 			settings = PopulateSettings (settings);
 			if (context == null)
 				context = PopulateParserContext (settings, url);
-			return CreateCustomizedTextReader (
-				new XmlTextReader (true, url, GetNodeType (settings), context),
-				settings);
+			XmlTextReader xtr = new XmlTextReader (true, url, GetNodeType (settings), context);
+			XmlReader ret = CreateCustomizedTextReader (xtr, settings);
+			xtr.CloseInput = true;
+			return xtr;
 		}
 
 		public static XmlReader Create (Stream stream, XmlReaderSettings settings, XmlParserContext context)
@@ -409,22 +410,18 @@ namespace System.Xml
 				xvr.ValidationType = ValidationType.DTD;
 				break;
 			case ValidationType.Schema:
-//				xvr = new XmlValidatingReader (reader);
-//				xvr.ValidationType = ValidationType.Schema;
 				return new XmlSchemaValidatingReader (reader, settings);
 			}
-			if (xvr != null)
-				xvr.SetSchemas (settings.Schemas);
 
 			// Actually I don't think they are treated in DTD validation though...
 			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessIdentityConstraints) == 0)
 				throw new NotImplementedException ();
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessInlineSchema) != 0)
-				throw new NotImplementedException ();
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessSchemaLocation) != 0)
-				throw new NotImplementedException ();
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ReportValidationWarnings) == 0)
-				throw new NotImplementedException ();
+			//if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessInlineSchema) != 0)
+			//	throw new NotImplementedException ();
+			//if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessSchemaLocation) != 0)
+			//	throw new NotImplementedException ();
+			//if ((settings.ValidationFlags & XmlSchemaValidationFlags.ReportValidationWarnings) == 0)
+			//	throw new NotImplementedException ();
 
 			return xvr != null ? xvr : reader;
 		}
