@@ -2114,7 +2114,7 @@ namespace System.Xml
 			value = src.Substring (start, idx - start - 1);
 		}
 
-		private void SkipTextDeclaration ()
+		internal void SkipTextDeclaration ()
 		{
 			if (PeekChar () != '<')
 				return;
@@ -2202,6 +2202,8 @@ namespace System.Xml
 #endif
 
 			Expect ("?>");
+
+			curNodePeekIndex = peekCharsIndex; // without this it causes incorrect value start indication.
 		}
 
 		// The reader is positioned on the first character after
