@@ -342,8 +342,12 @@ namespace Mono.Xml
 		public Formatting Formatting {
 			get { return indent ? Formatting.Indented : Formatting.None; }
 			set {
-				if (state != WriteState.Start)
-					throw InvalidOperation ("Formatting must be set before it is actually used to write output.");
+				// Someone thinks it should be settable even
+				// after writing some content (bug #78148).
+				// I totally disagree but here is the fix.
+
+				//if (state != WriteState.Start)
+				//	throw InvalidOperation ("Formatting must be set before it is actually used to write output.");
 				indent = (value == Formatting.Indented);
 			}
 		}
@@ -745,16 +749,20 @@ namespace Mono.Xml
 			CloseStartElementCore ();
 
 			nsmanager.PopScope ();
-			bool doIndent = !elements [open_count - 1].HasSimple;
-			if (open_count > 1)
-				doIndent &= !elements [open_count - 2].HasSimple;
+
+			if (state == WriteState.Element) {
+				if (full)
+					writer.Write ('>');
+				else
+					writer.Write (" />");
+			}
+
+			if (full || state == WriteState.Content)
+				WriteIndentEndElement ();
+
 			XmlNodeInfo info = elements [--open_count];
 
 			if (full || state == WriteState.Content) {
-				if (state == WriteState.Element)
-					writer.Write ('>');
-				if (doIndent && (full || !isEmpty))
-					DoWriteIndent ();
 				writer.Write ("</");
 				if (info.Prefix.Length > 0) {
 					writer.Write (info.Prefix);
@@ -762,8 +770,6 @@ namespace Mono.Xml
 				}
 				writer.Write (info.LocalName);
 				writer.Write ('>');
-			} else {
-				writer.Write (" />");
 			}
 
 			state = WriteState.Content;
@@ -867,7 +873,7 @@ namespace Mono.Xml
 			bool mockup = false;
 			if (prefix.Length == 0) {
 				prefix = LookupPrefix (ns);
-				if (prefix != null)
+				if (prefix != null && prefix.Length > 0)
 					return prefix;
 				mockup = true;
 			} else {
@@ -969,14 +975,17 @@ namespace Mono.Xml
 			if (text == null)
 				throw ArgumentError ("text");
 
-			WriteIndent ();
-
 			if (text.Length > 0 && text [text.Length - 1] == '-')
 				throw ArgumentError ("An input string to WriteComment method must not end with '-'. Escape it with '&#2D;'.");
 			if (StringUtil.IndexOf (text, "--") > 0)
 				throw ArgumentError ("An XML comment cannot end with \"-\".");
 
-			ShiftStateTopLevel ("Comment", false, false);
+			if (state == WriteState.Attribute || state == WriteState.Element)
+				CloseStartElement ();
+
+			WriteIndent ();
+
+			ShiftStateTopLevel ("Comment", false, false, false);
 
 			writer.Write ("<!--");
 			writer.Write (text);
@@ -999,7 +1008,7 @@ namespace Mono.Xml
 			if (StringUtil.IndexOf (text, "?>") > 0)
 				throw ArgumentError ("Processing instruction cannot contain \"?>\" as its value.");
 
-			ShiftStateTopLevel ("ProcessingInstruction", false, name == "xml");
+			ShiftStateTopLevel ("ProcessingInstruction", false, name == "xml", false);
 
 			writer.Write ("<?");
 			writer.Write (name);
@@ -1023,7 +1032,7 @@ namespace Mono.Xml
 			    XmlChar.IndexOfNonWhitespace (text) >= 0)
 				throw ArgumentError ("WriteWhitespace method accepts only whitespaces.");
 
-			ShiftStateTopLevel ("Whitespace", true, false);
+			ShiftStateTopLevel ("Whitespace", true, false, true);
 
 			writer.Write (text);
 		}
@@ -1032,7 +1041,7 @@ namespace Mono.Xml
 		{
 			if (text == null)
 				text = String.Empty;
-			ShiftStateContent ("Text", false);
+			ShiftStateContent ("CData", false);
 
 			if (StringUtil.IndexOf (text, "]]>") >= 0)
 				throw ArgumentError ("CDATA section must not contain ']]>'.");
@@ -1059,7 +1068,7 @@ namespace Mono.Xml
 
 			// LAMESPEC: It rejects XMLDecl while it allows
 			// DocType which could consist of non well-formed XML.
-			ShiftStateTopLevel ("Raw string", true, true);
+			ShiftStateTopLevel ("Raw string", true, true, true);
 
 			writer.Write (raw);
 		}
@@ -1098,7 +1107,7 @@ namespace Mono.Xml
 			if (!XmlChar.IsName (name))
 				throw ArgumentError ("Argument name must be a valid XML name.");
 
-			ShiftStateContent ("Character", true);
+			ShiftStateContent ("Entity reference", true);
 
 			writer.Write ('&');
 			writer.Write (name);
@@ -1180,7 +1189,7 @@ namespace Mono.Xml
 		{
 			CheckChunkRange (buffer, index, count);
 
-			ShiftStateContent ("Text", false);
+			ShiftStateContent ("BinHex", true);
 
 			XmlConvert.WriteBinHex (buffer, index, count, writer);
 		}
@@ -1189,7 +1198,7 @@ namespace Mono.Xml
 		{
 			CheckChunkRange (buffer, index, count);
 
-			ShiftStateContent ("Text", true);
+			ShiftStateContent ("Chars", true);
 
 			WriteEscapedBuffer (buffer, index, count,
 				state == WriteState.Attribute);
@@ -1199,7 +1208,7 @@ namespace Mono.Xml
 		{
 			CheckChunkRange (buffer, index, count);
 
-			ShiftStateContent ("Text", false);
+			ShiftStateContent ("Raw text", false);
 
 			writer.Write (buffer, index, count);
 		}
@@ -1208,21 +1217,25 @@ namespace Mono.Xml
 
 		void WriteIndent ()
 		{
-			if (!indent || (open_count > 0 &&
-			    elements [open_count - 1].HasSimple))
-				return;
-			if (state != WriteState.Start)
-				writer.Write (newline);
-			for (int i = 0; i < open_count; i++)
-				writer.Write (indent_string);
+			WriteIndentCore (0);
 		}
 
-		void DoWriteIndent ()
+		void WriteIndentEndElement ()
+		{
+			WriteIndentCore (-1);
+		}
+
+		void WriteIndentCore (int nestFix)
 		{
 			if (!indent)
 				return;
-			writer.Write (newline);
-			for (int i = 0; i < open_count; i++)
+			for (int i = open_count - 1; i >= 0; i--)
+				if (elements [i].HasSimple)
+					return;
+
+			if (state != WriteState.Start)
+				writer.Write (newline);
+			for (int i = 0; i < open_count + nestFix; i++)
 				writer.Write (indent_string);
 		}
 
@@ -1233,7 +1246,7 @@ namespace Mono.Xml
 			WriteStartDocumentCore (false, false);
 		}
 
-		void ShiftStateTopLevel (string occured, bool allowAttribute, bool dontCheckXmlDecl)
+		void ShiftStateTopLevel (string occured, bool allowAttribute, bool dontCheckXmlDecl, bool isCharacter)
 		{
 			switch (state) {
 #if NET_2_0
@@ -1242,6 +1255,8 @@ namespace Mono.Xml
 			case WriteState.Closed:
 				throw StateError (occured);
 			case WriteState.Start:
+				if (isCharacter)
+					CheckMixedContentState ();
 				if (output_xmldecl && !dontCheckXmlDecl)
 					OutputAutoStartDocument ();
 				state = WriteState.Prolog;
@@ -1251,12 +1266,24 @@ namespace Mono.Xml
 					break;
 				goto case WriteState.Closed;
 			case WriteState.Element:
+				if (isCharacter)
+					CheckMixedContentState ();
 				CloseStartElement ();
 				break;
+			case WriteState.Content:
+				if (isCharacter)
+					CheckMixedContentState ();
+				break;
 			}
 
-			if (open_count > 0 &&
-			    state != WriteState.Attribute)
+		}
+
+		void CheckMixedContentState ()
+		{
+//			if (open_count > 0 &&
+//			    state != WriteState.Attribute)
+//				elements [open_count - 1].HasSimple = true;
+			if (open_count > 0)
 				elements [open_count - 1].HasSimple = true;
 		}
 
@@ -1274,6 +1301,7 @@ namespace Mono.Xml
 					goto case WriteState.Closed;
 				if (output_xmldecl)
 					OutputAutoStartDocument ();
+				CheckMixedContentState ();
 				state = WriteState.Content;
 				break;
 			case WriteState.Attribute:
@@ -1282,14 +1310,12 @@ namespace Mono.Xml
 				goto case WriteState.Closed;
 			case WriteState.Element:
 				CloseStartElement ();
+				CheckMixedContentState ();
 				break;
 			case WriteState.Content:
+				CheckMixedContentState ();
 				break;
 			}
-
-			if (open_count > 0 &&
-			    state != WriteState.Attribute)
-				elements [open_count - 1].HasSimple = true;
 		}
 
 		void WriteEscapedString (string text, bool isAttribute)
