@@ -116,35 +116,28 @@ namespace System.Xml
 			return Create (builder, null);
 		}
 
-		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (Stream stream, XmlWriterSettings settings)
 		{
-			// FIXME: this might result in encoding null reference
 			Encoding enc = settings != null ? settings.Encoding : Encoding.UTF8;
 			return Create (new StreamWriter (stream, enc), settings);
 		}
 
-		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (string file, XmlWriterSettings settings)
 		{
-			// FIXME: this might result in encoding null reference
 			Encoding enc = settings != null ? settings.Encoding : Encoding.UTF8;
 			return Create (new StreamWriter (file, false, enc), settings);
 		}
 
-		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (StringBuilder builder, XmlWriterSettings settings)
 		{
-			return Create (new StringWriter (builder), null);
+			return Create (new StringWriter (builder), settings);
 		}
 
-		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (TextWriter writer, XmlWriterSettings settings)
 		{
 			return CreateTextWriter (writer, settings);
 		}
 
-		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (XmlWriter writer, XmlWriterSettings settings)
 		{
 			if (settings == null)
@@ -177,7 +170,7 @@ namespace System.Xml
 			return Create (xtw, settings);
 		}
 
-		public virtual void Dispose (bool disposing)
+		protected virtual void Dispose (bool disposing)
 		{
 			Close ();
 		}
@@ -254,13 +247,9 @@ namespace System.Xml
 			// In MS.NET (1.0), this check is done *here*, not at WriteStartAttribute.
 			// (XmlTextWriter.WriteStartAttribute("xmlns", "anyname", null) throws an exception.
 
-#if NET_1_0
-			if ((prefix == "xmlns" || (prefix == "" && localName == "xmlns")) && ns == null)
-				ns = "http://www.w3.org/2000/xmlns/";
-#endif
-
 			WriteStartAttribute (prefix, localName, ns);
-			WriteString (value);
+			if (value != null && value.Length > 0)
+				WriteString (value);
 			WriteEndAttribute ();
 		}
 
@@ -290,14 +279,16 @@ namespace System.Xml
 		public void WriteElementString (string localName, string value)
 		{
 			WriteStartElement(localName);
-			WriteString(value);
+			if (value != null && value.Length > 0)
+				WriteString(value);
 			WriteEndElement();
 		}
 
 		public void WriteElementString (string localName, string ns, string value)
 		{
 			WriteStartElement(localName, ns);
-			WriteString(value);
+			if (value != null && value.Length > 0)
+				WriteString(value);
 			WriteEndElement();
 		}
 
@@ -305,7 +296,8 @@ namespace System.Xml
 		public void WriteElementString (string prefix, string localName, string ns, string value)
 		{
 			WriteStartElement(prefix, localName, ns);
-			WriteString(value);
+			if (value != null && value.Length > 0)
+				WriteString(value);
 			WriteEndElement();
 		}
 #endif
@@ -438,9 +430,41 @@ namespace System.Xml
 			case XPathNodeType.ProcessingInstruction:
 				WriteProcessingInstruction (navigator.Name, navigator.Value);
 				break;
-			default:
-				WriteNode (navigator.ReadSubtree (), defattr);
+			case XPathNodeType.Root:
+				if (navigator.MoveToFirstChild ()) {
+					do {
+						WriteNode (navigator, defattr);
+					} while (navigator.MoveToNext ());
+					navigator.MoveToParent ();
+				}
 				break;
+			case XPathNodeType.Element:
+				WriteStartElement (navigator.Prefix, navigator.LocalName, navigator.NamespaceURI);
+				if (navigator.MoveToFirstNamespace (XPathNamespaceScope.Local)) {
+					do {
+						WriteNode (navigator, defattr);
+					} while (navigator.MoveToNextNamespace (XPathNamespaceScope.Local));
+					navigator.MoveToParent ();
+				}
+				if (navigator.MoveToFirstAttribute ()) {
+					do {
+						WriteNode (navigator, defattr);
+					} while (navigator.MoveToNextAttribute ());
+					navigator.MoveToParent ();
+				}
+				if (navigator.MoveToFirstChild ()) {
+					do {
+						WriteNode (navigator, defattr);
+					} while (navigator.MoveToNext ());
+					navigator.MoveToParent ();
+				}
+				if (navigator.IsEmptyElement)
+					WriteEndElement ();
+				else
+					WriteFullEndElement ();
+				break;
+			default:
+				throw new NotSupportedException ();
 			}
 		}
 #endif
