@@ -1856,6 +1856,37 @@ namespace MonoTests.System.Xml
 				sw.ToString ());
 		}
 
+		[Test]
+		public void Indent3 ()
+		{
+			XmlDocument doc = new XmlDocument ();
+			doc.PreserveWhitespace = true;
+			string s;
+
+			doc.LoadXml ("<root><element></element><!-- comment indented --><element>sample <!-- comment non-indented --></element></root>");
+			s = GetIndentedOutput (doc.DocumentElement);
+			Assert.AreEqual (s, String.Format ("<root>{0}  <element>{0}  </element>{0}  <!-- comment indented -->{0}  <element>sample <!-- comment non-indented --></element>{0}</root>", "\n"), "#1");
+
+			doc.LoadXml ("<root> \n<mid> \n<mid>   \n<child attr='value'>sample <nested attr='value' /> string</child>     <child2 attr='value'>sample string</child2>  <empty attr='value'/>\n<a>test</a> \n</mid> <returnValue>  <returnType>System.String</returnType>  </returnValue>  </mid>   </root>");
+			s = GetIndentedOutput (doc.DocumentElement);
+			Assert.AreEqual (s, String.Format ("<root> {0}<mid> {0}<mid>   {0}<child attr='value'>sample <nested attr='value' /> string</child>     <child2 attr='value'>sample string</child2>  <empty attr='value' />{0}<a>test</a> {0}</mid> <returnValue>  <returnType>System.String</returnType>  </returnValue>  </mid>   </root>", "\n"), "#2");
+
+			doc.LoadXml ("<!-- after /MemberType and after /returnValue --><root><MemberType>blah</MemberType>\n  <returnValue><returnType>System.String</returnType></returnValue>\n  <Docs><summary>text</summary><value>text<see cref='ttt' /></value><remarks/></Docs></root>");
+			s = GetIndentedOutput (doc.DocumentElement);
+			Assert.AreEqual (s, String.Format ("<root>{0}  <MemberType>blah</MemberType>{0}  <returnValue><returnType>System.String</returnType></returnValue>{0}  <Docs><summary>text</summary><value>text<see cref='ttt' /></value><remarks /></Docs></root>", "\n"), "#3");
+		}
+
+		string GetIndentedOutput (XmlNode n)
+		{
+			StringWriter sw = new StringWriter ();
+			sw.NewLine = "\n";
+			XmlTextWriter xtw = new XmlTextWriter (sw);
+			xtw.QuoteChar = '\'';
+			xtw.Formatting = Formatting.Indented;
+			n.WriteTo (xtw);
+			return sw.ToString ();
+		}
+
 		[Test]
 		public void CloseTwice ()
 		{
@@ -2199,6 +2230,60 @@ namespace MonoTests.System.Xml
 			Assert.AreEqual ("<stream:stream version='1.0' to='me@test.com' from='server' xmlns='jabber:client' xmlns:stream='http://etherx.jabber.org/streams'>", StringWriterText);
 		}
 
+		[Test] // bug #78148
+		public void UpdateFormattingOnTheFly ()
+		{
+			XmlTextWriter w = new XmlTextWriter (TextWriter.Null);
+			w.WriteStartElement ("test");
+			w.Formatting = Formatting.Indented;
+		}
+
+		[Test] // bug #78598
+		public void WriteGlobalAttributeInDefaultNS ()
+		{
+			StringWriter sw = new StringWriter ();
+			XmlTextWriter tw = new XmlTextWriter (sw);
+			string ns = "http://schemas.xmlsoap.org/soap/envelope/";
+			tw.WriteStartElement ("Envelope");
+			tw.WriteAttributeString ("xmlns", ns);
+			int start = sw.ToString ().Length;
+			tw.WriteStartElement ("UserInfo");
+			tw.WriteStartAttribute ("actor", ns);
+			tw.WriteEndAttribute ();
+			tw.WriteEndElement ();
+			tw.WriteEndElement ();
+			Assert.IsTrue (sw.ToString ().IndexOf (ns, start) > 0);
+		}
+
+		[Test]
+		public void WriteCommentPIAndIndent ()
+		{
+			StringWriter sw = new StringWriter ();
+			XmlTextWriter w = new XmlTextWriter (sw);
+			w.Formatting = Formatting.Indented;
+			w.WriteStartElement ("foo");
+			w.WriteComment ("test");
+			w.WriteProcessingInstruction ("PI", "");
+			w.WriteStartElement ("child");
+			w.WriteEndElement ();
+			w.WriteComment ("test");
+			w.WriteString ("STRING");
+			w.WriteEndElement ();
+			Assert.AreEqual (String.Format (@"<foo>{0}  <!--test-->{0}  <?PI ?>{0}  <child />{0}  <!--test-->STRING</foo>", Environment.NewLine), sw.ToString ());
+		}
+
+		[Test]
+		public void WriteBinHexAttribute () // for bug #79019
+		{
+			XmlWriter writer = new XmlTextWriter (TextWriter.Null);
+			writer.WriteStartElement ("test");
+			byte [] buffer1 = new byte [] {200, 155};
+			writer.WriteStartAttribute ("key", "");
+			writer.WriteBinHex (buffer1, 0, buffer1.Length);
+			writer.WriteEndAttribute ();
+			writer.WriteEndElement ();
+		}
+
 #if NET_2_0
 		[Test]
 		[ExpectedException (typeof (InvalidOperationException))]
