@@ -30,11 +30,11 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Collections;
 using System.Reflection;
 using System.Globalization;
 using System.ComponentModel.Design;
+using System.Security.Permissions;
 
 namespace System.ComponentModel
 {
@@ -72,6 +72,7 @@ public sealed class TypeDescriptor
 		return null;
 	}
 
+	[ReflectionPermission (SecurityAction.LinkDemand, TypeInformation = true, MemberAccess = true)]
 	public static EventDescriptor CreateEvent (Type componentType,
 						   string name,
 						   Type type,
@@ -80,6 +81,7 @@ public sealed class TypeDescriptor
 		return new ReflectionEventDescriptor (componentType, name, type, attributes);
 	}
 
+	[ReflectionPermission (SecurityAction.LinkDemand, TypeInformation = true, MemberAccess = true)]
 	public static EventDescriptor CreateEvent (Type componentType,
 						   EventDescriptor oldEventDescriptor,
 						   Attribute [] attributes)
@@ -87,6 +89,7 @@ public sealed class TypeDescriptor
 		return new ReflectionEventDescriptor (componentType, oldEventDescriptor, attributes);
 	}
 
+	[ReflectionPermission (SecurityAction.LinkDemand, TypeInformation = true, MemberAccess = true)]
 	public static PropertyDescriptor CreateProperty (Type componentType,
 							 string name,
 							 Type type,
@@ -95,6 +98,7 @@ public sealed class TypeDescriptor
 		return new ReflectionPropertyDescriptor (componentType, name, type, attributes);
 	}
 
+	[ReflectionPermission (SecurityAction.LinkDemand, TypeInformation = true, MemberAccess = true)]
 	public static PropertyDescriptor CreateProperty (Type componentType,
 							 PropertyDescriptor oldPropertyDescriptor,
 							 Attribute [] attributes)
@@ -516,7 +520,9 @@ public sealed class TypeDescriptor
 	}
 
 	public static IComNativeDescriptorHandler ComNativeDescriptorHandler {
+		[PermissionSet (SecurityAction.LinkDemand, Unrestricted = true)]
 		get { return descriptorHandler; }
+		[PermissionSet (SecurityAction.LinkDemand, Unrestricted = true)]
 		set { descriptorHandler = value; }
 	}
 
@@ -595,7 +601,8 @@ public sealed class TypeDescriptor
 	{
 		if (component != null && component.Site != null) {
 			ITypeResolutionService resver = (ITypeResolutionService) component.Site.GetService (typeof(ITypeResolutionService));
-			if (resver != null) return resver.GetType (typeName, true, false);
+			if (resver != null)
+				return resver.GetType (typeName, true, false);
 		}
 		
 		Type t = Type.GetType (typeName);
@@ -630,20 +637,25 @@ public sealed class TypeDescriptor
 		public EventDescriptorCollection GetEvents (Attribute[] attributes)
 		{
 			EventDescriptorCollection evs = GetEvents ();
-			if (attributes == null) return evs;
-			else return evs.Filter (attributes);
+			if (attributes == null)
+				return evs;
+			else
+				return evs.Filter (attributes);
 		}
 		
 		public PropertyDescriptorCollection GetProperties (Attribute[] attributes)
 		{
 			PropertyDescriptorCollection props = GetProperties ();
-			if (attributes == null) return props;
-			else return props.Filter (attributes);
+			if (attributes == null)
+				return props;
+			else
+				return props.Filter (attributes);
 		}
 		
 		public EventDescriptor GetDefaultEvent ()
 		{
-			if (_gotDefaultEvent) return _defaultEvent;
+			if (_gotDefaultEvent)
+				return _defaultEvent;
 			
 			DefaultEventAttribute attr = (DefaultEventAttribute) GetAttributes()[typeof(DefaultEventAttribute)];
 			if (attr == null || attr.Name == null) 
@@ -671,7 +683,8 @@ public sealed class TypeDescriptor
 		
 		public PropertyDescriptor GetDefaultProperty ()
 		{
-			if (_gotDefaultProperty) return _defaultProperty;
+			if (_gotDefaultProperty)
+				return _defaultProperty;
 			
 			DefaultPropertyAttribute attr = (DefaultPropertyAttribute) GetAttributes()[typeof(DefaultPropertyAttribute)];
 			if (attr == null || attr.Name == null) 
@@ -686,7 +699,8 @@ public sealed class TypeDescriptor
 		
 		protected AttributeCollection GetAttributes (IComponent comp)
 		{
-			if (_attributes != null) return _attributes;
+			if (_attributes != null)
+				return _attributes;
 			
 			bool cache = true;
 			object[] ats = _infoType.GetCustomAttributes (true);
@@ -703,7 +717,8 @@ public sealed class TypeDescriptor
 			ArrayList atts = new ArrayList ();
 			atts.AddRange (t.Values);
 			AttributeCollection attCol = new AttributeCollection (atts);
-			if (cache) _attributes = attCol;
+			if (cache)
+				_attributes = attCol;
 			return attCol;
 		}
 	}
@@ -726,7 +741,8 @@ public sealed class TypeDescriptor
 		
 		public override EventDescriptorCollection GetEvents ()
 		{
-			if (_events != null) return _events;
+			if (_events != null)
+				return _events;
 			
 			bool cache = true;
 			EventInfo[] events = _component.GetType().GetEvents ();
@@ -749,10 +765,11 @@ public sealed class TypeDescriptor
 		
 		public override PropertyDescriptorCollection GetProperties ()
 		{
-			if (_properties != null) return _properties;
+			if (_properties != null)
+				return _properties;
 			
 			bool cache = true;
-			PropertyInfo[] props = _component.GetType().GetProperties ();
+			PropertyInfo[] props = _component.GetType().GetProperties (BindingFlags.Instance | BindingFlags.Public);
 			Hashtable t = new Hashtable ();
 			foreach (PropertyInfo pr in props)
 				t [pr.Name] = new ReflectionPropertyDescriptor (pr);
@@ -766,7 +783,8 @@ public sealed class TypeDescriptor
 			PropertyDescriptor[] descriptors = new PropertyDescriptor[t.Values.Count];
 			t.Values.CopyTo (descriptors, 0);
 			PropertyDescriptorCollection attCol = new PropertyDescriptorCollection (descriptors, true);
-			if (cache) _properties = attCol;
+			if (cache)
+				_properties = attCol;
 			return attCol;
 		}
 	}
@@ -787,7 +805,8 @@ public sealed class TypeDescriptor
 		
 		public override EventDescriptorCollection GetEvents ()
 		{
-			if (_events != null) return _events;
+			if (_events != null)
+				return _events;
 			
 			EventInfo[] events = InfoType.GetEvents ();
 			EventDescriptor[] descs = new EventDescriptor [events.Length];
@@ -800,14 +819,16 @@ public sealed class TypeDescriptor
 		
 		public override PropertyDescriptorCollection GetProperties ()
 		{
-			if (_properties != null) return _properties;
+			if (_properties != null)
+				return _properties;
 			
-			PropertyInfo[] props = InfoType.GetProperties ();
-			PropertyDescriptor[] descs = new PropertyDescriptor [props.Length];
+			PropertyInfo[] props = InfoType.GetProperties (BindingFlags.Instance | BindingFlags.Public);
+			ArrayList descs = new ArrayList (props.Length);
 			for (int n=0; n<props.Length; n++)
-				descs [n] = new ReflectionPropertyDescriptor (props[n]);
+				if (props [n].GetIndexParameters ().Length == 0)
+					descs.Add (new ReflectionPropertyDescriptor (props[n]));
 
-			_properties = new PropertyDescriptorCollection (descs, true);
+			_properties = new PropertyDescriptorCollection ((PropertyDescriptor[]) descs.ToArray (typeof (PropertyDescriptor)), true);
 			return _properties;
 		}
 	}
