@@ -42,6 +42,7 @@ using System.Collections;
 using System.Collections.Specialized;
 using System.IO;
 using System.Runtime.CompilerServices;
+using System.Security.Permissions;
 #if (XML_DEP)
 using System.Xml;
 using System.Xml.XPath;
@@ -57,6 +58,9 @@ namespace System.Configuration
 		{
 		}
 
+#if NET_2_0
+		[Obsolete ("This method is obsolete, it has been replaced by System.Configuration!System.Configuration.ConfigurationManager.GetSection")]
+#endif
 		public static object GetConfig (string sectionName)
 		{
 			return config.GetConfig (sectionName);
@@ -64,39 +68,18 @@ namespace System.Configuration
 
 #if NET_2_0
 		[Obsolete ("This property is obsolete.  Please use System.Configuration.ConfigurationManager.AppSettings")]
-#endif
-#if NET_2_0 && CONFIGURATION_2_0
-		static NameValueCollection appSettingsCol;
 #endif
 		public static NameValueCollection AppSettings
 		{
 			get {
-#if NET_2_0 && CONFIGURATION_2_0
-#if CONFIGURATION_DEP
-				lock (lockobj) {
-					if (appSettingsCol == null) {
-						AppSettingsSection appSettings = (AppSettingsSection)GetConfig ("appSettings");
-
-						appSettingsCol = new NameValueCollection ();
-				
-						foreach (string key in appSettings.Settings.AllKeys) {
-							KeyValueConfigurationElement ele = appSettings.Settings[key];
-							appSettingsCol.Add (ele.Key, ele.Value);
-						}
-					}
-				}
-
-				return appSettingsCol;
-#else
-				return null;
-#endif
+#if NET_2_0 && CONFIGURATION_DEP
+				object appSettings = ConfigurationManager.GetSection ("appSettings");
 #else
 				object appSettings = GetConfig ("appSettings");
+#endif
 				if (appSettings == null)
 					appSettings = new NameValueCollection ();
-
 				return (NameValueCollection) appSettings;
-#endif
 			}
 		}
 
@@ -109,9 +92,6 @@ namespace System.Configuration
 			lock (lockobj) {
 				IConfigurationSystem old = config;
 				config = newSystem;
-#if NET_2_0 && CONFIGURATION_2_0
-				appSettingsCol = null;
-#endif
 				return old;
 			}
 		}
@@ -140,33 +120,8 @@ namespace System.Configuration
 #endif
 		public object GetConfig (string sectionName)
 		{
-#if NET_2_0 && CONFIGURATION_2_0
-#if CONFIGURATION_DEP
-			Init ();
-			object o = ConfigurationManager.GetSection (sectionName);
-			if (o == null || o is IgnoreSection) {
-				/* this can happen when the section
-				 * handler doesn't subclass from
-				 * ConfigurationSection.  let's be
-				 * nice and try to load it using the
-				 * 1.x style routines in case there's
-				 * a 1.x section handler registered
-				 * for it.
-				 */
-				object o1 = config.GetConfig (sectionName);
-				if (o1 != null)
-					return o1;
-			}
-
-			return o;
-#else
-			return null;
-#endif
-
-#else
 			Init ();
 			return config.GetConfig (sectionName);
-#endif
 		}
 
 		public void Init ()
@@ -271,6 +226,8 @@ namespace System.Configuration
 			factories = new Hashtable ();
 		}
 
+		// SECURITY-FIXME: limit this with an imperative assert for reading the specific file
+		[FileIOPermission (SecurityAction.Assert, Unrestricted = true)]
 		public bool Load (string fileName)
 		{
 			this.fileName = fileName;
@@ -570,7 +527,7 @@ namespace System.Configuration
 					value = reader.Value;
 				}
 				else 
-#if NET_2_0 && CONFIGURATION_2_0
+#if NET_2_0
 				if (reader.Name != "type")
 #endif
 					ThrowException ("Unrecognized attribute.", reader);
