@@ -1,8 +1,26 @@
 /* -*- Mode: Csharp; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+//
+//
+// This class has several problems:
+//
+//   * No buffering, the specification requires that there is buffering, this
+//     matters because a few methods expose strings and chars and the reading
+//     is encoding sensitive.   This means that when we do a read of a byte
+//     sequence that can not be turned into a full string by the current encoding
+//     we should keep a buffer with this data, and read from it on the next
+//     iteration.
+//
+//   * Calls to read_serial from the unmanaged C do not check for errors,
+//     like EINTR, that should be retried
+//
+//   * Calls to the encoder that do not consume all bytes because of partial
+//     reads 
+//
 
 #if NET_2_0
 
 using System;
+using System.Collections.Generic;
 using System.ComponentModel;
 using System.Text;
 using System.Runtime.InteropServices;
@@ -19,21 +37,21 @@ namespace System.IO.Ports
 		const Parity DefaultParity = Parity.None;
 		const StopBits DefaultStopBits = StopBits.One;
 
-		bool isOpen;
-		int baudRate;
+		bool is_open;
+		int baud_rate;
 		Parity parity;
-		StopBits stopBits;
-		Handshake handshake = Handshake.None;
-		int dataBits;
-		bool breakState = false;
+		StopBits stop_bits;
+		Handshake handshake;
+		int data_bits;
+		bool break_state = false;
 		bool dtr_enable = false;
 		bool rts_enable = false;
-		SerialPortStream stream;
+		ISerialStream stream;
 		Encoding encoding = Encoding.ASCII;
-		string newLine = Environment.NewLine;
-		string portName;
-		int readTimeout = InfiniteTimeout;
-		int writeTimeout = InfiniteTimeout;
+		string new_line = Environment.NewLine;
+		string port_name;
+		int read_timeout = InfiniteTimeout;
+		int write_timeout = InfiniteTimeout;
 		int readBufferSize = DefaultReadBufferSize;
 		int writeBufferSize = DefaultWriteBufferSize;
 		object error_received = new object ();
@@ -75,11 +93,11 @@ namespace System.IO.Ports
 
 		public SerialPort (string portName, int baudRate, Parity parity, int dataBits, StopBits stopBits) 
 		{
-			this.portName = portName;
-			this.baudRate = baudRate;
+			port_name = portName;
+			baud_rate = baudRate;
+			data_bits = dataBits;
+			stop_bits = stopBits;
 			this.parity = parity;
-			this.dataBits = dataBits;
-			this.stopBits = stopBits;
 		}
 
 		static string GetDefaultPortName ()
@@ -89,38 +107,40 @@ namespace System.IO.Ports
 
 		public Stream BaseStream {
 			get {
-				if (!isOpen)
+				if (!is_open)
 					throw new InvalidOperationException ();
 
-				return stream;
+				return (Stream) stream;
 			}
 		}
 
+		[DefaultValueAttribute (DefaultBaudRate)]
 		public int BaudRate {
 			get {
-				return baudRate;
+				return baud_rate;
 			}
 			set {
 				if (value <= 0)
 					throw new ArgumentOutOfRangeException ("value");
 				
-				baudRate = value;
-				if (isOpen)
-					stream.BaudRate = value;
+				if (is_open)
+					stream.SetAttributes (value, parity, data_bits, stop_bits, handshake);
+				
+				baud_rate = value;
 			}
 		}
 
 		public bool BreakState {
 			get {
-				return breakState;
+				return break_state;
 			}
 			set {
 				CheckOpen ();
-				if (value == breakState)
+				if (value == break_state)
 					return; // Do nothing.
 
-				breakState = value;
-				// Update the state
+				stream.SetBreakState (value);
+				break_state = value;
 			}
 		}
 
@@ -141,31 +161,34 @@ namespace System.IO.Ports
 		public bool CDHolding {
 			get {
 				CheckOpen ();
-				return stream.GetSignal (SerialSignal.Cd);
+				return (stream.GetSignals () & SerialSignal.Cd) != 0;
 			}
 		}
 
 		public bool CtsHolding {
 			get {
 				CheckOpen ();
-				return stream.GetSignal (SerialSignal.Cts);
+				return (stream.GetSignals () & SerialSignal.Cts) != 0;
 			}
 		}
 
+		[DefaultValueAttribute(DefaultDataBits)]
 		public int DataBits {
 			get {
-				return dataBits;
+				return data_bits;
 			}
 			set {
 				if (value < 5 || value > 8)
 					throw new ArgumentOutOfRangeException ("value");
 
-				dataBits = value;
-				if (isOpen)
-					stream.DataBits = value;
+				if (is_open)
+					stream.SetAttributes (baud_rate, parity, value, stop_bits, handshake);
+				
+				data_bits = value;
 			}
 		}
 
+		[MonoTODO("Not implemented")]
 		public bool DiscardNull {
 			get {
 				CheckOpen ();
@@ -180,10 +203,11 @@ namespace System.IO.Ports
 		public bool DsrHolding {
 			get {
 				CheckOpen ();
-				return stream.GetSignal (SerialSignal.Dsr);
+				return (stream.GetSignals () & SerialSignal.Dsr) != 0;
 			}
 		}
 
+		[DefaultValueAttribute(false)]
 		public bool DtrEnable {
 			get {
 				return dtr_enable;
@@ -191,7 +215,7 @@ namespace System.IO.Ports
 			set {
 				if (value == dtr_enable)
 					return;
-				if (isOpen)
+				if (is_open)
 					stream.SetSignal (SerialSignal.Dtr, value);
 				
 				dtr_enable = value;
@@ -210,6 +234,7 @@ namespace System.IO.Ports
 			}
 		}
 
+		[DefaultValueAttribute(Handshake.None)]
 		public Handshake Handshake {
 			get {
 				return handshake;
@@ -218,30 +243,33 @@ namespace System.IO.Ports
 				if (value < Handshake.None || value > Handshake.RequestToSendXOnXOff)
 					throw new ArgumentOutOfRangeException ("value");
 
+				if (is_open)
+					stream.SetAttributes (baud_rate, parity, data_bits, stop_bits, value);
+				
 				handshake = value;
-				if (isOpen)
-					stream.Handshake = value;
 			}
 		}
 
 		public bool IsOpen {
 			get {
-				return isOpen;
+				return is_open;
 			}
 		}
 
+		[DefaultValueAttribute("\n")]
 		public string NewLine {
 			get {
-				return newLine;
+				return new_line;
 			}
 			set {
 				if (value == null)
 					throw new ArgumentNullException ("value");
 				
-				newLine = value;
+				new_line = value;
 			}
 		}
 
+		[DefaultValueAttribute(DefaultParity)]
 		public Parity Parity {
 			get {
 				return parity;
@@ -250,12 +278,14 @@ namespace System.IO.Ports
 				if (value < Parity.None || value > Parity.Space)
 					throw new ArgumentOutOfRangeException ("value");
 
+				if (is_open)
+					stream.SetAttributes (baud_rate, value, data_bits, stop_bits, handshake);
+				
 				parity = value;
-				if (isOpen)
-					stream.Parity = value;
 			}
 		}
 
+		[MonoTODO("Not implemented")]
 		public byte ParityReplace {
 			get {
 				throw new NotImplementedException ();
@@ -265,28 +295,30 @@ namespace System.IO.Ports
 			}
 		}
 
+		
 		public string PortName {
 			get {
-				return portName;
+				return port_name;
 			}
 			set {
-				if (isOpen)
+				if (is_open)
 					throw new InvalidOperationException ("Port name cannot be set while port is open.");
 				if (value == null)
 					throw new ArgumentNullException ("value");
 				if (value.Length == 0 || value.StartsWith ("\\\\"))
 					throw new ArgumentException ("value");
 
-				portName = value;
+				port_name = value;
 			}
 		}
 
+		[DefaultValueAttribute(DefaultReadBufferSize)]
 		public int ReadBufferSize {
 			get {
 				return readBufferSize;
 			}
 			set {
-				if (isOpen)
+				if (is_open)
 					throw new InvalidOperationException ();
 				if (value <= 0)
 					throw new ArgumentOutOfRangeException ("value");
@@ -297,20 +329,24 @@ namespace System.IO.Ports
 			}
 		}
 
+		[DefaultValueAttribute(InfiniteTimeout)]
 		public int ReadTimeout {
 			get {
-				return readTimeout;
+				return read_timeout;
 			}
 			set {
 				if (value <= 0 && value != InfiniteTimeout)
 					throw new ArgumentOutOfRangeException ("value");
 
-				readTimeout = value;
-				if (isOpen)
+				if (is_open)
 					stream.ReadTimeout = value;
+				
+				read_timeout = value;
 			}
 		}
 
+		[MonoTODO("Not implemented")]
+		[DefaultValueAttribute(1)]
 		public int ReceivedBytesThreshold {
 			get {
 				throw new NotImplementedException ();
@@ -323,6 +359,7 @@ namespace System.IO.Ports
 			}
 		}
 
+		[DefaultValueAttribute(false)]
 		public bool RtsEnable {
 			get {
 				return rts_enable;
@@ -330,33 +367,36 @@ namespace System.IO.Ports
 			set {
 				if (value == rts_enable)
 					return;
-				if (isOpen)
+				if (is_open)
 					stream.SetSignal (SerialSignal.Rts, value);
 				
 				rts_enable = value;
 			}
 		}
 
+		[DefaultValueAttribute(DefaultStopBits)]
 		public StopBits StopBits {
 			get {
-				return stopBits;
+				return stop_bits;
 			}
 			set {
 				if (value < StopBits.One || value > StopBits.OnePointFive)
 					throw new ArgumentOutOfRangeException ("value");
 				
-				stopBits = value;
-				if (isOpen)
-					stream.StopBits = value;
+				if (is_open)
+					stream.SetAttributes (baud_rate, parity, data_bits, value, handshake);
+				
+				stop_bits = value;
 			}
 		}
 
+		[DefaultValueAttribute(DefaultWriteBufferSize)]
 		public int WriteBufferSize {
 			get {
 				return writeBufferSize;
 			}
 			set {
-				if (isOpen)
+				if (is_open)
 					throw new InvalidOperationException ();
 				if (value <= 0)
 					throw new ArgumentOutOfRangeException ("value");
@@ -367,17 +407,19 @@ namespace System.IO.Ports
 			}
 		}
 
+		[DefaultValueAttribute(InfiniteTimeout)]
 		public int WriteTimeout {
 			get {
-				return writeTimeout;
+				return write_timeout;
 			}
 			set {
 				if (value <= 0 && value != InfiniteTimeout)
 					throw new ArgumentOutOfRangeException ("value");
 
-				writeTimeout = value;
-				if (isOpen)
+				if (is_open)
 					stream.WriteTimeout = value;
+				
+				write_timeout = value;
 			}
 		}
 
@@ -390,10 +432,10 @@ namespace System.IO.Ports
 
 		protected override void Dispose (bool disposing)
 		{
-			if (!isOpen)
+			if (!is_open)
 				return;
 			
-			isOpen = false;
+			is_open = false;
 			stream.Close ();
 			stream = null;
 		}
@@ -401,32 +443,54 @@ namespace System.IO.Ports
 		public void DiscardInBuffer ()
 		{
 			CheckOpen ();
-			stream.DiscardInputBuffer ();
+			stream.DiscardInBuffer ();
 		}
 
 		public void DiscardOutBuffer ()
 		{
 			CheckOpen ();
-			stream.DiscardOutputBuffer ();
+			stream.DiscardOutBuffer ();
 		}
 
 		public static string [] GetPortNames ()
 		{
 			int p = (int) Environment.OSVersion.Platform;
-			if (p == 4 || p == 128) // Are we on Unix?
-				return Directory.GetFiles ("/dev/", "ttyS*");
-
+			
+			// Are we on Unix?
+			if (p == 4 || p == 128){
+				string [] ttys = Directory.GetFiles ("/dev/", "tty*");
+				List<string> serial_ports = new List<string> ();
+				
+				foreach (string dev in ttys){
+					if (dev.StartsWith ("/dev/ttyS") || dev.StartsWith ("/dev/ttyUSB"))
+						serial_ports.Add (dev);
+						
+				}
+				return serial_ports.ToArray ();
+			}
 			throw new NotImplementedException ("Detection of ports is not implemented for this platform yet.");
 		}
 
+		static bool IsWindows {
+			get {
+				PlatformID id =  Environment.OSVersion.Platform;
+				return id == PlatformID.Win32Windows || id == PlatformID.Win32NT; // WinCE not supported
+			}
+		}
+
 		public void Open ()
 		{
-			if (isOpen)
+			if (is_open)
 				throw new InvalidOperationException ("Port is already open");
 			
-			stream = new SerialPortStream (portName, baudRate, dataBits, parity, stopBits, dtr_enable,
-					rts_enable, handshake, readTimeout, writeTimeout, readBufferSize, writeBufferSize);
-			isOpen = true;
+			if (IsWindows) // Use windows kernel32 backend
+				stream = new WinSerialStream (port_name, baud_rate, data_bits, parity, stop_bits,
+						handshake, read_timeout, write_timeout, readBufferSize, writeBufferSize);
+			else // Use standard unix backend
+				stream = new SerialPortStream (port_name, baud_rate, data_bits, parity, stop_bits, dtr_enable,
+					rts_enable, handshake, read_timeout, write_timeout, readBufferSize, writeBufferSize);
+			
+			is_open = true;
 		}
 
 		public int Read (byte[] buffer, int offset, int count)
@@ -434,54 +498,95 @@ namespace System.IO.Ports
 			CheckOpen ();
 			if (buffer == null)
 				throw new ArgumentNullException ("buffer");
-			if (offset < 0 || offset >= buffer.Length)
-				throw new ArgumentOutOfRangeException ("offset");
-			if (count < 0 || count > buffer.Length)
-				throw new ArgumentOutOfRangeException ("count");
-			if (count > buffer.Length - offset)
-				throw new ArgumentException ("count > buffer.Length - offset");
+			if (offset < 0 || count < 0)
+				throw new ArgumentOutOfRangeException ("offset or count less than zero.");
+
+			if (buffer.Length - offset < count )
+				throw new ArgumentException ("offset+count",
+							      "The size of the buffer is less than offset + count.");
 			
 			return stream.Read (buffer, offset, count);
 		}
 
+		[Obsolete("Read of char buffers is currently broken")]
+		[MonoTODO("This is broken")]
 		public int Read (char[] buffer, int offset, int count)
 		{
 			CheckOpen ();
 			if (buffer == null)
 				throw new ArgumentNullException ("buffer");
-			if (offset < 0 || offset >= buffer.Length)
-				throw new ArgumentOutOfRangeException ("offset");
-			if (count < 0 || count > buffer.Length)
-				throw new ArgumentOutOfRangeException ("count");
-			if (count > buffer.Length - offset)
-				throw new ArgumentException ("count > buffer.Length - offset");
+			if (offset < 0 || count < 0)
+				throw new ArgumentOutOfRangeException ("offset or count less than zero.");
 
+			if (buffer.Length - offset < count )
+				throw new ArgumentException ("offset+count",
+							      "The size of the buffer is less than offset + count.");
+
+			// The following code does not work, we nee to reintroduce a buffer stream somewhere
+			// for this to work;  In addition the code is broken.
 			byte [] bytes = encoding.GetBytes (buffer, offset, count);
 			return stream.Read (bytes, 0, bytes.Length);
 		}
 
-		public int ReadByte ()
+		internal int read_byte ()
 		{
 			byte [] buff = new byte [1];
-			if (Read (buff, 0, 1) > 0)
+			if (stream.Read (buff, 0, 1) > 0)
 				return buff [0];
 
 			return -1;
 		}
+		
+		public int ReadByte ()
+		{
+			CheckOpen ();
+			return read_byte ();
+		}
 
 		public int ReadChar ()
 		{
-			throw new NotImplementedException ();
+			CheckOpen ();
+			
+			byte [] buffer = new byte [16];
+			int i = 0;
+
+			do {
+				int b = read_byte ();
+				if (b == -1)
+					return -1;
+				buffer [i++] = (byte) b;
+				char [] c = encoding.GetChars (buffer, 0, 1);
+				if (c.Length > 0)
+					return (int) c [0];
+			} while (i < buffer.Length);
+
+			return -1;
 		}
 
 		public string ReadExisting ()
 		{
-			throw new NotImplementedException ();
+			CheckOpen ();
+			
+			int count = BytesToRead;
+			byte [] bytes = new byte [count];
+			
+			int n = stream.Read (bytes, 0, count);
+			return new String (encoding.GetChars (bytes, 0, n));
 		}
 
 		public string ReadLine ()
 		{
-			return ReadTo (newLine);
+			CheckOpen ();
+			List<byte> bytes_read = new List<byte>();
+			byte [] buff = new byte [1];
+			
+			while (true){
+				int n = stream.Read (buff, 0, 1);
+				if (n == -1 || buff [0] == '\n')
+					break;
+				bytes_read.Add (buff [0]);
+			} 
+			return new String (encoding.GetChars (bytes_read.ToArray ()));
 		}
 
 		public string ReadTo (string value)
@@ -492,7 +597,25 @@ namespace System.IO.Ports
 			if (value.Length == 0)
 				throw new ArgumentException ("value");
 
-			throw new NotImplementedException ();
+			// Turn into byte array, so we can compare
+			byte [] byte_value = encoding.GetBytes (value);
+			int current = 0;
+			List<byte> seen = new List<byte> ();
+
+			while (true){
+				int n = read_byte ();
+				if (n == -1)
+					break;
+				seen.Add ((byte)n);
+				if (n == byte_value [current]){
+					current++;
+					if (current == byte_value.Length)
+						return encoding.GetString (seen.ToArray (), 0, seen.Count - byte_value.Length);
+				} else {
+					current = (byte_value [0] == n) ? 1 : 0;
+				}
+			}
+			return encoding.GetString (seen.ToArray ());
 		}
 
 		public void Write (string str)
@@ -510,12 +633,13 @@ namespace System.IO.Ports
 			CheckOpen ();
 			if (buffer == null)
 				throw new ArgumentNullException ("buffer");
-			if (offset < 0 || offset >= buffer.Length)
-				throw new ArgumentOutOfRangeException ("offset");
-			if (count < 0 || count > buffer.Length)
-				throw new ArgumentOutOfRangeException ("count");
-			if (count > buffer.Length - offset)
-				throw new ArgumentException ("count > buffer.Length - offset");
+
+			if (offset < 0 || count < 0)
+				throw new ArgumentOutOfRangeException ();
+
+			if (buffer.Length - offset < count)
+				throw new ArgumentException ("offset+count",
+							     "The size of the buffer is less than offset + count.");
 
 			stream.Write (buffer, offset, count);
 		}
@@ -538,12 +662,12 @@ namespace System.IO.Ports
 
 		public void WriteLine (string str)
 		{
-			Write (str + newLine);
+			Write (str + new_line);
 		}
 
 		void CheckOpen ()
 		{
-			if (!isOpen)
+			if (!is_open)
 				throw new InvalidOperationException ("Specified port is not open.");
 		}
 
