@@ -57,14 +57,6 @@ namespace Microsoft.Win32
 		// FIXME this is hard coded on Mono, can it be determined dynamically? 
 		readonly int NativeBytesPerCharacter = Marshal.SystemDefaultCharSize;
 
-		internal enum RegistryType {
-			String = 1,
-			EnvironmentString = 2,
-			Binary = 3,
-			Dword = 4,
-			StringArray = 7,
-		}
-		
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegCreateKey")]
 		static extern int RegCreateKey (IntPtr keyBase, string keyName, out IntPtr keyHandle);
 	       
@@ -86,47 +78,47 @@ namespace Microsoft.Win32
 		private static extern int RegDeleteValue (IntPtr keyHandle, string valueName);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegEnumKey")]
-		private static extern int RegEnumKey (IntPtr keyBase, int index, [Out] byte[] nameBuffer, int bufferLength);
+		private static extern int RegEnumKey (IntPtr keyBase, int index, StringBuilder nameBuffer, int bufferLength);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegEnumValue")]
 		private static extern int RegEnumValue (IntPtr keyBase, 
 				int index, StringBuilder nameBuffer, 
 				ref int nameLength, IntPtr reserved, 
-				ref RegistryType type, IntPtr data, IntPtr dataLength);
+				ref RegistryValueKind type, IntPtr data, IntPtr dataLength);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegSetValueEx")]
 		private static extern int RegSetValueEx (IntPtr keyBase, 
-				string valueName, IntPtr reserved, RegistryType type,
+				string valueName, IntPtr reserved, RegistryValueKind type,
 				StringBuilder data, int rawDataLength);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegSetValueEx")]
 		private static extern int RegSetValueEx (IntPtr keyBase, 
-				string valueName, IntPtr reserved, RegistryType type,
+				string valueName, IntPtr reserved, RegistryValueKind type,
 				string data, int rawDataLength);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegSetValueEx")]
 		private static extern int RegSetValueEx (IntPtr keyBase, 
-				string valueName, IntPtr reserved, RegistryType type,
+				string valueName, IntPtr reserved, RegistryValueKind type,
 				byte[] rawData, int rawDataLength);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegSetValueEx")]
 		private static extern int RegSetValueEx (IntPtr keyBase, 
-				string valueName, IntPtr reserved, RegistryType type,
+				string valueName, IntPtr reserved, RegistryValueKind type,
 				ref int data, int rawDataLength);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegQueryValueEx")]
 		private static extern int RegQueryValueEx (IntPtr keyBase,
-				string valueName, IntPtr reserved, ref RegistryType type,
+				string valueName, IntPtr reserved, ref RegistryValueKind type,
 				IntPtr zero, ref int dataSize);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegQueryValueEx")]
 		private static extern int RegQueryValueEx (IntPtr keyBase,
-				string valueName, IntPtr reserved, ref RegistryType type,
+				string valueName, IntPtr reserved, ref RegistryValueKind type,
 				[Out] byte[] data, ref int dataSize);
 
 		[DllImport ("advapi32.dll", CharSet=CharSet.Unicode, EntryPoint="RegQueryValueEx")]
 		private static extern int RegQueryValueEx (IntPtr keyBase,
-				string valueName, IntPtr reserved, ref RegistryType type,
+				string valueName, IntPtr reserved, ref RegistryValueKind type,
 				ref int data, ref int dataSize);
 
 		// Returns our handle from the RegistryKey
@@ -136,42 +128,50 @@ namespace Microsoft.Win32
 				: (IntPtr) key.Data;
 		}
 
-		/// <summary>
-		///	Acctually read a registry value. Requires knoledge of the
-		///	value's type and size.
-		/// </summary>
-		public object GetValue (RegistryKey rkey, string name, bool returnDefaultValue, object defaultValue)
+		static bool IsHandleValid (RegistryKey key)
 		{
-			RegistryType type = 0;
+			return key.Data != null;
+		}
+
+		/// <summary>
+		/// Acctually read a registry value. Requires knowledge of the
+		/// value's type and size.
+		/// </summary>
+		public object GetValue (RegistryKey rkey, string name, object defaultValue, RegistryValueOptions options)
+		{
+			RegistryValueKind type = 0;
 			int size = 0;
 			object obj = null;
 			IntPtr handle = GetHandle (rkey);
 			int result = RegQueryValueEx (handle, name, IntPtr.Zero, ref type, IntPtr.Zero, ref size);
 
-			if (result == Win32ResultCode.FileNotFound) {
-				if (returnDefaultValue) {
-					return defaultValue;
-				}
-				return null;
+			if (result == Win32ResultCode.FileNotFound || result == Win32ResultCode.MarkedForDeletion) {
+				return defaultValue;
 			}
 			
 			if (result != Win32ResultCode.MoreData && result != Win32ResultCode.Success ) {
 				GenerateException (result);
 			}
 			
-			if (type == RegistryType.String || type == RegistryType.EnvironmentString) {
+			if (type == RegistryValueKind.String) {
 				byte[] data;
 				result = GetBinaryValue (rkey, name, type, out data, size);
 				obj = RegistryKey.DecodeString (data);
-			} else if (type == RegistryType.Dword) {
+			} else if (type == RegistryValueKind.ExpandString) {
+				byte [] data;
+				result = GetBinaryValue (rkey, name, type, out data, size);
+				obj = RegistryKey.DecodeString (data);
+				if ((options & RegistryValueOptions.DoNotExpandEnvironmentNames) == 0)
+					obj = Environment.ExpandEnvironmentVariables ((string) obj);
+			} else if (type == RegistryValueKind.DWord) {
 				int data = 0;
 				result = RegQueryValueEx (handle, name, IntPtr.Zero, ref type, ref data, ref size);
 				obj = data;
-			} else if (type == RegistryType.Binary) {
+			} else if (type == RegistryValueKind.Binary) {
 				byte[] data;
 				result = GetBinaryValue (rkey, name, type, out data, size);
 				obj = data;
-			} else if (type == RegistryType.StringArray) {
+			} else if (type == RegistryValueKind.MultiString) {
 				obj = null;
 				byte[] data;
 				result = GetBinaryValue (rkey, name, type, out data, size);
@@ -193,6 +193,56 @@ namespace Microsoft.Win32
 			return obj;
 		}
 
+#if NET_2_0
+		//
+		// This version has to do extra checking, make sure that the requested
+		// valueKind matches the type of the value being stored
+		//
+		public void SetValue (RegistryKey rkey, string name, object value, RegistryValueKind valueKind)
+		{
+			Type type = value.GetType ();
+			int result;
+			IntPtr handle = GetHandle (rkey);
+
+			if (valueKind == RegistryValueKind.DWord && type == typeof (int)) {
+				int rawValue = (int)value;
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.DWord, ref rawValue, Int32ByteSize); 
+			} else if (valueKind == RegistryValueKind.Binary && type == typeof (byte[])) {
+				byte[] rawValue = (byte[]) value;
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.Binary, rawValue, rawValue.Length);
+			} else if (valueKind == RegistryValueKind.MultiString && type == typeof (string[])) {
+				string[] vals = (string[]) value;
+				StringBuilder fullStringValue = new StringBuilder ();
+				foreach (string v in vals)
+				{
+					fullStringValue.Append (v);
+					fullStringValue.Append ('\0');
+				}
+				fullStringValue.Append ('\0');
+
+				byte[] rawValue = Encoding.Unicode.GetBytes (fullStringValue.ToString ());
+			
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.MultiString, rawValue, rawValue.Length); 
+			} else if ((valueKind == RegistryValueKind.String || valueKind == RegistryValueKind.ExpandString) &&
+				   type == typeof (string)){
+				string rawValue = String.Format ("{0}{1}", value, '\0');
+				result = RegSetValueEx (handle, name, IntPtr.Zero, valueKind, rawValue,
+							rawValue.Length * NativeBytesPerCharacter);
+				
+			} else if (type.IsArray) {
+				throw new ArgumentException ("Only string and byte arrays can written as registry values");
+			} else {
+				throw new ArgumentException ("Type does not match the valueKind");
+			}
+
+			// handle the result codes
+			if (result != Win32ResultCode.Success)
+			{
+				GenerateException (result);
+			}
+		}
+#endif
+	
 		public void SetValue (RegistryKey rkey, string name, object value)
 		{
 			Type type = value.GetType ();
@@ -201,10 +251,10 @@ namespace Microsoft.Win32
 
 			if (type == typeof (int)) {
 				int rawValue = (int)value;
-				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryType.Dword, ref rawValue, Int32ByteSize); 
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.DWord, ref rawValue, Int32ByteSize); 
 			} else if (type == typeof (byte[])) {
 				byte[] rawValue = (byte[]) value;
-				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryType.Binary, rawValue, rawValue.Length);
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.Binary, rawValue, rawValue.Length);
 			} else if (type == typeof (string[])) {
 				string[] vals = (string[]) value;
 				StringBuilder fullStringValue = new StringBuilder ();
@@ -217,15 +267,18 @@ namespace Microsoft.Win32
 
 				byte[] rawValue = Encoding.Unicode.GetBytes (fullStringValue.ToString ());
 			
-				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryType.StringArray, rawValue, rawValue.Length); 
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.MultiString, rawValue, rawValue.Length); 
 			} else if (type.IsArray) {
 				throw new ArgumentException ("Only string and byte arrays can written as registry values");
 			} else {
 				string rawValue = String.Format ("{0}{1}", value, '\0');
-				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryType.String, rawValue,
+				result = RegSetValueEx (handle, name, IntPtr.Zero, RegistryValueKind.String, rawValue,
 							rawValue.Length * NativeBytesPerCharacter);
 			}
 
+			if (result == Win32ResultCode.MarkedForDeletion)
+				throw RegistryKey.CreateMarkedForDeletionException ();
+
 			// handle the result codes
 			if (result != Win32ResultCode.Success)
 			{
@@ -236,7 +289,7 @@ namespace Microsoft.Win32
 		/// <summary>
 		///	Get a binary value.
 		/// </summary>
-		private int GetBinaryValue (RegistryKey rkey, string name, RegistryType type, out byte[] data, int size)
+		private int GetBinaryValue (RegistryKey rkey, string name, RegistryValueKind type, out byte[] data, int size)
 		{
 			byte[] internalData = new byte [size];
 			IntPtr handle = GetHandle (rkey);
@@ -253,13 +306,17 @@ namespace Microsoft.Win32
 		
 		public int SubKeyCount (RegistryKey rkey)
 		{
-			int index, result;
-			byte[] stringBuffer = new byte [BufferMaxLength];
+			int index;
+			StringBuilder stringBuffer = new StringBuilder (BufferMaxLength);
 			IntPtr handle = GetHandle (rkey);
 			
 			for (index = 0; true; index ++) {
-				result = RegEnumKey (handle, index, stringBuffer, BufferMaxLength);
-				
+				int result = RegEnumKey (handle, index, stringBuffer,
+					stringBuffer.Capacity);
+
+				if (result == Win32ResultCode.MarkedForDeletion)
+					throw RegistryKey.CreateMarkedForDeletionException ();
+
 				if (result == Win32ResultCode.Success)
 					continue;
 				
@@ -275,7 +332,7 @@ namespace Microsoft.Win32
 		public int ValueCount (RegistryKey rkey)
 		{
 			int index, result, bufferCapacity;
-			RegistryType type;
+			RegistryValueKind type;
 			StringBuilder buffer = new StringBuilder (BufferMaxLength);
 			
 			IntPtr handle = GetHandle (rkey);
@@ -286,7 +343,10 @@ namespace Microsoft.Win32
 						       buffer, ref bufferCapacity,
 						       IntPtr.Zero, ref type, 
 						       IntPtr.Zero, IntPtr.Zero);
-				
+
+				if (result == Win32ResultCode.MarkedForDeletion)
+					throw RegistryKey.CreateMarkedForDeletionException ();
+
 				if (result == Win32ResultCode.Success || result == Win32ResultCode.MoreData)
 					continue;
 				
@@ -299,32 +359,36 @@ namespace Microsoft.Win32
 			return index;
 		}
 		
-		public RegistryKey OpenSubKey (RegistryKey rkey, string keyName, bool writtable)
+		public RegistryKey OpenSubKey (RegistryKey rkey, string keyName, bool writable)
 		{
 			int access = OpenRegKeyRead;
-			if (writtable) access |= OpenRegKeyWrite;
+			if (writable) access |= OpenRegKeyWrite;
 			IntPtr handle = GetHandle (rkey);
 			
 			IntPtr subKeyHandle;
 			int result = RegOpenKeyEx (handle, keyName, IntPtr.Zero, access, out subKeyHandle);
 
-			if (result == Win32ResultCode.FileNotFound)
+			if (result == Win32ResultCode.FileNotFound || result == Win32ResultCode.MarkedForDeletion)
 				return null;
 			
 			if (result != Win32ResultCode.Success)
 				GenerateException (result);
 			
-			return new RegistryKey (subKeyHandle, CombineName (rkey, keyName));
+			return new RegistryKey (subKeyHandle, CombineName (rkey, keyName), writable);
 		}
 
 		public void Flush (RegistryKey rkey)
 		{
+			if (!IsHandleValid (rkey))
+				return;
 			IntPtr handle = GetHandle (rkey);
 			RegFlushKey (handle);
 		}
 
 		public void Close (RegistryKey rkey)
 		{
+			if (!IsHandleValid (rkey))
+				return;
 			IntPtr handle = GetHandle (rkey);
 			RegCloseKey (handle);
 		}
@@ -335,13 +399,15 @@ namespace Microsoft.Win32
 			IntPtr subKeyHandle;
 			int result = RegCreateKey (handle , keyName, out subKeyHandle);
 
+			if (result == Win32ResultCode.MarkedForDeletion)
+				throw RegistryKey.CreateMarkedForDeletionException ();
+
 			if (result != Win32ResultCode.Success) {
 				GenerateException (result);
 			}
 			
-			RegistryKey subKey = new RegistryKey (subKeyHandle, CombineName (rkey, keyName));
-
-			return subKey;
+			return new RegistryKey (subKeyHandle, CombineName (rkey, keyName),
+				true);
 		}
 
 		public void DeleteKey (RegistryKey rkey, string keyName, bool shouldThrowWhenKeyMissing)
@@ -363,7 +429,10 @@ namespace Microsoft.Win32
 		{
 			IntPtr handle = GetHandle (rkey);
 			int result = RegDeleteValue (handle, value);
-			
+
+			if (result == Win32ResultCode.MarkedForDeletion)
+				return;
+
 			if (result == Win32ResultCode.FileNotFound){
 				if (shouldThrowWhenKeyMissing)
 					throw new ArgumentException ("value " + value);
@@ -377,15 +446,15 @@ namespace Microsoft.Win32
 		public string [] GetSubKeyNames (RegistryKey rkey)
 		{
 			IntPtr handle = GetHandle (rkey);
-			byte[] buffer = new byte [BufferMaxLength];
-			int bufferCapacity = BufferMaxLength;
+			StringBuilder buffer = new StringBuilder (BufferMaxLength);
 			ArrayList keys = new ArrayList ();
 				
 			for (int index = 0; true; index ++) {
-				int result = RegEnumKey (handle, index, buffer, bufferCapacity);
+				int result = RegEnumKey (handle, index, buffer, buffer.Capacity);
 
 				if (result == Win32ResultCode.Success) {
-					keys.Add (RegistryKey.DecodeString (buffer));
+					keys.Add (buffer.ToString ());
+					buffer.Length = 0;
 					continue;
 				}
 
@@ -408,7 +477,7 @@ namespace Microsoft.Win32
 			{
 				StringBuilder buffer = new StringBuilder (BufferMaxLength);
 				int bufferCapacity = buffer.Capacity;
-				RegistryType type = 0;
+				RegistryValueKind type = 0;
 				
 				int result = RegEnumValue (handle, index, buffer, ref bufferCapacity,
 							IntPtr.Zero, ref type, IntPtr.Zero, IntPtr.Zero);
@@ -420,7 +489,10 @@ namespace Microsoft.Win32
 				
 				if (result == Win32ResultCode.NoMoreEntries)
 					break;
-					
+
+				if (result == Win32ResultCode.MarkedForDeletion)
+					throw RegistryKey.CreateMarkedForDeletionException ();
+
 				GenerateException (result);
 			}
 
