@@ -101,7 +101,8 @@ namespace System.IO
 				// and having di.Exists return false afterwards.
 				// I hope we don't break anyone's code, as they should be catching
 				// the exception anyway.
-				if (error != MonoIOError.ERROR_ALREADY_EXISTS)
+				if (error != MonoIOError.ERROR_ALREADY_EXISTS &&
+				    error != MonoIOError.ERROR_FILE_EXISTS)
 					throw MonoIO.GetException (path, error);
 			}
 
@@ -173,7 +174,14 @@ namespace System.IO
 			
 			exists = MonoIO.ExistsDirectory (path, out error);
 			if (error != MonoIOError.ERROR_SUCCESS &&
-			    error != MonoIOError.ERROR_PATH_NOT_FOUND) {
+			    error != MonoIOError.ERROR_PATH_NOT_FOUND &&
+			    error != MonoIOError.ERROR_INVALID_HANDLE &&
+			    error != MonoIOError.ERROR_ACCESS_DENIED) {
+
+				// INVALID_HANDLE might happen if the file is moved
+				// while testing for the existence, a kernel issue
+				// according to Larry Ewing.
+				
 				throw MonoIO.GetException (path, error);
 			}
 
@@ -234,6 +242,24 @@ namespace System.IO
 			return GetFileSystemEntries (path, pattern, FileAttributes.Directory, FileAttributes.Directory);
 		}
 		
+#if NET_2_0
+		public static string [] GetDirectories (string path, string pattern, SearchOption option)
+		{
+			if (option == SearchOption.TopDirectoryOnly)
+				return GetDirectories (path, pattern);
+			ArrayList all = new ArrayList ();
+			GetDirectoriesRecurse (path, pattern, all);
+			return (string []) all.ToArray (typeof (string));
+		}
+		
+		static void GetDirectoriesRecurse (string path, string pattern, ArrayList all)
+		{
+			all.AddRange (GetDirectories (path, pattern));
+			foreach (string dir in GetDirectories (path))
+				GetDirectoriesRecurse (dir, pattern, all);
+		}
+#endif
+
 		public static string GetDirectoryRoot (string path)
 		{
 			return new String(Path.DirectorySeparatorChar,1);
@@ -250,10 +276,20 @@ namespace System.IO
 		}
 
 #if NET_2_0
-		[MonoTODO]
 		public static string[] GetFiles (string path, string searchPattern, SearchOption searchOption)
 		{
-			throw new NotImplementedException ();
+			if (searchOption == SearchOption.TopDirectoryOnly)
+				return GetFiles (path, searchPattern);
+			ArrayList all = new ArrayList ();
+			GetFilesRecurse (path, searchPattern, all);
+			return (string []) all.ToArray (typeof (string));
+		}
+		
+		static void GetFilesRecurse (string path, string pattern, ArrayList all)
+		{
+			all.AddRange (GetFiles (path, pattern));
+			foreach (string dir in GetDirectories (path))
+				GetFilesRecurse (dir, pattern, all);
 		}
 #endif
 
@@ -298,8 +334,12 @@ namespace System.IO
 			// return null if the path is the root directory
 			if (IsRootDirectory (path))
 				return null;
-			
-			return new DirectoryInfo (Path.GetDirectoryName (path));
+
+			string parent_name = Path.GetDirectoryName (path);
+			if (parent_name == "")
+				parent_name = GetCurrentDirectory();
+
+			return new DirectoryInfo (parent_name);
 		}
 
 		public static void Move (string src, string dest)
