@@ -294,7 +294,9 @@ namespace System.Reflection.Emit {
 #else
 				if (((ilgen == null) || (ILGenerator.Mono_GetCurrentOffset (ilgen) == 0)) && (code == null))
 #endif
-					throw new InvalidOperationException ("Method '" + Name + "' does not have a method body.");
+					throw new InvalidOperationException (
+									     String.Format ("Method '{0}.{1}' does not have a method body.",
+											    DeclaringType.Name, Name));
 			}
 			if (ilgen != null)
 				ilgen.label_fixup ();
@@ -364,6 +366,13 @@ namespace System.Reflection.Emit {
 						ThrowOnUnmappableChar = (bool)value;
 #endif
 				}
+
+				attrs |= MethodAttributes.PinvokeImpl;
+				if (preserveSig)
+					iattrs |= MethodImplAttributes.PreserveSig;
+				return;
+			}
+
 #if NET_2_0
 			if (attrname == "System.Runtime.CompilerServices.SpecialNameAttribute") {
 				attrs |= MethodAttributes.SpecialName;
@@ -371,11 +380,6 @@ namespace System.Reflection.Emit {
 			}
 #endif
 
-				attrs |= MethodAttributes.PinvokeImpl;
-				if (preserveSig)
-					iattrs |= MethodImplAttributes.PreserveSig;
-				return;
-			}
 			if (cattrs != null) {
 				CustomAttributeBuilder[] new_array = new CustomAttributeBuilder [cattrs.Length + 1];
 				cattrs.CopyTo (new_array, 0);
