@@ -271,6 +271,13 @@ namespace System.Reflection {
 			if (info.ReferencedAssembly != null)
 				return info.ReferencedAssembly.GetManifestResourceStream (name);
 			if ((info.FileName != null) && (info.ResourceLocation == 0)) {
+				if (fromByteArray)
+#if NET_2_0
+					throw new FileNotFoundException (info.FileName);
+#else
+					return null;
+#endif
+
 				string filename = Path.Combine (Path.GetDirectoryName (Location),
 											info.FileName);
 				return new FileStream (filename, FileMode.Open, FileAccess.Read);
