@@ -126,6 +126,11 @@ namespace System.Reflection {
 				SecurityManager.ReflectedLinkDemandInvoke (this);
 			}
 
+#if NET_2_0
+			if (ContainsGenericParameters)
+				throw new InvalidOperationException ("Late bound operations cannot be performed on types or methods for which ContainsGenericParameters is true.");
+#endif
+
 			try {
 				return InternalInvoke (obj, parameters);
 			} catch (InvalidOperationException) {
@@ -295,6 +300,17 @@ namespace System.Reflection {
 			[MethodImplAttribute(MethodImplOptions.InternalCall)]
 			get;
 		}
+
+		public override bool ContainsGenericParameters {
+			get {
+				if (IsGenericMethod) {
+					foreach (Type arg in GetGenericArguments ())
+						if (arg.ContainsGenericParameters)
+							return true;
+				}
+				return DeclaringType.ContainsGenericParameters;
+			}
+		}
 #endif
 
 #if NET_2_0
