@@ -3,6 +3,7 @@
 // Sean MacIsaac (macisaac@ximian.com)
 // Paolo Molaro (lupus@ximian.com)
 // Dietmar Maurer (dietmar@ximian.com)
+// Jonathan Chambers (joncham@gmail.com)
 //
 // (C) 2001-2002 Ximian, Inc.
 
@@ -29,6 +30,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+using Mono.Interop;
 using System.Runtime.CompilerServices;
 using System;
 using System.Security;
@@ -37,6 +39,7 @@ using System.Threading;
 
 #if NET_2_0
 using System.Runtime.ConstrainedExecution;
+using System.Runtime.InteropServices.ComTypes;
 #endif
 
 namespace System.Runtime.InteropServices
@@ -58,10 +61,8 @@ namespace System.Runtime.InteropServices
 		private Marshal () {}
 #endif
 
-		[MonoTODO]
-		public static int AddRef (IntPtr pUnk) {
-			throw new NotImplementedException ();
-		}
+        [MethodImplAttribute (MethodImplOptions.InternalCall)]
+        public extern static int AddRef (IntPtr pUnk);
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static IntPtr AllocCoTaskMem (int cb);
@@ -153,18 +154,27 @@ namespace System.Runtime.InteropServices
 			copy_from_unmanaged (source, startIndex, destination, length);
 		}
 
-		[MonoTODO]
 		public static object CreateWrapperOfType (object o, Type t) {
-			throw new NotImplementedException ();
+			__ComObject co = o as __ComObject;
+			if (co == null)
+				throw new ArgumentException ("o must derive from __ComObject", "o");
+			if (t == null)
+				throw new ArgumentNullException ("t");
+
+			Type[] itfs = o.GetType ().GetInterfaces ();
+			foreach (Type itf in itfs) {
+				if (itf.IsImport && co.GetInterface (itf) == IntPtr.Zero)
+					throw new InvalidCastException ();
+			}
+
+			return ComInteropProxy.GetProxy (co.IUnknown, t).GetTransparentProxy ();
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static void DestroyStructure (IntPtr ptr, Type structuretype);
 
-		[MonoTODO]
-		public static void FreeBSTR (IntPtr ptr) {
-			throw new NotImplementedException ();
-		}
+        [MethodImplAttribute(MethodImplOptions.InternalCall)]
+        public extern static void FreeBSTR (IntPtr ptr);
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static void FreeCoTaskMem (IntPtr ptr);
@@ -224,7 +234,13 @@ namespace System.Runtime.InteropServices
 
 		[MonoTODO]
 		public static IntPtr GetComInterfaceForObject (object o, Type T) {
-			throw new NotImplementedException ();
+            __ComObject co = o as __ComObject;
+            if (co == null)
+			    throw new NotSupportedException ("Only RCWs are currently supported");
+
+			IntPtr pUnk = co.GetInterface (T);
+			AddRef (pUnk);
+			return pUnk;
 		}
 
 #if NET_2_0
@@ -234,14 +250,31 @@ namespace System.Runtime.InteropServices
 		}
 #endif
 
-		[MonoTODO]
 		public static object GetComObjectData (object obj, object key) {
-			throw new NotImplementedException ();
+			if (obj == null)
+				throw new ArgumentNullException ("obj");
+			if (key == null)
+				throw new ArgumentNullException ("key");
+
+			__ComObject com_object = obj as __ComObject;
+			if (com_object == null)
+				throw new ArgumentException ("obj is not a COM object", "obj");
+
+			return com_object.Hashtable[key];
 		}
 
-		[MonoTODO]
-		public static int GetComSlotForMethodInfo (MemberInfo m) {
-			throw new NotImplementedException ();
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		private extern static int GetComSlotForMethodInfoInternal (MemberInfo m);
+
+		public static int GetComSlotForMethodInfo (MemberInfo m)
+		{
+			if (m == null)
+				throw new ArgumentNullException("m", "Value cannot be null.");
+			if (!(m is MethodInfo))
+				throw new ArgumentException("m", "The MemberInfo must be an interface method.");
+			if (!m.DeclaringType.IsInterface)
+				throw new ArgumentException("m", "The MemberInfo must be an interface method.");
+			return GetComSlotForMethodInfoInternal (m);
 		}
 
 		[MonoTODO]
@@ -264,9 +297,9 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("SetErrorInfo")]
 		public static int GetHRForException (Exception e) {
-			throw new NotImplementedException ();
+			return e.hresult;
 		}
 
 		[MonoTODO]
@@ -275,7 +308,14 @@ namespace System.Runtime.InteropServices
 		}
 
 		[MonoTODO]
-		public static IntPtr GetIDispatchForObject (object o) {
+		public static IntPtr GetIDispatchForObject (object o)
+		{			// only handle case of RCW objects for now
+			__ComObject co = o as __ComObject;
+			if (co != null) {
+				IntPtr pUnk = co.IDispatch;
+				AddRef (pUnk);
+				return pUnk;
+			}
 			throw new NotImplementedException ();
 		}
 
@@ -293,6 +333,13 @@ namespace System.Runtime.InteropServices
 
 		[MonoTODO]
 		public static IntPtr GetIUnknownForObject (object o) {
+			// only handle case of RCW objects for now
+			__ComObject co = o as __ComObject;
+			if (co != null) {
+				IntPtr pUnk = co.IUnknown;
+				AddRef (pUnk);
+				return pUnk;
+			}
 			throw new NotImplementedException ();
 		}
 
@@ -319,19 +366,20 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public static void GetNativeVariantForObject (object obj, IntPtr pDstNativeVariant) {
-			throw new NotImplementedException ();
+			Variant vt = new Variant();
+			vt.SetValue(obj);
+			Marshal.StructureToPtr(vt, pDstNativeVariant, false);
 		}
 
-		[MonoTODO]
 		public static object GetObjectForIUnknown (IntPtr pUnk) {
-			throw new NotImplementedException ();
+			ComInteropProxy proxy = ComInteropProxy.GetProxy (pUnk, typeof(__ComObject));
+            return proxy.GetTransparentProxy ();
 		}
 
-		[MonoTODO]
 		public static object GetObjectForNativeVariant (IntPtr pSrcNativeVariant) {
-			throw new NotImplementedException ();
+			Variant vt = (Variant)Marshal.PtrToStructure(pSrcNativeVariant, typeof(Variant));
+			return vt.GetValue();
 		}
 
 		[MonoTODO]
@@ -349,10 +397,18 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
-		public static object GetTypedObjectForIUnknown (IntPtr pUnk, Type t) {
-			throw new NotImplementedException ();
-		}
+        public static object GetTypedObjectForIUnknown (IntPtr pUnk, Type t)
+        {
+			ComInteropProxy proxy = new ComInteropProxy (pUnk, t);
+            __ComObject co = (__ComObject)proxy.GetTransparentProxy ();
+            foreach (Type itf in t.GetInterfaces ()) {
+                if ((itf.Attributes & TypeAttributes.Import) == TypeAttributes.Import) {
+                    if (co.GetInterface (itf) == IntPtr.Zero)
+                        return null;
+                }
+            }
+            return co;
+        }
 
 		[MonoTODO]
 		public static Type GetTypeForITypeInfo (IntPtr piTypeInfo) {
@@ -375,6 +431,13 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public static Guid GetTypeLibGuid (ITypeLib typelib) {
+			throw new NotImplementedException ();
+		}
+#endif
+
 		[MonoTODO]
 		public static Guid GetTypeLibGuidForAssembly (Assembly asm) {
 			throw new NotImplementedException ();
@@ -388,6 +451,13 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public static int GetTypeLibLcid (ITypeLib typelib) {
+			throw new NotImplementedException ();
+		}
+#endif
+
 #if NET_2_0
 		[Obsolete]
 #endif
@@ -397,6 +467,12 @@ namespace System.Runtime.InteropServices
 		}
 
 #if NET_2_0
+		[MonoTODO]
+		public static string GetTypeLibName (ITypeLib typelib)
+		{
+			throw new NotImplementedException ();
+		}
+
 		[MonoTODO]
 		public static void GetTypeLibVersionForAssembly (Assembly inputAssembly, out int majorVersion, out int minorVersion) {
 			throw new NotImplementedException ();
@@ -408,9 +484,10 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public static bool IsComObject (object o) {
-			throw new NotImplementedException ();
+			Type t = o.GetType ();
+			object[] attrs = t.GetCustomAttributes (typeof (ComImportAttribute), true);
+			return (attrs != null && attrs.Length > 0);
 		}
 
 		[MonoTODO]
@@ -459,10 +536,8 @@ namespace System.Runtime.InteropServices
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static object PtrToStructure (IntPtr ptr, Type structureType);
 
-		[MonoTODO]
-		public static int QueryInterface (IntPtr pUnk, ref Guid iid, out IntPtr ppv) {
-			throw new NotImplementedException ();
-		}
+        [MethodImplAttribute (MethodImplOptions.InternalCall)]
+        public extern static int QueryInterface (IntPtr pUnk, ref Guid iid, out IntPtr ppv);
 
 		public static byte ReadByte (IntPtr ptr) {
 			return ReadByte (ptr, 0);
@@ -562,14 +637,16 @@ namespace System.Runtime.InteropServices
 #if NET_2_0
 		[ReliabilityContractAttribute (Consistency.WillNotCorruptState, Cer.Success)]
 #endif
-		[MonoTODO]
-		public static int Release (IntPtr pUnk) {
-			throw new NotImplementedException ();
-		}
+        [MethodImplAttribute (MethodImplOptions.InternalCall)]
+        public extern static int Release (IntPtr pUnk);
 
-		[MonoTODO]
 		public static int ReleaseComObject (object o) {
-			throw new NotImplementedException ();
+			if (o == null)
+				throw new ArgumentException ("o");
+			__ComObject co = o as __ComObject;
+			if (co == null)
+				throw new ArgumentException ("o");
+			return ComInteropProxy.ReleaseComObject (co);
 		}
 
 #if NET_2_0
@@ -580,9 +657,18 @@ namespace System.Runtime.InteropServices
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public static bool SetComObjectData (object obj, object key, object data) {
-			throw new NotImplementedException ();
+			if (obj == null)
+				throw new ArgumentNullException ("obj");
+			if (key == null)
+				throw new ArgumentNullException ("key");
+
+			__ComObject com_object = obj as __ComObject;
+			if (com_object == null)
+				throw new ArgumentException ("obj is not a COM object", "obj");
+
+			com_object.Hashtable[key] = data;
+			return true;
 		}
 
 		public static int SizeOf (object structure) {
@@ -592,10 +678,8 @@ namespace System.Runtime.InteropServices
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static int SizeOf (Type t);
 
-		[MonoTODO]
-		public static IntPtr StringToBSTR (string s) {
-			throw new NotImplementedException ();
-		}
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		public extern static IntPtr StringToBSTR (string s);
 
 		public static IntPtr StringToCoTaskMemAnsi (string s) {
 			int length = s.Length + 1;
@@ -684,9 +768,9 @@ namespace System.Runtime.InteropServices
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static void StructureToPtr (object structure, IntPtr ptr, bool fDeleteOld);
 
-		[MonoTODO]
 		public static void ThrowExceptionForHR (int errorCode) {
-			throw new NotImplementedException ();
+            if (errorCode < 0)
+			    throw new COMException ("", errorCode);
 		}
 
 		[MonoTODO]
