@@ -5,10 +5,7 @@
 //         Lluis Sanchez (lsg@ctv.es)
 //
 // (C) Ximian, Inc.
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004,2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,9 +27,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Collections;
-using System.Runtime.Serialization;
 
 namespace System.Runtime.Serialization
 {
@@ -81,7 +76,7 @@ namespace System.Runtime.Serialization
 		public virtual long GetId (object obj, out bool firstTime)
 		{
 			if (obj == null)
-				throw new ArgumentNullException ("The obj parameter is null.");
+				throw new ArgumentNullException ("obj");
 
 			object val = table [obj];
 
@@ -99,7 +94,7 @@ namespace System.Runtime.Serialization
 		public virtual long HasId (object obj, out bool firstTime)
 		{
 			if (obj == null)
-				throw new ArgumentNullException ("The obj parameter is null.");
+				throw new ArgumentNullException ("obj");
 
  			object val = table [obj];
  
