@@ -28,8 +28,12 @@ namespace System.Text
 {
 
 using System;
+using System.Runtime.InteropServices;
 
 [Serializable]
+#if NET_2_0
+[ComVisible (true)]
+#endif
 [MonoTODO ("Fix serialization compatibility with MS.NET")]
 public class ASCIIEncoding : Encoding
 {
@@ -45,6 +49,7 @@ public class ASCIIEncoding : Encoding
 	}
 
 #if NET_2_0
+	[ComVisible (false)]
 	public override bool IsSingleByte {
 		get { return true; }
 	}
@@ -305,14 +310,33 @@ public class ASCIIEncoding : Encoding
 			return String.Empty;
 		
 		unsafe {
-			fixed (byte *ss = &bytes [0]) {
-				return new String ((sbyte*)ss, index, count);
+			fixed (byte* bytePtr = bytes) {
+				string s = string.InternalAllocateStr (count);
+
+				fixed (char* charPtr = s) {
+					byte* currByte = bytePtr + index;
+					byte* lastByte = currByte + count;
+					char* currChar = charPtr;
+
+					while (currByte < lastByte) {
+#if NET_2_0
+						byte b = currByte++ [0];
+						currChar++ [0] = b <= 0x7F ? (char) b : (char) '?';
+#else
+						// GetString is incompatible with GetChars
+						currChar++ [0] = (char) (currByte++ [0] & 0x7F);
+#endif
+					}
+				}
+
+				return s;
 			}
 		}
 	}
 
 #if NET_2_0
 	[CLSCompliantAttribute (false)]
+	[ComVisible (false)]
 	public unsafe override int GetBytes (char *chars, int charCount, byte *bytes, int byteCount)
 	{
 		if (chars == null)
@@ -335,6 +359,7 @@ public class ASCIIEncoding : Encoding
 	}
 
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe override int GetChars (byte *bytes, int byteCount, char *chars, int charCount)
 	{
 		if (bytes == null)
@@ -357,12 +382,14 @@ public class ASCIIEncoding : Encoding
 	}
 
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe override int GetCharCount (byte *bytes, int count)
 	{
 		return count;
 	}
 
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe override int GetByteCount (char *chars, int count)
 	{
 		return count;
@@ -374,25 +401,21 @@ public class ASCIIEncoding : Encoding
 		if (bytes == null) {
 			throw new ArgumentNullException ("bytes");
 		}
-		int count = bytes.Length;
-		if (count == 0)
-		    return String.Empty;
-		unsafe {
-			fixed (byte *ss = &bytes [0]) {
-				return new String ((sbyte*)ss, 0, count);
-			}
-		}
+
+		return GetString (bytes, 0, bytes.Length);
 	}
 #endif
 
 #if NET_2_0
 	[MonoTODO ("we have simple override to match method signature.")]
+	[ComVisible (false)]
 	public override Decoder GetDecoder ()
 	{
 		return base.GetDecoder ();
 	}
 
 	[MonoTODO ("we have simple override to match method signature.")]
+	[ComVisible (false)]
 	public override Encoder GetEncoder ()
 	{
 		return base.GetEncoder ();
