@@ -35,6 +35,9 @@ using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
 
 [Serializable]
+#if NET_2_0
+[ComVisible (true)]
+#endif
 public abstract class Encoding
 #if NET_2_0
 	: ICloneable
@@ -99,6 +102,7 @@ public abstract class Encoding
 		get { return is_readonly; }
 	}
 
+	[ComVisible (false)]
 	public virtual bool IsSingleByte {
 		get { return false; }
 	}
@@ -189,7 +193,13 @@ public abstract class Encoding
 	{
 		Encoding enc = (obj as Encoding);
 		if (enc != null) {
+#if NET_2_0
+			return codePage == enc.codePage &&
+				DecoderFallback.Equals (enc.DecoderFallback) &&
+				EncoderFallback.Equals (enc.EncoderFallback);
+#else
 			return (codePage == enc.codePage);
+#endif
 		} else {
 			return false;
 		}
@@ -451,6 +461,9 @@ public abstract class Encoding
 #if NET_2_0
 			case UTF32Encoding.UTF32_CODE_PAGE:
 				return UTF32;
+
+			case UTF32Encoding.BIG_UTF32_CODE_PAGE:
+				return BigEndianUTF32;
 #endif
 
 			case UnicodeEncoding.UNICODE_CODE_PAGE:
@@ -501,7 +514,7 @@ public abstract class Encoding
 #if !ECMA_COMPAT
 
 #if NET_2_0
-
+	[ComVisible (false)]
 	public virtual object Clone ()
 	{
 		Encoding e = (Encoding) MemberwiseClone ();
@@ -539,10 +552,52 @@ public abstract class Encoding
 		return e;
 	}
 
-	[MonoTODO]
+	static EncodingInfo [] encoding_infos;
+
+	// FIXME: As everyone would agree, this implementation is so *hacky*
+	// and could be very easily broken. But since there is a test for
+	// this method to make sure that this method always returns
+	// the same number and content of encoding infos, this won't
+	// matter practically.
 	public static EncodingInfo[] GetEncodings ()
 	{
-		throw new NotImplementedException ();
+		if (encoding_infos == null) {
+			int [] codepages = new int [] {
+				37, 437, 500, 708,
+				850, 852, 855, 857, 858, 860, 861, 862, 863, 
+				864, 865, 866, 869, 870, 874, 875,
+				932, 936, 949, 950,
+				1026, 1047, 1140, 1141, 1142, 1143, 1144,
+				1145, 1146, 1147, 1148, 1149,
+				1200, 1201, 1250, 1251, 1252, 1253, 1254,
+				1255, 1256, 1257, 1258,
+				10000, 10079, 12000, 12001,
+				20127, 20273, 20277, 20278, 20280, 20284,
+				20285, 20290, 20297, 20420, 20424, 20866,
+				20871, 21025, 21866, 28591, 28592, 28593,
+				28594, 28595, 28596, 28597, 28598, 28599,
+				28605, 38598,
+				50220, 50221, 50222, 51932, 51949, 54936,
+				57002, 57003, 57004, 57005, 57006, 57007,
+				57008, 57009, 57010, 57011,
+				65000, 65001};
+
+			encoding_infos = new EncodingInfo [codepages.Length];
+			for (int i = 0; i < codepages.Length; i++)
+				encoding_infos [i] = new EncodingInfo (codepages [i]);
+		}
+		return encoding_infos;
+	}
+
+	public bool IsAlwaysNormalized ()
+	{
+		return IsAlwaysNormalized (NormalizationForm.FormC);
+	}
+
+	public virtual bool IsAlwaysNormalized (NormalizationForm form)
+	{
+		// umm, ASCIIEncoding should have overriden this method, no?
+		return form == NormalizationForm.FormC && this is ASCIIEncoding;
 	}
 #endif
 
@@ -639,7 +694,11 @@ public abstract class Encoding
 	// Get a hash code for this instance.
 	public override int GetHashCode ()
 	{
+#if NET_2_0
+		return DecoderFallback.GetHashCode () << 24 + EncoderFallback.GetHashCode () << 16 + codePage;
+#else
 		return codePage;
+#endif
 	}
 
 	// Get the maximum number of bytes needed to encode a
@@ -776,6 +835,7 @@ public abstract class Encoding
 	static volatile Encoding unixConsoleEncoding;
 #if NET_2_0
 	static volatile Encoding utf32Encoding;
+	static volatile Encoding bigEndianUTF32Encoding;
 #endif
 
 	static readonly object lockobj = new object ();
@@ -965,6 +1025,23 @@ public abstract class Encoding
 			return utf32Encoding;
 		}
 	}
+
+	// Get the standard big-endian UTF-32 encoding object.
+	private static Encoding BigEndianUTF32
+	{
+		get {
+			if (bigEndianUTF32Encoding == null) {
+				lock (lockobj) {
+					if (bigEndianUTF32Encoding == null) {
+						bigEndianUTF32Encoding = new UTF32Encoding (true, true);
+						bigEndianUTF32Encoding.is_readonly = true;
+					}
+				}
+			}
+
+			return bigEndianUTF32Encoding;
+		}
+	}
 #endif
 
 	// Forwarding decoder implementation.
@@ -1025,6 +1102,7 @@ public abstract class Encoding
 
 #if NET_2_0
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe virtual int GetByteCount (char *chars, int count)
 	{
 		if (chars == null)
@@ -1040,6 +1118,7 @@ public abstract class Encoding
 	}
 
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe virtual int GetCharCount (byte *bytes, int count)
 	{
 		if (bytes == null)
@@ -1054,6 +1133,7 @@ public abstract class Encoding
 	}
 
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe virtual int GetChars (byte *bytes, int byteCount, char *chars, int charCount)
 	{
 		if (bytes == null)
@@ -1080,6 +1160,7 @@ public abstract class Encoding
 	}
 
 	[CLSCompliantAttribute(false)]
+	[ComVisible (false)]
 	public unsafe virtual int GetBytes (char *chars, int charCount, byte *bytes, int byteCount)
 	{
 		if (bytes == null)
