@@ -38,10 +38,14 @@
 //
 using System.Runtime.Serialization;
 using System.Runtime.CompilerServices;
+using System.Runtime.InteropServices;
 
 namespace System.Text {
 	
 	[Serializable]
+#if NET_2_0
+	[ComVisible (true)]
+#endif
 	[MonoTODO ("Fix serialization compatibility with MS.NET")]
 	public sealed class StringBuilder
 #if NET_2_0
@@ -157,13 +161,8 @@ namespace System.Text {
 					_length = value;
 				} else {
 					// Expand the capacity to the new length and
-					// pad the string with spaces.
-					
-					// LAMESPEC: The spec says to put the spaces on the
-					// left of the string however the MS implementation
-					// puts them on the right.  We'll do that for 
-					// compatibility (!)
-					Append(' ', value - _length);
+					// pad the string with NULL characters.
+					Append('\0', value - _length);
 				}
 			}
 		}
@@ -476,11 +475,13 @@ namespace System.Text {
 		}
 
 #if NET_2_0
+		[ComVisible (false)]
 		public StringBuilder AppendLine ()
 		{
 			return Append (System.Environment.NewLine);
 		}
 
+		[ComVisible (false)]
 		public StringBuilder AppendLine (string value)
 		{
 			return Append (value).Append (System.Environment.NewLine);
@@ -683,6 +684,7 @@ namespace System.Text {
 		}
 
 #if NET_2_0
+		[ComVisible (false)]
 		public void CopyTo (int sourceIndex, char [] destination, int destinationIndex, int count)
 		{
 			if (destination == null)
