@@ -49,16 +49,19 @@ namespace System.Threading {
 #if NET_2_0
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_Thread))]
-#endif
+	public sealed class Thread : CriticalFinalizerObject, _Thread {
+#else
 	public sealed class Thread : _Thread {
+#endif
 
 		#region Sync with metadata/object-internals.h
 		int lock_thread_id;
 		// stores a thread handle
 		private IntPtr system_thread_handle;
-		
-		private IntPtr culture_info;
-		private IntPtr ui_culture_info;
+
+		/* Note this is an opaque object (an array), not a CultureInfo */
+		private object cached_culture_info;
+		private IntPtr unused0;
 		private bool threadpool_thread;
 		/* accessed only from unmanaged code */
 		private IntPtr name;
