@@ -1439,15 +1439,20 @@ namespace System
 		{
 			if (array == null)
 				throw new ArgumentNullException ("array");
+			Sort<T> (array, array.Length, comparison);
+		}
+
+		internal static void Sort<T> (T [] array, int length, Comparison<T> comparison)
+		{
 			if (comparison == null)
 				throw new ArgumentNullException ("comparison");
 
-			if (array.Length <= 1)
+			if (length <= 1 || array.Length <= 1)
 				return;
 			
 			try {
 				int low0 = 0;
-				int high0 = array.Length - 1;
+				int high0 = length - 1;
 				qsort<T> (array, low0, high0, comparison);
 			}
 			catch (Exception e) {
@@ -1849,7 +1854,7 @@ namespace System
 			if (array == null)
 				throw new ArgumentNullException ("array");
 	
-			return IndexOf (array, value, 0, array.Length);
+			return IndexOf<T> (array, value, 0, array.Length);
 		}
 
 		public static int IndexOf<T> (T [] array, T value, int startIndex)
@@ -1857,7 +1862,7 @@ namespace System
 			if (array == null)
 				throw new ArgumentNullException ("array");
 
-			return IndexOf (array, value, startIndex, array.Length - startIndex);
+			return IndexOf<T> (array, value, startIndex, array.Length - startIndex);
 		}
 
 		public static int IndexOf<T> (T [] array, T value, int startIndex, int count)
@@ -1870,8 +1875,9 @@ namespace System
 				throw new ArgumentOutOfRangeException ();
 
 			int max = startIndex + count;
+			EqualityComparer<T> equalityComparer = EqualityComparer<T>.Default;
 			for (int i = startIndex; i < max; i++) {
-				if (Object.Equals (value, array [i]))
+				if (equalityComparer.Equals (value, array [i]))
 					return i;
 			}
 
@@ -1883,7 +1889,7 @@ namespace System
 			if (array == null)
 				throw new ArgumentNullException ("array");
 
-			return LastIndexOf (array, value, array.Length - 1);
+			return LastIndexOf<T> (array, value, array.Length - 1);
 		}
 
 		public static int LastIndexOf<T> (T [] array, T value, int startIndex)
@@ -1891,7 +1897,7 @@ namespace System
 			if (array == null)
 				throw new ArgumentNullException ("array");
 
-			return LastIndexOf (array, value, startIndex, startIndex + 1);
+			return LastIndexOf<T> (array, value, startIndex, startIndex + 1);
 		}
 
 		public static int LastIndexOf<T> (T [] array, T value, int startIndex, int count)
@@ -1902,8 +1908,9 @@ namespace System
 			if (count < 0 || startIndex > array.Length || startIndex - count + 1 < 0)
 				throw new ArgumentOutOfRangeException ();
 
+			EqualityComparer<T> equalityComparer = EqualityComparer<T>.Default;
 			for (int i = startIndex; i >= startIndex - count + 1; i--) {
-				if (Object.Equals (value, array [i]))
+				if (equalityComparer.Equals (value, array [i]))
 					return i;
 			}
 
