@@ -98,6 +98,8 @@ namespace System
 			"yyyy-MM-ddTHH:mm:sszzz",
 			"yyyy-MM-ddTHH:mm:ss.fffffff",
 			"yyyy-MM-ddTHH:mm:ss.fffffffzzz",
+			// bug #78618
+			"yyyy-M-d H:m:s.fffffff",
 			// UTC / allow any separator
 			"yyyy/MM/ddTHH:mm:ssZ",
 			"yyyy/M/dZ",
@@ -443,7 +445,11 @@ namespace System
 		public static DateTime UtcNow 
 		{
 			get {
+#if NET_2_0
+				return new DateTime (GetNow (), DateTimeKind.Utc);
+#else
 				return new DateTime (GetNow ());
+#endif
 			}
 		}
 
@@ -1374,8 +1380,8 @@ namespace System
 				if (tzoffset == -1)
 					tzoffset = 0;
 				if (tzsign == 1)
-					tzoffset = -tzoffset;
-
+					tzoffset = -tzoffset
+;
 				utcoffset = new TimeSpan (tzoffset, tzoffmin, 0);
 			}
 
@@ -1417,7 +1423,55 @@ namespace System
 				throw new FormatException ();
 			return result;
 		}
+
+#if NET_2_0
+		public static bool TryParse (string s, out DateTime result)
+		{
+			try {
+				result = Parse (s);
+			} catch {
+				result = MinValue;
+				return false;
+			}
+			return true;
+		}
 		
+		public static bool TryParse (string s, IFormatProvider provider, DateTimeStyles styles, out DateTime result)
+		{
+			try {
+				result = Parse (s, provider, styles);
+			} catch {
+				result = MinValue;
+				return false;
+			}
+			return true;
+		}
+		
+		public static bool TryParseExact (string s, string format,
+						  IFormatProvider fp,
+						  DateTimeStyles style,
+						  out DateTime result)
+		{
+			string[] formats;
+
+			formats = new string [1];
+			formats[0] = format;
+
+			return TryParseExact (s, formats, fp, style, out result);
+		}
+
+		public static bool TryParseExact (string s, string[] formats,
+						  IFormatProvider fp,
+						  DateTimeStyles style,
+						  out DateTime result)
+		{
+			DateTimeFormatInfo dfi = DateTimeFormatInfo.GetInstance (fp);
+
+			bool longYear = false;
+			return ParseExact (s, formats, dfi, style, out result, true, ref longYear);
+		}
+#endif
+
 		private static bool ParseExact (string s, string [] formats,
 			DateTimeFormatInfo dfi, DateTimeStyles style, out DateTime ret,
 			bool exact, ref bool longYear)
