@@ -380,9 +380,12 @@ namespace System
 
 		public override string ToString ()
 		{
-			return ToString ("G", null);
+			return ToString ("G");
 		}
 
+#if NET_2_0
+	[Obsolete("Provider is ignored, just use ToString")]
+#endif
 		public string ToString (IFormatProvider provider)
 		{
 			return ToString ("G", provider);
@@ -390,9 +393,15 @@ namespace System
 
 		public string ToString (String format)
 		{
-			return ToString (format, null);
+			if (format == String.Empty || format == null)
+				format = "G";
+			
+			return Format (this.GetType (), this.get_value (), format);
 		}
 
+#if NET_2_0
+		[Obsolete("Provider is ignored, just use ToString")]
+#endif
 		public string ToString (String format, IFormatProvider provider)
 		{
 			// provider is not used for Enums
@@ -505,7 +514,7 @@ namespace System
 
 		static string FormatFlags (Type enumType, object value)
 		{
-			string retVal = "";
+			string retVal = String.Empty;
 			MonoEnumInfo info;
 			MonoEnumInfo.GetInfo (enumType, out info);
 			string asString = value.ToString ();
@@ -527,7 +536,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -543,7 +552,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -559,7 +568,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -575,7 +584,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -591,7 +600,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -607,7 +616,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -623,7 +632,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -639,7 +648,7 @@ namespace System
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
-						retVal = info.names[i] + (retVal == String.Empty ? "" : ", ") + retVal;
+						retVal = info.names[i] + (retVal == String.Empty ? String.Empty : ", ") + retVal;
 						flags -= enumValue;
 					}
 				}
@@ -648,7 +657,7 @@ namespace System
 				break;
 			}
 
-			if (retVal == "")
+			if (retVal == String.Empty)
 				return asString;
 
 			return retVal;
@@ -668,7 +677,9 @@ namespace System
 			
 			Type vType = value.GetType();
 			if (vType != enumType && vType != Enum.GetUnderlyingType (enumType))
-				throw new ArgumentException ();
+				throw new ArgumentException (string.Format(CultureInfo.InvariantCulture,
+					"Object must be the same type as the enum. The type passed in " +
+					"was {0}; the enum type was {1}.", vType.FullName, enumType.FullName));
 
 			if (format.Length != 1)
 				throw new FormatException ("Format String can be only \"G\",\"g\",\"X\"," + 
@@ -691,7 +702,7 @@ namespace System
 			if ((formatChar == 'f' || formatChar == 'F'))
 				return FormatFlags (enumType, value);
 
-			retVal = "";
+			retVal = String.Empty;
 			switch (formatChar) {
 			case 'X':
 			case 'x':
