@@ -185,7 +185,6 @@ namespace System
 			MethodInfo[] methods = GetMethodsByName (name, bindingAttr, ignoreCase, this);
 			MethodInfo found = null;
 			MethodBase[] match;
-			int typesLen = (types != null) ? types.Length : 0;
 			int count = 0;
 			
 			foreach (MethodInfo m in methods) {
@@ -199,7 +198,7 @@ namespace System
 			if (count == 0)
 				return null;
 			
-			if (count == 1 && typesLen == 0) 
+			if (count == 1 && types == null) 
 				return (MethodInfo) CheckMethodSecurity (found);
 
 			match = new MethodBase [count];
@@ -213,7 +212,7 @@ namespace System
 					match [count++] = m;
 				}
 			}
-			
+
 			if (types == null) 
 				return (MethodInfo) CheckMethodSecurity (Binder.FindMostDerivedMatch (match));
 
@@ -291,10 +290,8 @@ namespace System
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		protected extern override bool IsByRefImpl ();
 
-		protected override bool IsCOMObjectImpl ()
-		{
-			return false;
-		}
+		[MethodImplAttribute (MethodImplOptions.InternalCall)]
+		protected extern override bool IsCOMObjectImpl ();
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		protected extern override bool IsPointerImpl ();
@@ -328,8 +325,6 @@ namespace System
 				throw new ArgumentException ("invokeAttr");
 			if ((invokeAttr & BindingFlags.InvokeMethod) != 0 && (invokeAttr & (BindingFlags.SetProperty|BindingFlags.SetField)) != 0)
 				throw new ArgumentException ("invokeAttr");
-			if ((invokeAttr & BindingFlags.SetField) != 0 && ((args == null) || args.Length != 1))
-				throw new ArgumentException ("invokeAttr");
 			if ((namedParameters != null) && ((args == null) || args.Length < namedParameters.Length))
 				throw new ArgumentException ("namedParameters cannot be more than named arguments in number");
 
@@ -389,6 +384,9 @@ namespace System
 				}
 				/* try GetProperty */
 			} else if ((invokeAttr & BindingFlags.SetField) != 0) {
+				if ((args == null) || args.Length != 1)
+					throw new ArgumentException ("invokeAttr");
+
 				FieldInfo f = GetField (name, invokeAttr);
 				if (f != null) {
 					f.SetValue (target, args [0]);
