@@ -39,6 +39,7 @@ using System.Runtime.CompilerServices;
 using System.Collections.Generic;
 using System.Runtime.ConstrainedExecution;
 using System.Runtime.InteropServices;
+using Mono.Globalization.Unicode;
 #endif
 
 namespace System
@@ -951,6 +952,44 @@ namespace System
 			return (value == null) || (value.Length == 0);
 		}
 
+		public string Normalize ()
+		{
+			return Normalize (NormalizationForm.FormC);
+		}
+
+		public string Normalize (NormalizationForm form)
+		{
+			switch (form) {
+			default:
+				return Normalization.Normalize (this, 0);
+			case NormalizationForm.FormD:
+				return Normalization.Normalize (this, 1);
+			case NormalizationForm.FormKC:
+				return Normalization.Normalize (this, 2);
+			case NormalizationForm.FormKD:
+				return Normalization.Normalize (this, 3);
+			}
+		}
+
+		public bool IsNormalized ()
+		{
+			return IsNormalized (NormalizationForm.FormC);
+		}
+
+		public bool IsNormalized (NormalizationForm form)
+		{
+			switch (form) {
+			default:
+				return Normalization.IsNormalized (this, 0);
+			case NormalizationForm.FormD:
+				return Normalization.IsNormalized (this, 1);
+			case NormalizationForm.FormKC:
+				return Normalization.IsNormalized (this, 2);
+			case NormalizationForm.FormKD:
+				return Normalization.IsNormalized (this, 3);
+			}
+		}
+
 		public string Remove (int startIndex)
 		{
 			if (startIndex < 0)
@@ -1049,6 +1088,9 @@ namespace System
 #endif
 		bool StartsWith (String value, bool ignoreCase, CultureInfo culture)
 		{
+			if (culture == null)
+				culture = CultureInfo.CurrentCulture;
+			
 			return (culture.CompareInfo.IsPrefix (this, value,
 				ignoreCase ? CompareOptions.IgnoreCase :
 				CompareOptions.None));
@@ -1251,7 +1293,7 @@ namespace System
 						formatter = provider.GetFormat (typeof (ICustomFormatter))
 							as ICustomFormatter;
 					if (arg == null)
-						str = "";
+						str = String.Empty;
 					else if (formatter != null)
 						str = formatter.Format (arg_format, arg, provider);
 					else if (arg is IFormattable)
@@ -1681,11 +1723,6 @@ namespace System
 			return Convert.ToSingle (this, provider);
 		}
 
-		string IConvertible.ToString (IFormatProvider format)
-		{
-			return this;
-		}
-
 		object IConvertible.ToType (Type conversionType, IFormatProvider provider)
 		{
 			return Convert.ToType (this, conversionType,  provider);
@@ -1706,11 +1743,6 @@ namespace System
 			return Convert.ToUInt64 (this, provider);
 		}
 
-		TypeCode IConvertible.GetTypeCode ()
-		{
-			return TypeCode.String;
-		}
-
 		public int Length {
 			get {
 				return length;
@@ -1771,7 +1803,7 @@ namespace System
 				else {
 					width = 0;
 					left_align = false;
-					format = "";
+					format = String.Empty;
 				}
 
 				// F = argument format (string)
@@ -1830,12 +1862,18 @@ namespace System
 			if (newLength > length)
 				throw new ArgumentOutOfRangeException ("newLength", "newLength as to be <= length");
 
-			length = newLength;
-
 			// zero terminate, we can pass string objects directly via pinvoke
+			// we also zero the rest of the string, since the new GC needs to be
+			// able to handle the changing size (it will skip the 0 bytes).
 			fixed (char * pStr = &start_char) {
-				pStr [length] = '\0';
+				char *p = pStr + newLength;
+				char *end = pStr + length;
+				while (p < end) {
+					p [0] = '\0';
+					p++;
+				}
 			}
+			length = newLength;
 		}
 
 #if NET_2_0
@@ -1858,6 +1896,86 @@ namespace System
 			}
 		}
 
+		// Certain constructors are redirected to CreateString methods with
+		// matching argument list. The this pointer should not be used.
+
+		private unsafe String CreateString (sbyte* value)
+		{
+			if (value == null)
+				return String.Empty;
+
+			byte* bytes = (byte*) value;
+			int length = 0;
+
+			try {
+				while (bytes++ [0] != 0)
+					length++;
+			} catch (NullReferenceException) {
+				throw new ArgumentOutOfRangeException ("value", "Value does not refer to a valid string.");
+#if NET_2_0
+			} catch (AccessViolationException) {
+				throw new ArgumentOutOfRangeException ("value", "Value does not refer to a valid string.");
+#endif
+			}
+
+			return CreateString (value, 0, length, null);
+		}
+
+		private unsafe String CreateString (sbyte* value, int startIndex, int length)
+		{
+			return CreateString (value, startIndex, length, null);
+		}
+
+		private unsafe String CreateString (sbyte* value, int startIndex, int length, Encoding enc)
+		{
+			if (length < 0)
+				throw new ArgumentOutOfRangeException ("length", "Non-negative number required.");
+			if (startIndex < 0)
+				throw new ArgumentOutOfRangeException ("startIndex", "Non-negative number required.");
+			if (value + startIndex < value)
+				throw new ArgumentOutOfRangeException ("startIndex", "Value, startIndex and length do not refer to a valid string.");
+
+			bool isDefaultEncoding;
+
+			if (isDefaultEncoding = (enc == null)) {
+#if NET_2_0
+				if (value == null)
+					throw new ArgumentNullException ("value");
+				if (length == 0)
+#else
+				if (value == null || length == 0)
+#endif
+					return String.Empty;
+
+				enc = Encoding.Default;
+			}
+
+			byte [] bytes = new byte [length];
+
+			if (length != 0)
+				fixed (byte* bytePtr = bytes)
+					try {
+						memcpy (bytePtr, (byte*) (value + startIndex), length);
+					} catch (NullReferenceException) {
+#if !NET_2_0
+						if (!isDefaultEncoding)
+							throw;
+#endif
+
+						throw new ArgumentOutOfRangeException ("value", "Value, startIndex and length do not refer to a valid string.");
+#if NET_2_0
+					} catch (AccessViolationException) {
+						if (!isDefaultEncoding)
+							throw;
+
+						throw new ArgumentOutOfRangeException ("value", "Value, startIndex and length do not refer to a valid string.");
+#endif
+					}
+
+			// GetString () is called even when length == 0
+			return enc.GetString (bytes);
+		}
+
 		/* helpers used by the runtime as well as above or eslewhere in corlib */
 		internal static unsafe void memset (byte *dest, int val, int len)
 		{
