@@ -15,6 +15,7 @@ using System.Text;
 using System;
 using System.Globalization;
 using System.Threading;
+using System.Diagnostics;
 
 namespace MonoTests.System.IO {
 
@@ -100,6 +101,43 @@ public class DirectoryTest : Assertion {
 		}
 	}
 
+	[Test]
+	public void CreateDirectoryAlreadyExists ()
+	{
+		string path = TempFolder + DSC + "DirectoryTest.Test.Exists";
+		DeleteDirectory (path);
+		try {
+			DirectoryInfo info1 = Directory.CreateDirectory (path);
+			DirectoryInfo info2 = Directory.CreateDirectory (path);
+
+			AssertEquals ("test#01", true, info2.Exists);
+			AssertEquals ("test#02", true, info2.FullName.EndsWith ("DirectoryTest.Test.Exists"));
+			AssertEquals ("test#03", "DirectoryTest.Test.Exists", info2.Name);
+		} finally {
+			DeleteDirectory (path);		
+		}
+	}
+
+	[Test]
+	public void CreateDirectoryAlreadyExistsAsFile ()
+	{
+		string path = TempFolder + DSC + "DirectoryTest.Test.ExistsAsFile";
+		DeleteDirectory (path);
+		DeleteFile (path);
+		try {
+			FileStream fstream = File.Create (path);
+			fstream.Close();
+
+			DirectoryInfo dinfo = Directory.CreateDirectory (path);
+			AssertEquals ("test#01", false, dinfo.Exists);
+			AssertEquals ("test#02", true, dinfo.FullName.EndsWith ("DirectoryTest.Test.ExistsAsFile"));
+			AssertEquals ("test#03", "DirectoryTest.Test.ExistsAsFile", dinfo.Name);
+		} finally {
+			DeleteDirectory (path);		
+			DeleteFile (path);
+		}
+	}
+
 	[Test]
 	public void Delete ()
 	{
@@ -198,6 +236,28 @@ public class DirectoryTest : Assertion {
 	{
 		AssertEquals ("test#01", false, Directory.Exists (null as string));
 	}
+
+	[Test]
+	[Category("NotDotNet")]
+	public void ExistsAccessDenied ()
+	{
+		// bug #78239
+
+		if (Path.DirectorySeparatorChar == '\\')
+			return; // this test does not work on Windows.
+
+		string path = TempFolder + DSC + "ExistsAccessDenied";
+		Process p;
+
+		Directory.CreateDirectory (path);
+		Process.Start ("/bin/chmod", "000 " + path).WaitForExit ();
+		try {
+			AssertEquals ("#1", false, Directory.Exists(path + DSC + "b"));
+		} finally {
+			Process.Start ("/bin/chmod", "755 " + path).WaitForExit ();
+			Directory.Delete (path);
+		}
+	}
 	
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]	
@@ -582,6 +642,7 @@ public class DirectoryTest : Assertion {
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Ignore ("On IA64, causes nunit to abort due to bug #76388")]
 	public void MoveException4 ()
 	{
 		string path = TempFolder + DSC + "DirectoryTest.Test.13";
@@ -1256,10 +1317,23 @@ public class DirectoryTest : Assertion {
 		dir.GetFiles ("*.nonext");
 	}
 
+
+	[Test]
+	public void FilenameOnly () // bug 78209
+	{
+		Directory.GetParent ("somefile");
+	}
+
 	private void DeleteDirectory (string path)
 	{
 		if (Directory.Exists (path))
 			Directory.Delete (path, true);		
 	}
+
+	private void DeleteFile (string path)
+	{
+		if (File.Exists (path))
+			File.Delete (path);
+	}
 }
 }
