@@ -537,6 +537,17 @@ public class AssemblyBuilderTest
 		m = ab2.GetModules ();
 	}
 
+	[Test] // bug #78724
+	public void GetTypes ()
+	{
+		TypeBuilder tb = mb.DefineType ("sometype");
+		tb.CreateType ();
+
+		Type[] types = ab.GetTypes ();
+		Assert.AreEqual (1, types.Length);
+		Assert.AreEqual ("sometype", types[0].Name);
+	}
+
 	[Test]
 	[Category ("NotWorking")]
 	public void AssemblyName_Culture ()
@@ -654,9 +665,11 @@ public class AssemblyBuilderTest
 	}
 
 	[Test]
+	[Category ("NotDotNet")]
 	public void GetType_IgnoreCase ()
 	{
 		TypeBuilder tb = mb.DefineType ("Foo.Test2", TypeAttributes.Public, typeof (object));
+		// the previous line throws a TypeLoadException under MS 1.1 SP1
 
 		Type t;
 
