@@ -1073,14 +1073,46 @@ public class DateTimeTest : Assertion
 	public void X509Certificate () 
 	{
 		// if this test fails then *ALL* or *MOST* X509Certificate tests will also fails
-		AssertEquals ("yyyyMMddHHmmssZ", "03/12/1996 18:38:47", DateTime.ParseExact ("19960312183847Z", "yyyyMMddHHmmssZ", null).ToUniversalTime ().ToString ());
-		// technically this is invalid (PKIX) because of the missing seconds but it exists so...
-		AssertEquals ("yyMMddHHmmZ", "02/23/1996 19:15:00", DateTime.ParseExact ("9602231915Z", "yyMMddHHmmZ", null).ToUniversalTime ().ToString ());
+		DateTime dt = DateTime.ParseExact ("19960312183847Z", "yyyyMMddHHmmssZ", null);
+#if NET_2_0
+		AssertEquals ("Kind-Local-1", DateTimeKind.Local, dt.Kind);
+		dt = dt.ToUniversalTime ();
+		AssertEquals ("Kind-Utc-1", DateTimeKind.Utc, dt.Kind);
+#else
+		dt = dt.ToUniversalTime ();
+#endif
+		AssertEquals ("yyyyMMddHHmmssZ", "03/12/1996 18:38:47", dt.ToString ());
 
+		// technically this is invalid (PKIX) because of the missing seconds but it exists so...
+		dt = DateTime.ParseExact ("9602231915Z", "yyMMddHHmmZ", null);
+#if NET_2_0
+		AssertEquals ("Kind-Local-2", DateTimeKind.Local, dt.Kind);
+		dt = dt.ToUniversalTime ();
+		AssertEquals ("Kind-Utc-2", DateTimeKind.Utc, dt.Kind);
+#else
+		dt = dt.ToUniversalTime ();
+#endif
+		AssertEquals ("yyMMddHHmmZ", "02/23/1996 19:15:00", dt.ToString ());
+	}
+
+	[Test]
+#if NET_2_0
+	[Category ("NotWorking")]	// see bug #
+#endif
+	public void ZLiteral ()
+	{
 		// However, "Z" and "'Z'" are different.
-		AssertEquals ("Z timezone handling",
-			DateTime.ParseExact ("19960312183847Z", "yyyyMMddHHmmss'Z'", null).ToLocalTime (),
-			DateTime.ParseExact ("19960312183847Z", "yyyyMMddHHmmssZ", null));
+		DateTime dt = DateTime.ParseExact ("19960312183847Z", "yyyyMMddHHmmss'Z'", null);
+		DateTime dtz = DateTime.ParseExact ("19960312183847Z", "yyyyMMddHHmmssZ", null);
+#if NET_2_0
+		AssertEquals ("Kind-Unspecified-3", DateTimeKind.Unspecified, dt.Kind);
+		dt = dt.ToLocalTime ();
+		AssertEquals ("Kind-Local-3a", DateTimeKind.Local, dt.Kind);
+		AssertEquals ("Kind-Local-3b", DateTimeKind.Local, dtz.Kind);
+#else
+		dt = dt.ToLocalTime ();
+#endif
+		AssertEquals ("Z timezone handling", dt, dtz);
 	}
 
 	[Test]
@@ -1463,6 +1495,39 @@ public class DateTimeTest : Assertion
 	{
 		((IConvertible)DateTime.Now).ToType (typeof (ulong), null);
 	}
+
+#if NET_2_0
+	[Test]
+	public void Kind ()
+	{
+		if (DateTime.Now == DateTime.UtcNow)
+			return; // This test does not make sense.
+		if (TimeZone.CurrentTimeZone.GetUtcOffset (DateTime.UtcNow)
+		    != TimeZone.CurrentTimeZone.GetUtcOffset (DateTime.Now))
+			return; // In this case it does not satisfy the test premises.
+
+		AssertEquals ("#0", DateTimeKind.Local, DateTime.Now.Kind);
+
+		DateTime utc = DateTime.UtcNow;
+		DateTime now = new DateTime (utc.Ticks + TimeZone.
+			CurrentTimeZone.GetUtcOffset (utc).Ticks, DateTimeKind.Local);
+		DateTime utctouniv = utc.ToUniversalTime ();
+		DateTime nowtouniv = now.ToUniversalTime ();
+		DateTime utctoloc = utc.ToLocalTime ();
+		DateTime nowtoloc = now.ToLocalTime ();
+
+		AssertEquals ("#1", DateTimeKind.Utc, utc.Kind);
+		AssertEquals ("#2", DateTimeKind.Local, now.Kind);
+		AssertEquals ("#3", DateTimeKind.Utc, utctouniv.Kind);
+		AssertEquals ("#4", DateTimeKind.Utc, nowtouniv.Kind);
+		AssertEquals ("#5", DateTimeKind.Local, utctoloc.Kind);
+		AssertEquals ("#6", DateTimeKind.Local, nowtoloc.Kind);
+		AssertEquals ("#7", utc, utctouniv);
+		AssertEquals ("#8", utc, nowtouniv);
+		AssertEquals ("#9", now, nowtoloc);
+		AssertEquals ("#10", now, utctoloc);
+	}
+#endif
 }
 
 }
