@@ -1,9 +1,13 @@
 // StringTest.cs - NUnit Test Cases for the System.String class
 //
-// Jeffrey Stedfast <fejj@ximian.com>
-// David Brandt <bucky@keystreams.com>
+// Authors:
+//   Jeffrey Stedfast <fejj@ximian.com>
+//   David Brandt <bucky@keystreams.com>
+//   Kornl Pl <http://www.kornelpal.hu/>
 //
 // (C) Ximian, Inc.  http://www.ximian.com
+// Copyright (C) 2006 Kornl Pl
+// Copyright (C) 2006 Novell (http://www.novell.com)
 //
 
 using NUnit.Framework;
@@ -18,121 +22,285 @@ namespace MonoTests.System
 public class StringTest : Assertion
 {
 	[Test]
-	[ExpectedException (typeof (ArgumentNullException))]
-	public void CtrExceptions ()
+	public unsafe void CharArrayConstructor ()
 	{
-		String s = new String ((char[])null, 0, 0);
+		AssertEquals ("char[]", String.Empty, new String ((char[]) null));
+		AssertEquals (String.Empty, new String (new Char [0]));
+		AssertEquals ("A", new String (new Char [1] {'A'}));
 	}
 
-	public void TestConstructors ()
+	[Test]
+	public void CharArrayIntIntConstructor ()
 	{
-		AssertEquals ("", new String ((char[])null));
-		AssertEquals ("", new String (new Char [0]));
-		AssertEquals ("A", new String (new Char [1] {'A'}));
-
-		AssertEquals ("", new String ('A', 0));
-		AssertEquals ("AAA", new String ('A', 3));
-		try {
-			new String ('A', -1);
-			Fail ("Should reject negative count");
-		}
-		catch (ArgumentOutOfRangeException) {
-		}
-
 		char[] arr = new char [3] { 'A', 'B', 'C' };
 		AssertEquals ("BC", new String (arr, 1, 2));
-		try {
-			new String ((char[])null, 1, 0);
-			Fail ();
-		}
-		catch (ArgumentNullException) {
-		}
-		try {
-			new String (arr, -1, 1);
-			Fail ();
-		}
-		catch (ArgumentOutOfRangeException) {
-		}
-		try {
-			new String (arr, 0, -1);
-			Fail ();
-		}
-		catch (ArgumentOutOfRangeException) {
-		}
-		try {
-			new String (arr, 1, 3);
-			Fail ();
-		}
-		catch (ArgumentOutOfRangeException) {
+	}
+
+	[Test]
+	[ExpectedException (typeof (ArgumentNullException))]
+	public void CharArray_Null_IntIntConstructor ()
+	{
+		new String ((char[])null, 0, 0);
+	}
+
+	[Test]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public void CharArrayInt_Negative_IntConstructor ()
+	{
+		char[] arr = new char [3] { 'A', 'B', 'C' };
+		new String (arr, -1, 1);
+	}
+
+	[Test]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public void CharArrayIntInt_Negative_Constructor ()
+	{
+		char[] arr = new char [3] { 'A', 'B', 'C' };
+		new String (arr, 0, -1);
+	}
+
+	[Test]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public void CharArrayIntInt_TooLong_Constructor ()
+	{
+		char[] arr = new char [3] { 'A', 'B', 'C' };
+		new String (arr, 1, 3);
+	}
+
+	[Test]
+	public void CharIntConstructor ()
+	{
+		AssertEquals ("", new String ('A', 0));
+		AssertEquals ("AAA", new String ('A', 3));
+	}
+
+	[Test]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public void CharInt_NegativeConstructor ()
+	{
+		new String ('A', -1);
+	}
+
+	[Test]
+	public unsafe void CharPtrConstructor ()
+	{
+		AssertEquals ("char*", String.Empty, new String ((char*) null));
+		AssertEquals ("char*,int,int", String.Empty, new String ((char*) null, 0, 0));
+	}
+
+	public unsafe void TestSbytePtrConstructorASCII ()
+	{
+		Encoding encoding = Encoding.ASCII;
+		String s = "ASCII*\0";
+		byte[] bytes = encoding.GetBytes (s);
+
+		fixed (byte* bytePtr = bytes)
+			AssertEquals (s, new String ((sbyte*) bytePtr, 0, bytes.Length, encoding));
+	}
+
+	public unsafe void TestSbytePtrConstructorDefault ()
+	{
+		Encoding encoding = Encoding.Default;
+		byte [] bytes = new byte [256];
+		
+		for (int i = 0; i < 255; i++)
+			bytes [i] = (byte) (i + 1);
+		bytes [255] = (byte) 0;
+
+		// Ensure that bytes are valid for Encoding.Default
+		bytes = encoding.GetBytes (encoding.GetChars (bytes));
+		String s = encoding.GetString(bytes);
+
+		// Ensure null terminated array
+		bytes [bytes.Length - 1] = (byte) 0;
+
+		fixed (byte* bytePtr = bytes) 
+		{
+			AssertEquals (s.Substring (0, s.Length - 1), new String ((sbyte*) bytePtr));
+			AssertEquals (s, new String ((sbyte*) bytePtr, 0, bytes.Length));
+			AssertEquals (s, new String ((sbyte*) bytePtr, 0, bytes.Length, null));
+			AssertEquals (s, new String ((sbyte*) bytePtr, 0, bytes.Length, encoding));
 		}
 	}
-	
-	[Category("NotDotNet")]
-	public void TestUnsafeConstructors ()
+
+	public unsafe void TestSbytePtrConstructorNull1 ()
 	{
-		unsafe {
-			AssertEquals (String.Empty, new String ((sbyte*)null, 0, 10, Encoding.ASCII));
-		}
+		AssertEquals (String.Empty, new String ((sbyte*) null));
+	}
 
-		unsafe {
-			sbyte[] s1 = new sbyte [10];
-			fixed (sbyte* s2 = &s1[0]) {
-				AssertEquals (String.Empty, new String (s2, 0, 0, Encoding.ASCII));
-			}
-		}
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentNullException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull2 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, 0));
+	}
 
-		unsafe {
-			sbyte[] s1 = new sbyte [10];
-			fixed (sbyte* s2 = &s1[0]) {
-				try {
-					new String (s2, 0, 10, null);
-					Fail ();
-				}
-				catch (ArgumentNullException) {
-				}
-			}
-		}
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentNullException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull3 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, 1));
+	}
 
-		unsafe {
-			sbyte[] s1 = new sbyte [10];
-			fixed (sbyte* s2 = &s1[0]) {
-				try {
-					new String (s2, -1, 10, Encoding.ASCII);
-					Fail ();
-				}
-				catch (ArgumentOutOfRangeException) {
-				}
-			}
-		}
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentNullException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull4 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 1, 0));
+	}
 
-		unsafe {
-			sbyte[] s1 = new sbyte [10];
-			fixed (sbyte* s2 = &s1[0]) {
-				try {
-					new String (s2, 0, -1, Encoding.ASCII);
-					Fail ();
-				}
-				catch (ArgumentOutOfRangeException) {
-				}
-			}
-		}
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentNullException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull5 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, 0, null));
+	}
 
-		unsafe {    
-			String s = "Hello, World!";
-			byte[] bytes = Encoding.ASCII.GetBytes (s);
-			sbyte[] s1 = new sbyte [bytes.Length];
-			for (int i = 0; i < s1.Length; ++i)
-				s1 [i] = (sbyte)bytes [i];
-			fixed (sbyte* s2 = &s1[0]) {
-				string res = new String(s2, 0, s1.Length, Encoding.ASCII);
-				AssertEquals (s, res);
-			}    
-		}    
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentNullException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull6 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, 1, null));
+	}
 
-		unsafe {
-			AssertEquals (String.Empty, new String ((sbyte*)null));
-			AssertEquals (String.Empty, new String ((char*)null, 0, 0));
-		}
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentNullException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull7 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 1, 0, null));
+	}
+
+	public unsafe void TestSbytePtrConstructorNull8 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, 0, Encoding.Default));
+	}
+
+#if NET_2_0
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+#else
+	[ExpectedException (typeof (NullReferenceException))]
+#endif
+	public unsafe void TestSbytePtrConstructorNull9 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, 1, Encoding.Default));
+	}
+
+	public unsafe void TestSbytePtrConstructorNull10 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 1, 0, Encoding.Default));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorInvalid1 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1)));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorInvalid2 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 0, 1));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorInvalid3 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 0, 1, null));
+	}
+
+#if NET_2_0
+	[Ignore ("Runtime throws NullReferenceException instead of AccessViolationException")]
+	[ExpectedException (typeof (AccessViolationException))]
+#else
+	[ExpectedException (typeof (NullReferenceException))]
+#endif
+	public unsafe void TestSbytePtrConstructorInvalid4 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 0, 1, Encoding.Default));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorNegative1 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, -1, 0));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorNegative2 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, -1));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorNegative3 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, -1, 0, null));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorNegative4 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, -1, null));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorNegative5 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, -1, 0, Encoding.Default));
+	}
+
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorNegative6 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) null, 0, -1, Encoding.Default));
+	}
+
+	[Ignore ("Blocked by mcs bug 78899")]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorOverflow1 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 1, 0));
+	}
+
+	[Ignore ("Blocked by mcs bug 78899")]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorOverflow2 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 1, 1));
+	}
+
+	[Ignore ("Blocked by mcs bug 78899")]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorOverflow3 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 1, 0, null));
+	}
+
+	[Ignore ("Blocked by mcs bug 78899")]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorOverflow4 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 1, 1, null));
+	}
+
+	[Ignore ("Blocked by mcs bug 78899")]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorOverflow5 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 1, 0, Encoding.Default));
+	}
+
+	[Ignore ("Blocked by mcs bug 78899")]
+	[ExpectedException (typeof (ArgumentOutOfRangeException))]
+	public unsafe void TestSbytePtrConstructorOverflow6 ()
+	{
+		AssertEquals (String.Empty, new String ((sbyte*) (-1), 1, 1, Encoding.Default));
 	}
 
 	public void TestLength ()
@@ -1628,6 +1796,15 @@ public class StringTest : Assertion
 		"A B".Split (new String [] { "A" }, 0, (StringSplitOptions)4096);
 	}
 
+	[Test]
+	public void StartsWith_WithEmptyCulture ()
+	{
+		// This should not crash
+		string s = "boo";
+
+		s.StartsWith ("this", true, null);
+	}
+	
 	[Test]
 	public void SplitString ()
 	{
