@@ -9,6 +9,7 @@
 
 using NUnit.Framework;
 using System;
+using System.Collections;
 using System.IO;
 using System.Reflection;
 using System.Runtime.InteropServices;
@@ -527,6 +528,13 @@ PublicKeyToken=b77a5c561934e089"));
 			typeof (BindingFlags).GetConstructor (BindingFlags.Default, null, CallingConventions.Any, new Type[1] { null }, null);
 		}
 
+		[Test]
+		public void GetMethod_Bug77367 ()
+		{
+			MethodInfo i = typeof (Bug77367).GetMethod ("Run", Type.EmptyTypes);
+			Assert.IsNull (i);
+		}
+
 #if NET_2_0
 		[Test]
 		public void FullNameGenerics ()
@@ -620,6 +628,21 @@ PublicKeyToken=b77a5c561934e089"));
 			Assert.IsNull (byref_type_param.DeclaringType);
 		}
 
+		[Test]
+		public void Bug79023 ()
+		{
+			ArrayList list = new ArrayList();
+			list.Add("foo");
+
+			// The next line used to throw because we had SetProperty
+			list.GetType().InvokeMember("Item",
+						    BindingFlags.SetField|BindingFlags.SetProperty|
+						    BindingFlags.Instance|BindingFlags.Public,
+						    null, list, new object[] { 0, "bar" });
+			if (!object.Equals("bar", list[0]))
+				throw new ApplicationException();
+		}
+		
 		[ComVisible (true)]
 		public class ComFoo<T> {
 		}
@@ -642,5 +665,13 @@ PublicKeyToken=b77a5c561934e089"));
 
 		struct FooStruct {
 		}
+
+		public class Bug77367
+		{
+			public void Run (bool b)
+			{
+			}
+		}
+
 	}
 }
