@@ -1,4 +1,4 @@
-// cs0110.cs: The evaluation of the constant value for 'A.B.C.X' involves a circular definition
+// cs0110.cs: The evaluation of the constant value for `A.B.C.X' involves a circular definition
 // Line: 9
 
 class A {
@@ -16,7 +16,11 @@ class A {
 			}
 
 			enum F {
-				A, B, C, D = X, E
+			    A, 
+			    B,
+			    C,
+			    D = X,
+			    E
 			}
 
 			const int X = Y + 1;
