@@ -280,8 +280,8 @@ namespace Mono.CSharp {
 			if (expr_type.IsArray && target_type == TypeManager.array_type)
 				return new EmptyCast (expr, target_type);
 
-			// from an array-type of type T to IEnumerable<T>
-			if (expr_type.IsArray && TypeManager.IsIEnumerable (expr_type, target_type))
+			// from an array-type of type T to IList<T>
+			if (expr_type.IsArray && TypeManager.IsIList (expr_type, target_type))
 				return new EmptyCast (expr, target_type);
 
 			// from any delegate type to System.Delegate
@@ -376,8 +376,8 @@ namespace Mono.CSharp {
 			if (expr_type.IsArray && (target_type == TypeManager.array_type))
 				return true;
 
-			// from an array-type of type T to IEnumerable<T>
-			if (expr_type.IsArray && TypeManager.IsIEnumerable (expr_type, target_type))
+			// from an array-type of type T to IList<T>
+			if (expr_type.IsArray && TypeManager.IsIList (expr_type, target_type))
 				return true;
 
 			// from any delegate type to System.Delegate
@@ -629,6 +629,19 @@ namespace Mono.CSharp {
 		{
 			Type expr_type = expr.Type;
 
+			if (TypeManager.IsNullableType (expr_type)) {
+				Type nullable = TypeManager.GetTypeArguments (expr_type) [0];
+				Expression source = new EmptyExpression (nullable);
+
+				if (TypeManager.IsNullableType (target_type)) {
+					if (ImplicitStandardConversionExists (source, target_type))
+						return true;
+					if (ImplicitStandardConversionExists (expr, target_type))
+						return true;
+				}
+			} else if (TypeManager.IsNullableTypeOf (target_type, expr_type))
+				return true;
+
 			if (expr_type == TypeManager.void_type)
 				return false;
 
@@ -752,7 +765,7 @@ namespace Mono.CSharp {
 					}
 				}
 			}
-			
+
 			if (ImplicitReferenceConversionExists (expr, target_type))
 				return true;
 
@@ -821,9 +834,6 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.void_ptr_type && expr_type.IsPointer)
 				return true;
 
-			if (TypeManager.IsNullableType (expr_type) && TypeManager.IsNullableType (target_type))
-				return true;
-
 			if (expr_type == TypeManager.anonymous_method_type){
 				if (!TypeManager.IsDelegateType (target_type))
 					return false;
@@ -1146,10 +1156,6 @@ namespace Mono.CSharp {
 			Type source_type = source.Type;
 			MethodInfo method = null;
 
-			if (TypeManager.IsNullableType (source_type) && TypeManager.IsNullableType (target))
-				return new Nullable.LiftedConversion (
-					source, target, true, look_for_explicit, loc).Resolve (ec);
-
 			object o;
 			DoubleHash hash = look_for_explicit ? explicit_conv : implicit_conv;
 
@@ -1163,7 +1169,7 @@ namespace Mono.CSharp {
 
 			if (method == null)
 				return null;
-			
+
 			Type most_specific_source = TypeManager.GetParameterData (method).ParameterType (0);
 
 			//
@@ -1241,9 +1247,20 @@ namespace Mono.CSharp {
 					return new Nullable.NullableLiteral (target_type, loc);
 			}
 
-			if (TypeManager.IsNullableType (expr_type) && TypeManager.IsNullableType (target_type))
-				return new Nullable.LiftedConversion (
-					expr, target_type, false, false, loc).Resolve (ec);
+			if (TypeManager.IsNullableType (target_type)) {
+				Type target = TypeManager.GetTypeArguments (target_type) [0];
+
+				if (TypeManager.IsNullableType (expr.Type)) {
+					e = new Nullable.LiftedConversion (
+						expr, target_type, false, false, loc).Resolve (ec);
+					if (e != null)
+						return e;
+				} else {
+					e = ImplicitConversion (ec, expr, target, loc);
+					if (e != null)
+						return Nullable.Wrap.Create (e, ec);
+				}
+			}
 
 			if (expr.eclass == ExprClass.MethodGroup){
 				if (!TypeManager.IsDelegateType (target_type)){
@@ -1866,10 +1883,6 @@ namespace Mono.CSharp {
 				return ExplicitConversionCore (ec, new EmptyCast (expr, TypeManager.EnumToUnderlying (expr_type)), target_type, loc);
 			}
 
-			if (TypeManager.IsNullableType (expr_type) && TypeManager.IsNullableType (target_type))
-				return new Nullable.LiftedConversion (
-					expr, target_type, false, true, loc).Resolve (ec);
-
 			if (TypeManager.IsEnumType (target_type)){
 				Expression ce = ExplicitConversionCore (ec, expr, TypeManager.EnumToUnderlying (target_type), loc);
 				if (ce != null)
@@ -1961,10 +1974,6 @@ namespace Mono.CSharp {
 			if (ne != null)
 				return ne;
 
-			if (TypeManager.IsNullableType (expr.Type) && TypeManager.IsNullableType (target_type))
-				return new Nullable.LiftedConversion (
-					expr, target_type, false, true, l).Resolve (ec);
-
 			ne = ExplicitNumericConversion (expr, target_type);
 			if (ne != null)
 				return ne;
@@ -1987,7 +1996,33 @@ namespace Mono.CSharp {
 		static public Expression ExplicitConversion (EmitContext ec, Expression expr,
 			Type target_type, Location loc)
 		{
-			Expression e = ExplicitConversionCore (ec, expr, target_type, loc);
+			Expression e;
+			Type expr_type = expr.Type;
+			if (TypeManager.IsNullableType (target_type)) {
+				if (TypeManager.IsNullableType (expr_type)) {
+					e = new Nullable.LiftedConversion (
+						expr, target_type, false, true, loc).Resolve (ec);
+					if (e != null)
+						return e;
+				} else if (expr_type == TypeManager.object_type) {
+					return new UnboxCast (expr, target_type);
+				} else {
+					Type target = TypeManager.GetTypeArguments (target_type) [0];
+
+					e = ExplicitConversionCore (ec, expr, target, loc);
+					if (e != null)
+						return Nullable.Wrap.Create (e, ec);
+				}
+			} else if (TypeManager.IsNullableType (expr_type)) {
+				Expression source = Nullable.Unwrap.Create (expr, ec);
+				if (source != null) {
+					e = ExplicitConversionCore (ec, source, target_type, loc);
+					if (e != null)
+						return e;
+				}
+			}
+
+			e = ExplicitConversionCore (ec, expr, target_type, loc);
 			if (e != null)
 				return e;
 
