@@ -397,9 +397,9 @@ namespace Mono.CSharp {
 		public virtual string GetSignatureForError ()
 		{
 			if (Parent == null || Parent.Parent == null)
-				return Name;
+				return member_name.ToString ();
 
-			return String.Concat (Parent.GetSignatureForError (), '.', Name);
+			return String.Concat (Parent.GetSignatureForError (), '.', member_name.ToString ());
 		}
 
 		/// <summary>
@@ -608,17 +608,17 @@ namespace Mono.CSharp {
 		}
 
 		public override IResolveContext ResolveContext {
-			get {
-				return this;
-			}
+			get { return this; }
 		}
 
 		#region IResolveContext Members
 
-		public virtual DeclSpace DeclContainer {
-			get {
-				return Parent;
-			}
+		public DeclSpace DeclContainer {
+			get { return Parent; }
+		}
+
+		public virtual DeclSpace GenericDeclContainer {
+			get { return DeclContainer; }
 		}
 
 		public bool IsInObsoleteScope {
@@ -676,6 +676,8 @@ namespace Mono.CSharp {
 		
 		protected Hashtable defined_names;
 
+		public TypeContainer PartialContainer;
+
 		readonly bool is_generic;
 		readonly int count_type_params;
 		readonly int count_current_type_params;
@@ -703,6 +705,7 @@ namespace Mono.CSharp {
 			NamespaceEntry = ns;
 			Basename = name.Basename;
 			defined_names = new Hashtable ();
+			PartialContainer = null;
 			if (name.TypeArguments != null) {
 				is_generic = true;
 				count_type_params = count_current_type_params = name.TypeArguments.Count;
@@ -711,10 +714,8 @@ namespace Mono.CSharp {
 				count_type_params += parent.count_type_params;
 		}
 
-		public override DeclSpace DeclContainer {
-			get {
-				return this;
-			}
+		public override DeclSpace GenericDeclContainer {
+			get { return this; }
 		}
 
 		/// <summary>
@@ -1026,15 +1027,6 @@ namespace Mono.CSharp {
 			return ~ (~ mAccess | pAccess) == 0;
 		}
 
-		//
-		// Return the nested type with name @name.  Ensures that the nested type
-		// is defined if necessary.  Do _not_ use this when you have a MemberCache handy.
-		//
-		public virtual Type FindNestedType (string name)
-		{
-			return null;
-		}
-
 		private Type LookupNestedTypeInHierarchy (string name)
 		{
 			// if the member cache has been created, lets use it.
@@ -1049,10 +1041,11 @@ namespace Mono.CSharp {
 			     current_type = current_type.BaseType) {
 				current_type = TypeManager.DropGenericTypeArguments (current_type);
 				if (current_type is TypeBuilder) {
-					DeclSpace decl = this;
-					if (current_type != TypeBuilder)
-						decl = TypeManager.LookupDeclSpace (current_type);
-					t = decl.FindNestedType (name);
+					TypeContainer tc = current_type == TypeBuilder
+						? PartialContainer
+						: TypeManager.LookupTypeContainer (current_type);
+					if (tc != null)
+						t = tc.FindNestedType (name);
 				} else {
 					t = TypeManager.GetNestedType (current_type, name);
 				}
@@ -1080,7 +1073,7 @@ namespace Mono.CSharp {
 			Type t = LookupNestedTypeInHierarchy (name);
 			if (t != null)
 				e = new TypeExpression (t, Location.Null);
-			else if (Parent != null && Parent != RootContext.Tree.Types)
+			else if (Parent != null)
 				e = Parent.LookupType (name, loc, ignore_cs0104);
 			else
 				e = NamespaceEntry.LookupNamespaceOrType (this, name, loc, ignore_cs0104);
@@ -1265,7 +1258,7 @@ namespace Mono.CSharp {
 
 			TypeParameter [] current_params;
 			if (this is TypeContainer)
-				current_params = ((TypeContainer) this).PartialContainer.CurrentTypeParameters;
+				current_params = PartialContainer.CurrentTypeParameters;
 			else
 				current_params = CurrentTypeParameters;
 
@@ -1944,6 +1937,7 @@ namespace Mono.CSharp {
 
 			IMemberContainer current = Container;
 
+			bool do_interface_search = current.IsInterface;
 
 			// `applicable' is a list of all members with the given member name `name'
 			// in the current class and all its base classes.  The list is sorted in
@@ -1959,7 +1953,10 @@ namespace Mono.CSharp {
 				// iteration of this loop if there are no members with the name we're
 				// looking for in the current class).
 				if (entry.Container != current) {
-					if (declared_only || DoneSearching (global))
+					if (declared_only)
+						break;
+
+					if (!do_interface_search && DoneSearching (global))
 						break;
 
 					current = entry.Container;
@@ -1975,8 +1972,28 @@ namespace Mono.CSharp {
 
 				// Apply the filter to it.
 				if (filter (entry.Member, criteria)) {
-					if ((entry.EntryType & EntryType.MaskType) != EntryType.Method)
+					if ((entry.EntryType & EntryType.MaskType) != EntryType.Method) {
 						do_method_search = false;
+					}
+					
+					// Because interfaces support multiple inheritance we have to be sure that
+					// base member is from same interface, so only top level member will be returned
+					if (do_interface_search && global.Count > 0) {
+						bool member_already_exists = false;
+
+						foreach (MemberInfo mi in global) {
+							if (mi is MethodBase)
+								continue;
+
+							if (IsInterfaceBaseInterface (TypeManager.GetInterfaces (mi.DeclaringType), entry.Member.DeclaringType)) {
+								member_already_exists = true;
+								break;
+							}
+						}
+						if (member_already_exists)
+							continue;
+					}
+
 					global.Add (entry.Member);
 				}
 			}
@@ -1998,6 +2015,22 @@ namespace Mono.CSharp {
 			global.CopyTo (copy);
 			return copy;
 		}
+
+		/// <summary>
+		/// Returns true if iterface exists in any base interfaces (ifaces)
+		/// </summary>
+		static bool IsInterfaceBaseInterface (Type[] ifaces, Type ifaceToFind)
+		{
+			foreach (Type iface in ifaces) {
+				if (iface == ifaceToFind)
+					return true;
+
+				Type[] base_ifaces = TypeManager.GetInterfaces (iface);
+				if (base_ifaces.Length > 0 && IsInterfaceBaseInterface (base_ifaces, ifaceToFind))
+					return true;
+			}
+			return false;
+		}
 		
 		// find the nested type @name in @this.
 		public Type FindNestedType (string name)
