@@ -1536,12 +1536,8 @@ namespace Mono.CSharp
 			if (tokenize)
 				return true;
 			
-			//
-			// This will point to the NamespaceEntry of the last file that was parsed, and may
-			// not be meaningful when resolving classes from other files.  So, reset it to prevent
-			// silent bugs.
-			//
-			RootContext.Tree.Types.NamespaceEntry = null;
+			if (RootContext.ToplevelTypes.NamespaceEntry != null)
+ 				throw new InternalErrorException ("who set it?");
 
 			//
 			// If we are an exe, require a source file for the entry point
@@ -1609,10 +1605,13 @@ namespace Mono.CSharp
 				int pos = first_source.LastIndexOf ('.');
 
 				if (pos > 0)
-					output_file = first_source.Substring (0, pos) + RootContext.TargetExt;
+					output_file = first_source.Substring (0, pos);
 				else
-					output_file = first_source + RootContext.TargetExt;
+					output_file = first_source;
 			}
+			
+			if (!Path.HasExtension (output_file))
+				output_file += RootContext.TargetExt;
 
 			if (!CodeGen.Init (output_file, output_file, want_debugging_support))
 				return false;
@@ -1741,7 +1740,7 @@ namespace Mono.CSharp
 
 				if (ep == null) {
 					if (RootContext.MainClass != null) {
-						DeclSpace main_cont = RootContext.Tree.Types.GetDefinition (RootContext.MainClass) as DeclSpace;
+						DeclSpace main_cont = RootContext.ToplevelTypes.GetDefinition (RootContext.MainClass) as DeclSpace;
 						if (main_cont == null) {
 							Report.Error (1555, "Could not find `{0}' specified for Main method", RootContext.MainClass); 
 							return false;
@@ -1898,7 +1897,7 @@ namespace Mono.CSharp
 
 			public void Emit ()
 			{
-				CodeGen.Assembly.Builder.AddResourceFile (name, file, attribute);
+				CodeGen.Assembly.Builder.AddResourceFile (name, Path.GetFileName(file), attribute);
 			}
 
 			public string FileName {
