@@ -504,6 +504,12 @@ namespace Mono.CSharp {
 			return false;
 		}
 
+		public Location Location {
+			get {
+				return loc;
+			}
+		}
+
 		/// <summary>
 		///   This is used when we're implementing a generic interface method.
 		///   Each method type parameter in implementing method must have the same
@@ -568,41 +574,23 @@ namespace Mono.CSharp {
 		}
 
 		public GenericConstraints GenericConstraints {
-			get {
-				return gc != null ? gc : constraints;
-			}
+			get { return gc != null ? gc : constraints; }
 		}
 
 		public Constraints Constraints {
-			get {
-				return constraints;
-			}
+			get { return constraints; }
 		}
 
 		public bool HasConstructorConstraint {
-			get {
-				if (constraints != null)
-					return constraints.HasConstructorConstraint;
-
-				return false;
-			}
+			get { return constraints != null && constraints.HasConstructorConstraint; }
 		}
 
 		public DeclSpace DeclSpace {
-			get {
-				return decl;
-			}
+			get { return decl; }
 		}
 
 		public Type Type {
-			get {
-				return type;
-			}
-		}
-
-		// FIXME: This should be removed once we fix the handling of RootContext.Tree.Types
-		public override DeclSpace DeclContainer {
-			get { return DeclSpace; }
+			get { return type; }
 		}
 
 		/// <summary>
@@ -990,8 +978,14 @@ namespace Mono.CSharp {
 				if (t.IsGenericParameter)
 					return dargs [t.GenericParameterPosition];
 				if (t.IsGenericType) {
+					Type[] args = t.GetGenericArguments ();
+					Type[] inflated = new Type [args.Length];
+
+					for (int i = 0; i < args.Length; i++)
+						inflated [i] = inflate (args [i]);
+
 					t = t.GetGenericTypeDefinition ();
-					t = t.MakeGenericType (dargs);
+					t = t.MakeGenericType (inflated);
 				}
 
 				return t;
@@ -1067,7 +1061,7 @@ namespace Mono.CSharp {
 
 		public void Error_CannotUseAsUnmanagedType (Location loc)
 		{
-			Report.Error (-203, loc, "Can not use type parameter as unamanged type");
+			Report.Error (-203, loc, "Can not use type parameter as unmanaged type");
 		}
 	}
 
@@ -1208,7 +1202,7 @@ namespace Mono.CSharp {
 
 				if (te.Type.IsPointer) {
 					Report.Error (306, Location, "The type `{0}' may not be used " +
-						      "as a type argument.", TypeManager.CSharpName (te.Type));
+						      "as a type argument", TypeManager.CSharpName (te.Type));
 					return false;
 				} else if (te.Type == TypeManager.void_type) {
 					Report.Error (1547, Location,
@@ -1642,18 +1636,22 @@ namespace Mono.CSharp {
 				}
 			}
 
-			MethodGroupExpr mg = Expression.MemberLookup (
-				containerType, atype, ".ctor", MemberTypes.Constructor,
-				BindingFlags.Public | BindingFlags.Instance |
-				BindingFlags.DeclaredOnly, loc)
-				as MethodGroupExpr;
+			TypeParameter tparam = TypeManager.LookupTypeParameter (atype);
+			if (tparam != null)
+				return tparam.HasConstructorConstraint;
 
-			if (!atype.IsAbstract && (mg != null) && mg.IsInstance) {
-				foreach (MethodBase mb in mg.Methods) {
-					ParameterData pd = TypeManager.GetParameterData (mb);
-					if (pd.Count == 0)
-						return true;
-				}
+			MemberList list = TypeManager.FindMembers (
+				atype, MemberTypes.Constructor,
+				BindingFlags.Public | BindingFlags.Instance |
+				BindingFlags.DeclaredOnly, null, null);
+
+			if (atype.IsAbstract || (list == null))
+				return false;
+
+			foreach (MethodBase mb in list) {
+				ParameterData pd = TypeManager.GetParameterData (mb);
+				if ((pd.Count == 0) && mb.IsPublic && !mb.IsStatic)
+					return true;
 			}
 
 			return false;
@@ -1932,6 +1930,8 @@ namespace Mono.CSharp {
 		// A list of core types that the compiler requires or uses
 		//
 		static public Type activator_type;
+		static public Type generic_ilist_type;
+		static public Type generic_icollection_type;
 		static public Type generic_ienumerator_type;
 		static public Type generic_ienumerable_type;
 		static public Type generic_nullable_type;
@@ -1960,6 +1960,10 @@ namespace Mono.CSharp {
 		{
 			activator_type = CoreLookupType ("System", "Activator");
 
+			generic_ilist_type = CoreLookupType (
+				"System.Collections.Generic", "IList", 1);
+			generic_icollection_type = CoreLookupType (
+				"System.Collections.Generic", "ICollection", 1);
 			generic_ienumerator_type = CoreLookupType (
 				"System.Collections.Generic", "IEnumerator", 1);
 			generic_ienumerable_type = CoreLookupType (
@@ -2100,18 +2104,20 @@ namespace Mono.CSharp {
 		}
 
 		//
-		// Whether `array' is an array of T and `enumerator' is `IEnumerable<T>'.
-		// For instance "string[]" -> "IEnumerable<string>".
+		// Whether `array' is an array of T and `list' is `IList<T>'.
+		// For instance "string[]" -> "IList<string>".
 		//
-		public static bool IsIEnumerable (Type array, Type enumerator)
+		public static bool IsIList (Type array, Type list)
 		{
-			if (!array.IsArray || !enumerator.IsGenericType)
+			if (!array.IsArray || !list.IsGenericType)
 				return false;
 
-			if (enumerator.GetGenericTypeDefinition () != generic_ienumerable_type)
+			Type gt = list.GetGenericTypeDefinition ();
+			if ((gt != generic_ilist_type) && (gt != generic_icollection_type) &&
+			    (gt != generic_ienumerable_type))
 				return false;
 
-			Type[] args = GetTypeArguments (enumerator);
+			Type[] args = GetTypeArguments (list);
 			return args [0] == GetElementType (array);
 		}
 
@@ -2389,9 +2395,13 @@ namespace Mono.CSharp {
 					return InferType (pt.GetElementType (), at.GetElementType (), infered);
 				}
 
-				if (!pt.IsGenericType ||
-				    (pt.GetGenericTypeDefinition () != generic_ienumerable_type))
-				    return false;
+				if (!pt.IsGenericType)
+					return false;
+
+				Type gt = pt.GetGenericTypeDefinition ();
+				if ((gt != generic_ilist_type) && (gt != generic_icollection_type) &&
+				    (gt != generic_ienumerable_type))
+					return false;
 
 				Type[] args = GetTypeArguments (pt);
 				return InferType (args [0], at.GetElementType (), infered);
@@ -2472,31 +2482,25 @@ namespace Mono.CSharp {
 		public static bool InferParamsTypeArguments (EmitContext ec, ArrayList arguments,
 							     ref MethodBase method)
 		{
-			if ((arguments == null) || !TypeManager.IsGenericMethod (method))
+			if (!TypeManager.IsGenericMethod (method))
 				return true;
 
-			int arg_count;
-			
-			if (arguments == null)
-				arg_count = 0;
-			else
-				arg_count = arguments.Count;
-			
-			ParameterData pd = TypeManager.GetParameterData (method);
+			// if there are no arguments, there's no way to infer the type-arguments
+			if (arguments == null || arguments.Count == 0)
+				return false;
 
+			ParameterData pd = TypeManager.GetParameterData (method);
 			int pd_count = pd.Count;
+			int arg_count = arguments.Count;
 
 			if (pd_count == 0)
 				return false;
-			
+
 			if (pd.ParameterModifier (pd_count - 1) != Parameter.Modifier.PARAMS)
 				return false;
-			
+
 			if (pd_count - 1 > arg_count)
 				return false;
-			
-			if (pd_count == 1 && arg_count == 0)
-				return true;
 
 			Type[] method_args = method.GetGenericArguments ();
 			Type[] infered_types = new Type [method_args.Length];
@@ -2655,6 +2659,14 @@ namespace Mono.CSharp {
 			return generic_nullable_type == DropGenericTypeArguments (t);
 		}
 
+		public static bool IsNullableTypeOf (Type t, Type nullable)
+		{
+			if (!IsNullableType (t))
+				return false;
+
+			return GetTypeArguments (t) [0] == nullable;
+		}
+
 		public static bool IsNullableValueType (Type t)
 		{
 			if (!IsNullableType (t))
@@ -2666,7 +2678,7 @@ namespace Mono.CSharp {
 
 	public abstract class Nullable
 	{
-		protected sealed class NullableInfo
+		public sealed class NullableInfo
 		{
 			public readonly Type Type;
 			public readonly Type UnderlyingType;
@@ -2688,7 +2700,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		protected class Unwrap : Expression, IMemoryLocation, IAssignMethod
+		public class Unwrap : Expression, IMemoryLocation, IAssignMethod
 		{
 			Expression expr;
 			NullableInfo info;
@@ -2696,10 +2708,15 @@ namespace Mono.CSharp {
 			LocalTemporary temp;
 			bool has_temp;
 
-			public Unwrap (Expression expr, Location loc)
+			protected Unwrap (Expression expr)
 			{
 				this.expr = expr;
-				this.loc = loc;
+				this.loc = expr.Location;
+			}
+
+			public static Unwrap Create (Expression expr, EmitContext ec)
+			{
+				return new Unwrap (expr).Resolve (ec) as Unwrap;
 			}
 
 			public override Expression DoResolve (EmitContext ec)
@@ -2799,15 +2816,20 @@ namespace Mono.CSharp {
 			}
 		}
 
-		protected class Wrap : Expression
+		public class Wrap : Expression
 		{
 			Expression expr;
 			NullableInfo info;
 
-			public Wrap (Expression expr, Location loc)
+			protected Wrap (Expression expr)
 			{
 				this.expr = expr;
-				this.loc = loc;
+				this.loc = expr.Location;
+			}
+
+			public static Wrap Create (Expression expr, EmitContext ec)
+			{
+				return new Wrap (expr).Resolve (ec) as Wrap;
 			}
 
 			public override Expression DoResolve (EmitContext ec)
@@ -2884,7 +2906,7 @@ namespace Mono.CSharp {
 				if (expr == null)
 					return null;
 
-				unwrap = (Unwrap) new Unwrap (expr, loc).Resolve (ec);
+				unwrap = Unwrap.Create (expr, ec);
 				if (unwrap == null)
 					return null;
 
@@ -2892,7 +2914,7 @@ namespace Mono.CSharp {
 				if (underlying == null)
 					return null;
 
-				wrap = new Wrap (underlying, loc).Resolve (ec);
+				wrap = Wrap.Create (underlying, ec);
 				if (wrap == null)
 					return null;
 
@@ -3016,15 +3038,13 @@ namespace Mono.CSharp {
 			public override Expression DoResolve (EmitContext ec)
 			{
 				if (TypeManager.IsNullableType (left.Type)) {
-					left_unwrap = new Unwrap (left, loc);
-					left = left_unwrap.Resolve (ec);
+					left = left_unwrap = Unwrap.Create (left, ec);
 					if (left == null)
 						return null;
 				}
 
 				if (TypeManager.IsNullableType (right.Type)) {
-					right_unwrap = new Unwrap (right, loc);
-					right = right_unwrap.Resolve (ec);
+					right = right_unwrap = Unwrap.Create (right, ec);
 					if (right == null)
 						return null;
 				}
@@ -3041,7 +3061,7 @@ namespace Mono.CSharp {
 				if (((Oper == Binary.Operator.BitwiseAnd) || (Oper == Binary.Operator.BitwiseOr)) &&
 				    ((left.Type == TypeManager.bool_type) && (right.Type == TypeManager.bool_type))) {
 					Expression empty = new EmptyExpression (TypeManager.bool_type);
-					bool_wrap = new Wrap (empty, loc).Resolve (ec);
+					bool_wrap = Wrap.Create (empty, ec);
 					null_value = new NullableLiteral (bool_wrap.Type, loc).Resolve (ec);
 
 					type = bool_wrap.Type;
@@ -3070,7 +3090,7 @@ namespace Mono.CSharp {
 					if (underlying == null)
 						return null;
 
-					underlying = new Wrap (underlying, loc).Resolve (ec);
+					underlying = Wrap.Create (underlying, ec);
 					if (underlying == null)
 						return null;
 
@@ -3150,95 +3170,70 @@ namespace Mono.CSharp {
 			{
 				ILGenerator ig = ec.ig;
 
-				Label left_not_null_label = ig.DefineLabel ();
-				Label false_label = ig.DefineLabel ();
-				Label true_label = ig.DefineLabel ();
+				// Given 'X? x;' for any value type X: 'x != null' is the same as 'x.HasValue'
+				if (left is NullLiteral) {
+					if (right_unwrap == null)
+						throw new InternalErrorException ();
+					right_unwrap.EmitCheck (ec);
+					if (Oper == Binary.Operator.Equality) {
+						ig.Emit (OpCodes.Ldc_I4_0);
+						ig.Emit (OpCodes.Ceq);
+					}
+					return;
+				}
+
+				if (right is NullLiteral) {
+					if (left_unwrap == null)
+						throw new InternalErrorException ();
+					left_unwrap.EmitCheck (ec);
+					if (Oper == Binary.Operator.Equality) {
+						ig.Emit (OpCodes.Ldc_I4_0);
+						ig.Emit (OpCodes.Ceq);
+					}
+					return;
+				}
+
+				Label both_have_value_label = ig.DefineLabel ();
 				Label end_label = ig.DefineLabel ();
 
-				bool false_label_used = false;
-				bool true_label_used = false;
+				if (left_unwrap != null && right_unwrap != null) {
+					Label dissimilar_label = ig.DefineLabel ();
 
-				if (left_unwrap != null) {
 					left_unwrap.EmitCheck (ec);
-					if (right is NullLiteral) {
-						if (Oper == Binary.Operator.Equality) {
-							true_label_used = true;
-							ig.Emit (OpCodes.Brfalse, true_label);
-						} else {
-							false_label_used = true;
-							ig.Emit (OpCodes.Brfalse, false_label);
-						}
-					} else if (right_unwrap != null) {
-						ig.Emit (OpCodes.Dup);
-						ig.Emit (OpCodes.Brtrue, left_not_null_label);
-						right_unwrap.EmitCheck (ec);
-						ig.Emit (OpCodes.Ceq);
-						if (Oper == Binary.Operator.Inequality) {
-							ig.Emit (OpCodes.Ldc_I4_0);
-							ig.Emit (OpCodes.Ceq);
-						}
-						ig.Emit (OpCodes.Br, end_label);
-
-						ig.MarkLabel (left_not_null_label);
-						ig.Emit (OpCodes.Pop);
-					} else {
-						if (Oper == Binary.Operator.Equality) {
-							false_label_used = true;
-							ig.Emit (OpCodes.Brfalse, false_label);
-						} else {
-							true_label_used = true;
-							ig.Emit (OpCodes.Brfalse, true_label);
-						}
-					}
-				}
-
-				if (right_unwrap != null) {
+					ig.Emit (OpCodes.Dup);
 					right_unwrap.EmitCheck (ec);
-					if (left is NullLiteral) {
-						if (Oper == Binary.Operator.Equality) {
-							true_label_used = true;
-							ig.Emit (OpCodes.Brfalse, true_label);
-						} else {
-							false_label_used = true;
-							ig.Emit (OpCodes.Brfalse, false_label);
-						}
-					} else {
-						if (Oper == Binary.Operator.Equality) {
-							false_label_used = true;
-							ig.Emit (OpCodes.Brfalse, false_label);
-						} else {
-							true_label_used = true;
-							ig.Emit (OpCodes.Brfalse, true_label);
-						}
-					}
-				}
+					ig.Emit (OpCodes.Bne_Un, dissimilar_label);
 
-				bool left_is_null = left is NullLiteral;
-				bool right_is_null = right is NullLiteral;
-				if (left_is_null || right_is_null) {
-					if (((Oper == Binary.Operator.Equality) && (left_is_null == right_is_null)) ||
-					    ((Oper == Binary.Operator.Inequality) && (left_is_null != right_is_null))) {
-						true_label_used = true;
-						ig.Emit (OpCodes.Br, true_label);
-					} else {
-						false_label_used = true;
-						ig.Emit (OpCodes.Br, false_label);
-					}
-				} else {
-					underlying.Emit (ec);
+					ig.Emit (OpCodes.Brtrue, both_have_value_label);
+
+					// both are null
+					if (Oper == Binary.Operator.Equality)
+						ig.Emit (OpCodes.Ldc_I4_1);
+					else
+						ig.Emit (OpCodes.Ldc_I4_0);
 					ig.Emit (OpCodes.Br, end_label);
+
+					ig.MarkLabel (dissimilar_label);
+					ig.Emit (OpCodes.Pop);
+				} else if (left_unwrap != null) {
+					left_unwrap.EmitCheck (ec);
+					ig.Emit (OpCodes.Brtrue, both_have_value_label);
+				} else if (right_unwrap != null) {
+					right_unwrap.EmitCheck (ec);
+					ig.Emit (OpCodes.Brtrue, both_have_value_label);
+				} else {
+					throw new InternalErrorException ("shouldn't get here");
 				}
 
-				ig.MarkLabel (false_label);
-				if (false_label_used) {
+				// one is null while the other isn't
+				if (Oper == Binary.Operator.Equality)
 					ig.Emit (OpCodes.Ldc_I4_0);
-					if (true_label_used)
-						ig.Emit (OpCodes.Br, end_label);
-				}
-
-				ig.MarkLabel (true_label);
-				if (true_label_used)
+				else
 					ig.Emit (OpCodes.Ldc_I4_1);
+				ig.Emit (OpCodes.Br, end_label);
+
+				ig.MarkLabel (both_have_value_label);
+				underlying.Emit (ec);
 
 				ig.MarkLabel (end_label);
 			}
@@ -3328,9 +3323,8 @@ namespace Mono.CSharp {
 
 			public override Expression DoResolve (EmitContext ec)
 			{
-				unwrap = new Unwrap (expr, loc);
-				expr = unwrap.Resolve (ec);
-				if (expr == null)
+				unwrap = Unwrap.Create (expr, ec);
+				if (unwrap == null)
 					return null;
 
 				if (unwrap.Type != TypeManager.bool_type)
@@ -3403,7 +3397,7 @@ namespace Mono.CSharp {
 				if (TypeManager.IsNullableType (ltype)) {
 					NullableInfo info = new NullableInfo (ltype);
 
-					unwrap = (Unwrap) new Unwrap (left, loc).Resolve (ec);
+					unwrap = Unwrap.Create (left, ec);
 					if (unwrap == null)
 						return null;
 
@@ -3490,7 +3484,7 @@ namespace Mono.CSharp {
 				if (expr == null)
 					return null;
 
-				unwrap = (Unwrap) new Unwrap (expr, loc).Resolve (ec);
+				unwrap = Unwrap.Create (expr, ec);
 				if (unwrap == null)
 					return null;
 
