@@ -3,6 +3,7 @@
 //
 // Author:
 //   Miguel de Icaza (miguel@ximian.com)
+//   Marek Safar (marek.safar@seznam.cz)
 //
 // (C) 2001 Ximian, Inc.
 //
@@ -112,14 +113,13 @@ namespace Mono.CSharp {
 			if (t.IsPointer)
 				throw new InternalErrorException ("Use GetPointerType() to get a pointer");
 			
-			
 			TypeAttributes ta = t.Attributes & TypeAttributes.VisibilityMask;
 			if (ta == TypeAttributes.NestedPrivate)
 				return null;
 			
 			if (ta == TypeAttributes.NotPublic ||
-					ta == TypeAttributes.NestedAssembly ||
-					ta == TypeAttributes.NestedFamANDAssem)
+			    ta == TypeAttributes.NestedAssembly ||
+			    ta == TypeAttributes.NestedFamANDAssem)
 				if (!TypeManager.IsFriendAssembly (t.Assembly))
 					return null;
 
@@ -187,6 +187,12 @@ namespace Mono.CSharp {
 				EnsureNamespace (t.Namespace);
 		}
 
+		public override void Error_NamespaceDoesNotExist(Location loc, string name)
+		{
+			Report.Error (400, loc, "The type or namespace name `{0}' could not be found in the global namespace (are you missing an assembly reference?)",
+				name);
+		}
+
 		public override Type LookupTypeReflection (string name, Location loc)
 		{
 			Type found_type = null;
@@ -298,6 +304,12 @@ namespace Mono.CSharp {
 			return this;
 		}
 
+		public virtual void Error_NamespaceDoesNotExist (Location loc, string name)
+		{
+			Report.Error (234, loc, "The type or namespace name `{0}' does not exist in the namespace `{1}'. Are you missing an assembly reference?",
+				name, FullName);
+		}
+
 		public override void Emit (EmitContext ec)
 		{
 			throw new InternalErrorException ("Expression tree referenced namespace " + fullname + " during Emit ()");
@@ -420,6 +432,8 @@ namespace Mono.CSharp {
 		public bool DeclarationFound = false;
 		bool UsingFound;
 
+		public readonly DeclSpace SlaveDeclSpace;
+
 		ListDictionary extern_aliases;
 
 		static ArrayList entries = new ArrayList ();
@@ -436,7 +450,7 @@ namespace Mono.CSharp {
 		// We use this to flag using clauses for namespaces that do not
 		// exist.
 		//
-		public class UsingEntry {
+		public class UsingEntry : IResolveContext {
 			public readonly MemberName Name;
 			readonly Expression Expr;
 			readonly NamespaceEntry NamespaceEntry;
@@ -457,11 +471,7 @@ namespace Mono.CSharp {
 				if (resolved != null)
 					return resolved;
 
-				DeclSpace root = RootContext.Tree.Types;
-				root.NamespaceEntry = NamespaceEntry;
-				FullNamedExpression fne = Expr.ResolveAsTypeStep (root, false);
-				root.NamespaceEntry = null;
-
+				FullNamedExpression fne = Expr.ResolveAsTypeStep (this, false);
 				if (fne == null) {
 					Error_NamespaceNotFound (Location, Name.ToString ());
 					return null;
@@ -474,6 +484,19 @@ namespace Mono.CSharp {
 				}
 				return resolved;
 			}
+
+			DeclSpace IResolveContext.DeclContainer {
+				get { return NamespaceEntry.SlaveDeclSpace; }
+			}
+			DeclSpace IResolveContext.GenericDeclContainer {
+				get { return NamespaceEntry.SlaveDeclSpace; }
+			}
+			bool IResolveContext.IsInObsoleteScope {
+				get { return false; }
+			}
+			bool IResolveContext.IsInUnsafeScope {
+				get { return false; }
+			}
 		}
 
 		public abstract class AliasEntry {
@@ -504,8 +527,7 @@ namespace Mono.CSharp {
 			protected abstract FullNamedExpression DoResolve ();
 		}
 
-		public class LocalAliasEntry : AliasEntry
-		{
+		public class LocalAliasEntry : AliasEntry, IResolveContext {
 			public readonly Expression Alias;
 			
 			public LocalAliasEntry (NamespaceEntry entry, string name, MemberName alias, Location loc) :
@@ -516,19 +538,38 @@ namespace Mono.CSharp {
 
 			protected override FullNamedExpression DoResolve ()
 			{
-				DeclSpace root = RootContext.Tree.Types;
-				root.NamespaceEntry = NamespaceEntry;
-				resolved = Alias.ResolveAsTypeStep (root, false);
-				root.NamespaceEntry = null;
-
+				resolved = Alias.ResolveAsTypeStep (this, false);
 				if (resolved == null)
-					Error_NamespaceNotFound (Location, Alias.ToString ());
+					return null;
+
+				if (resolved.Type != null) {
+					TypeAttributes attr = resolved.Type.Attributes & TypeAttributes.VisibilityMask;
+					if (attr == TypeAttributes.NestedPrivate || attr == TypeAttributes.NestedFamily ||
+					 	((attr == TypeAttributes.NestedFamORAssem || attr == TypeAttributes.NestedAssembly) && 
+						TypeManager.LookupDeclSpace (resolved.Type) == null)) {
+						Expression.ErrorIsInaccesible (Alias.Location, Alias.ToString ());
+						return null;
+					}
+				}
+
 				return resolved;
 			}
+
+			DeclSpace IResolveContext.DeclContainer {
+				get { return NamespaceEntry.SlaveDeclSpace; }
+			}
+			DeclSpace IResolveContext.GenericDeclContainer {
+				get { return NamespaceEntry.SlaveDeclSpace; }
+			}
+			bool IResolveContext.IsInObsoleteScope {
+				get { return false; }
+			}
+			bool IResolveContext.IsInUnsafeScope {
+				get { return false; }
+			}
 		}
 
-		public class ExternAliasEntry : AliasEntry 
-		{
+		public class ExternAliasEntry : AliasEntry {
 			public ExternAliasEntry (NamespaceEntry entry, string name, Location loc) :
 				base (entry, name, loc)
 			{
@@ -558,9 +599,10 @@ namespace Mono.CSharp {
 				ns = RootNamespace.Global.GetNamespace (name, true);
 			else
 				ns = RootNamespace.Global;
+			SlaveDeclSpace = new RootDeclSpace (this);
 		}
 
-		private NamespaceEntry (NamespaceEntry parent, SourceFile file, Namespace ns)
+		private NamespaceEntry (NamespaceEntry parent, SourceFile file, Namespace ns, bool slave)
 		{
 			this.parent = parent;
 			this.file = file;
@@ -568,6 +610,7 @@ namespace Mono.CSharp {
 			this.ID = -1;
 			this.IsImplicit = true;
 			this.ns = ns;
+			this.SlaveDeclSpace = slave ? new RootDeclSpace (this) : null;
 		}
 
 		//
@@ -584,7 +627,7 @@ namespace Mono.CSharp {
 		NamespaceEntry Doppelganger {
 			get {
 				if (!IsImplicit && doppelganger == null)
-					doppelganger = new NamespaceEntry (ImplicitParent, file, ns);
+					doppelganger = new NamespaceEntry (ImplicitParent, file, ns, true);
 				return doppelganger;
 			}
 		}
@@ -607,7 +650,7 @@ namespace Mono.CSharp {
 				if (implicit_parent == null) {
 					implicit_parent = (parent.NS == ns.Parent)
 						? parent
-						: new NamespaceEntry (parent, file, ns.Parent);
+						: new NamespaceEntry (parent, file, ns.Parent, false);
 				}
 				return implicit_parent;
 			}
@@ -634,9 +677,9 @@ namespace Mono.CSharp {
 			foreach (UsingEntry old_entry in using_clauses) {
 				if (name.Equals (old_entry.Name)) {
 					Report.Warning (105, 3, loc, "The using directive for `{0}' appeared previously in this namespace", name.GetName ());
-						return;
-					}
+					return;
 				}
+			}
 
 			UsingEntry ue = new UsingEntry (Doppelganger, name, loc);
 			using_clauses.Add (ue);
