@@ -76,17 +76,17 @@ namespace Mono.CSharp {
 			28, 67, 78,
 			105, 108, 109, 114, 162, 164, 168, 169, 183, 184, 197,
 			219, 251, 252, 253, 282,
-			419, 420, 429, 436, 440, 465,
+			419, 420, 429, 436, 440, 465, 467, 469,
 			612, 618, 626, 628, 642, 649, 652, 658, 659, 660, 661, 665, 672,
 			1030, 1058,
 			1522, 1570, 1571, 1572, 1573, 1574, 1580, 1581, 1584, 1587, 1589, 1590, 1591, 1592,
-			1616, 1633, 1634, 1635, 1691, 1692,
+			1616, 1633, 1634, 1635, 1690, 1691, 1692,
 			1717, 1718,
 			1901,
 			2002, 2023,
 			3005, 3012, 3019, 3021, 3022, 3023, 3026, 3027,
 			// gmcs warnings start here
-			414, 1700
+			402, 414, 1700
 		};
 
 		static Report ()
