@@ -114,6 +114,7 @@ public partial class TypeManager {
 	static internal Type fixed_buffer_attr_type;
 	static internal Type default_charset_type;
 	static internal Type internals_visible_attr_type;
+	static internal Type type_forwarder_attr_type;
 
 	// 
 	// Expressions representing the internal types.  Used during declaration
@@ -816,7 +817,7 @@ public partial class TypeManager {
 	static Type CoreLookupType (string ns_name, string name)
 	{
 		Namespace ns = RootNamespace.Global.GetNamespace (ns_name, true);
-		FullNamedExpression fne = ns.Lookup (RootContext.Tree.Types, name, Location.Null);
+		FullNamedExpression fne = ns.Lookup (RootContext.ToplevelTypes, name, Location.Null);
 		Type t = fne == null ? null : fne.Type;
 		if (t == null)
 			Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
@@ -1026,6 +1027,7 @@ public partial class TypeManager {
 		default_charset_type = CoreLookupType ("System.Runtime.InteropServices", "DefaultCharSetAttribute");
 		internals_visible_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "InternalsVisibleToAttribute");
 		runtime_compatibility_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "RuntimeCompatibilityAttribute");
+		type_forwarder_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "TypeForwardedToAttribute");
 		//
 		// When compiling corlib, store the "real" types here.
 		//
@@ -1531,6 +1533,9 @@ public partial class TypeManager {
 		if (t == TypeManager.object_type || t == TypeManager.string_type)
 			return false;
 
+		if (t.IsGenericType || t.IsGenericParameter)
+			return false;
+
 		if (IsBuiltinOrEnum (t))
 			return true;
 
@@ -1631,8 +1636,12 @@ public partial class TypeManager {
 
 	public static bool IsPrivateAccessible (Type type, Type parent)
 	{
+		if (type == null)
+			return false;
+
 		if (type.Equals (parent))
 			return true;
+
 		return DropGenericTypeArguments (type) == DropGenericTypeArguments (parent);
 	}
 
@@ -1679,6 +1688,9 @@ public partial class TypeManager {
 	//
 	public static bool IsNestedChildOf (Type type, Type parent)
 	{
+		if (type == null)
+			return false;
+
 		type = DropGenericTypeArguments (type);
 		parent = DropGenericTypeArguments (parent);
 
@@ -2667,7 +2679,7 @@ public partial class TypeManager {
 						IsPrivateAccessible (invocation_type, m.DeclaringType) ||
 						IsNestedChildOf (invocation_type, m.DeclaringType);
 
-				if (invocation_assembly == fi.DeclaringType.Assembly ||
+				if (invocation_assembly == fi.DeclaringType.Assembly || invocation_assembly == null ||
 						TypeManager.IsFriendAssembly (fi.DeclaringType.Assembly)) {
 					if (fa == FieldAttributes.Assembly || fa == FieldAttributes.FamORAssem)
 						return true;
@@ -2782,8 +2794,7 @@ public partial class TypeManager {
 
 			Timer.StopTimer (TimerType.MemberLookup);
 
-			list = MemberLookup_FindMembers (
-				current_type, mt, bf, name, out used_cache);
+			list = MemberLookup_FindMembers (current_type, mt, bf, name, out used_cache);
 
 			Timer.StartTimer (TimerType.MemberLookup);
 
@@ -2843,8 +2854,7 @@ public partial class TypeManager {
 				if ((list.Length == 2) && (list [1] is FieldInfo))
 					return new MemberInfo [] { list [0] };
 
-				// Oooops
-				return null;
+				return list;
 			}
 
 			//
@@ -2852,34 +2862,26 @@ public partial class TypeManager {
 			// mode.
 			//
 
- 			if (first_members_list != null) {
- 				if (use_first_members_list) {
- 					method_list = CopyNewMethods (method_list, first_members_list);
- 					use_first_members_list = false;
- 				}
- 				
+			if (first_members_list != null) {
+				if (use_first_members_list) {
+					method_list = CopyNewMethods (method_list, first_members_list);
+					use_first_members_list = false;
+				}
+				
 				method_list = CopyNewMethods (method_list, list);
 			} else {
 				first_members_list = list;
- 				use_first_members_list = true;
-
+				use_first_members_list = true;
 				mt &= (MemberTypes.Method | MemberTypes.Constructor);
 			}
 		} while (searching);
 
- 		if (use_first_members_list) {
- 			foreach (MemberInfo mi in first_members_list) {
- 				if (! (mi is MethodBase)) {
- 					method_list = CopyNewMethods (method_list, first_members_list);
- 					return (MemberInfo []) method_list.ToArray (typeof (MemberInfo));
- 				}
- 			}
- 			return (MemberInfo []) first_members_list;
- 		}
+		if (use_first_members_list)
+			return first_members_list;
 
 		if (method_list != null && method_list.Count > 0) {
-                        return (MemberInfo []) method_list.ToArray (typeof (MemberInfo));
-                }
+			return (MemberInfo []) method_list.ToArray (typeof (MemberInfo));
+		}
 		//
 		// This happens if we already used the cache in the first iteration, in this case
 		// the cache already looked in all interfaces.
@@ -3030,6 +3032,10 @@ public sealed class TypeHandle : IMemberContainer {
 	private TypeHandle (Type type)
 	{
 		this.type = type;
+#if MS_COMPATIBLE
+		if (type.IsGenericType)
+			this.type = this.type.GetGenericTypeDefinition ();
+#endif
 		full_name = type.FullName != null ? type.FullName : type.Name;
 		if (type.BaseType != null) {
 			base_cache = TypeManager.LookupMemberCache (type.BaseType);
