@@ -18,13 +18,16 @@ using System.Reflection.Emit;
 using System.Text;
 using System.Security;
 
+using SSPermissionSet = System.Security.PermissionSet;
+using MIPermissionSet = Mono.ILASM.PermissionSet;
+
+using MIAssembly = Mono.ILASM.Assembly;
 
 namespace Mono.ILASM {
 
         public class CodeGen {
 
                 private PEFile pefile;
-                private string assembly_name;
                 private ExternAssembly current_assemblyref;
                 private ExternModule current_moduleref;
                 private string current_namespace;
@@ -37,15 +40,7 @@ namespace Mono.ILASM {
                 private IDeclSecurityTarget current_declsectarget;
                 private PEAPI.NativeType current_field_native_type;
 
-                private byte [] assembly_public_key;
-                private int assembly_major_version;
-                private int assembly_minor_version;
-                private int assembly_build_version;
-                private int assembly_revision_version;
-                private string assembly_locale;
-                private int assembly_hash_algorithm;
-                private ArrayList assembly_custom_attributes;
-                private DeclSecurity assembly_declsec;
+                private MIAssembly this_assembly;
                         
                 private TypeManager type_manager;
                 private ExternTable extern_table;
@@ -53,7 +48,7 @@ namespace Mono.ILASM {
                 private Hashtable global_method_table;
                 private Hashtable global_methodref_table;
                 private Hashtable global_fieldref_table;
-                private ArrayList data_list;
+                private Hashtable data_table;
                 private FileRef file_ref;
                 private ArrayList manifestResources;
                 private Hashtable typeref_table;
@@ -68,17 +63,14 @@ namespace Mono.ILASM {
                 private string output_file;
 		private string debug_file;
                 private bool is_dll;
-                private bool is_assembly;
                 private bool entry_point;
 
                 private Module this_module;
 
-                public CodeGen (string output_file, bool is_dll, bool is_assembly,
-				bool debugging_info)
+                public CodeGen (string output_file, bool is_dll, bool debugging_info)
                 {
                         this.output_file = output_file;
                         this.is_dll = is_dll;
-                        this.is_assembly = is_assembly;
 
 			if (debugging_info)
 				symwriter = new SymbolWriter (CreateDebugFile (output_file));
@@ -90,7 +82,7 @@ namespace Mono.ILASM {
                         global_field_table = new Hashtable ();
                         global_method_table = new Hashtable ();
 
-                        data_list = new ArrayList ();
+                        data_table = new Hashtable ();
 
                         defcont_list = new ArrayList ();
 
@@ -248,10 +240,11 @@ namespace Mono.ILASM {
 
                 public void SetAssemblyName (string name)
                 {
-                        if (assembly_name != null && assembly_name != name)
+                        if (this_assembly != null && this_assembly.Name != name)
                                 Report.Error ("Multiple assembly declarations");
-                        assembly_name = name;
-                        if (assembly_name != "mscorlib")
+
+                        this_assembly = new Assembly (name);
+                        if (name != "mscorlib")
                                 ExternTable.AddCorlib ();
                 }
 
@@ -266,9 +259,13 @@ namespace Mono.ILASM {
                         this.file_ref = file_ref;
                 }
 
+                public MIAssembly ThisAssembly {
+                        get { return this_assembly; }
+                }
+
                 public bool IsThisAssembly (string name)
                 {
-                        return (name == assembly_name);
+                        return (this_assembly != null && name == this_assembly.Name);
                 }
 
                 public Module ThisModule {
@@ -358,7 +355,9 @@ namespace Mono.ILASM {
 
                 public void AddDataDef (DataDef datadef)
                 {
-                        data_list.Add (datadef);
+                        if (data_table [datadef.Name] != null)
+                                Report.Error ("Duplicate global label '" + datadef.Name + "'");
+                        data_table [datadef.Name] = datadef;
                 }
 
 		public void AddManifestResource (ManifestResource mr)
@@ -370,12 +369,11 @@ namespace Mono.ILASM {
 
                 public PEAPI.DataConstant GetDataConst (string name)
                 {
-                        foreach (DataDef def in data_list) {
-                                if (def.Name == name)
-                                        return (DataConstant) def.PeapiConstant;
-                        }
-                        return null;
+                        DataDef def = (DataDef) data_table [name];
+                        if (def == null)
+                                return null;
 
+                        return (DataConstant) def.PeapiConstant;
                 }
 
                 public void BeginMethodDef (MethodDef methoddef)
@@ -429,58 +427,33 @@ namespace Mono.ILASM {
                         defcont_list.Add (typedef);
                 }
 
-                public void SetAssemblyPublicKey (byte [] public_key)
-                {
-                        assembly_public_key = public_key;
-                }
-
-                public void SetAssemblyVersion (int major, int minor, int build, int revision)
-                {
-                        assembly_major_version = major;
-                        assembly_minor_version = minor;
-                        assembly_build_version = build;
-                        assembly_revision_version = revision;
-                }
-
-                public void SetAssemblyLocale (string locale)
-                {
-                        assembly_locale = locale;
-                }
-
-                public void SetAssemblyHashAlgorithm (int algorithm)
-                {
-                        assembly_hash_algorithm = algorithm;
-                }
-
-                public void AddAssemblyCustomAttribute (CustomAttr attribute)
-                {
-                        if (assembly_custom_attributes == null)
-                                assembly_custom_attributes = new ArrayList ();
-                        assembly_custom_attributes.Add (attribute);
-                }
-
-                public void AddAssemblyPermission (PEAPI.SecurityAction sec_action, object perm)
-                {
-                        if (assembly_declsec == null)
-                                assembly_declsec = new DeclSecurity ();
-
-                        PermissionSet ps = perm as PermissionSet;
-                        if (ps == null)
-                                assembly_declsec.AddPermission (sec_action, (IPermission) perm);
-                        else
-                                assembly_declsec.AddPermissionSet (sec_action, ps);
-                }
-
                 public void AddPermission (PEAPI.SecurityAction sec_action, object perm)
                 {
                         if (CurrentDeclSecurityTarget == null)
                                 return;
 
-                        PermissionSet ps = perm as PermissionSet;
-                        if (ps == null)
-                                CurrentDeclSecurityTarget.AddPermission (sec_action, (IPermission) perm);
-                        else
-                                CurrentDeclSecurityTarget.AddPermissionSet (sec_action, ps);
+                        AddPermission (sec_action, perm, CurrentDeclSecurityTarget.DeclSecurity);
+                }
+
+                private void AddPermission (PEAPI.SecurityAction sec_action, object perm, DeclSecurity decl_sec)
+                {
+                        SSPermissionSet ps = perm as SSPermissionSet;
+                        if (ps != null) {
+                                decl_sec.AddPermissionSet (sec_action, ps);
+                                return;
+                        }
+
+                        IPermission iper = perm as IPermission;
+                        if (iper != null) {
+                                decl_sec.AddPermission (sec_action, iper);
+                                return;
+                        }
+
+                        MIPermissionSet ps20 = perm as MIPermissionSet;
+                        if (ps20 != null) {
+                                decl_sec.AddPermissionSet (sec_action, ps20);
+                                return;
+                        }
                 }
 
                 public void Write ()
@@ -492,7 +465,7 @@ namespace Mono.ILASM {
                                         this_module = new Module (Path.GetFileName (output_file));
 
                                 out_stream = new FileStream (output_file, FileMode.Create, FileAccess.Write);
-                                pefile = new PEFile (assembly_name, ThisModule.Name, is_dll, is_assembly, null, out_stream);
+                                pefile = new PEFile (ThisAssembly != null ? ThisAssembly.Name : null, ThisModule.Name, is_dll, ThisAssembly != null, null, out_stream);
                                 PEAPI.Assembly asmb = pefile.GetThisAssembly ();
 
                                 ThisModule.PeapiModule = pefile.GetThisModule ();
@@ -520,16 +493,10 @@ namespace Mono.ILASM {
                                         typedef.DefineContents (this);
                                 }
 
-                                if (assembly_custom_attributes != null) {
-                                        foreach (CustomAttr cattr in assembly_custom_attributes)
-                                                cattr.AddTo (this, asmb);
-                                }
-
+                                if (ThisAssembly != null)
+                                        ThisAssembly.Resolve (this, pefile.GetThisAssembly ());
                                 ThisModule.Resolve (this, pefile.GetThisModule ());
 
-                                if (assembly_declsec != null)
-                                        assembly_declsec.AddTo (this, asmb);        
-
                                 if (sub_system != -1)
                                         pefile.SetSubSystem ((PEAPI.SubSystem) sub_system);
                                 if (cor_flags != -1)
@@ -537,11 +504,6 @@ namespace Mono.ILASM {
                                 if (stack_reserve != -1)
                                         pefile.SetStackReserve (stack_reserve);
 
-                                asmb.AddAssemblyInfo(assembly_major_version,
-                                                assembly_minor_version, assembly_build_version,
-                                                assembly_revision_version, assembly_public_key,
-                                                (uint) assembly_hash_algorithm, assembly_locale);
-
                                 pefile.WritePEFile ();
 
 				if (symwriter != null) {
