@@ -81,7 +81,6 @@ namespace Mono.CSharp {
 			// The order is important: this setups the CaptureContext tree hierarchy.
 			//
 			if (container == null) {
-				Report.Error (1706, l, "Anonymous methods are not allowed in attribute declaration");
 				return;
 			}
 			container.SetHaveAnonymousMethods (l, this);
@@ -208,6 +207,13 @@ namespace Mono.CSharp {
 	{
 		TypeContainer host;
 
+		//
+		// The value return by the Compatible call, this ensure that
+		// the code works even if invoked more than once (Resolve called
+		// more than once, due to the way Convert.ImplicitConversion works
+		//
+		Expression anonymous_delegate;
+
 		public AnonymousMethod (TypeContainer host, Parameters parameters, ToplevelBlock container,
 					ToplevelBlock block, Location l)
 			: base (parameters, container, block, l)
@@ -300,6 +306,9 @@ namespace Mono.CSharp {
 		//
 		public Expression Compatible (EmitContext ec, Type delegate_type)
 		{
+			if (anonymous_delegate != null)
+				return anonymous_delegate;
+			
 			//
 			// At this point its the first time we know the return type that is 
 			// needed for the anonymous method.  We create the method here.
@@ -376,14 +385,21 @@ namespace Mono.CSharp {
 			ContainerAnonymousMethod = ec.CurrentAnonymousMethod;
 			ContainingBlock = ec.CurrentBlock;
 
-			if (aec.ResolveTopBlock (ec, Block, Parameters, null, out unreachable))
-				return new AnonymousDelegate (this, delegate_type, loc).Resolve (ec);
-
+			if (aec.ResolveTopBlock (ec, Block, Parameters, null, out unreachable)){
+				anonymous_delegate = new AnonymousDelegate (
+					this, delegate_type, loc).Resolve (ec);
+				return anonymous_delegate;
+			}
 			return null;
 		}
 
 		public override Expression DoResolve (EmitContext ec)
 		{
+			if (!ec.IsAnonymousMethodAllowed) {
+				Report.Error (1706, loc, "Anonymous methods are not allowed in the attribute declaration");
+				return null;
+			}
+
 			if (Parameters != null && !Parameters.Resolve (ec)) {
 				return null;
 			}
@@ -436,9 +452,9 @@ namespace Mono.CSharp {
 			TypeBuilder container = ec.TypeContainer.TypeBuilder;
 			string name = String.Format ("<>AnonHelp<{0}>", scope.id);
 
-			scope.ScopeTypeBuilder = container.DefineNestedType (
-				name, TypeAttributes.AutoLayout | TypeAttributes.Class |
-				TypeAttributes.NestedAssembly, TypeManager.object_type, null);
+			scope.ScopeTypeBuilder = container.DefineNestedType (name,
+				TypeAttributes.Sealed | TypeAttributes.BeforeFieldInit | TypeAttributes.NestedPrivate,
+					 TypeManager.object_type);
 
 			Type [] constructor_types = Type.EmptyTypes;
 			scope.ScopeConstructor = scope.ScopeTypeBuilder.DefineConstructor (
@@ -493,7 +509,8 @@ namespace Mono.CSharp {
 			if ((am.method.ModFlags & Modifiers.STATIC) == 0)
 				delegate_instance_expression = new AnonymousInstance (am);
 			
-			Expression ml = Expression.MemberLookup (ec.ContainerType, type, ".ctor", loc);
+			Expression ml = Expression.MemberLookup (ec.ContainerType, type, ".ctor", MemberTypes.Constructor,
+				BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly, loc);
 			constructor_method = ((MethodGroupExpr) ml).Methods [0];
 			delegate_method = am.GetMethodBuilder ();
 			base.Emit (ec);
@@ -642,11 +659,6 @@ namespace Mono.CSharp {
 			Pad ();
 			Console.WriteLine ("END");
 		}
-		
-		public string MakeHelperName ()
-		{
-			return String.Format ("<>AnonHelp<{0}>", id);
-		}
 
 		private string MakeFieldName (string local_name)
 		{
