@@ -331,6 +331,7 @@ namespace Mono.CSharp {
 			Attributable owner = Owner;
 			EmitContext ec = new EmitContext (owner.ResolveContext, owner.ResolveContext.DeclContainer, owner.ResolveContext.DeclContainer,
 				Location, null, null, owner.ResolveContext.DeclContainer.ModFlags, false);
+			ec.IsAnonymousMethodAllowed = false;
 
 			ConstructorInfo ctor = ResolveConstructor (ec);
 			if (ctor == null)
@@ -518,6 +519,7 @@ namespace Mono.CSharp {
 						Location);
 
 					if (member != null) {
+						Report.SymbolRelatedToPreviousError (member.Type);
 						Expression.ErrorIsInaccesible (Location, member.GetSignatureForError ());
 						return false;
 					}
@@ -1326,6 +1328,14 @@ namespace Mono.CSharp {
 				return (e as BoolLiteral).Value;
 			return false;
 		}
+
+		public Type GetArgumentType ()
+		{
+			TypeOf e = GetValue () as TypeOf;
+			if (e == null)
+				return null;
+			return e.TypeArgument;
+		}
 	}
 	
 
@@ -1346,22 +1356,22 @@ namespace Mono.CSharp {
 
 		void Enter ()
 		{
-			// RootContext.Tree.Types has a single NamespaceEntry which gets overwritten
+			// RootContext.ToplevelTypes has a single NamespaceEntry which gets overwritten
 			// each time a new file is parsed.  However, we need to use the NamespaceEntry
 			// in effect where the attribute was used.  Since code elsewhere cannot assume
 			// that the NamespaceEntry is right, just overwrite it.
 			//
-			// Precondition: RootContext.Tree.Types == null
+			// Precondition: RootContext.ToplevelTypes == null
 
-			if (RootContext.Tree.Types.NamespaceEntry != null)
+			if (RootContext.ToplevelTypes.NamespaceEntry != null)
 				throw new InternalErrorException (Location + " non-null NamespaceEntry");
 
-			RootContext.Tree.Types.NamespaceEntry = ns;
+			RootContext.ToplevelTypes.NamespaceEntry = ns;
 		}
 
 		void Leave ()
 		{
-			RootContext.Tree.Types.NamespaceEntry = null;
+			RootContext.ToplevelTypes.NamespaceEntry = null;
 		}
 
 		protected override TypeExpr ResolveAsTypeTerminal (Expression expr, IResolveContext ec, bool silent)
@@ -1753,9 +1763,18 @@ namespace Mono.CSharp {
 			if (mb.IsSpecialName) {
 				PropertyInfo pi = PropertyExpr.AccessorTable [mb] as PropertyInfo;
 				if (pi != null) {
-					// FIXME: This is buggy as properties from this assembly are included as well
+					if (TypeManager.LookupDeclSpace (pi.DeclaringType) == null)
+						return GetMemberObsoleteAttribute (pi);
+
+					return null;
+				}
+
+				EventInfo ei = EventExpr.AccessorTable [mb] as EventInfo;
+				if (ei != null) {
+					if (TypeManager.LookupDeclSpace (ei.DeclaringType) == null)
+						return GetMemberObsoleteAttribute (ei);
+
 					return null;
-					//return GetMemberObsoleteAttribute (pi);
 				}
 			}
 
