@@ -165,6 +165,9 @@ namespace Mono.CSharp {
 		{
 			try {
 				Assembly.Builder.Save (Basename (name));
+
+				if (SymbolWriter != null)
+					SymbolWriter.WriteSymbolFile ();
 			}
 			catch (COMException) {
 				if ((RootContext.StrongNameKeyFile == null) || (!RootContext.StrongNameDelaySign))
@@ -181,9 +184,6 @@ namespace Mono.CSharp {
 			catch (System.UnauthorizedAccessException ua) {
 				Report.Error (16, "Could not write to file `"+name+"', cause: " + ua.Message);
 			}
-
-			if (SymbolWriter != null)
-				SymbolWriter.WriteSymbolFile ();
 		}
 	}
 
@@ -205,25 +205,50 @@ namespace Mono.CSharp {
 		public DeclSpace TypeContainer;
 		public ILGenerator   ig;
 
-		/// <summary>
-		///   This variable tracks the `checked' state of the compilation,
-		///   it controls whether we should generate code that does overflow
-		///   checking, or if we generate code that ignores overflows.
-		///
-		///   The default setting comes from the command line option to generate
-		///   checked or unchecked code plus any source code changes using the
-		///   checked/unchecked statements or expressions.   Contrast this with
-		///   the ConstantCheckState flag.
-		/// </summary>
-		
-		public bool CheckState;
+		[Flags]
+		public enum Flags : byte {
+			/// <summary>
+			///   This flag tracks the `checked' state of the compilation,
+			///   it controls whether we should generate code that does overflow
+			///   checking, or if we generate code that ignores overflows.
+			///
+			///   The default setting comes from the command line option to generate
+			///   checked or unchecked code plus any source code changes using the
+			///   checked/unchecked statements or expressions.   Contrast this with
+			///   the ConstantCheckState flag.
+			/// </summary>
+			CheckState = 1 << 0,
 
-		/// <summary>
-		///   The constant check state is always set to `true' and cant be changed
-		///   from the command line.  The source code can change this setting with
-		///   the `checked' and `unchecked' statements and expressions. 
-		/// </summary>
-		public bool ConstantCheckState;
+			/// <summary>
+			///   The constant check state is always set to `true' and cant be changed
+			///   from the command line.  The source code can change this setting with
+			///   the `checked' and `unchecked' statements and expressions. 
+			/// </summary>
+			ConstantCheckState = 1 << 1,
+
+			AllCheckStateFlags = CheckState | ConstantCheckState,
+
+			/// <summary>
+			///  Whether we are inside an unsafe block
+			/// </summary>
+			InUnsafe = 1 << 2,
+
+			InCatch = 1 << 3,
+			InFinally = 1 << 4,
+
+			/// <summary>
+			///   Whether control flow analysis is enabled
+			/// </summary>
+			DoFlowAnalysis = 1 << 5,
+
+			/// <summary>
+			///   Whether control flow analysis is disabled on structs
+			///   (only meaningful when DoFlowAnalysis is set)
+			/// </summary>
+			OmitStructFlowAnalysis = 1 << 6
+		}
+
+		Flags flags;
 
 		/// <summary>
 		///   Whether we are emitting code inside a static or instance method
@@ -262,16 +287,6 @@ namespace Mono.CSharp {
 		/// </summary>
 		public bool IsConstructor;
 
-		/// <summary>
-		///   Whether we're control flow analysis enabled
-		/// </summary>
-		public bool DoFlowAnalysis;
-
-		/// <summary>
-		///   Whether we're control flow analysis disabled on struct
-		/// </summary>
-		public bool OmitStructFlowAnalysis;
-
 		/// <summary>
 		///   Keeps track of the Type to LocalBuilder temporary storage created
 		///   to store structures (used to compute the address of the structure
@@ -306,21 +321,11 @@ namespace Mono.CSharp {
 
 		public bool IsLastStatement;
 
-		/// <summary>
-		///  Whether we are inside an unsafe block
-		/// </summary>
-		public bool InUnsafe;
-
 		/// <summary>
 		///  Whether we are in a `fixed' initialization
 		/// </summary>
 		public bool InFixedInitializer;
 
-		public bool InRefOutArgumentResolving;
-
-		public bool InCatch;
-		public bool InFinally;
-
 		/// <summary>
 		///  Whether we are inside an anonymous method.
 		/// </summary>
@@ -363,7 +368,9 @@ namespace Mono.CSharp {
 			Resolving,
 			Emitting
 		}
-		
+
+		bool isAnonymousMethodAllowed = true;
+
 		Phase current_phase;
 		FlowBranching current_flow_branching;
 
@@ -384,8 +391,9 @@ namespace Mono.CSharp {
 
 			TypeContainer = parent;
 			this.declSpace = ds;
-			CheckState = RootContext.Checked;
-			ConstantCheckState = true;
+			if (RootContext.Checked)
+				flags |= Flags.CheckState;
+			flags |= Flags.ConstantCheckState;
 
 			IsStatic = (code_flags & Modifiers.STATIC) != 0;
 			MethodIsStatic = IsStatic;
@@ -395,11 +403,12 @@ namespace Mono.CSharp {
 			CurrentBlock = null;
 			CurrentFile = 0;
 			current_phase = Phase.Created;
-			
+
 			if (parent != null){
 				// Can only be null for the ResolveType contexts.
 				ContainerType = parent.TypeBuilder;
-				InUnsafe = rc.IsInUnsafeScope;
+				if (rc.IsInUnsafeScope)
+					flags |= Flags.InUnsafe;
 			}
 			loc = l;
 
@@ -424,6 +433,67 @@ namespace Mono.CSharp {
 			set { declSpace = value; }
 		}
 
+		public bool CheckState {
+			get { return (flags & Flags.CheckState) != 0; }
+		}
+
+		public bool ConstantCheckState {
+			get { return (flags & Flags.ConstantCheckState) != 0; }
+		}
+
+		public bool InUnsafe {
+			get { return (flags & Flags.InUnsafe) != 0; }
+		}
+
+		public bool InCatch {
+			get { return (flags & Flags.InCatch) != 0; }
+		}
+
+		public bool InFinally {
+			get { return (flags & Flags.InFinally) != 0; }
+		}
+
+		public bool DoFlowAnalysis {
+			get { return (flags & Flags.DoFlowAnalysis) != 0; }
+		}
+
+		public bool OmitStructFlowAnalysis {
+			get { return (flags & Flags.OmitStructFlowAnalysis) != 0; }
+		}
+
+		// utility helper for CheckExpr, UnCheckExpr, Checked and Unchecked statements
+		// it's public so that we can use a struct at the callsite
+		public struct FlagsHandle : IDisposable
+		{
+			EmitContext ec;
+			Flags invmask, oldval;
+			internal FlagsHandle (EmitContext ec, Flags mask, Flags val)
+			{
+				this.ec = ec;
+				invmask = ~mask;
+				oldval = ec.flags & mask;
+				ec.flags = (ec.flags & invmask) | (val & mask);
+			}
+			public void Dispose ()
+			{
+				ec.flags = (ec.flags & invmask) | oldval;
+			}
+		}
+
+		// Temporarily set all the given flags to the given value.  Should be used in an 'using' statement
+		public FlagsHandle With (Flags bits, bool enable)
+		{
+			return new FlagsHandle (this, bits, enable ? bits : 0);
+		}
+
+		public FlagsHandle WithFlowAnalysis (bool do_flow_analysis, bool omit_struct_analysis)
+		{
+			Flags newflags = 
+				(do_flow_analysis ? Flags.DoFlowAnalysis : 0) |
+				(omit_struct_analysis ? Flags.OmitStructFlowAnalysis : 0);
+			return new FlagsHandle (this, Flags.DoFlowAnalysis | Flags.OmitStructFlowAnalysis, newflags);
+		}
+
 		public bool IsInObsoleteScope {
 			get { return ResolveContext.IsInObsoleteScope; }
 		}
@@ -432,6 +502,10 @@ namespace Mono.CSharp {
 			get { return InUnsafe || ResolveContext.IsInUnsafeScope; }
 		}
 
+		public bool IsAnonymousMethodAllowed {
+			get { return isAnonymousMethodAllowed; }
+			set { isAnonymousMethodAllowed = value; }
+		}
 
 		public FlowBranching CurrentBranching {
 			get { return current_flow_branching; }
@@ -470,7 +544,7 @@ namespace Mono.CSharp {
 			else
 				type = FlowBranching.BranchingType.Block;
 
-			DoFlowAnalysis = true;
+			flags |= Flags.DoFlowAnalysis;
 
 			current_flow_branching = FlowBranching.CreateBranching (
 				CurrentBranching, type, block, block.StartLocation);
@@ -479,8 +553,21 @@ namespace Mono.CSharp {
 
 		public FlowBranchingException StartFlowBranching (ExceptionStatement stmt)
 		{
-			FlowBranchingException branching = new FlowBranchingException (
-				CurrentBranching, stmt);
+			FlowBranchingException branching = new FlowBranchingException (CurrentBranching, stmt);
+			current_flow_branching = branching;
+			return branching;
+		}
+
+		public FlowBranchingLabeled StartFlowBranching (LabeledStatement stmt)
+		{
+			FlowBranchingLabeled branching = new FlowBranchingLabeled (CurrentBranching, stmt);
+			current_flow_branching = branching;
+			return branching;
+		}
+
+		public FlowBranchingToplevel StartFlowBranching (ToplevelBlock stmt)
+		{
+			FlowBranchingToplevel branching = new FlowBranchingToplevel (CurrentBranching, stmt);
 			current_flow_branching = branching;
 			return branching;
 		}
@@ -644,42 +731,35 @@ namespace Mono.CSharp {
 				if (!block.ResolveMeta (this, ip))
 					return false;
 
-				bool old_do_flow_analysis = DoFlowAnalysis;
-				DoFlowAnalysis = true;
+				using (this.With (EmitContext.Flags.DoFlowAnalysis, true)) {
+					FlowBranchingToplevel top_level;
+					if (anonymous_method_host != null)
+						top_level = new FlowBranchingToplevel (anonymous_method_host.CurrentBranching, block);
+					else 
+						top_level = block.TopLevelBranching;
 
-				if (anonymous_method_host != null)
-					current_flow_branching = FlowBranching.CreateBranching (
-						anonymous_method_host.CurrentBranching,
-						FlowBranching.BranchingType.Block, block, loc);
-				else 
-					current_flow_branching = block.TopLevelBranching;
-
-				if (!block.Resolve (this)) {
+					current_flow_branching = top_level;
+					bool ok = block.Resolve (this);
 					current_flow_branching = null;
-					DoFlowAnalysis = old_do_flow_analysis;
-					return false;
+
+					if (!ok)
+						return false;
+
+					FlowBranching.Reachability reachability = top_level.End ();
+					if (reachability.IsUnreachable)
+						unreachable = true;
 				}
-
-				FlowBranching.Reachability reachability = current_flow_branching.MergeTopBlock ();
-				current_flow_branching = null;
-
-				DoFlowAnalysis = old_do_flow_analysis;
-
-				if (reachability.AlwaysReturns ||
-				    reachability.AlwaysThrows ||
-				    reachability.IsUnreachable)
-					unreachable = true;
 #if PRODUCTION
 			} catch (Exception e) {
-					Console.WriteLine ("Exception caught by the compiler while compiling:");
-					Console.WriteLine ("   Block that caused the problem begin at: " + loc);
-					
-					if (CurrentBlock != null){
-						Console.WriteLine ("                     Block being compiled: [{0},{1}]",
-								   CurrentBlock.StartLocation, CurrentBlock.EndLocation);
-					}
-					Console.WriteLine (e.GetType ().FullName + ": " + e.Message);
-					throw;
+				Console.WriteLine ("Exception caught by the compiler while compiling:");
+				Console.WriteLine ("   Block that caused the problem begin at: " + loc);
+
+				if (CurrentBlock != null){
+					Console.WriteLine ("                     Block being compiled: [{0},{1}]",
+							   CurrentBlock.StartLocation, CurrentBlock.EndLocation);
+				}
+				Console.WriteLine (e.GetType ().FullName + ": " + e.Message);
+				throw;
 			}
 #endif
 
@@ -788,58 +868,44 @@ namespace Mono.CSharp {
 		/// </summary>
 		public LocalBuilder GetTemporaryLocal (Type t)
 		{
-			LocalBuilder location = null;
-			
-			if (temporary_storage != null){
+			if (temporary_storage != null) {
 				object o = temporary_storage [t];
-				if (o != null){
-					if (o is ArrayList){
-						ArrayList al = (ArrayList) o;
-						
-						for (int i = 0; i < al.Count; i++){
-							if (al [i] != null){
-								location = (LocalBuilder) al [i];
-								al [i] = null;
-								break;
-							}
-						}
-					} else
-						location = (LocalBuilder) o;
-					if (location != null)
-						return location;
+				if (o != null) {
+					if (o is Stack) {
+						Stack s = (Stack) o;
+						o = s.Count == 0 ? null : s.Pop ();
+					} else {
+						temporary_storage.Remove (t);
+					}
 				}
+				if (o != null)
+					return (LocalBuilder) o;
 			}
-			
 			return ig.DeclareLocal (t);
 		}
 
 		public void FreeTemporaryLocal (LocalBuilder b, Type t)
 		{
-			if (temporary_storage == null){
+			Stack s;
+
+			if (temporary_storage == null) {
 				temporary_storage = new Hashtable ();
 				temporary_storage [t] = b;
 				return;
 			}
 			object o = temporary_storage [t];
-			if (o == null){
+			if (o == null) {
 				temporary_storage [t] = b;
 				return;
 			}
-			if (o is ArrayList){
-				ArrayList al = (ArrayList) o;
-				for (int i = 0; i < al.Count; i++){
-					if (al [i] == null){
-						al [i] = b;
-						return;
-					}
-				}
-				al.Add (b);
-				return;
+			if (o is Stack) {
+				s = (Stack) o;
+			} else {
+				s = new Stack ();
+				s.Push (o);
+				temporary_storage [t] = s;
 			}
-			ArrayList replacement = new ArrayList ();
-			replacement.Add (o);
-			temporary_storage.Remove (t);
-			temporary_storage [t] = replacement;
+			s.Push (b);
 		}
 
 		/// <summary>
@@ -1012,29 +1078,21 @@ namespace Mono.CSharp {
 		}
 
 		public override IResolveContext ResolveContext {
-			get {
-				return this;
-			}
+			get { return this; }
 		}
 
 		#region IResolveContext Members
 
 		public DeclSpace DeclContainer {
-			get {
-				return RootContext.Tree.Types;
-			}
+			get { return RootContext.ToplevelTypes; }
 		}
 
 		public bool IsInObsoleteScope {
-			get {
-				return false;
-			}
+			get { return false; }
 		}
 
 		public bool IsInUnsafeScope {
-			get {
-				return false;
-			}
+			get { return false; }
 		}
 
 		#endregion
