@@ -39,6 +39,18 @@ namespace Mono.CSharp {
 			return this.GetType ().Name + " (" + AsString () + ")";
 		}
 
+		public override bool GetAttributableValue (Type valueType, out object value)
+		{
+			Constant c = ToType (valueType, loc);
+			if (c == null) {
+				value = null;
+				return false;
+			}
+
+			value = c.GetTypedValue ();
+			return true;
+		}
+
 		/// <summary>
 		///  This is used to obtain the actual value of the literal
 		///  cast into an object.
@@ -147,7 +159,7 @@ namespace Mono.CSharp {
 			if (type == TypeManager.object_type)
 				return this;
 
-			if (!Convert.ImplicitStandardConversionExists (Convert.ConstantEC, this, type)){
+			if (!Convert.ImplicitStandardConversionExists (this, type)){
 				Error_ValueCannotBeConverted (loc, type, false);
 				return null;
 			}
@@ -206,34 +218,53 @@ namespace Mono.CSharp {
 			return retval;
 		}
 
-		protected void CheckRange (EmitContext ec, ulong value, Type type, ulong max)
+		protected static void CheckRange (bool inCheckedContext, ulong value, ulong max)
 		{
-			if (!ec.ConstantCheckState)
+			if (!inCheckedContext)
 				return;
 
 			if (value > max)
 				throw new OverflowException ();
 		}
 
-		protected void CheckRange (EmitContext ec, double value, Type type, long min, long max)
+		protected static void CheckRange (bool inCheckedContext, double value, long min, long max)
 		{
-			if (!ec.ConstantCheckState)
+			if (!inCheckedContext)
 				return;
 
 			if (((value < min) || (value > max)))
 				throw new OverflowException ();
+
+			if (double.IsNaN (value))
+				throw new OverflowException ();
 		}
 
-		protected void CheckUnsigned (EmitContext ec, long value, Type type)
+		protected static void CheckRange (bool inCheckedContext, double value, ulong min, ulong max)
 		{
-			if (!ec.ConstantCheckState)
+			if (!inCheckedContext)
+				return;
+
+			if (((value < min) || (value > max)))
+				throw new OverflowException ();
+
+			if (double.IsNaN (value))
+				throw new OverflowException ();
+		}
+
+		protected static void CheckUnsigned (bool inCheckedContext, long value)
+		{
+			if (!inCheckedContext)
 				return;
 
 			if (value < 0)
 				throw new OverflowException ();
 		}
 
-		public abstract Constant Reduce (EmitContext ec, Type target_type);
+		/// <summary>
+		/// Maybe ConvertTo name is better. It tries to convert `this' constant to target_type.
+		/// It throws OverflowException 
+		/// </summary>
+		public abstract Constant Reduce (bool inCheckedContext, Type target_type);
 
 		/// <summary>
 		///   Attempts to do a compile-time folding of a constant cast.
@@ -244,11 +275,9 @@ namespace Mono.CSharp {
 				return  TryReduce (ec, target_type);
 			}
 			catch (OverflowException) {
-				if (ec.ConstantCheckState) {
-					Report.Error (221, loc, "Constant value `{0}' cannot be converted to a `{1}' (use `unchecked' syntax to override)",
-						GetValue ().ToString (), TypeManager.CSharpName (target_type));
-				}
-				return null;
+				Report.Error (221, loc, "Constant value `{0}' cannot be converted to a `{1}' (use `unchecked' syntax to override)",
+					GetValue ().ToString (), TypeManager.CSharpName (target_type));
+				throw;
 			}
 		}
 
@@ -265,7 +294,7 @@ namespace Mono.CSharp {
 				return new EnumConstant (c, target_type);
 			}
 
-			return Reduce (ec, target_type);
+			return Reduce (ec.ConstantCheckState, target_type);
 		}
 
 
@@ -306,6 +335,18 @@ namespace Mono.CSharp {
 
 		public abstract Constant Increment ();
 		
+		/// <summary>
+		/// Need to pass type as the constant can require a boxing
+		/// and in such case no optimization is possible
+		/// </summary>
+		public bool IsDefaultInitializer (Type type)
+		{
+			if (type == Type)
+				return IsDefaultValue;
+
+			return Type == TypeManager.null_type;
+		}
+
 		public abstract bool IsDefaultValue {
 			get;
 		}
@@ -322,6 +363,26 @@ namespace Mono.CSharp {
 			get { return false; }
 		}
 	}
+
+	public abstract class IntegralConstant : Constant {
+		protected IntegralConstant (Location loc) :
+			base (loc)
+		{
+		}
+
+		public override void Error_ValueCannotBeConverted (Location loc, Type target, bool expl)
+		{
+			try {
+				Reduce (true, target);
+				base.Error_ValueCannotBeConverted (loc, target, expl);
+			}
+			catch 
+			{
+				Report.Error (31, loc, "Constant value `{0}' cannot be converted to a `{1}'",
+					GetValue ().ToString (), TypeManager.CSharpName (target));
+			}
+		}
+	}
 	
 	public class BoolConstant : Constant {
 		public readonly bool Value;
@@ -375,14 +436,14 @@ namespace Mono.CSharp {
 			get { return Value == false; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			return null;
 		}
 
 	}
 
-	public class ByteConstant : Constant {
+	public class ByteConstant : IntegralConstant {
 		public readonly byte Value;
 
 		public ByteConstant (byte v, Location loc):
@@ -459,10 +520,10 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type)
@@ -597,18 +658,18 @@ namespace Mono.CSharp {
 			get { return Value == '\0'; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MinValue, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MinValue, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type) {
-				CheckRange (ec, Value, target_type, Int16.MinValue, Int16.MaxValue);
+				CheckRange (inCheckedContext, Value, Int16.MinValue, Int16.MaxValue);
 				return new ShortConstant ((short) Value, Location);
 			}
 			if (target_type == TypeManager.int32_type)
@@ -631,7 +692,7 @@ namespace Mono.CSharp {
 
 	}
 
-	public class SByteConstant : Constant {
+	public class SByteConstant : IntegralConstant {
 		public readonly sbyte Value;
 
 		public SByteConstant (sbyte v, Location loc):
@@ -711,26 +772,26 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type)
 				return new ShortConstant ((short) Value, Location);
 			if (target_type == TypeManager.ushort_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new UShortConstant ((ushort) Value, Location);
 			} if (target_type == TypeManager.int32_type)
 				  return new IntConstant ((int) Value, Location);
 			if (target_type == TypeManager.uint32_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new UIntConstant ((uint) Value, Location);
 			} if (target_type == TypeManager.int64_type)
 				  return new LongConstant ((long) Value, Location);
 			if (target_type == TypeManager.uint64_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new ULongConstant ((ulong) Value, Location);
 			}
 			if (target_type == TypeManager.float_type)
@@ -738,7 +799,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -749,7 +810,7 @@ namespace Mono.CSharp {
 
 	}
 
-	public class ShortConstant : Constant {
+	public class ShortConstant : IntegralConstant {
 		public readonly short Value;
 
 		public ShortConstant (short v, Location loc):
@@ -826,30 +887,30 @@ namespace Mono.CSharp {
 			}
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MinValue, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MinValue, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.ushort_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new UShortConstant ((ushort) Value, Location);
 			}
 			if (target_type == TypeManager.int32_type)
 				return new IntConstant ((int) Value, Location);
 			if (target_type == TypeManager.uint32_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new UIntConstant ((uint) Value, Location);
 			}
 			if (target_type == TypeManager.int64_type)
 				return new LongConstant ((long) Value, Location);
 			if (target_type == TypeManager.uint64_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new ULongConstant ((ulong) Value, Location);
 			}
 			if (target_type == TypeManager.float_type)
@@ -857,7 +918,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, Char.MinValue, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MinValue, Char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -868,7 +929,7 @@ namespace Mono.CSharp {
 
 	}
 
-	public class UShortConstant : Constant {
+	public class UShortConstant : IntegralConstant {
 		public readonly ushort Value;
 
 		public UShortConstant (ushort v, Location loc):
@@ -945,18 +1006,18 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MinValue, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MinValue, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type) {
-				CheckRange (ec, Value, target_type, Int16.MinValue, Int16.MaxValue);
+				CheckRange (inCheckedContext, Value, Int16.MinValue, Int16.MaxValue);
 				return new ShortConstant ((short) Value, Location);
 			}
 			if (target_type == TypeManager.int32_type)
@@ -972,7 +1033,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, Char.MinValue, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MinValue, Char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -982,7 +1043,7 @@ namespace Mono.CSharp {
 		}
 	}
 
-	public class IntConstant : Constant {
+	public class IntConstant : IntegralConstant {
 		public readonly int Value;
 
 		public IntConstant (int v, Location loc):
@@ -1122,32 +1183,32 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MinValue, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MinValue, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type) {
-				CheckRange (ec, Value, target_type, Int16.MinValue, Int16.MaxValue);
+				CheckRange (inCheckedContext, Value, Int16.MinValue, Int16.MaxValue);
 				return new ShortConstant ((short) Value, Location);
 			}
 			if (target_type == TypeManager.ushort_type) {
-				CheckRange (ec, Value, target_type, UInt16.MinValue, UInt16.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt16.MinValue, UInt16.MaxValue);
 				return new UShortConstant ((ushort) Value, Location);
 			}
 			if (target_type == TypeManager.uint32_type) {
-				CheckRange (ec, Value, target_type, Int32.MinValue, Int32.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt32.MinValue, UInt32.MaxValue);
 				return new UIntConstant ((uint) Value, Location);
 			}
 			if (target_type == TypeManager.int64_type)
 				return new LongConstant ((long) Value, Location);
 			if (target_type == TypeManager.uint64_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new ULongConstant ((ulong) Value, Location);
 			}
 			if (target_type == TypeManager.float_type)
@@ -1155,7 +1216,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, Char.MinValue, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MinValue, Char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -1165,7 +1226,7 @@ namespace Mono.CSharp {
 		}
 	}
 
-	public class UIntConstant : Constant {
+	public class UIntConstant : IntegralConstant {
 		public readonly uint Value;
 
 		public UIntConstant (uint v, Location loc):
@@ -1242,26 +1303,26 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Char.MinValue, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MinValue, Char.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type) {
-				CheckRange (ec, Value, target_type, Int16.MinValue, Int16.MaxValue);
+				CheckRange (inCheckedContext, Value, Int16.MinValue, Int16.MaxValue);
 				return new ShortConstant ((short) Value, Location);
 			}
 			if (target_type == TypeManager.ushort_type) {
-				CheckRange (ec, Value, target_type, UInt16.MinValue, UInt16.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt16.MinValue, UInt16.MaxValue);
 				return new UShortConstant ((ushort) Value, Location);
 			}
 			if (target_type == TypeManager.int32_type) {
-				CheckRange (ec, Value, target_type, Int32.MinValue, Int32.MaxValue);
+				CheckRange (inCheckedContext, Value, Int32.MinValue, Int32.MaxValue);
 				return new IntConstant ((int) Value, Location);
 			}
 			if (target_type == TypeManager.int64_type)
@@ -1273,7 +1334,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, Char.MinValue, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MinValue, Char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -1284,7 +1345,7 @@ namespace Mono.CSharp {
 
 	}
 
-	public class LongConstant : Constant {
+	public class LongConstant : IntegralConstant {
 		public readonly long Value;
 
 		public LongConstant (long v, Location loc):
@@ -1297,19 +1358,17 @@ namespace Mono.CSharp {
 
 		public override void Emit (EmitContext ec)
 		{
-			ILGenerator ig = ec.ig;
-
-			EmitLong (ig, Value);
+			EmitLong (ec.ig, Value);
 		}
 
 		static public void EmitLong (ILGenerator ig, long l)
 		{
-			if ((l >> 32) == 0){
+			if (l >= int.MinValue && l <= int.MaxValue) {
 				IntLiteral.EmitInt (ig, unchecked ((int) l));
-				ig.Emit (OpCodes.Conv_U8);
-			} else {
-				ig.Emit (OpCodes.Ldc_I8, l);
+				ig.Emit (OpCodes.Conv_I8);
+				return;
 			}
+			ig.Emit (OpCodes.Ldc_I8, l);
 		}
 
 		public override string AsString ()
@@ -1376,34 +1435,34 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MinValue, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MinValue, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type) {
-				CheckRange (ec, Value, target_type, Int16.MinValue, Int16.MaxValue);
+				CheckRange (inCheckedContext, Value, Int16.MinValue, Int16.MaxValue);
 				return new ShortConstant ((short) Value, Location);
 			}
 			if (target_type == TypeManager.ushort_type) {
-				CheckRange (ec, Value, target_type, UInt16.MinValue, UInt16.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt16.MinValue, UInt16.MaxValue);
 				return new UShortConstant ((ushort) Value, Location);
 			}
 			if (target_type == TypeManager.int32_type) {
-				CheckRange (ec, Value, target_type, Int32.MinValue, Int32.MaxValue);
+				CheckRange (inCheckedContext, Value, Int32.MinValue, Int32.MaxValue);
 				return new IntConstant ((int) Value, Location);
 			}
 			if (target_type == TypeManager.uint32_type) {
-				CheckRange (ec, Value, target_type, UInt32.MinValue, UInt32.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt32.MinValue, UInt32.MaxValue);
 				return new UIntConstant ((uint) Value, Location);
 			}
 			if (target_type == TypeManager.uint64_type) {
-				CheckUnsigned (ec, Value, target_type);
+				CheckUnsigned (inCheckedContext, Value);
 				return new ULongConstant ((ulong) Value, Location);
 			}
 			if (target_type == TypeManager.float_type)
@@ -1411,7 +1470,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, Char.MinValue, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MinValue, Char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -1422,7 +1481,7 @@ namespace Mono.CSharp {
 
 	}
 
-	public class ULongConstant : Constant {
+	public class ULongConstant : IntegralConstant {
 		public readonly ulong Value;
 
 		public ULongConstant (ulong v, Location loc):
@@ -1501,34 +1560,34 @@ namespace Mono.CSharp {
 			get { return Value == 0; }
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, (ulong) SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, (ulong) SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
 			if (target_type == TypeManager.short_type) {
-				CheckRange (ec, Value, target_type, (ulong) Int16.MaxValue);
+				CheckRange (inCheckedContext, Value, (ulong) Int16.MaxValue);
 				return new ShortConstant ((short) Value, Location);
 			}
 			if (target_type == TypeManager.ushort_type) {
-				CheckRange (ec, Value, target_type, UInt16.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt16.MaxValue);
 				return new UShortConstant ((ushort) Value, Location);
 			}
 			if (target_type == TypeManager.int32_type) {
-				CheckRange (ec, Value, target_type, Int32.MaxValue);
+				CheckRange (inCheckedContext, Value, Int32.MaxValue);
 				return new IntConstant ((int) Value, Location);
 			}
 			if (target_type == TypeManager.uint32_type) {
-				CheckRange (ec, Value, target_type, UInt32.MaxValue);
+				CheckRange (inCheckedContext, Value, UInt32.MaxValue);
 				return new UIntConstant ((uint) Value, Location);
 			}
 			if (target_type == TypeManager.int64_type) {
-				CheckRange (ec, Value, target_type, (ulong) Int64.MaxValue);
+				CheckRange (inCheckedContext, Value, (ulong) Int64.MaxValue);
 				return new LongConstant ((long) Value, Location);
 			}
 			if (target_type == TypeManager.float_type)
@@ -1536,7 +1595,7 @@ namespace Mono.CSharp {
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, Char.MaxValue);
+				CheckRange (inCheckedContext, Value, Char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -1615,28 +1674,46 @@ namespace Mono.CSharp {
 			}
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
-			if (target_type == TypeManager.byte_type)
+			if (target_type == TypeManager.byte_type) {
+				CheckRange (inCheckedContext, Value, byte.MinValue, byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
-			if (target_type == TypeManager.sbyte_type)
+			}
+			if (target_type == TypeManager.sbyte_type) {
+				CheckRange (inCheckedContext, Value,  sbyte.MinValue, sbyte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
-			if (target_type == TypeManager.short_type)
+			}
+			if (target_type == TypeManager.short_type) {
+				CheckRange (inCheckedContext, Value, short.MinValue, short.MaxValue);
 				return new ShortConstant ((short) Value, Location);
-			if (target_type == TypeManager.ushort_type)
+			}
+			if (target_type == TypeManager.ushort_type) {
+				CheckRange (inCheckedContext, Value, ushort.MinValue, ushort.MaxValue);
 				return new UShortConstant ((ushort) Value, Location);
-			if (target_type == TypeManager.int32_type)
+			}
+			if (target_type == TypeManager.int32_type) {
+				CheckRange (inCheckedContext, Value, int.MinValue, int.MaxValue);
 				return new IntConstant ((int) Value, Location);
-			if (target_type == TypeManager.uint32_type)
+			}
+			if (target_type == TypeManager.uint32_type) {
+				CheckRange (inCheckedContext, Value, uint.MinValue, uint.MaxValue);
 				return new UIntConstant ((uint) Value, Location);
-			if (target_type == TypeManager.int64_type)
+			}
+			if (target_type == TypeManager.int64_type) {
+				CheckRange (inCheckedContext, Value, long.MinValue, long.MaxValue);
 				return new LongConstant ((long) Value, Location);
-			if (target_type == TypeManager.uint64_type)
+			}
+			if (target_type == TypeManager.uint64_type) {
+				CheckRange (inCheckedContext, Value, ulong.MinValue, ulong.MaxValue);
 				return new ULongConstant ((ulong) Value, Location);
+			}
 			if (target_type == TypeManager.double_type)
 				return new DoubleConstant ((double) Value, Location);
-			if (target_type == TypeManager.char_type)
+			if (target_type == TypeManager.char_type) {
+				CheckRange (inCheckedContext, Value, char.MinValue, char.MaxValue);
 				return new CharConstant ((char) Value, Location);
+			}
 			if (target_type == TypeManager.decimal_type)
 				return new DecimalConstant ((decimal) Value, Location);
 
@@ -1718,32 +1795,44 @@ namespace Mono.CSharp {
 			}
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.byte_type) {
-				CheckRange (ec, Value, target_type, Byte.MinValue, Byte.MaxValue);
+				CheckRange (inCheckedContext, Value, Byte.MinValue, Byte.MaxValue);
 				return new ByteConstant ((byte) Value, Location);
 			}
 			if (target_type == TypeManager.sbyte_type) {
-				CheckRange (ec, Value, target_type, SByte.MinValue, SByte.MaxValue);
+				CheckRange (inCheckedContext, Value, SByte.MinValue, SByte.MaxValue);
 				return new SByteConstant ((sbyte) Value, Location);
 			}
-			if (target_type == TypeManager.short_type)
+			if (target_type == TypeManager.short_type) {
+				CheckRange (inCheckedContext, Value, short.MinValue, short.MaxValue);
 				return new ShortConstant ((short) Value, Location);
-			if (target_type == TypeManager.ushort_type)
+			}
+			if (target_type == TypeManager.ushort_type) {
+				CheckRange (inCheckedContext, Value, ushort.MinValue, ushort.MaxValue);
 				return new UShortConstant ((ushort) Value, Location);
-			if (target_type == TypeManager.int32_type)
+			}
+			if (target_type == TypeManager.int32_type) {
+				CheckRange (inCheckedContext, Value, int.MinValue, int.MaxValue);
 				return new IntConstant ((int) Value, Location);
-			if (target_type == TypeManager.uint32_type)
+			}
+			if (target_type == TypeManager.uint32_type) {
+				CheckRange (inCheckedContext, Value, uint.MinValue, uint.MaxValue);
 				return new UIntConstant ((uint) Value, Location);
-			if (target_type == TypeManager.int64_type)
+			}
+			if (target_type == TypeManager.int64_type) {
+				CheckRange (inCheckedContext, Value, long.MinValue, long.MaxValue);
 				return new LongConstant ((long) Value, Location);
-			if (target_type == TypeManager.uint64_type)
+			}
+			if (target_type == TypeManager.uint64_type) {
+				CheckRange (inCheckedContext, Value, ulong.MinValue, ulong.MaxValue);
 				return new ULongConstant ((ulong) Value, Location);
+			}
 			if (target_type == TypeManager.float_type)
 				return new FloatConstant ((float) Value, Location);
 			if (target_type == TypeManager.char_type) {
-				CheckRange (ec, Value, target_type, char.MinValue, char.MaxValue);
+				CheckRange (inCheckedContext, Value, char.MinValue, char.MaxValue);
 				return new CharConstant ((char) Value, Location);
 			}
 			if (target_type == TypeManager.decimal_type)
@@ -1825,7 +1914,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			if (target_type == TypeManager.sbyte_type)
 				return new SByteConstant ((sbyte)Value, loc);
@@ -1902,7 +1991,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		public override Constant Reduce (EmitContext ec, Type target_type)
+		public override Constant Reduce (bool inCheckedContext, Type target_type)
 		{
 			return null;
 		}
