@@ -99,6 +99,12 @@ namespace Mono.CSharp {
 			return GetName (false);
 		}
 
+		public bool IsGeneric {
+			get {
+				return false;
+			}
+		}
+
 		public string GetName (bool is_generic)
 		{
 			string name = is_generic ? Basename : Name;
@@ -309,9 +315,9 @@ namespace Mono.CSharp {
 		public virtual string GetSignatureForError ()
 		{
 			if (Parent == null || Parent.Parent == null)
-				return Name;
+				return member_name.ToString ();
 
-			return String.Concat (Parent.GetSignatureForError (), '.', Name);
+			return String.Concat (Parent.GetSignatureForError (), '.', member_name.ToString ());
 		}
 
 		/// <summary>
@@ -520,17 +526,13 @@ namespace Mono.CSharp {
 		}
 
 		public override IResolveContext ResolveContext {
-			get {
-				return this;
-			}
+			get { return this; }
 		}
 
 		#region IResolveContext Members
 
-		public virtual DeclSpace DeclContainer {
-			get {
-				return Parent;
-			}
+		public DeclSpace DeclContainer {
+			get { return Parent; }
 		}
 
 		public bool IsInObsoleteScope {
@@ -582,13 +584,13 @@ namespace Mono.CSharp {
 		
 		protected Hashtable defined_names;
 
+		public TypeContainer PartialContainer;		
+
 		//
 		// Whether we are Generic
 		//
 		public bool IsGeneric {
-			get {
-				return false;
-			}
+			get { return false; }
 		}
 
 		static string[] attribute_targets = new string [] { "type" };
@@ -600,12 +602,7 @@ namespace Mono.CSharp {
 			NamespaceEntry = ns;
 			Basename = name.Name;
 			defined_names = new Hashtable ();
-		}
-
-		public override DeclSpace DeclContainer {
-			get {
-				return this;
-			}
+			PartialContainer = null;
 		}
 
 		/// <summary>
@@ -887,15 +884,6 @@ namespace Mono.CSharp {
 			return ~ (~ mAccess | pAccess) == 0;
 		}
 
-		//
-		// Return the nested type with name @name.  Ensures that the nested type
-		// is defined if necessary.  Do _not_ use this when you have a MemberCache handy.
-		//
-		public virtual Type FindNestedType (string name)
-		{
-			return null;
-		}
-
 		private Type LookupNestedTypeInHierarchy (string name)
 		{
 			// if the member cache has been created, lets use it.
@@ -909,10 +897,11 @@ namespace Mono.CSharp {
 			     current_type != null && current_type != TypeManager.object_type;
 			     current_type = current_type.BaseType) {
 				if (current_type is TypeBuilder) {
-					DeclSpace decl = this;
-					if (current_type != TypeBuilder)
-						decl = TypeManager.LookupDeclSpace (current_type);
-					t = decl.FindNestedType (name);
+					TypeContainer tc = current_type == TypeBuilder
+						? PartialContainer
+						: TypeManager.LookupTypeContainer (current_type);
+					if (tc != null)
+						t = tc.FindNestedType (name);
 				} else {
 					t = TypeManager.GetNestedType (current_type, name);
 				}
@@ -940,7 +929,7 @@ namespace Mono.CSharp {
 			Type t = LookupNestedTypeInHierarchy (name);
 			if (t != null)
 				e = new TypeExpression (t, Location.Null);
-			else if (Parent != null && Parent != RootContext.Tree.Types)
+			else if (Parent != null)
 				e = Parent.LookupType (name, loc, ignore_cs0104);
 			else
 				e = NamespaceEntry.LookupNamespaceOrType (this, name, loc, ignore_cs0104);
@@ -1632,6 +1621,7 @@ namespace Mono.CSharp {
 
 			IMemberContainer current = Container;
 
+			bool do_interface_search = current.IsInterface;
 
 			// `applicable' is a list of all members with the given member name `name'
 			// in the current class and all its base classes.  The list is sorted in
@@ -1647,7 +1637,10 @@ namespace Mono.CSharp {
 				// iteration of this loop if there are no members with the name we're
 				// looking for in the current class).
 				if (entry.Container != current) {
-					if (declared_only || DoneSearching (global))
+					if (declared_only)
+						break;
+
+					if (!do_interface_search && DoneSearching (global))
 						break;
 
 					current = entry.Container;
@@ -1663,8 +1656,28 @@ namespace Mono.CSharp {
 
 				// Apply the filter to it.
 				if (filter (entry.Member, criteria)) {
-					if ((entry.EntryType & EntryType.MaskType) != EntryType.Method)
+					if ((entry.EntryType & EntryType.MaskType) != EntryType.Method) {
 						do_method_search = false;
+					}
+					
+					// Because interfaces support multiple inheritance we have to be sure that
+					// base member is from same interface, so only top level member will be returned
+					if (do_interface_search && global.Count > 0) {
+						bool member_already_exists = false;
+
+						foreach (MemberInfo mi in global) {
+							if (mi is MethodBase)
+								continue;
+
+							if (IsInterfaceBaseInterface (TypeManager.GetInterfaces (mi.DeclaringType), entry.Member.DeclaringType)) {
+								member_already_exists = true;
+								break;
+							}
+						}
+						if (member_already_exists)
+							continue;
+					}
+
 					global.Add (entry.Member);
 				}
 			}
@@ -1686,6 +1699,22 @@ namespace Mono.CSharp {
 			global.CopyTo (copy);
 			return copy;
 		}
+
+		/// <summary>
+		/// Returns true if iterface exists in any base interfaces (ifaces)
+		/// </summary>
+		static bool IsInterfaceBaseInterface (Type[] ifaces, Type ifaceToFind)
+		{
+			foreach (Type iface in ifaces) {
+				if (iface == ifaceToFind)
+					return true;
+
+				Type[] base_ifaces = TypeManager.GetInterfaces (iface);
+				if (base_ifaces.Length > 0 && IsInterfaceBaseInterface (base_ifaces, ifaceToFind))
+					return true;
+			}
+			return false;
+		}
 		
 		// find the nested type @name in @this.
 		public Type FindNestedType (string name)
