@@ -354,6 +354,9 @@ namespace Mono.CSharp {
 				Parameter.Modifier invoke_pd_type_mod = invoke_pd.ParameterModifier (i);
 				Parameter.Modifier pd_type_mod = pd.ParameterModifier (i);
 
+				invoke_pd_type_mod &= ~Parameter.Modifier.PARAMS;
+				pd_type_mod &= ~Parameter.Modifier.PARAMS;
+
 				if (invoke_pd_type == pd_type &&
 				    invoke_pd_type_mod == pd_type_mod)
 					continue;
@@ -492,13 +495,14 @@ namespace Mono.CSharp {
 		public override MemberList FindMembers (MemberTypes mt, BindingFlags bf,
 							MemberFilter filter, object criteria)
 		{
-			ArrayList members = new ArrayList ();
+			ArrayList members = new ArrayList (2);
+
+			if ((mt & MemberTypes.Constructor) != 0) {
+				if (ConstructorBuilder != null && filter (ConstructorBuilder, criteria))
+					members.Add (ConstructorBuilder);
+			}
 
 			if ((mt & MemberTypes.Method) != 0) {
-				if (ConstructorBuilder != null)
-				if (filter (ConstructorBuilder, criteria))
-					members.Add (ConstructorBuilder);
-
 				if (InvokeBuilder != null)
 				if (filter (InvokeBuilder, criteria))
 					members.Add (InvokeBuilder);
@@ -613,8 +617,8 @@ namespace Mono.CSharp {
 
 		protected bool ResolveConstructorMethod (EmitContext ec)
 		{
-			Expression ml = Expression.MemberLookup (
-				ec.ContainerType, type, ".ctor", loc);
+			Expression ml = Expression.MemberLookupFinal(ec, 
+				null, type, ".ctor", MemberTypes.Constructor, AllBindingFlags | BindingFlags.DeclaredOnly, loc);
 
 			if (!(ml is MethodGroupExpr)) {
 				Report.Error (-100, loc, "Internal error: Could not find delegate constructor!");
