@@ -3,6 +3,7 @@
 //
 // Author:
 //   Miguel de Icaza (miguel@ximian.com)
+//   Marek Safar (marek.safar@seznam.cz)
 //
 // (C) 2001 Ximian, Inc.
 //
@@ -183,6 +184,12 @@ namespace Mono.CSharp {
 				EnsureNamespace (t.Namespace);
 		}
 
+		public override void Error_NamespaceDoesNotExist(Location loc, string name)
+		{
+			Report.Error (400, loc, "The type or namespace name `{0}' could not be found in the global namespace (are you missing an assembly reference?)",
+				name);
+		}
+
 		public override Type LookupTypeReflection (string name, Location loc)
 		{
 			Type found_type = null;
@@ -294,6 +301,12 @@ namespace Mono.CSharp {
 			return this;
 		}
 
+		public virtual void Error_NamespaceDoesNotExist (Location loc, string name)
+		{
+			Report.Error (234, loc, "The type or namespace name `{0}' does not exist in the namespace `{1}'. Are you missing an assembly reference?",
+				name, FullName);
+		}
+
 		public override void Emit (EmitContext ec)
 		{
 			throw new InternalErrorException ("Expression tree referenced namespace " + fullname + " during Emit ()");
@@ -416,6 +429,8 @@ namespace Mono.CSharp {
 		public bool DeclarationFound = false;
 		bool UsingFound;
 
+		public readonly DeclSpace SlaveDeclSpace;
+
 		ListDictionary extern_aliases;
 
 		static ArrayList entries = new ArrayList ();
@@ -432,7 +447,7 @@ namespace Mono.CSharp {
 		// We use this to flag using clauses for namespaces that do not
 		// exist.
 		//
-		public class UsingEntry {
+		public class UsingEntry : IResolveContext {
 			public readonly MemberName Name;
 			readonly Expression Expr;
 			readonly NamespaceEntry NamespaceEntry;
@@ -453,11 +468,7 @@ namespace Mono.CSharp {
 				if (resolved != null)
 					return resolved;
 
-				DeclSpace root = RootContext.Tree.Types;
-				root.NamespaceEntry = NamespaceEntry;
-				FullNamedExpression fne = Expr.ResolveAsTypeStep (root, false);
-				root.NamespaceEntry = null;
-
+				FullNamedExpression fne = Expr.ResolveAsTypeStep (this, false);
 				if (fne == null) {
 					Error_NamespaceNotFound (Location, Name.ToString ());
 					return null;
@@ -470,6 +481,16 @@ namespace Mono.CSharp {
 				}
 				return resolved;
 			}
+
+			DeclSpace IResolveContext.DeclContainer {
+				get { return NamespaceEntry.SlaveDeclSpace; }
+			}
+			bool IResolveContext.IsInObsoleteScope {
+				get { return false; }
+			}
+			bool IResolveContext.IsInUnsafeScope {
+				get { return false; }
+			}
 		}
 
 		public abstract class AliasEntry {
@@ -500,8 +521,7 @@ namespace Mono.CSharp {
 			protected abstract FullNamedExpression DoResolve ();
 		}
 
-		public class LocalAliasEntry : AliasEntry
-		{
+		public class LocalAliasEntry : AliasEntry, IResolveContext {
 			public readonly Expression Alias;
 			
 			public LocalAliasEntry (NamespaceEntry entry, string name, MemberName alias, Location loc) :
@@ -512,19 +532,35 @@ namespace Mono.CSharp {
 
 			protected override FullNamedExpression DoResolve ()
 			{
-				DeclSpace root = RootContext.Tree.Types;
-				root.NamespaceEntry = NamespaceEntry;
-				resolved = Alias.ResolveAsTypeStep (root, false);
-				root.NamespaceEntry = null;
-
+				resolved = Alias.ResolveAsTypeStep (this, false);
 				if (resolved == null)
-					Error_NamespaceNotFound (Location, Alias.ToString ());
+					return null;
+
+				if (resolved.Type != null) {
+					TypeAttributes attr = resolved.Type.Attributes & TypeAttributes.VisibilityMask;
+					if (attr == TypeAttributes.NestedPrivate || attr == TypeAttributes.NestedFamily ||
+					 	((attr == TypeAttributes.NestedFamORAssem || attr == TypeAttributes.NestedAssembly) && 
+						TypeManager.LookupDeclSpace (resolved.Type) == null)) {
+						Expression.ErrorIsInaccesible (Alias.Location, Alias.ToString ());
+						return null;
+					}
+				}
+
 				return resolved;
 			}
+
+			DeclSpace IResolveContext.DeclContainer {
+				get { return NamespaceEntry.SlaveDeclSpace; }
+			}
+			bool IResolveContext.IsInObsoleteScope {
+				get { return false; }
+			}
+			bool IResolveContext.IsInUnsafeScope {
+				get { return false; }
+			}
 		}
 
-		public class ExternAliasEntry : AliasEntry 
-		{
+		public class ExternAliasEntry : AliasEntry {
 			public ExternAliasEntry (NamespaceEntry entry, string name, Location loc) :
 				base (entry, name, loc)
 			{
@@ -554,9 +590,10 @@ namespace Mono.CSharp {
 				ns = RootNamespace.Global.GetNamespace (name, true);
 			else
 				ns = RootNamespace.Global;
+			SlaveDeclSpace = new RootDeclSpace (this);
 		}
 
-		private NamespaceEntry (NamespaceEntry parent, SourceFile file, Namespace ns)
+		private NamespaceEntry (NamespaceEntry parent, SourceFile file, Namespace ns, bool slave)
 		{
 			this.parent = parent;
 			this.file = file;
@@ -564,6 +601,7 @@ namespace Mono.CSharp {
 			this.ID = -1;
 			this.IsImplicit = true;
 			this.ns = ns;
+			this.SlaveDeclSpace = slave ? new RootDeclSpace (this) : null;
 		}
 
 		//
@@ -580,7 +618,7 @@ namespace Mono.CSharp {
 		NamespaceEntry Doppelganger {
 			get {
 				if (!IsImplicit && doppelganger == null)
-					doppelganger = new NamespaceEntry (ImplicitParent, file, ns);
+					doppelganger = new NamespaceEntry (ImplicitParent, file, ns, true);
 				return doppelganger;
 			}
 		}
@@ -603,7 +641,7 @@ namespace Mono.CSharp {
 				if (implicit_parent == null) {
 					implicit_parent = (parent.NS == ns.Parent)
 						? parent
-						: new NamespaceEntry (parent, file, ns.Parent);
+						: new NamespaceEntry (parent, file, ns.Parent, false);
 				}
 				return implicit_parent;
 			}
