@@ -114,6 +114,8 @@ public class TypeManager {
 	static internal Type compiler_generated_attr_type;
 	static internal Type fixed_buffer_attr_type;
 	static internal Type default_charset_type;
+	static internal Type internals_visible_attr_type;
+	static internal Type type_forwarder_attr_type;
 #endif
 
 	// 
@@ -429,7 +431,7 @@ public class TypeManager {
 
 	public static MemberCache LookupBaseInterfacesCache (Type t)
 	{
-		Type [] ifaces = t.GetInterfaces ();
+		Type [] ifaces = GetInterfaces (t);
 
 		if (ifaces != null && ifaces.Length == 1)
 			return LookupMemberCache (ifaces [0]);
@@ -675,7 +677,7 @@ public class TypeManager {
 	static Type CoreLookupType (string ns_name, string name)
 	{
 		Namespace ns = RootNamespace.Global.GetNamespace (ns_name, true);
-		FullNamedExpression fne = ns.Lookup (RootContext.Tree.Types, name, Location.Null);
+		FullNamedExpression fne = ns.Lookup (RootContext.ToplevelTypes, name, Location.Null);
 		Type t = fne == null ? null : fne.Type;
 		if (t == null)
 			Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
@@ -882,7 +884,9 @@ public class TypeManager {
 		compiler_generated_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "CompilerGeneratedAttribute");
 		fixed_buffer_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "FixedBufferAttribute");
 		default_charset_type = CoreLookupType ("System.Runtime.InteropServices", "DefaultCharSetAttribute");
+		internals_visible_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "InternalsVisibleToAttribute");
 		runtime_compatibility_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "RuntimeCompatibilityAttribute");
+		type_forwarder_attr_type = CoreLookupType ("System.Runtime.CompilerServices", "TypeForwardedToAttribute");
 #endif
 		//
 		// When compiling corlib, store the "real" types here.
@@ -1439,6 +1443,9 @@ public class TypeManager {
 		if (type == parent)
 			return false;
 
+		if (type == null)
+			return false;
+
 		type = type.DeclaringType;
 		while (type != null) {
 			if (type == parent)
@@ -2240,7 +2247,7 @@ public class TypeManager {
 					return private_ok || (invocation_type == m.DeclaringType) ||
 						IsNestedChildOf (invocation_type, m.DeclaringType);
 
-				if (invocation_assembly == fi.DeclaringType.Assembly) {
+				if (invocation_assembly == fi.DeclaringType.Assembly || invocation_assembly == null) {
 					if (fa == FieldAttributes.Assembly || fa == FieldAttributes.FamORAssem)
 						return true;
 				} else {
@@ -2419,8 +2426,7 @@ public class TypeManager {
 				if ((list.Length == 2) && (list [1] is FieldInfo))
 					return new MemberInfo [] { list [0] };
 
-				// Oooops
-				return null;
+				return list;
 			}
 
 			//
@@ -2441,20 +2447,13 @@ public class TypeManager {
 				mt &= (MemberTypes.Method | MemberTypes.Constructor);
 			}
 		} while (searching);
-		
-		if (use_first_members_list) {
-			foreach (MemberInfo mi in first_members_list) {
-				if (! (mi is MethodBase)) {
-					method_list = CopyNewMethods (method_list, first_members_list);
-					return (MemberInfo []) method_list.ToArray (typeof (MemberInfo));
-				}
-			}
-			return (MemberInfo []) first_members_list;
-		}
+
+		if (use_first_members_list)
+			return first_members_list;
 
 		if (method_list != null && method_list.Count > 0) {
-                        return (MemberInfo []) method_list.ToArray (typeof (MemberInfo));
-                }
+			return (MemberInfo []) method_list.ToArray (typeof (MemberInfo));
+		}
 		//
 		// This happens if we already used the cache in the first iteration, in this case
 		// the cache already looked in all interfaces.
