@@ -24,6 +24,7 @@
 #include "dump.h"
 #include "get.h"
 #include "dis-cil.h"
+#include "declsec.h"
 #include <mono/metadata/class-internals.h>
 #include <mono/metadata/object-internals.h>
 #include <mono/metadata/loader.h>
@@ -146,8 +147,16 @@ dump_declarative_security (MonoImage *m, guint32 objectType, guint32 token, cons
 		action = get_il_security_action (cols [MONO_DECL_SECURITY_ACTION]);
 		idx = cols [MONO_DECL_SECURITY_PARENT];
 		if (((idx & MONO_HAS_DECL_SECURITY_MASK) == objectType) && ((idx >> MONO_HAS_DECL_SECURITY_BITS) == token)) {
-			char *dump = data_dump (blob, len, indent);
-			fprintf (output, "%s.permissionset %s = %s", indent, action, dump);
+			char *dump;
+			if (blob [0] == MONO_DECLSEC_FORMAT_20) {
+				/* 2.0 declarative security format */
+				dump = dump_declsec_entry20 (m, blob, indent);
+				fprintf (output, "%s.permissionset %s = %s\n", indent, action, dump);
+			} else {
+				/* 1.x declarative security metadata format */
+				dump = data_dump (blob, len, indent);
+				fprintf (output, "%s.permissionset %s = %s", indent, action, dump);
+			}
 			g_free (dump);
 		}
 	}
@@ -696,7 +705,7 @@ dis_code (MonoImage *m, guint32 token, guint32 rva, MonoGenericContext *context)
 	fprintf (output, "\t.maxstack %d\n", mh->max_stack);
 	if (mh->num_locals)
 		dis_locals (m, mh, ptr);
-	dissasemble_cil (m, mh, context);
+	disassemble_cil (m, mh, context);
 	
 /*
   hex_dump (mh->code, 0, mh->code_size);
@@ -843,7 +852,7 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 		MonoGenericContainer *container;
 		MonoGenericContext *method_context = context;
 		char *flags, *impl_flags;
-		const char *sig;
+		const char *sig, *method_name;
 		char *sig_str;
 		guint32 token;
 
@@ -867,6 +876,7 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 
 		ms = mono_metadata_parse_method_signature_full (m, method_context ? method_context->container : NULL, i + 1, sig, &sig);
 		sig_str = dis_stringify_method_signature (m, ms, i + 1, method_context, FALSE);
+		method_name = mono_metadata_string_heap (m, cols [MONO_METHOD_NAME]);
 
 		fprintf (output, "    // method line %d\n", i + 1);
 		fprintf (output, "    .method %s", flags);
@@ -897,7 +907,10 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 			fprintf (output, "          // Disassembly of native methods is not supported\n");
 		else
 			dis_code (m, token, cols [MONO_METHOD_RVA], method_context);
-		fprintf (output, "    } // end of method %s::%s\n\n", klass_name, sig_str);
+		if (klass_name)
+			fprintf (output, "    } // end of method %s::%s\n\n", klass_name, method_name);
+		else
+			fprintf (output, "    } // end of global method %s\n\n", method_name);
 		mono_metadata_free_method_signature (ms);
 		g_free (sig_str);
 	}
@@ -944,7 +957,7 @@ dis_property_methods (MonoImage *m, guint32 prop, MonoGenericContext *context)
 		mono_metadata_decode_row (msemt, start, cols, MONO_METHOD_SEMA_SIZE);
 		if (!should_include_method (cols [MONO_METHOD_SEMA_METHOD]))
 			continue;
-		sig = dis_stringify_method_signature (m, NULL, cols [MONO_METHOD_SEMA_METHOD], context, TRUE);
+		sig = dis_stringify_method_signature_full (m, NULL, cols [MONO_METHOD_SEMA_METHOD], context, TRUE, FALSE);
 		fprintf (output, "\t\t%s %s\n", type [cols [MONO_METHOD_SEMA_SEMANTICS]], sig);
 		g_free (sig);
 	}
@@ -1058,7 +1071,7 @@ dis_event_methods (MonoImage *m, guint32 event, MonoGenericContext *context)
 		mono_metadata_decode_row (msemt, start, cols, MONO_METHOD_SEMA_SIZE);
 		if (!should_include_method (cols [MONO_METHOD_SEMA_METHOD]))
 			continue;
-		sig = dis_stringify_method_signature (m, NULL, cols [MONO_METHOD_SEMA_METHOD], context, TRUE);
+		sig = dis_stringify_method_signature_full (m, NULL, cols [MONO_METHOD_SEMA_METHOD], context, TRUE, FALSE);
 		switch (cols [MONO_METHOD_SEMA_SEMANTICS]) {
 		case METHOD_SEMANTIC_OTHER:
 			type = ".other"; break;
@@ -1271,9 +1284,6 @@ dis_globals (MonoImage *m)
 	guint32 cols [MONO_TYPEDEF_SIZE];
 	guint32 cols_next [MONO_TYPEDEF_SIZE];
 	gboolean next_is_valid, last;
-        gchar *name;
-
-        name = g_strdup ("<Module>");
 
         mono_metadata_decode_row (t, 0, cols, MONO_TYPEDEF_SIZE);
 
@@ -1303,7 +1313,7 @@ dis_globals (MonoImage *m)
 		last = m->tables [MONO_TABLE_METHOD].rows;
 	
 	if (cols [MONO_TYPEDEF_METHOD_LIST] && cols [MONO_TYPEDEF_METHOD_LIST] <= m->tables [MONO_TABLE_METHOD].rows)
-		dis_method_list (name, m, cols [MONO_TYPEDEF_METHOD_LIST] - 1, last, NULL);
+		dis_method_list (NULL, m, cols [MONO_TYPEDEF_METHOD_LIST] - 1, last, NULL);
 
 }
 
@@ -1784,7 +1794,7 @@ usage (void)
 	}
 	g_string_append (args, "[--forward-decls]");
 	fprintf (stderr,
-		 "monodis -- Mono Common Intermediate Language Dissassembler\n" 
+		 "monodis -- Mono Common Intermediate Language Disassembler\n" 
 		 "Usage is: monodis %s file ..\n", args->str);
 	exit (1);
 }
