@@ -135,7 +135,54 @@ static mono_mutex_t scan_mutex = MONO_MUTEX_INITIALIZER;
 
 static void handle_cleanup (void)
 {
+	int i, j, k;
+	
 	_wapi_process_signal_self ();
+
+	/* Every shared handle we were using ought really to be closed
+	 * by now, but to make sure just blow them all away.  The
+	 * exiting finalizer thread in particular races us to the
+	 * program exit and doesn't always win, so it can be left
+	 * cluttering up the shared file.  Anything else left over is
+	 * really a bug.
+	 */
+	for(i = SLOT_INDEX (0); _wapi_private_handles[i] != NULL; i++) {
+		for(j = SLOT_OFFSET (0); j < _WAPI_HANDLE_INITIAL_COUNT; j++) {
+			struct _WapiHandleUnshared *handle_data = &_wapi_private_handles[i][j];
+			int type = handle_data->type;
+			
+			
+			if (_WAPI_SHARED_HANDLE (type)) {
+				gpointer handle = GINT_TO_POINTER (i*_WAPI_HANDLE_INITIAL_COUNT+j);
+				
+				if (type == WAPI_HANDLE_THREAD) {
+					/* Special-case thread handles
+					 * because they need extra
+					 * cleanup.  This also avoids
+					 * a race condition between
+					 * the application exit and
+					 * the finalizer thread - if
+					 * it finishes up between now
+					 * and actual app termination
+					 * it will find all its handle
+					 * details have been blown
+					 * away, so this sets those
+					 * anyway.
+					 */
+					_wapi_thread_set_termination_details (handle, 0);
+				}
+				
+				for(k = handle_data->ref; k > 0; k--) {
+#ifdef DEBUG
+					g_message ("%s: unreffing %s handle %p", __func__, _wapi_handle_typename[type], handle);
+#endif
+					
+					_wapi_handle_unref (handle);
+				}
+			}
+		}
+	}
+	
 	_wapi_shm_semaphores_remove ();
 }
 
@@ -420,7 +467,7 @@ gpointer _wapi_handle_new_from_offset (WapiHandleType type, guint32 offset,
 	shared = &_wapi_shared_layout->handles[offset];
 	if (timestamp) {
 		/* Bump up the timestamp for this offset */
-		InterlockedExchange (&shared->timestamp, now);
+		InterlockedExchange ((gint32 *)&shared->timestamp, now);
 	}
 		
 	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
@@ -501,11 +548,10 @@ first_pass_done:
 	handle = GUINT_TO_POINTER (handle_idx);
 		
 	_WAPI_PRIVATE_HANDLES(handle_idx).u.shared.offset = offset;
-	InterlockedIncrement (&shared->handle_refs);
+	InterlockedIncrement ((gint32 *)&shared->handle_refs);
 	
 #ifdef DEBUG
-	g_message ("%s: Allocated new handle %p referencing 0x%x", __func__,
-		   handle, offset);
+	g_message ("%s: Allocated new handle %p referencing 0x%x (shared refs %d)", __func__, handle, offset, shared->handle_refs);
 #endif
 	
 done:
@@ -673,6 +719,11 @@ gpointer _wapi_search_handle (WapiHandleType type,
 				if (check (ret, user_data) == TRUE) {
 					_wapi_handle_ref (ret);
 					found = TRUE;
+
+					if (_WAPI_SHARED_HANDLE (type)) {
+						shared = &_wapi_shared_layout->handles[i];
+					}
+					
 					break;
 				}
 			}
@@ -730,7 +781,7 @@ gpointer _wapi_search_handle (WapiHandleType type,
 					
 					if (shared->type == type) {
 						guint32 now = (guint32)(time (NULL) & 0xFFFFFFFF);
-						InterlockedExchange (&shared->timestamp, now);
+						InterlockedExchange ((gint32 *)&shared->timestamp, now);
 
 						found = TRUE;
 						handle_data = &_WAPI_PRIVATE_HANDLES(GPOINTER_TO_UINT(ret));
@@ -865,7 +916,7 @@ void _wapi_handle_ref (gpointer handle)
 
 	handle_data = &_WAPI_PRIVATE_HANDLES(idx);
 	
-	InterlockedIncrement (&handle_data->ref);
+	InterlockedIncrement ((gint32 *)&handle_data->ref);
 
 	/* It's possible for processes to exit before getting around
 	 * to updating timestamps in the collection thread, so if a
@@ -875,7 +926,7 @@ void _wapi_handle_ref (gpointer handle)
 	if (_WAPI_SHARED_HANDLE(handle_data->type)) {
 		struct _WapiHandleShared *shared_data = &_wapi_shared_layout->handles[handle_data->u.shared.offset];
 		
-		InterlockedExchange (&shared_data->timestamp, now);
+		InterlockedExchange ((gint32 *)&shared_data->timestamp, now);
 	}
 	
 #ifdef DEBUG_REFS
@@ -908,7 +959,7 @@ void _wapi_handle_unref (gpointer handle)
 	 * could lock a mutex, but I'm not sure that allowing a handle
 	 * reference to reach 0 isn't an application bug anyway.
 	 */
-	destroy = (InterlockedDecrement (&_WAPI_PRIVATE_HANDLES(idx).ref) ==0);
+	destroy = (InterlockedDecrement ((gint32 *)&_WAPI_PRIVATE_HANDLES(idx).ref) ==0);
 	
 #ifdef DEBUG_REFS
 	g_message ("%s: %s handle %p ref now %d (destroy %s)", __func__,
@@ -974,6 +1025,10 @@ void _wapi_handle_unref (gpointer handle)
 			/* It's possible that this handle is already
 			 * pointing at a deleted shared section
 			 */
+#ifdef DEBUG_REFS
+			g_message ("%s: %s handle %p shared refs before dec %d", __func__, _wapi_handle_typename[type], handle, shared->handle_refs);
+#endif
+
 			if (shared->handle_refs > 0) {
 				shared->handle_refs--;
 				if (shared->handle_refs == 0) {
@@ -1155,11 +1210,48 @@ void _wapi_handle_ops_prewait (gpointer handle)
  */
 gboolean CloseHandle(gpointer handle)
 {
+	if (handle == NULL) {
+		/* Problem: because we map file descriptors to the
+		 * same-numbered handle we can't tell the difference
+		 * between a bogus handle and the handle to stdin.
+		 * Assume that it's the console handle if that handle
+		 * exists...
+		 */
+		if (_WAPI_PRIVATE_HANDLES (0).type != WAPI_HANDLE_CONSOLE) {
+			SetLastError (ERROR_INVALID_PARAMETER);
+			return(FALSE);
+		}
+	}
+	
 	_wapi_handle_unref (handle);
 	
 	return(TRUE);
 }
 
+/* Lots more to implement here, but this is all we need at the moment */
+gboolean DuplicateHandle (gpointer srcprocess, gpointer src,
+			  gpointer targetprocess, gpointer *target,
+			  guint32 access G_GNUC_UNUSED, gboolean inherit G_GNUC_UNUSED, guint32 options G_GNUC_UNUSED)
+{
+	if (srcprocess != _WAPI_PROCESS_CURRENT ||
+	    targetprocess != _WAPI_PROCESS_CURRENT) {
+		/* Duplicating other process's handles is not supported */
+		SetLastError (ERROR_INVALID_HANDLE);
+		return(FALSE);
+	}
+	
+	if (src == _WAPI_PROCESS_CURRENT) {
+		*target = _wapi_process_duplicate ();
+	} else if (src == _WAPI_THREAD_CURRENT) {
+		*target = _wapi_thread_duplicate ();
+	} else {
+		_wapi_handle_ref (src);
+		*target = src;
+	}
+	
+	return(TRUE);
+}
+
 gboolean _wapi_handle_count_signalled_handles (guint32 numhandles,
 					       gpointer *handles,
 					       gboolean waitall,
@@ -1442,7 +1534,7 @@ gboolean _wapi_handle_get_or_set_share (dev_t device, ino_t inode,
 			 * This makes the increment atomic wrt
 			 * collections
 			 */
-			InterlockedIncrement (&file_share->handle_refs);
+			InterlockedIncrement ((gint32 *)&file_share->handle_refs);
 			
 			exists = TRUE;
 			break;
@@ -1471,7 +1563,7 @@ gboolean _wapi_handle_get_or_set_share (dev_t device, ino_t inode,
 	}
 
 	if (*share_info != NULL) {
-		InterlockedExchange (&(*share_info)->timestamp, now);
+		InterlockedExchange ((gint32 *)&(*share_info)->timestamp, now);
 	}
 	
 	thr_ret = _wapi_shm_sem_unlock (_WAPI_SHARED_SEM_FILESHARE);
@@ -1709,8 +1801,7 @@ void _wapi_handle_update_refs (void)
 				g_message ("%s: (%d) Updating timestamp of handle 0x%x", __func__, _wapi_getpid (), handle->u.shared.offset);
 #endif
 
-				InterlockedExchange (&shared_data->timestamp,
-						     now);
+				InterlockedExchange ((gint32 *)&shared_data->timestamp, now);
 			} else if (handle->type == WAPI_HANDLE_FILE) {
 				struct _WapiHandle_file *file_handle = &handle->u.file;
 				
@@ -1724,7 +1815,7 @@ void _wapi_handle_update_refs (void)
 				g_message ("%s: (%d) Inc refs on fileshare 0x%x", __func__, _wapi_getpid (), (file_handle->share_info - &_wapi_fileshare_layout->share_info[0]) / sizeof(struct _WapiFileShare));
 #endif
 
-				InterlockedExchange (&file_handle->share_info->timestamp, now);
+				InterlockedExchange ((gint32 *)&file_handle->share_info->timestamp, now);
 			}
 		}
 	}
