@@ -81,10 +81,19 @@ static gboolean process_set_termination_details (gpointer handle, int status)
 	}
 	_wapi_time_t_to_filetime (time(NULL), &process_handle->exit_time);
 	
+#ifdef DEBUG
+	g_message ("%s: Setting handle %p signalled", __func__, handle);
+#endif
+
 	_wapi_shared_handle_set_signal_state (handle, TRUE);
 
 	_wapi_handle_unlock_shared_handles ();
 
+	/* Drop the reference we hold so we have somewhere to store
+	 * the exit details, now the process has in fact exited
+	 */
+	_wapi_handle_unref (handle);
+	
 	return (ok);
 }
 
@@ -92,7 +101,7 @@ static gboolean process_set_termination_details (gpointer handle, int status)
  * updating process handle info.  This function is called from the
  * collection thread every few seconds.
  */
-static gboolean waitfor_pid (gpointer test, gpointer user_data G_GNUC_UNUSED)
+static gboolean waitfor_pid (gpointer test, gpointer user_data)
 {
 	struct _WapiHandle_process *process;
 	gboolean ok;
@@ -129,20 +138,32 @@ static gboolean waitfor_pid (gpointer test, gpointer user_data G_GNUC_UNUSED)
 	g_message ("%s: Process %d finished", __func__, ret);
 #endif
 
-	process_set_termination_details (test, status);
+	*(int *)user_data = status;
 	
-	/* return FALSE to keep searching */
-	return (FALSE);
+	return (TRUE);
 }
 
 void _wapi_process_reap (void)
 {
+	gpointer proc;
+	int status;
+	
 #ifdef DEBUG
 	g_message ("%s: Reaping child processes", __func__);
 #endif
 
-	_wapi_search_handle (WAPI_HANDLE_PROCESS, waitfor_pid, NULL, NULL,
-			     FALSE);
+	do {
+		proc = _wapi_search_handle (WAPI_HANDLE_PROCESS, waitfor_pid,
+					    &status, NULL, FALSE);
+		if (proc != NULL) {
+#ifdef DEBUG
+			g_message ("%s: process handle %p exit code %d",
+				   __func__, proc, status);
+#endif
+			
+			process_set_termination_details (proc, status);
+		}
+	} while (proc != NULL);
 }
 
 /* Limitations: This can only wait for processes that are our own
@@ -171,7 +192,7 @@ static guint32 process_wait (gpointer handle, guint32 timeout)
 	pid = process_handle->id;
 	
 #ifdef DEBUG
-	g_message ("%s: PID is %d", __func__, pid);
+	g_message ("%s: PID is %d, timeout %d", __func__, pid, timeout);
 #endif
 
 	if (timeout == INFINITE) {
@@ -207,7 +228,7 @@ static guint32 process_wait (gpointer handle, guint32 timeout)
 
 	/* Process must have exited */
 #ifdef DEBUG
-	g_message ("%s: Wait done", __func__);
+	g_message ("%s: Wait done, status %d", __func__, status);
 #endif
 
 	ok = process_set_termination_details (handle, status);
@@ -411,8 +432,6 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 				dir[i] = '/';
 			}
 		}
-	} else {
-		dir = g_get_current_dir ();
 	}
 	
 
@@ -757,7 +776,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 
 #ifdef DEBUG
 		g_message ("%s: exec()ing [%s] in dir [%s]", __func__, cmd,
-			   dir);
+			   dir==NULL?".":dir);
 		for (i = 0; argv[i] != NULL; i++) {
 			g_message ("arg %d: [%s]", i, argv[i]);
 		}
@@ -768,16 +787,16 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 #endif
 
 		/* set cwd */
-		if (chdir (dir) == -1) {
+		if (dir != NULL && chdir (dir) == -1) {
 			/* set error */
-			exit (-1);
+			_exit (-1);
 		}
 		
 		/* exec */
 		execve (argv[0], argv, env_strings);
 		
 		/* set error */
-		exit (-1);
+		_exit (-1);
 	}
 	/* parent */
 	
@@ -868,7 +887,7 @@ static void process_set_current (void)
 	
 	if (handle_env != NULL) {
 		struct _WapiHandle_process *process_handlep;
-		guchar *procname = NULL;
+		gchar *procname = NULL;
 		gboolean ok;
 		
 		current_process = _wapi_handle_new_from_offset (WAPI_HANDLE_PROCESS, atoi (handle_env), TRUE);
@@ -933,11 +952,15 @@ static void process_set_current (void)
 		g_warning ("%s: error creating process handle", __func__);
 		return;
 	}
-		
-	/* Make sure the new handle has a reference so it wont go away
-	 * until this process exits
-	 */
+}
+
+gpointer _wapi_process_duplicate ()
+{
+	mono_once (&process_current_once, process_set_current);
+	
 	_wapi_handle_ref (current_process);
+	
+	return(current_process);
 }
 
 /* Returns a pseudo handle that doesn't need to be closed afterwards */
@@ -945,7 +968,7 @@ gpointer GetCurrentProcess (void)
 {
 	mono_once (&process_current_once, process_set_current);
 		
-	return((gpointer)-1);
+	return(_WAPI_PROCESS_CURRENT);
 }
 
 guint32 GetProcessId (gpointer handle)
@@ -1252,7 +1275,7 @@ guint32 GetModuleBaseName (gpointer process, gpointer module,
 		 */
 		pid_t pid;
 		gunichar2 *procname;
-		guchar *procname_utf8 = NULL;
+		gchar *procname_utf8 = NULL;
 		glong len, bytes;
 		
 #ifdef DEBUG
