@@ -29,10 +29,10 @@
 
 #undef DEBUG
 
-static guchar *_wapi_shm_file (_wapi_shm_t type)
+static gchar *_wapi_shm_file (_wapi_shm_t type)
 {
-	static guchar file[_POSIX_PATH_MAX];
-	guchar *name = NULL, *filename, *dir, *wapi_dir;
+	static gchar file[_POSIX_PATH_MAX];
+	gchar *name = NULL, *filename, *dir, *wapi_dir;
 	gchar machine_name[256];
 	const gchar *fake_name;
 	struct utsname ubuf;
@@ -43,6 +43,9 @@ static guchar *_wapi_shm_file (_wapi_shm_t type)
 	if (ret == -1) {
 		ubuf.machine[0] = '\0';
 		ubuf.sysname[0] = '\0';
+	} else {
+		g_strdelimit (ubuf.sysname, "/", '_');
+		g_strdelimit (ubuf.machine, "/", '_');
 	}
 
 	fake_name = g_getenv ("MONO_SHARED_HOSTNAME");
@@ -100,7 +103,7 @@ static guchar *_wapi_shm_file (_wapi_shm_t type)
 	return(file);
 }
 
-static int _wapi_shm_file_open (const guchar *filename, guint32 wanted_size)
+static int _wapi_shm_file_open (const gchar *filename, guint32 wanted_size)
 {
 	int fd;
 	struct stat statbuf;
@@ -223,7 +226,7 @@ gpointer _wapi_shm_attach (_wapi_shm_t type)
 	gpointer shm_seg;
 	int fd;
 	struct stat statbuf;
-	guchar *filename=_wapi_shm_file (type);
+	gchar *filename=_wapi_shm_file (type);
 	guint32 size;
 	
 	switch(type) {
@@ -400,7 +403,7 @@ again:
 	g_assert (thr_ret == 0);
 	
 #ifdef DEBUG
-	g_message ("%s: Incrementing the process count", __func__);
+	g_message ("%s: Incrementing the process count (%d)", __func__, _wapi_getpid ());
 #endif
 
 	/* We only ever _unlock_ this semaphore, letting the kernel
@@ -412,7 +415,7 @@ again:
 	_wapi_shm_sem_unlock (_WAPI_SHARED_SEM_PROCESS_COUNT);
 
 #ifdef DEBUG
-	g_message ("%s: Process count is now %d", __func__, semctl (_wapi_sem_id, _WAPI_SHARED_SEM_PROCESS_COUNT, GETVAL));
+	g_message ("%s: Process count is now %d (%d)", __func__, semctl (_wapi_sem_id, _WAPI_SHARED_SEM_PROCESS_COUNT, GETVAL), _wapi_getpid ());
 #endif
 	
 	_wapi_shm_sem_unlock (_WAPI_SHARED_SEM_PROCESS_COUNT_LOCK);
@@ -426,9 +429,10 @@ void _wapi_shm_semaphores_remove (void)
 	int proc_count;
 	
 #ifdef DEBUG
-	g_message ("%s: Checking process count", __func__);
+	g_message ("%s: Checking process count (%d)", __func__,
+		   _wapi_getpid ());
 #endif
-
+	
 	thr_ret = _wapi_shm_sem_lock (_WAPI_SHARED_SEM_PROCESS_COUNT_LOCK);
 	g_assert (thr_ret == 0);
 	
@@ -451,7 +455,8 @@ void _wapi_shm_semaphores_remove (void)
 		 * files
 		 */
 #ifdef DEBUG
-		g_message ("%s: Removing semaphores!", __func__);
+		g_message ("%s: Removing semaphores! (%d)", __func__,
+			   _wapi_getpid ());
 #endif
 
 		semctl (_wapi_sem_id, 0, IPC_RMID);
