@@ -8,10 +8,11 @@
  *
  * (c) 2001-2003 Ximian, Inc. (http://www.ximian.com)
  */
-
+#undef ASSEMBLY_LOAD_DEBUG
 #include <config.h>
 #include <glib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <mono/os/gc_wrapper.h>
 
@@ -30,8 +31,10 @@
 #include <mono/metadata/threadpool.h>
 #include <mono/utils/mono-uri.h>
 #include <mono/utils/mono-logger.h>
+#include <mono/utils/mono-path.h>
+#include <mono/utils/mono-stdlib.h>
 
-#define MONO_CORLIB_VERSION 49
+#define MONO_CORLIB_VERSION 54
 
 CRITICAL_SECTION mono_delegate_section;
 
@@ -201,7 +204,15 @@ mono_runtime_cleanup (MonoDomain *domain)
 	/* This ends up calling any pending pending (for at most 2 seconds) */
 	mono_gc_cleanup ();
 
+	mono_thread_cleanup ();
+
 	mono_network_cleanup ();
+
+	mono_marshal_cleanup ();
+
+	mono_type_initialization_cleanup ();
+
+	mono_monitor_cleanup ();
 }
 
 static MonoDomainFunc quit_function = NULL;
@@ -350,13 +361,15 @@ mono_domain_set (MonoDomain *domain, gboolean force)
 MonoObject *
 ves_icall_System_AppDomain_GetData (MonoAppDomain *ad, MonoString *name)
 {
-	MonoDomain *add = ad->data;
+	MonoDomain *add;
 	MonoObject *o;
 	char *str;
 
 	MONO_ARCH_SAVE_REGS;
 
 	g_assert (ad != NULL);
+	add = ad->data;
+	g_assert (add != NULL);
 
 	if (name == NULL)
 		mono_raise_exception (mono_get_exception_argument_null ("name"));
@@ -398,11 +411,13 @@ ves_icall_System_AppDomain_GetData (MonoAppDomain *ad, MonoString *name)
 void
 ves_icall_System_AppDomain_SetData (MonoAppDomain *ad, MonoString *name, MonoObject *data)
 {
-	MonoDomain *add = ad->data;
+	MonoDomain *add;
 
 	MONO_ARCH_SAVE_REGS;
 
 	g_assert (ad != NULL);
+	add = ad->data;
+	g_assert (add != NULL);
 
 	if (name == NULL)
 		mono_raise_exception (mono_get_exception_argument_null ("name"));
@@ -459,11 +474,9 @@ ves_icall_System_AppDomain_getRootDomain ()
 MonoAppDomain *
 ves_icall_System_AppDomain_createDomain (MonoString *friendly_name, MonoAppDomainSetup *setup)
 {
-	MonoDomain *domain = mono_domain_get ();
 	MonoClass *adclass;
 	MonoAppDomain *ad;
 	MonoDomain *data;
-	GSList *tmp;
 	
 	MONO_ARCH_SAVE_REGS;
 
@@ -488,12 +501,7 @@ ves_icall_System_AppDomain_createDomain (MonoString *friendly_name, MonoAppDomai
 
 	mono_context_init (data);
 
-	/* The new appdomain should have all assemblies loaded */
-	mono_domain_assemblies_lock (domain);
-	/*g_print ("copy assemblies from domain %p (%s)\n", domain, domain->friendly_name);*/
-	for (tmp = domain->domain_assemblies; tmp; tmp = tmp->next)
-		add_assemblies_to_domain (data, tmp->data, NULL);
-	mono_domain_assemblies_unlock (domain);
+	add_assemblies_to_domain (data, mono_defaults.corlib->assembly, NULL);
 
 	return ad;
 }
@@ -641,7 +649,9 @@ mono_domain_fire_assembly_load (MonoAssembly *assembly, gpointer user_data)
 	if (!domain->domain)
 		/* This can happen during startup */
 		return;
-
+#ifdef ASSEMBLY_LOAD_DEBUG
+	fprintf (stderr, "Loading %s into domain %s\n", assembly->aname.name, domain->friendly_name);
+#endif
 	klass = domain->domain->mbr.obj.vtable->klass;
 
 	mono_domain_assemblies_lock (domain);
@@ -671,56 +681,6 @@ mono_domain_fire_assembly_load (MonoAssembly *assembly, gpointer user_data)
 	mono_runtime_invoke (assembly_load_method, domain->domain, params, NULL);
 }
 
-static gchar *
-reduce_path (const gchar *dirname)
-{
-	gchar **parts;
-	gchar *part;
-	GList *list, *tmp;
-	GString *result;
-	gchar *res;
-	gint i;
-
-	parts = g_strsplit (dirname, G_DIR_SEPARATOR_S, 0);
-	list = NULL;
-	for (i = 0; (part = parts [i]) != NULL; i++) {
-		if (!strcmp (part, "."))
-			continue;
-
-		if (!strcmp (part, "..")) {
-			if (list && list->next) /* Don't remove root */
-				list = g_list_delete_link (list, list);
-		} else {
-			list = g_list_prepend (list, part);
-		}
-	}
-
-	result = g_string_new ("");
-	list = g_list_reverse (list);
-
-	for (tmp = list; tmp; tmp = tmp->next) {
-		gchar *data = (gchar *) tmp->data;
-
-		if (data && *data) {
-#ifdef PLATFORM_WIN32
-			if (result->len == 0)
-				g_string_append_printf (result, "%s\\", data);
-			else if (result->str [result->len - 1] == '\\')
-				g_string_append_printf (result, "%s", data);
-			else
-#endif
-				g_string_append_printf (result, "%c%s",
-							G_DIR_SEPARATOR, data);
-		}
-	}
-	
-	res = result->str;
-	g_string_free (result, FALSE);
-	g_list_free (list);
-	g_strfreev (parts);
-	return res;
-}
-
 static void
 set_domain_search_path (MonoDomain *domain)
 {
@@ -808,7 +768,7 @@ set_domain_search_path (MonoDomain *domain)
 			gchar *reduced;
 			gchar *freeme;
 
-			reduced = reduce_path (tmp [i]);
+			reduced = mono_path_canonicalize (tmp [i]);
 			if (appbaselen == -1)
 				appbaselen = strlen (tmp [0]);
 
@@ -835,16 +795,62 @@ set_domain_search_path (MonoDomain *domain)
 	g_strfreev (pvt_split);
 }
 
+static char *
+make_shadow_copy (const char *filename)
+{
+	gchar *tmp;
+	guint16 *orig, *dest;
+	MonoDomain *domain = mono_domain_get ();
+	char *db;
+	int fd;
+	gboolean copy_result;
+	MonoException *exc;
+
+	db = mono_string_to_utf8 (domain->setup->dynamic_base);
+	tmp = g_build_filename (db, "shadow-XXXXXX", NULL);
+	fd = mono_mkstemp (tmp);
+	if (fd == -1) {
+		exc = mono_get_exception_execution_engine ("Failed to create shadow copy (mkstemp).");
+		g_free (tmp);
+		g_free (db);
+		mono_raise_exception (exc);
+	}
+	close (fd);
+	remove (tmp);
+	orig = g_utf8_to_utf16 (filename, strlen (filename), NULL, NULL, NULL);
+	dest = g_utf8_to_utf16 (tmp, strlen (tmp), NULL, NULL, NULL);
+	copy_result = CopyFile (orig, dest, TRUE);
+	g_free (dest);
+	g_free (orig);
+	g_free (db);
+
+	if (copy_result == FALSE) {
+		g_free (tmp);
+		exc = mono_get_exception_execution_engine ("Failed to create shadow copy (CopyFile).");
+		mono_raise_exception (exc);
+	}
+	return tmp;
+}
+
 static gboolean
 try_load_from (MonoAssembly **assembly, const gchar *path1, const gchar *path2,
-					const gchar *path3, const gchar *path4, gboolean refonly)
+					const gchar *path3, const gchar *path4,
+					gboolean refonly, gboolean is_private)
 {
+	
 	gchar *fullpath;
 
 	*assembly = NULL;
 	fullpath = g_build_filename (path1, path2, path3, path4, NULL);
-	if (g_file_test (fullpath, G_FILE_TEST_IS_REGULAR))
+	if (g_file_test (fullpath, G_FILE_TEST_IS_REGULAR)) {
+		if (is_private) {
+			char *new_path = make_shadow_copy (fullpath);
+			g_free (fullpath);
+			fullpath = new_path;
+		}
+
 		*assembly = mono_assembly_open_full (fullpath, NULL, refonly);
+	}
 
 	g_free (fullpath);
 	return (*assembly != NULL);
@@ -858,6 +864,7 @@ real_load (gchar **search_path, const gchar *culture, const gchar *name, gboolea
 	gchar *filename;
 	const gchar *local_culture;
 	gint len;
+	gboolean is_private = FALSE;
 
 	if (!culture || *culture == '\0') {
 		local_culture = "";
@@ -869,28 +876,30 @@ real_load (gchar **search_path, const gchar *culture, const gchar *name, gboolea
 	len = strlen (filename);
 
 	for (path = search_path; *path; path++) {
-		if (**path == '\0')
+		if (**path == '\0') {
+			is_private = TRUE;
 			continue; /* Ignore empty ApplicationBase */
+		}
 
 		/* See test cases in bug #58992 and bug #57710 */
 		/* 1st try: [culture]/[name].dll (culture may be empty) */
 		strcpy (filename + len - 4, ".dll");
-		if (try_load_from (&result, *path, local_culture, "", filename, refonly))
+		if (try_load_from (&result, *path, local_culture, "", filename, refonly, is_private))
 			break;
 
 		/* 2nd try: [culture]/[name].exe (culture may be empty) */
 		strcpy (filename + len - 4, ".exe");
-		if (try_load_from (&result, *path, local_culture, "", filename, refonly))
+		if (try_load_from (&result, *path, local_culture, "", filename, refonly, is_private))
 			break;
 
 		/* 3rd try: [culture]/[name]/[name].dll (culture may be empty) */
 		strcpy (filename + len - 4, ".dll");
-		if (try_load_from (&result, *path, local_culture, name, filename, refonly))
+		if (try_load_from (&result, *path, local_culture, name, filename, refonly, is_private))
 			break;
 
 		/* 4th try: [culture]/[name]/[name].exe (culture may be empty) */
 		strcpy (filename + len - 4, ".exe");
-		if (try_load_from (&result, *path, local_culture, name, filename, refonly))
+		if (try_load_from (&result, *path, local_culture, name, filename, refonly, is_private))
 			break;
 	}
 
@@ -970,13 +979,19 @@ ves_icall_System_Reflection_Assembly_LoadFrom (MonoString *fname, MonoBoolean re
 
 	ass = mono_assembly_open_full (filename, &status, refOnly);
 	
-	g_free (name);
-
 	if (!ass){
-		MonoException *exc = mono_get_exception_file_not_found (fname);
+		MonoException *exc;
+
+		if (status == MONO_IMAGE_IMAGE_INVALID)
+			exc = mono_get_exception_bad_image_format (name);
+		else
+			exc = mono_get_exception_file_not_found (fname);
+		g_free (name);
 		mono_raise_exception (exc);
 	}
 
+	g_free (name);
+
 	return mono_assembly_get_object (domain, ass);
 }
 
@@ -1391,8 +1406,10 @@ mono_domain_unload (MonoDomain *domain)
 		if (mono_thread_has_appdomain_ref (mono_thread_current (), domain) && (mono_thread_interruption_requested ()))
 			/* The unload thread tries to abort us */
 			/* The icall wrapper will execute the abort */
+			CloseHandle (thread_handle);
 			return;
 	}
+	CloseHandle (thread_handle);
 
 	mono_domain_set (caller_domain, FALSE);
 
