@@ -30,6 +30,7 @@ typedef enum {
 	MONO_WRAPPER_RUNTIME_INVOKE,
 	MONO_WRAPPER_NATIVE_TO_MANAGED,
 	MONO_WRAPPER_MANAGED_TO_NATIVE,
+	MONO_WRAPPER_MANAGED_TO_MANAGED,
 	MONO_WRAPPER_REMOTING_INVOKE,
 	MONO_WRAPPER_REMOTING_INVOKE_WITH_CHECK,
 	MONO_WRAPPER_XDOMAIN_INVOKE,
@@ -46,7 +47,10 @@ typedef enum {
 	MONO_WRAPPER_STELEMREF,
 	MONO_WRAPPER_UNBOX,
 	MONO_WRAPPER_LDFLDA,
-	MONO_WRAPPER_UNKNOWN
+	MONO_WRAPPER_WRITE_BARRIER,
+	MONO_WRAPPER_UNKNOWN,
+	MONO_WRAPPER_COMINTEROP_INVOKE,
+	MONO_WRAPPER_COMINTEROP
 } MonoWrapperType;
 
 typedef enum {
@@ -58,7 +62,8 @@ typedef enum {
 
 typedef enum {
 	MONO_REMOTING_TARGET_UNKNOWN,
-	MONO_REMOTING_TARGET_APPDOMAIN
+	MONO_REMOTING_TARGET_APPDOMAIN,
+	MONO_REMOTING_TARGET_COMINTEROP
 } MonoRemotingTarget;
 
 struct _MonoMethod {
@@ -259,11 +264,16 @@ struct _MonoClass {
 	guint delegate        : 1; /* class is a Delegate */
 	guint gc_descr_inited : 1; /* gc_descr is initialized */
 	guint has_cctor       : 1; /* class has a cctor */
-	guint dummy           : 1; /* temporary hack */
-	/* next byte */
 	guint has_references  : 1; /* it has GC-tracked references in the instance */
+	/* next byte */
 	guint has_static_refs : 1; /* it has static fields that are GC-tracked */
 	guint no_special_static_fields : 1; /* has no thread/context static fields */
+	/* directly or indirectly derives from ComImport attributed class.
+	 * this means we need to create a proxy for instances of this class
+	 * for COM Interop. set this flag on loading so all we need is a quick check
+	 * during object creation rather than having to traverse supertypes
+	 */
+	guint is_com_object : 1; 
 
 	guint8     exception_type;	/* MONO_EXCEPTION_* */
 	void*      exception_data;	/* Additional information about the exception */
@@ -514,14 +524,17 @@ typedef struct {
 typedef enum {
 	MONO_LOADER_ERROR_TYPE,
 	MONO_LOADER_ERROR_METHOD,
-	MONO_LOADER_ERROR_FIELD
+	MONO_LOADER_ERROR_FIELD,
+	MONO_LOADER_ERROR_ASSEMBLY
 } MonoLoaderErrorKind;
 
 typedef struct {
 	MonoLoaderErrorKind kind;
-	char *class_name, *assembly_name; /* If kind == TYPE */
+	char *class_name; /* If kind == TYPE */
+	char *assembly_name; /* If kind == TYPE or ASSEMBLY */
 	MonoClass *klass; /* If kind != TYPE */
 	const char *member_name; /* If kind != TYPE */
+	gboolean ref_only; /* If kind == ASSEMBLY */
 } MonoLoaderError;
 
 #define mono_class_has_parent(klass,parent) (((klass)->idepth >= (parent)->idepth) && ((klass)->supertypes [(parent)->idepth - 1] == (parent)))
@@ -554,6 +567,9 @@ typedef gboolean (*MonoGetCachedClassInfo) (MonoClass *klass, MonoCachedClassInf
 void
 mono_classes_init (void) MONO_INTERNAL;
 
+void
+mono_classes_cleanup (void) MONO_INTERNAL;
+
 void
 mono_class_layout_fields   (MonoClass *klass) MONO_INTERNAL;
 
@@ -685,6 +701,8 @@ typedef struct {
 	MonoClass *internals_visible_class;
 	MonoClass *generic_array_class;
 	MonoClass *generic_nullable_class;
+	MonoClass *variant_class;
+	MonoClass *com_object_class;
 } MonoDefaults;
 
 extern MonoDefaults mono_defaults MONO_INTERNAL;
@@ -692,6 +710,9 @@ extern MonoDefaults mono_defaults MONO_INTERNAL;
 void
 mono_loader_init           (void) MONO_INTERNAL;
 
+void
+mono_loader_cleanup        (void) MONO_INTERNAL;
+
 void
 mono_loader_lock           (void) MONO_INTERNAL;
 
@@ -699,15 +720,21 @@ void
 mono_loader_unlock         (void) MONO_INTERNAL;
 
 void
-mono_loader_set_error_type_load (char *class_name, char *assembly_name) MONO_INTERNAL;
+mono_loader_set_error_assembly_load (const char *assembly_name, gboolean ref_only) MONO_INTERNAL;
 
 void
-mono_loader_set_error_method_load (MonoClass *klass, const char *member_name) MONO_INTERNAL;
+mono_loader_set_error_type_load (const char *class_name, const char *assembly_name) MONO_INTERNAL;
+
+void
+mono_loader_set_error_method_load (const char *class_name, const char *member_name) MONO_INTERNAL;
 
 void
 mono_loader_set_error_field_load (MonoClass *klass, const char *member_name) MONO_INTERNAL;
 
-MonoLoaderError*
+MonoException *
+mono_loader_error_prepare_exception (MonoLoaderError *error) MONO_INTERNAL;
+
+MonoLoaderError *
 mono_loader_get_last_error (void) MONO_INTERNAL;
 
 void
