@@ -23,6 +23,7 @@ static const char *wrapper_type_names [] = {
 	"runtime-invoke",
 	"native-to-managed",
 	"managed-to-native",
+	"managed-to-managed",
 	"remoting-invoke",
 	"remoting-invoke-with-check",
 	"xdomain-invoke",
@@ -39,7 +40,10 @@ static const char *wrapper_type_names [] = {
 	"stelemref",
 	"unbox",
 	"ldflda",
-	"unknown"
+	"write-barrier",
+	"unknown",
+	"cominterop-invoke",
+	"cominterop"
 };
 
 static void
@@ -337,8 +341,9 @@ dis_one (GString *str, MonoDisHelper *dh, MonoMethod *method, const unsigned cha
 {
 	MonoMethodHeader *header = mono_method_get_header (method);
 	const MonoOpcode *opcode;
-	guint32 i, label, token;
+	guint32 label, token;
 	gint32 sval;
+	int i;
 	char *tmp;
 	const unsigned char* il_code = mono_method_header_get_code (header, NULL, NULL);
 
