@@ -1,4 +1,3 @@
-
 /*
  * domain.c: MonoDomain functions
  *
@@ -29,6 +28,7 @@
 #include <mono/metadata/metadata-internals.h>
 #include <mono/metadata/gc-internal.h>
 #include <mono/metadata/appdomain.h>
+#include <mono/metadata/mono-debug-debugger.h>
 #include <metadata/threads.h>
 
 /* #define DEBUG_DOMAIN_UNLOAD */
@@ -199,7 +199,7 @@ mono_jit_info_table_find (MonoDomain *domain, char *addr)
 
 	if (ji == NULL) {
 		/* Maybe its an AOT module */
-		MonoImage *image = mono_jit_info_find_aot_module (addr);
+		MonoImage *image = mono_jit_info_find_aot_module ((guint8*)addr);
 		if (image)
 			ji = jit_info_find_in_aot_func (domain, image, addr);
 	}
@@ -440,12 +440,26 @@ domain_id_alloc (MonoDomain *domain)
 	return id;
 }
 
+static guint32 domain_gc_bitmap [sizeof(MonoDomain)/4/32 + 1];
+static gpointer domain_gc_desc = NULL;
+
 MonoDomain *
 mono_domain_create (void)
 {
 	MonoDomain *domain;
 
-	domain = mono_gc_alloc_fixed (sizeof (MonoDomain), NULL);
+	mono_appdomains_lock ();
+	if (!domain_gc_desc) {
+		unsigned int i, bit = 0;
+		for (i = G_STRUCT_OFFSET (MonoDomain, MONO_DOMAIN_FIRST_OBJECT); i < G_STRUCT_OFFSET (MonoDomain, MONO_DOMAIN_FIRST_GC_TRACKED); i += sizeof (gpointer)) {
+			bit = i / sizeof (gpointer);
+			domain_gc_bitmap [bit / 32] |= 1 << (bit % 32);
+		}
+		domain_gc_desc = mono_gc_make_descr_from_bitmap (domain_gc_bitmap, bit + 1);
+	}
+	mono_appdomains_unlock ();
+
+	domain = mono_gc_alloc_fixed (sizeof (MonoDomain), domain_gc_desc);
 	domain->domain = NULL;
 	domain->setup = NULL;
 	domain->friendly_name = NULL;
@@ -453,7 +467,7 @@ mono_domain_create (void)
 
 	domain->mp = mono_mempool_new ();
 	domain->code_mp = mono_code_manager_new ();
-	domain->env = mono_g_hash_table_new ((GHashFunc)mono_string_hash, (GCompareFunc)mono_string_equal);
+	domain->env = mono_g_hash_table_new_type ((GHashFunc)mono_string_hash, (GCompareFunc)mono_string_equal, MONO_HASH_KEY_VALUE_GC);
 	domain->domain_assemblies = NULL;
 	domain->class_vtable_hash = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	domain->proxy_vtable_hash = g_hash_table_new ((GHashFunc)mono_ptrarray_hash, (GCompareFunc)mono_ptrarray_equal);
@@ -512,6 +526,7 @@ mono_init_internal (const char *filename, const char *exe_filename, const char *
 	mono_raw_buffer_init ();
 	mono_images_init ();
 	mono_assemblies_init ();
+	mono_classes_init ();
 	mono_loader_init ();
 
 	/* FIXME: When should we release this memory? */
@@ -782,6 +797,12 @@ mono_init_internal (const char *filename, const char *exe_filename, const char *
 	mono_defaults.internals_visible_class = mono_class_from_name (
 	        mono_defaults.corlib, "System.Runtime.CompilerServices", "InternalsVisibleToAttribute");
 
+	mono_defaults.variant_class = mono_class_from_name (
+	        mono_defaults.corlib, "System", "Variant");
+
+	mono_defaults.com_object_class = mono_class_from_name (
+	        mono_defaults.corlib, "System", "__ComObject");
+
 	/*
 	 * Note that mono_defaults.generic_*_class is only non-NULL if we're
 	 * using the 2.0 corlib.
@@ -847,6 +868,35 @@ mono_init_version (const char *domain_name, const char *version)
 	return mono_init_internal (domain_name, NULL, version);
 }
 
+/**
+ * mono_cleanup:
+ *
+ * Cleans up all metadata modules. 
+ */
+void
+mono_cleanup (void)
+{
+	mono_loader_cleanup ();
+	mono_classes_cleanup ();
+	mono_assemblies_cleanup ();
+	mono_images_cleanup ();
+	mono_raw_buffer_cleanup ();
+	mono_metadata_cleanup ();
+
+	TlsFree (appdomain_thread_id);
+	DeleteCriticalSection (&appdomains_mutex);
+}
+
+/**
+ * mono_get_root_domain:
+ *
+ * The root AppDomain is the initial domain created by the runtime when it is
+ * initialized.  Programs execute on this AppDomain, but can create new ones
+ * later.   Currently there is no unmanaged API to create new AppDomains, this
+ * must be done from managed code.
+ *
+ * Returns: the root appdomain, to obtain the current domain, use mono_domain_get ()
+ */
 MonoDomain*
 mono_get_root_domain (void)
 {
@@ -856,7 +906,8 @@ mono_get_root_domain (void)
 /**
  * mono_domain_get:
  *
- * Returns: the current domain.
+ * Returns: the current domain, to obtain the root domain use
+ * mono_get_root_domain().
  */
 MonoDomain *
 mono_domain_get ()
@@ -1424,6 +1475,12 @@ get_runtimes_from_exe (const char *exe_file, const MonoRuntimeInfo** runtimes)
 		return;
 	}
 
+	/* 
+	 * FIXME: This would cause us to unload the image, and it will be loaded again later.
+	 * Disabling it will mean the initial exe will not be unloaded on shutdown.
+	 */
+	//mono_image_close (image);
+
 	runtimes [0] = get_runtime_by_version (image->version);
 	runtimes [1] = NULL;
 }
