@@ -63,6 +63,7 @@ run_finalize (void *obj, void *data)
 	MonoObject *o, *o2;
 	o = (MonoObject*)((char*)obj + GPOINTER_TO_UINT (data));
 
+#ifndef HAVE_SGEN_GC
 	mono_domain_lock (o->vtable->domain);
 
 	o2 = g_hash_table_lookup (o->vtable->domain->finalizable_objects_hash, o);
@@ -72,6 +73,7 @@ run_finalize (void *obj, void *data)
 	if (!o2)
 		/* Already finalized somehow */
 		return;
+#endif
 
 	/* make sure the finalizer is not called again if the object is resurrected */
 	object_register_finalizer (obj, NULL);
@@ -155,6 +157,8 @@ object_register_finalizer (MonoObject *obj, void (*callback)(void *, void*))
 	mono_domain_unlock (obj->vtable->domain);
 
 	GC_REGISTER_FINALIZER_NO_ORDER ((char*)obj - offset, callback, GUINT_TO_POINTER (offset), NULL, NULL);
+#elif defined(HAVE_SGEN_GC)
+	mono_gc_register_for_finalization (obj, callback);
 #endif
 }
 
@@ -471,7 +475,8 @@ alloc_handle (HandleData *handles, MonoObject *obj)
 			memcpy (domain_ids, handles->domain_ids, sizeof (guint16) * handles->size);
 			for (i = 0; i < handles->size; ++i) {
 				MonoObject *obj = mono_gc_weak_link_get (&(handles->entries [i]));
-				mono_gc_weak_link_remove (&(handles->entries [i]));
+				if (handles->entries [i])
+					mono_gc_weak_link_remove (&(handles->entries [i]));
 				/*g_print ("reg/unreg entry %d of type %d at %p to object %p (%p), was: %p\n", i, handles->type, &(entries [i]), obj, entries [i], handles->entries [i]);*/
 				if (obj) {
 					mono_gc_weak_link_add (&(entries [i]), obj);
@@ -596,7 +601,8 @@ mono_gchandle_set_target (guint32 gchandle, MonoObject *obj)
 	lock_handles (handles);
 	if (slot < handles->size && (handles->bitmap [slot / 32] & (1 << (slot % 32)))) {
 		if (handles->type <= HANDLE_WEAK_TRACK) {
-			mono_gc_weak_link_remove (&handles->entries [slot]);
+			if (handles->entries [slot])
+				mono_gc_weak_link_remove (&handles->entries [slot]);
 			if (obj)
 				mono_gc_weak_link_add (&handles->entries [slot], obj);
 		} else {
@@ -662,10 +668,12 @@ mono_gchandle_free (guint32 gchandle)
 		return;
 	lock_handles (handles);
 	if (slot < handles->size && (handles->bitmap [slot / 32] & (1 << (slot % 32)))) {
-		if (handles->type <= HANDLE_WEAK_TRACK)
-			mono_gc_weak_link_remove (&handles->entries [slot]);
-		else
+		if (handles->type <= HANDLE_WEAK_TRACK) {
+			if (handles->entries [slot])
+				mono_gc_weak_link_remove (&handles->entries [slot]);
+		} else {
 			handles->entries [slot] = NULL;
+		}
 		handles->bitmap [slot / 32] &= ~(1 << (slot % 32));
 	} else {
 		/* print a warning? */
@@ -848,6 +856,10 @@ void mono_gc_cleanup (void)
 	}
 
 #endif
+
+	DeleteCriticalSection (&handle_section);
+	DeleteCriticalSection (&allocator_section);
+	DeleteCriticalSection (&finalizer_mutex);
 }
 
 #else
