@@ -25,6 +25,7 @@
 #include "loader.h"
 #include <mono/io-layer/io-layer.h>
 #include <mono/utils/mono-logger.h>
+#include <mono/utils/mono-path.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -38,6 +39,9 @@ static GHashTable *loaded_images_hash;
 static GHashTable *loaded_images_guid_hash;
 static GHashTable *loaded_images_refonly_hash;
 static GHashTable *loaded_images_refonly_guid_hash;
+
+static gboolean debug_assembly_unload = FALSE;
+
 #define mono_images_lock() EnterCriticalSection (&images_mutex)
 #define mono_images_unlock() LeaveCriticalSection (&images_mutex)
 static CRITICAL_SECTION images_mutex;
@@ -82,50 +86,6 @@ mono_image_rva_map (MonoImage *image, guint32 addr)
 	return NULL;
 }
 
-static gchar *
-canonicalize_path (const char *path)
-{
-	gchar *abspath, *pos, *lastpos, *dest;
-	int backc;
-
-	if (g_path_is_absolute (path)) {
-		abspath = g_strdup (path);
-	} else {
-		gchar *tmpdir = g_get_current_dir ();
-		abspath = g_build_filename (tmpdir, path, NULL);
-		g_free (tmpdir);
-	}
-
-	abspath = g_strreverse (abspath);
-
-	backc = 0;
-	dest = lastpos = abspath;
-	pos = strchr (lastpos, G_DIR_SEPARATOR);
-
-	while (pos != NULL) {
-		int len = pos - lastpos;
-		if (len == 1 && lastpos [0] == '.') {
-			// nop
-		} else if (len == 2 && lastpos [0] == '.' && lastpos [1] == '.') {
-			backc++;
-		} else if (len > 0) {
-			if (backc > 0) {
-				backc--;
-			} else {
-				if (dest != lastpos) 
-					/* The two strings can overlap */
-					memmove (dest, lastpos, len + 1);
-				dest += len + 1;
-			}
-		}
-		lastpos = pos + 1;
-		pos = strchr (lastpos, G_DIR_SEPARATOR);
-	}
-	
-	if (dest != lastpos) strcpy (dest, lastpos);
-	return g_strreverse (abspath);
-}
-
 /**
  * mono_images_init:
  *
@@ -140,6 +100,24 @@ mono_images_init (void)
 	loaded_images_guid_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	loaded_images_refonly_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	loaded_images_refonly_guid_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+	debug_assembly_unload = getenv ("MONO_DEBUG_ASSEMBLY_UNLOAD") != NULL;
+}
+
+/**
+ * mono_images_cleanup:
+ *
+ *  Free all resources used by this module.
+ */
+void
+mono_images_cleanup (void)
+{
+	DeleteCriticalSection (&images_mutex);
+
+	g_hash_table_destroy (loaded_images_hash);
+	g_hash_table_destroy (loaded_images_guid_hash);
+	g_hash_table_destroy (loaded_images_refonly_hash);
+	g_hash_table_destroy (loaded_images_refonly_guid_hash);
 }
 
 /**
@@ -381,7 +359,7 @@ load_metadata_ptrs (MonoImage *image, MonoCLIImageInfo *iinfo)
 	g_assert (image->heap_guid.data);
 	g_assert (image->heap_guid.size >= 16);
 
-	image->guid = mono_guid_to_string (image->heap_guid.data);
+	image->guid = mono_guid_to_string ((guint8*)image->heap_guid.data);
 
 	return TRUE;
 }
@@ -462,13 +440,24 @@ static void
 load_modules (MonoImage *image, MonoImageOpenStatus *status)
 {
 	MonoTableInfo *t;
+	MonoTableInfo *file_table;
 	int i;
 	char *base_dir;
 	gboolean refonly = image->ref_only;
+	GList *list_iter, *valid_modules = NULL;
 
 	if (image->modules)
 		return;
 
+	file_table = &image->tables [MONO_TABLE_FILE];
+	for (i = 0; i < file_table->rows; i++) {
+		guint32 cols [MONO_FILE_SIZE];
+		mono_metadata_decode_row (file_table, i, cols, MONO_FILE_SIZE);
+		if (cols [MONO_FILE_FLAGS] == FILE_CONTAINS_NO_METADATA)
+			continue;
+		valid_modules = g_list_prepend (valid_modules, mono_metadata_string_heap (image, cols [MONO_FILE_NAME]));
+	}
+
 	t = &image->tables [MONO_TABLE_MODULEREF];
 	image->modules = g_new0 (MonoImage *, t->rows);
 	image->module_count = t->rows;
@@ -477,9 +466,19 @@ load_modules (MonoImage *image, MonoImageOpenStatus *status)
 		char *module_ref;
 		const char *name;
 		guint32 cols [MONO_MODULEREF_SIZE];
+		int valid = 0;
 
 		mono_metadata_decode_row (t, i, cols, MONO_MODULEREF_SIZE);
 		name = mono_metadata_string_heap (image, cols [MONO_MODULEREF_NAME]);
+		for (list_iter = valid_modules; list_iter; list_iter = list_iter->next) {
+			/* be safe with string dups, but we could just compare string indexes  */
+			if (strcmp (list_iter->data, name) == 0) {
+				valid = TRUE;
+				break;
+			}
+		}
+		if (!valid)
+			continue;
 		module_ref = g_build_filename (base_dir, name, NULL);
 		image->modules [i] = mono_image_open_full (module_ref, status, refonly);
 		if (image->modules [i]) {
@@ -487,8 +486,7 @@ load_modules (MonoImage *image, MonoImageOpenStatus *status)
 			/* g_print ("loaded module %s from %s (%p)\n", module_ref, image->name, image->assembly); */
 		}
 		/* 
-		 * FIXME: what do we do here? it could be a native dll...
-		 * We should probably do lazy-loading of modules.
+		 * FIXME: We should probably do lazy-loading of modules.
 		 */
 		if (status)
 			*status = MONO_IMAGE_OK;
@@ -496,6 +494,7 @@ load_modules (MonoImage *image, MonoImageOpenStatus *status)
 	}
 
 	g_free (base_dir);
+	g_list_free (valid_modules);
 }
 
 static void
@@ -581,7 +580,6 @@ mono_image_init (MonoImage *image)
 	image->class_cache = g_hash_table_new (NULL, NULL);
 	image->field_cache = g_hash_table_new (NULL, NULL);
 	image->name_cache = g_hash_table_new (g_str_hash, g_str_equal);
-	image->array_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
 
 	image->delegate_begin_invoke_cache = 
 		g_hash_table_new ((GHashFunc)mono_signature_hash, 
@@ -599,9 +597,20 @@ mono_image_init (MonoImage *image)
 	image->managed_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	image->native_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	image->remoting_invoke_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->cominterop_invoke_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->cominterop_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	image->synchronized_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	image->unbox_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
 
+	image->ldfld_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->ldflda_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->ldfld_remote_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->stfld_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->stfld_remote_wrapper_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->isinst_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->castclass_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	image->proxy_isinst_cache = g_hash_table_new (mono_aligned_addr_hash, NULL);
+
 	image->typespec_cache = g_hash_table_new (NULL, NULL);
 	image->memberref_signatures = g_hash_table_new (NULL, NULL);
 	image->helper_signatures = g_hash_table_new (g_str_hash, g_str_equal);
@@ -802,8 +811,9 @@ do_mono_image_open (const char *fname, MonoImageOpenStatus *status,
 	image->raw_data = mono_raw_buffer_load (fileno (filed), FALSE, 0, stat_buf.st_size);
 	iinfo = g_new0 (MonoCLIImageInfo, 1);
 	image->image_info = iinfo;
-	image->name = canonicalize_path (fname);
+	image->name = mono_path_resolve_symlinks (fname);
 	image->ref_only = refonly;
+	image->ref_count = 1;
 
 	return do_mono_image_load (image, status, care_about_cli);
 }
@@ -923,7 +933,7 @@ mono_image_open_full (const char *fname, MonoImageOpenStatus *status, gboolean r
 	
 	g_return_val_if_fail (fname != NULL, NULL);
 	
-	absfname = canonicalize_path (fname);
+	absfname = mono_path_canonicalize (fname);
 
 	/*
 	 * The easiest solution would be to do all the loading inside the mutex,
@@ -937,6 +947,7 @@ mono_image_open_full (const char *fname, MonoImageOpenStatus *status, gboolean r
 	g_free (absfname);
 	
 	if (image){
+		mono_image_addref (image);
 		mono_images_unlock ();
 		return image;
 	}
@@ -954,7 +965,9 @@ mono_image_open_full (const char *fname, MonoImageOpenStatus *status, gboolean r
  * @fname: filename that points to the module we want to open
  * @status: An error condition is returned in this field
  *
- * Returns: An open image of type %MonoImage or NULL on error.
+ * Returns: An open image of type %MonoImage or NULL on error. 
+ * The caller holds a temporary reference to the returned image which should be cleared 
+ * when no longer needed by calling mono_image_close ().
  * if NULL, then check the value of @status for details on the error
  */
 MonoImage *
@@ -988,11 +1001,13 @@ free_hash_table (gpointer key, gpointer val, gpointer user_data)
 	g_hash_table_destroy ((GHashTable*)val);
 }
 
+/*
 static void
 free_mr_signatures (gpointer key, gpointer val, gpointer user_data)
 {
 	mono_metadata_free_method_signature ((MonoMethodSignature*)val);
 }
+*/
 
 static void
 free_blob_cache_entry (gpointer key, gpointer val, gpointer user_data)
@@ -1006,6 +1021,12 @@ free_remoting_wrappers (gpointer key, gpointer val, gpointer user_data)
 	g_free (val);
 }
 
+static void
+free_array_cache_entry (gpointer key, gpointer val, gpointer user_data)
+{
+	g_slist_free ((GSList*)val);
+}
+
 /**
  * mono_image_addref:
  * @image: The image file we wish to add a reference to
@@ -1049,7 +1070,7 @@ mono_image_close (MonoImage *image)
 	if (InterlockedDecrement (&image->ref_count) > 0)
 		return;
 
-	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Unloading image %s %p.", image->name, image);
+	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Unloading image %s [%p].", image->name, image);
 	
 	mono_images_lock ();
 	loaded_images = image->ref_only ? loaded_images_refonly_hash : loaded_images_hash;
@@ -1090,15 +1111,25 @@ mono_image_close (MonoImage *image)
 
 		g_free (image->raw_data);
 	}
-	g_free (image->name);
-	g_free (image->guid);
-	g_free (image->version);
-	g_free (image->files);
+
+	if (debug_assembly_unload) {
+		image->name = g_strdup_printf ("%s - UNLOADED", image->name);
+	} else {
+		g_free (image->name);
+		g_free (image->guid);
+		g_free (image->version);
+		g_free (image->files);
+	}
 
 	g_hash_table_destroy (image->method_cache);
 	g_hash_table_destroy (image->class_cache);
 	g_hash_table_destroy (image->field_cache);
-	g_hash_table_destroy (image->array_cache);
+	if (image->array_cache) {
+		g_hash_table_foreach (image->array_cache, free_array_cache_entry, NULL);
+		g_hash_table_destroy (image->array_cache);
+	}
+	if (image->ptr_cache)
+		g_hash_table_destroy (image->ptr_cache);
 	g_hash_table_foreach (image->name_cache, free_hash_table, NULL);
 	g_hash_table_destroy (image->name_cache);
 	g_hash_table_destroy (image->native_wrapper_cache);
@@ -1111,7 +1142,18 @@ mono_image_close (MonoImage *image)
 	g_hash_table_destroy (image->runtime_invoke_cache);
 	g_hash_table_destroy (image->synchronized_cache);
 	g_hash_table_destroy (image->unbox_wrapper_cache);
+	g_hash_table_destroy (image->cominterop_invoke_cache);
+	g_hash_table_destroy (image->cominterop_wrapper_cache);
 	g_hash_table_destroy (image->typespec_cache);
+	g_hash_table_destroy (image->ldfld_wrapper_cache);
+	g_hash_table_destroy (image->ldflda_wrapper_cache);
+	g_hash_table_destroy (image->ldfld_remote_wrapper_cache);
+	g_hash_table_destroy (image->stfld_wrapper_cache);
+	g_hash_table_destroy (image->stfld_remote_wrapper_cache);
+	g_hash_table_destroy (image->isinst_cache);
+	g_hash_table_destroy (image->castclass_cache);
+	g_hash_table_destroy (image->proxy_isinst_cache);
+
 	/* The ownership of signatures is not well defined */
 	//g_hash_table_foreach (image->memberref_signatures, free_mr_signatures, NULL);
 	g_hash_table_destroy (image->memberref_signatures);
@@ -1137,10 +1179,18 @@ mono_image_close (MonoImage *image)
 		if (image->modules [i])
 			mono_image_close (image->modules [i]);
 	}
+	if (image->modules)
+		g_free (image->modules);
+	if (image->references)
+		g_free (image->references);
 	/*g_print ("destroy image %p (dynamic: %d)\n", image, image->dynamic);*/
 	if (!image->dynamic) {
-		mono_mempool_destroy (image->mempool);
-		g_free (image);
+		if (debug_assembly_unload)
+			mono_mempool_invalidate (image->mempool);
+		else {
+			mono_mempool_destroy (image->mempool);
+			g_free (image);
+		}
 	} else {
 		/* Dynamic images are GC_MALLOCed */
 		struct _MonoDynamicImage *di = (struct _MonoDynamicImage*)image;
