@@ -27,6 +27,9 @@ typedef struct _MonoMethodBuilder MonoMethodBuilder;
 void
 mono_marshal_init (void);
 
+void
+mono_marshal_cleanup (void);
+
 gint32
 mono_class_native_size (MonoClass *klass, guint32 *align);
 
@@ -87,6 +90,9 @@ mono_marshal_asany (MonoObject *obj, MonoMarshalNative string_encoding, int para
 void
 mono_marshal_free_asany (MonoObject *o, gpointer ptr, MonoMarshalNative string_encoding, int param_attrs);
 
+MonoMethod*
+mono_marshal_get_write_barrier (void);
+
 /* method builder functions */
 
 void
@@ -140,6 +146,9 @@ mono_mb_emit_icon (MonoMethodBuilder *mb, gint32 value);
 guint32
 mono_mb_emit_branch (MonoMethodBuilder *mb, guint8 op);
 
+guint32
+mono_mb_emit_short_branch (MonoMethodBuilder *mb, guint8 op);
+
 void
 mono_mb_emit_add_to_local (MonoMethodBuilder *mb, guint16 local, gint32 incr);
 
@@ -158,6 +167,12 @@ mono_mb_emit_i4 (MonoMethodBuilder *mb, gint32 data);
 void
 mono_mb_emit_ldstr (MonoMethodBuilder *mb, char *str);
 
+guint
+mono_type_to_ldind (MonoType *type);
+
+guint
+mono_type_to_stind (MonoType *type);
+
 /* functions to create various architecture independent helper functions */
 
 MonoMethod *
@@ -305,6 +320,9 @@ ves_icall_System_Runtime_InteropServices_Marshal_PtrToStringUni_len (guint16 *pt
 MonoString *
 ves_icall_System_Runtime_InteropServices_Marshal_PtrToStringBSTR (gpointer ptr);
 
+guint32
+ves_icall_System_Runtime_InteropServices_Marshal_GetComSlotForMethodInfoInternal (MonoReflectionMethod *m);
+
 guint32 
 ves_icall_System_Runtime_InteropServices_Marshal_GetLastWin32Error (void);
 
@@ -323,6 +341,9 @@ ves_icall_System_Runtime_InteropServices_Marshal_PtrToStructure_type (gpointer s
 int
 ves_icall_System_Runtime_InteropServices_Marshal_OffsetOf (MonoReflectionType *type, MonoString *field_name);
 
+gpointer
+ves_icall_System_Runtime_InteropServices_Marshal_StringToBSTR (MonoString *string);
+
 gpointer
 ves_icall_System_Runtime_InteropServices_Marshal_StringToHGlobalAnsi (MonoString *string);
 
@@ -344,12 +365,43 @@ ves_icall_System_Runtime_InteropServices_Marshal_AllocHGlobal (int size);
 void
 ves_icall_System_Runtime_InteropServices_Marshal_FreeHGlobal (void *ptr);
 
+void
+ves_icall_System_Runtime_InteropServices_Marshal_FreeBSTR (void *ptr);
+
 void*
 ves_icall_System_Runtime_InteropServices_Marshal_UnsafeAddrOfPinnedArrayElement (MonoArray *arrayobj, int index);
 
 MonoDelegate*
 ves_icall_System_Runtime_InteropServices_Marshal_GetDelegateForFunctionPointerInternal (void *ftn, MonoReflectionType *type);
 
+int
+ves_icall_System_Runtime_InteropServices_Marshal_AddRef (gpointer pUnk);
+
+int
+ves_icall_System_Runtime_InteropServices_Marshal_QueryInterface (gpointer pUnk, gpointer riid, gpointer* ppv);
+
+int
+ves_icall_System_Runtime_InteropServices_Marshal_Release (gpointer pUnk);
+
+
+MonoObject *
+ves_icall_System_ComObject_CreateRCW (MonoReflectionType *type);
+
+void
+ves_icall_System_ComObject_Finalizer(MonoComObject* obj);
+
+gpointer
+ves_icall_System_ComObject_FindInterface (MonoComObject* obj, MonoReflectionType* type);
+
+void
+ves_icall_System_ComObject_CacheInterface (MonoComObject* obj, MonoReflectionType* type, gpointer pItf);
+
+gpointer
+ves_icall_System_ComObject_GetIUnknown (MonoComObject* obj);
+
+void
+ves_icall_System_ComObject_SetIUnknown (MonoComObject* obj, gpointer pUnk);
+
 G_END_DECLS
 
 #endif /* __MONO_MARSHAL_H__ */
