@@ -31,6 +31,8 @@ G_BEGIN_DECLS
 
 #define MONO_CLASS_IS_INTERFACE(c) ((c->flags & TYPE_ATTRIBUTE_INTERFACE) || (c->byval_arg.type == MONO_TYPE_VAR) || (c->byval_arg.type == MONO_TYPE_MVAR))
 
+#define MONO_CLASS_IS_IMPORT(c) ((c->flags & TYPE_ATTRIBUTE_IMPORT))
+
 typedef struct _MonoClass MonoClass;
 typedef struct _MonoDomain MonoDomain;
 typedef struct _MonoMethod MonoMethod;
@@ -170,7 +172,8 @@ typedef enum {
 	MONO_MARSHAL_CONV_OBJECT_STRUCT,
 	MONO_MARSHAL_CONV_DEL_FTN,
 	MONO_MARSHAL_CONV_FTN_DEL,
-	MONO_MARSHAL_FREE_ARRAY
+	MONO_MARSHAL_FREE_ARRAY,
+	MONO_MARSHAL_CONV_BSTR_STR
 } MonoMarshalConv;
 
 typedef struct {
