@@ -3,6 +3,7 @@
 
 #include <mono/metadata/object.h>
 #include <mono/metadata/reflection.h>
+#include <mono/metadata/mempool.h>
 #include <mono/io-layer/io-layer.h>
 
 #if 1
@@ -149,7 +150,8 @@ typedef struct {
 	MonoObject  *async_state;
 	MonoObject  *handle;
 	MonoObject  *async_delegate;
-	gpointer     data;
+	gpointer    *data;
+	MonoObject  *object_data;
 	MonoBoolean  sync_completed;
 	MonoBoolean  completed;
 	MonoBoolean  endinvoke_called;
@@ -184,8 +186,21 @@ typedef struct {
 	MonoObject *unwrapped_server;
 	gint32      target_domain_id;
 	MonoString *target_uri;
+	MonoObject *object_identity;
+	MonoObject *obj_TP;
+	MonoObject *stub_data;
 } MonoRealProxy;
 
+typedef struct {
+	MonoRealProxy real_proxy;
+	MonoObject *com_object;
+} MonoComInteropProxy;
+
+typedef struct {
+	MonoMarshalByRefObject object;
+	GHashTable* itf_hash;
+} MonoComObject;
+
 typedef struct {
 	MonoObject	 object;
 	MonoRealProxy	*rp;	
@@ -391,6 +406,11 @@ typedef struct {
 	gint32 lcid;
 } MonoSortKey;
 
+typedef struct {
+	MonoObject object;
+	guint16 intType;
+} MonoInterfaceTypeAttribute;
+
 /* used to free a dynamic method */
 typedef void        (*MonoFreeMethodFunc)	 (MonoDomain *domain, MonoMethod *method);
 
@@ -401,7 +421,7 @@ typedef gboolean    (*MonoInitVTableFunc)    (MonoVTable *vtable);
 
 MonoAsyncResult *
 mono_async_result_new	    (MonoDomain *domain, HANDLE handle, 
-			     MonoObject *state, gpointer data);
+			     MonoObject *state, gpointer data, MonoObject *object_data);
 
 MonoWaitHandle *
 mono_wait_handle_new	    (MonoDomain *domain, HANDLE handle);
@@ -455,6 +475,9 @@ mono_install_init_vtable    (MonoInitVTableFunc func);
 void
 mono_type_initialization_init (void);
 
+void
+mono_type_initialization_cleanup (void);
+
 guint32
 mono_thread_get_tls_key    (void);
 
@@ -755,6 +778,7 @@ typedef struct {
 	MonoString *filename;
 	guint32 attrs;
 	guint32 offset;
+	MonoObject *stream;
 } MonoReflectionResource;
 
 typedef struct {
@@ -1066,6 +1090,9 @@ mono_get_constant_value_from_blob (MonoDomain* domain, MonoTypeEnum type, const
 void
 mono_release_type_locks (MonoThread *thread);
 
+char *
+mono_string_to_utf8_mp	(MonoMemPool *mp, MonoString *s);
+
 MonoArray*
 mono_array_clone_in_domain (MonoDomain *domain, MonoArray *array);
 
