@@ -1,8 +1,19 @@
+/*
+ * profiler.c: Profiler interface for Mono
+ *
+ * Author:
+ *   Paolo Molaro (lupus@ximian.com)
+ *
+ * (C) 2001-2003 Ximian, Inc.
+ * (C) 2003-2006 Novell, Inc.
+ */
 
 #include "config.h"
 #include "mono/metadata/profiler-private.h"
 #include "mono/metadata/debug-helpers.h"
 #include "mono/metadata/mono-debug.h"
+#include "mono/metadata/debug-mono-symfile.h"
+#include "mono/metadata/metadata-internals.h"
 #include "mono/metadata/class-internals.h"
 #include "mono/metadata/domain-internals.h"
 #include "mono/metadata/gc-internal.h"
@@ -62,6 +73,17 @@ static CRITICAL_SECTION profiler_coverage_mutex;
 /* this is directly accessible to other mono libs. */
 MonoProfileFlags mono_profiler_events;
 
+/**
+ * mono_profiler_install:
+ * @prof: a MonoProfiler structure pointer, or a pointer to a derived structure.
+ * @callback: the function to invoke at shutdown
+ *
+ * Use mono_profiler_install to activate profiling in the Mono runtime.
+ * Typically developers of new profilers will create a new structure whose
+ * first field is a MonoProfiler and put any extra information that they need
+ * to access from the various profiling callbacks there.
+ *
+ */
 void
 mono_profiler_install (MonoProfiler *prof, MonoProfileFunc callback)
 {
@@ -72,18 +94,46 @@ mono_profiler_install (MonoProfiler *prof, MonoProfileFunc callback)
 	InitializeCriticalSection (&profiler_coverage_mutex);
 }
 
+/**
+ * mono_profiler_set_events:
+ * @events: an ORed set of values made up of MONO_PROFILER_ flags
+ *
+ * The events descriped in the @events argument is a set of flags
+ * that represent which profiling events must be triggered.  For
+ * example if you have registered a set of methods for tracking
+ * JIT compilation start and end with mono_profiler_install_jit_compile,
+ * you will want to pass the MONO_PROFILE_JIT_COMPILATION flag to
+ * this routine.
+ *
+ * You can call mono_profile_set_events more than once and you can
+ * do this at runtime to modify which methods are invoked.
+ */
 void
 mono_profiler_set_events (MonoProfileFlags events)
 {
 	mono_profiler_events = events;
 }
 
+/**
+ * mono_profiler_get_events:
+ *
+ * Returns a list of active events that will be intercepted. 
+ */
 MonoProfileFlags
 mono_profiler_get_events (void)
 {
 	return mono_profiler_events;
 }
 
+/**
+ * mono_profiler_install_enter_leave:
+ * @enter: the routine to be called on each method entry
+ * @fleave: the routine to be called each time a method returns
+ *
+ * Use this routine to install routines that will be called everytime
+ * a method enters and leaves.   The routines will receive as an argument
+ * the MonoMethod representing the method that is entering or leaving.
+ */
 void
 mono_profiler_install_enter_leave (MonoProfileMethodFunc enter, MonoProfileMethodFunc fleave)
 {
@@ -91,6 +141,14 @@ mono_profiler_install_enter_leave (MonoProfileMethodFunc enter, MonoProfileMetho
 	method_leave = fleave;
 }
 
+/**
+ * mono_profiler_install_jit_compile:
+ * @start: the routine to be called when the JIT process starts.
+ * @end: the routine to be called when the JIT process ends.
+ *
+ * Use this routine to install routines that will be called when JIT 
+ * compilation of a method starts and completes.
+ */
 void 
 mono_profiler_install_jit_compile (MonoProfileMethodFunc start, MonoProfileMethodResult end)
 {
@@ -441,15 +499,28 @@ mono_profiler_coverage_free (MonoMethod *method)
 	mono_profiler_coverage_unlock ();
 }
 
+/**
+ * mono_profiler_coverage_get:
+ * @prof: The profiler handle, installed with mono_profiler_install
+ * @method: the method to gather information from.
+ * @func: A routine that will be called back with the results
+ *
+ * If the MONO_PROFILER_INS_COVERAGE flag was active during JIT compilation
+ * it is posisble to obtain coverage information about a give method.
+ *
+ * The function @func will be invoked repeatedly with instances of the
+ * MonoProfileCoverageEntry structure.
+ */
 void 
 mono_profiler_coverage_get (MonoProfiler *prof, MonoMethod *method, MonoProfileCoverageFunc func)
 {
 	MonoProfileCoverageInfo* info;
 	int i, offset;
-	guint32 line, col;
+	guint32 code_size;
 	const unsigned char *start, *end, *cil_code;
 	MonoMethodHeader *header;
 	MonoProfileCoverageEntry entry;
+	MonoDebugMethodInfo *debug_minfo;
 
 	mono_profiler_coverage_lock ();
 	info = g_hash_table_lookup (coverage_hash, method);
@@ -459,8 +530,10 @@ mono_profiler_coverage_get (MonoProfiler *prof, MonoMethod *method, MonoProfileC
 		return;
 
 	header = mono_method_get_header (method);
-	start = mono_method_header_get_code (header, &col, NULL);
-	end = start + col;
+	start = mono_method_header_get_code (header, &code_size, NULL);
+	debug_minfo = mono_debug_lookup_method (method);
+
+	end = start + code_size;
 	for (i = 0; i < info->entries; ++i) {
 		cil_code = info->data [i].cil_code;
 		if (cil_code && cil_code >= start && cil_code < end) {
@@ -468,12 +541,22 @@ mono_profiler_coverage_get (MonoProfiler *prof, MonoMethod *method, MonoProfileC
 			entry.iloffset = offset;
 			entry.method = method;
 			entry.counter = info->data [i].count;
-			/* the debug interface doesn't support column info, sigh */
-			col = line = 1;
-			entry.filename = mono_debug_source_location_from_il_offset (method, offset, &line);
-			entry.line = line;
-			entry.col = col;
+			entry.line = entry.col = 1;
+			entry.filename = NULL;
+			if (debug_minfo) {
+				MonoDebugSourceLocation *location;
+
+				location = mono_debug_symfile_lookup_location (debug_minfo, offset);
+				if (location) {
+					entry.line = location->row;
+					entry.col = location->column;
+					entry.filename = g_strdup (location->source_file);
+					mono_debug_free_source_location (location);
+				}
+			}
+
 			func (prof, &entry);
+			g_free (entry.filename);
 		}
 	}
 }
@@ -667,7 +750,7 @@ create_profiler (void)
 {
 	MonoProfiler *prof = g_new0 (MonoProfiler, 1);
 
-	prof->methods = g_hash_table_new (NULL, NULL);
+	prof->methods = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	MONO_TIMER_INIT (prof->jit_timer);
 	prof->mempool = mono_mempool_new ();
 	return prof;
@@ -1348,6 +1431,16 @@ mono_profiler_install_simple (const char *desc)
 typedef void (*ProfilerInitializer) (const char*);
 #define INITIALIZER_NAME "mono_profiler_startup"
 
+/**
+ * mono_profiler_load:
+ * @desc: arguments to configure the profiler
+ *
+ * Invoke this method to initialize the profiler.   This will drive the
+ * loading of the internal ("default") or any external profilers.
+ *
+ * This routine is invoked by Mono's driver, but must be called manually
+ * if you embed Mono into your application.
+ */
 void 
 mono_profiler_load (const char *desc)
 {
