@@ -64,11 +64,10 @@ ves_icall_System_String_ctor_charp (gpointer dummy, gunichar2 *value)
 	domain = mono_domain_get ();
 
 	if (value == NULL)
-		length = 0;
-	else {
-		for (i = 0; *(value + i) != '\0'; i++);
-		length = i;
-	}
+		return empty_string (domain);
+
+	for (i = 0; *(value + i) != '\0'; i++);
+	length = i;
 
 	return mono_string_new_utf16 (domain, value, length);
 }
@@ -239,6 +238,14 @@ ves_icall_System_String_ctor_encoding (gpointer dummy, gint8 *value, gint32 sind
 	return s;
 }
 
+/* This function is redirected to String.CreateString ()
+   by mono_marshal_get_native_wrapper () */
+void
+ves_icall_System_String_ctor_RedirectToCreateString (void)
+{
+	g_assert_not_reached ();
+}
+
 MonoString * 
 ves_icall_System_String_InternalJoin (MonoString *separator, MonoArray * value, gint32 sindex, gint32 count)
 {
