@@ -4,7 +4,8 @@
 #include <mono/metadata/object-internals.h>
 #include <mono/metadata/reflection.h>
 
-extern int mono_max_worker_threads;
+/* No managed code here */
+void mono_thread_pool_init (void);
 
 MonoAsyncResult *
 mono_thread_pool_add     (MonoObject *target, MonoMethodMessage *msg, 
@@ -32,7 +33,4 @@ MonoBoolean
 ves_icall_System_Threading_ThreadPool_SetMinThreads (gint workerThreads, 
 								gint completionPortThreads);
 
-MonoBoolean
-ves_icall_System_Threading_ThreadPool_BindHandle (gpointer handle);
-
 #endif
