@@ -2505,7 +2505,6 @@ wait_handle_fields[] = {
 static FieldDesc 
 thread_fields[] = {
 	{"system_thread_handle", G_STRUCT_OFFSET (MonoThread, handle)},
-	{"current_culture", G_STRUCT_OFFSET (MonoThread, culture_info)},
 	{"threadpool_thread", G_STRUCT_OFFSET (MonoThread, threadpool_thread)},
 	{"state", G_STRUCT_OFFSET (MonoThread, state)},
 	{"abort_exc", G_STRUCT_OFFSET (MonoThread, abort_exc)},
@@ -2540,10 +2539,10 @@ cinfo_fields[] = {
 
 static const FieldDesc
 dtfinfo_fields[] = {
-	{"_AMDesignator", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, AMDesignator)},
-	{"_PMDesignator", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, PMDesignator)},
-	{"_DayNames", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, DayNames)},
-	{"_MonthNames", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, MonthNames)},
+	{"amDesignator", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, AMDesignator)},
+	{"pmDesignator", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, PMDesignator)},
+	{"dayNames", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, DayNames)},
+	{"monthNames", G_STRUCT_OFFSET (MonoDateTimeFormatInfo, MonthNames)},
 	{NULL, 0}
 };
 
