@@ -24,10 +24,6 @@
 #include "mini.h"
 #include "mini-x86.h"
 
-#if defined(__FreeBSD__)
-#include <ucontext.h>
-#endif 
-
 #ifdef PLATFORM_WIN32
 static void (*restore_stack) (void *);
 
@@ -689,27 +685,15 @@ mono_arch_sigctx_to_monoctx (void *sigctx, MonoContext *mctx)
 #ifdef MONO_ARCH_USE_SIGACTION
 	ucontext_t *ctx = (ucontext_t*)sigctx;
 	
-#if defined(__FreeBSD__)
-	mctx->eax = ctx->uc_mcontext.mc_eax;
-	mctx->ebx = ctx->uc_mcontext.mc_ebx;
-	mctx->ecx = ctx->uc_mcontext.mc_ecx;
-	mctx->edx = ctx->uc_mcontext.mc_edx;
-	mctx->ebp = ctx->uc_mcontext.mc_ebp;
-	mctx->esp = ctx->uc_mcontext.mc_esp;
-	mctx->esi = ctx->uc_mcontext.mc_esi;
-	mctx->edi = ctx->uc_mcontext.mc_edi;
-	mctx->eip = ctx->uc_mcontext.mc_eip;
-#else
-	mctx->eax = ctx->uc_mcontext.gregs [REG_EAX];
-	mctx->ebx = ctx->uc_mcontext.gregs [REG_EBX];
-	mctx->ecx = ctx->uc_mcontext.gregs [REG_ECX];
-	mctx->edx = ctx->uc_mcontext.gregs [REG_EDX];
-	mctx->ebp = ctx->uc_mcontext.gregs [REG_EBP];
-	mctx->esp = ctx->uc_mcontext.gregs [REG_ESP];
-	mctx->esi = ctx->uc_mcontext.gregs [REG_ESI];
-	mctx->edi = ctx->uc_mcontext.gregs [REG_EDI];
-	mctx->eip = ctx->uc_mcontext.gregs [REG_EIP];
-#endif
+	mctx->eax = UCONTEXT_REG_EAX (ctx);
+	mctx->ebx = UCONTEXT_REG_EBX (ctx);
+	mctx->ecx = UCONTEXT_REG_ECX (ctx);
+	mctx->edx = UCONTEXT_REG_EDX (ctx);
+	mctx->ebp = UCONTEXT_REG_EBP (ctx);
+	mctx->esp = UCONTEXT_REG_ESP (ctx);
+	mctx->esi = UCONTEXT_REG_ESI (ctx);
+	mctx->edi = UCONTEXT_REG_EDI (ctx);
+	mctx->eip = UCONTEXT_REG_EIP (ctx);
 #else	
 	struct sigcontext *ctx = (struct sigcontext *)sigctx;
 
@@ -731,28 +715,15 @@ mono_arch_monoctx_to_sigctx (MonoContext *mctx, void *sigctx)
 #ifdef MONO_ARCH_USE_SIGACTION
 	ucontext_t *ctx = (ucontext_t*)sigctx;
 
-#if defined(__FreeBSD__)
-	ctx->uc_mcontext.mc_eax = mctx->eax;
-	ctx->uc_mcontext.mc_ebx = mctx->ebx;
-	ctx->uc_mcontext.mc_ecx = mctx->ecx;
-	ctx->uc_mcontext.mc_edx = mctx->edx;
-	ctx->uc_mcontext.mc_ebp  = mctx->ebp;
-	ctx->uc_mcontext.mc_esp = mctx->esp;
-	ctx->uc_mcontext.mc_esi = mctx->esi;
-	ctx->uc_mcontext.mc_edi = mctx->edi;
-	ctx->uc_mcontext.mc_eip = mctx->eip;
-
-#else
-	ctx->uc_mcontext.gregs [REG_EAX] = mctx->eax;
-	ctx->uc_mcontext.gregs [REG_EBX] = mctx->ebx;
-	ctx->uc_mcontext.gregs [REG_ECX] = mctx->ecx;
-	ctx->uc_mcontext.gregs [REG_EDX] = mctx->edx;
-	ctx->uc_mcontext.gregs [REG_EBP] = mctx->ebp;
-	ctx->uc_mcontext.gregs [REG_ESP] = mctx->esp;
-	ctx->uc_mcontext.gregs [REG_ESI] = mctx->esi;
-	ctx->uc_mcontext.gregs [REG_EDI] = mctx->edi;
-	ctx->uc_mcontext.gregs [REG_EIP] = mctx->eip;
-#endif
+	UCONTEXT_REG_EAX (ctx) = mctx->eax;
+	UCONTEXT_REG_EBX (ctx) = mctx->ebx;
+	UCONTEXT_REG_ECX (ctx) = mctx->ecx;
+	UCONTEXT_REG_EDX (ctx) = mctx->edx;
+	UCONTEXT_REG_EBP (ctx) = mctx->ebp;
+	UCONTEXT_REG_ESP (ctx) = mctx->esp;
+	UCONTEXT_REG_ESI (ctx) = mctx->esi;
+	UCONTEXT_REG_EDI (ctx) = mctx->edi;
+	UCONTEXT_REG_EIP (ctx) = mctx->eip;
 #else
 	struct sigcontext *ctx = (struct sigcontext *)sigctx;
 
@@ -773,11 +744,7 @@ mono_arch_ip_from_context (void *sigctx)
 {
 #ifdef MONO_ARCH_USE_SIGACTION
 	ucontext_t *ctx = (ucontext_t*)sigctx;
-#if defined(__FreeBSD__)
-	return (gpointer)ctx->uc_mcontext.mc_eip;
-#else
-	return (gpointer)ctx->uc_mcontext.gregs [REG_EIP];
-#endif
+	return (gpointer)UCONTEXT_REG_EIP (ctx);
 #else
 	struct sigcontext *ctx = sigctx;
 	return (gpointer)ctx->SC_EIP;
