@@ -1104,6 +1104,18 @@ class Tests {
 		if (failed)
 			return 10;
 
+		try {
+			ulong ul = UInt64.MaxValue;
+			failed = true;
+			checked {
+				a = (int)ul;
+			}
+		}
+		catch (OverflowException) {
+			failed = false;
+		}
+		if (failed)
+			return 11;
 
 		{
 			int i; 
@@ -2254,5 +2266,23 @@ class Tests {
 		}
 		return 1;
 	}
+
+	// bug #78633
+	public static int test_0_throw_to_branch_opt_outer_clause () {
+		int i = 0;
+
+		try {
+			try {
+				string [] files = new string[1];
+
+				string s = files[2];
+			} finally {
+				i ++;
+			}
+		} catch {
+		}
+
+		return (i == 1) ? 0 : 1;
+	}		
 }
 
