@@ -10,6 +10,8 @@
 
 #include <mono/metadata/appdomain.h>
 
+G_BEGIN_DECLS
+
 MonoDomain * 
 mono_jit_init              (const char *file);
 
@@ -19,5 +21,7 @@ mono_jit_exec              (MonoDomain *domain, MonoAssembly *assembly,
 void        
 mono_jit_cleanup           (MonoDomain *domain);
 
+G_END_DECLS
+
 #endif
 
