@@ -1732,6 +1732,8 @@ mono_arch_output_basic_block (MonoCompile *cfg, MonoBasicBlock *bb)
 		mono_debug_record_line_number (cfg, ins, offset);
 
 		switch (ins->opcode) {
+		case OP_MEMORY_BARRIER:
+			break;
 		case OP_TLS_GET:
 			g_assert_not_reached ();
 			break;
@@ -1866,9 +1868,6 @@ mono_arch_output_basic_block (MonoCompile *cfg, MonoBasicBlock *bb)
 			g_assert (imm8 >= 0);
 			ARM_CMP_REG_IMM (code, ins->sreg1, imm8, rot_amount);
 			break;
-		case OP_X86_TEST_NULL:
-			g_assert_not_reached ();
-			break;
 		case CEE_BREAK:
 			*(int*)code = 0xe7f001f0;
 			*(int*)code = 0xef9f0001;
@@ -2290,7 +2289,7 @@ mono_arch_output_basic_block (MonoCompile *cfg, MonoBasicBlock *bb)
 			code += 4 * GPOINTER_TO_INT (ins->klass);
 			break;
 		case OP_CEQ:
-			ARM_MOV_REG_IMM8 (code, ins->dreg, 0);
+			ARM_MOV_REG_IMM8_COND (code, ins->dreg, 0, ARMCOND_NE);
 			ARM_MOV_REG_IMM8_COND (code, ins->dreg, 1, ARMCOND_EQ);
 			break;
 		case OP_CLT:
@@ -2400,14 +2399,6 @@ mono_arch_output_basic_block (MonoCompile *cfg, MonoBasicBlock *bb)
 		case CEE_CONV_R8:
 			ARM_FLTD (code, ins->dreg, ins->sreg1);
 			break;
-		case OP_X86_FP_LOAD_I8:
-			g_assert_not_reached ();
-			/*x86_fild_membase (code, ins->inst_basereg, ins->inst_offset, TRUE);*/
-			break;
-		case OP_X86_FP_LOAD_I4:
-			g_assert_not_reached ();
-			/*x86_fild_membase (code, ins->inst_basereg, ins->inst_offset, FALSE);*/
-			break;
 		case OP_FCONV_TO_I1:
 			code = emit_float_to_int (cfg, code, ins->dreg, ins->sreg1, 1, TRUE);
 			break;
@@ -2492,7 +2483,7 @@ mono_arch_output_basic_block (MonoCompile *cfg, MonoBasicBlock *bb)
 			break;
 		case OP_FCEQ:
 			ARM_FCMP (code, ARM_FPA_CMF, ins->sreg1, ins->sreg2);
-			ARM_MOV_REG_IMM8 (code, ins->dreg, 0);
+			ARM_MOV_REG_IMM8_COND (code, ins->dreg, 0, ARMCOND_NE);
 			ARM_MOV_REG_IMM8_COND (code, ins->dreg, 1, ARMCOND_EQ);
 			break;
 		case OP_FCLT:
@@ -3138,7 +3129,7 @@ mono_arch_emit_this_vret_args (MonoCompile *cfg, MonoCallInst *inst, int this_re
 		this->sreg1 = this_reg;
 		this->dreg = mono_regstate_next_int (cfg->rs);
 		mono_bblock_add_inst (cfg->cbb, this);
-		mono_call_inst_add_outarg_reg (inst, this->dreg, this_dreg, FALSE);
+		mono_call_inst_add_outarg_reg (cfg, inst, this->dreg, this_dreg, FALSE);
 	}
 
 	if (vt_reg != -1) {
@@ -3148,14 +3139,19 @@ mono_arch_emit_this_vret_args (MonoCompile *cfg, MonoCallInst *inst, int this_re
 		vtarg->sreg1 = vt_reg;
 		vtarg->dreg = mono_regstate_next_int (cfg->rs);
 		mono_bblock_add_inst (cfg->cbb, vtarg);
-		mono_call_inst_add_outarg_reg (inst, vtarg->dreg, ARMREG_R0, FALSE);
+		mono_call_inst_add_outarg_reg (cfg, inst, vtarg->dreg, ARMREG_R0, FALSE);
 	}
 }
 
 MonoInst*
 mono_arch_get_inst_for_method (MonoCompile *cfg, MonoMethod *cmethod, MonoMethodSignature *fsig, MonoInst **args)
 {
-	return NULL;
+	MonoInst *ins = NULL;
+	if (cmethod->klass == mono_defaults.thread_class &&
+			strcmp (cmethod->name, "MemoryBarrier") == 0) {
+		MONO_INST_NEW (cfg, ins, OP_MEMORY_BARRIER);
+	}
+	return ins;
 }
 
 gboolean
