@@ -42,6 +42,9 @@ static const char*const * ins_spec = s390x_cpu_desc;
 #elif defined(__s390__)
 const char * const s390_cpu_desc [OP_LAST];
 static const char*const * ins_spec = s390_cpu_desc;
+#elif defined(__alpha__)
+const char * const alpha_desc [OP_LAST];
+static const char*const * ins_spec = alpha_desc;
 #else
 #error "Not implemented"
 #endif
@@ -51,23 +54,22 @@ static const char*const * ins_spec = s390_cpu_desc;
 static inline GSList*
 g_slist_append_mempool (MonoMemPool *mp, GSList *list, gpointer data)
 {
-  GSList *new_list;
-  GSList *last;
-
-  new_list = mono_mempool_alloc (mp, sizeof (GSList));
-  new_list->data = data;
-  new_list->next = NULL;
-
-  if (list) {
-	  last = list;
-	  while (last->next)
-		  last = last->next;
-      last->next = new_list;
-
-      return list;
-    }
-  else
-      return new_list;
+	GSList *new_list;
+	GSList *last;
+	
+	new_list = mono_mempool_alloc (mp, sizeof (GSList));
+	new_list->data = data;
+	new_list->next = NULL;
+	
+	if (list) {
+		last = list;
+		while (last->next)
+			last = last->next;
+		last->next = new_list;
+		
+		return list;
+	} else
+		return new_list;
 }
 
 /**
@@ -849,7 +851,7 @@ mono_local_regalloc (MonoCompile *cfg, MonoBasicBlock *bb)
 	}
 	reginfof = cfg->reginfof;
 	if (!reginfof) {
-		cfg->reginfof_len = MAX (256, rs->next_vireg * 2);
+		cfg->reginfof_len = MAX (256, rs->next_vfreg * 2);
 		reginfof = cfg->reginfof = mono_mempool_alloc (cfg->mempool, sizeof (RegTrack) * cfg->reginfof_len);
 	} 
 	else
@@ -1203,6 +1205,7 @@ mono_local_regalloc (MonoCompile *cfg, MonoBasicBlock *bb)
 					assign_ireg (cfg, rs, ins->dreg, new_dest);
 					clob_dreg = ins->dreg;
 					create_copy_ins (cfg, dest_sreg2, new_dest, ins, ip, FALSE);
+					mono_regstate_free_int (rs, dest_sreg2);
 					need_spill = FALSE;
 				}
 
