@@ -65,6 +65,12 @@ mono_arch_patch_callsite (guint8 *code, guint8 *addr)
 	}
 }
 
+void
+mono_arch_patch_plt_entry (guint8 *code, guint8 *addr)
+{
+	g_assert_not_reached ();
+}
+
 void
 mono_arch_nullify_class_init_trampoline (guint8 *code, gssize *regs)
 {
@@ -72,6 +78,12 @@ mono_arch_nullify_class_init_trampoline (guint8 *code, gssize *regs)
 	sparc_nop (code);
 }
 
+void
+mono_arch_nullify_plt_entry (guint8 *code)
+{
+	g_assert_not_reached ();
+}
+
 #define ALIGN_TO(val,align) (((val) + ((align) - 1)) & ~((align) - 1))
 
 guchar*
@@ -251,13 +263,11 @@ mono_arch_create_specific_trampoline (gpointer arg1, MonoTrampolineType tramp_ty
  * This method is only called when running in the Mono Debugger.
  */
 gpointer
-mono_debugger_create_notification_function (gpointer *notification_address)
+mono_debugger_create_notification_function (MonoCodeManager *codeman)
 {
 	guint8 *ptr, *buf;
 
-	ptr = buf = mono_global_codeman_reserve (16);
-	if (notification_address)
-		*notification_address = buf;
+	ptr = buf = mono_code_manager_reserve (codeman, 16);
 
 	g_assert_not_reached ();
 
