@@ -77,6 +77,10 @@ mono_arch_patch_callsite (guint8 *code, guint8 *addr)
 				//VALGRIND_DISCARD_TRANSLATIONS (code + 2, code + 6);
 #endif
 		}
+	} else if (code [1] == 0xe9) {
+		/* A PLT entry: jmp <DISP> */
+		if (!mono_running_on_valgrind ())
+			InterlockedExchange ((gint32*)(code + 2), (guint)addr - ((guint)code + 1) - 5);
 	} else {
 		printf ("Invalid trampoline sequence: %x %x %x %x %x %x %x\n", code [0], code [1], code [2], code [3],
 				code [4], code [5], code [6]);
@@ -84,6 +88,16 @@ mono_arch_patch_callsite (guint8 *code, guint8 *addr)
 	}
 }
 
+void
+mono_arch_patch_plt_entry (guint8 *code, guint8 *addr)
+{
+	/* A PLT entry: jmp <DISP> */
+	g_assert (code [0] == 0xe9);
+
+	if (!mono_running_on_valgrind ())
+		InterlockedExchange ((gint32*)(code + 1), (guint)addr - (guint)code - 5);
+}
+
 void
 mono_arch_nullify_class_init_trampoline (guint8 *code, gssize *regs)
 {
@@ -135,6 +149,26 @@ mono_arch_nullify_class_init_trampoline (guint8 *code, gssize *regs)
 		}
 }
 
+void
+mono_arch_nullify_plt_entry (guint8 *code)
+{
+	if (!mono_running_on_valgrind ()) {
+		guint32 ops;
+
+		ops = 0xfeeb;
+		InterlockedExchange ((gint32*)code, ops);
+
+		/* Then change the other bytes to a nop */
+		code [2] = 0x90;
+		code [3] = 0x90;
+		code [4] = 0x90;
+
+		/* Change the first byte to a nop */
+		ops = 0xc3;
+		InterlockedExchange ((gint32*)code, ops);
+	}
+}
+
 void
 mono_arch_patch_delegate_trampoline (guint8 *code, guint8 *tramp, gssize *regs, guint8 *addr)
 {
@@ -267,6 +301,8 @@ mono_arch_create_trampoline_code (MonoTrampolineType tramp_type)
 		x86_call_code (buf, mono_class_init_trampoline);
 	else if (tramp_type == MONO_TRAMPOLINE_AOT)
 		x86_call_code (buf, mono_aot_trampoline);
+	else if (tramp_type == MONO_TRAMPOLINE_AOT_PLT)
+		x86_call_code (buf, mono_aot_plt_trampoline);
 	else if (tramp_type == MONO_TRAMPOLINE_DELEGATE)
 		x86_call_code (buf, mono_delegate_trampoline);
 	else
@@ -363,11 +399,11 @@ mono_arch_invalidate_method (MonoJitInfo *ji, void *func, gpointer func_arg)
  * This method is only called when running in the Mono Debugger.
  */
 guint8 *
-mono_debugger_create_notification_function ()
+mono_debugger_create_notification_function (MonoCodeManager *codeman)
 {
 	guint8 *buf, *code;
 
-	code = buf = mono_global_codeman_reserve (2);
+	code = buf = mono_code_manager_reserve (codeman, 2);
 	x86_breakpoint (buf);
 	x86_ret (buf);
 	return code;
