@@ -5,6 +5,11 @@
 #include <errno.h>
 #include <time.h>
 
+#ifdef WIN32
+#include <windows.h>
+#include "initguid.h"
+#endif
+
 #ifdef WIN32
 #define STDCALL __stdcall
 #else
@@ -303,6 +308,11 @@ typedef struct {
 	gunichar2 *d2;
 } simplestruct;
 
+typedef struct {
+	double x;
+	double y;
+} point;
+
 STDCALL simplestruct
 mono_test_return_vtype (int i)
 {
@@ -351,7 +361,7 @@ mono_test_ref_vtype (int a, simplestruct *ss, int b, RefVTypeDelegate func)
 		ss->b = 0;
 		ss->c = 1;
 		ss->d = "TEST2";
-	
+
 		return func (a, ss, b);
 	}
 
@@ -450,6 +460,25 @@ mono_test_marshal_struct2_2 (int i, int j, int k, simplestruct2 ss)
 	return 1;
 }
 
+STDCALL int 
+mono_test_marshal_lpstruct (simplestruct *ss)
+{
+	if (ss->a == 0 && ss->b == 1 && ss->c == 0 &&
+	    !strcmp (ss->d, "TEST"))
+		return 0;
+
+	return 1;
+}
+
+STDCALL int 
+mono_test_marshal_lpstruct_blittable (point *p)
+{
+	if (p->x == 1.0 && p->y == 2.0)
+		return 0;
+	else
+		return 1;
+}
+
 STDCALL int
 mono_test_marshal_struct_array (simplestruct2 *ss)
 {
@@ -524,6 +553,7 @@ get_sp (void)
 	int i;
 	void *p;
 
+	/* Yes, this is correct, we are only trying to determine the value of the stack here */
 	p = &i;
 	return p;
 }
@@ -1104,14 +1134,14 @@ TestVectorList (VectorList *vl)
 	return res;
 }
 
-typedef struct _OSVERSIONINFO
+typedef struct OSVERSIONINFO_STRUCT
 { 
 	int a; 
 	int b; 
-} OSVERSIONINFO; 
+} OSVERSIONINFO_STRUCT;
 
 STDCALL int 
-GetVersionEx (OSVERSIONINFO *osvi)
+MyGetVersionEx (OSVERSIONINFO_STRUCT *osvi)
 {
 
 	// printf ("GOT %d %d\n", osvi->a, osvi->b);
@@ -1123,7 +1153,7 @@ GetVersionEx (OSVERSIONINFO *osvi)
 }
 
 STDCALL int 
-BugGetVersionEx (int a, int b, int c, int d, int e, int f, int g, int h, OSVERSIONINFO *osvi)
+BugGetVersionEx (int a, int b, int c, int d, int e, int f, int g, int h, OSVERSIONINFO_STRUCT *osvi)
 {
 
 	// printf ("GOT %d %d\n", osvi->a, osvi->b);
@@ -1134,11 +1164,6 @@ BugGetVersionEx (int a, int b, int c, int d, int e, int f, int g, int h, OSVERSI
 	return osvi->a + osvi->b;
 }
 
-typedef struct {
-	double x;
-	double y;
-} point;
-
 STDCALL int
 mono_test_marshal_point (point pt)
 {
@@ -1220,10 +1245,6 @@ marshal_test_bool_struct(struct BoolStruct *s)
     return res;
 }
 
-#ifdef WIN32
-extern __declspec(dllimport) __stdcall void SetLastError(int x);
-#endif
-
 STDCALL void
 mono_test_last_error (int err)
 {
@@ -1799,3 +1820,350 @@ mono_test_marshal_return_string_array_delegate (ReturnStringArrayDelegate d)
 
 	return res;
 }
+
+STDCALL int
+add_delegate (int i, int j)
+{
+	return i + j;
+}
+
+STDCALL gpointer
+mono_test_marshal_return_fnptr (void)
+{
+	return &add_delegate;
+}
+
+/*
+ * COM INTEROP TESTS
+ */
+
+#ifdef WIN32
+
+STDCALL int
+mono_test_marshal_bstr_in(BSTR bstr)
+{
+	if (!wcscmp(bstr, L"mono_test_marshal_bstr_in"))
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_bstr_out(BSTR* bstr)
+{
+	*bstr = SysAllocString(L"mono_test_marshal_bstr_out");
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_sbyte(VARIANT variant)
+{
+	if (variant.vt == VT_I1 && variant.cVal == 100)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_byte(VARIANT variant)
+{
+	if (variant.vt == VT_UI1 && variant.bVal == 100)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_short(VARIANT variant)
+{
+	if (variant.vt == VT_I2 && variant.iVal == 314)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_ushort(VARIANT variant)
+{
+	if (variant.vt == VT_UI2 && variant.uiVal == 314)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_int(VARIANT variant)
+{
+	if (variant.vt == VT_I4 && variant.lVal == 314)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_uint(VARIANT variant)
+{
+	if (variant.vt == VT_UI4 && variant.ulVal == 314)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_long(VARIANT variant)
+{
+	if (variant.vt == VT_I8 && variant.llVal == 314)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_ulong(VARIANT variant)
+{
+	if (variant.vt == VT_UI8 && variant.ullVal == 314)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_float(VARIANT variant)
+{
+	if (variant.vt == VT_R4 && (variant.fltVal - 3.14)/3.14 < .001)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_double(VARIANT variant)
+{
+	if (variant.vt == VT_R8 && (variant.dblVal - 3.14)/3.14 < .001)
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_in_bstr(VARIANT variant)
+{
+	if (variant.vt == VT_BSTR && !wcscmp(variant.bstrVal, L"PI"))
+		return 0;
+	return 1;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_sbyte(VARIANT* variant)
+{
+	variant->vt = VT_I1;
+	variant->cVal = 100;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_byte(VARIANT* variant)
+{	
+	variant->vt = VT_UI1;
+	variant->bVal = 100;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_short(VARIANT* variant)
+{
+	variant->vt = VT_I2;
+	variant->iVal = 314;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_ushort(VARIANT* variant)
+{
+	variant->vt = VT_UI2;
+	variant->uiVal = 314;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_int(VARIANT* variant)
+{
+	variant->vt = VT_I4;
+	variant->lVal = 314;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_uint(VARIANT* variant)
+{
+	variant->vt = VT_UI4;
+	variant->ulVal = 314;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_long(VARIANT* variant)
+{
+	variant->vt = VT_I8;
+	variant->llVal = 314;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_ulong(VARIANT* variant)
+{
+	variant->vt = VT_UI8;
+	variant->ullVal = 314;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_float(VARIANT* variant)
+{
+	variant->vt = VT_R4;
+	variant->fltVal = 3.14;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_double(VARIANT* variant)
+{
+	variant->vt = VT_R8;
+	variant->dblVal = 3.14;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_variant_out_bstr(VARIANT* variant)
+{
+	variant->vt = VT_BSTR;
+	variant->bstrVal = SysAllocString(L"PI");
+
+	return 0;
+}
+
+#ifdef _MSC_VER
+#define COM_STDCALL __stdcall
+#else
+#define COM_STDCALL __attribute__((stdcall))
+#endif
+
+typedef struct MonoComObject MonoComObject;
+
+typedef struct
+{
+	int (COM_STDCALL *QueryInterface)(MonoComObject* pUnk, gpointer riid, gpointer* ppv);
+	int (COM_STDCALL *AddRef)(MonoComObject* pUnk);
+	int (COM_STDCALL *Release)(MonoComObject* pUnk);
+	int (COM_STDCALL *Add)(MonoComObject* pUnk, int a, int b, int* c);
+	int (COM_STDCALL *Subtract)(MonoComObject* pUnk, int a, int b, int* c);
+	int (COM_STDCALL *Same)(MonoComObject* pUnk, MonoComObject* *pOut);
+	int (COM_STDCALL *Different)(MonoComObject* pUnk, MonoComObject* *pOut);
+} MonoIUnknown;
+
+struct MonoComObject
+{
+	MonoIUnknown* vtbl;
+	int m_ref;
+};
+
+DEFINE_GUID(IID_IMath, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1);
+DEFINE_GUID(IID_IMonoUnknown, 0, 0, 0, 0xC0, 0, 0, 0, 0, 0, 0, 0x46);
+DEFINE_GUID(IID_IMonoDispatch, 0x00020400, 0, 0, 0xC0, 0, 0, 0, 0, 0, 0, 0x46);
+
+int COM_STDCALL MonoQueryInterface(MonoComObject* pUnk, gpointer riid, gpointer* ppv)
+{
+	*ppv = NULL;
+	if (!memcmp(riid, &IID_IMonoUnknown, sizeof(GUID))) {
+		*ppv = pUnk;
+		return S_OK;
+	}
+	else if (!memcmp(riid, &IID_IMath, sizeof(GUID))) {
+		*ppv = pUnk;
+		return S_OK;
+	}
+	else if (!memcmp(riid, &IID_IMonoDispatch, sizeof(GUID))) {
+		*ppv = pUnk;
+		return S_OK;
+	}
+	return E_NOINTERFACE;
+}
+
+int COM_STDCALL MonoAddRef(MonoComObject* pUnk)
+{
+	return ++(pUnk->m_ref);
+}
+
+int COM_STDCALL MonoRelease(MonoComObject* pUnk)
+{
+	return --(pUnk->m_ref);
+}
+
+int COM_STDCALL Add(MonoComObject* pUnk, int a, int b, int* c)
+{
+	*c = a + b;
+	return 0;
+}
+
+int COM_STDCALL Subtract(MonoComObject* pUnk, int a, int b, int* c)
+{
+	*c = a - b;
+	return 0;
+}
+
+static void create_com_object (MonoComObject** pOut);
+static MonoComObject* same_com_object = NULL;
+
+int COM_STDCALL Same(MonoComObject* pUnk, MonoComObject** pOut)
+{
+	if (!same_com_object)
+		create_com_object (&same_com_object);
+	*pOut = same_com_object;
+	return 0;
+}
+
+int COM_STDCALL Different(MonoComObject* pUnk, MonoComObject** pOut)
+{
+	create_com_object (pOut);
+	return 0;
+}
+
+static void create_com_object (MonoComObject** pOut)
+{
+	*pOut = g_new0 (MonoComObject, 1);
+	(*pOut)->vtbl = g_new0 (MonoIUnknown, 1);
+
+	(*pOut)->m_ref = 1;
+	(*pOut)->vtbl->QueryInterface = MonoQueryInterface;
+	(*pOut)->vtbl->AddRef = MonoAddRef;
+	(*pOut)->vtbl->Release = MonoRelease;
+	(*pOut)->vtbl->Add = Add;
+	(*pOut)->vtbl->Subtract = Subtract;
+	(*pOut)->vtbl->Same = Same;
+	(*pOut)->vtbl->Different = Different;
+}
+
+STDCALL int
+mono_test_marshal_com_object_create(MonoComObject* *pUnk)
+{
+	create_com_object (pUnk);
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_com_object_destroy(MonoComObject *pUnk)
+{
+	int ref = --(pUnk->m_ref);
+	g_free(pUnk->vtbl);
+	g_free(pUnk);
+
+	return ref;
+}
+
+STDCALL int
+mono_test_marshal_com_object_ref_count(MonoComObject *pUnk)
+{
+	return pUnk->m_ref;
+}
+
+#endif //NOT_YET
