@@ -201,6 +201,12 @@ public class Tests {
 	[DllImport ("libtest", EntryPoint="mono_test_empty_struct")]
 	public static extern int mono_test_empty_struct (int a, EmptyStruct es, int b);
 
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_lpstruct")]
+	public static extern int mono_test_marshal_lpstruct ([In, MarshalAs(UnmanagedType.LPStruct)] SimpleStruct ss);
+
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_lpstruct_blittable")]
+	public static extern int mono_test_marshal_lpstruct_blittable ([In, MarshalAs(UnmanagedType.LPStruct)] Point p);
+
 	[DllImport ("libtest", EntryPoint="mono_test_marshal_struct_array")]
 	public static extern int mono_test_marshal_struct_array (SimpleStruct2[] ss);
 
@@ -383,6 +389,22 @@ public class Tests {
 		return 0;
 	}
 
+	public static int test_0_marshal_lpstruct () {
+		SimpleStruct ss = new  SimpleStruct ();
+		ss.b = true;
+		ss.d = "TEST";
+		
+		return mono_test_marshal_lpstruct (ss);
+	}
+
+	public static int test_0_marshal_lpstruct_blittable () {
+		Point p = new Point ();
+		p.x = 1.0;
+		p.y = 2.0;
+		
+		return mono_test_marshal_lpstruct_blittable (p);
+	}
+
 	public static int test_0_marshal_struct_array () {
 		SimpleStruct2[] ss_arr = new SimpleStruct2 [2];
 
