@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <glib.h>
 
 #ifdef PLATFORM_WIN32
 #include <windows.h>
@@ -250,7 +251,7 @@ new_codechunk (int dynamic, int size)
 		 */
 #ifndef PLATFORM_WIN32
 		{
-			char *page_start = (char *) (((unsigned long long) (ptr)) & ~ (pagesize - 1));
+			char *page_start = (char *) (((gssize) (ptr)) & ~ (pagesize - 1));
 			int pages = ((char*)ptr + chunk_size - page_start + pagesize - 1) / pagesize;
 			int err = mprotect (page_start, pages * pagesize, PROT_READ | PROT_WRITE | PROT_EXEC);
 			assert (!err);
@@ -263,8 +264,10 @@ new_codechunk (int dynamic, int size)
 		}
 #endif
 
+#ifdef BIND_ROOM
 			/* Make sure the thunks area is zeroed */
 			memset (ptr, 0, bsize);
+#endif
 	}
 
 	chunk = malloc (sizeof (CodeChunk));
